@@ -2,14 +2,19 @@
 #
 # Run this to generate all the initial makefiles.
 #
-# $Id: autogen.sh 11400 2004-07-18 00:24:25Z guy $
+# $Id: autogen.sh 14143 2005-04-19 22:27:05Z gram $
 
 DIE=true
 PROJECT="Ethereal"
 
-ACLOCAL=aclocal
+# If you are going to use the non-default name for automake becase your OS
+# installaion has multiple versions, you need to call both aclocal and automake
+# with that version number, as they come from the same package.
+#AM_VERSION='-1.8'
+
+ACLOCAL=aclocal$AM_VERSION
 AUTOHEADER=autoheader
-AUTOMAKE=automake
+AUTOMAKE=automake$AM_VERSION
 AUTOCONF=autoconf
 
 # Check for python. There's no "--version" option!
