@@ -3,7 +3,7 @@
  *
  * Copyright 2004 Ian Schorr
  *
- * $Id: capinfos.c 13003 2005-01-11 00:13:42Z guy $
+ * $Id: capinfos.c 16996 2006-01-11 01:38:16Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -41,10 +41,6 @@
 #include <sys/time.h>
 #endif
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #include <glib.h>
 
 #include <epan/packet.h>
@@ -70,7 +66,7 @@ static gboolean cap_packet_size = FALSE;
 typedef struct _capture_info {
 	const char		*filename;
 	guint16			file_type;
-	guint64			filesize;
+	gint64			filesize;
 	guint64			packet_bytes;
 	double			start_time;
 	double			stop_time;
@@ -87,9 +83,9 @@ typedef struct _capture_info {
 } capture_info;
 
 static double
-secs_usecs(guint32 s, guint32 us)
+secs_nsecs(const struct wtap_nstime * nstime)
 {
-  return (us / 1000000.0) + (double)s;
+  return (nstime->nsecs / 1000000000.0) + (double)nstime->secs;
 }
 
 static void
@@ -106,7 +102,7 @@ print_stats(capture_info *cf_info)
 
   if (cap_file_type) printf("File type: %s\n", file_type_string);
   if (cap_packet_count) printf("Number of packets: %u \n", cf_info->packet_count);
-  if (cap_file_size) printf("File size: %" PRIu64 " bytes\n", cf_info->filesize);
+  if (cap_file_size) printf("File size: %" PRId64 " bytes\n", cf_info->filesize);
   if (cap_data_size) printf("Data size: %" PRIu64 " bytes\n", cf_info->packet_bytes);
   if (cap_duration) printf("Capture duration: %f seconds\n", cf_info->duration);
   if (cap_start_time) printf("Start time: %s", ctime (&start_time_t));
@@ -121,7 +117,7 @@ process_cap_file(wtap *wth, const char *filename)
 {
   int			err;
   gchar			*err_info;
-  struct stat   cf_stat;
+  gint64		size;
   long  		data_offset;
   
   guint32		packet = 0;
@@ -135,7 +131,7 @@ process_cap_file(wtap *wth, const char *filename)
   /* Tally up data that we need to parse through the file to find */
   while (wtap_read(wth, &err, &err_info, &data_offset))  {
     phdr = wtap_phdr(wth);
-    cur_time = secs_usecs(phdr->ts.tv_sec, phdr->ts.tv_usec);
+    cur_time = secs_nsecs(&phdr->ts);
     if(packet==0) {
       start_time = cur_time;
       stop_time = cur_time;
@@ -166,14 +162,15 @@ process_cap_file(wtap *wth, const char *filename)
   }
 
   /* File size */
-  if (fstat(wtap_fd(wth), &cf_stat) < 0) {
+  size = wtap_file_size(wth, &err);
+  if (size == -1) {
     fprintf(stderr,
-            "capinfos: Can't fstat \"%s\": %s.\n",
-	    filename, strerror(errno));
+            "capinfos: Can't get size of \"%s\": %s.\n",
+	    filename, strerror(err));
     return 1;
   }
   
-  cf_info.filesize = cf_stat.st_size;
+  cf_info.filesize = size;
   
   /* File Type */
   cf_info.file_type = wtap_file_type(wth);
@@ -213,22 +210,38 @@ static void usage(gboolean is_error)
     output = stderr;
   }
 
-
-  fprintf(output, "Usage: capinfos [-t] [-c] [-s] [-d] [-u] [-a] [-e] [-y]\n");
-  fprintf(output, "                [-i] [-z] [-h] <capfile>\n");
-  fprintf(output, "  where\t-t display the capture type of <capfile>\n");
-  fprintf(output, "       \t-c count the number of packets\n");
-  fprintf(output, "       \t-s display the size of the file \n");
-  fprintf(output, "       \t-d display the total length of all packets in the file\n");
-  fprintf(output, "       \t   (in bytes)\n");
-  fprintf(output, "       \t-u display the capture duration (in seconds) \n");
-  fprintf(output, "       \t-a display the capture start time\n");
-  fprintf(output, "       \t-e display the capture end time\n");
-  fprintf(output, "       \t-y display average data rate (in bytes)\n");
-  fprintf(output, "       \t-i display average data rate (in bits)\n");
-  fprintf(output, "       \t-z display average packet size (in bytes)\n");
-  fprintf(output, "       \t-h produces this help listing.\n");
-  fprintf(output, "\n      \t    If no data flags are given, default is to display all statistics\n");
+  fprintf(output, "Capinfos %s"
+#ifdef SVNVERSION
+	  " (" SVNVERSION ")"
+#endif
+	  "\n", VERSION);
+  fprintf(output, "Prints information about capture files.\n");
+  fprintf(output, "See http://www.ethereal.com for more information.\n");
+  fprintf(output, "\n");
+  fprintf(output, "Usage: capinfos [options] <infile> ...\n");
+  fprintf(output, "\n");
+  fprintf(output, "General:\n");
+  fprintf(output, "  -t display the capture file type\n");
+  fprintf(output, "\n");
+  fprintf(output, "Size:\n");
+  fprintf(output, "  -c display the number of packets\n");
+  fprintf(output, "  -s display the size of the file (in bytes)\n");
+  fprintf(output, "  -d display the total length of all packets (in bytes)\n");
+  fprintf(output, "\n");
+  fprintf(output, "Time:\n");
+  fprintf(output, "  -u display the capture duration (in seconds) \n");
+  fprintf(output, "  -a display the capture start time\n");
+  fprintf(output, "  -e display the capture end time\n");
+  fprintf(output, "\n");
+  fprintf(output, "Statistic:\n");
+  fprintf(output, "  -y display average data rate (in bytes/s)\n");
+  fprintf(output, "  -i display average data rate (in bits/s)\n");
+  fprintf(output, "  -z display average packet size (in bytes)\n");
+  fprintf(stderr, "\n");
+  fprintf(stderr, "Miscellaneous:\n");
+  fprintf(stderr, "  -h display this help and exit\n");
+  fprintf(output, "\n");
+  fprintf(output, "If no options are given, default is to display all infos\n");
 }
 
 int main(int argc, char *argv[])
