@@ -3,7 +3,7 @@
  * time, so that we only need one Ethereal binary and one Tethereal binary
  * for Windows, regardless of whether WinPcap is installed or not.
  *
- * $Id: capture-wpcap.c,v 1.6 2003/10/10 09:48:54 guy Exp $
+ * $Id: capture-wpcap.c 17451 2006-03-02 21:29:15Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,19 +28,23 @@
 # include "config.h"
 #endif
 
-#include <glib.h>
-#include <gmodule.h>
-
 #ifdef HAVE_LIBPCAP
 #include <pcap.h>
 #endif
 
-#include "pcap-util.h"
-#include "pcap-util-int.h"
+#include <glib.h>
+#include <gmodule.h>
+
+#include "capture-pcap-util.h"
+#include "capture-pcap-util-int.h"
 
 /* XXX - yes, I know, I should move cppmagic.h to a generic location. */
 #include "tools/lemon/cppmagic.h"
 
+
+#define MAX_WIN_IF_NAME_LEN 511
+
+
 gboolean has_wpcap = FALSE;
 
 #ifdef HAVE_LIBPCAP
@@ -65,11 +69,20 @@ static int     (*p_pcap_lookupnet) (char *, bpf_u_int32 *, bpf_u_int32 *,
 static pcap_t* (*p_pcap_open_live) (char *, int, int, int, char *);
 #endif
 static int     (*p_pcap_loop) (pcap_t *, int, pcap_handler, guchar *);
+static void    (*p_pcap_freecode) (struct bpf_program *);
 #ifdef HAVE_PCAP_FINDALLDEVS
 static int     (*p_pcap_findalldevs) (pcap_if_t **, char *);
 static void    (*p_pcap_freealldevs) (pcap_if_t *);
 #endif
+#ifdef HAVE_PCAP_DATALINK_NAME_TO_VAL
+static int (*p_pcap_datalink_name_to_val) (const char *);
+#endif
+#ifdef HAVE_PCAP_DATALINK_VAL_TO_NAME
+static const char *(*p_pcap_datalink_val_to_name) (int);
+#endif
 static const char *(*p_pcap_lib_version) (void);
+static int     (*p_pcap_setbuff) (pcap_t *, int dim);
+static int     (*p_pcap_next_ex) (pcap_t *, struct pcap_pkthdr **pkt_header, const u_char **pkt_data);
 
 typedef struct {
 	const char	*name;
@@ -97,11 +110,20 @@ load_wpcap(void)
 		SYM(pcap_lookupnet, FALSE),
 		SYM(pcap_open_live, FALSE),
 		SYM(pcap_loop, FALSE),
+		SYM(pcap_freecode, TRUE),
 #ifdef HAVE_PCAP_FINDALLDEVS
 		SYM(pcap_findalldevs, TRUE),
 		SYM(pcap_freealldevs, TRUE),
+#endif
+#ifdef HAVE_PCAP_DATALINK_NAME_TO_VAL
+		SYM(pcap_datalink_name_to_val, TRUE),
+#endif
+#ifdef HAVE_PCAP_DATALINK_VAL_TO_NAME
+		SYM(pcap_datalink_val_to_name, TRUE),
 #endif
 		SYM(pcap_lib_version, TRUE),
+		SYM(pcap_setbuff, TRUE),
+		SYM(pcap_next_ex, TRUE),
 		{ NULL, NULL, FALSE }
 	};
 
@@ -165,7 +187,6 @@ pcap_dispatch(pcap_t *a, int b, pcap_handler c, guchar *d)
 	return p_pcap_dispatch(a, b, c, d);
 }
 
-
 int
 pcap_snapshot(pcap_t *a)
 {
@@ -173,7 +194,6 @@ pcap_snapshot(pcap_t *a)
 	return p_pcap_snapshot(a);
 }
 
-
 int
 pcap_datalink(pcap_t *a)
 {
@@ -232,6 +252,15 @@ pcap_loop(pcap_t *a, int b, pcap_handler c, guchar *d)
 	return p_pcap_loop(a, b, c, d);
 }
 
+void
+pcap_freecode(struct bpf_program *a)
+{
+	g_assert(has_wpcap);
+    if(p_pcap_freecode) {
+	    p_pcap_freecode(a);
+    }
+}
+
 #ifdef HAVE_PCAP_FINDALLDEVS
 int
 pcap_findalldevs(pcap_if_t **a, char *b)
@@ -248,6 +277,166 @@ pcap_freealldevs(pcap_if_t *a)
 }
 #endif
 
+#if defined(HAVE_PCAP_DATALINK_NAME_TO_VAL) || defined(HAVE_PCAP_DATALINK_VAL_TO_NAME)
+/*
+ * Table of DLT_ types, names, and descriptions, for use if the version
+ * of WinPcap we have installed lacks "pcap_datalink_name_to_val()"
+ * or "pcap_datalink_val_to_name()".
+ */
+struct dlt_choice {
+	const char *name;
+	const char *description;
+	int	dlt;
+};
+
+#define DLT_CHOICE(code, description) { #code, description, code }
+#define DLT_CHOICE_SENTINEL { NULL, NULL, 0 }
+
+static struct dlt_choice dlt_choices[] = {
+	DLT_CHOICE(DLT_NULL, "BSD loopback"),
+	DLT_CHOICE(DLT_EN10MB, "Ethernet"),
+	DLT_CHOICE(DLT_IEEE802, "Token ring"),
+	DLT_CHOICE(DLT_ARCNET, "ARCNET"),
+	DLT_CHOICE(DLT_SLIP, "SLIP"),
+	DLT_CHOICE(DLT_PPP, "PPP"),
+	DLT_CHOICE(DLT_FDDI, "FDDI"),
+	DLT_CHOICE(DLT_ATM_RFC1483, "RFC 1483 IP-over-ATM"),
+	DLT_CHOICE(DLT_RAW, "Raw IP"),
+#ifdef DLT_SLIP_BSDOS
+	DLT_CHOICE(DLT_SLIP_BSDOS, "BSD/OS SLIP"),
+#endif
+#ifdef DLT_PPP_BSDOS
+	DLT_CHOICE(DLT_PPP_BSDOS, "BSD/OS PPP"),
+#endif
+#ifdef DLT_ATM_CLIP
+	DLT_CHOICE(DLT_ATM_CLIP, "Linux Classical IP-over-ATM"),
+#endif
+#ifdef DLT_PPP_SERIAL
+	DLT_CHOICE(DLT_PPP_SERIAL, "PPP over serial"),
+#endif
+#ifdef DLT_PPP_ETHER
+	DLT_CHOICE(DLT_PPP_ETHER, "PPPoE"),
+#endif
+#ifdef DLT_C_HDLC
+	DLT_CHOICE(DLT_C_HDLC, "Cisco HDLC"),
+#endif
+#ifdef DLT_IEEE802_11
+	DLT_CHOICE(DLT_IEEE802_11, "802.11"),
+#endif
+#ifdef DLT_FRELAY
+	DLT_CHOICE(DLT_FRELAY, "Frame Relay"),
+#endif
+#ifdef DLT_LOOP
+	DLT_CHOICE(DLT_LOOP, "OpenBSD loopback"),
+#endif
+#ifdef DLT_ENC
+	DLT_CHOICE(DLT_ENC, "OpenBSD encapsulated IP"),
+#endif
+#ifdef DLT_LINUX_SLL
+	DLT_CHOICE(DLT_LINUX_SLL, "Linux cooked"),
+#endif
+#ifdef DLT_LTALK
+	DLT_CHOICE(DLT_LTALK, "Localtalk"),
+#endif
+#ifdef DLT_PFLOG
+	DLT_CHOICE(DLT_PFLOG, "OpenBSD pflog file"),
+#endif
+#ifdef DLT_PRISM_HEADER
+	DLT_CHOICE(DLT_PRISM_HEADER, "802.11 plus Prism header"),
+#endif
+#ifdef DLT_IP_OVER_FC
+	DLT_CHOICE(DLT_IP_OVER_FC, "RFC 2625 IP-over-Fibre Channel"),
+#endif
+#ifdef DLT_SUNATM
+	DLT_CHOICE(DLT_SUNATM, "Sun raw ATM"),
+#endif
+#ifdef DLT_IEEE802_11_RADIO
+	DLT_CHOICE(DLT_IEEE802_11_RADIO, "802.11 plus radio information header"),
+#endif
+#ifdef DLT_ARCNET_LINUX
+	DLT_CHOICE(DLT_ARCNET_LINUX, "Linux ARCNET"),
+#endif
+#ifdef DLT_LINUX_IRDA
+	DLT_CHOICE(DLT_LINUX_IRDA, "Linux IrDA"),
+#endif
+#ifdef DLT_LINUX_LAPD
+	DLT_CHOICE(DLT_LINUX_LAPD, "Linux vISDN LAPD"),
+#endif
+#ifdef DLT_LANE8023
+	DLT_CHOICE(DLT_LANE8023, "Linux 802.3 LANE"),
+#endif
+#ifdef DLT_CIP
+	DLT_CHOICE(DLT_CIP, "Linux Classical IP-over-ATM"),
+#endif
+#ifdef DLT_HDLC
+	DLT_CHOICE(DLT_HDLC, "Cisco HDLC"),
+#endif
+	DLT_CHOICE_SENTINEL
+};
+#endif /* defined(HAVE_PCAP_DATALINK_NAME_TO_VAL) || defined(HAVE_PCAP_DATALINK_VAL_TO_NAME) */
+
+#ifdef HAVE_PCAP_DATALINK_NAME_TO_VAL
+int
+pcap_datalink_name_to_val(const char *name)
+{
+	int i;
+
+	g_assert(has_wpcap);
+
+	if (p_pcap_datalink_name_to_val != NULL)
+		return p_pcap_datalink_name_to_val(name);
+	else {
+		/*
+		 * We don't have it in WinPcap; do it ourselves.
+		 */
+		for (i = 0; dlt_choices[i].name != NULL; i++) {
+			if (strcasecmp(dlt_choices[i].name + sizeof("DLT_") - 1,
+			    name) == 0)
+				return dlt_choices[i].dlt;
+		}
+		return -1;
+	}
+}
+#endif
+
+#ifdef HAVE_PCAP_DATALINK_VAL_TO_NAME
+const char *
+pcap_datalink_val_to_name(int dlt)
+{
+	int i;
+
+	g_assert(has_wpcap);
+
+	if (p_pcap_datalink_val_to_name != NULL)
+		return p_pcap_datalink_val_to_name(dlt);
+	else {
+		/*
+		 * We don't have it in WinPcap; do it ourselves.
+		 */
+		for (i = 0; dlt_choices[i].name != NULL; i++) {
+			if (dlt_choices[i].dlt == dlt)
+				return dlt_choices[i].name + sizeof("DLT_") - 1;
+		}
+		return NULL;
+	}
+}
+#endif
+
+/* setbuff is win32 specific! */
+int pcap_setbuff(pcap_t *a, int b)
+{
+	g_assert(has_wpcap);
+	return p_pcap_setbuff(a, b);
+}
+
+/* pcap_next_ex is available since libpcap 0.8 / WinPcap 3.0! */
+/* (if you get a declaration warning here, try to update to at least WinPcap 3.1b4 develpack) */
+int pcap_next_ex (pcap_t *a, struct pcap_pkthdr **b, const u_char **c)
+{
+	g_assert(has_wpcap);
+	return p_pcap_next_ex(a, b, c);
+}
+
 /*
  * This will use "pcap_findalldevs()" if we have it, otherwise it'll
  * fall back on "pcap_lookupdev()".
@@ -272,7 +461,7 @@ get_interface_list(int *err, char *err_str)
 	 * PacketGetAdapterNames.  According to the documentation
 	 * I could find:
 	 *
-	 *	http://winpcap.polito.it/docs/man/html/Packet32_8c.html#a43
+	 *	http://www.winpcap.org/docs/man/html/Packet32_8c.html#a43
 	 *
 	 * this means that:
 	 *
@@ -348,7 +537,7 @@ get_interface_list(int *err, char *err_str)
 				j = 0;
 				while (names[i] != 0) {
 					if (j < MAX_WIN_IF_NAME_LEN)
-					ascii_name[j++] = names[i++];
+					ascii_name[j++] = (char) names[i++];
 				}
 				ascii_name[j] = '\0';
 				i++;
@@ -402,6 +591,28 @@ get_interface_list(int *err, char *err_str)
 	return il;
 }
 
+/*
+ * Get an error message string for a CANT_GET_INTERFACE_LIST error from
+ * "get_interface_list()".
+ */
+gchar *
+cant_get_if_list_error_message(const char *err_str)
+{
+	/*
+	 * If the error message includes "Not enough storage is available
+	 * to process this command" or "The operation completed successfully",
+	 * suggest that they install a WinPcap version later than 3.0.
+	 */
+	if (strstr(err_str, "Not enough storage is available to process this command") != NULL ||
+	    strstr(err_str, "The operation completed successfully") != NULL) {
+		return g_strdup_printf("Can't get list of interfaces: %s\n"
+"This might be a problem with WinPcap 3.0; you should try updating to\n"
+"a later version of WinPcap - see the WinPcap site at www.winpcap.org",
+		    err_str);
+	}
+	return g_strdup_printf("Can't get list of interfaces: %s", err_str);
+}
+
 /*
  * Append the version of WinPcap with which we were compiled to a GString.
  */
@@ -423,12 +634,47 @@ get_runtime_pcap_version(GString *str)
 	 * not and, if we have it and we have "pcap_lib_version()",
 	 * what version we have.
 	 */
+	GModule *handle;		/* handle returned by dlopen */
+	static gchar *packetVer;
+	gchar *blankp;
+
 	if (has_wpcap) {
 		g_string_sprintfa(str, "with ");
 		if (p_pcap_lib_version != NULL)
 			g_string_sprintfa(str, p_pcap_lib_version());
-		else
-			g_string_append(str, "WinPcap (version unknown)");
+		else {
+			/*
+			 * An alternative method of obtaining the version
+			 * number, by using the PacketLibraryVersion
+			 * string from packet.dll.
+			 *
+			 * Unfortunately, in WinPcap 3.0, it returns
+			 * "3.0 alpha3", even in the final version of
+			 * WinPcap 3.0, so if there's a blank in the
+			 * string, we strip it and everything after
+			 * it from the string, so we don't misleadingly
+			 * report that 3.0 alpha3 is being used when
+			 * the final version is being used.
+			 */
+			if (packetVer == NULL) {
+				packetVer = "version unknown";
+				handle = g_module_open("Packet.dll", 0);
+				if (handle != NULL) {
+					if (g_module_symbol(handle,
+					    "PacketLibraryVersion",
+					    (gpointer*)&packetVer)) {
+						packetVer = g_strdup(packetVer);
+						blankp = strchr(packetVer, ' ');
+						if (blankp != NULL)
+							*blankp = '\0';
+					} else {
+						packetVer = "version unknown";
+					}
+					g_module_close(handle);
+				}
+			}
+			g_string_sprintfa(str, "WinPcap (%s)", packetVer);
+		}
 	} else
 		g_string_append(str, "without WinPcap");
 	g_string_append(str, " ");
