@@ -1,7 +1,7 @@
 /* capture_ui_utils.c
  * Utilities for capture user interfaces
  *
- * $Id: capture_ui_utils.c 14144 2005-04-19 22:32:52Z ulfl $
+ * $Id: capture_ui_utils.c 17327 2006-02-17 02:18:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,12 +28,11 @@
 
 #ifdef HAVE_LIBPCAP
 
-#include <pcap.h>
 #include <string.h>
 #include <glib.h>
 
 #include <epan/prefs.h>
-#include "pcap-util.h"
+#include "capture-pcap-util.h"
 #include "capture_ui_utils.h"
 
 /*
@@ -110,7 +109,7 @@ get_interface_descriptive_name(const char *if_name)
   GList *if_entry;
   if_info_t *if_info;
   int err;
-  char err_buf[PCAP_ERRBUF_SIZE];
+  char err_buf[CAPTURE_PCAP_ERRBUF_SIZE];
 
   /* Do we have a user-supplied description? */
   descr = capture_dev_user_descr_find(if_name);
@@ -269,10 +268,10 @@ free_capture_combo_list(GList *combo_list)
  * Given text that contains an interface name possibly prefixed by an
  * interface description, extract the interface name.
  */
-char *
-get_if_name(char *if_text)
+const char *
+get_if_name(const char *if_text)
 {
-  char *if_name;
+  const char *if_name;
 
 #ifdef _WIN32
   /*
