@@ -1,7 +1,7 @@
 /* clopts_common.h
  * Handle command-line arguments common to Ethereal and Tethereal
  *
- * $Id: clopts_common.h 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: clopts_common.h 16526 2005-11-17 05:59:21Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,15 +29,9 @@
 extern "C" {
 #endif /* __cplusplus */
 
-/*
- * Handle the "-G" option, to cause protocol field, etc. information
- * to be printed.
- */
-void handle_dashG_option(int argc, char **argv, const char *progname);
+int get_natural_int(const char *string, const char *name);
 
-int get_natural_int(const char *appname, const char *string, const char *name);
-
-int get_positive_int(const char *appname, const char *string, const char *name);
+int get_positive_int(const char *string, const char *name);
 
 #ifdef __cplusplus
 }
