@@ -1,7 +1,7 @@
 /* color_filters.c
  * Routines for color filters
  *
- * $Id: color_filters.c 13929 2005-03-27 02:19:51Z ulfl $
+ * $Id: color_filters.c 16403 2005-11-06 22:43:25Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,6 +34,7 @@
 #include <string.h>
 
 #include <epan/filesystem.h>
+#include "file_util.h"
 
 #include <epan/packet.h>
 #include "color.h"
@@ -111,8 +112,8 @@ color_filters_init(void)
 
 /* Create a new filter */
 color_filter_t *
-color_filter_new(gchar *name,          /* The name of the filter to create */
-                 gchar *filter_string, /* The string representing the filter */
+color_filter_new(const gchar *name,    /* The name of the filter to create */
+                 const gchar *filter_string, /* The string representing the filter */
                  color_t *bg_color,    /* The background color */
                  color_t *fg_color)    /* The foreground color */
 {
@@ -365,7 +366,7 @@ read_filters(void)
 
 	/* decide what file to open (from dfilter code) */
 	path = get_persconffile_path("colorfilters", FALSE);
-	if ((f = fopen(path, "r")) == NULL) {
+	if ((f = eth_fopen(path, "r")) == NULL) {
 		if (errno != ENOENT) {
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			    "Could not open filter file\n\"%s\": %s.", path,
@@ -392,7 +393,7 @@ read_global_filters(void)
 
 	/* decide what file to open (from dfilter code) */
 	path = get_datafile_path("colorfilters");
-	if ((f = fopen(path, "r")) == NULL) {
+	if ((f = eth_fopen(path, "r")) == NULL) {
 		if (errno != ENOENT) {
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			    "Could not open global filter file\n\"%s\": %s.", path,
@@ -416,7 +417,7 @@ color_filters_import(gchar *path, gpointer arg)
 	FILE *f;
 	gboolean ret;
 
-	if ((f = fopen(path, "r")) == NULL) {
+	if ((f = eth_fopen(path, "r")) == NULL) {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Could not open\n%s\nfor reading: %s.",
 		    path, strerror(errno));
@@ -456,7 +457,7 @@ write_filter(gpointer filter_arg, gpointer data_arg)
 }
 
 /* save filters in a filter file */
-gboolean
+static gboolean
 write_filters_file(FILE *f, gboolean only_marked)
 {
 	struct write_filter_data data;
@@ -488,7 +489,7 @@ color_filters_write(void)
 	}
 
 	path = get_persconffile_path("colorfilters", TRUE);
-	if ((f = fopen(path, "w+")) == NULL) {
+	if ((f = eth_fopen(path, "w+")) == NULL) {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Could not open\n%s\nfor writing: %s.",
 		    path, strerror(errno));
@@ -525,7 +526,7 @@ color_filters_export(gchar *path, gboolean only_marked)
 {
 	FILE *f;
 
-	if ((f = fopen(path, "w+")) == NULL) {
+	if ((f = eth_fopen(path, "w+")) == NULL) {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Could not open\n%s\nfor writing: %s.",
 		    path, strerror(errno));
