@@ -2,7 +2,7 @@
  * Routines to fetch IPv4 and IPv6 addresses from a tvbuff and then mask
  * out bits other than those covered by a prefix length
  *
- * $Id: addr_and_mask.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: addr_and_mask.c 15764 2005-09-12 06:21:35Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,7 +31,6 @@
 #include <string.h>
 
 #include "tvbuff.h"
-#include "ipv6-utils.h"
 #include "addr_and_mask.h"
 
 /*
@@ -65,10 +64,10 @@ ipv6_addr_and_mask(tvbuff_t *tvb, int offset, struct e_in6_addr *addr,
 		return -1;
 
 	addr_len = (prefix_len + 7) / 8;
-	memset(addr->u6_addr.u6_addr8, 0, 16);
-	tvb_memcpy(tvb, addr->u6_addr.u6_addr8, offset, addr_len);
+	memset(addr->bytes, 0, 16);
+	tvb_memcpy(tvb, addr->bytes, offset, addr_len);
 	if (prefix_len % 8) {
-		addr->u6_addr.u6_addr8[addr_len - 1] &=
+		addr->bytes[addr_len - 1] &=
 		    ((0xff00 >> (prefix_len % 8)) & 0xff);
 	}
 
