@@ -1,7 +1,7 @@
 /* addr_resolv.c
  * Routines for network object lookup
  *
- * $Id: addr_resolv.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: addr_resolv.c 17441 2006-03-02 00:06:15Z ulfl $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -104,6 +104,9 @@
 #ifdef HAVE_GNU_ADNS
 # include <errno.h>
 # include <adns.h>
+# ifdef inet_aton
+#  undef inet_aton
+# endif
 #endif
 
 #if defined(_WIN32) && defined(INET6)
@@ -117,7 +120,9 @@
 #include "ipv6-utils.h"
 #include "addr_resolv.h"
 #include "filesystem.h"
+#include <wiretap/file_util.h>
 #include <epan/prefs.h>
+#include <epan/emem.h>
 
 #define ENAME_HOSTS		"hosts"
 #define ENAME_ETHERS 		"ethers"
@@ -137,15 +142,15 @@
 
 typedef struct hashipv4 {
   guint			addr;
-  gchar   		name[MAXNAMELEN];
   gboolean              is_dummy_entry;	/* name is IPv4 address in dot format */
   struct hashipv4 	*next;
+  gchar   		name[MAXNAMELEN];
 } hashipv4_t;
 
 /* hash table used for IPv6 lookup */
 
 #define HASH_IPV6_ADDRESS(addr)	\
-	((((addr).s6_addr8[14] << 8)|((addr).s6_addr8[15])) & (HASHHOSTSIZE - 1))
+	((((addr).bytes[14] << 8)|((addr).bytes[15])) & (HASHHOSTSIZE - 1))
 
 typedef struct hashipv6 {
   struct e_in6_addr	addr;
@@ -218,6 +223,7 @@ static hashipv6_t	*ipv6_table[HASHHOSTSIZE];
 static hashport_t	*udp_port_table[HASHPORTSIZE];
 static hashport_t	*tcp_port_table[HASHPORTSIZE];
 static hashport_t	*sctp_port_table[HASHPORTSIZE];
+static hashport_t	*dccp_port_table[HASHPORTSIZE];
 static hashether_t	*eth_table[HASHETHSIZE];
 static hashmanuf_t	*manuf_table[HASHMANUFSIZE];
 static hashether_t	*(*wka_table[48])[HASHETHSIZE];
@@ -293,6 +299,10 @@ static gchar *serv_name_lookup(guint port, port_type proto)
     table = sctp_port_table;
     serv_proto = "sctp";
     break;
+  case PT_DCCP:
+    table = dccp_port_table;
+    serv_proto = "dcp";
+    break;
   default:
     /* not yet implemented */
     return NULL;
@@ -326,7 +336,7 @@ static gchar *serv_name_lookup(guint port, port_type proto)
   if (!(g_resolv_flags & RESOLV_TRANSPORT) ||
       (servp = getservbyport(g_htons(port), serv_proto)) == NULL) {
     /* unknown port */
-    sprintf(tp->name, "%d", port);
+    g_snprintf(tp->name, MAXNAMELEN, "%d", port);
   } else {
     strncpy(tp->name, servp->s_name, MAXNAMELEN);
     tp->name[MAXNAMELEN-1] = '\0';
@@ -387,7 +397,7 @@ static gchar *host_name_lookup(guint addr, gboolean *found)
   tp->next = NULL;
 
 #ifdef HAVE_GNU_ADNS
-  if ((g_resolv_flags & RESOLV_CONCURRENT) && 
+  if ((g_resolv_flags & RESOLV_CONCURRENT) &&
       prefs.name_resolve_concurrency > 0 &&
       gnu_adns_initialized) {
     qmsg = g_malloc(sizeof(adns_queue_msg_t));
@@ -539,7 +549,7 @@ static const gchar *solve_address_to_name(address *addr)
     return get_hostname(ipv4_addr);
 
   case AT_IPv6:
-    memcpy(&ipv6_addr.s6_addr, addr->data, sizeof ipv6_addr.s6_addr);
+    memcpy(&ipv6_addr.bytes, addr->data, sizeof ipv6_addr.bytes);
     return get_hostname6(&ipv6_addr);
 
   case AT_STRINGZ:
@@ -758,7 +768,7 @@ static void set_ethent(char *path)
   if (eth_p)
     rewind(eth_p);
   else
-    eth_p = fopen(path, "r");
+    eth_p = eth_fopen(path, "r");
 }
 
 static void end_ethent(void)
@@ -1053,6 +1063,7 @@ static hashether_t *add_eth_name(const guint8 *addr, const gchar *name)
 {
   int hash_idx;
   hashether_t *tp;
+  int new_one = TRUE;
 
   hash_idx = HASH_ETH_ADDRESS(addr);
 
@@ -1068,6 +1079,7 @@ static hashether_t *add_eth_name(const guint8 *addr, const gchar *name)
 	  return tp;
 	} else {
 	  /* replace this dummy (manuf) entry with a real name */
+	  new_one = FALSE;
 	  break;
 	}
       }
@@ -1080,10 +1092,12 @@ static hashether_t *add_eth_name(const guint8 *addr, const gchar *name)
     }
   }
 
-  memcpy(tp->addr, addr, sizeof(tp->addr));
   strncpy(tp->name, name, MAXNAMELEN);
   tp->name[MAXNAMELEN-1] = '\0';
-  tp->next = NULL;
+  if (new_one) {
+      memcpy(tp->addr, addr, sizeof(tp->addr));
+      tp->next = NULL;
+  }
   tp->is_dummy_entry = FALSE;
 
   return tp;
@@ -1131,7 +1145,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (;;) {
       /* Only the topmost 5 bytes participate fully */
       if ((etp = wka_name_lookup(addr, mask+40)) != NULL) {
-	sprintf(tp->name, "%s_%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x",
 	      etp->name, addr[5] & (0xFF >> mask));
 	tp->is_dummy_entry = TRUE;
 	return (tp->name);
@@ -1145,7 +1159,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (;;) {
       /* Only the topmost 4 bytes participate fully */
       if ((etp = wka_name_lookup(addr, mask+32)) != NULL) {
-	sprintf(tp->name, "%s_%02x:%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x",
 	      etp->name, addr[4] & (0xFF >> mask), addr[5]);
 	tp->is_dummy_entry = TRUE;
 	return (tp->name);
@@ -1159,7 +1173,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (;;) {
       /* Only the topmost 3 bytes participate fully */
       if ((etp = wka_name_lookup(addr, mask+24)) != NULL) {
-	sprintf(tp->name, "%s_%02x:%02x:%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x:%02x",
 	      etp->name, addr[3] & (0xFF >> mask), addr[4], addr[5]);
 	tp->is_dummy_entry = TRUE;
 	return (tp->name);
@@ -1171,7 +1185,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
 
     /* Now try looking in the manufacturer table. */
     if ((manufp = manuf_name_lookup(addr)) != NULL) {
-      sprintf(tp->name, "%s_%02x:%02x:%02x",
+      g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x:%02x",
 	      manufp->name, addr[3], addr[4], addr[5]);
       tp->is_dummy_entry = TRUE;
       return (tp->name);
@@ -1183,7 +1197,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (;;) {
       /* Only the topmost 2 bytes participate fully */
       if ((etp = wka_name_lookup(addr, mask+16)) != NULL) {
-	sprintf(tp->name, "%s_%02x:%02x:%02x:%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x:%02x:%02x",
 	      etp->name, addr[2] & (0xFF >> mask), addr[3], addr[4],
 	      addr[5]);
 	tp->is_dummy_entry = TRUE;
@@ -1198,7 +1212,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (;;) {
       /* Only the topmost byte participates fully */
       if ((etp = wka_name_lookup(addr, mask+8)) != NULL) {
-	sprintf(tp->name, "%s_%02x:%02x:%02x:%02x:%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x:%02x:%02x:%02x",
 	      etp->name, addr[1] & (0xFF >> mask), addr[2], addr[3],
 	      addr[4], addr[5]);
 	tp->is_dummy_entry = TRUE;
@@ -1212,7 +1226,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     for (mask = 7; mask > 0; mask--) {
       /* Not even the topmost byte participates fully */
       if ((etp = wka_name_lookup(addr, mask)) != NULL) {
-	sprintf(tp->name, "%s_%02x:%02x:%02x:%02x:%02x:%02x",
+	g_snprintf(tp->name, MAXNAMELEN, "%s_%02x:%02x:%02x:%02x:%02x:%02x",
 	      etp->name, addr[0] & (0xFF >> mask), addr[1], addr[2],
 	      addr[3], addr[4], addr[5]);
 	tp->is_dummy_entry = TRUE;
@@ -1221,7 +1235,7 @@ static gchar *eth_name_lookup(const guint8 *addr)
     }
 
     /* No match whatsoever. */
-    sprintf(tp->name, "%s", ether_to_str(addr));
+    g_snprintf(tp->name, MAXNAMELEN, "%s", ether_to_str(addr));
     tp->is_dummy_entry = TRUE;
 
   } else {
@@ -1324,7 +1338,7 @@ static void set_ipxnetent(char *path)
   if (ipxnet_p)
     rewind(ipxnet_p);
   else
-    ipxnet_p = fopen(path, "r");
+    ipxnet_p = eth_fopen(path, "r");
 }
 
 static void end_ipxnetent(void)
@@ -1413,9 +1427,7 @@ static void initialize_ipxnets(void)
    * directory as well?
    */
   if (g_ipxnets_path == NULL) {
-    g_ipxnets_path = g_malloc(strlen(get_systemfile_dir()) +
-			      strlen(ENAME_IPXNETS) + 2);
-    sprintf(g_ipxnets_path, "%s" G_DIR_SEPARATOR_S "%s",
+	g_ipxnets_path = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s",
 	    get_systemfile_dir(), ENAME_IPXNETS);
   }
 
@@ -1491,7 +1503,7 @@ static gchar *ipxnet_name_lookup(const guint addr)
 
   if ( (ipxnet = get_ipxnetbyaddr(addr)) == NULL) {
     /* unknown name */
-      sprintf(tp->name, "%X", addr);
+      g_snprintf(tp->name, MAXNAMELEN, "%X", addr);
 
   } else {
     strncpy(tp->name, ipxnet->name, MAXNAMELEN);
@@ -1553,7 +1565,7 @@ read_hosts_file (const char *hostspath)
    *  See the hosts(4) or hosts(5) man page for hosts file format
    *  (not available on all systems).
    */
-  if ((hf = fopen(hostspath, "r")) == NULL)
+  if ((hf = eth_fopen(hostspath, "r")) == NULL)
     return FALSE;
 
   while (fgetline(&line, &size, hf) >= 0) {
@@ -1628,6 +1640,15 @@ host_name_lookup_init(void) {
   }
   g_free(hostspath);
 
+  /*
+   * Load the global hosts file, if we have one.
+   */
+  hostspath = get_datafile_path(ENAME_HOSTS);
+  if (!read_hosts_file(hostspath) && errno != ENOENT) {
+    report_open_failure(hostspath, errno, FALSE);
+  }
+  g_free(hostspath);
+
 #ifdef HAVE_GNU_ADNS
   /*
    * We're using GNU ADNS, which doesn't check the system hosts file;
@@ -1635,7 +1656,7 @@ host_name_lookup_init(void) {
    */
 #ifdef WIN32
 
-  sysroot = getenv("WINDIR");
+  sysroot = getenv_utf8("WINDIR");
   if (sysroot != NULL) {
     /*
      * The file should be under WINDIR.
@@ -1698,7 +1719,7 @@ host_name_lookup_process(gpointer data _U_) {
     almsg = (adns_queue_msg_t *) cur->data;
     if (! almsg->submitted && almsg->type == AF_INET) {
       addr_bytes = (guint8 *) &almsg->ip4_addr;
-      sprintf(addr_str, "%u.%u.%u.%u.in-addr.arpa.", addr_bytes[3],
+      g_snprintf(addr_str, sizeof addr_str, "%u.%u.%u.%u.in-addr.arpa.", addr_bytes[3],
           addr_bytes[2], addr_bytes[1], addr_bytes[0]);
       /* XXX - what if it fails? */
       adns_submit (ads, addr_str, adns_r_ptr, 0, NULL, &almsg->query);
@@ -1743,7 +1764,7 @@ host_name_lookup_cleanup(void) {
     adns_queue_head = g_list_remove(adns_queue_head, qdata);
     g_free(qdata);
   }
-  
+
   if (gnu_adns_initialized)
     adns_finish(ads);
 }
@@ -1776,17 +1797,10 @@ extern const gchar *get_hostname6(struct e_in6_addr *addr)
 {
   gboolean found;
 
-#ifdef INET6
   if (!(g_resolv_flags & RESOLV_NETWORK))
     return ip6_to_str(addr);
-#ifdef SOLARIS8_INET6
-  if (IN6_IS_ADDR_LINKLOCAL((struct in6_addr*)addr) || IN6_IS_ADDR_MULTICAST((struct in6_addr*)addr))
-#else
-  if (IN6_IS_ADDR_LINKLOCAL(addr) || IN6_IS_ADDR_MULTICAST(addr))
-#endif
+  if (E_IN6_IS_ADDR_LINKLOCAL(addr) || E_IN6_IS_ADDR_MULTICAST(addr))
     return ip6_to_str(addr);
-#endif
-
   return host_name_lookup6(addr, &found);
 }
 
@@ -1794,6 +1808,7 @@ extern void add_ipv4_name(guint addr, const gchar *name)
 {
   int hash_idx;
   hashipv4_t *tp;
+  int new_one = TRUE;
 
   hash_idx = HASH_IPV4_ADDRESS(addr);
 
@@ -1809,6 +1824,7 @@ extern void add_ipv4_name(guint addr, const gchar *name)
 	  return;
 	} else {
 	  /* replace this dummy entry with the new one */
+	  new_one = FALSE;
 	  break;
 	}
       }
@@ -1823,8 +1839,10 @@ extern void add_ipv4_name(guint addr, const gchar *name)
 
   strncpy(tp->name, name, MAXNAMELEN);
   tp->name[MAXNAMELEN-1] = '\0';
-  tp->addr = addr;
-  tp->next = NULL;
+  if (new_one) {
+      tp->addr = addr;
+      tp->next = NULL;
+  }
   tp->is_dummy_entry = FALSE;
 
 } /* add_ipv4_name */
@@ -1833,6 +1851,7 @@ extern void add_ipv6_name(struct e_in6_addr *addrp, const gchar *name)
 {
   int hash_idx;
   hashipv6_t *tp;
+  int new_one = TRUE;
 
   hash_idx = HASH_IPV6_ADDRESS(*addrp);
 
@@ -1848,6 +1867,7 @@ extern void add_ipv6_name(struct e_in6_addr *addrp, const gchar *name)
 	  return;
 	} else {
 	  /* replace this dummy entry with the new one */
+	  new_one = FALSE;
 	  break;
 	}
       }
@@ -1862,26 +1882,21 @@ extern void add_ipv6_name(struct e_in6_addr *addrp, const gchar *name)
 
   strncpy(tp->name, name, MAXNAMELEN);
   tp->name[MAXNAMELEN-1] = '\0';
-  tp->addr = *addrp;
-  tp->next = NULL;
+  if (new_one) {
+      tp->addr = *addrp;
+      tp->next = NULL;
+  }
   tp->is_dummy_entry = FALSE;
 
 } /* add_ipv6_name */
 
 extern gchar *get_udp_port(guint port)
 {
-  static gchar  str[3][MAXNAMELEN];
-  static gchar *cur;
+  gchar *cur;
 
   if (!(g_resolv_flags & RESOLV_TRANSPORT)) {
-    if (cur == &str[0][0]) {
-      cur = &str[1][0];
-    } else if (cur == &str[1][0]) {
-      cur = &str[2][0];
-    } else {
-      cur = &str[0][0];
-    }
-    sprintf(cur, "%u", port);
+    cur=ep_alloc(MAXNAMELEN);
+    g_snprintf(cur, MAXNAMELEN, "%u", port);
     return cur;
   }
 
@@ -1889,20 +1904,28 @@ extern gchar *get_udp_port(guint port)
 
 } /* get_udp_port */
 
-extern gchar *get_tcp_port(guint port)
+extern gchar *get_dccp_port(guint port)
 {
-  static gchar  str[3][MAXNAMELEN];
-  static gchar *cur;
+  gchar *cur;
 
   if (!(g_resolv_flags & RESOLV_TRANSPORT)) {
-    if (cur == &str[0][0]) {
-      cur = &str[1][0];
-    } else if (cur == &str[1][0]) {
-      cur = &str[2][0];
-    } else {
-      cur = &str[0][0];
-    }
-    sprintf(cur, "%u", port);
+    cur=ep_alloc(MAXNAMELEN);
+    g_snprintf(cur, MAXNAMELEN, "%u", port);
+    return cur;
+  }
+
+  return serv_name_lookup(port, PT_DCCP);
+
+} /* get_dccp_port */
+
+
+extern gchar *get_tcp_port(guint port)
+{
+  gchar *cur;
+
+  if (!(g_resolv_flags & RESOLV_TRANSPORT)) {
+    cur=ep_alloc(MAXNAMELEN);
+    g_snprintf(cur, MAXNAMELEN, "%u", port);
     return cur;
   }
 
@@ -1912,18 +1935,11 @@ extern gchar *get_tcp_port(guint port)
 
 extern gchar *get_sctp_port(guint port)
 {
-  static gchar  str[3][MAXNAMELEN];
-  static gchar *cur;
+  gchar *cur;
 
   if (!(g_resolv_flags & RESOLV_TRANSPORT)) {
-    if (cur == &str[0][0]) {
-      cur = &str[1][0];
-    } else if (cur == &str[1][0]) {
-      cur = &str[2][0];
-    } else {
-      cur = &str[0][0];
-    }
-    sprintf(cur, "%u", port);
+    cur=ep_alloc(MAXNAMELEN);
+    g_snprintf(cur, MAXNAMELEN, "%u", port);
     return cur;
   }
 
@@ -1935,30 +1951,30 @@ extern gchar *get_sctp_port(guint port)
 const gchar *get_addr_name(address *addr)
 {
   const gchar *result;
-  
+
   result = solve_address_to_name(addr);
-  
+
   if (result!=NULL){
 	  return result;
   }
-  
+
   /* if it gets here, either it is of type AT_NONE, */
   /* or it should be solvable in address_to_str -unless addr->type is wrongly defined- */
-  
+
   if (addr->type == AT_NONE){
 	  return "NONE";
   }
-   
+
   return(address_to_str(addr));
 } /* get_addr_name */
-  
-  
+
+
 void get_addr_name_buf(address *addr, gchar *buf, guint size)
 {
   const gchar *result;
-    
+
   result = get_addr_name(addr);
-  
+
   strncpy(buf,result,size);
   buf[size]='\0';
   return;
@@ -2061,7 +2077,7 @@ extern void add_ether_byip(guint ip, const guint8 *eth)
 
   /* first check that IP address can be resolved */
 
-  if ((host = host_name_lookup(ip, &found)) == NULL)
+  if (!(g_resolv_flags & RESOLV_NETWORK) || ((host = host_name_lookup(ip, &found)) == NULL))
     return;
 
   /* ok, we can add this entry in the ethers hashtable */
@@ -2108,8 +2124,7 @@ extern guint32 get_ipxnet_addr(const gchar *name, gboolean *known)
 
 extern const gchar *get_manuf_name(const guint8 *addr)
 {
-  static gchar  str[3][MAXMANUFLEN];
-  static gchar *cur;
+  gchar *cur;
   hashmanuf_t  *manufp;
 
   if ((g_resolv_flags & RESOLV_MAC) && !eth_resolution_initialized) {
@@ -2118,14 +2133,8 @@ extern const gchar *get_manuf_name(const guint8 *addr)
   }
 
   if (!(g_resolv_flags & RESOLV_MAC) || ((manufp = manuf_name_lookup(addr)) == NULL)) {
-    if (cur == &str[0][0]) {
-      cur = &str[1][0];
-    } else if (cur == &str[1][0]) {
-      cur = &str[2][0];
-    } else {
-      cur = &str[0][0];
-    }
-    sprintf(cur, "%02x:%02x:%02x", addr[0], addr[1], addr[2]);
+    cur=ep_alloc(MAXMANUFLEN);
+    g_snprintf(cur, MAXMANUFLEN, "%02x:%02x:%02x", addr[0], addr[1], addr[2]);
     return cur;
   }
 
