@@ -2,7 +2,7 @@
  * Definitions for structures storing addresses, and for the type of
  * variables holding port-type values
  *
- * $Id: address.h 14867 2005-07-07 04:03:35Z guy $
+ * $Id: address.h 16641 2005-12-01 20:02:45Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,12 +40,13 @@ typedef enum {
   AT_ATALK,		/* Appletalk DDP */
   AT_VINES,		/* Banyan Vines */
   AT_OSI,		/* OSI NSAP */
-  AT_ARCNET,		/* ARCNET */
+  AT_ARCNET,	/* ARCNET */
   AT_FC,		/* Fibre Channel */
   AT_SS7PC,		/* SS7 Point Code */
-  AT_STRINGZ,		/* null-terminated string */
+  AT_STRINGZ,	/* null-terminated string */
   AT_EUI64,		/* IEEE EUI-64 */
-  AT_URI		/* URI/URL/URN */
+  AT_URI,		/* URI/URL/URN */
+  AT_TIPC		/* TIPC Address Zone,Subnetwork,Processor */
 } address_type;
 
 typedef struct _address {
@@ -110,12 +111,14 @@ typedef enum {
   PT_SCTP,		/* SCTP */
   PT_TCP,		/* TCP */
   PT_UDP,		/* UDP */
+  PT_DCCP,		/* DCCP */
   PT_IPX,		/* IPX sockets */
   PT_NCP,		/* NCP connection */
   PT_EXCHG,		/* Fibre Channel exchange */
   PT_DDP,		/* DDP AppleTalk connection */
   PT_SBCCS,		/* FICON */
-  PT_IDP		/* XNS IDP sockets */
+  PT_IDP,		/* XNS IDP sockets */
+  PT_TIPC		/* TIPC PORT */
 } port_type;
 
 /* Types of circuit IDs Ethereal knows about. */
@@ -125,7 +128,9 @@ typedef enum {
   CT_ISDN,		/* ISDN channel number */
   CT_X25,		/* X.25 logical channel number */
   CT_ISUP,		/* ISDN User Part CIC */
-  CT_IAX2		/* IAX2 call id */
+  CT_IAX2,		/* IAX2 call id */
+  CT_H223,		/* H.223 logical channel number */
+  CT_BICC		/* BICC Circuit identifier */
   /* Could also have ATM VPI/VCI pairs */
 } circuit_type;
 
