@@ -1,7 +1,7 @@
 /* asn1.c
  * Routines for ASN.1 BER dissection
  *
- * $Id: asn1.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: asn1.c 17578 2006-03-11 11:18:56Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -66,12 +66,9 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/tvbuff.h>
 #include <epan/asn1.h>
+#include <epan/emem.h>
 
 /*
  * NAME:        asn1_open                                   [API]
@@ -319,7 +316,7 @@ asn1_header_decode(ASN1_SCK *asn1, guint *cls, guint *con, guint *tag,
 			gboolean *defp, guint *lenp)
 {
     int   ret;
-    guint def, len;
+    guint def, len = 0;
 
     ret = asn1_id_decode (asn1, cls, con, tag);
     if (ret != ASN1_ERR_NOERROR)
@@ -900,8 +897,11 @@ asn1_oid_value_decode ( ASN1_SCK *asn1, int enc_len, subid_t **oid, guint *len)
      * but that's another matter; in any case, that would happen only
      * if we had an immensely large tvbuff....)
      */
-    if (enc_len != 0)
-	tvb_ensure_bytes_exist(asn1->tvb, asn1->offset, enc_len);
+    if (enc_len < 1) {
+	*oid = NULL;
+	return ASN1_ERR_LENGTH_MISMATCH;
+    }
+    tvb_ensure_bytes_exist(asn1->tvb, asn1->offset, enc_len);
 
     eoc = asn1->offset + enc_len;
 
@@ -1077,8 +1077,8 @@ asn1_err_to_str(int err)
 	break;
 
     default:
-	snprintf(errstrbuf, sizeof errstrbuf, "Unknown error (%d)", err);
-	errstr = errstrbuf;
+	g_snprintf(errstrbuf, sizeof errstrbuf, "Unknown error (%d)", err);
+	errstr = ep_strdup(errstrbuf);
 	break;
     }
     return errstr;
