@@ -1,5 +1,5 @@
 /*
- * $Id: dfilter.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: dfilter.c 15475 2005-08-20 16:19:22Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,16 +27,13 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef NEED_SNPRINTF_H
-#include "snprintf.h"
-#endif
-
 #include "dfilter-int.h"
 #include "syntax-tree.h"
 #include "gencode.h"
 #include "semcheck.h"
 #include "dfvm.h"
 #include <epan/epan_dissect.h>
+#include "dfilter.h"
 
 #define DFILTER_TOKEN_ID_OFFSET	1
 
@@ -53,7 +50,7 @@ int     df_lex(void);
 static void*	ParserObj = NULL;
 
 void
-dfilter_fail(char *format, ...)
+dfilter_fail(const char *format, ...)
 {
 	va_list	args;
 
@@ -63,7 +60,7 @@ dfilter_fail(char *format, ...)
 
 	va_start(args, format);
 
-	vsnprintf(dfilter_error_msg_buf, sizeof(dfilter_error_msg_buf),
+	g_vsnprintf(dfilter_error_msg_buf, sizeof(dfilter_error_msg_buf),
 			format, args);
 	dfilter_error_msg = dfilter_error_msg_buf;
 	va_end(args);
