@@ -1,5 +1,5 @@
 /*
- * $Id: semcheck.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: semcheck.c 16652 2005-12-02 13:16:58Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,6 +27,7 @@
 #include <string.h>
 
 #include "dfilter-int.h"
+#include "semcheck.h"
 #include "syntax-tree.h"
 #include "sttype-range.h"
 #include "sttype-test.h"
@@ -73,7 +74,9 @@ compatible_ftypes(ftenum_t a, ftenum_t b)
 		case FT_ETHER:
 		case FT_BYTES:
 		case FT_UINT_BYTES:
-			return (b == FT_ETHER || b == FT_BYTES || b == FT_UINT_BYTES);
+		case FT_GUID:
+		case FT_OID:
+			return (b == FT_ETHER || b == FT_BYTES || b == FT_UINT_BYTES || b == FT_GUID || b == FT_OID);
 
 		case FT_BOOLEAN:
 		case FT_FRAMENUM:
@@ -164,6 +167,8 @@ mk_fvalue_from_val_string(header_field_info *hfinfo, char *s)
 		case FT_UINT64:
 		case FT_INT64:
 		case FT_PCRE:
+		case FT_GUID:
+		case FT_OID:
 			return FALSE;
 
 		case FT_BOOLEAN:
@@ -233,6 +238,8 @@ is_bytes_type(enum ftenum type)
 		case FT_BYTES:
 		case FT_UINT_BYTES:
 		case FT_IPv6:
+		case FT_GUID:
+		case FT_OID:
 			return TRUE;
 
 		case FT_NONE:
