@@ -1,7 +1,7 @@
 /* packet-aarp.c
  * Routines for Appletalk ARP packet disassembly
  *
- * $Id: packet-aarp.c 14786 2005-06-26 19:56:52Z guy $
+ * $Id: packet-aarp.c 17316 2006-02-16 11:43:38Z guy $
  *
  * Simon Wilkinson <sxw@dcs.ed.ac.uk>
  *
@@ -32,7 +32,8 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/strutil.h>
-#include "etypes.h"
+#include <epan/emem.h>
+#include <epan/etypes.h>
 
 static int proto_aarp = -1;
 static int hf_aarp_hard_type = -1;
@@ -105,19 +106,11 @@ static const value_string hrd_vals[] = {
 static gchar *
 atalkid_to_str(const guint8 *ad) {
   gint node;
-  static gchar  str[3][16];
-  static gchar *cur;
-
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
+  gchar *cur;
 
+  cur=ep_alloc(16);
   node=ad[1]<<8|ad[2];
-  sprintf(cur, "%d.%d",node,ad[3]);
+  g_snprintf(cur, 16, "%d.%d",node,ad[3]);
   return cur;
 }
 
@@ -241,13 +234,15 @@ dissect_aarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
     if (ar_pln != 0) {
       if (AARP_PRO_IS_ATALK(ar_pro, ar_pln)) {
-        proto_tree_add_bytes_format(aarp_tree, hf_aarp_src_proto_id, tvb, spa_offset, ar_pln,
-			       spa_val,
-			       "Sender ID: %s", spa_str);
+        proto_tree_add_bytes_format_value(aarp_tree, hf_aarp_src_proto_id, tvb,
+					  spa_offset, ar_pln,
+					  spa_val,
+					  "%s", spa_str);
       } else {
-        proto_tree_add_bytes_format(aarp_tree, hf_aarp_src_proto, tvb, spa_offset, ar_pln,
-			       spa_val,
-			       "Sender protocol address: %s", spa_str);
+        proto_tree_add_bytes_format_value(aarp_tree, hf_aarp_src_proto, tvb,
+					  spa_offset, ar_pln,
+					  spa_val,
+					  "%s", spa_str);
       }
     }
 
@@ -259,13 +254,15 @@ dissect_aarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
     if (ar_pln != 0) {
       if (AARP_PRO_IS_ATALK(ar_pro, ar_pln)) {
-        proto_tree_add_bytes_format(aarp_tree, hf_aarp_dst_proto_id, tvb, tpa_offset, ar_pln,
-			       tpa_val,
-			       "Target ID: %s", tpa_str);
+        proto_tree_add_bytes_format_value(aarp_tree, hf_aarp_dst_proto_id, tvb,
+					  tpa_offset, ar_pln,
+					  tpa_val,
+					  "%s", tpa_str);
       } else {
-        proto_tree_add_bytes_format(aarp_tree, hf_aarp_dst_proto, tvb, tpa_offset, ar_pln,
-			       tpa_val,
-			       "Target protocol address: %s", tpa_str);
+        proto_tree_add_bytes_format_value(aarp_tree, hf_aarp_dst_proto, tvb,
+					  tpa_offset, ar_pln,
+					  tpa_val,
+					  "%s", tpa_str);
       }
     }
   }
