@@ -1,7 +1,7 @@
 /* packet-afp.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet-afp.h 15085 2005-07-26 09:52:06Z etxrab $
+ * $Id: packet-afp.h 15473 2005-08-20 15:19:31Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -120,4 +120,19 @@ struct aspinfo {
 	gint32  code;			/* error code/ offset NU */
 };
 
+/*
+ * value_string table for AFP command codes.
+ */
+ETH_VAR_IMPORT const value_string CommandCode_vals[];
+
+/*
+ * Value from table matching requests and responses; also passed to AFP
+ * tap for responses.
+ */
+typedef struct {
+	guint8	command;
+	guint32 frame_req, frame_res;
+	nstime_t req_time;
+} afp_request_val;
+
 #endif
