@@ -4,7 +4,7 @@
  * Copyright 2000, Ralf Hoelzer <ralf@well.com>
  * Copyright 2004, Devin Heitmueller <dheitmueller@netilla.com>
  *
- * $Id: packet-aim-messaging.c 15027 2005-07-24 02:43:41Z sahlberg $
+ * $Id: packet-aim-messaging.c 15574 2005-08-28 01:35:29Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -108,7 +108,7 @@ static const value_string rendezvous_msg_types[] = {
 	{ RENDEZVOUS_MSG_REQUEST, "Request" },
 	{ RENDEZVOUS_MSG_CANCEL, "Cancel" },
 	{ RENDEZVOUS_MSG_ACCEPT, "Accept" },
-	{ 0, "Unknown" },
+	{ 0, NULL },
 };
 
 #define CLIENTAUTORESP_REASON_BUSTED_PAYLOAD	2
@@ -117,7 +117,7 @@ static const value_string rendezvous_msg_types[] = {
 static const value_string clientautoresp_reason_types[] = {
 	{ CLIENTAUTORESP_REASON_BUSTED_PAYLOAD, "Busted Payload" },
 	{ CLIENTAUTORESP_REASON_CHANNEL_SPECIFIC, "Channel-specific" },
-	{ 0, "Unknown" },
+	{ 0, NULL },
 };
 
 #define EXTENDED_DATA_MTYPE_PLAIN 0x01
@@ -158,7 +158,7 @@ static const value_string extended_data_message_types[] = {
 	{EXTENDED_DATA_MTYPE_AUTONA, "Auto not available message"},
 	{EXTENDED_DATA_MTYPE_AUTODND, "Auto do not disturb message"},
 	{EXTENDED_DATA_MTYPE_AUTOFFC, "Auto free for chat message"},
-	{ 0, "Unknown" },
+	{ 0, NULL },
 };
 
 #define EXTENDED_DATA_MFLAG_NORMAL 0x01
@@ -348,7 +348,7 @@ static const aim_client_plugin known_client_plugins[] = {
 	{ NULL, {0x0, 0x0, 0x0, { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 } } }
 };
 
-const aim_client_plugin *aim_find_plugin ( e_uuid_t uuid)
+static const aim_client_plugin *aim_find_plugin ( e_uuid_t uuid)
 {
 	int i;
 
@@ -363,7 +363,7 @@ const aim_client_plugin *aim_find_plugin ( e_uuid_t uuid)
 	return NULL;
 }
 
-int dissect_aim_plugin(proto_tree *entry, tvbuff_t *tvb, int offset, e_uuid_t* out_plugin_uuid)
+static int dissect_aim_plugin(proto_tree *entry, tvbuff_t *tvb, int offset, e_uuid_t* out_plugin_uuid)
 {
 	const aim_client_plugin *plugin = NULL;
 	e_uuid_t uuid;
@@ -410,7 +410,7 @@ static int dissect_aim_rendezvous_extended_message(tvbuff_t *tvb, proto_tree *ms
 	proto_tree_add_item(msg_tree, hf_aim_rendezvous_extended_data_message_priority_code, tvb, offset, 2, TRUE); offset+=2;
 	text_length = tvb_get_letohs(tvb, offset);
 	proto_tree_add_item(msg_tree, hf_aim_rendezvous_extended_data_message_text_length, tvb, offset, 2, TRUE); offset+=2;
-	text = ep_tvb_get_string(tvb, offset, text_length);
+	text = tvb_get_ephemeral_string(tvb, offset, text_length);
 	proto_tree_add_text(msg_tree, tvb, offset, text_length, "Text: %s", text); offset+=text_length;
 	offset = tvb->length;
 	
