@@ -4,7 +4,7 @@
  * Copyright 2004, Jelmer Vernooij <jelmer@samba.org>
  * Copyright 2004, Devin Heitmueller <dheitmueller@netilla.com>
  *
- * $Id: packet-aim.c 13084 2005-01-17 08:13:02Z guy $
+ * $Id: packet-aim.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,6 +40,7 @@
 
 #include <epan/packet.h>
 #include <epan/strutil.h>
+#include <epan/emem.h>
 
 #include "packet-tcp.h"
 #include "packet-aim.h"
@@ -56,8 +57,6 @@
 #define CHANNEL_CLOSE_CONN  0x04
 #define CHANNEL_KEEP_ALIVE  0x05
 
-#define CLI_COOKIE		    0x01
-
 #define FAMILY_ALL_ERROR_INVALID_HEADER				   0x0001
 #define FAMILY_ALL_ERROR_SERVER_RATE_LIMIT_EXCEEDED    0x0002
 #define FAMILY_ALL_ERROR_CLIENT_RATE_LIMIT_EXCEEDED    0x0003
@@ -368,7 +367,7 @@ static int hf_aim_buddyname = -1;
 static int hf_aim_userinfo_warninglevel = -1;
 static int hf_aim_snac_error = -1;
 static int hf_aim_tlvcount = -1;
-static int hf_aim_authcookie = -1;
+static int hf_aim_version = -1;
 static int hf_aim_userclass_unconfirmed = -1;
 static int hf_aim_userclass_administrator = -1;
 static int hf_aim_userclass_aol = -1;
@@ -653,7 +652,7 @@ static void dissect_aim_newconn(tvbuff_t *tvb, packet_info *pinfo,
     col_add_fstr(pinfo->cinfo, COL_INFO, "New Connection");
 
   if (tvb_length_remaining(tvb, offset) > 0) {
-	  proto_tree_add_item(tree, hf_aim_authcookie, tvb, offset, 4, FALSE);
+	  proto_tree_add_item(tree, hf_aim_version, tvb, offset, 4, FALSE);
 	  offset+=4;
 	  offset = dissect_aim_tlv_sequence(tvb, pinfo, offset, tree, client_tlvs);
   }
@@ -666,7 +665,8 @@ static void dissect_aim_newconn(tvbuff_t *tvb, packet_info *pinfo,
 int dissect_aim_snac_error(tvbuff_t *tvb, packet_info *pinfo, 
 			     proto_tree *aim_tree)
 {
-  char *name;
+  const char *name;
+
   if ((name = match_strval(tvb_get_ntohs(tvb, 0), aim_snac_errors)) != NULL) {
      if (check_col(pinfo->cinfo, COL_INFO))
 		col_add_fstr(pinfo->cinfo, COL_INFO, name);
@@ -681,15 +681,15 @@ int dissect_aim_snac_error(tvbuff_t *tvb, packet_info *pinfo,
 int dissect_aim_userinfo(tvbuff_t *tvb, packet_info *pinfo, 
 				 int offset, proto_tree *tree) 
 {
-	offset = dissect_aim_buddyname(tvb, pinfo, offset, tree);
+    offset = dissect_aim_buddyname(tvb, pinfo, offset, tree);
 
     proto_tree_add_item(tree, hf_aim_userinfo_warninglevel, tvb, offset, 2, FALSE);
     offset += 2;
 
-	return dissect_aim_tlv_list(tvb, pinfo, offset, tree, onlinebuddy_tlvs);
+    return dissect_aim_tlv_list(tvb, pinfo, offset, tree, onlinebuddy_tlvs);
 }
 
-int dissect_aim_fnac_flags(tvbuff_t *tvb, int offset, int len, proto_item *ti, guint16 flags)
+static int dissect_aim_fnac_flags(tvbuff_t *tvb, int offset, int len, proto_item *ti, guint16 flags)
 {
 	proto_tree *entry = proto_item_add_subtree(ti, ett_aim_fnac_flags);
 	proto_tree_add_boolean(entry, hf_aim_fnac_flag_next_is_related, tvb, offset, len, flags);
@@ -774,7 +774,7 @@ static void dissect_aim_snac(tvbuff_t *tvb, packet_info *pinfo,
   }
   
   if (check_col(pinfo->cinfo, COL_INFO)) {
-	 if(subtype) {
+	 if(subtype && family) {
 		 col_set_str(pinfo->cinfo, COL_INFO, family->name);
 		 col_append_fstr(pinfo->cinfo, COL_INFO, ", %s", subtype->name);
 	 } else {
@@ -993,7 +993,7 @@ static const aim_client_capability known_client_caps[] = {
 	{ NULL, {0x0, 0x0, 0x0, { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 } } }
 };
 
-const aim_client_capability *aim_find_capability ( e_uuid_t clsid)
+static const aim_client_capability *aim_find_capability ( e_uuid_t clsid)
 {
 	int i;
 
@@ -1008,11 +1008,6 @@ const aim_client_capability *aim_find_capability ( e_uuid_t clsid)
 	return NULL;
 }
 
-int dissect_aim_tlv_value_capability_data ( proto_item *ti _U_, guint16 valueid _U_, tvbuff_t *tvb _U_, packet_info *pinfo _U_)
-{
-	return 0;
-}
-
 int dissect_aim_capability(proto_tree *entry, tvbuff_t *tvb, int offset)
 {
 	const aim_client_capability *caps = NULL;
@@ -1118,9 +1113,8 @@ int dissect_aim_tlv_value_string (proto_item *ti, guint16 valueid _U_, tvbuff_t
    gint string_len;
 
    string_len = tvb_length(tvb);
-   buf = tvb_get_string(tvb, 0, string_len);
+   buf = tvb_get_ephemeral_string(tvb, 0, string_len);
    proto_item_set_text(ti, "Value: %s", format_text(buf, string_len));
-   g_free(buf);
    return string_len;
 }
 
@@ -1189,6 +1183,11 @@ int dissect_aim_tlv_value_messageblock (proto_item *ti, guint16 valueid _U_, tvb
     
     /* Block length (includes charset and charsubset) */
     blocklen = tvb_get_ntohs(tvb, offset);
+    if (blocklen <= 4) {
+      proto_tree_add_text(entry, tvb, offset, 2, "Invalid block length: %d",
+        blocklen);
+      break;
+    }
     proto_tree_add_item(entry, hf_aim_messageblock_len, tvb, offset, 2,
 			FALSE);
     offset += 2;
@@ -1204,14 +1203,13 @@ int dissect_aim_tlv_value_messageblock (proto_item *ti, guint16 valueid _U_, tvb
     offset += 2;
 
     /* The actual message */
-    buf = tvb_get_string(tvb, offset, blocklen - 4 );
+    buf = tvb_get_ephemeral_string(tvb, offset, blocklen - 4 );
     proto_item_set_text(ti, "Message: %s",
                         format_text(buf, blocklen - 4));
     proto_tree_add_item(entry, hf_aim_messageblock_message, tvb, offset, 
 			blocklen-4,
 			FALSE);
     offset += tvb_length_remaining(tvb, offset);
-    g_free(buf);
   }
 
   return offset;
@@ -1328,8 +1326,8 @@ proto_register_aim(void)
     { &hf_aim_seqno,
       { "Sequence Number", "aim.seqno", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }
     },
-	{ &hf_aim_authcookie,
-	  { "Authentication Cookie", "aim.authcookie", FT_BYTES, BASE_DEC, NULL, 0x0, "", HFILL },
+	{ &hf_aim_version,
+	  { "Protocol Version", "aim.version", FT_BYTES, BASE_DEC, NULL, 0x0, "", HFILL },
 	},
     { &hf_aim_data_len,
       { "Data Field Length", "aim.datalen", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }
