@@ -9,7 +9,7 @@
  *   Short Message Service
  *			3GPP2 C.S0015-0		TIA/EIA-637-A
  *
- * $Id: packet-ansi_637.c 15085 2005-07-26 09:52:06Z etxrab $
+ * $Id: packet-ansi_637.c 15669 2005-09-02 09:28:30Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,11 +49,12 @@
 #include <string.h>
 
 #include "epan/packet.h"
+#include "epan/emem.h"
 
 
-static char *ansi_proto_name_tele = "ANSI IS-637-A (SMS) Teleservice Layer";
-static char *ansi_proto_name_trans = "ANSI IS-637-A (SMS) Transport Layer";
-static char *ansi_proto_name_short = "IS-637-A";
+static const char *ansi_proto_name_tele = "ANSI IS-637-A (SMS) Teleservice Layer";
+static const char *ansi_proto_name_trans = "ANSI IS-637-A (SMS) Transport Layer";
+static const char *ansi_proto_name_short = "IS-637-A";
 
 static const value_string ansi_srvc_cat_strings[] = {
     { 0x0000,	"Unknown or unspecified" },
@@ -182,7 +183,6 @@ static gint ett_params = -1;
 
 static guint32 ansi_637_trans_tele_id;
 static char ansi_637_bigbuf[1024];
-static gchar ansi_637_add_string[1024];
 /* static dissector_handle_t data_handle; */
 static dissector_table_t tele_dissector_table;
 static packet_info *g_pinfo;
@@ -312,7 +312,7 @@ tele_param_user_data(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
     guint32	required_octs;
     guint32	saved_offset;
     guint32	i;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     SHORT_DATA_CHECK(len, 2);
 
@@ -516,8 +516,8 @@ tele_param_rel_timestamp(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 off
 {
     guint8	oct;
     guint32	value = 0;
-    gchar	*str = NULL;
-    gchar	*str2 = NULL;
+    const gchar	*str = NULL;
+    const gchar	*str2 = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
@@ -550,23 +550,25 @@ tele_param_rel_timestamp(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 off
     }
 }
 
+static const value_string tele_param_pri_ind_strings[] = {
+	{ 0,	"Normal" },
+	{ 1,	"Interactive" },
+	{ 2,	"Urgent" },
+	{ 3,	"Emergency" },
+	{ 0, NULL }
+};
+
 static void
 tele_param_pri_ind(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
     oct = tvb_get_guint8(tvb, offset);
 
-    switch ((oct & 0xc0) >> 6)
-    {
-    case 0: str = "Normal"; break;
-    case 1: str = "Interactive"; break;
-    case 2: str = "Urgent"; break;
-    case 3: str = "Emergency"; break;
-    }
+    str=val_to_str((oct&0xc0)>>6, tele_param_pri_ind_strings, "Unknown");
 
     other_decode_bitfield_value(ansi_637_bigbuf, oct, 0xc0, 8);
     proto_tree_add_text(tree, tvb, offset, 1,
@@ -584,7 +586,7 @@ static void
 tele_param_priv_ind(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
@@ -656,7 +658,7 @@ static void
 tele_param_alert(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
@@ -686,7 +688,7 @@ static void
 tele_param_lang_ind(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
@@ -851,7 +853,7 @@ static void
 tele_param_disp_mode(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     EXACT_DATA_CHECK(len, 1);
 
@@ -901,7 +903,7 @@ static void (*ansi_637_tele_param_fcn[])(tvbuff_t *tvb, proto_tree *tree, guint
 };
 
 static void
-trans_param_tele_id(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_tele_id(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len)
 {
     guint32	value;
     const gchar	*str = NULL;
@@ -921,11 +923,11 @@ trans_param_tele_id(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset,
 	str,
 	value);
 
-    sprintf(add_string, " - %s (%d)", str, value);
+    g_snprintf(add_string, string_len, " - %s (%d)", str, value);
 }
 
 static void
-trans_param_srvc_cat(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_srvc_cat(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len)
 {
     guint32	value;
     const gchar	*str = NULL;
@@ -941,22 +943,21 @@ trans_param_srvc_cat(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset,
     proto_tree_add_text(tree, tvb, offset, 2,
 	str);
 
-    sprintf(add_string, " - %s (%d)", str, value);
+    g_snprintf(add_string, string_len, " - %s (%d)", str, value);
 }
 
 static void
-trans_param_address(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_address(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string _U_, int string_len _U_)
 {
     guint8	oct, oct2, num_fields, odd;
     gboolean	email_addr;
     guint32	saved_offset;
     guint32	required_octs;
     guint32	i;
-    gchar	*str;
+    const gchar	*str;
 
     SHORT_DATA_CHECK(len, 2);
 
-    add_string = add_string;
     email_addr = FALSE;
 
     oct = tvb_get_guint8(tvb, offset);
@@ -1220,16 +1221,14 @@ trans_param_address(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset,
 }
 
 static void
-trans_param_subaddress(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_subaddress(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string _U_, int string_len)
 {
     guint8	oct, oct2, num_fields;
     guint32	i;
-    gchar	*str;
+    const gchar	*str;
 
     SHORT_DATA_CHECK(len, 2);
 
-    add_string = add_string;
-
     oct = tvb_get_guint8(tvb, offset);
 
     switch ((oct & 0xe0) >> 5)
@@ -1313,7 +1312,7 @@ trans_param_subaddress(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offse
 }
 
 static void
-trans_param_bearer_reply_opt(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_bearer_reply_opt(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len)
 {
     guint8	oct;
 
@@ -1326,7 +1325,7 @@ trans_param_bearer_reply_opt(tvbuff_t *tvb, proto_tree *tree, guint len, guint32
 	ansi_637_bigbuf,
 	(oct & 0xfc) >> 2);
 
-    sprintf(add_string, " - Reply Sequence Number (%d)", (oct & 0xfc) >> 2);
+    g_snprintf(add_string, string_len, " - Reply Sequence Number (%d)", (oct & 0xfc) >> 2);
 
     other_decode_bitfield_value(ansi_637_bigbuf, oct, 0x03, 8);
     proto_tree_add_text(tree, tvb, offset, 1,
@@ -1335,10 +1334,10 @@ trans_param_bearer_reply_opt(tvbuff_t *tvb, proto_tree *tree, guint len, guint32
 }
 
 static void
-trans_param_cause_codes(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_cause_codes(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len)
 {
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     oct = tvb_get_guint8(tvb, offset);
 
@@ -1358,7 +1357,7 @@ trans_param_cause_codes(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offs
 	break;
     }
 
-    sprintf(add_string, " - Reply Sequence Number (%d)", (oct & 0xfc) >> 2);
+    g_snprintf(add_string, string_len, " - Reply Sequence Number (%d)", (oct & 0xfc) >> 2);
 
     other_decode_bitfield_value(ansi_637_bigbuf, oct, 0x03, 8);
     proto_tree_add_text(tree, tvb, offset, 1,
@@ -1423,12 +1422,10 @@ trans_param_cause_codes(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offs
 }
 
 static void
-trans_param_bearer_data(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string)
+trans_param_bearer_data(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string _U_, int string_len)
 {
     tvbuff_t	*tele_tvb;
 
-    add_string = add_string;
-
     proto_tree_add_text(tree, tvb, offset, len,
 	"Bearer Data");
 
@@ -1443,7 +1440,7 @@ trans_param_bearer_data(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offs
 
 #define	NUM_TRANS_PARAM (sizeof(ansi_trans_param_strings)/sizeof(value_string))
 static gint ett_ansi_637_trans_param[NUM_TRANS_PARAM];
-static void (*ansi_637_trans_param_fcn[])(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string) = {
+static void (*ansi_637_trans_param_fcn[])(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len) = {
     trans_param_tele_id,	/* Teleservice Identifier */
     trans_param_srvc_cat,	/* Service Category */
     trans_param_address,	/* Originating Address */
@@ -1640,7 +1637,7 @@ dissect_ansi_637_tele(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 static gboolean
 dissect_ansi_637_trans_param(tvbuff_t *tvb, proto_tree *tree, guint32 *offset)
 {
-    void (*param_fcn)(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string) = NULL;
+    void (*param_fcn)(tvbuff_t *tvb, proto_tree *tree, guint len, guint32 offset, gchar *add_string, int string_len) = NULL;
     guint8	oct;
     guint8	len;
     guint32	curr_offset;
@@ -1690,8 +1687,11 @@ dissect_ansi_637_trans_param(tvbuff_t *tvb, proto_tree *tree, guint32 *offset)
 	}
 	else
 	{
+            gchar *ansi_637_add_string;
+
+	    ansi_637_add_string = ep_alloc(1024);
 	    ansi_637_add_string[0] = '\0';
-	    (*param_fcn)(tvb, subtree, len, curr_offset, ansi_637_add_string);
+	    (*param_fcn)(tvb, subtree, len, curr_offset, ansi_637_add_string, 1024);
 
 	    if (ansi_637_add_string[0] != '\0')
 	    {
