@@ -10,7 +10,7 @@
  * Copyright 2004, Michael Lum <mlum [AT] telostech.com>
  * In association with Telos Technology Inc.
  *
- * $Id: packet-ansi_801.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-ansi_801.c 15114 2005-07-27 07:57:01Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -52,8 +52,8 @@
 #include "epan/packet.h"
 
 
-static char *ansi_proto_name = "ANSI IS-801 (Location Services (PLD))";
-static char *ansi_proto_name_short = "IS-801";
+static const char *ansi_proto_name = "ANSI IS-801 (Location Services (PLD))";
+static const char *ansi_proto_name_short = "IS-801";
 
 #define	ANSI_801_FORWARD	0
 #define	ANSI_801_REVERSE	1
@@ -80,27 +80,6 @@ static proto_tree *g_tree;
 
 /* FUNCTIONS */
 
-static gchar *
-my_match_strval(guint32 val, const value_string *vs, gint *idx)
-{
-    gint i = 0;
-
-    while (vs[i].strptr)
-    {
-	if (vs[i].value == val)
-	{
-	    *idx = i;
-	    return(vs[i].strptr);
-	}
-
-	i++;
-    }
-
-    *idx = -1;
-    return(NULL);
-}
-
-
 /* PARAM FUNCTIONS */
 
 #define	EXTRANEOUS_DATA_CHECK(edc_len, edc_max_len) \
@@ -308,7 +287,7 @@ for_request(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 {
     guint32	offset;
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gint	idx;
     proto_tree	*subtree;
     proto_item	*item;
@@ -321,7 +300,7 @@ for_request(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 	"%s :  Reserved",
 	bigbuf);
 
-    str = my_match_strval(oct & 0x0f, for_req_type_strings, &idx);
+    str = match_strval_idx(oct & 0x0f, for_req_type_strings, &idx);
 
     if (str == NULL)
     {
@@ -366,7 +345,7 @@ for_response(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 {
     guint32	offset;
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gint	idx;
     proto_tree	*subtree;
     proto_item	*item;
@@ -384,7 +363,7 @@ for_response(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 	"%s :  Unsolicited response indicator",
 	bigbuf);
 
-    str = my_match_strval(oct & 0x0f, for_rsp_type_strings, &idx);
+    str = match_strval_idx(oct & 0x0f, for_rsp_type_strings, &idx);
 
     if (str == NULL)
     {
@@ -429,7 +408,7 @@ rev_request(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 {
     guint32	offset;
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gint	idx;
     proto_tree	*subtree;
     proto_item	*item;
@@ -442,7 +421,7 @@ rev_request(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 	"%s :  Reserved",
 	bigbuf);
 
-    str = my_match_strval(oct & 0x0f, rev_req_type_strings, &idx);
+    str = match_strval_idx(oct & 0x0f, rev_req_type_strings, &idx);
 
     if (str == NULL)
     {
@@ -487,7 +466,7 @@ rev_response(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 {
     guint32	offset;
     guint8	oct;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gint	idx;
     proto_tree	*subtree;
     proto_item	*item;
@@ -505,7 +484,7 @@ rev_response(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 	"%s :  Unsolicited response indicator",
 	bigbuf);
 
-    str = my_match_strval(oct & 0x0f, rev_rsp_type_strings, &idx);
+    str = match_strval_idx(oct & 0x0f, rev_rsp_type_strings, &idx);
 
     if (str == NULL)
     {
@@ -551,7 +530,7 @@ dissect_ansi_801_for_message(tvbuff_t *tvb, proto_tree *tree)
     guint32	offset;
     guint8	oct, num_req, num_rsp;
     guint	rem_len;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     offset = 0;
     oct = tvb_get_guint8(tvb, offset);
@@ -684,7 +663,7 @@ dissect_ansi_801_rev_message(tvbuff_t *tvb, proto_tree *tree)
     guint32	offset;
     guint8	oct, num_req, num_rsp;
     guint	rem_len;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
     offset = 0;
     oct = tvb_get_guint8(tvb, offset);
