@@ -1,7 +1,7 @@
 /* packet-ap1394.c
  * Routines for Apple IP-over-IEEE 1394 packet disassembly
  *
- * $Id: packet-ap1394.c 14884 2005-07-09 00:53:17Z guy $
+ * $Id: packet-ap1394.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,7 +32,7 @@
 #include <epan/strutil.h>
 
 #include "packet-ap1394.h"
-#include "etypes.h"
+#include <epan/etypes.h>
 
 static int proto_ap1394 = -1;
 static int hf_ap1394_dst = -1;
