@@ -1,7 +1,7 @@
 /* packet-arp.c
  * Routines for ARP packet disassembly (RFC 826)
  *
- * $Id: packet-arp.c 13269 2005-02-03 22:52:20Z jmayer $
+ * $Id: packet-arp.c 16236 2005-10-16 00:33:00Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,9 +32,10 @@
 #include <epan/strutil.h>
 #include <epan/arptypes.h>
 #include <epan/addr_resolv.h>
+#include <epan/emem.h>
 #include "packet-arp.h"
-#include "etypes.h"
-#include "arcnet_pids.h"
+#include <epan/etypes.h>
+#include <epan/arcnet_pids.h>
 
 static int proto_arp = -1;
 static int hf_arp_hard_type = -1;
@@ -140,7 +141,7 @@ static const value_string atmop_vals[] = {
 #define ARP_PRO_IS_IPv4(ar_pro, ar_pln) \
 	((ar_pro) == ETHERTYPE_IP && (ar_pln) == 4)
 
-gchar *
+const gchar *
 arphrdaddr_to_str(const guint8 *ad, int ad_len, guint16 type)
 {
   if (ad_len == 0)
@@ -153,7 +154,7 @@ arphrdaddr_to_str(const guint8 *ad, int ad_len, guint16 type)
   return bytes_to_str(ad, ad_len);
 }
 
-static gchar *
+static const gchar *
 arpproaddr_to_str(const guint8 *ad, int ad_len, guint16 type)
 {
   if (ad_len == 0)
@@ -165,15 +166,12 @@ arpproaddr_to_str(const guint8 *ad, int ad_len, guint16 type)
   return bytes_to_str(ad, ad_len);
 }
 
-#define	N_ATMARPNUM_TO_STR_STRINGS	2
 #define	MAX_E164_STR_LEN		20
 
-static gchar *
+static const gchar *
 atmarpnum_to_str(const guint8 *ad, int ad_tl)
 {
   int           ad_len = ad_tl & ATMARP_LEN_MASK;
-  static gchar  str[N_ATMARPNUM_TO_STR_STRINGS][MAX_E164_STR_LEN+3+1];
-  static int    cur_idx;
   gchar        *cur;
 
   if (ad_len == 0)
@@ -183,14 +181,11 @@ atmarpnum_to_str(const guint8 *ad, int ad_tl)
     /*
      * I'm assuming this means it's an ASCII (IA5) string.
      */
-    cur_idx++;
-    if (cur_idx >= N_ATMARPNUM_TO_STR_STRINGS)
-      cur_idx = 0;
-    cur = &str[cur_idx][0];
+    cur = ep_alloc(MAX_E164_STR_LEN+3+1);
     if (ad_len > MAX_E164_STR_LEN) {
       /* Can't show it all. */
       memcpy(cur, ad, MAX_E164_STR_LEN);
-      strcpy(&cur[MAX_E164_STR_LEN], "...");
+      g_snprintf(&cur[MAX_E164_STR_LEN], 3+1, "...");
     } else {
       memcpy(cur, ad, ad_len);
       cur[ad_len + 1] = '\0';
@@ -206,7 +201,7 @@ atmarpnum_to_str(const guint8 *ad, int ad_tl)
   }
 }
 
-static gchar *
+static const gchar *
 atmarpsubaddr_to_str(const guint8 *ad, int ad_tl)
 {
   int           ad_len = ad_tl & ATMARP_LEN_MASK;
@@ -257,7 +252,7 @@ static const value_string hrd_vals[] = {
   {ARPHRD_EUI_64,   "EUI-64"               },
   {0,                NULL                  } };
 
-gchar *
+const gchar *
 arphrdtype_to_str(guint16 hwtype, const char *fmt) {
     return val_to_str(hwtype, hrd_vals, fmt);
 }
@@ -301,6 +296,11 @@ dissect_atm_number(tvbuff_t *tvb, int offset, int tl, int hf_e164,
 	}
 }
 
+/*
+ * XXX - shouldn't there be a centralized routine for dissecting NSAPs?
+ * See also "dissect_nsap()" in epan/dissectors/packet-isup.c and
+ * "print_nsap_net_buf()" and "print_nsap_net()" in epan/osi=utils.c.
+ */
 void
 dissect_atm_nsap(tvbuff_t *tvb, int offset, int len, proto_tree *tree)
 {
@@ -389,13 +389,13 @@ dissect_atmarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int         tot_len;
   proto_tree  *arp_tree;
   proto_item  *ti;
-  gchar       *op_str;
+  const gchar *op_str;
   int         sha_offset, ssa_offset, spa_offset;
   int         tha_offset, tsa_offset, tpa_offset;
   const guint8      *sha_val, *ssa_val, *spa_val;
   const guint8      *tha_val, *tsa_val, *tpa_val;
-  gchar       *sha_str, *ssa_str, *spa_str;
-  gchar       *tha_str, *tsa_str, *tpa_str;
+  const gchar       *sha_str, *ssa_str, *spa_str;
+  const gchar       *tha_str, *tsa_str, *tpa_str;
   proto_tree  *tl_tree;
   proto_item  *tl;
 
@@ -637,7 +637,7 @@ dissect_arp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int         tot_len;
   proto_tree  *arp_tree;
   proto_item  *ti;
-  gchar       *op_str;
+  const gchar *op_str;
   int         sha_offset, spa_offset, tha_offset, tpa_offset;
   const guint8      *sha_val, *spa_val, *tha_val, *tpa_val;
   gboolean    is_gratuitous;
@@ -718,7 +718,7 @@ dissect_arp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* Add sender address if sender MAC address is neither a broadcast/
        multicast address nor an all-zero address and if sender IP address
        isn't all zeroes. */
-    tvb_memcpy(tvb, (guint8 *)&ip, spa_offset, sizeof(ip));
+    ip = tvb_get_ipv4(tvb, spa_offset);
     mac = tvb_get_ptr(tvb, sha_offset, 6);
     if ((mac[0] & 0x01) == 0 && memcmp(mac, mac_allzero, 6) != 0 && ip != 0)
       add_ether_byip(ip, mac);
@@ -730,7 +730,7 @@ dissect_arp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* Do not add target address if the packet is a Request. According to the RFC,
        target addresses in requests have no meaning */
        
-    tvb_memcpy(tvb, (guint8 *)&ip, tpa_offset, sizeof(ip));
+    ip = tvb_get_ipv4(tvb, tpa_offset);
     mac = tvb_get_ptr(tvb, tha_offset, 6);
     if ((mac[0] & 0x01) == 0 && memcmp(mac, mac_allzero, 6) != 0 && ip != 0 
       && ar_op != ARPOP_REQUEST)
