@@ -1,7 +1,7 @@
 /* packet-atm.c
  * Routines for ATM packet disassembly
  *
- * $Id: packet-atm.c 13883 2005-03-23 20:25:29Z guy $
+ * $Id: packet-atm.c 17316 2006-02-16 11:43:38Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,7 +29,7 @@
 #include <stdio.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "oui.h"
+#include <epan/oui.h>
 #include <epan/addr_resolv.h>
 
 #include "packet-atm.h"
@@ -1573,9 +1573,9 @@ dissect_atm_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
       break;
     }
 
-    proto_tree_add_uint_format(atm_tree, hf_atm_aal, tvb, 0, 0,
+    proto_tree_add_uint_format_value(atm_tree, hf_atm_aal, tvb, 0, 0,
 	pinfo->pseudo_header->atm.aal,
-	"AAL: %s",
+	"%s",
 	val_to_str(pinfo->pseudo_header->atm.aal, aal_vals,
 	  "Unknown AAL (%u)"));
   }
