@@ -1,7 +1,7 @@
 /* packet-chdlc.c
  * Routines for Cisco HDLC packet disassembly
  *
- * $Id: packet-chdlc.c 14002 2005-04-01 21:04:55Z sahlberg $
+ * $Id: packet-chdlc.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,10 +28,10 @@
 
 #include <glib.h>
 #include <epan/packet.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 #include <epan/prefs.h>
-#include "chdlctypes.h"
-#include "nlpid.h"
+#include <epan/chdlctypes.h>
+#include <epan/nlpid.h>
 #include <epan/addr_resolv.h>
 #include "packet-chdlc.h"
 #include "packet-ppp.h"
@@ -298,7 +298,7 @@ dissect_slarp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   case SLARP_REQUEST:
   case SLARP_REPLY:
     if (check_col(pinfo->cinfo, COL_INFO)) {
-      tvb_memcpy(tvb, (guint8 *)&address, 4, 4);
+      address = tvb_get_ipv4(tvb, 4);
       col_add_fstr(pinfo->cinfo, COL_INFO, "%s, from %s, mask %s",
         match_strval(code, slarp_ptype_vals),
         get_hostname(address),
