@@ -1,7 +1,7 @@
 /* packet-clnp.c
  * Routines for ISO/OSI network and transport protocol packet disassembly
  *
- * $Id: packet-clnp.c 15079 2005-07-26 08:01:16Z jmayer $
+ * $Id: packet-clnp.c 17112 2006-01-28 22:34:22Z guy $
  * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -34,12 +34,15 @@
 #include <epan/prefs.h>
 #include <epan/packet.h>
 #include <epan/reassemble.h>
+#include <epan/emem.h>
+#include "packet-frame.h"
 #include "packet-osi.h"
 #include "packet-osi-options.h"
 #include "packet-isis.h"
 #include "packet-esis.h"
-#include "nlpid.h"
+#include <epan/nlpid.h>
 #include <epan/ipproto.h>
+#include <epan/expert.h>
 
 /* protocols and fields */
 
@@ -333,9 +336,9 @@ static const value_string tp_vpart_type_vals[] = {
   { 0,				NULL }
 };
 
-static int hf_cotp_vp_src_tsap = -1; 
+static int hf_cotp_vp_src_tsap = -1;
 static int hf_cotp_vp_dst_tsap = -1;
-static int hf_cotp_vp_src_tsap_bytes = -1; 
+static int hf_cotp_vp_src_tsap_bytes = -1;
 static int hf_cotp_vp_dst_tsap_bytes = -1;
 
 
@@ -365,6 +368,7 @@ static GHashTable *clnp_reassembled_table = NULL;
  */
 static GHashTable *cotp_segment_table = NULL;
 static GHashTable *cotp_reassembled_table = NULL;
+static guint16    cotp_dst_ref = 0;
 
 #define TSAP_DISPLAY_AUTO	0
 #define TSAP_DISPLAY_STRING	1
@@ -374,8 +378,8 @@ static GHashTable *cotp_reassembled_table = NULL;
 /* options */
 static guint tp_nsap_selector = NSEL_TP;
 static gboolean always_decode_transport = FALSE;
-static gboolean clnp_reassemble = FALSE;
-static gboolean cotp_reassemble = FALSE;
+static gboolean clnp_reassemble = TRUE;
+static gboolean cotp_reassemble = TRUE;
 static gint32   tsap_display = TSAP_DISPLAY_AUTO;
 
 const enum_val_t tsap_display_options[] = {
@@ -389,6 +393,12 @@ const enum_val_t tsap_display_options[] = {
 /* function definitions */
 
 #define MAX_TSAP_LEN	32
+
+static void cotp_frame_end(void)
+{
+  cotp_dst_ref = 0;
+}
+
 static gboolean is_all_printable(const guchar *stringtocheck, int length)
 {
   gboolean allprintable;
@@ -401,40 +411,35 @@ static gboolean is_all_printable(const guchar *stringtocheck, int length)
       break;
     }
   }
-  return allprintable; 
+  return allprintable;
 } /* is_all_printable */
 
 
 static gchar *print_tsap(const guchar *tsap, int length)
 {
 
-  static gchar  str[3][MAX_TSAP_LEN * 2 + 3];	/* TSAP in hex + '0x' + NULL */
-  static gchar *cur;
-  gchar tmp[3];
+  gchar *cur;
   gboolean allprintable;
+  size_t index = 0, returned_length;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
-
-
+  cur=ep_alloc(MAX_TSAP_LEN * 2 + 3);
   cur[0] = '\0';
   if (length <= 0 || length > MAX_TSAP_LEN)
-    sprintf(cur, "<unsupported TSAP length>");
+    g_snprintf(cur, MAX_TSAP_LEN * 2 + 3, "<unsupported TSAP length>");
   else {
     allprintable = is_all_printable(tsap,length);
-    if (!allprintable)
-      strcat(cur,"0x");
+    if (!allprintable) {
+      returned_length = g_snprintf(cur, MAX_TSAP_LEN * 2 + 3, "0x");
+      index += MIN(returned_length, MAX_TSAP_LEN * 2 + 3 - 1);
+    }
     while (length != 0) {
-      if (allprintable)
-	sprintf(tmp, "%c", *tsap ++);
-      else
-	sprintf(tmp, "%02x", *tsap ++);
-      strcat(cur, tmp);
+      if (allprintable) {
+        returned_length = g_snprintf(&cur[index], MAX_TSAP_LEN * 2 + 3 - index, "%c", *tsap ++);
+        index += MIN(returned_length, MAX_TSAP_LEN * 2 + 3 - index - 1 );
+      } else {
+        returned_length = g_snprintf(&cur[index], MAX_TSAP_LEN * 2 + 3 - index, "%02x", *tsap ++);
+        index += MIN(returned_length, MAX_TSAP_LEN * 2 + 3 - index - 1);
+      }
       length --;
     }
   }
@@ -646,16 +651,16 @@ static gboolean ositp_decode_var_part(tvbuff_t *tvb, int offset,
       break;
 
     case VP_SRC_TSAP:
-      /* if our preference is set to STRING or the  
+      /* if our preference is set to STRING or the
 	 TSAP is not printable, add as bytes and hidden as string;
          otherwise vice-versa */
       if (tsap_display==TSAP_DISPLAY_STRING ||
 	 (tsap_display==TSAP_DISPLAY_AUTO && is_all_printable(tvb_get_ptr(tvb,offset,length),length))) {
-     	proto_tree_add_string(tree, hf_cotp_vp_src_tsap, tvb, offset, length, 
+     	proto_tree_add_string(tree, hf_cotp_vp_src_tsap, tvb, offset, length,
 		print_tsap(tvb_get_ptr(tvb, offset, length),length));
         proto_tree_add_item_hidden(tree, hf_cotp_vp_src_tsap_bytes, tvb, offset, length, TRUE);
       } else {
-     	proto_tree_add_string_hidden(tree, hf_cotp_vp_src_tsap, tvb, offset, length, 
+     	proto_tree_add_string_hidden(tree, hf_cotp_vp_src_tsap, tvb, offset, length,
 		print_tsap(tvb_get_ptr(tvb, offset, length),length));
         proto_tree_add_item(tree, hf_cotp_vp_src_tsap_bytes, tvb, offset, length, TRUE);
       }
@@ -664,16 +669,16 @@ static gboolean ositp_decode_var_part(tvbuff_t *tvb, int offset,
       break;
 
     case VP_DST_TSAP:
-      /* if our preference is set to STRING or the  
+      /* if our preference is set to STRING or the
 	 TSAP is not printable, add as bytes and hidden as string;
-         otherwise vice-versa */      
+         otherwise vice-versa */
       if (tsap_display==TSAP_DISPLAY_STRING ||
 	 (tsap_display==TSAP_DISPLAY_AUTO && is_all_printable(tvb_get_ptr(tvb,offset,length),length))) {
-     	proto_tree_add_string(tree, hf_cotp_vp_dst_tsap, tvb, offset, length, 
+     	proto_tree_add_string(tree, hf_cotp_vp_dst_tsap, tvb, offset, length,
 		print_tsap(tvb_get_ptr(tvb, offset, length),length));
         proto_tree_add_item_hidden(tree, hf_cotp_vp_dst_tsap_bytes, tvb, offset, length, TRUE);
       } else {
-     	proto_tree_add_string_hidden(tree, hf_cotp_vp_dst_tsap, tvb, offset, length, 
+     	proto_tree_add_string_hidden(tree, hf_cotp_vp_dst_tsap, tvb, offset, length,
 		print_tsap(tvb_get_ptr(tvb, offset, length),length));
         proto_tree_add_item(tree, hf_cotp_vp_dst_tsap_bytes, tvb, offset, length, TRUE);
       }
@@ -790,7 +795,7 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 			 packet_info *pinfo, proto_tree *tree)
 {
   proto_tree *cotp_tree;
-  proto_item *ti;
+  proto_item *ti = NULL;
   guint16 dst_ref, src_ref;
   guchar  reason;
   const char *str;
@@ -851,6 +856,9 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
   offset += li + 1;
 
+  expert_add_info_format(pinfo, ti, PI_SEQUENCE, PI_CHAT,
+	  "Disconnect Request(DR): 0x%x -> 0x%x", src_ref, dst_ref);
+
   /* User data */
   call_dissector(data_handle, tvb_new_subset(tvb, offset, -1, -1), pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
@@ -922,7 +930,7 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 	fragment = TRUE;
       is_extended = FALSE;
       is_class_234 = FALSE;
-      dst_ref = 0;
+      dst_ref = cotp_dst_ref;
       break;
 
     default : /* bad TPDU */
@@ -939,11 +947,11 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
       col_append_fstr(pinfo->cinfo, COL_INFO, "DT TPDU (%u) dst-ref: 0x%04x %s",
 		 tpdu_nr,
 		 dst_ref,
-		 (fragment)? "(fragment)" : "EOT");
+		 (fragment)? "[COTP Fragment]" : "EOT");
     } else {
       col_append_fstr(pinfo->cinfo, COL_INFO, "DT TPDU (%u) %s",
 		 tpdu_nr,
-		 (fragment)? "(fragment)" : "EOT");
+		 (fragment)? "[COTP Fragment]" : "EOT");
     }
   }
 
@@ -1014,29 +1022,33 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 				     cotp_reassembled_table,
 				     fragment_length, fragment);
     if (fd_head) {
-      if (fd_head->next) {
+      if (fd_head->next && !fragment) {
 	/* This is the last packet */
 	reassembled_tvb = tvb_new_real_data(fd_head->data,
 					    fd_head->len,
 					    fd_head->len);
 	tvb_set_child_real_data_tvbuff(next_tvb, reassembled_tvb);
 	add_new_data_source(pinfo, reassembled_tvb, "Reassembled COTP");
-	
+
 	show_fragment_seq_tree(fd_head,
 			       &cotp_frag_items,
-			       cotp_tree,
+			       tree,
 			       pinfo, reassembled_tvb, &ti);
 	pinfo->fragmented = fragment;
 	next_tvb = reassembled_tvb;
+
+	cotp_dst_ref++;
+	register_frame_end_routine(cotp_frame_end);
       }
     }
     if (fragment && reassembled_tvb == NULL) {
-      proto_tree_add_text(cotp_tree, tvb, offset, -1,
+    	/* don't use -1 if fragment length is zero (throws Exception) */
+		proto_tree_add_text(cotp_tree, tvb, offset, (fragment_length) ? -1 : 0,
 			  "User data (%u byte%s)", fragment_length,
 			  plurality(fragment_length, "", "s"));
-    } 
+    }
 
-  } 
+  }
 
   if (uses_inactive_subset) {
     if (dissector_try_heuristic(cotp_is_heur_subdissector_list, next_tvb,
@@ -1048,9 +1060,9 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     }
   } else {
     /*
-     * We dissect payload if one of the following is TRUE: 
+     * We dissect payload if one of the following is TRUE:
      *
-     * - Reassembly option for COTP in preferences is unchecked 
+     * - Reassembly option for COTP in preferences is unchecked
      * - Reassembly option is checked and this packet is the last fragment
      */
     if ( (!cotp_reassemble) ||
@@ -1062,7 +1074,7 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
         call_dissector(data_handle,next_tvb, pinfo, tree);
       }
     }
-  }   
+  }
 
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
@@ -1183,6 +1195,7 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 {
   proto_tree *cotp_tree;
   proto_item *ti;
+  proto_item *item = NULL;
   guint16  dst_ref;
   guint    tpdu_nr;
   gushort  credit = 0;
@@ -1213,7 +1226,7 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     ti = proto_tree_add_item(tree, proto_cotp, tvb, offset, li + 1, FALSE);
     cotp_tree = proto_item_add_subtree(ti, ett_cotp);
     proto_tree_add_uint(cotp_tree, hf_cotp_li, tvb, offset, 1,li);
-    proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset +  1, 1, tpdu);
+    item = proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset +  1, 1, tpdu);
     if (li == LI_NORMAL_RJ)
       proto_tree_add_text(cotp_tree, tvb, offset +  1, 1,
 			  "Credit: %u", cdt);
@@ -1231,6 +1244,9 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
   offset += li + 1;
 
+  expert_add_info_format(pinfo, item, PI_SEQUENCE, PI_NOTE,
+	  "Reject(RJ): -> 0x%x", dst_ref);
+
   return offset;
 
 } /* ositp_decode_RJ */
@@ -1245,14 +1261,15 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
   proto_tree *cotp_tree = NULL;
   proto_item *ti;
+  proto_item *item = NULL;
   guint16 dst_ref, src_ref;
   guchar  class_option;
   tvbuff_t *next_tvb;
 
   src_ref = tvb_get_ntohs(tvb, offset + P_SRC_REF);
-  
-  class_option = (tvb_get_guint8(tvb, offset + P_CLASS_OPTION) >> 4 ) & 0x0F;
-  if (class_option > 4)
+
+  class_option = tvb_get_guint8(tvb, offset + P_CLASS_OPTION);
+  if (((class_option & 0xF0) >> 4) > 4) /* class 0..4 allowed */
     return -1;
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
@@ -1273,7 +1290,7 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu);
+    item = proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu);
   }
   offset += 1;
   li -= 1;
@@ -1288,9 +1305,19 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 2;
   li -= 2;
 
+  /* expert info, but only if not encapsulated in TCP/SMB */
+  /* XXX - the best way to detect seems to be if we have a port set */
+  if (pinfo->destport == 0) {
+	  expert_add_info_format(pinfo, item, PI_SEQUENCE, PI_CHAT,
+		  tpdu == CR_TPDU ? "Connection Request(CR): 0x%x -> 0x%x" : "Connection Confirm(CC): 0x%x -> 0x%x",
+		  src_ref, dst_ref);
+  }
+
   if (tree) {
     proto_tree_add_text(cotp_tree, tvb, offset, 1,
-			"Class option: 0x%02x", class_option);
+			"Class: %1u", (class_option & 0xF0) >> 4);
+    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+			"Option: %1u", (class_option & 0x0F));
   }
   offset += 1;
   li -= 1;
@@ -1322,6 +1349,7 @@ static int ositp_decode_DC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 {
   proto_tree *cotp_tree = NULL;
   proto_item *ti;
+  proto_item *item = NULL;
   guint16 dst_ref, src_ref;
 
   if (li > LI_MAX_DC)
@@ -1346,7 +1374,7 @@ static int ositp_decode_DC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu);
+    item = proto_tree_add_uint(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu);
   }
   offset += 1;
   li -= 1;
@@ -1365,6 +1393,9 @@ static int ositp_decode_DC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     ositp_decode_var_part(tvb, offset, li, 4, cotp_tree);
   offset += li;
 
+  expert_add_info_format(pinfo, item, PI_SEQUENCE, PI_CHAT,
+	  "Disconnect Confirm(DC): 0x%x -> 0x%x", src_ref, dst_ref);
+
   return offset;
 
 } /* ositp_decode_DC */
@@ -1691,6 +1722,7 @@ static gboolean dissect_ositp_internal(tvbuff_t *tvb, packet_info *pinfo,
     if (!first_tpdu) {
       if (check_col(pinfo->cinfo, COL_INFO))
         col_append_str(pinfo->cinfo, COL_INFO, ", ");
+	  expert_add_info_format(pinfo, NULL, PI_SEQUENCE, PI_NOTE, "Multiple TPDUs in one packet");
     }
     if ((li = tvb_get_guint8(tvb, offset + P_LI)) == 0) {
       if (check_col(pinfo->cinfo, COL_INFO))
@@ -2030,6 +2062,7 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (clnp_reassemble && (cnf_type & CNF_SEG_OK) &&
 	((cnf_type & CNF_MORE_SEGS) || segment_offset != 0) &&
 	tvb_bytes_exist(tvb, offset, segment_length - cnf_hdr_len) &&
+	segment_length > cnf_hdr_len &&
 	cksum_status != CKSUM_NOT_OK) {
     fd_head = fragment_add_check(tvb, offset, pinfo, du_id, clnp_segment_table,
 			   clnp_reassembled_table, segment_offset,
@@ -2098,7 +2131,7 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
           pinfo->fragmented = save_fragmented;
           return;	/* yes, it appears to be COTP or CLTP */
       }
-	
+
       break;
 
     case ER_NPDU:
@@ -2148,6 +2181,7 @@ clnp_reassemble_init(void)
 {
   fragment_table_init(&clnp_segment_table);
   reassembled_table_init(&clnp_reassembled_table);
+  cotp_dst_ref = 0;
 }
 
 static void
@@ -2240,7 +2274,7 @@ void proto_register_clnp(void)
   proto_register_field_array(proto_clnp, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
   register_dissector("clnp", dissect_clnp, proto_clnp);
-  register_heur_dissector_list("clnp", &clnp_heur_subdissector_list);  
+  register_heur_dissector_list("clnp", &clnp_heur_subdissector_list);
   register_init_routine(clnp_reassemble_init);
   register_init_routine(cotp_reassemble_init);
 
@@ -2278,7 +2312,7 @@ void proto_register_cotp(void)
         "Source address reference", HFILL}},
     { &hf_cotp_destref,
       { "Destination reference", "cotp.destref", FT_UINT16, BASE_HEX, NULL, 0x0,
-        "Destination address reference", HFILL}}, 
+        "Destination address reference", HFILL}},
     { &hf_cotp_li,
       { "Length", "cotp.li", FT_UINT8, BASE_DEC, NULL, 0x0,
         "Length Indicator, length of this header", HFILL}},
