@@ -4,7 +4,7 @@
  *
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-dccp.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-dccp.c 16821 2005-12-16 16:31:46Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -445,7 +445,7 @@ proto_register_dccp(void)
 				NULL, 0, "Client ID", HFILL }},
 
 			{ &hf_dccp_opnums_host, {
-				"Host", "dccp.opnums.host", FT_IPv4, BASE_DEC,
+				"Host", "dccp.opnums.host", FT_UINT32, BASE_DEC,
 				NULL, 0, "Host", HFILL }},
 
 			{ &hf_dccp_opnums_pid, {
