@@ -15,7 +15,7 @@
  *        offloading and you're capturing outgoing packets.
  *        This should probably be documented somewhere besides here.)
  *
- * $Id: packet-dcm.c 13452 2005-02-20 22:36:34Z ulfl $
+ * $Id: packet-dcm.c 17065 2006-01-20 21:18:18Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -64,6 +64,32 @@
  * 9 Nov 2004
  * - Fixed the heuristic code -- sometimes a conversation already exists
  * - Fixed the dissect code to display all the tags in the pdu
+ *
+ * 28 Apr 2005
+ * - fix memory leak when Assoc packet is processed repeatedly in ethereal
+ *
+ * - removed unused partial packet flag
+ *
+ * - added better support for DICOM VR
+ *	- sequences
+ *	- report actual VR in packet display, if supplied by xfer syntax 
+ *	- show that we are not displaying entire tag string with '[...]',
+ *	  some tags can hold up to 2^32-1 chars
+ *
+ * - remove my goofy attempt at trying to get access to the fragmented packets
+ *   (anyone have an idea on how to fix this ???)
+ *
+ * - process all the data in the Assoc packet even if display is off
+ *
+ * - limit display of data in Assoc packet to defined size of the data even
+ *   if reported size is larger
+ *
+ * - show the last tag in a packet as [incomplete] if we don't have all the data
+ *
+ * - added framework for reporting DICOM async negotiation (not finished)
+ *   (I'm not aware of an implementation which currently supports this)
+ *
+ * - still need to fix display of continuation packets
  */
 
 #include <stdio.h>
@@ -75,13 +101,11 @@
 
 #include "isprint.h"
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/strutil.h>
 #include <epan/conversation.h>
+#include <epan/emem.h>
 
 #include "packet-tcp.h"
 
@@ -98,6 +122,7 @@ static int hf_dcm_pdu = -1,
     hf_dcm_pdu_maxlen = -1,
     hf_dcm_impl = -1,
     hf_dcm_vers = -1,
+    hf_dcm_async = -1,
     hf_dcm_data_len = -1,
     hf_dcm_data_ctx = -1,
     hf_dcm_data_flags = -1,
@@ -136,8 +161,8 @@ struct dcmItem {
     struct dcmItem *next, *prev;
     int valid;
     guint8 id;		/* 0x20 Presentation Context */
-    guint8 *abs;	/* 0x30 Abstract syntax */
-    guint8 *xfer;	/* 0x40 Transfer syntax */
+    const guint8 *abs;	/* 0x30 Abstract syntax */
+    char *xfer;		/* 0x40 Transfer syntax */
     guint8 syntax;
 #define DCM_ILE  0x01		/* implicit, little endian */
 #define DCM_EBE  0x02           /* explicit, big endian */
@@ -150,7 +175,6 @@ struct dcmState {
     dcmItem_t *first, *last;
     guint8 pdu;		/* protocol data unit */
     guint32 tlen, clen, rlen;    /* length: total, current, remaining */
-    int partial;	/* is a partial packet */
     int coff;		/* current offset */
     int valid;		/* this conversation is a dicom conversation */
     /* enum { DCM_NONE, DCM_ASSOC, DCM_ }; */
@@ -162,7 +186,7 @@ typedef struct dcmState dcmState_t;
 struct dcmTag {
     int tag;
     int dtype;
-    char *desc;
+    const char *desc;
 #define DCM_TSTR  1
 #define DCM_TINT2 2
 #define DCM_TINT4 3
@@ -171,11 +195,11 @@ struct dcmTag {
 #define DCM_TSTAT 6  /* call dcm_rsp2str() on TINT2 */
 #define DCM_TRET  7
 #define DCM_TCMD  8
+#define DCM_SQ    9 	/* sequence */
+#define DCM_OTH   10    /* other */
 };
 typedef struct dcmTag dcmTag_t;
 
-/* static GMemChunk *dcm_assocs = NULL; */
-static GMemChunk *dcm_pdus = NULL;
 static GHashTable *dcm_tagTable = NULL;
 
 dcmItem_t * lookupCtx(dcmState_t *dd, guint8 ctx);
@@ -243,6 +267,7 @@ static dcmTag_t tagData[] = {
     {  0x200011, DCM_TSTR, "Series Num" },
     {  0x200012, DCM_TSTR, "Acq Num" },
     {  0x200013, DCM_TSTR, "Image Num" },
+    {  0x7fe00010, DCM_OTH, "Pixels" },
     {  0xfffee000, DCM_TRET, "Item Begin" },
     {  0xfffee00d, DCM_TRET, "Item End" },
     {  0xfffee0dd, DCM_TRET, "Sequence End" },
@@ -251,14 +276,6 @@ static dcmTag_t tagData[] = {
 static void
 dcm_init(void)
 {
-#ifdef notdef
-    if (dcm_assocs) g_mem_chunk_destroy(dcm_assocs);
-    dcm_assocs = g_mem_chunk_new("dcm_assocs", sizeof(struct dcmState),
-	100 * sizeof(struct dcmState), G_ALLOC_AND_FREE);
-#endif
-    if (dcm_pdus) g_mem_chunk_destroy(dcm_pdus);
-    dcm_pdus = g_mem_chunk_new("dcm_pdus", sizeof(struct dcmItem),
-	128 * sizeof(struct dcmItem), G_ALLOC_AND_FREE);
     if (NULL == dcm_tagTable) {
 	unsigned int i;
 	dcm_tagTable = g_hash_table_new(NULL, NULL);
@@ -268,17 +285,16 @@ dcm_init(void)
     }
 }
 
-dcmState_t *
+static dcmState_t *
 mkds(void)
 {
     dcmState_t *ds;
 
-    if (NULL == (ds = (dcmState_t *) malloc(sizeof(dcmState_t)))) {
+    if (NULL == (ds = (dcmState_t *) g_malloc(sizeof(dcmState_t)))) {
 	return NULL;
     }
     ds->pdu = 0;
     ds->tlen = ds->rlen = 0;
-    ds->partial = 0;
     ds->valid = TRUE;
     memset(ds->orig, 0, sizeof(ds->orig));
     memset(ds->targ, 0, sizeof(ds->targ));
@@ -287,10 +303,10 @@ mkds(void)
     return ds;
 }
 
-char *
+static const char *
 dcm_pdu2str(guint8 item)
 {
-    char *s = "";
+    const char *s = "";
     switch (item) {
     case 1: s = "ASSOC Request"; break;
     case 2: s = "ASSOC Accept"; break;
@@ -311,10 +327,10 @@ dcm_pdu2str(guint8 item)
     return s;
 }
 
-char *
+static const char *
 dcm_result2str(guint8 result)
 {
-    char *s = "";
+    const char *s = "";
     switch (result) {
     case 1:  s = "Reject Permanent"; break;
     case 2:  s = "Reject Transient"; break;
@@ -323,10 +339,10 @@ dcm_result2str(guint8 result)
     return s;
 }
 
-char *
+static const char *
 dcm_source2str(guint8 source)
 {
-    char *s = "";
+    const char *s = "";
     switch (source) {
     case 1:  s = "User"; break;
     case 2:  s = "Provider (ACSE)"; break;
@@ -336,10 +352,10 @@ dcm_source2str(guint8 source)
     return s;
 }
 
-char * 
+static const char * 
 dcm_reason2str(guint8 source, guint8 reason)
 {
-    char *s = "";
+    const char *s = "";
     if (1 == source) switch (reason) {
 	case 1:  s = "No reason"; break;
 	case 2:  s = "App Name not supported"; break;
@@ -358,10 +374,10 @@ dcm_reason2str(guint8 source, guint8 reason)
     return s;
 }
 
-char *
+static const char *
 dcm_abort2str(guint8 reason)
 {
-    char *s = "";
+    const char *s = "";
     switch (reason) {
     case 0:  s = "not specified"; break;
     case 1:  s = "unrecognized"; break;
@@ -374,10 +390,10 @@ dcm_abort2str(guint8 reason)
     return s;
 }
 
-char *
+static const char *
 dcm_PCresult2str(guint8 result)
 {
-    char *s = "";
+    const char *s = "";
     switch (result) {
     case 0:  s = "accept"; break;
     case 1:  s = "user-reject"; break;
@@ -389,10 +405,10 @@ dcm_PCresult2str(guint8 result)
     return s;
 }
 
-char *
+static const char *
 dcm_flags2str(guint8 flags)
 {
-    char *s = "";
+    const char *s = "";
     switch (flags) {
     case 0:  s = "Data,    more Fragments"; break;	/* 00 */
     case 1:  s = "Command, more Fragments"; break;      /* 01 */
@@ -403,10 +419,10 @@ dcm_flags2str(guint8 flags)
     return s;
 }
 
-char *
+static const char *
 dcm_cmd2str(guint16 us)
 {
-    char *s = "";
+    const char *s = "";
     /* there should be a better way to do this */
     switch (us) {
     case 0x0001:  s = "C-STORE-RQ"; break;
@@ -437,10 +453,10 @@ dcm_cmd2str(guint16 us)
     return s;
 }
 
-char *
+static const char *
 dcm_rsp2str(guint16 us)
 {
-    char *s = "";
+    const char *s = "";
     switch (us) {
     case 0x0000:  s = "Success"; break;
     case 0xa701:
@@ -456,12 +472,14 @@ dcm_rsp2str(guint16 us)
     return s;
 }
 
-void
+static void
 dcm_setSyntax(dcmItem_t *di, char *name)
 {
     if (NULL == di) return;
+    if (di->xfer != NULL)
+	g_free(di->xfer);	/* free prev allocated xfer */
     di->syntax = 0;
-    di->xfer = name;
+    di->xfer = g_strdup(name);
     if (0 == *name) return;
     /* this would be faster to skip the common parts, and have a FSA to 
      * find the syntax.
@@ -482,85 +500,95 @@ dcm_setSyntax(dcmItem_t *di, char *name)
 	di->syntax = DCM_ELE;	 /* explicit little endian, deflated */
 }
 
-char *
-dcm_tag2str(guint16 grp, guint16 elm, guint8 syntax, tvbuff_t *tvb, int offset, guint32 len)
+static char *
+dcm_tag2str(guint16 grp, guint16 elm, guint8 syntax, tvbuff_t *tvb, int offset, guint32 len, int vr, int tr)
 {
-    static char buf[512+1];	/* bad form ??? */
+    char *buf;
     const guint8 *vval;
-    size_t vval_len;
     char *p;
     guint32 tag, val32;
     guint16 val16;
     dcmTag_t *dtag;
-    size_t pl;
     static dcmTag_t utag = { 0, 0, "(unknown)" };
 
+#define MAX_BUF_LEN 1024
+    buf=ep_alloc(MAX_BUF_LEN);
     *buf = 0;
     if (0 == elm) {
 	if (DCM_ILE & syntax) 
 	     val32 = tvb_get_letohl(tvb, offset); 
 	else val32 = tvb_get_ntohl(tvb, offset); 
-	snprintf(buf, sizeof(buf), "Group Length 0x%x (%d)", val32, val32);
+	g_snprintf(buf, MAX_BUF_LEN, "Group Length 0x%x (%d)", val32, val32);
 	return buf;
     }
     tag = (grp << 16) | elm;
     if (NULL == (dtag = g_hash_table_lookup(dcm_tagTable, GUINT_TO_POINTER(tag))))
 	dtag = &utag;
 
-    strcpy(buf, dtag->desc);
-    g_assert(sizeof(buf) >= strlen(buf));
-    pl = sizeof(buf) - strlen(buf);
-    p = buf + strlen(buf);
+    DISSECTOR_ASSERT(MAX_BUF_LEN > strlen(dtag->desc));
+    p=buf;
+    p+=MIN(MAX_BUF_LEN-(p-buf),
+	   g_snprintf(p, MAX_BUF_LEN-(p-buf), "%s", dtag->desc));
+    if (vr > 0) {
+	vval = tvb_format_text(tvb, vr, 2);
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " [%s]", vval));
+    }
 
-    switch (dtag->dtype) {
+    switch (tr > 0 ? tr : dtag->dtype) {
     case DCM_TSTR:
     default:		/* try ascii */
-	*p++ = ' ';
 	vval = tvb_format_text(tvb, offset, len);
-	vval_len = strlen(vval);
-	strncpy(p, vval, vval_len > pl ? pl : vval_len);
-	p += len;
-	*p = 0;
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " %s", vval));
 	break;
     case DCM_TINT2:
 	if (DCM_ILE & syntax) 
 	     val16 = tvb_get_letohs(tvb, offset);
 	else val16 = tvb_get_ntohs(tvb, offset);
-	sprintf(p, " 0x%x (%d)", val16, val16);
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " 0x%x (%d)", val16, val16));
 	break;
     case DCM_TINT4:
 	if (DCM_ILE & syntax) 
 	     val32 = tvb_get_letohl(tvb, offset); 
 	else val32 = tvb_get_ntohl(tvb, offset); 
-	sprintf(p, " 0x%x (%d)", val32, val32);
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " 0x%x (%d)", val32, val32));
 	break;
     case DCM_TFLT: {
 	gfloat valf;
 	if (DCM_ILE & syntax) 
 	     valf = tvb_get_letohieee_float(tvb, offset); 
 	else valf = tvb_get_ntohieee_float(tvb, offset); 
-	sprintf(p, " (%f)", valf);
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " (%f)", valf));
 	} break;
     case DCM_TDBL: {
 	gdouble vald;
 	if (DCM_ILE & syntax) 
 	     vald = tvb_get_letohieee_double(tvb, offset); 
 	else vald = tvb_get_ntohieee_double(tvb, offset); 
-	sprintf(p, " (%f)", vald);
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " (%f)", vald));
 	} break;
     case DCM_TSTAT: /* call dcm_rsp2str() on TINT2 */
 	if (DCM_ILE & syntax) 
 	     val16 = tvb_get_letohs(tvb, offset);
 	else val16 = tvb_get_ntohs(tvb, offset);
-	sprintf(p, " 0x%x '%s'", val16, dcm_rsp2str(val16));
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " 0x%x '%s'", val16, dcm_rsp2str(val16)));
 	break;
     case DCM_TCMD:   /* call dcm_cmd2str() on TINT2 */
 	if (DCM_ILE & syntax) 
 	     val16 = tvb_get_letohs(tvb, offset);
 	else val16 = tvb_get_ntohs(tvb, offset);
-	sprintf(p, " 0x%x '%s'", val16, dcm_cmd2str(val16));
+	p+=MIN(MAX_BUF_LEN-(p-buf),
+	       g_snprintf(p, MAX_BUF_LEN-(p-buf), " 0x%x '%s'", val16, dcm_cmd2str(val16)));
 	break;
-    case DCM_TRET:   /* Retired */
+    case DCM_SQ:	/* Sequence */
+    case DCM_OTH:	/* Other BYTE, WORD, ... */
+    case DCM_TRET:	/* Retired */
 	break;
     }
     return buf;
@@ -570,85 +598,50 @@ static guint
 dcm_get_pdu_len(tvbuff_t *tvb, int offset)
 {
     guint32 len;
-#if 0
-    guint8 pdu_type;
-    guint len_rem;
-#endif
 
     len = tvb_get_ntohl(tvb, 2 + offset);
-#if 0
-    pdu_type = tvb_get_guint8(tvb, offset);
-    len_rem = tvb_ensure_length_remaining(tvb, offset);
-    /*
-     * XXX - a get_pdu_len() routine *CANNOT* scan forward arbitrarily
-     * long into a packet, as it might run past the end of the tvbuff
-     * and cause an exception.  It *must* be able to compute the PDU
-     * length from the first N bytes of the PDU, for some value of N.
-     *
-     * If there's fragmentation in DICOM, the TCP reassembly should be
-     * done at the *fragment* level - i.e., do reassembly of a *fragment*
-     * at the TCP layer - and reassembly of the fragments should be done
-     * in the DICOM dissector.
-     */
-    if (4 == pdu_type) {
-	guint8 frag;
-	/* old swamp */
-  	frag = tvb_get_guint8(tvb, 11 + offset);
-	while (0 == (0x2 & frag)) { /* there are more fragments */ 
-	    guint32 nlen;
-	    if (len_rem < len + 6 + 12) {
-		len += 12;	    /* fixed part of next pdu */
-		break;
-	    }
-	    len += 6;
-	    nlen = tvb_get_ntohl(tvb, 2 + offset + len);
-	    frag = tvb_get_guint8(tvb, 11 + offset + len);
-	    len += nlen;
-	}
-    }
-#endif
     return len + 6;		/* add in fixed header part */
 }
 
-void 
+static void 
 dissect_dcm_assoc(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb, int offset)
 { 
-    proto_tree *dcm_tree;
+    proto_tree *dcm_tree = NULL;
     dcmItem_t *di = NULL;
     guint8 id, *name, result;
     int reply = 0;
 
+    if (!ti)
+	return;
+
     dcm_tree = proto_item_add_subtree(ti, ett_assoc);
     while (-1 < offset && offset < (int) dcm_data->clen) {
 	guint16 len;
 	guint32 mlen;
 	id = tvb_get_guint8(tvb, offset);
 	len = tvb_get_ntohs(tvb, 2 + offset);
-	proto_tree_add_uint_format(dcm_tree, hf_dcm_pdi, tvb,
-	    offset, 4+len, id, "Item 0x%x (%s)", id, dcm_pdu2str(id));
+	if (ti)
+	    proto_tree_add_uint_format(dcm_tree, hf_dcm_pdi, tvb,
+	        offset, 4+len, id, "Item 0x%x (%s)", id, dcm_pdu2str(id));
 	offset += 4;
 	switch (id) {
 	case 0x10:		/* App context */
-	    name = g_malloc(1 + len);
-	    tvb_memcpy(tvb, name, offset, len);
-	    *(name + len) = 0;
-	    proto_tree_add_string(dcm_tree, hf_dcm_pdi_name, tvb, offset, len, name);
-	    g_free(name);
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_pdi_name, tvb, offset, len > 65 ? 65 : len, FALSE);
 	    offset += len;
 	    break;
 	case 0x30:		/* Abstract syntax */
-	    dcm_data->last->abs = name = g_malloc(1 + len);
-	    tvb_memcpy(tvb, name, offset, len);
-	    *(name + len) = 0;
-	    proto_tree_add_string(dcm_tree, hf_dcm_pdi_syntax, tvb, offset, len, name);
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_pdi_syntax, tvb, offset, len > 65 ? 65 : len, FALSE);
 	    offset += len;
 	    break;
 	case 0x40:		/* Transfer syntax */
-	    name = g_malloc(1 + len);
-	    tvb_memcpy(tvb, name, offset, len);
-	    *(name + len) = 0;
-	    proto_tree_add_string(dcm_tree, hf_dcm_pdi_syntax, tvb, offset, len, name);
-	    if (reply && di && di->valid) dcm_setSyntax(di, name);
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_pdi_syntax, tvb, offset, len > 65 ? 65 : len, FALSE);
+	    if (reply && di && di->valid) {
+		name = tvb_get_ephemeral_string(tvb, offset, len);
+		dcm_setSyntax(di, name);
+	    }
 	    reply = 0;
 	    offset += len;
 	    break;
@@ -656,9 +649,11 @@ dissect_dcm_assoc(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb, int offse
 	    id = tvb_get_guint8(tvb, offset);
 	    di = lookupCtx(dcm_data, id);
 	    if (!di->valid) {
-		di = g_chunk_new(dcmItem_t, dcm_pdus);
+		di = se_alloc(sizeof(struct dcmItem));
 		di->id = id;
 		di->valid = 1;
+		di->xfer = NULL;
+		di->syntax = DCM_UNK;
 		di->next = di->prev = NULL;
 		if (dcm_data->last) {
 		    dcm_data->last->next = di;
@@ -667,16 +662,19 @@ dissect_dcm_assoc(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb, int offse
 		} else 
 		    dcm_data->first = dcm_data->last = di;
 	    }
-	    proto_tree_add_item(dcm_tree, hf_dcm_pctxt, tvb, offset, 1, FALSE);
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_pctxt, tvb, offset, 1, FALSE);
 	    offset += 4;
 	    break;
 	case 0x21:		/* Presentation context reply */
 	    id = tvb_get_guint8(tvb, offset);
 	    result = tvb_get_guint8(tvb, 2 + offset);
-	    proto_tree_add_item(dcm_tree, hf_dcm_pctxt, tvb, offset, 1, FALSE);
-	    proto_tree_add_uint_format(dcm_tree, hf_dcm_pcres, tvb, 
-		2 + offset, 1, result, 
-		"Result 0x%x (%s)", result, dcm_PCresult2str(result));
+	    if (ti) {
+		proto_tree_add_item(dcm_tree, hf_dcm_pctxt, tvb, offset, 1, FALSE);
+		proto_tree_add_uint_format(dcm_tree, hf_dcm_pcres, tvb, 
+		    2 + offset, 1, result, 
+		    "Result 0x%x (%s)", result, dcm_PCresult2str(result));
+	    }
 	    if (0 == result) {
 		reply = 1;
 		di = lookupCtx(dcm_data, id);
@@ -688,23 +686,22 @@ dissect_dcm_assoc(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb, int offse
 	    break;
 	case 0x51:		/* Max length */
 	    mlen = tvb_get_ntohl(tvb, offset);
-	    proto_tree_add_item(dcm_tree, hf_dcm_pdu_maxlen, tvb, offset, 4, FALSE);
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_pdu_maxlen, tvb, offset, 4, FALSE);
 	    offset += len;
 	    break;
-	case 0x52:		/* UID? */
-	    name = g_malloc(1 + len);
-	    tvb_memcpy(tvb, name, offset, len);
-	    *(name + len) = 0;
-	    proto_tree_add_string(dcm_tree, hf_dcm_impl, tvb, offset, len, name);
-	    g_free(name);
+	case 0x52:		/* UID */
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_impl, tvb, offset, len > 65 ? 65 : len, FALSE);
 	    offset += len;
 	    break;
-	case 0x55:		/* version? */
-	    name = g_malloc(1 + len);
-	    tvb_memcpy(tvb, name, offset, len);
-	    *(name + len) = 0;
-	    proto_tree_add_string(dcm_tree, hf_dcm_vers, tvb, offset, len, name);
-	    g_free(name);
+	case 0x55:		/* version */
+	    if (ti)
+		proto_tree_add_item(dcm_tree, hf_dcm_vers, tvb, offset, len > 17 ? 17 : len, FALSE);
+	    offset += len;
+	    break;
+	case 0x53:		/* async negotion */
+	    /* hf_dcm_async */
 	    offset += len;
 	    break;
 	default:
@@ -748,10 +745,10 @@ lookupCtx(dcmState_t *dd, guint8 ctx)
 #define D_LEN4   5
 #define D_VALUE  6
 
-void 
+static void 
 dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 {
-    int len, offset, toffset, state;
+    int len, offset, toffset, state, vr, tr;
     proto_tree *dcm_tree;
     dcmItem_t *di;
     guint8 ctx, syntax = DCM_UNK;
@@ -762,13 +759,26 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
     proto_tree_add_item(dcm_tree, hf_dcm_data_len, tvb, 6, 4, FALSE);
     ctx = tvb_get_guint8(tvb, 10);
     di = lookupCtx(dcm_data, ctx);
-    /* proto_tree_add_item(dcm_tree, hf_dcm_data_ctx, tvb, 10, 1, FALSE); */
+    /*
+     * XXX - telling the user to "click on ASSOC request" is bogus if we
+     * have already identified the ASSOC request and can connect it to
+     * this mnessage; if clicking on a request prior to this one causes
+     * additional state information to be set up that would affect the
+     * dissection of this request, we should set up that state *at the
+     * time we dissect that request*, if possible, and if clicking on it
+     * doesn't change any state, clicking on the request doesn't convey
+     * any additional information.
+     */
     proto_tree_add_uint_format(dcm_tree, hf_dcm_data_ctx, tvb, 10, 1, 
-	ctx, "Context 0x%x (%s)", ctx, di->xfer);
+	ctx, "Context 0x%x (%s)", ctx,
+	di->xfer == NULL ? "not found - click on ASSOC Request" :
+			   di->xfer);
+    if (DCM_UNK == di->syntax)
+	return;
     len = offset = toffset = 11;
     state = D_HEADER;
     nlen = 1;
-    while (len + nlen <= dcm_data->clen) {
+    while (len + nlen <= dcm_data->tlen && len + nlen <= dcm_data->clen) {
     switch (state) {
     case D_HEADER: {
 	guint8 flags;
@@ -793,6 +803,7 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 	nlen = 4;
 	} break;		/* don't fall through -- check length */
     case D_TAG: {
+	vr = tr = 0;
 	if (DCM_ILE & syntax) {
 	    grp = tvb_get_letohs(tvb, offset);
 	    elm = tvb_get_letohs(tvb, offset+2);
@@ -811,20 +822,44 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 	} break;		/* don't fall through -- check length */
     case D_VR:  {
 	guint8 V, R;
+	vr = offset;
 	V = tvb_get_guint8(tvb, offset); offset++;
 	R = tvb_get_guint8(tvb, offset); offset++;
 	len += 2;
 	/* 4byte lengths OB, OW, OF, SQ, UN, UT */
 	state = D_LEN2;
 	nlen = 2;
-	if ((('O' == V) && ('B' == R || 'W' == R || 'F' == R))
-	    || (('U' == V) && ('N' == R || 'T' == R))
-	    || ('S' == V && 'Q' == R)) {
+	if ((('O' == V) && ('B' == R || 'W' == R || 'F' == R) && (tr = DCM_OTH))
+	    || (('U' == V) && ('N' == R || (('T' == R) && (tr = DCM_TSTR))))
+	    || ('S' == V && 'Q' == R && (tr = DCM_SQ))) {
 	    state = D_LEN4;
 	    offset += 2;	/* skip 00 (2 bytes) */
 	    len += 2;
 	    nlen = 4;
-	}
+	} else if ('F' == V && 'L' == R) {
+	    tr = DCM_TFLT;
+	} else if ('F' == V && 'D' == R) {
+	    tr = DCM_TDBL;
+	} else if (('S' == V && 'L' == R) || ('U' == V && 'L' == R)) {
+	    tr = DCM_TINT4;
+	} else if (('S' == V && 'S' == R) || ('U' == V && 'S' == R)) {
+	    tr = DCM_TINT2;
+	} else if ('A' == V && 'T' == R) {
+	    tr = DCM_OTH;
+	} else
+	    tr = DCM_TSTR;
+/* 
+	else if (('A' == V && ('E' == R || 'S' == R))
+	    || ('C' == V && 'S' == R)
+	    || ('D' == V && ('A' == R || 'S' == R || 'T' == R))
+	    || ('I' == V && 'S' == R)
+	    || ('L' == V && ('O' == R || 'T' == R))
+	    || ('P' == V && 'N' == R)
+	    || ('S' == V && ('H' == R ||| 'T' == R))
+	    || ('T' == V && 'M' == R)
+	    || ('U' == V && ('I' == R || 'T' == R)))
+	    tr = DCM_TSTR;
+ */
 	} break;		/* don't fall through -- check length */
     case D_LEN2: {
 	if (DCM_ILE & syntax)	/* is it LE */
@@ -854,7 +889,7 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 	    proto_tree_add_bytes_format(dcm_tree, hf_dcm_data_tag, tvb, 
 		toffset, totlen, val, 
 		"(%04x,%04x) %-8x %s", grp, elm, tlen, 
-		    dcm_tag2str(grp, elm, syntax, tvb, offset, 0));
+		    dcm_tag2str(grp, elm, syntax, tvb, offset, 0, vr, tr));
 	    tlen = 0;
 	/* } else if (0xfffe == grp) { */ /* need to make a sub-tree here */
 	} else {
@@ -863,7 +898,7 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 	    proto_tree_add_bytes_format(dcm_tree, hf_dcm_data_tag, tvb, 
 		toffset, totlen, val, 
 		"(%04x,%04x) %-8x %s", grp, elm, tlen, 
-		    dcm_tag2str(grp, elm, syntax, tvb, offset, tlen));
+		    dcm_tag2str(grp, elm, syntax, tvb, offset, tlen, vr, tr));
 	}
 	len += tlen;
 	offset += tlen;
@@ -872,6 +907,15 @@ dissect_dcm_data(dcmState_t *dcm_data, proto_item *ti, tvbuff_t *tvb)
 	} break;
     }
     }
+    if (D_VALUE == state) {
+	const guint8 *val;
+	int totlen = (offset - toffset);
+	val = tvb_get_ptr(tvb, toffset, totlen);
+	proto_tree_add_bytes_format(dcm_tree, hf_dcm_data_tag, tvb,
+	    toffset, totlen, val,
+	    "(%04x,%04x) %-8x %s [incomplete]", grp, elm, tlen, 
+		dcm_tag2str(grp, elm, syntax, tvb, offset, tlen, vr, tr));
+    }
 }
 
 /* 
@@ -947,7 +991,7 @@ dissect_dcm_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     dcmState_t *dcm_data;
     proto_tree *dcm_tree;
     conversation_t *conv;
-    char *buf;
+    char *buf=NULL;
     int offset = 0;
 
     if (NULL == (conv = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
@@ -971,22 +1015,21 @@ dissect_dcm_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     dcm_data->pdu = tvb_get_guint8(tvb, 0);
     dcm_data->tlen = tvb_get_ntohl(tvb, 2) + 6;
     dcm_data->clen = tvb_reported_length(tvb);
-    /* if (dcm_data->clen < dcm_data->tlen) dcm_data->partial = 1; */
 
     switch (dcm_data->pdu) {
     case 1:					/* ASSOC Request */
 	tvb_memcpy(tvb, dcm_data->orig, 10, 16);
 	tvb_memcpy(tvb, dcm_data->targ, 26, 16);
 	dcm_data->orig[AEEND] = dcm_data->targ[AEEND] = 0;
-	buf = g_malloc(128);
-	snprintf(buf, 128, "DCM ASSOC Request %s <-- %s",
+	buf = ep_alloc(128);
+	g_snprintf(buf, 128, "DCM ASSOC Request %s <-- %s",
 	    dcm_data->orig, dcm_data->targ);
 	offset = 74;
 	break;
     case 2: 				/* ASSOC Accept */
 	tvb_memcpy(tvb, dcm_data->resp, 26, 16);
-	buf = g_malloc(128);
-	snprintf(buf, 128, "DCM ASSOC Accept %s <-- %s (%s)",
+	buf = ep_alloc(128);
+	g_snprintf(buf, 128, "DCM ASSOC Accept %s <-- %s (%s)",
 	    dcm_data->orig, dcm_data->targ, dcm_data->resp);
 	offset = 74; 
 	break;
@@ -994,8 +1037,8 @@ dissect_dcm_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	dcm_data->result = tvb_get_guint8(tvb, 7);
 	dcm_data->source = tvb_get_guint8(tvb, 8);
 	dcm_data->reason = tvb_get_guint8(tvb, 9);
-	buf = g_malloc(128);
-	snprintf(buf, 128, "DCM ASSOC Reject %s <-- %s %s %s %s",
+	buf = ep_alloc(128);
+	g_snprintf(buf, 128, "DCM ASSOC Reject %s <-- %s %s %s %s",
 	    dcm_data->orig, dcm_data->targ,
 	    dcm_result2str(dcm_data->result),
 	    dcm_source2str(dcm_data->source),
@@ -1004,32 +1047,28 @@ dissect_dcm_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	break;
     case 4:					/* DATA */
 	offset = 6; 
-	buf = g_malloc(128);
-	strcpy(buf, "DCM Data");
+	buf="DCM Data";
 	break;
     case 5:					/* RELEASE Request */
-	buf = g_malloc(128);
-	strcpy(buf, "DCM RELEASE Request");
+	buf="DCM RELEASE Request";
 	offset = 6; 
 	break;
     case 6:					/* RELEASE Response */
-	buf = g_malloc(128);
-	strcpy(buf, "DCM RELEASE Response");
+	buf="DCM RELEASE Response";
 	offset = 6; 
 	break;
     case 7:					/* ABORT */
 	dcm_data->source = tvb_get_guint8(tvb, 8);
 	dcm_data->reason = tvb_get_guint8(tvb, 9);
-	buf = g_malloc(128);
-	snprintf(buf, 128, "DCM ABORT %s <-- %s %s %s", 
+	buf = ep_alloc(128);
+	g_snprintf(buf, 128, "DCM ABORT %s <-- %s %s %s", 
 	    dcm_data->orig, dcm_data->targ,
 	    (dcm_data->source == 1) ? "USER" :
 		(dcm_data->source == 2) ? "PROVIDER" : "",
 	    dcm_data->source == 1 ? dcm_abort2str(dcm_data->reason) : "");
 	break;
     default:
-	buf = g_malloc(128);
-	strcpy(buf, "DCM Continuation");
+	buf="DCM Continuation";
 	offset = -1;				/* cannot continue parsing */
 	break;
     }
@@ -1066,6 +1105,8 @@ dissect_dcm_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
 /* Continue adding tree items to process the packet here */
+    } else if (1 == dcm_data->pdu || 2 == dcm_data->pdu) {
+	dissect_dcm_assoc(dcm_data, NULL, tvb, offset);
     }
 
 /* If this protocol has a sub-dissector call it here, see section 1.8 */
@@ -1105,6 +1146,8 @@ static hf_register_info hf[] = {
 	FT_STRING, BASE_NONE, NULL, 0, "", HFILL } },
     { &hf_dcm_vers, { "Version", "dcm.pdi.version",
 	FT_STRING, BASE_NONE, NULL, 0, "", HFILL } },
+    { &hf_dcm_async, { "Asynch", "dcm.pdi.async",
+	FT_STRING, BASE_NONE, NULL, 0, "", HFILL } },
     { &hf_dcm_data_len, { "DATA LENGTH", "dcm.data.len",
 	FT_UINT32, BASE_HEX, NULL, 0, "", HFILL } },
     { &hf_dcm_data_ctx, { "Data Context", "dcm.data.ctx",
