@@ -1,7 +1,7 @@
 /* packet-dcom-cba-acco.c
  * Routines for DCOM CBA
  *
- * $Id: packet-dcom-cba-acco.c 15095 2005-07-26 18:02:42Z ulfl $
+ * $Id: packet-dcom-cba-acco.c 17316 2006-02-16 11:43:38Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,8 +33,10 @@
 
 #include <glib.h>
 #include <epan/packet.h>
+#include <epan/expert.h>
 #include "packet-dcerpc.h"
 #include "packet-dcom.h"
+#include "packet-dcom-cba-acco.h"
 
 static int hf_cba_acco_opnum = -1;
 
@@ -695,14 +697,14 @@ dissect_ICBAAccoServerSRT_ConnectCR_rqst(tvbuff_t *tvb, int offset,
 
     /* add flags subtree */
 	u32Flags = tvb_get_guint8(tvb, offset);
-	item = proto_tree_add_uint_format(tree, hf_cba_acco_serversrt_cr_flags, 
+	item = proto_tree_add_uint_format_value(tree, hf_cba_acco_serversrt_cr_flags, 
 		tvb, offset, 4, u32Flags,
-		"Flags: 0x%02x (%s, %s)", u32Flags,
-		(u32Flags & 0x1) ? "Timestamped" : "not Timestamped",
-		(u32Flags & 0x2) ? "Reconfigure" : "not Reconfigure");
+		"0x%02x (%s, %s)", u32Flags,
+		(u32Flags & 0x2) ? "Reconfigure" : "not Reconfigure",
+		(u32Flags & 0x1) ? "Timestamped" : "not Timestamped");
 	flags_tree = proto_item_add_subtree(item, ett_cba_acco_serversrt_cr_flags);
-    proto_tree_add_boolean(flags_tree, hf_cba_acco_serversrt_cr_flags_timestamped, tvb, offset, 4, u32Flags);
     proto_tree_add_boolean(flags_tree, hf_cba_acco_serversrt_cr_flags_reconfigure, tvb, offset, 4, u32Flags);
+    proto_tree_add_boolean(flags_tree, hf_cba_acco_serversrt_cr_flags_timestamped, tvb, offset, 4, u32Flags);
 	offset += 4;
 
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
@@ -1209,6 +1211,7 @@ dissect_CBA_Connection_Data(tvbuff_t *tvb,
 	proto_tree *sub_tree;
 	proto_item *conn_data_item = NULL;
 	proto_tree *conn_data_tree = NULL;
+	proto_item *item = NULL;
 	guint16	u16Len;
 	guint32	u32ID;
 	guint8  u8QC;
@@ -1216,6 +1219,10 @@ dissect_CBA_Connection_Data(tvbuff_t *tvb,
 	guint16	u16HdrLen;
 	int offset = 0;
 	int offset_hole;
+	gboolean qc_reported = FALSE;
+	int qc_good = 0;
+	int qc_uncertain = 0;
+	int qc_bad = 0;
 /*	guint32 bTimeStamped = 0;*/
 
 	/*** ALL data in this buffer is NOT aligned and always little endian ordered ***/
@@ -1246,8 +1253,10 @@ dissect_CBA_Connection_Data(tvbuff_t *tvb,
 	u16CountFix = u16Count;
 
 	/* update column info now */
+#if 0
     if (check_col(pinfo->cinfo, COL_INFO))
       col_append_fstr(pinfo->cinfo, COL_INFO, " Cnt=%u", u16Count);
+#endif
 
 	/* is this an OnDataChanged buffer format (version), we know? */
 	if (u8Version != CBA_MRSH_VERSION_DCOM && 
@@ -1320,11 +1329,31 @@ dissect_CBA_Connection_Data(tvbuff_t *tvb,
 
 		u8QC = tvb_get_guint8 (tvb, offset);
 		if (sub_tree) {
-			proto_tree_add_item(sub_tree, hf_cba_acco_qc, tvb, offset, 1, TRUE);
+			item = proto_tree_add_item(sub_tree, hf_cba_acco_qc, tvb, offset, 1, TRUE);
 		}
 		offset += 1;
 		u16HdrLen += 1;
 
+		if(	u8QC != 0x80 && /* GoodNonCascOk */
+			u8QC != 0x1C &&	/* BadOutOfService (usually permanent, so don't report for every frame) */
+			qc_reported == 0) {
+			expert_add_info_format(pinfo, item, PI_RESPONSE_CODE, PI_CHAT, "%s QC: %s",
+				u8Version == CBA_MRSH_VERSION_DCOM ? "DCOM" : "SRT",
+				val_to_str(u8QC, cba_acco_qc_vals, "Unknown (0x%02x)"));
+			qc_reported = 0;
+		}
+
+		switch(u8QC >> 6) {
+		case(00):
+			qc_bad++;
+			break;
+		case(01):
+			qc_uncertain++;
+			break;
+		default:
+			qc_good++;
+		}
+
 		/* user data length is item length without headers */
 		u16DataLen = u16Len - u16HdrLen;
 
@@ -1363,6 +1392,10 @@ dissect_CBA_Connection_Data(tvbuff_t *tvb,
 	}
 	proto_item_set_len(conn_data_item, offset);
 
+    if (check_col(pinfo->cinfo, COL_INFO))
+      col_append_fstr(pinfo->cinfo, COL_INFO, ", QC (G:%u,U:%u,B:%u)", 
+		qc_good, qc_uncertain, qc_bad);
+
 	return offset;
 }
 
