@@ -1,7 +1,7 @@
 /* packet-dcom-dispatch.c
  * Routines for DCOM IDispatch
  *
- * $Id: packet-dcom-dispatch.c 12777 2004-12-19 14:08:03Z ulfl $
+ * $Id: packet-dcom-dispatch.c 17839 2006-04-07 22:03:58Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,6 +33,7 @@
 #include <epan/packet.h>
 #include "packet-dcerpc.h"
 #include "packet-dcom.h"
+#include "packet-dcom-dispatch.h"
 
 
 static int hf_dispatch_opnum = -1;
@@ -42,7 +43,6 @@ static int hf_dispatch_name = -1;
 static int hf_dispatch_names = -1;
 static int hf_dispatch_lcid = -1;
 static int hf_dispatch_id = -1;
-static int hf_dispatch_flags2 = -1;
 
 static int hf_dispatch_arg = -1;
 static int hf_dispatch_args = -1;
@@ -52,8 +52,23 @@ static int hf_dispatch_varrefidx = -1;
 static int hf_dispatch_varrefarg = -1;
 
 static int hf_dispatch_varresult = -1;
+static int hf_dispatch_code = -1;
+static int hf_dispatch_reserved16 = -1;
+static int hf_dispatch_source = -1;
+static int hf_dispatch_description = -1;
+static int hf_dispatch_help_file = -1;
+static int hf_dispatch_help_context = -1;
+static int hf_dispatch_reserved32 = -1;
+static int hf_dispatch_deferred_fill_in = -1;
+static int hf_dispatch_arg_err = -1;
+
+static int hf_dispatch_tinfo = -1;
+static int hf_dispatch_itinfo = -1;
+static int hf_dispatch_dispparams = -1;
+static int hf_dispatch_excepinfo = -1;
+static int hf_dispatch_scode = -1;
+
 
-static gint ett_dispatch_flags = -1;
 static int hf_dispatch_flags = -1;
 static int hf_dispatch_flags_method = -1;
 static int hf_dispatch_flags_propget = -1;
@@ -65,6 +80,9 @@ static int hf_dispatch_flags_propputref = -1;
 #define DISPATCH_FLAGS_PROPPUT		4
 #define DISPATCH_FLAGS_PROPPUTREF	8
 
+static gint ett_dispatch_flags = -1;
+static gint ett_dispatch_params = -1;
+static gint ett_dispatch_excepinfo = -1;
 
 static e_uuid_t uuid_dispatch = { 0x00020400, 0x0000, 0x0000, { 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46 } };
 static guint16  ver_dispatch = 0;
@@ -73,14 +91,85 @@ static int proto_dispatch = -1;
 
 
 
-static int
+int
+dissect_IDispatch_GetTypeInfoCount_resp(tvbuff_t *tvb, int offset,
+	packet_info *pinfo, proto_tree *tree, guint8 *drep)
+{
+    guint32 u32TInfo;
+	guint32 u32HResult;
+
+
+    offset = dissect_dcom_that(tvb, offset, pinfo, tree, drep);
+
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
+                        hf_dispatch_tinfo, &u32TInfo);
+
+	/* HRESULT of call */
+	offset = dissect_dcom_HRESULT(tvb, offset, pinfo, tree, drep, 
+                        &u32HResult);
+
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+	  col_append_fstr(pinfo->cinfo, COL_INFO, " -> %s", 
+	  val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%08x)") );
+	}
+
+  	return offset;
+}
+
+int
+dissect_IDispatch_GetTypeInfo_rqst(tvbuff_t *tvb, int offset,
+	packet_info *pinfo, proto_tree *tree, guint8 *drep)
+{
+    guint32 u32TInfo;
+	guint32 u32Lcid;
+
+    offset = dissect_dcom_this(tvb, offset, pinfo, tree, drep);
+
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
+                        hf_dispatch_tinfo, &u32TInfo);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
+						hf_dispatch_lcid, &u32Lcid);
+
+	return offset;
+}
+
+
+int
+dissect_IDispatch_GetTypeInfo_resp(tvbuff_t *tvb, int offset,
+	packet_info *pinfo, proto_tree *tree, guint8 *drep)
+{
+	guint32 u32HResult;
+	guint32	u32Pointer;
+
+
+    offset = dissect_dcom_that(tvb, offset, pinfo, tree, drep);
+
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
+				&u32Pointer);
+	if (u32Pointer) {
+        offset = dissect_dcom_MInterfacePointer(tvb, offset, pinfo, tree, drep, hf_dispatch_itinfo);
+    }
+
+	/* HRESULT of call */
+	offset = dissect_dcom_HRESULT(tvb, offset, pinfo, tree, drep, 
+                        &u32HResult);
+
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+	  col_append_fstr(pinfo->cinfo, COL_INFO, " -> %s", 
+	  val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%08x)") );
+	}
+
+  	return offset;
+}
+
+
+int
 dissect_IDispatch_GetIDsOfNames_rqst(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep)
 {
 	e_uuid_t riid;
 	guint32 u32Lcid;
 	gchar 	szName[1000] = { 0 };
-	guint32 u32MaxNameLen = sizeof(szName);
 	guint32	u32Names;
 	guint32	u32ArraySize;
 	guint32	u32Pointer;
@@ -104,7 +193,7 @@ dissect_IDispatch_GetIDsOfNames_rqst(tvbuff_t *tvb, int offset,
 					&u32Pointer);
 		if (u32Pointer) {
 			u32VariableOffset = dissect_dcom_LPWSTR(tvb, u32VariableOffset, pinfo, tree, drep, 
-							hf_dispatch_name, szName, u32MaxNameLen);
+							hf_dispatch_name, szName, sizeof(szName));
 			if (check_col(pinfo->cinfo, COL_INFO)) {
 			  col_append_fstr(pinfo->cinfo, COL_INFO, " \"%s\"", szName);
 			}
@@ -124,7 +213,7 @@ dissect_IDispatch_GetIDsOfNames_rqst(tvbuff_t *tvb, int offset,
 
 
 
-static int
+int
 dissect_IDispatch_GetIDsOfNames_resp(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep)
 {
@@ -144,7 +233,7 @@ dissect_IDispatch_GetIDsOfNames_resp(tvbuff_t *tvb, int offset,
 		offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
                         hf_dispatch_id, &u32DispId);
 		if (check_col(pinfo->cinfo, COL_INFO)) {
-		  col_append_fstr(pinfo->cinfo, COL_INFO, " ID=0x%08x", u32DispId);
+		  col_append_fstr(pinfo->cinfo, COL_INFO, " ID=0x%x", u32DispId);
 		}
 	}
 
@@ -162,115 +251,135 @@ dissect_IDispatch_GetIDsOfNames_resp(tvbuff_t *tvb, int offset,
 
 
 
-static int
+int
 dissect_IDispatch_Invoke_rqst(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep)
 {
 	guint32 u32DispIdMember;
 	e_uuid_t riid;
 	guint32 u32Lcid;
-	guint16 u16Flags;
-	guint16 u16Flags2;
+	guint32 u32Flags;
 	guint32 u32Args;
 	guint32 u32NamedArgs;
 	guint32 u32Pointer;
+	guint32 u32Pointer2;
 	guint32 u32ArraySize;
 	guint32 u32VariableOffset;
 	guint32 u32VarRef;
 	guint32 u32VarRefIdx;
 	guint32 u32TmpOffset;
+    guint32 u32SubStart;
 
 	proto_item *feature_item;
 	proto_tree *feature_tree;
+	proto_item *dispparams_item;
+	proto_tree *dispparams_tree;
 
 
     offset = dissect_dcom_this(tvb, offset, pinfo, tree, drep);
 
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
                     hf_dispatch_id, &u32DispIdMember);
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+	  col_append_fstr(pinfo->cinfo, COL_INFO, " ID=0x%x", u32DispIdMember);
+	}
+
 	offset = dissect_dcom_UUID(tvb, offset, pinfo, tree, drep, 
 					hf_dispatch_riid, &riid);
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
 					hf_dispatch_lcid, &u32Lcid);
 	
 	/* dispatch flags */
-	u32TmpOffset = dissect_dcom_WORD(tvb, offset, pinfo, NULL, drep, 
-                        hf_dispatch_flags, &u16Flags);
-    feature_item = proto_tree_add_uint (tree, hf_dispatch_flags, tvb, offset, 2, u16Flags);
+	u32TmpOffset = dissect_dcom_DWORD(tvb, offset, pinfo, NULL, drep, 
+                        hf_dispatch_flags, &u32Flags);
+    feature_item = proto_tree_add_uint (tree, hf_dispatch_flags, tvb, offset, 4, u32Flags);
     feature_tree = proto_item_add_subtree (feature_item, ett_dispatch_flags);
     if (feature_tree) {
-        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propputref, tvb, offset, 2, u16Flags);
-        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propput, tvb, offset, 2, u16Flags);
-        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propget, tvb, offset, 2, u16Flags);
-        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_method, tvb, offset, 2, u16Flags);
+        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propputref, tvb, offset, 4, u32Flags);
+        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propput, tvb, offset, 4, u32Flags);
+        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_propget, tvb, offset, 4, u32Flags);
+        proto_tree_add_boolean (feature_tree, hf_dispatch_flags_method, tvb, offset, 4, u32Flags);
     }
 
-	if (u16Flags & DISPATCH_FLAGS_METHOD) {
+	if (u32Flags & DISPATCH_FLAGS_METHOD) {
 		proto_item_append_text(feature_item, ", Method");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_str(pinfo->cinfo, COL_INFO, " Method");
+		}
 	}
-	if (u16Flags & DISPATCH_FLAGS_PROPGET) {
+	if (u32Flags & DISPATCH_FLAGS_PROPGET) {
 		proto_item_append_text(feature_item, ", PropertyGet");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_str(pinfo->cinfo, COL_INFO, " PropertyGet");
+		}
 	}
-	if (u16Flags & DISPATCH_FLAGS_PROPPUT) {
+	if (u32Flags & DISPATCH_FLAGS_PROPPUT) {
 		proto_item_append_text(feature_item, ", PropertyPut");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_str(pinfo->cinfo, COL_INFO, " PropertyPut");
+		}
 	}
-	if (u16Flags & DISPATCH_FLAGS_PROPPUTREF) {
+	if (u32Flags & DISPATCH_FLAGS_PROPPUTREF) {
 		proto_item_append_text(feature_item, ", PropertyPutRef");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_str(pinfo->cinfo, COL_INFO, " PropertyPutRef");
+		}
 	}
 
 	offset = u32TmpOffset;
 
-	offset = dissect_dcom_WORD(tvb, offset, pinfo, tree, drep, 
-                    hf_dispatch_flags2, &u16Flags2);
-
-	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
-						&u32Pointer);
+    dispparams_item = proto_tree_add_item(tree, hf_dispatch_dispparams, tvb, offset, 0, FALSE);
+    dispparams_tree = proto_item_add_subtree (dispparams_item, ett_dispatch_params);
+    u32SubStart = offset;
 
 	/* DISPPARAMS */
 	/* VARIANT rgvarg[u32Args] */
-	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, dispparams_tree, drep, 
 						&u32Pointer);
-	if (u32Pointer) {
-		offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, tree, drep, 
+
+    /* DISPID rgdispidNamedArgs[u32NamedArgs] */
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, dispparams_tree, drep, 
+						&u32Pointer2);
+
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, dispparams_tree, drep, 
+                    hf_dispatch_args, &u32Args);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, dispparams_tree, drep, 
+                    hf_dispatch_named_args, &u32NamedArgs);
+
+    if (u32Pointer) {
+		offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, dispparams_tree, drep, 
 								&u32ArraySize);
 		u32VariableOffset = offset + u32ArraySize * 4;
 		while(u32ArraySize--) {
-			offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
+			offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, dispparams_tree, drep, 
 								&u32Pointer);
 			if (u32Pointer) {
-				u32VariableOffset = dissect_dcom_VARIANT(tvb, u32VariableOffset, pinfo, tree, drep, hf_dispatch_arg);
+				u32VariableOffset = dissect_dcom_VARIANT(tvb, u32VariableOffset, pinfo, dispparams_tree, drep, hf_dispatch_arg);
 			}
 		}
 		offset = u32VariableOffset;
 	}
 
 	/* DISPID rgdispidNamedArgs[u32NamedArgs] */
-	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
-						&u32Pointer);
-	if (u32Pointer) {
-		offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, tree, drep, 
+	if (u32Pointer2) {
+		offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, dispparams_tree, drep, 
 								&u32ArraySize);
 		u32VariableOffset = offset + u32ArraySize * 4;
 		while(u32ArraySize--) {
-			u32VariableOffset = dissect_dcom_dcerpc_pointer(tvb, u32VariableOffset, pinfo, tree, drep, 
+			u32VariableOffset = dissect_dcom_dcerpc_pointer(tvb, u32VariableOffset, pinfo, dispparams_tree, drep, 
 									&u32Pointer);
 			if (u32Pointer) {
-				offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
+				offset = dissect_dcom_DWORD(tvb, offset, pinfo, dispparams_tree, drep, 
 						hf_dispatch_id, &u32DispIdMember);
 			}
 		}
 		offset = u32VariableOffset;
 	}
 
-	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
-                    hf_dispatch_args, &u32Args);
-	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
-                    hf_dispatch_named_args, &u32NamedArgs);
+	proto_item_append_text(dispparams_item, ", Args: %u NamedArgs: %u", u32Args, u32NamedArgs);
+	proto_item_set_len(dispparams_item, offset - u32SubStart);
 
 	/* end of DISPPARAMS */
-/*	offset = u32VariableOffset; */
-
-
 
 	/* u32VarRef */
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
@@ -296,60 +405,115 @@ dissect_IDispatch_Invoke_rqst(tvbuff_t *tvb, int offset,
 		}
 	}
 
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+	  col_append_fstr(pinfo->cinfo, COL_INFO, 
+          " Args=%u NamedArgs=%u VarRef=%u", u32Args, u32NamedArgs, u32VarRef);
+	}
+
 	return u32VariableOffset;
 }
 
-#if 0
-typedef struct tagEXCEPINFO {
-    WORD  wCode;            /* An error code describing the error. */
-    WORD  wReserved;
-    BSTR  bstrSource;       /* A source of the exception */
-    BSTR  bstrDescription;  /* A description of the error */
-    BSTR  bstrHelpFile;     /* Fully qualified drive, path, and file name */
-    DWORD dwHelpContext;    /* help context of topic within the help file */
-    ULONG pvReserved;
-    ULONG pfnDeferredFillIn;
-    SCODE scode;
-} EXCEPINFO;
-#endif
-
-static int
+int
 dissect_IDispatch_Invoke_resp(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep)
 {
 	guint32 u32Pointer;
+	guint32 u32Pointer2;
+	guint32 u32Pointer3;
 	guint32 u32VariableOffset;
+	guint32 u32ArraySize;
+    guint32 u32SubStart;
+    guint16 u16Code;
+    guint16 u16Reserved;
+    guint32 u32HelpContext;
+    guint32 u32Reserved;
+    guint32 u32DeferredFillIn;
+    guint32 u32ArgErr;
+    guint32 u32HResult;
+    guint32 u32SCode;
+	guint32 u32VarRef;
+	gchar 	szName[1000] = { 0 };
+	proto_item *excepinfo_item;
+	proto_tree *excepinfo_tree;
 
 
     offset = dissect_dcom_that(tvb, offset, pinfo, tree, drep);
 
-	/* XXX: this is not correct and has to be improved! */
 	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
 						&u32Pointer);
 	if (u32Pointer) {
-		offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
-						&u32Pointer);
-		u32VariableOffset = offset;
-		u32VariableOffset = dissect_dcom_VARIANT(tvb, u32VariableOffset, pinfo, tree, drep, hf_dispatch_varresult);
-		offset = u32VariableOffset;
+		offset = dissect_dcom_VARIANT(tvb, offset, pinfo, tree, drep, hf_dispatch_varresult);
 	}
 
 	/* ExcepInfo */
-	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
+    excepinfo_item = proto_tree_add_item(tree, hf_dispatch_excepinfo, tvb, offset, 0, FALSE);
+    excepinfo_tree = proto_item_add_subtree (excepinfo_item, ett_dispatch_excepinfo);
+    u32SubStart = offset;
+
+	offset = dissect_dcom_WORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_code, &u16Code);
+	offset = dissect_dcom_WORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_reserved16, &u16Reserved);
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, excepinfo_tree, drep, 
 						&u32Pointer);
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, excepinfo_tree, drep, 
+						&u32Pointer2);
+	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, excepinfo_tree, drep, 
+						&u32Pointer3);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_help_context, &u32HelpContext);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_reserved32, &u32Reserved);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_deferred_fill_in, &u32DeferredFillIn);
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, excepinfo_tree, drep, 
+                    hf_dispatch_scode, &u32SCode);
+
 	if (u32Pointer) {
-		offset = dissect_dcom_tobedone_data(tvb, offset, pinfo, tree, drep, 
-					10000);
+		offset = dissect_dcom_BSTR(tvb, offset, pinfo, excepinfo_tree, drep, 
+						hf_dispatch_source, szName, sizeof(szName));
+	}
+	if (u32Pointer2) {
+		offset = dissect_dcom_BSTR(tvb, offset, pinfo, excepinfo_tree, drep, 
+						hf_dispatch_description, szName, sizeof(szName));
+	}
+	if (u32Pointer3) {
+		offset = dissect_dcom_BSTR(tvb, offset, pinfo, excepinfo_tree, drep, 
+						hf_dispatch_help_file, szName, sizeof(szName));
 	}
 
-	/* ArgErr */
-	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
-						&u32Pointer);
-	if (u32Pointer) {
-		offset = dissect_dcom_tobedone_data(tvb, offset, pinfo, tree, drep, 
-					10000);
-	}
+	proto_item_append_text(excepinfo_item, ", SCode: %s", 
+        val_to_str(u32SCode, dcom_hresult_vals, "Unknown (0x%08x)"));
+	proto_item_set_len(excepinfo_item, offset - u32SubStart);
+    /* end of ExcepInfo */
 
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
+                    hf_dispatch_arg_err, &u32ArgErr);
+
+	/* rgVarRef: VARIANT[u32VarRef] */
+	offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, tree, drep, 
+							&u32ArraySize);
+    u32VarRef = u32ArraySize;
+	u32VariableOffset = offset + u32ArraySize * 4;
+	while(u32ArraySize--) {
+		offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, tree, drep, 
+							&u32Pointer);
+		if (u32Pointer) {
+			u32VariableOffset = dissect_dcom_VARIANT(tvb, u32VariableOffset, pinfo, tree, drep, hf_dispatch_varrefarg);
+		}
+	}
+    offset = u32VariableOffset;
+
+	/* HRESULT of call */
+	offset = dissect_dcom_HRESULT(tvb, offset, pinfo, tree, drep, 
+                        &u32HResult);
+
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+	  col_append_fstr(pinfo->cinfo, COL_INFO, " SCode=%s VarRef=%u -> %s", 
+          val_to_str(u32SCode, dcom_hresult_vals, "Unknown (0x%08x)"),
+          u32VarRef,
+	      val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%08x)") );
+	}
 
 	return offset;
 }
@@ -362,8 +526,8 @@ static dcerpc_sub_dissector dispatch_dissectors[] = {
     { 1, "AddRef", NULL, NULL },
     { 2, "Release", NULL, NULL },
 
-    { 3, "GetTypeInfoCount", NULL, NULL },
-    { 4, "GetTypeInfo", NULL, NULL },
+    { 3, "GetTypeInfoCount", dissect_dcom_simple_rqst, dissect_IDispatch_GetTypeInfoCount_resp },
+    { 4, "GetTypeInfo", dissect_IDispatch_GetTypeInfo_rqst, dissect_IDispatch_GetTypeInfo_resp },
     { 5, "GetIDsOfNames", dissect_IDispatch_GetIDsOfNames_rqst, dissect_IDispatch_GetIDsOfNames_resp },
     { 6, "Invoke", dissect_IDispatch_Invoke_rqst, dissect_IDispatch_Invoke_resp },
     { 0, NULL, NULL, NULL },
@@ -387,11 +551,9 @@ proto_register_dcom_dispatch(void)
         { &hf_dispatch_lcid,
         { "LCID", "dispatch_lcid", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
         { &hf_dispatch_id,
-        { "ID", "dispatch_id", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { "DispID", "dispatch_id", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
         { &hf_dispatch_flags,
-        { "Flags", "dispatch_flags", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }},
-        { &hf_dispatch_flags2,
-        { "Flags2", "dispatch_flags2", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { "Flags", "dispatch_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
 
 		{ &hf_dispatch_arg,
 		{ "Argument", "dispatch_arg", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
@@ -409,18 +571,50 @@ proto_register_dcom_dispatch(void)
 		{ "VarResult", "dispatch_varresult", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
 
         { &hf_dispatch_flags_method,
-        { "Method", "dispatch_flags_method", FT_BOOLEAN, 16, TFS (&flags_set_truth), DISPATCH_FLAGS_METHOD, "", HFILL }},
+        { "Method", "dispatch_flags_method", FT_BOOLEAN, 32, TFS (&flags_set_truth), DISPATCH_FLAGS_METHOD, "", HFILL }},
         { &hf_dispatch_flags_propget,
-        { "PropertyGet", "dispatch_flags_propget", FT_BOOLEAN, 16, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPGET, "", HFILL }},
+        { "PropertyGet", "dispatch_flags_propget", FT_BOOLEAN, 32, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPGET, "", HFILL }},
         { &hf_dispatch_flags_propput,
-        { "PropertyPut", "dispatch_flags_propput", FT_BOOLEAN, 16, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPPUT, "", HFILL }},
+        { "PropertyPut", "dispatch_flags_propput", FT_BOOLEAN, 32, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPPUT, "", HFILL }},
         { &hf_dispatch_flags_propputref,
-        { "PropertyPutRef", "dispatch_flags_propputref", FT_BOOLEAN, 16, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPPUTREF, "", HFILL }},
-	};
+        { "PropertyPutRef", "dispatch_flags_propputref", FT_BOOLEAN, 32, TFS (&flags_set_truth), DISPATCH_FLAGS_PROPPUTREF, "", HFILL }},
+
+        { &hf_dispatch_code,
+        { "Code", "dispatch_code", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_reserved16,
+        { "Reserved", "dispatch_reserved16", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_source,
+        { "Source", "dispatch_source", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_description,
+        { "Description", "dispatch_description", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_help_file,
+        { "HelpFile", "dispatch_help_file", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_help_context,
+        { "HelpContext", "dispatch_help_context", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_reserved32,
+        { "Reserved", "dispatch_reserved32", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_deferred_fill_in,
+        { "DeferredFillIn", "dispatch_deferred_fill_in", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_arg_err,
+        { "ArgErr", "dispatch_arg_err", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
+
+        { &hf_dispatch_tinfo,
+        { "TInfo", "dispatch_tinfo", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_itinfo,
+        { "TInfo", "dispatch_itinfo", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_dispparams,
+        { "DispParams", "dispatch_dispparams", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_excepinfo,
+        { "ExcepInfo", "dispatch_excepinfo", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
+        { &hf_dispatch_scode,
+        { "SCode", "dispatch_scode", FT_UINT32, BASE_HEX, VALS(dcom_hresult_vals), 0x0, "", HFILL }}
+    };
 
 	static gint *ett[] = {
 		&ett_dispatch,
-		&ett_dispatch_flags
+		&ett_dispatch_flags,
+        &ett_dispatch_params,
+        &ett_dispatch_excepinfo
 	};
 
 
