@@ -1,7 +1,7 @@
 /* packet-dcom.c
  * Routines for DCOM generics
  *
- * $Id: packet-dcom.c 13198 2005-01-29 20:39:14Z guy $
+ * $Id: packet-dcom.c 17840 2006-04-07 22:05:34Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -84,9 +84,11 @@
 
 #include <glib.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-dcerpc.h"
 #include "packet-dcom.h"
 #include "prefs.h"
+#include "expert.h"
 
 
 static int proto_dcom = -1;
@@ -206,7 +208,8 @@ static int hf_dcom_vt_r4 = -1;
 static int hf_dcom_vt_r8 = -1;
 static int hf_dcom_vt_date = -1;
 static int hf_dcom_vt_bstr = -1;
-
+static int hf_dcom_vt_byref = -1;
+static int hf_dcom_vt_dispatch = -1;
 
 
 /*
@@ -290,6 +293,7 @@ const value_string dcom_variant_type_vals[] = {
 	{ ETHEREAL_VT_CY,		"VT_CY"},
 	{ ETHEREAL_VT_DATE,		"VT_DATE"},
 	{ ETHEREAL_VT_BSTR,		"VT_BSTR"},
+	{ ETHEREAL_VT_DISPATCH, "VT_DISPATCH"},
 	{ ETHEREAL_VT_ERROR,	"VT_ERROR"},
 	{ ETHEREAL_VT_BOOL,		"VT_BOOL"},
 	{ ETHEREAL_VT_I1,		"VT_I1"},
@@ -317,6 +321,10 @@ const value_string dcom_variant_type_vals[] = {
     { ETHEREAL_VT_ARRAY | ETHEREAL_VT_UI4,   "VT_ARRAY|VT_UI4"},
     { ETHEREAL_VT_ARRAY | ETHEREAL_VT_I8,    "VT_ARRAY|VT_I8"},
     { ETHEREAL_VT_ARRAY | ETHEREAL_VT_UI8,   "VT_ARRAY|VT_UI8"},
+
+    { ETHEREAL_VT_BYREF | ETHEREAL_VT_I2,    "VT_BYREF|VT_I2"},
+    { ETHEREAL_VT_BYREF | ETHEREAL_VT_BSTR,  "VT_BYREF|VT_BSTR"},
+    { ETHEREAL_VT_BYREF | ETHEREAL_VT_VARIANT,"VT_BYREF|VT_VARIANT"},
 	{ 0,          NULL }
 /* XXX: append more types here */
 };
@@ -330,19 +338,27 @@ const value_string dcom_hresult_vals[] = {
 	{ 0x8000FFFF, "E_UNEXPECTED" },
 	{ 0x80004001, "E_NOTIMPL" },
 	{ 0x80004002, "E_NOINTERFACE" },
+	{ 0x80004003, "E_POINTER" },
 	{ 0x80004004, "E_ABORT" },
 	{ 0x80004005, "E_FAIL" },
-	{ 0x80004003, "E_POINTER" },
 	{ 0x80070005, "E_ACCESSDENIED" },
 	{ 0x80070006, "E_HANDLE" },
 	{ 0x8007000E, "E_OUTOFMEMORY" },
 	{ 0x80070057, "E_INVALIDARG" },
 
 	{ 0x80010108, "RPC_E_DISCONNECTED" },
+	{ 0x80010113, "RPC_E_INVALID_IPID" },
 
+	{ 0x80020003, "DISP_E_MEMBERNOTFOUND" },
 	{ 0x80020004, "DISP_E_PARAMNOTFOUND" },
+	{ 0x80020005, "DISP_E_TYPEMISMATCH" },
+	{ 0x80020006, "DISP_E_UNKNOWNNAME" },
+	{ 0x80020008, "DISP_E_BADVARTYPE" },
+	{ 0x80020009, "DISP_E_EXCEPTION" },
+	{ 0x8002000A, "DISP_E_OVERFLOW" },
 
 	{ 0x80040154, "REGDB_E_CLASSNOTREG" },
+	{ 0x80040201, "CO_E_FAILEDTOGETSECCTX" },
 
 /* following are CBA application specific values */
 	{ 0x0004CA00, "CBA_S_PERSISTPENDING" },
@@ -391,6 +407,9 @@ const value_string dcom_hresult_vals[] = {
 	{ 0x8004CB23, "CBA_E_FRAMECOUNTUNSUPPORTED" },
 	{ 0x8004CB24, "CBA_E_LINKFAILURE" },
 	{ 0x8004CB25, "CBA_E_MODECHANGE" },
+
+	{ 0x80080004, "CO_E_BAD_PATH" },
+
 	{ 0,          NULL }
 };
 
@@ -406,6 +425,21 @@ static const value_string dcom_objref_signature_vals[] = {
 	{ 0,          NULL }
 };
 
+/* although flags, they doesn't seem to be used as such */
+static const value_string dcom_stdobjref_flag_vals[] = {
+	{ 0x0000, "SORF_NULL" },
+	{ 0x0001, "SORF_OXRES1" },
+	{ 0x0020, "SORF_OXRES2" },
+	{ 0x0040, "SORF_OXRES3" },
+	{ 0x0080, "SORF_OXRES4" },
+	{ 0x0100, "SORF_OXRES5" },
+	{ 0x0200, "SORF_OXRES6" },
+	{ 0x0400, "SORF_OXRES7" },
+	{ 0x0800, "SORF_OXRES8" },
+	{ 0x1000, "SORF_NOPING" },
+	{ 0,          NULL }
+};
+
 static const value_string dcom_dcerpc_pointer_vals[] = {
 	{ 0x72657355, "User" },
 	{ 0x42535452, "BSTR" },
@@ -459,7 +493,7 @@ static const value_string dcom_vt_bool_vals[] = {
 
 
 /* dissect extension to DCOM "this" and "that" */
-int
+static int
 dissect_dcom_extent(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep)
 {
@@ -643,8 +677,8 @@ dissect_dcom_simple_resp(tvbuff_t *tvb, int offset,
    Returns the associated string ptr on a match.
    Formats uuid number and returns the resulting string, on failure.
    (copied from val_to_str) */
-gchar* dcom_uuid_to_str(e_uuid_t *uuid) {
-  gchar *ret;
+const gchar* dcom_uuid_to_str(e_uuid_t *uuid) {
+  const gchar *ret;
   static gchar  str[3][64];
   static gchar *cur;
 
@@ -660,7 +694,7 @@ gchar* dcom_uuid_to_str(e_uuid_t *uuid) {
   } else {  
     cur = &str[0][0];
   }
-  snprintf(cur, 64, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
+  g_snprintf(cur, 64, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
                       uuid->Data1, uuid->Data2, uuid->Data3,
                       uuid->Data4[0], uuid->Data4[1],
                       uuid->Data4[2], uuid->Data4[3],
@@ -715,11 +749,14 @@ extern int
 dissect_dcom_tobedone_data(tvbuff_t *tvb, int offset,
 	packet_info *pinfo _U_, proto_tree *tree, guint8 *drep _U_, int length)
 {
+	proto_item *item;
 
     proto_tree_add_uint(tree, hf_dcom_tobedone_len, tvb, offset, length, length);
 
-	proto_tree_add_bytes(tree, hf_dcom_tobedone, tvb, offset, length, 
+	item = proto_tree_add_bytes(tree, hf_dcom_tobedone, tvb, offset, length, 
 		tvb_get_ptr(tvb, offset, length));
+	expert_add_info_format(pinfo, item, PI_UNDECODED, PI_WARN, "%u bytes still undecoded", length);
+
 	offset += length;
 
 	return offset;
@@ -792,8 +829,26 @@ dissect_dcom_HRESULT(tvbuff_t *tvb, int offset,	packet_info *pinfo,
 					 proto_tree *tree, guint8 *drep, 
 					 guint32 * pu32HResult)
 {
-	offset = dissect_dcom_DWORD(tvb, offset, pinfo, tree, drep, 
-                    hf_dcom_hresult, pu32HResult);
+    guint32 u32HResult;
+	proto_item *item = NULL;
+
+	/* dissect the DWORD, but don't add to tree */
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, NULL /*tree*/, drep, 
+                    hf_dcom_hresult, &u32HResult);
+
+    if (tree) {
+		/* special formatted output of indexed value */
+        item = proto_tree_add_item (tree, hf_dcom_hresult, tvb, offset-4, 4, (drep[0] & 0x10));
+    }
+
+	/* expert info only if severity is set */
+	/* XXX - move this to the callers of this function, to provide a more detailed error output */
+	if(u32HResult & 0x80000000) {
+		expert_add_info_format(pinfo, item, PI_RESPONSE_CODE, PI_NOTE, "Hresult: %s",
+			val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%x)"));
+	}
+    if (pu32HResult)
+        *pu32HResult = u32HResult;
 
 	return offset;
 }
@@ -806,6 +861,7 @@ dissect_dcom_indexed_HRESULT(tvbuff_t *tvb, int offset,	packet_info *pinfo,
 					 guint32 * pu32HResult, int field_index)
 {
     guint32 u32HResult;
+	proto_item *item = NULL;
 
 
 	/* dissect the DWORD, but don't add to tree */
@@ -814,11 +870,17 @@ dissect_dcom_indexed_HRESULT(tvbuff_t *tvb, int offset,	packet_info *pinfo,
 
     if (tree) {
 		/* special formatted output of indexed value */
-        proto_tree_add_uint_format(tree, hf_dcom_hresult, tvb, offset-4, 4, (drep[0] & 0x10),
+        item = proto_tree_add_uint_format(tree, hf_dcom_hresult, tvb, offset-4, 4, (drep[0] & 0x10),
 			"HResult[%u]: %s (0x%08x)", field_index,
 			val_to_str(u32HResult, dcom_hresult_vals, "Unknown"),
 			u32HResult);
     }
+	/* expert info only if severity flag is set */
+	/* XXX - move this to the callers of this function, to provide a more detailed error output */
+	if(u32HResult & 0x80000000) {
+		expert_add_info_format(pinfo, item, PI_RESPONSE_CODE, PI_NOTE, "Hresult: %s",
+			val_to_str(u32HResult, dcom_hresult_vals, "Unknown (0x%x)"));
+	}
     if (pu32HResult)
         *pu32HResult = u32HResult;
 
@@ -842,7 +904,7 @@ dissect_dcom_COMVERSION(tvbuff_t *tvb, int offset, packet_info *pinfo,
 }
 	
 
-int
+static int
 dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
 						proto_tree *tree, guint8 *drep, int hfindex _U_)
 {
@@ -926,6 +988,7 @@ dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 	offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, sub_tree, drep, &u32ArraySize);
 
+    tvb_ensure_bytes_exist(tvb, offset, u32ArraySize * u32ElementSize);
 	u32VariableOffset = offset + u32ArraySize * u32ElementSize;
 
 	u32Tmp = u32ArraySize;
@@ -950,6 +1013,8 @@ dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			case(ETHEREAL_VT_I8):
 				offset = dissect_dcom_I8(tvb, offset, pinfo, sub_tree, drep, 
 									hf_dcom_vt_i8, NULL);
+                /* take care of the 8 byte alignment */
+                u32VariableOffset = offset;
 				break;
 			case(ETHEREAL_VT_BSTR):
 				offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, sub_tree, drep, &u32Pointer);
@@ -1105,6 +1170,13 @@ dissect_dcom_VARIANT(tvbuff_t *tvb, int offset, packet_info *pinfo,
 								hf_dcom_vt_bstr, cData, sizeof(cData) );
 			}
 			break;
+		case(ETHEREAL_VT_DISPATCH):
+			offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, sub_tree, drep, &u32Pointer);
+			if (u32Pointer) {
+				offset = dissect_dcom_MInterfacePointer(tvb, offset, pinfo, sub_tree, drep, 
+								hf_dcom_vt_dispatch);
+			}
+			break;
 		case(ETHEREAL_VT_ARRAY):
 			offset = dissect_dcom_SAFEARRAY(tvb, offset, pinfo, sub_tree, drep,
 								0);
@@ -1113,6 +1185,13 @@ dissect_dcom_VARIANT(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			offset = dissect_dcom_HRESULT(tvb, offset, pinfo, sub_tree, drep,
 								0);
 			break;
+		case(ETHEREAL_VT_VARIANT):
+			offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, sub_tree, drep, &u32Pointer);
+			if (u32Pointer) {
+			    offset = dissect_dcom_VARIANT(tvb, offset, pinfo, sub_tree, drep,
+								    hf_dcom_vt_byref /* must be BYREF */);
+            }
+			break;
 		case(ETHEREAL_VT_UNKNOWN):
 			offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, sub_tree, drep, &u32Pointer);
 			break;
@@ -1135,10 +1214,10 @@ dissect_dcom_VARIANT(tvbuff_t *tvb, int offset, packet_info *pinfo,
 int
 dissect_dcom_append_UUID(tvbuff_t *tvb, int offset,
 	packet_info *pinfo, proto_tree *tree, guint8 *drep,
-	int hfindex, gchar *field_name, int field_index)
+	int hfindex, const gchar *field_name, int field_index)
 {
 	e_uuid_t uuid;
-	gchar *uuid_name;
+	const gchar *uuid_name;
 
 
 	offset = dissect_dcom_UUID(tvb, offset, pinfo, tree, drep, 
@@ -1165,8 +1244,9 @@ dissect_dcom_append_UUID(tvbuff_t *tvb, int offset,
 
 
 /* get a zero terminated wide character string from tvb */
+/* maxlength is including zero termination */
 /* XXX: is there a function existing somewhere, already implementing this? */
-int
+static int
 dcom_tvb_get_nwstringz0(tvbuff_t *tvb, gint offset, guint32 maxlength, gchar *pszStr)
 {
 	guint32 u32Idx;
@@ -1174,7 +1254,7 @@ dcom_tvb_get_nwstringz0(tvbuff_t *tvb, gint offset, guint32 maxlength, gchar *ps
 
 
 	/* get LPWSTR to ascii */
-	g_assert(maxlength > 0);
+	DISSECTOR_ASSERT(maxlength > 0);
 	u32Idx = 0;
 	pszStr[0] = 0;
 	while (u32Idx < maxlength-1) {
@@ -1232,13 +1312,18 @@ dissect_dcom_indexed_LPWSTR(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, sub_tree, drep, 
                         &u32ArraySize);
 
+    DISSECTOR_ASSERT(u32MaxStr != 0);
 	u32ArraySize++;	/* u32MaxStr is including zero termination */
 	if (u32ArraySize < u32MaxStr) {
 		u32MaxStr = u32ArraySize;
 	}
 
 	u32StrStart = offset;
-	offset = dcom_tvb_get_nwstringz0(tvb, offset, u32MaxStr, pszStr);
+    if(u32MaxStr != 0) {
+	    offset = dcom_tvb_get_nwstringz0(tvb, offset, u32MaxStr, pszStr);
+    } else {
+        *pszStr=0;
+    }
 
 #if GLIB_MAJOR_VERSION < 2
     pszEscaped = g_strescape(pszStr);
@@ -1317,13 +1402,18 @@ dissect_dcom_BSTR(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 
 	u32RealOffset = offset + u32ArraySize*2;
 
+    DISSECTOR_ASSERT(u32MaxStr != 0);
 	u32ArraySize++;	/* u32MaxStr is including zero termination */
 	if (u32ArraySize < u32MaxStr) {
 		u32MaxStr = u32ArraySize;
 	}
 
 	u32StrStart = offset;
-	offset = dcom_tvb_get_nwstringz0(tvb, offset, u32MaxStr, pszStr);
+    if(u32MaxStr != 0) {
+	    offset = dcom_tvb_get_nwstringz0(tvb, offset, u32MaxStr, pszStr);
+    } else {
+        *pszStr=0;
+    }
 
 #if GLIB_MAJOR_VERSION < 2
     pszEscaped = g_strescape(pszStr);
@@ -1334,6 +1424,8 @@ dissect_dcom_BSTR(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 
 	/* update subtree header */
 	proto_item_append_text(sub_item, "\"%s\"", pszEscaped);
+	if ((int) (u32RealOffset - u32SubStart) <= 0)
+	    THROW(ReportedBoundsError);
 	proto_item_set_len(sub_item, u32RealOffset - u32SubStart);
 
     u32Max = MIN(u32Max, strlen(pszEscaped) + 1);
@@ -1695,7 +1787,7 @@ proto_register_dcom (void)
 		{ &hf_dcom_stdobjref,
 		{ "STDOBJREF", "dcom.stdobjref", FT_NONE, BASE_HEX, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_stdobjref_flags,
-		{ "Flags", "dcom.stdobjref.flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
+		{ "Flags", "dcom.stdobjref.flags", FT_UINT32, BASE_HEX, VALS(dcom_stdobjref_flag_vals), 0x0, "", HFILL }},
 		{ &hf_dcom_stdobjref_public_refs,
 		{ "PublicRefs", "dcom.stdobjref.public_refs", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_stdobjref_oxid,
@@ -1751,7 +1843,11 @@ proto_register_dcom (void)
 		{ &hf_dcom_vt_date,
 		{ "VT_DATE", "dcom.vt.date", FT_DOUBLE, BASE_NONE, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_vt_bstr,
-		{ "VT_BSTR", "dcom.vt.bstr", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }}
+		{ "VT_BSTR", "dcom.vt.bstr", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
+		{ &hf_dcom_vt_byref,
+		{ "BYREF", "dcom.vt.byref", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }},
+		{ &hf_dcom_vt_dispatch,
+		{ "VT_DISPATCH", "dcom.vt.dispatch", FT_NONE, BASE_NONE, NULL, 0x0, "", HFILL }}
 	};
 
 	static hf_register_info hf_dcom_sa_array[] = {
