@@ -1,7 +1,7 @@
 /* packet-diameter.c
  * Routines for Diameter packet disassembly
  *
- * $Id: packet-diameter.c 15043 2005-07-24 19:01:28Z guy $
+ * $Id: packet-diameter.c 17684 2006-03-20 19:46:04Z etxrab $
  *
  * Copyright (c) 2001 by David Frascone <dave@frascone.com>
  *
@@ -54,13 +54,10 @@
 #include <epan/report_err.h>
 #include <epan/prefs.h>
 #include <epan/sminmpec.h>
+#include <epan/emem.h>
 #include "packet-tcp.h"
 #include "packet-sip.h"
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 /* This must be defined before we include packet-diameter-defs.h */
 
 /* Valid data types */
@@ -70,6 +67,7 @@ typedef enum {
   DIAMETER_INTEGER32,
   DIAMETER_INTEGER64,
   DIAMETER_UNSIGNED32,
+  DIAMETER_UNSIGNED32ENUM,
   DIAMETER_UNSIGNED64,
   DIAMETER_FLOAT32,
   DIAMETER_FLOAT64,
@@ -99,6 +97,7 @@ static const value_string TypeValues[]={
   {  DIAMETER_INTEGER32,       "Integer32" },
   {  DIAMETER_INTEGER64,       "Integer64" },
   {  DIAMETER_UNSIGNED32,      "Unsigned32" },
+  {  DIAMETER_UNSIGNED32ENUM,	"Unsigned32" }, /* This is needed to get value translation for Uint32:s with a value*/
   {  DIAMETER_UNSIGNED64,      "Unsigned64" },
   {  DIAMETER_FLOAT32,         "Float32" },
   {  DIAMETER_FLOAT64,         "Float64" },
@@ -130,7 +129,7 @@ typedef struct value_name {
 
 typedef struct old_avp_info {
   guint32           code;
-  gchar            *name;
+  const gchar      *name;
   diameterDataType  type;
   const value_string *values;
 } oldAvpInfo;
@@ -242,11 +241,12 @@ static guint gbl_diameterSctpPort=SCTP_PORT_DIAMETER;
 static gboolean gbl_diameter_desegment = TRUE;
 
 /* Allow zero as a valid application ID */
-static gboolean allow_zero_as_app_id = FALSE;
+static gboolean allow_zero_as_app_id = TRUE;
 
-/* Supress console output at unknown AVP:s,Flags etc */
+/* Suppress console output at unknown AVP:s,Flags etc */
 static gboolean suppress_console_output = TRUE;
 
+static gboolean gbl_use_xml_dictionary = TRUE;
 #define DICT_FN  "diameter/dictionary.xml"
 static const gchar *gbl_diameterDictionary;
 
@@ -374,7 +374,7 @@ xmlParseFilePush( const char *filename, int checkValid
  * only called when the XML dictionary fails to load properly.
  */
 static int
-addStaticAVP(int code, gchar *name, diameterDataType type, const value_string *values)
+addStaticAVP(int code, const gchar *name, diameterDataType type, const value_string *values)
 {
   avpInfo *entry;
   ValueName *vEntry=NULL;
@@ -400,8 +400,20 @@ addStaticAVP(int code, gchar *name, diameterDataType type, const value_string *v
   entry->vendorName = NULL;
   entry->type = type;
   entry->values = vEntry;
-  if (vEntry)
-	entry->type = DIAMETER_ENUMERATED;
+  /* Unsigned32 might have values to ( Result-code 268 ) */
+  if (vEntry){
+		switch(type){
+		case DIAMETER_UNSIGNED32:
+			entry->type = DIAMETER_UNSIGNED32ENUM;
+			break;
+		case DIAMETER_VENDOR_ID:
+			/* Ignore data from the xml file, use sminmpec.h vals */
+			break;
+		default:
+			entry->type = DIAMETER_ENUMERATED;
+	}
+  }
+
 
   /* And, add it to the list */
   entry->next = avpListHead;
@@ -415,7 +427,7 @@ addStaticAVP(int code, gchar *name, diameterDataType type, const value_string *v
  * only called when the XML dictionary fails to load properly.
  */
 static int
-addVendorAVP(int code, gchar *name, diameterDataType type, const value_string *values,int vendorId)
+addVendorAVP(int code, const gchar *name, diameterDataType type, const value_string *values,int vendorId)
 {
   avpInfo *entry;
   ValueName *vEntry=NULL;
@@ -445,10 +457,23 @@ addVendorAVP(int code, gchar *name, diameterDataType type, const value_string *v
   if (vendorName)
 	entry->vendorName = g_strdup(vendorName);
   else
-	entry->vendorName = NULL;  entry->type = type;
+	entry->vendorName = NULL;  
+  entry->type = type;
   entry->values = vEntry;
-  if (vEntry)
-	entry->type = DIAMETER_ENUMERATED;
+
+  /* Unsigned32 might have values to ( Result-code 268 ) */
+  if (vEntry){
+		switch(type){
+		case DIAMETER_UNSIGNED32:
+			entry->type = DIAMETER_UNSIGNED32ENUM;
+			break;
+		case DIAMETER_VENDOR_ID:
+			/* Ignore data from the xml file, use sminmpec.h vals */
+			break;
+		default:
+			entry->type = DIAMETER_ENUMERATED;
+	}
+  }
 
   /* And, add it to the list */
   entry->next = avpListHead;
@@ -488,9 +513,9 @@ xmlParseAVP(xmlNodePtr cur)
   cur = cur->xmlChildrenNode;
 
   while (cur != NULL ) {
-	if (strcasecmp(cur->name, "type") == 0) {
+	if (strcasecmp((const char *)cur->name, "type") == 0) {
 	  type = XmlStub.xmlGetProp(cur, "type-name");
-	} else if (strcasecmp(cur->name, "enum") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "enum") == 0) {
 	  char *valueName=NULL, *valueCode=NULL;
 	  ValueName *ve = NULL;
 	  valueName = XmlStub.xmlGetProp(cur, "name");
@@ -507,7 +532,7 @@ xmlParseAVP(xmlNodePtr cur)
 
 	  ve->next = vEntry;
 	  vEntry = ve;
-	} else if (strcasecmp(cur->name, "grouped") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "grouped") == 0) {
 	  /* WORK Recurse here for grouped AVPs */
 	  type = "grouped";
 	}
@@ -548,10 +573,20 @@ xmlParseAVP(xmlNodePtr cur)
 	entry->vendorName = NULL;
   entry->type = avpType;
   entry->values = vEntry;
-  if (vEntry)
-	entry->type = DIAMETER_ENUMERATED;
+  /* Unsigned32 might have values to ( Result-code 268 ) */
+    if (vEntry)
+		switch(avpType){
+		case DIAMETER_UNSIGNED32:
+			entry->type = DIAMETER_UNSIGNED32ENUM;
+			break;
+		case DIAMETER_VENDOR_ID:
+			/* Ignore data from the xml file, use sminmpec.h vals */
+			break;
+		default:
+			entry->type = DIAMETER_ENUMERATED;
+	}
 
-  /* And, add it to the list */
+	/* And, add it to the list */
   entry->next = avpListHead;
   avpListHead = entry;
 
@@ -608,6 +643,9 @@ xmlParseCommand(xmlNodePtr cur)
    */
   name = XmlStub.xmlGetProp(cur, "name");
   code = XmlStub.xmlGetProp(cur, "code");
+  /*
+  g_warning("xmlParseCommand Name: %s code %s",name,code);
+  */
   if (!name || !code) {
 	report_failure("Invalid command.  Name or code missing!");
 	return -1;
@@ -623,12 +661,12 @@ xmlParseCommand(xmlNodePtr cur)
 
 /* This routine adds an application to the name<-> id table */
 static int
-dictionaryAddApplication(char *name, int id)
+dictionaryAddApplication(char *name, guint32 id)
 {
   ApplicationId *entry;
 
-  if (!name || (id < 0) || (id == 0 && !allow_zero_as_app_id)) {
-	report_failure( "Diameter Error: Invalid application (name=%p, id=%d)",
+  if (!name || (id == 0 && !allow_zero_as_app_id)) {
+	report_failure( "Diameter Error: Invalid application (name=%s, id=%d)",
 			   name, id);
 	return (-1);
   } /* Sanity Checks */
@@ -714,9 +752,8 @@ xmlDictionaryParseSegment(xmlNodePtr cur, int base)
 				name?name:"NULL", id?id:"NULL");
 	  return -1;
 	}
-
 	/* Add the application */
-	if (dictionaryAddApplication(name, atol(id)) != 0) {
+	if (dictionaryAddApplication(name, (guint32)atol(id)) != 0) {
 	  /* ERROR! */
 	  return -1;
 	}
@@ -728,19 +765,19 @@ xmlDictionaryParseSegment(xmlNodePtr cur, int base)
    */
   cur = cur->xmlChildrenNode;
   while (cur != NULL) {
-	if (strcasecmp(cur->name, "avp") == 0) {
+	if (strcasecmp((const char *)cur->name, "avp") == 0) {
 	  /* we have an avp!!! */
 	  xmlParseAVP(cur);
-	} else if (strcasecmp(cur->name, "vendor") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "vendor") == 0) {
 	  /* we have a vendor */
 	  xmlParseVendor(cur);
 	  /* For now, ignore typedefn and text */
-	} else if (strcasecmp(cur->name, "command") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "command") == 0) {
 	  /* Found a command */
 	  xmlParseCommand(cur);
-	} else if (strcasecmp(cur->name, "text") == 0) {
-	} else if (strcasecmp(cur->name, "comment") == 0) {
-	} else if (strcasecmp(cur->name, "typedefn") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "text") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "comment") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "typedefn") == 0) {
 	  /* WORK -- parse in valid types . . . */
 	} else {
 	  /* IF we got here, we're an error */
@@ -762,15 +799,15 @@ xmlDictionaryParse(xmlNodePtr cur)
 {
   /* We should expect a base protocol, followed by multiple applications */
   while (cur != NULL) {
-	if (strcasecmp(cur->name, "base") == 0) {
+	if (strcasecmp((const char *)cur->name, "base") == 0) {
 	  /* Base protocol.  Descend and parse */
 	  xmlDictionaryParseSegment(cur, 1);
-	} else if (strcasecmp(cur->name, "application") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "application") == 0) {
 	  /* Application.  Descend and parse */
 	  xmlDictionaryParseSegment(cur, 0);
-	} else if (strcasecmp(cur->name, "text") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "text") == 0) {
 	  /* Ignore text */
-	} else if (strcasecmp(cur->name, "comment") == 0) {
+	} else if (strcasecmp((const char *)cur->name, "comment") == 0) {
 	  /* Ignore text */
 	} else {
 	  report_failure( "Diameter: XML Expecting a base or an application  (got \"%s\")",
@@ -794,16 +831,18 @@ loadXMLDictionary(void)
   xmlNodePtr cur;
 
   /*
-   * build an XML tree from a the file;
+   * build an XML tree from the file;
    */
   XmlStub.xmlKeepBlanksDefault(0);                    /* Strip leading and trailing blanks */
   XmlStub.xmlSubstituteEntitiesDefault(1);            /* Substitute entities automagically */
   doc = xmlParseFilePush(gbl_diameterDictionary, 1);  /* Parse the XML (do validity checks)*/
 
-  /* Check for invalid xml */
+  /* Check for invalid xml.
+     Note that xmlParseFilePush reports details of problems found,
+     and it should be obvious from the default filename that the error relates
+     to Diameter.
+  */
   if (doc == NULL) {
-	report_failure("Diameter: Unable to parse xmldictionary %s",
-			  gbl_diameterDictionary);
 	return -1;
   }
 
@@ -882,24 +921,75 @@ initializeDictionaryDefaults(void)
 } /* initializeDictionaryDefaults */
 
 /*
- * This routine will attempt to load the XML dictionary, and on
- * failure, will call initializeDictionaryDefaults to load in
- * our static dictionary.
+ * This routine will attempt to load the XML dictionary if configured to.
+ * Otherwise, or if load fails, it will call initializeDictionaryDefaults
+ * to load in our static dictionary instead.
  */
 static void
 initializeDictionary(void)
 {
+  /*
+   * First, empty the dictionary of any previous contents
+   */
+
+  ApplicationId *tmpApplicationId = ApplicationIdHead;
+  VendorId      *tmpVendorId = vendorListHead;
+  CommandCode   *tmpCommandCode = commandListHead;
+  avpInfo       *tmpAvpInfo = avpListHead;
+
+  /* ApplicationId list */
+  while (tmpApplicationId != NULL) {
+    g_free(tmpApplicationId->name);
+    tmpApplicationId = tmpApplicationId->next;
+  }
+  ApplicationIdHead = NULL;
+
+  /* VendorId list */
+  while (tmpVendorId != NULL) {
+    g_free(tmpVendorId->name);
+    g_free(tmpVendorId->longName);
+    tmpVendorId = tmpVendorId->next;
+  }
+  vendorListHead = NULL;
+
+  /* CommandCode list */
+  while (tmpCommandCode != NULL) {
+    g_free(tmpCommandCode->name);
+    g_free(tmpCommandCode->vendorName);
+    tmpCommandCode = tmpCommandCode->next;
+  }
+  commandListHead = NULL;
+
+  /* avpInfo list */
+  while (tmpAvpInfo != NULL) {
+    ValueName *valueNamePtr = tmpAvpInfo->values;
+    g_free(tmpAvpInfo->name);
+    g_free(tmpAvpInfo->vendorName);
+    while (valueNamePtr) {
+      g_free(valueNamePtr->name);
+      valueNamePtr = valueNamePtr->next;
+    }
+    tmpAvpInfo = tmpAvpInfo->next;
+  }
+  avpListHead = NULL;
+
+
   /*
    * Using ugly ordering here.  If loadLibXML succeeds, then
    * loadXMLDictionary will be called.  This is one of the few times when
    * I think this is prettier than the nested if alternative.
    */
-  if (loadLibXML() ||
-	  (loadXMLDictionary() != 0)) {
-	/* Something failed.  Use the static dictionary */
-	report_failure("Diameter: Using static dictionary! (Unable to use XML)");
-	initializeDictionaryDefaults();
-  }
+   if (gbl_use_xml_dictionary) {
+      if (loadLibXML() || (loadXMLDictionary() != 0)) {
+	     /* Something failed.  Use the static dictionary */
+	     report_failure("Diameter: Using static dictionary! (Unable to use XML)");
+	     initializeDictionaryDefaults();
+      }
+   }
+   else {
+      initializeDictionaryDefaults();
+   }
+
 } /* initializeDictionary */
 
 
@@ -912,7 +1002,7 @@ initializeDictionary(void)
 static gchar *
 diameter_vendor_to_str(guint32 vendorId, gboolean longName) {
   VendorId *probe;
-  static gchar buffer[64];
+  gchar *buffer;
 
   for (probe=vendorListHead; probe; probe=probe->next) {
 	if (vendorId == probe->id) {
@@ -923,8 +1013,8 @@ diameter_vendor_to_str(guint32 vendorId, gboolean longName) {
 	}
   }
 
-  snprintf(buffer, sizeof(buffer),
-		   "Vendor 0x%08x", vendorId);
+  buffer=ep_alloc(64);
+  g_snprintf(buffer, 64, "Vendor 0x%08x", vendorId);
   return buffer;
 } /*diameter_vendor_to_str */
 
@@ -933,7 +1023,7 @@ static gchar *
 diameter_command_to_str(guint32 commandCode, guint32 vendorId)
 {
   CommandCode *probe;
-  static gchar buffer[64];
+  gchar *buffer=NULL;
   gchar *vendorName=NULL;
 
   switch(gbl_version) {
@@ -965,7 +1055,8 @@ diameter_command_to_str(guint32 commandCode, guint32 vendorId)
   if ( suppress_console_output == FALSE )
 	  g_warning("Diameter: Unable to find name for command code 0x%08x, Vendor \"%u\"!",
 			commandCode, vendorId);
-  snprintf(buffer, sizeof(buffer),
+  buffer=ep_alloc(64);
+  g_snprintf(buffer, 64,
 		   "Cmd-0x%08x", commandCode);
     break;
     case DIAMETER_RFC:
@@ -980,7 +1071,8 @@ diameter_command_to_str(guint32 commandCode, guint32 vendorId)
     if ( suppress_console_output == FALSE )
           g_warning("Diameter: Unable to find name for command code 0x%08x!",
                         commandCode);
-    snprintf(buffer, sizeof(buffer),
+    buffer=ep_alloc(64);
+    g_snprintf(buffer, 64,
                    "Cmd-0x%08x", commandCode);
     break;
   }
@@ -991,15 +1083,16 @@ diameter_command_to_str(guint32 commandCode, guint32 vendorId)
 static gchar *
 diameter_app_to_str(guint32 appId) {
   ApplicationId *probe;
-  static gchar buffer[64];
+  gchar *buffer;
 
   for (probe=ApplicationIdHead; probe; probe=probe->next) {
-	if (appId == probe->id) {
-	  return probe->name;
-	}
+    if (appId == probe->id) {
+      return probe->name;
+    }
   }
 
-  snprintf(buffer, sizeof(buffer), "Unknown");
+  buffer=ep_alloc(64);
+  g_snprintf(buffer, 64, "Unknown");
   return buffer;
 } /*diameter_app_to_str */
 
@@ -1043,7 +1136,7 @@ diameter_avp_get_type(guint32 avpCode, guint32 vendorId){
 static gchar *
 diameter_avp_get_name(guint32 avpCode, guint32 vendorId)
 {
-  static gchar buffer[64];
+  gchar *buffer;
   avpInfo *probe;
   gchar *vendorName=NULL;
 
@@ -1073,10 +1166,11 @@ diameter_avp_get_name(guint32 avpCode, guint32 vendorId)
 			avpCode, vendorId);
 
   /* If we don't find it, build a name string */
-  sprintf(buffer, "Unknown AVP:0x%08x", avpCode);
+  buffer=ep_alloc(64);
+  g_snprintf(buffer, 64, "Unknown AVP:0x%08x", avpCode);
   return buffer;
 } /* diameter_avp_get_name */
-static gchar *
+static const gchar *
 diameter_avp_get_value(guint32 avpCode, guint32 vendorId, guint32 avpValue)
 {
   avpInfo *probe;
@@ -1157,21 +1251,36 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int              BadPacket = FALSE;
   guint32          commandCode=0, pktLength=0;
   guint8           version=0, flags=0;
-  gchar            flagstr[64] = "<None>";
-  gchar           *fstr[] = {"RSVD7", "RSVD6", "RSVD5", "RSVD4", "RSVD3", "Error", "Proxyable", "Request" };
-  gchar            commandString[64], vendorName[64], applicationName[64];
+  gchar            *flagstr="<None>";
+  const gchar     *fstr[] = {"RSVD7", "RSVD6", "RSVD5", "RSVD4", "RSVD3", "Error", "Proxyable", "Request" };
+  gchar            *commandString=NULL, *vendorName=NULL, *applicationName=NULL, *commandStringType=NULL;
   gint        i;
   guint      bpos;
   static  int initialized=FALSE;
 
+  /* Keep track of preference settings affecting dictionary source */
+  static  gboolean previous_use_xml_dictionary=FALSE;
+  #define MAX_DICT_NAME_SIZE 256
+  static  gchar    previous_diameterDictionary[MAX_DICT_NAME_SIZE];
+
   /*
    * Only parse in dictionary if there are diameter packets to
    * dissect.
+   * Keeps track of preference settings and frees/reinitializes the
+   * dictionary when appropriate.
    */
-  if (!initialized) {
-	  /* Read in our dictionary, if it exists. */
-	  initializeDictionary();
-	  initialized=TRUE;
+  if (!initialized ||
+      (gbl_use_xml_dictionary != previous_use_xml_dictionary) ||
+      (strncmp(gbl_diameterDictionary,
+               previous_diameterDictionary,
+               MAX_DICT_NAME_SIZE) != 0)) {
+      /* Populate dictionary according to preferences */
+      initializeDictionary();
+      initialized=TRUE;
+
+      /* Record current preference settings */
+      previous_use_xml_dictionary = gbl_use_xml_dictionary;
+      strncpy(previous_diameterDictionary, gbl_diameterDictionary, MAX_DICT_NAME_SIZE);
   }
 
   /* Make entries in Protocol column and Info column on summary display */
@@ -1191,10 +1300,9 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   dh.hopByHopId = g_ntohl(dh.hopByHopId);
   dh.endToEndId = g_ntohl(dh.endToEndId);
   if (dh.vendorId) {
-	strcpy(vendorName,
-		   diameter_vendor_to_str(dh.vendorId, TRUE));
+	vendorName=diameter_vendor_to_str(dh.vendorId, TRUE);
   } else {
-	strcpy(vendorName, "None");
+	vendorName="None";
   }
   /* Do the bit twiddling */
   version = DIAM_GET_VERSION(dh);
@@ -1211,10 +1319,13 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       dh2.hopByHopId = g_ntohl(dh2.hopByHopId);
       dh2.endToEndId = g_ntohl(dh2.endToEndId);
       if (dh2.applicationId) {
- 	strcpy(applicationName,
-		   diameter_app_to_str(dh2.applicationId));
+        applicationName=diameter_app_to_str(dh2.applicationId);
+        /* If not found, it might be a vendor ID? */
+        if (strcmp(applicationName, "Unknown") == 0){
+          applicationName=diameter_vendor_to_str(dh2.applicationId,FALSE);
+        }
       } else {
-  	strcpy(applicationName, "None");
+        applicationName="None";
       }
       /* Do the bit twiddling */
       version = DIAM_GET_VERSION(dh2);
@@ -1227,36 +1338,43 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   /* Set up our flags */
   if (check_col(pinfo->cinfo, COL_INFO) || tree) {
+	int fslen;
+
+#define FLAG_STR_LEN 64
+	flagstr=ep_alloc(FLAG_STR_LEN);
 	flagstr[0]=0;
+	fslen=0;
 	for (i = 0; i < 8; i++) {
 	  bpos = 1 << i;
 	  if (flags & bpos) {
 		if (flagstr[0]) {
-		  strcat(flagstr, ", ");
+		  fslen+=MIN(FLAG_STR_LEN-fslen,
+			     g_snprintf(flagstr+fslen, FLAG_STR_LEN-fslen, ", "));
 		}
-		strcat(flagstr, fstr[i]);
+		fslen+=MIN(FLAG_STR_LEN-fslen,
+			   g_snprintf(flagstr+fslen, FLAG_STR_LEN-fslen, "%s", fstr[i]));
 	  }
 	}
-	if (strlen(flagstr) == 0) {
-	  strcpy(flagstr,"<None>");
+	if (flagstr[0] == 0) {
+	  flagstr="<None>";
 	}
   }
 
   /* Set up our commandString */
   switch(gbl_version) {
     case DIAMETER_V16:
-  strcpy(commandString, diameter_command_to_str(commandCode, dh.vendorId));
+      commandString=diameter_command_to_str(commandCode, dh.vendorId);
     break;
     case DIAMETER_RFC:
       /* FIXME: in RFC, is applicationID needed to decode the command code?  */
-      strcpy(commandString, diameter_command_to_str(commandCode, dh2.applicationId));
+      commandString=diameter_command_to_str(commandCode, dh2.applicationId);
     break;
   }
 
   if (flags & DIAM_FLAGS_R)
-	strcat(commandString, "-Request");
+	commandStringType="Request";
   else
-	strcat(commandString, "-Answer");
+	commandStringType="Answer";
 
   /* Short packet.  Should have at LEAST one avp */
   if (pktLength < MIN_DIAMETER_SIZE) {
@@ -1278,34 +1396,34 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (check_col(pinfo->cinfo, COL_INFO)) {
     switch(gbl_version) {
       case DIAMETER_V16:
-	col_add_fstr(pinfo->cinfo, COL_INFO,
-				 "%s%s%s%s%s vendor=%s (hop-id=%u) (end-id=%u) RPE=%d%d%d",
-				 (BadPacket)?"***** Bad Packet!: ":"",
-				 (flags & DIAM_FLAGS_P)?"Proxyable ":"",
-				 (flags & DIAM_FLAGS_E)?" Error":"",
-				 ((BadPacket ||
-				   (flags & (DIAM_FLAGS_P|DIAM_FLAGS_E))) ?
-				   ": " : ""),
-				 commandString, vendorName,
-				 dh.hopByHopId, dh.endToEndId,
-				 (flags & DIAM_FLAGS_R)?1:0,
-				 (flags & DIAM_FLAGS_P)?1:0,
-				 (flags & DIAM_FLAGS_E)?1:0);
+        col_add_fstr(pinfo->cinfo, COL_INFO,
+                     "%s%s%s%s%s-%s vendor=%s (hop-id=%u) (end-id=%u) RPE=%d%d%d",
+                     (BadPacket)?"***** Bad Packet!: ":"",
+                     (flags & DIAM_FLAGS_P)?"Proxyable ":"",
+                     (flags & DIAM_FLAGS_E)?" Error":"",
+                     ((BadPacket ||
+                       (flags & (DIAM_FLAGS_P|DIAM_FLAGS_E))) ?
+                       ": " : ""),
+                     commandString, commandStringType, vendorName,
+                     dh.hopByHopId, dh.endToEndId,
+                     (flags & DIAM_FLAGS_R)?1:0,
+                     (flags & DIAM_FLAGS_P)?1:0,
+                     (flags & DIAM_FLAGS_E)?1:0);
       break;
       case DIAMETER_RFC:
-	col_add_fstr(pinfo->cinfo, COL_INFO,
-				 "%s%s%s%s%s app=%s (hop-id=%u) (end-id=%u) RPE=%d%d%d",
-				 (BadPacket)?"***** Bad Packet!: ":"",
-				 (flags & DIAM_FLAGS_P)?"Proxyable ":"",
-				 (flags & DIAM_FLAGS_E)?" Error":"",
-				 ((BadPacket ||
-				   (flags & (DIAM_FLAGS_P|DIAM_FLAGS_E))) ?
-				   ": " : ""),
-				 commandString, applicationName,
-				 dh2.hopByHopId, dh2.endToEndId,
-				 (flags & DIAM_FLAGS_R)?1:0,
-				 (flags & DIAM_FLAGS_P)?1:0,
-				 (flags & DIAM_FLAGS_E)?1:0);
+        col_add_fstr(pinfo->cinfo, COL_INFO,
+                     "%s%s%s%s%s-%s app=%s (hop-id=%u) (end-id=%u) RPE=%d%d%d",
+                     (BadPacket)?"***** Bad Packet!: ":"",
+                     (flags & DIAM_FLAGS_P)?"Proxyable ":"",
+                     (flags & DIAM_FLAGS_E)?" Error":"",
+                     ((BadPacket ||
+                       (flags & (DIAM_FLAGS_P|DIAM_FLAGS_E))) ?
+                       ": " : ""),
+                     commandString, commandStringType, applicationName,
+                     dh2.hopByHopId, dh2.endToEndId,
+                     (flags & DIAM_FLAGS_R)?1:0,
+                     (flags & DIAM_FLAGS_P)?1:0,
+                     (flags & DIAM_FLAGS_E)?1:0);
       break;
     }
   }
@@ -1335,9 +1453,9 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	offset += 3;
 
 	/* Flags */
-	tf = proto_tree_add_uint_format(diameter_tree, hf_diameter_flags, tvb,
-									offset , 1, flags, "Flags: 0x%02x (%s)", flags,
-									flagstr);
+	tf = proto_tree_add_uint_format_value(diameter_tree, hf_diameter_flags, tvb,
+					      offset, 1, flags, "0x%02x (%s)", flags,
+					      flagstr);
 	flags_tree = proto_item_add_subtree(tf, ett_diameter_avp_flags);
 	proto_tree_add_boolean(flags_tree, hf_diameter_flags_request, tvb, offset, 1, flags);
 	proto_tree_add_boolean(flags_tree, hf_diameter_flags_proxyable, tvb, offset, 1, flags);
@@ -1351,39 +1469,39 @@ dissect_diameter_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	offset += 1;
 
 	/* Command Code */
-	proto_tree_add_uint_format(diameter_tree, hf_diameter_code,
-							   tvb, offset, 3, commandCode, "Command Code: %s", commandString);
+	proto_tree_add_uint_format_value(diameter_tree, hf_diameter_code,
+	                                 tvb, offset, 3, commandCode, "%s-%s (%d)",
+	                                 commandString, commandStringType, commandCode);
 	offset += 3;
 
         switch(gbl_version) {
           case DIAMETER_V16:
-	/* Vendor Id */
-	proto_tree_add_uint_format(diameter_tree,hf_diameter_vendor_id,
-							   tvb, offset, 4,	dh.vendorId, "Vendor-Id: %s", vendorName);
-	offset += 4;
-	/* Hop-by-hop Identifier */
-	proto_tree_add_uint(diameter_tree, hf_diameter_hopbyhopid,
+
+			/* Vendor Id */
+			proto_tree_add_item(diameter_tree, hf_diameter_vendor_id, tvb, offset, 4, FALSE);
+			offset += 4;
+			/* Hop-by-hop Identifier */
+			proto_tree_add_uint(diameter_tree, hf_diameter_hopbyhopid,
 						tvb, offset, 4, dh.hopByHopId);
-	offset += 4;
-	/* End-to-end Identifier */
-	proto_tree_add_uint(diameter_tree, hf_diameter_endtoendid,
+			offset += 4;
+			/* End-to-end Identifier */
+			proto_tree_add_uint(diameter_tree, hf_diameter_endtoendid,
 						tvb, offset, 4, dh.endToEndId);
-	offset += 4;
-          break;
+			offset += 4;
+			break;
           case DIAMETER_RFC:
-	    /* Application Id */
-	    proto_tree_add_uint_format(diameter_tree,hf_diameter_application_id,
-                                       tvb, offset, 4,	dh2.applicationId, "Application-Id: %s", applicationName);
-	    offset += 4;
-	    /* Hop-by-hop Identifier */
-	    proto_tree_add_uint(diameter_tree, hf_diameter_hopbyhopid,
+		    /* Application Id */
+			proto_tree_add_item(diameter_tree, hf_diameter_application_id, tvb, offset, 4, FALSE);
+		    offset += 4;
+		    /* Hop-by-hop Identifier */
+		    proto_tree_add_uint(diameter_tree, hf_diameter_hopbyhopid,
 						tvb, offset, 4, dh2.hopByHopId);
-	    offset += 4;
-	    /* End-to-end Identifier */
-	    proto_tree_add_uint(diameter_tree, hf_diameter_endtoendid,
+		    offset += 4;
+		    /* End-to-end Identifier */
+		    proto_tree_add_uint(diameter_tree, hf_diameter_endtoendid,
 						tvb, offset, 4, dh2.endToEndId);
-	    offset += 4;
-          break;
+		    offset += 4;
+	          break;
         }
 
 
@@ -1512,11 +1630,11 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 {
   /* adds the attribute value pairs to the tree */
   e_avphdr avph;
-  gchar avpTypeString[64];
-  gchar avpNameString[64];
-  gchar *valstr;
+  const gchar *avpTypeString;
+  const gchar *avpNameString;
+  const gchar *valstr;
   guint32 vendorId=0;
-  gchar    vendorName[64];
+  gchar    *vendorName;
   int hdrLength;
   int fixAmt;
   proto_tree *avpi_tree;
@@ -1525,7 +1643,7 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
   proto_tree *group_tree;
   proto_item *grouptf;
   proto_item *avptf;
-  char buffer[1024];
+  char *buffer;
   int BadPacket = FALSE;
   guint32 avpLength;
   guint8 flags;
@@ -1535,8 +1653,8 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
   gint32 packetLength;
   size_t avpDataLength;
   int avpType;
-  gchar flagstr[64] = "<None>";
-  gchar *fstr[] = {"RSVD7", "RSVD6", "RSVD5", "RSVD4", "RSVD3", "Protected", "Mandatory", "Vendor-Specific" };
+  gchar *flagstr="<None>";
+  const gchar *fstr[] = {"RSVD7", "RSVD6", "RSVD5", "RSVD4", "RSVD3", "Protected", "Mandatory", "Vendor-Specific" };
   gint        i;
   guint      bpos;
 
@@ -1574,18 +1692,25 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 
 	/* Set up our flags string */
 	if (check_col(pinfo->cinfo, COL_INFO) || avp_tree) {
+	  int fslen;
+
+#define FLAG_STR_LEN 64
+	  flagstr=ep_alloc(FLAG_STR_LEN);
 	  flagstr[0]=0;
+	  fslen=0;
 	  for (i = 0; i < 8; i++) {
 		bpos = 1 << i;
 		if (flags & bpos) {
 		  if (flagstr[0]) {
-			strcat(flagstr, ", ");
+			fslen+=MIN(FLAG_STR_LEN-fslen,
+				   g_snprintf(flagstr+fslen, FLAG_STR_LEN-fslen, ", "));
 		  }
-		  strcat(flagstr, fstr[i]);
+		  fslen+=MIN(FLAG_STR_LEN-fslen,
+			     g_snprintf(flagstr+fslen, FLAG_STR_LEN-fslen, "%s", fstr[i]));
 		}
 	  }
-	  if (strlen(flagstr) == 0) {
-		strcpy(flagstr,"<None>");
+	  if (flagstr[0] == 0) {
+		flagstr="<None>";
 	  }
 	}
 
@@ -1602,10 +1727,9 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 	}
 
 	if (vendorId) {
-	  strcpy(vendorName,
-			 diameter_vendor_to_str(vendorId, TRUE));
+	  vendorName=diameter_vendor_to_str(vendorId, TRUE);
 	} else {
-	  vendorName[0]='\0';
+	  vendorName="";
 	}
 
 	/* Check for bad length */
@@ -1649,10 +1773,10 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 	}
 
 	/* Make avp Name & type */
-	strcpy(avpTypeString, val_to_str(diameter_avp_get_type(avph.avp_code,vendorId),
+	avpTypeString=val_to_str(diameter_avp_get_type(avph.avp_code,vendorId),
 									 TypeValues,
-									 "Unknown-Type: 0x%08x"));
-	strcpy(avpNameString, diameter_avp_get_name(avph.avp_code, vendorId));
+									 "Unknown-Type: 0x%08x");
+	avpNameString=diameter_avp_get_name(avph.avp_code, vendorId);
 
 	avptf = proto_tree_add_text(avp_tree, tvb,
 								offset, avpLength + fixAmt,
@@ -1664,13 +1788,13 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 
 	if (avpi_tree !=NULL) {
 	  /* Command Code */
-	  proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_code,
-								 tvb, offset, 4, avph.avp_code, "AVP Code: %s (%u)", avpNameString,avph.avp_code);
+	  proto_tree_add_uint_format_value(avpi_tree, hf_diameter_avp_code,
+					   tvb, offset, 4, avph.avp_code, "%s (%u)", avpNameString,avph.avp_code);
 	  offset += 4;
 
-	  tf = proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_flags, tvb,
-									  offset , 1, flags, "Flags: 0x%02x (%s)", flags,
-									  flagstr);
+	  tf = proto_tree_add_uint_format_value(avpi_tree, hf_diameter_avp_flags, tvb,
+						offset, 1, flags, "0x%02x (%s)", flags,
+						flagstr);
 	  flags_tree = proto_item_add_subtree(tf, ett_diameter_avp_flags);
 	  proto_tree_add_boolean(flags_tree, hf_diameter_avp_flags_vendor_specific, tvb, offset, 1, flags);
 	  proto_tree_add_boolean(flags_tree, hf_diameter_avp_flags_mandatory, tvb, offset, 1, flags);
@@ -1687,8 +1811,8 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 	  offset += 3;
 
 	  if (flags & AVP_FLAGS_V) {
-		proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_vendor_id,
-								   tvb, offset, 4, vendorId, "Vendor-Id: %s", vendorName);
+		proto_tree_add_uint_format_value(avpi_tree, hf_diameter_avp_vendor_id,
+						 tvb, offset, 4, vendorId, "%s", vendorName);
 		offset += 4;
 	  }
 
@@ -1711,7 +1835,8 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 
 	  switch(avpType) {
 	  case DIAMETER_GROUPED:
-		sprintf(buffer, "%s Grouped AVPs", avpNameString);
+		buffer=ep_alloc(256);
+		g_snprintf(buffer, 256, "%s Grouped AVPs", avpNameString);
 		/* Recursively call ourselves */
 		grouptf = proto_tree_add_text(avpi_tree,
 									  tvb, offset, tvb_length(tvb),
@@ -1818,6 +1943,24 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 		}
 		break;
 
+	  case DIAMETER_UNSIGNED32ENUM:
+		if (avpDataLength == 4) {
+		  guint32 data;
+
+		  data = tvb_get_ntohl(tvb, offset);
+		  valstr = diameter_avp_get_value(avph.avp_code, vendorId, data);
+		  proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_data_uint32,
+					     tvb, offset, avpDataLength, data,
+					     "Value: 0x%08x (%u): %s", data,
+					     data, valstr);
+		} else {
+		  proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+					      tvb, offset, avpDataLength,
+					      tvb_get_ptr(tvb, offset, avpDataLength),
+					      "Error!  Bad Enumerated Length");
+		}
+		break;
+
 	  case DIAMETER_INTEGER64:
 		if (avpDataLength == 8) {
 		  proto_tree_add_item(avpi_tree, hf_diameter_avp_data_int64,
@@ -1845,7 +1988,6 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 	  case DIAMETER_TIME:
 		if (avpDataLength == 4) {
 		  nstime_t data;
-		  gchar buffer[64];
 		  struct tm *gmtp;
 
 		  data.secs = tvb_get_ntohl(tvb, offset);
@@ -1855,6 +1997,7 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 				data.nsecs = 0;
 
 				gmtp = gmtime(&data.secs);
+				buffer=ep_alloc(64);
 				strftime(buffer, 64,
 				"%a, %d %b %Y %H:%M:%S UTC", gmtp);
 
@@ -1895,14 +2038,7 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 
 	  case DIAMETER_VENDOR_ID:
 		if (avpDataLength == 4) {
-		  guint32 data;
-
-		  data = tvb_get_ntohl(tvb, offset);
-		  valstr = diameter_vendor_to_str(data, TRUE);
-		  proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_data_uint32,
-					     tvb, offset, avpDataLength, data,
-					     "Vendor ID: %s (0x%08x)", valstr,
-					     data);
+		  proto_tree_add_item(avpi_tree, hf_diameter_vendor_id, tvb, offset, avpDataLength, FALSE);
 		} else {
 		  proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
 					      tvb, offset, avpDataLength,
@@ -1919,8 +2055,8 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 		  valstr = diameter_app_to_str(data);
 		  proto_tree_add_uint_format(avpi_tree, hf_diameter_avp_data_uint32,
 					     tvb, offset, avpDataLength, data,
-					     "Application ID: %s (0x%08x)",
-					     valstr, data);
+					     "Application ID: %s %d (0x%08x)",
+					     valstr, data, data);
 		} else {
 		  proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
 					      tvb, offset, avpDataLength,
@@ -2050,10 +2186,10 @@ proto_register_diameter(void)
 		  { "Command Code","diameter.code", FT_UINT24, BASE_DEC,
 		    NULL, 0x0, "", HFILL }},
 		{ &hf_diameter_vendor_id,
-		  { "VendorId",	"diameter.vendorId", FT_UINT32, BASE_DEC, NULL,
+		  { "VendorId",	"diameter.vendorId", FT_UINT32, BASE_DEC, VALS(sminmpec_values),
 			0x0,"", HFILL }},
 		{ &hf_diameter_application_id,
-		  { "ApplicationId",	"diameter.applicationId", FT_UINT32, BASE_DEC, NULL,
+		  { "ApplicationId",	"diameter.applicationId", FT_UINT32, BASE_DEC, VALS(diameter_application_id_vals),
 			0x0,"", HFILL }},
 		{ &hf_diameter_hopbyhopid,
 		  { "Hop-by-Hop Identifier", "diameter.hopbyhopid", FT_UINT32,
@@ -2154,7 +2290,7 @@ proto_register_diameter(void)
 	module_t *diameter_module;
 	gchar *default_diameterDictionary;
 
-	proto_diameter = proto_register_protocol ("Diameter Protocol", "Diameter", "diameter");
+	proto_diameter = proto_register_protocol ("Diameter Protocol", "DIAMETER", "diameter");
 	proto_register_field_array(proto_diameter, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
@@ -2162,7 +2298,7 @@ proto_register_diameter(void)
 	diameter_module = prefs_register_protocol(proto_diameter,
 											  proto_reg_handoff_diameter);
 	/* Register a configuration option for Diameter version */
-  prefs_register_enum_preference(diameter_module, "version", "Diameter version", "Standard version used for decoding", (gint *)&gbl_version, options, FALSE);
+	prefs_register_enum_preference(diameter_module, "version", "Diameter version", "Standard version used for decoding", (gint *)&gbl_version, options, FALSE);
 
 	prefs_register_uint_preference(diameter_module, "tcp.port",
 								   "Diameter TCP Port",
@@ -2196,6 +2332,16 @@ proto_register_diameter(void)
 	 */
 	g_free(default_diameterDictionary);
 
+	/*
+	 * Make use of the dictionary optional.  Avoids error popups if xml library
+	 * or dictionary file aren't available.
+	 */
+	prefs_register_bool_preference(diameter_module, "dictionary.use",
+	                               "Attempt to load/use Diameter XML Dictionary",
+	                               "Only attempt to load and use the Diameter XML "
+	                               "Dictionary when this option is selected",
+	                               &gbl_use_xml_dictionary);
+
 	/* Desegmentation */
 	prefs_register_bool_preference(diameter_module, "desegment",
                                    "Reassemble Diameter messages\nspanning multiple TCP segments",
@@ -2210,7 +2356,7 @@ proto_register_diameter(void)
 			&allow_zero_as_app_id);
 	/* Register some preferences we no longer support, so we can report
 	   them as obsolete rather than just illegal. */
-	/* Supress console output or not */
+	/* Suppress console output or not */
 	prefs_register_bool_preference(diameter_module, "suppress_console_output",
 			"Suppress console output for unknown AVP:s Flags etc.",
 			"If console output for errors should be suppressed or not",
