@@ -4,7 +4,7 @@
  * RFC 1034, RFC 1035
  * RFC 2136 for dynamic DNS
  *
- * $Id: packet-dns.c 14793 2005-06-27 07:50:11Z guy $
+ * $Id: packet-dns.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Copyright 2004, Nicolas DICHTEL - 6WIND - <nicolas.dichtel@6wind.com>
  * Ethereal - Network traffic analyzer
@@ -34,19 +34,16 @@
 #include <string.h>
 #include <memory.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <glib.h>
-#include <epan/ipv6-utils.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/ipproto.h>
 #include <epan/addr_resolv.h>
 #include "packet-dns.h"
 #include "packet-tcp.h"
 #include <epan/prefs.h>
 #include <epan/strutil.h>
+#include <epan/emem.h>
 
 static int proto_dns = -1;
 static int hf_dns_length = -1;
@@ -77,6 +74,14 @@ static int hf_dns_rr_type = -1;
 static int hf_dns_rr_class = -1;
 static int hf_dns_rr_ttl = -1;
 static int hf_dns_rr_len = -1;
+static int hf_dns_tsig_error = -1;
+static int hf_dns_tsig_fudge = -1;
+static int hf_dns_tsig_mac_size = -1;
+static int hf_dns_tsig_mac = -1;
+static int hf_dns_tsig_original_id = -1;
+static int hf_dns_tsig_algorithm_name = -1;
+static int hf_dns_tsig_other_len = -1;
+static int hf_dns_tsig_other_data = -1;
 
 static gint ett_dns = -1;
 static gint ett_dns_qd = -1;
@@ -86,6 +91,9 @@ static gint ett_dns_ans = -1;
 static gint ett_dns_flags = -1;
 static gint ett_t_key_flags = -1;
 static gint ett_t_key = -1;
+static gint ett_dns_mac = -1;
+
+static dissector_table_t dns_tsig_dissector_table=NULL;
 
 /* desegmentation of DNS over TCP */
 static gboolean dns_desegment = TRUE;
@@ -439,11 +447,13 @@ dns_type_description (guint type)
   };
   const char *short_name;
   const char *long_name;
-  static char strbuf[1024+1];
+#define MAX_STRBUF_LEN 1024
+  char *strbuf;
 
+  strbuf=ep_alloc(MAX_STRBUF_LEN);
   short_name = dns_type_name(type);
   if (short_name == NULL) {
-    snprintf(strbuf, sizeof strbuf, "Unknown (%u)", type);
+    g_snprintf(strbuf, MAX_STRBUF_LEN, "Unknown (%u)", type);
     return strbuf;
   }
   if (type < sizeof(type_names)/sizeof(type_names[0]))
@@ -482,9 +492,9 @@ dns_type_description (guint type)
   }
 
   if (long_name != NULL)
-    snprintf(strbuf, sizeof strbuf, "%s (%s)", short_name, long_name);
+    g_snprintf(strbuf, MAX_STRBUF_LEN, "%s (%s)", short_name, long_name);
   else
-    snprintf(strbuf, sizeof strbuf, "%s", short_name);
+    g_snprintf(strbuf, MAX_STRBUF_LEN, "%s", short_name);
   return strbuf;
 }
 
@@ -505,23 +515,33 @@ dns_class_name(int class)
   return val_to_str(class, dns_classes, "Unknown (%u)");
 }
 
+/* This function returns the number of bytes consumed and the expanded string
+ * in *name.
+ * The string is allocated with ep scope and does not need to be free()d.
+ * it will be automatically free()d when the packet has been dissected.
+ */
 int
 get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
-    char *name, int maxname)
+    char **name)
 {
   int start_offset = offset;
-  char *np = name;
+  char *np;
   int len = -1;
   int chars_processed = 0;
   int data_size = tvb_reported_length_remaining(tvb, dns_data_offset);
   int component_len;
   int indir_offset;
+  int maxname;
 
   const int min_len = 1;	/* Minimum length of encoded name (for root) */
 	/* If we're about to return a value (probably negative) which is less
 	 * than the minimum length, we're looking at bad data and we're liable
 	 * to put the dissector into a loop.  Instead we throw an exception */
 
+  maxname=MAXDNAME;
+  *name=ep_alloc(maxname);
+  np=*name;
+
   maxname--;	/* reserve space for the trailing '\0' */
   for (;;) {
     component_len = tvb_get_guint8(tvb, offset);
@@ -533,7 +553,7 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 
     case 0x00:
       /* Label */
-      if (np != name) {
+      if (np != *name) {
       	/* Not the first component - put in a '.'. */
         if (maxname > 0) {
           *np++ = '.';
@@ -567,9 +587,9 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 	  label_len = (bit_count - 1) / 8 + 1;
 
 	  if (maxname > 0) {
-	    print_len = snprintf(np, maxname + 1, "\\[x");
+	    print_len = g_snprintf(np, maxname + 1, "\\[x");
 	    if (print_len != -1 && print_len <= maxname) {
-	      /* Some versions of snprintf return -1 if they'd truncate
+	      /* Some versions of g_snprintf return -1 if they'd truncate
 	         the output.  Others return <buf_size> or greater. */
 	      np += print_len;
 	      maxname -= print_len;
@@ -581,10 +601,10 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 	  }
 	  while(label_len--) {
 	    if (maxname > 0) {
-	      print_len = snprintf(np, maxname + 1, "%02x",
+	      print_len = g_snprintf(np, maxname + 1, "%02x",
 	        tvb_get_guint8(tvb, offset));
 	      if (print_len != -1 && print_len <= maxname) {
-		/* Some versions of snprintf return -1 if they'd truncate
+		/* Some versions of g_snprintf return -1 if they'd truncate
 		 the output.  Others return <buf_size> or greater. */
 		np += print_len;
 		maxname -= print_len;
@@ -597,9 +617,9 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 	    offset++;
 	  }
 	  if (maxname > 0) {
-	    print_len = snprintf(np, maxname + 1, "/%d]", bit_count);
+	    print_len = g_snprintf(np, maxname + 1, "/%d]", bit_count);
 	    if (print_len != -1 && print_len <= maxname) {
-	      /* Some versions of snprintf return -1 if they'd truncate
+	      /* Some versions of g_snprintf return -1 if they'd truncate
 	         the output.  Others return <buf_size> or greater. */
 	      np += print_len;
 	      maxname -= print_len;
@@ -613,7 +633,7 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 	break;
 
       default:
-	strcpy(name, "<Unknown extended label>");
+	*name="<Unknown extended label>";
 	/* Parsing will propably fail from here on, since the */
 	/* label length is unknown... */
 	len = offset - start_offset;
@@ -643,7 +663,7 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
          will make us look at some character again, which means we're
 	 looping. */
       if (chars_processed >= data_size) {
-        strcpy(name, "<Name contains a pointer that loops>");
+        *name="<Name contains a pointer that loops>";
         if (len < min_len)
           THROW(ReportedBoundsError);
         return len;
@@ -660,8 +680,8 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
   if (len < 0)
     len = offset - start_offset;
   /* Zero-length name means "root server" */
-  if (*name == '\0')
-    strcpy(name, "<Root>");
+  if (**name == '\0')
+    *name="<Root>";
   if (len < min_len)
     THROW(ReportedBoundsError);
   return len;
@@ -670,16 +690,15 @@ get_dns_name(tvbuff_t *tvb, int offset, int dns_data_offset,
 
 static int
 get_dns_name_type_class(tvbuff_t *tvb, int offset, int dns_data_offset,
-    char *name_ret, int *name_len_ret, int *type_ret, int *class_ret)
+    char **name_ret, int *name_len_ret, int *type_ret, int *class_ret)
 {
   int len;
   int name_len;
   int type;
   int class;
-  char name[MAXDNAME];
   int start_offset = offset;
 
-  name_len = get_dns_name(tvb, offset, dns_data_offset, name, sizeof(name));
+  name_len = get_dns_name(tvb, offset, dns_data_offset, name_ret);
   offset += name_len;
 
   type = tvb_get_ntohs(tvb, offset);
@@ -688,7 +707,6 @@ get_dns_name_type_class(tvbuff_t *tvb, int offset, int dns_data_offset,
   class = tvb_get_ntohs(tvb, offset);
   offset += 2;
 
-  strcpy (name_ret, name);
   *type_ret = type;
   *class_ret = class;
   *name_len_ret = name_len;
@@ -738,7 +756,7 @@ rfc1867_angle(tvbuff_t *tvb, int offset, const char *nsew)
   angle = angle / 60;
   minutes = angle % 60;
   degrees = angle / 60;
-  sprintf(buf, "%u deg %u min %u.%03u sec %c", degrees, minutes, secs,
+  g_snprintf(buf, sizeof(buf), "%u deg %u min %u.%03u sec %c", degrees, minutes, secs,
 		tsecs, direction);
   return buf;
 }
@@ -748,7 +766,7 @@ dissect_dns_query(tvbuff_t *tvb, int offset, int dns_data_offset,
   column_info *cinfo, proto_tree *dns_tree)
 {
   int len;
-  char name[MAXDNAME];
+  char *name;
   char *name_out;
   int name_len;
   int type;
@@ -761,7 +779,7 @@ dissect_dns_query(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   data_start = data_offset = offset;
 
-  len = get_dns_name_type_class(tvb, offset, dns_data_offset, name, &name_len,
+  len = get_dns_name_type_class(tvb, offset, dns_data_offset, &name, &name_len,
     &type, &class);
   data_offset += len;
 
@@ -929,7 +947,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   column_info *cinfo, proto_tree *dns_tree, packet_info *pinfo)
 {
   int len;
-  char name[MAXDNAME];
+  char *name;
   char *name_out;
   int name_len;
   int type;
@@ -947,7 +965,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   data_start = data_offset = offset;
   cur_offset = offset;
 
-  len = get_dns_name_type_class(tvb, offset, dns_data_offset, name, &name_len,
+  len = get_dns_name_type_class(tvb, offset, dns_data_offset, &name, &name_len,
     &type, &class);
   data_offset += len;
   cur_offset += len;
@@ -1014,10 +1032,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_NS:
     {
-      char ns_name[MAXDNAME];
+      char *ns_name;
       int ns_name_len;
 
-      ns_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, ns_name, sizeof(ns_name));
+      ns_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, &ns_name);
       name_out = format_text(ns_name, strlen(ns_name));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1031,10 +1049,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_CNAME:
     {
-      char cname[MAXDNAME];
+      char *cname;
       int cname_len;
 
-      cname_len = get_dns_name(tvb, cur_offset, dns_data_offset, cname, sizeof(cname));
+      cname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &cname);
       name_out = format_text(cname, strlen(cname));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1048,9 +1066,9 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_SOA:
     {
-      char mname[MAXDNAME];
+      char *mname;
       int mname_len;
-      char rname[MAXDNAME];
+      char *rname;
       int rname_len;
       guint32 serial;
       guint32 refresh;
@@ -1058,7 +1076,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 expire;
       guint32 minimum;
 
-      mname_len = get_dns_name(tvb, cur_offset, dns_data_offset, mname, sizeof(mname));
+      mname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &mname);
       name_out = format_text(mname, strlen(mname));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1068,7 +1086,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 		       name_out);
 	cur_offset += mname_len;
 
-	rname_len = get_dns_name(tvb, cur_offset, dns_data_offset, rname, sizeof(rname));
+	rname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &rname);
         name_out = format_text(rname, strlen(rname));
 	proto_tree_add_text(rr_tree, tvb, cur_offset, rname_len, "Responsible authority's mailbox: %s",
 		       name_out);
@@ -1103,10 +1121,10 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_PTR:
     {
-      char pname[MAXDNAME];
+      char *pname;
       int pname_len;
 
-      pname_len = get_dns_name(tvb, cur_offset, dns_data_offset, pname, sizeof(pname));
+      pname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &pname);
       name_out = format_text(pname, strlen(pname));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1128,10 +1146,12 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int mask;
       int port_num;
       int i;
-      static GString *bitnames = NULL;
+      char *bitnames, *strptr;
 
-      if (bitnames == NULL)
-	bitnames = g_string_sized_new(128);
+#define MAX_STR_LEN 128
+      bitnames=ep_alloc(MAX_STR_LEN);
+      bitnames[0]=0;
+      strptr=bitnames;
 
       if (rr_len < 4) {
 	if (dns_tree != NULL)
@@ -1161,23 +1181,28 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	  bits = tvb_get_guint8(tvb, cur_offset);
 	  if (bits != 0) {
 	    mask = 1<<7;
-	    g_string_truncate(bitnames, 0);
+            bitnames[0]=0;
+            strptr=bitnames;
 	    for (i = 0; i < 8; i++) {
 	      if (bits & mask) {
-		if (bitnames->len != 0)
-		  g_string_append(bitnames, ", ");
+		if (strptr!=bitnames)
+		  strptr += MIN(MAX_STR_LEN-(strptr-bitnames),
+				g_snprintf(strptr, MAX_STR_LEN-(strptr-bitnames), ", "));
 		switch (protocol) {
 
 		case IP_PROTO_TCP:
-		  g_string_append(bitnames, get_tcp_port(port_num));
+		  strptr += MIN(MAX_STR_LEN-(strptr-bitnames),
+				g_snprintf(strptr, MAX_STR_LEN-(strptr-bitnames), get_tcp_port(port_num)));
 		  break;
 
 		case IP_PROTO_UDP:
-		  g_string_append(bitnames, get_udp_port(port_num));
+		  strptr += MIN(MAX_STR_LEN-(strptr-bitnames),
+				g_snprintf(strptr, MAX_STR_LEN-(strptr-bitnames), get_udp_port(port_num)));
 		  break;
 
 		default:
-		  g_string_sprintfa(bitnames, "%u", port_num);
+		  strptr += MIN(MAX_STR_LEN-(strptr-bitnames),
+				g_snprintf(strptr, MAX_STR_LEN-(strptr-bitnames), "%u", port_num));
 		  break;
 	        }
 	      }
@@ -1185,7 +1210,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	      port_num++;
 	    }
 	    proto_tree_add_text(rr_tree, tvb, cur_offset, 1,
-		"Bits: 0x%02x (%s)", bits, bitnames->str);
+		"Bits: 0x%02x (%s)", bits, bitnames);
 	  } else
 	    port_num += 8;
 	  cur_offset += 1;
@@ -1228,11 +1253,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   case T_MX:
     {
       guint16 preference = 0;
-      char mx_name[MAXDNAME];
+      char *mx_name;
       int mx_name_len;
 
       preference = tvb_get_ntohs(tvb, cur_offset);
-      mx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, mx_name, sizeof(mx_name));
+      mx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, &mx_name);
       name_out = format_text(mx_name, strlen(mx_name));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %u %s", preference, name_out);
@@ -1271,7 +1296,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int rr_len = data_len;
       guint16 type_covered;
       nstime_t nstime;
-      char signer_name[MAXDNAME];
+      char *signer_name;
       int signer_name_len;
 
       if (dns_tree != NULL) {
@@ -1330,7 +1355,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	cur_offset += 2;
 	rr_len -= 2;
 
-	signer_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, signer_name, sizeof(signer_name));
+	signer_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, &signer_name);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, signer_name_len,
 		"Signer's name: %s",
 		format_text(signer_name, strlen(signer_name)));
@@ -1433,7 +1458,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int rr_len = data_len;
       guint8 gw_type, algo;
       const guint8 *addr;
-      char gw[MAXDNAME];
+      char *gw;
       int gw_name_len;
       static const value_string gw_algo[] = {
 	  { 1,     "DSA" },
@@ -1481,7 +1506,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	     rr_len -= 16;
 	     break;
 	   case 3:
-	     gw_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, gw, sizeof(gw));
+	     gw_name_len = get_dns_name(tvb, cur_offset, dns_data_offset, &gw);
 	     proto_tree_add_text(rr_tree, tvb, cur_offset, gw_name_len,
 				 "Gateway: %s", format_text(gw, strlen(gw)));
 
@@ -1526,7 +1551,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       unsigned short pre_len;
       unsigned short suf_len;
       unsigned short suf_octet_count;
-      char pname[MAXDNAME];
+      char *pname;
       int pname_len;
       int a6_offset;
       int suf_offset;
@@ -1539,18 +1564,18 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       suf_octet_count = suf_len ? (suf_len - 1) / 8 + 1 : 0;
       /* Pad prefix */
       for (suf_offset = 0; suf_offset < 16 - suf_octet_count; suf_offset++) {
-        suffix.s6_addr8[suf_offset] = 0;
+        suffix.bytes[suf_offset] = 0;
       }
       for (; suf_offset < 16; suf_offset++) {
-        suffix.s6_addr8[suf_offset] = tvb_get_guint8(tvb, cur_offset);
+        suffix.bytes[suf_offset] = tvb_get_guint8(tvb, cur_offset);
         cur_offset++;
       }
 
       if (pre_len > 0) {
         pname_len = get_dns_name(tvb, cur_offset, dns_data_offset,
-                                 pname, sizeof(pname));
+                                 &pname);
       } else {
-        strcpy(pname, "");
+        pname="";
         pname_len = 0;
       }
       name_out = format_text(pname, strlen(pname));
@@ -1585,11 +1610,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_DNAME:
     {
-      char dname[MAXDNAME];
+      char *dname;
       int dname_len;
 
       dname_len = get_dns_name(tvb, cur_offset, dns_data_offset,
-			       dname, sizeof(dname));
+			       &dname);
       name_out = format_text(dname, strlen(dname));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1633,7 +1658,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	  cur_offset += 4;
 
 	  proto_tree_add_text(rr_tree, tvb, cur_offset, 4, "Altitude: %g m",
-				(tvb_get_ntohl(tvb, cur_offset) - 10000000)/100.0);
+				((gint32)tvb_get_ntohl(tvb, cur_offset) - 10000000)/100.0);
 	} else
 	  proto_tree_add_text(rr_tree, tvb, cur_offset, data_len, "Data");
       }
@@ -1643,7 +1668,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   case T_NSEC:
     {
       int rr_len = data_len;
-      char next_domain_name[MAXDNAME];
+      char *next_domain_name;
       int next_domain_name_len;
       int rr_type;
       guint8 bits;
@@ -1651,7 +1676,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int i;
 
       next_domain_name_len = get_dns_name(tvb, cur_offset, dns_data_offset,
-			next_domain_name, sizeof(next_domain_name));
+			&next_domain_name);
       name_out = format_text(next_domain_name, strlen(next_domain_name));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1691,7 +1716,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   case T_NXT:
     {
       int rr_len = data_len;
-      char next_domain_name[MAXDNAME];
+      char *next_domain_name;
       int next_domain_name_len;
       int rr_type;
       guint8 bits;
@@ -1699,7 +1724,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       int i;
 
       next_domain_name_len = get_dns_name(tvb, cur_offset, dns_data_offset,
-			next_domain_name, sizeof(next_domain_name));
+			&next_domain_name);
       name_out = format_text(next_domain_name, strlen(next_domain_name));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -1732,11 +1757,11 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
   case T_KX:
     {
       guint16 preference = 0;
-      char kx_name[MAXDNAME];
+      char *kx_name;
       int kx_name_len;
 
       preference = tvb_get_ntohs(tvb, cur_offset);
-      kx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, kx_name, sizeof(kx_name));
+      kx_name_len = get_dns_name(tvb, cur_offset + 2, dns_data_offset, &kx_name);
       name_out = format_text(kx_name, strlen(kx_name));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %u %s", preference, name_out);
@@ -1841,7 +1866,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_TKEY:
     {
-      char tkey_algname[MAXDNAME];
+      char *tkey_algname;
       int tkey_algname_len;
       guint16 tkey_mode, tkey_error, tkey_keylen, tkey_otherlen;
       int rr_len = data_len;
@@ -1858,7 +1883,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 	proto_tree *key_tree;
 	proto_item *key_item;
 
-	tkey_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, tkey_algname, sizeof(tkey_algname));
+	tkey_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &tkey_algname);
 	proto_tree_add_text(rr_tree, tvb, cur_offset, tkey_algname_len,
 		"Algorithm name: %s",
 		format_text(tkey_algname, strlen(tkey_algname)));
@@ -1975,19 +2000,17 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
   case T_TSIG:
     {
-      guint16 tsig_fudge;
-      guint16 tsig_originalid, tsig_error, tsig_timehi, tsig_siglen, tsig_otherlen;
+      guint16 tsig_error, tsig_timehi, tsig_siglen, tsig_otherlen;
       guint32 tsig_timelo;
-      char tsig_algname[MAXDNAME];
+      char *tsig_raw_algname, *tsig_algname;
       int tsig_algname_len;
       nstime_t nstime;
       int rr_len = data_len;
 
       if (dns_tree != NULL) {
-	tsig_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, tsig_algname, sizeof(tsig_algname));
-	proto_tree_add_text(rr_tree, tvb, cur_offset, tsig_algname_len,
-		"Algorithm name: %s",
-		format_text(tsig_algname, strlen(tsig_algname)));
+	tsig_algname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &tsig_raw_algname);
+	tsig_algname=format_text(tsig_raw_algname, strlen(tsig_raw_algname));
+	proto_tree_add_string(rr_tree, hf_dns_tsig_algorithm_name, tvb, cur_offset, tsig_algname_len, tsig_algname);
 	cur_offset += tsig_algname_len;
 	rr_len -= tsig_algname_len;
 
@@ -2004,55 +2027,65 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
 
 	if (rr_len < 2)
 	  goto bad_rr;
-	tsig_fudge = tvb_get_ntohs(tvb, cur_offset);
-	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Fudge: %u",
-		tsig_fudge);
+
+	proto_tree_add_item(rr_tree, hf_dns_tsig_fudge, tvb, cur_offset, 2, FALSE);
 	cur_offset += 2;
 	rr_len -= 2;
 
 	if (rr_len < 2)
 	  goto bad_rr;
 	tsig_siglen = tvb_get_ntohs(tvb, cur_offset);
-	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Signature length: %u", tsig_siglen);
+	proto_tree_add_item(rr_tree, hf_dns_tsig_mac_size, tvb, cur_offset, 2, FALSE);
 	cur_offset += 2;
 	rr_len -= 2;
 
 	if (tsig_siglen != 0) {
+	  proto_item *mac_item;
+	  proto_tree *mac_tree;
+	  tvbuff_t *sub_tvb;
+
 	  if (rr_len < tsig_siglen)
 	    goto bad_rr;
-	  proto_tree_add_text(rr_tree, tvb, cur_offset, tsig_siglen, "Signature");
+
+	  mac_item = proto_tree_add_item(rr_tree, hf_dns_tsig_mac, tvb, cur_offset, tsig_siglen, FALSE);
+	  mac_tree = proto_item_add_subtree(mac_item, ett_dns_mac);
+
+	  sub_tvb=tvb_new_subset(tvb, cur_offset, tsig_siglen, tsig_siglen);
+
+	  if(!dissector_try_string(dns_tsig_dissector_table, tsig_algname, sub_tvb, pinfo, mac_tree)){
+		proto_tree_add_text(mac_tree, sub_tvb, 0, tvb_length(sub_tvb), "No dissector for algorithm:%s", tsig_algname);
+	  }
+
 	  cur_offset += tsig_siglen;
 	  rr_len -= tsig_siglen;
 	}
 
 	if (rr_len < 2)
 	  goto bad_rr;
-	tsig_originalid = tvb_get_ntohs(tvb, cur_offset);
-	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Original id: %d",
-		tsig_originalid);
+	proto_tree_add_item(rr_tree, hf_dns_tsig_original_id, tvb, cur_offset, 2, FALSE);
 	cur_offset += 2;
 	rr_len -= 2;
 
 	if (rr_len < 2)
 	  goto bad_rr;
 	tsig_error = tvb_get_ntohs(tvb, cur_offset);
-        proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Error: %s",
-		val_to_str(tsig_error, rcode_vals,
-		val_to_str(tsig_error, tsigerror_vals, "Unknown error (%x)")));
+	proto_tree_add_uint_format(rr_tree, hf_dns_tsig_error, tvb, cur_offset, 2, tsig_error, "Error: %s (%d)",
+		val_to_str(tsig_error, rcode_vals,val_to_str(tsig_error, tsigerror_vals, "Unknown error")),
+		tsig_error);
 	cur_offset += 2;
 	rr_len -= 2;
 
 	if (rr_len < 2)
 	  goto bad_rr;
 	tsig_otherlen = tvb_get_ntohs(tvb, cur_offset);
-	proto_tree_add_text(rr_tree, tvb, cur_offset, 2, "Other length: %u", tsig_otherlen);
+	proto_tree_add_item(rr_tree, hf_dns_tsig_other_len, tvb, cur_offset, 2, FALSE);
 	cur_offset += 2;
 	rr_len -= 2;
 
 	if (tsig_otherlen != 0) {
 	  if (rr_len < tsig_otherlen)
 	    goto bad_rr;
-	  proto_tree_add_text(rr_tree, tvb, cur_offset, tsig_otherlen, "Other");
+	  proto_tree_add_item(rr_tree, hf_dns_tsig_other_data, tvb, cur_offset, tsig_otherlen, FALSE);
 	  cur_offset += tsig_otherlen;
 	  rr_len -= tsig_otherlen;
 	}
@@ -2130,7 +2163,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint32 local_flag;
       guint32 lookup_timeout;
       guint32 cache_timeout;
-      char dname[MAXDNAME];
+      char *dname;
       int dname_len;
 
       if (rr_len < 4)
@@ -2163,7 +2196,7 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       cur_offset += 4;
       rr_len -= 4;
 
-      dname_len = get_dns_name(tvb, cur_offset, dns_data_offset, dname, sizeof(dname));
+      dname_len = get_dns_name(tvb, cur_offset, dns_data_offset, &dname);
       name_out = format_text(dname, strlen(dname));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %s", name_out);
@@ -2180,14 +2213,14 @@ dissect_dns_answer(tvbuff_t *tvb, int offset, int dns_data_offset,
       guint16 priority = 0;
       guint16 weight = 0;
       guint16 port = 0;
-      char target[MAXDNAME];
+      char *target;
       int target_len;
 
       priority = tvb_get_ntohs(tvb, cur_offset);
       weight = tvb_get_ntohs(tvb, cur_offset+2);
       port = tvb_get_ntohs(tvb, cur_offset+4);
 
-      target_len = get_dns_name(tvb, cur_offset + 6, dns_data_offset, target, sizeof(target));
+      target_len = get_dns_name(tvb, cur_offset + 6, dns_data_offset, &target);
       name_out = format_text(target, strlen(target));
       if (cinfo != NULL)
 	col_append_fstr(cinfo, COL_INFO, " %u %u %u %s", priority, weight, port, name_out);
@@ -2237,7 +2270,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
 
   start_off = cur_off;
   if (dns_tree) {
-    char *s = (isupdate ?  "Zone" : "Queries");
+    const char *s = (isupdate ?  "Zone" : "Queries");
     ti = proto_tree_add_text(dns_tree, tvb, start_off, -1, s);
     qatree = proto_item_add_subtree(ti, ett_dns_qry);
   }
@@ -2253,7 +2286,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
 
 static int
 dissect_answer_records(tvbuff_t *tvb, int cur_off, int dns_data_offset,
-    int count, column_info *cinfo, proto_tree *dns_tree, char *name,
+    int count, column_info *cinfo, proto_tree *dns_tree, const char *name,
     packet_info *pinfo)
 {
   int start_off, add_off;
@@ -2286,7 +2319,8 @@ dissect_dns_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   proto_tree *dns_tree = NULL, *field_tree;
   proto_item *ti, *tf;
   guint16    id, flags, opcode, rcode, quest, ans, auth, add;
-  char buf[128+1];
+  char *buf;
+  int bufpos;
   int cur_off;
   gboolean isupdate;
 
@@ -2295,6 +2329,11 @@ dissect_dns_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   if (check_col(pinfo->cinfo, COL_INFO))
     col_clear(pinfo->cinfo, COL_INFO);
 
+#define MAX_BUF_SIZE (128+1)
+  buf=ep_alloc(MAX_BUF_SIZE);
+  buf[0]=0;
+  bufpos=0;
+
   /* To do: check for errs, etc. */
   id    = tvb_get_ntohs(tvb, offset + DNS_ID);
   flags = tvb_get_ntohs(tvb, offset + DNS_FLAGS);
@@ -2302,13 +2341,17 @@ dissect_dns_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   rcode  = (guint16)  (flags & F_RCODE);
 
   if (check_col(pinfo->cinfo, COL_INFO)) {
-    strcpy(buf, val_to_str(opcode, opcode_vals, "Unknown operation (%u)"));
+    bufpos=0;
+    bufpos+=MIN(MAX_BUF_SIZE-bufpos,
+		g_snprintf(buf+bufpos, MAX_BUF_SIZE-bufpos, "%s%s",
+		val_to_str(opcode, opcode_vals, "Unknown operation (%u)"),
+		(flags&F_RESPONSE)?" response":""));
+		
     if (flags & F_RESPONSE) {
-      strcat(buf, " response");
       if ((flags & F_RCODE) != RCODE_NOERROR) {
-        strcat(buf, ", ");
-        strcat(buf, val_to_str(flags & F_RCODE, rcode_vals,
-            "Unknown error (%u)"));
+	bufpos+=MIN(MAX_BUF_SIZE-bufpos,
+		    g_snprintf(buf+bufpos, MAX_BUF_SIZE-bufpos, ", %s",
+			val_to_str(flags & F_RCODE, rcode_vals, "Unknown error (%u)")));
       }
     }
     col_add_str(pinfo->cinfo, COL_INFO, buf);
@@ -2339,12 +2382,14 @@ dissect_dns_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     proto_tree_add_uint(dns_tree, hf_dns_transaction_id, tvb,
 			offset + DNS_ID, 2, id);
 
-    strcpy(buf, val_to_str(opcode, opcode_vals, "Unknown operation"));
+    bufpos=0;
+    bufpos+=MIN(MAX_BUF_SIZE-bufpos,
+		g_snprintf(buf+bufpos, MAX_BUF_SIZE-bufpos, "%s",
+			val_to_str(opcode, opcode_vals, "Unknown operation")));
     if (flags & F_RESPONSE) {
-      strcat(buf, " response");
-      strcat(buf, ", ");
-      strcat(buf, val_to_str(flags & F_RCODE, rcode_vals,
-            "Unknown error"));
+	    bufpos+=MIN(MAX_BUF_SIZE-bufpos,
+			g_snprintf(buf+bufpos, MAX_BUF_SIZE-bufpos, " response, %s",
+				val_to_str(flags & F_RCODE, rcode_vals, "Unknown error")));
     }
     tf = proto_tree_add_uint_format(dns_tree, hf_dns_flags, tvb,
 				    offset + DNS_FLAGS, 2,
@@ -2616,6 +2661,38 @@ proto_register_dns(void)
       { "Updates",       	"dns.count.updates",
 	FT_UINT16, BASE_DEC, NULL, 0x0,
 	"Number of updates records in packet", HFILL }},
+    { &hf_dns_tsig_original_id,
+      { "Original Id",       	"dns.tsig.original_id",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"Original Id", HFILL }},
+    { &hf_dns_tsig_error,
+      { "Error",       	"dns.tsig.error",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"Expanded RCODE for TSIG", HFILL }},
+    { &hf_dns_tsig_fudge,
+      { "Fudge",       	"dns.tsig.fudge",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"Number of bytes for the MAC", HFILL }},
+    { &hf_dns_tsig_mac_size,
+      { "MAC Size",       	"dns.tsig.mac_size",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"Number of bytes for the MAC", HFILL }},
+    { &hf_dns_tsig_other_len,
+      { "Other Len",       	"dns.tsig.other_len",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"Number of bytes for Other Data", HFILL }},
+    { &hf_dns_tsig_mac,
+      { "MAC",       	"dns.tsig.mac",
+	FT_NONE, BASE_NONE, NULL, 0x0,
+	"MAC", HFILL }},
+    { &hf_dns_tsig_other_data,
+      { "Other Data",       	"dns.tsig.other_data",
+	FT_BYTES, BASE_HEX, NULL, 0x0,
+	"Other Data", HFILL }},
+    { &hf_dns_tsig_algorithm_name,
+      { "Algorithm Name",      	"dns.tsig.algorithm_name",
+	FT_STRING, BASE_NONE, NULL, 0x0,
+	"Name of algorithm used for the MAC", HFILL }},
     { &hf_dns_count_add_rr,
       { "Additional RRs",      	"dns.count.add_rr",
 	FT_UINT16, BASE_DEC, NULL, 0x0,
@@ -2630,6 +2707,7 @@ proto_register_dns(void)
     &ett_dns_flags,
     &ett_t_key_flags,
     &ett_t_key,
+    &ett_dns_mac,
   };
   module_t *dns_module;
 
@@ -2643,6 +2721,8 @@ proto_register_dns(void)
     "Whether the DNS dissector should reassemble messages spanning multiple TCP segments."
     " To use this option, you must also enable \"Allow subdissectors to reassemble TCP streams\" in the TCP protocol settings.",
     &dns_desegment);
+
+  dns_tsig_dissector_table = register_dissector_table("dns.tsig.mac", "DNS TSIG MAC Dissectors", FT_STRING, BASE_NONE);
 }
 
 void
@@ -2663,4 +2743,5 @@ proto_reg_handoff_dns(void)
 
   gssapi_handle = find_dissector("gssapi");
   ntlmssp_handle = find_dissector("ntlmssp");
+
 }
