@@ -2,7 +2,7 @@
  * Definitions for packet disassembly structures and routines used both by
  * DNS and NBNS.
  *
- * $Id: packet-dns.h 12925 2005-01-02 09:40:26Z guy $
+ * $Id: packet-dns.h 16166 2005-10-08 12:31:06Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,9 +27,9 @@
 #ifndef __PACKET_DNS_H__
 #define __PACKET_DNS_H__
 
-char *dns_class_name(int class);
+const char *dns_class_name(int class);
 
-int get_dns_name(tvbuff_t *, int, int, char *, int);
+int get_dns_name(tvbuff_t *, int, int, char **);
 
 #define MAXDNAME        1025            /* maximum domain name length */
 
