@@ -2,7 +2,7 @@
  * Routines for dsi packet dissection
  * Copyright 2001, Randy McEoin <rmceoin@pe.com>
  *
- * $Id: packet-dsi.c 15027 2005-07-24 02:43:41Z sahlberg $
+ * $Id: packet-dsi.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,6 +33,7 @@
 
 #include <glib.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 
 #include <epan/prefs.h>
 #include "packet-tcp.h"
@@ -414,7 +415,7 @@ dissect_dsi_reply_get_status(tvbuff_t *tvb, proto_tree *tree, gint offset)
 			case 4: /* DNS */
 			case 5: /* SSH tunnel */
 				if (len > 2) {
-					tmp = ep_tvb_get_string(tvb, ofs +2, len -2);
+					tmp = tvb_get_ephemeral_string(tvb, ofs +2, len -2);
 					ti = proto_tree_add_text(adr_tree, tvb, ofs, len, "%s: %s", 
 								(type==4)?"dns":"ssh tunnel", tmp);
 					break;
@@ -467,7 +468,7 @@ dissect_dsi_reply_get_status(tvbuff_t *tvb, proto_tree *tree, gint offset)
 
 		ofs = utf_ofs;
 		ulen = tvb_get_ntohs(tvb, ofs);
-		tmp = ep_tvb_get_string(tvb, ofs + 2, ulen);
+		tmp = tvb_get_ephemeral_string(tvb, ofs + 2, ulen);
 		ti = proto_tree_add_text(tree, tvb, ofs, ulen + 2, "UTF8 server name: %s", tmp);
 		sub_tree = proto_item_add_subtree(ti, ett_dsi_utf8_name);
 		proto_tree_add_uint(sub_tree, hf_dsi_utf8_server_name_len, tvb, ofs, 2, ulen);
