@@ -2,7 +2,7 @@
  * Routines for EAP Extensible Authentication Protocol dissection
  * RFC 2284, RFC 3748
  *
- * $Id: packet-eap.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-eap.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -30,8 +30,9 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/conversation.h>
-#include "ppptypes.h"
+#include <epan/ppptypes.h>
 #include <epan/reassemble.h>
+#include <epan/emem.h>
 
 static int proto_eap = -1;
 static int hf_eap_code = -1;
@@ -163,7 +164,6 @@ static const value_string eap_type_vals[] = {
  * Attach to all conversations both pieces of information, to keep
  * track of EAP-TLS reassembly and the LEAP state machine.
  */
-static GMemChunk *conv_state_chunk = NULL;
 
 typedef struct {
 	int	eap_tls_seq;
@@ -171,8 +171,6 @@ typedef struct {
 	int	leap_state;
 } conv_state_t;
 
-static GMemChunk *frame_state_chunk = NULL;
-
 typedef struct {
 	int	info;	/* interpretation depends on EAP message type */
 } frame_state_t;
@@ -250,25 +248,6 @@ eaptls_defragment_init(void)
   fragment_table_init(&eaptls_fragment_table);
 }
 
-static void
-eap_init_protocol(void)
-{
-  if (conv_state_chunk != NULL)
-    g_mem_chunk_destroy(conv_state_chunk);
-  if (frame_state_chunk != NULL)
-    g_mem_chunk_destroy(frame_state_chunk);
-
-  conv_state_chunk = g_mem_chunk_new("conv_state_chunk",
-				     sizeof (conv_state_t),
-				     10 * sizeof (conv_state_t),
-				     G_ALLOC_ONLY);
-
-  frame_state_chunk = g_mem_chunk_new("frame_state_chunk",
-				      sizeof (frame_state_t),
-				     100 * sizeof (frame_state_t),
-				     G_ALLOC_ONLY);
-}
-
 static void
 dissect_eap_mschapv2(proto_tree *eap_tree, tvbuff_t *tvb, int offset,
 		     gint size)
@@ -597,8 +576,7 @@ dissect_eap_aka(proto_tree *eap_tree, tvbuff_t *tvb, int offset, gint size)
 }
 
 static int
-dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
-		 gboolean fragmented)
+dissect_eap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
   guint8      eap_code;
   guint8      eap_id;
@@ -676,7 +654,7 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     /*
      * Attach state information to the conversation.
      */
-    conversation_state = g_mem_chunk_alloc(conv_state_chunk);
+    conversation_state = se_alloc(sizeof (conv_state_t));
     conversation_state->eap_tls_seq = -1;
     conversation_state->eap_reass_cookie = 0;
     conversation_state->leap_state = -1;
@@ -695,18 +673,6 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   eap_len = tvb_get_ntohs(tvb, 2);
   len = eap_len;
 
-  if (fragmented) {
-    /*
-     * This is an EAP fragment inside, for example, RADIUS.  If we don't
-     * have all of the packet data, return the negative of the amount of
-     * additional data we need.
-     */
-    int reported_len = tvb_reported_length_remaining(tvb, 0);
-
-    if (reported_len < len)
-      return -(len - reported_len);
-  }
-
   if (tree) {
     ti = proto_tree_add_item(tree, proto_eap, tvb, 0, len, FALSE);
     eap_tree = proto_item_add_subtree(ti, ett_eap);
@@ -947,7 +913,7 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		 * This frame requires reassembly; remember the reassembly
 		 * ID for subsequent accesses to it.
 		 */
-		packet_state = g_mem_chunk_alloc(frame_state_chunk);
+		packet_state = se_alloc(sizeof (frame_state_t));
 		packet_state->info = eap_reass_cookie;
 		p_add_proto_data(pinfo->fd, proto_eap, packet_state);
 	      }
@@ -1003,6 +969,7 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 	    if (fd_head != NULL)            /* Reassembled  */
 	      {
+		proto_item *frag_tree_item;
 
 		next_tvb = tvb_new_real_data(fd_head->data,
 					     fd_head->len,
@@ -1011,7 +978,7 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		add_new_data_source(pinfo, next_tvb, "Reassembled EAP-TLS");
 
 		show_fragment_seq_tree(fd_head, &eaptls_frag_items,
-		    eap_tree, pinfo, next_tvb);
+		    eap_tree, pinfo, next_tvb, &frag_tree_item);
 
 		call_dissector(ssl_handle, next_tvb, pinfo, eap_tree);
 
@@ -1090,7 +1057,7 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	     * Remember the state for subsequent accesses to this
 	     * frame.
 	     */
-	    packet_state = g_mem_chunk_alloc(frame_state_chunk);
+	    packet_state = se_alloc(sizeof (frame_state_t));
 	    packet_state->info = leap_state;
 	    p_add_proto_data(pinfo->fd, proto_eap, packet_state);
 
@@ -1189,18 +1156,6 @@ dissect_eap_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   return tvb_length(tvb);
 }
 
-static int
-dissect_eap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
-{
-  return dissect_eap_data(tvb, pinfo, tree, FALSE);
-}
-
-static int
-dissect_eap_fragment(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
-{
-  return dissect_eap_data(tvb, pinfo, tree, TRUE);
-}
-
 void
 proto_register_eap(void)
 {
@@ -1261,10 +1216,8 @@ proto_register_eap(void)
 				      "EAP", "eap");
   proto_register_field_array(proto_eap, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
-  register_init_routine(&eap_init_protocol);
 
   new_register_dissector("eap", dissect_eap, proto_eap);
-  new_register_dissector("eap_fragment", dissect_eap_fragment, proto_eap);
   register_init_routine(eaptls_defragment_init);
 }
 
