@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c 12914 2005-01-01 23:11:46Z guy $
+ * $Id: packet-eth.c 17378 2006-02-23 09:54:08Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,7 +29,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/prefs.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 #include <epan/addr_resolv.h>
 #include "packet-eth.h"
 #include "packet-ieee8023.h"
@@ -49,10 +49,13 @@ static int hf_eth_src = -1;
 static int hf_eth_len = -1;
 static int hf_eth_type = -1;
 static int hf_eth_addr = -1;
+static int hf_eth_multicast = -1;
+static int hf_eth_local_admin = -1;
 static int hf_eth_trailer = -1;
 
 static gint ett_ieee8023 = -1;
 static gint ett_ether2 = -1;
+static gint ett_addr = -1;
 
 static dissector_handle_t fw1_handle;
 static heur_dissector_list_t heur_subdissector_list;
@@ -61,6 +64,15 @@ static int eth_tap = -1;
 
 #define ETH_HEADER_SIZE	14
 
+static const true_false_string multicast_tfs = {
+	"This is a MULTICAST frame",
+	"This is a UNICAST frame"
+};
+static const true_false_string local_admin_tfs = {
+	"This is NOT a factory default address",
+	"This is a FACTORY DEFAULT address"
+};
+
 /* These are the Netware-ish names for the different Ethernet frame types.
 	EthernetII: The ethernet with a Type field instead of a length field
 	Ethernet802.2: An 802.3 header followed by an 802.2 header
@@ -160,16 +172,19 @@ capture_eth(const guchar *pd, int offset, int len, packet_counts *ld)
 }
 
 static void
-dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	int fcs_len)
 {
   proto_item		*ti;
   eth_hdr 		*volatile ehdr;
   volatile gboolean	is_802_2;
   proto_tree		*volatile fh_tree = NULL;
-  const char		*src_addr, *dst_addr;
+  const guint8		*src_addr, *dst_addr;
   static eth_hdr 	ehdrs[4];
   static int		ehdr_num=0;
+  proto_tree		*volatile tree;
+  proto_item		*addr_item;
+  proto_tree		*addr_tree=NULL;
 
   ehdr_num++;
   if(ehdr_num>=4){
@@ -177,6 +192,7 @@ dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   }
   ehdr=&ehdrs[ehdr_num];
 
+  tree=parent_tree;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "Ethernet");
@@ -198,7 +214,7 @@ dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
    * a first look before we assume that it's actually an
    * Ethernet packet.
    */
-  if (dissector_try_heuristic(heur_subdissector_list, tvb, pinfo, tree))
+  if (dissector_try_heuristic(heur_subdissector_list, tvb, pinfo, parent_tree))
     goto end_of_eth;
 
   if (ehdr->type <= IEEE_802_3_MAX_LEN) {
@@ -212,7 +228,7 @@ dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		tvb_get_guint8(tvb, 2) == 0x0C &&
 		tvb_get_guint8(tvb, 3) == 0x00 &&
 		tvb_get_guint8(tvb, 4) == 0x00 ) {
-      dissect_isl(tvb, pinfo, tree, fcs_len);
+      dissect_isl(tvb, pinfo, parent_tree, fcs_len);
       goto end_of_eth;
     }
   }
@@ -263,41 +279,69 @@ dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
       fh_tree = proto_item_add_subtree(ti, ett_ieee8023);
     }
 
-    proto_tree_add_ether(fh_tree, hf_eth_dst, tvb, 0, 6, dst_addr);
-    proto_tree_add_ether(fh_tree, hf_eth_src, tvb, 6, 6, src_addr);
+    /* if IP is not referenced from any filters we dont need to worry about
+       generating any tree items.  We must do this after we created the actual
+       protocol above so that proto hier stat still works though.
+    */
+    if(!proto_field_is_referenced(parent_tree, proto_eth)){
+      tree=NULL;
+      fh_tree=NULL;
+    }
 
-/* add items for eth.addr filter */
-    proto_tree_add_ether_hidden(fh_tree, hf_eth_addr, tvb, 0, 6, dst_addr);
-    proto_tree_add_ether_hidden(fh_tree, hf_eth_addr, tvb, 6, 6, src_addr);
+    addr_item=proto_tree_add_ether(fh_tree, hf_eth_dst, tvb, 0, 6, dst_addr);
+    if(addr_item){
+        addr_tree = proto_item_add_subtree(addr_item, ett_addr);
+    }
+    proto_tree_add_ether(addr_tree, hf_eth_addr, tvb, 0, 6, dst_addr);
+    proto_tree_add_item(addr_tree, hf_eth_multicast, tvb, 0, 3, FALSE);
+    proto_tree_add_item(addr_tree, hf_eth_local_admin, tvb, 0, 3, FALSE);
 
-    dissect_802_3(ehdr->type, is_802_2, tvb, ETH_HEADER_SIZE, pinfo, tree, fh_tree,
+    addr_item=proto_tree_add_ether(fh_tree, hf_eth_src, tvb, 6, 6, src_addr);
+    if(addr_item){
+        addr_tree = proto_item_add_subtree(addr_item, ett_addr);
+    }
+    proto_tree_add_ether(addr_tree, hf_eth_addr, tvb, 6, 6, src_addr);
+    proto_tree_add_item(addr_tree, hf_eth_multicast, tvb, 6, 3, FALSE);
+    proto_tree_add_item(addr_tree, hf_eth_local_admin, tvb, 6, 3, FALSE);
+
+    dissect_802_3(ehdr->type, is_802_2, tvb, ETH_HEADER_SIZE, pinfo, parent_tree, fh_tree,
 		  hf_eth_len, hf_eth_trailer, fcs_len);
   } else {
     if (eth_interpret_as_fw1_monitor) {
 	if ((dst_addr[0] == 'i') || (dst_addr[0] == 'I') ||
 	    (dst_addr[0] == 'o') || (dst_addr[0] == 'O')) {
-	  call_dissector(fw1_handle, tvb, pinfo, tree);
+	  call_dissector(fw1_handle, tvb, pinfo, parent_tree);
 	  goto end_of_eth;
 	}
     }
 
     if (check_col(pinfo->cinfo, COL_INFO))
       col_set_str(pinfo->cinfo, COL_INFO, "Ethernet II");
-    if (tree) {
-      ti = proto_tree_add_protocol_format(tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
-		"Ethernet II, Src: %s, Dst: %s",
-		ether_to_str(src_addr), ether_to_str(dst_addr));
+    if (parent_tree) {
+      ti = proto_tree_add_protocol_format(parent_tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
+		"Ethernet II, Src: %s (%s), Dst: %s (%s)",
+		get_ether_name(src_addr), ether_to_str(src_addr), get_ether_name(dst_addr), ether_to_str(dst_addr));
 
       fh_tree = proto_item_add_subtree(ti, ett_ether2);
     }
 
-    proto_tree_add_ether(fh_tree, hf_eth_dst, tvb, 0, 6, dst_addr);
-    proto_tree_add_ether(fh_tree, hf_eth_src, tvb, 6, 6, src_addr);
-/* add items for eth.addr filter */
-    proto_tree_add_ether_hidden(fh_tree, hf_eth_addr, tvb, 0, 6, dst_addr);
-    proto_tree_add_ether_hidden(fh_tree, hf_eth_addr, tvb, 6, 6, src_addr);
+    addr_item=proto_tree_add_ether(fh_tree, hf_eth_dst, tvb, 0, 6, dst_addr);
+    if(addr_item){
+        addr_tree = proto_item_add_subtree(addr_item, ett_addr);
+    }
+    proto_tree_add_ether(addr_tree, hf_eth_addr, tvb, 0, 6, dst_addr);
+    proto_tree_add_item(addr_tree, hf_eth_multicast, tvb, 0, 3, FALSE);
+    proto_tree_add_item(addr_tree, hf_eth_local_admin, tvb, 0, 3, FALSE);
 
-    ethertype(ehdr->type, tvb, ETH_HEADER_SIZE, pinfo, tree, fh_tree, hf_eth_type,
+    addr_item=proto_tree_add_ether(fh_tree, hf_eth_src, tvb, 6, 6, src_addr);
+    if(addr_item){
+        addr_tree = proto_item_add_subtree(addr_item, ett_addr);
+    }
+    proto_tree_add_ether(addr_tree, hf_eth_addr, tvb, 6, 6, src_addr);
+    proto_tree_add_item(addr_tree, hf_eth_multicast, tvb, 6, 3, FALSE);
+    proto_tree_add_item(addr_tree, hf_eth_local_admin, tvb, 6, 3, FALSE);
+
+    ethertype(ehdr->type, tvb, ETH_HEADER_SIZE, pinfo, parent_tree, fh_tree, hf_eth_type,
           hf_eth_trailer, fcs_len);
   }
 
@@ -368,6 +412,7 @@ add_ethernet_trailer(proto_tree *fh_tree, int trailer_id, tvbuff_t *tvb,
       }
     }
     if (trailer_length != 0) {
+      tvb_ensure_bytes_exist(tvb, 0, trailer_length);
       proto_tree_add_item(fh_tree, trailer_id, trailer_tvb, 0,
 			  trailer_length, FALSE);
     }
@@ -376,11 +421,11 @@ add_ethernet_trailer(proto_tree *fh_tree, int trailer_id, tvbuff_t *tvb,
       guint32 fcs = crc32_802_tvb(tvb, tvb_length(tvb) - 4);
       if (fcs == sent_fcs) {
 	proto_tree_add_text(fh_tree, trailer_tvb, trailer_length, 4,
-			    "Frame check sequence: 0x%08x (correct)",
+			    "Frame check sequence: 0x%08x [correct]",
 			    sent_fcs);
       } else {
 	proto_tree_add_text(fh_tree, trailer_tvb, trailer_length, 4,
-			    "Frame check sequence: 0x%08x (incorrect, should be 0x%08x)",
+			    "Frame check sequence: 0x%08x [incorrect, should be 0x%08x]",
 			    sent_fcs, fcs);
       }
     }
@@ -432,17 +477,28 @@ proto_register_eth(void)
 		{ "Type",		"eth.type", FT_UINT16, BASE_HEX, VALS(etype_vals), 0x0,
 			"", HFILL }},
 		{ &hf_eth_addr,
-		{ "Source or Destination Address", "eth.addr", FT_ETHER, BASE_NONE, NULL, 0x0,
+		{ "Address", "eth.addr", FT_ETHER, BASE_NONE, NULL, 0x0,
 			"Source or Destination Hardware Address", HFILL }},
 
                 { &hf_eth_trailer,
 		{ "Trailer", "eth.trailer", FT_BYTES, BASE_NONE, NULL, 0x0,
 			"Ethernet Trailer or Checksum", HFILL }},
 
+                { &hf_eth_multicast,
+		{ "Multicast", "eth.multicast", FT_BOOLEAN, 24,
+			VALS(&multicast_tfs), 0x010000,
+			"Whether this is a multicast frame or not", HFILL }},
+
+                { &hf_eth_local_admin,
+		{ "Locally Administrated Address", "eth.local_admin", FT_BOOLEAN, 24,
+			VALS(&local_admin_tfs), 0x020000,
+			"Whether this is a \"factory default\" address or not", HFILL }},
+
 	};
 	static gint *ett[] = {
 		&ett_ieee8023,
 		&ett_ether2,
+		&ett_addr,
 	};
 	module_t *eth_module;
 
