@@ -15,7 +15,7 @@
  * 
  * Copyright 2004, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-etheric.c 13176 2005-01-26 22:24:42Z ulfl $
+ * $Id: packet-etheric.c 16625 2005-11-29 18:58:18Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,6 +47,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <prefs.h>
 #include "packet-e164.h"
 #include "packet-q931.h"
@@ -162,7 +163,7 @@ static const true_false_string isup_ISDN_originating_access_ind_value = {
   "originating access ISDN",
   "originating access non-ISDN"
 };
-static const value_string isup_calling_partys_category_value[] = {
+static const value_string etheric_calling_partys_category_value[] = {
   { 0,	"Reserved"},
   { 1,	"Reserved"},
   { 2,	"Reserved"},
@@ -289,7 +290,7 @@ static void dissect_etheric_message(tvbuff_t *message_tvb, packet_info *pinfo, p
 /* ------------------------------------------------------------------
   Mapping number to ASCII-character
  ------------------------------------------------------------------ */
-char number_to_char_2(int number)
+static char number_to_char_2(int number)
 {
   if (number < 10)
     return ((char) number + 0x30);
@@ -389,7 +390,7 @@ dissect_etheric_calling_partys_category_parameter(tvbuff_t *parameter_tvb,proto_
 	  0, 1, calling_partys_category);
 
   proto_item_set_text(parameter_item, "Calling Party's category: 0x%x (%s)", calling_partys_category,
-	  val_to_str(calling_partys_category, isup_calling_partys_category_value, "reserved/spare"));
+	  val_to_str(calling_partys_category, etheric_calling_partys_category_value, "reserved/spare"));
 }
 /* ------------------------------------------------------------------
   Dissector Parameter Transmission medium requirement
@@ -431,7 +432,7 @@ dissect_etheric_called_party_number_parameter(tvbuff_t *parameter_tvb, proto_tre
   address_digits_tree = proto_item_add_subtree(address_digits_item, ett_etheric_address_digits);
 
   length = tvb_reported_length_remaining(parameter_tvb, offset);
-  called_number = g_malloc((length+1) *2);
+  called_number = ep_alloc((length+1) *2);
   while((length = tvb_reported_length_remaining(parameter_tvb, offset)) > 0){
     address_digit_pair = tvb_get_guint8(parameter_tvb, offset);
     proto_tree_add_uint(address_digits_tree, hf_etheric_called_party_odd_address_signal_digit, parameter_tvb, offset, 1, address_digit_pair);
@@ -456,7 +457,6 @@ dissect_etheric_called_party_number_parameter(tvbuff_t *parameter_tvb, proto_tre
 								  (offset - 2), e164_info);
   proto_item_set_text(address_digits_item, "Called Party Number: %s", called_number);
   proto_item_set_text(parameter_item, "Called Party Number: %s", called_number);
-  g_free(called_number);
 }
 /* ------------------------------------------------------------------
   Dissector Parameter calling party number
@@ -489,7 +489,7 @@ dissect_etheric_calling_party_number_parameter(tvbuff_t *parameter_tvb, proto_tr
 
   length = tvb_length_remaining(parameter_tvb, offset);
   /* prevent running behind the end of calling_number array by throwing an exception */
-  calling_number = g_malloc((length+1) *2);
+  calling_number = ep_alloc((length+1) *2);
   while(length > 0){
     address_digit_pair = tvb_get_guint8(parameter_tvb, offset);
     proto_tree_add_uint(address_digits_tree, hf_etheric_calling_party_odd_address_signal_digit, parameter_tvb, offset, 1, address_digit_pair);
@@ -516,7 +516,6 @@ dissect_etheric_calling_party_number_parameter(tvbuff_t *parameter_tvb, proto_tr
     e164_info.E164_number_str = calling_number;
     e164_info.E164_number_length = i - 1;
     dissect_e164_number(parameter_tvb, address_digits_tree, 2, (offset - 2), e164_info);
-  g_free(calling_number);
 }
 /* ------------------------------------------------------------------
   Dissector Parameter location number
@@ -562,7 +561,7 @@ dissect_etheric_location_number_parameter(tvbuff_t *parameter_tvb, proto_tree *p
   address_digits_tree = proto_item_add_subtree(address_digits_item, ett_etheric_address_digits);
 
   length = tvb_length_remaining(parameter_tvb, offset);
-  calling_number = g_malloc((length+1) *2);
+  calling_number = ep_alloc((length+1) *2);
   while(length > 0){
     address_digit_pair = tvb_get_guint8(parameter_tvb, offset);
     proto_tree_add_uint(address_digits_tree, hf_etheric_calling_party_odd_address_signal_digit, parameter_tvb, offset, 1, address_digit_pair);
@@ -583,7 +582,6 @@ dissect_etheric_location_number_parameter(tvbuff_t *parameter_tvb, proto_tree *p
 
   proto_item_set_text(address_digits_item, "Location number: %s", calling_number);
   proto_item_set_text(parameter_item, "Location number: %s", calling_number);
-  g_free(calling_number);
 }
 
 /* ------------------------------------------------------------------
@@ -636,95 +634,12 @@ dissect_etheric_backward_call_indicators_parameter(tvbuff_t *parameter_tvb,proto
   Dissector Parameter Cause Indicators - no detailed dissection since defined in Rec. Q.850
  */
 
-/*
- * Cause codes for Cause.
- * The decoding of cause indicators parameter field are defined in ITU-T
- * Recommendation Q.850; those are different from the ones in the Q.931
- * dissector, as that has some values not specified by the standard but
- * that appear to be used for purposes other than the ones in Q.850.
- */
-static const value_string q850_cause_code_vals[] = {
-	{ 0x00,	"Valid cause code not yet received" },
-	{ 0x01,	"Unallocated (unassigned) number" },
-	{ 0x02,	"No route to specified transit network" },
-	{ 0x03,	"No route to destination" },
-	{ 0x04,	"Send special information tone" },
-	{ 0x05,	"Misdialled trunk prefix" },
-	{ 0x06,	"Channel unacceptable" },
-	{ 0x07,	"Call awarded and being delivered in an established channel" },
-	{ 0x08,	"Preemption" },
-	{ 0x09,	"Preemption - circuit reserved for reuse" },
-	{ 0x0E,	"QoR: ported number" },
-	{ 0x10,	"Normal call clearing" },
-	{ 0x11,	"User busy" },
-	{ 0x12,	"No user responding" },
-	{ 0x13,	"No answer from user (user alerted)" },
-	{ 0x14,	"Subscriber absent" },
-	{ 0x15,	"Call rejected" },
-	{ 0x16,	"Number changed" },
-	{ 0x17,	"Redirection to new destination" },
-	{ 0x18,	"Call rejected due to feature at the destination" },
-	{ 0x19,	"Exchange routing error" },
-	{ 0x1A,	"Non-selected user clearing" },
-	{ 0x1B,	"Destination out of order" },
-	{ 0x1C,	"Invalid number format (address incomplete)" },
-	{ 0x1D,	"Facility rejected" },
-	{ 0x1E,	"Response to STATUS ENQUIRY" },
-	{ 0x1F,	"Normal unspecified" },
-	{ 0x21,	"Circuit out of order" },
-	{ 0x22,	"No circuit/channel available" },
-	{ 0x26,	"Network out of order" },
-	{ 0x27,	"Permanent frame mode connection out of service" },
-	{ 0x28,	"Permanent frame mode connection operational" },
-	{ 0x29,	"Temporary failure" },
-	{ 0x2A,	"Switching equipment congestion" },
-	{ 0x2B,	"Access information discarded" },
-	{ 0x2C,	"Requested circuit/channel not available" },
-	{ 0x2E,	"Precedence call blocked" },
-	{ 0x2F,	"Resources unavailable, unspecified" },
-	{ 0x31,	"Quality of service unavailable" },
-	{ 0x32,	"Requested facility not subscribed" },
-	{ 0x35,	"Outgoing calls barred within CUG" },
-	{ 0x37,	"Incoming calls barred within CUG" },
-	{ 0x38,	"Call waiting not subscribed" },
-	{ 0x39,	"Bearer capability not authorized" },
-	{ 0x3A,	"Bearer capability not presently available" },
-	{ 0x3E,	"Inconsistency in designated outgoing access information and subscriber class" },
-	{ 0x3F,	"Service or option not available, unspecified" },
-	{ 0x41,	"Bearer capability not implemented" },
-	{ 0x42,	"Channel type not implemented" },
-	{ 0x45,	"Requested facility not implemented" },
-	{ 0x46,	"Only restricted digital information bearer capability is available" },
-	{ 0x4F,	"Service or option not implemented, unspecified" },
-	{ 0x51,	"Invalid call reference value" },
-	{ 0x52,	"Identified channel does not exist" },
-	{ 0x53,	"Call identity does not exist for suspended call" },
-	{ 0x54,	"Call identity in use" },
-	{ 0x55,	"No call suspended" },
-	{ 0x56,	"Call having the requested call identity has been cleared" },
-	{ 0x57,	"Called user not member of CUG" },
-	{ 0x58,	"Incompatible destination" },
-	{ 0x5A,	"Non-existing CUG" },
-	{ 0x5B,	"Invalid transit network selection (national use)" },
-	{ 0x5F,	"Invalid message, unspecified" },
-	{ 0x60,	"Mandatory information element is missing" },
-	{ 0x61,	"Message type non-existent or not implemented" },
-	{ 0x62,	"Message not compatible with call state or message type non-existent or not implemented" },
-	{ 0x63,	"Information element nonexistant or not implemented" },
-	{ 0x64,	"Invalid information element contents" },
-	{ 0x65,	"Message not compatible with call state" },
-	{ 0x66,	"Recovery on timer expiry" },
-	{ 0x67,	"Parameter non-existent or not implemented - passed on" },
-	{ 0x6E,	"Message with unrecognized parameter discarded" },
-	{ 0x6F,	"Protocol error, unspecified" },
-	{ 0x7F,	"Internetworking, unspecified" },
-	{ 0,	NULL }
-};
+
 
 /* ------------------------------------------------------------------
   Dissector Message Type release message
  */
-gint
+static gint
 dissect_etheric_release_message(tvbuff_t *message_tvb, proto_tree *etheric_tree)
 { proto_item* parameter_item;
   proto_tree* parameter_tree;
@@ -777,7 +692,7 @@ dissect_etheric_unknown_parameter(tvbuff_t *parameter_tvb, proto_item *parameter
 /* ------------------------------------------------------------------
   Dissector Message Type Initial address message
  */
-gint
+static gint
 dissect_etheric_initial_address_message(tvbuff_t *message_tvb, proto_tree *etheric_tree)
 { proto_item* parameter_item;
   proto_tree* parameter_tree;
@@ -865,7 +780,7 @@ dissect_etheric_initial_address_message(tvbuff_t *message_tvb, proto_tree *ether
 /* ------------------------------------------------------------------
   Dissector Message Type Address complete
  */
-gint
+static gint
 dissect_etheric_address_complete_message(tvbuff_t *message_tvb, proto_tree *etheric_tree)
 { proto_item* parameter_item;
   proto_tree* parameter_tree;
@@ -892,7 +807,7 @@ dissect_etheric_address_complete_message(tvbuff_t *message_tvb, proto_tree *ethe
 /* ------------------------------------------------------------------
   Dissector Message Type Call Progress
 */
-gint
+static gint
 dissect_etheric_call_progress_message(tvbuff_t *message_tvb, proto_tree *isup_tree)
 { proto_item* parameter_item;
   proto_tree* parameter_tree;
@@ -1152,7 +1067,7 @@ proto_register_etheric(void)
 
 		{ &hf_etheric_calling_partys_category,
 			{ "Calling Party's category",  "etheric.calling_partys_category",
-			FT_UINT8, BASE_HEX, VALS(isup_calling_partys_category_value), 0x0,
+			FT_UINT8, BASE_HEX, VALS(etheric_calling_partys_category_value), 0x0,
 			"", HFILL }},
 
 		{ &hf_etheric_mandatory_variable_parameter_pointer,
