@@ -1,7 +1,7 @@
 /* ethertype.c
  * Routines for calling the right protocol for the ethertype.
  *
- * $Id: packet-ethertype.c 13577 2005-03-03 08:34:48Z guy $
+ * $Id: packet-ethertype.c 17717 2006-03-24 07:09:35Z etxrab $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -37,64 +37,72 @@
 #include "packet-ipx.h"
 #include "packet-vlan.h"
 #include "packet-vines.h"
-#include "etypes.h"
-#include "ppptypes.h"
+#include <epan/etypes.h>
+#include <epan/ppptypes.h>
 
 static dissector_table_t ethertype_dissector_table;
 
 static dissector_handle_t data_handle;
 
 const value_string etype_vals[] = {
-    {ETHERTYPE_IP,		"IP"				},
-    {ETHERTYPE_IPv6,		"IPv6"				},
-    {ETHERTYPE_CENTRINO_PROMISC, "IEEE 802.11 (Centrino promiscuous)" },
-    {ETHERTYPE_XNS_IDP,		"XNS Internet Datagram Protocol" },
-    {ETHERTYPE_X25L3,		"X.25 Layer 3"			},
-    {ETHERTYPE_ARP,		"ARP"				},
-    {ETHERTYPE_REVARP,		"RARP"				},
-    {ETHERTYPE_DEC_LB,		"DEC LanBridge"			},
-    {ETHERTYPE_ATALK,		"Appletalk"			},
-    {ETHERTYPE_SNA,		"SNA-over-Ethernet"		},
-    {ETHERTYPE_AARP,		"AARP"				},
-    {ETHERTYPE_IPX,		"Netware IPX/SPX"		},
-    {ETHERTYPE_VINES_IP,	"Vines IP"			},
-    {ETHERTYPE_VINES_ECHO,	"Vines Echo"			},
-    {ETHERTYPE_TRAIN,		"Netmon Train"			},
-    {ETHERTYPE_LOOP,		"Loopback"			}, /* Ethernet Loopback */
-    {ETHERTYPE_WCP,		"Wellfleet Compression Protocol" },
-    {ETHERTYPE_ISMP,		"Cabletron Interswitch Message Protocol" },
-    {ETHERTYPE_ISMP_TBFLOOD,	"Cabletron SFVLAN 1.8 Tag-Based Flood" },
+    {ETHERTYPE_IP,					"IP"				},
+    {ETHERTYPE_IPv6,				"IPv6"				},
+    {ETHERTYPE_CISCOWL,				"Cisco Wireless"		},
+    {ETHERTYPE_CENTRINO_PROMISC,	"IEEE 802.11 (Centrino promiscuous)" },
+    {ETHERTYPE_XNS_IDP,				"XNS Internet Datagram Protocol" },
+    {ETHERTYPE_X25L3,				"X.25 Layer 3"			},
+    {ETHERTYPE_ARP,					"ARP"				},
+    {ETHERTYPE_REVARP,				"RARP"				},
+    {ETHERTYPE_DEC_LB,				"DEC LanBridge"			},
+    {ETHERTYPE_ATALK,				"Appletalk"			},
+    {ETHERTYPE_SNA,					"SNA-over-Ethernet"		},
+    {ETHERTYPE_AARP,				"AARP"				},
+    {ETHERTYPE_IPX,					"Netware IPX/SPX"		},
+    {ETHERTYPE_VINES_IP,			"Vines IP"			},
+    {ETHERTYPE_VINES_ECHO,			"Vines Echo"			},
+    {ETHERTYPE_TRAIN,				"Netmon Train"			},
+    {ETHERTYPE_LOOP,				"Loopback"			}, /* Ethernet Loopback */
+    {ETHERTYPE_FOUNDRY,				"Foundry proprietary"		},
+    {ETHERTYPE_WCP,					"Wellfleet Compression Protocol" },
+    {ETHERTYPE_ISMP,				"Cabletron Interswitch Message Protocol" },
+    {ETHERTYPE_ISMP_TBFLOOD,		"Cabletron SFVLAN 1.8 Tag-Based Flood" },
     				/* for ISMP, see RFC 2641, RFC 2642, RFC 2643 */
-    {ETHERTYPE_PPPOED,		"PPPoE Discovery"		},
-    {ETHERTYPE_PPPOES,		"PPPoE Session"			},
-    {ETHERTYPE_INTEL_ANS,	"Intel ANS probe"		},
+    {ETHERTYPE_PPPOED,				"PPPoE Discovery"		},
+    {ETHERTYPE_PPPOES,				"PPPoE Session"			},
+    {ETHERTYPE_INTEL_ANS,			"Intel ANS probe"		},
     {ETHERTYPE_MS_NLB_HEARTBEAT,	"MS NLB heartbeat"	},
-    {ETHERTYPE_VLAN,		"802.1Q Virtual LAN"		},
-    {ETHERTYPE_EAPOL,		"802.1X Authentication"         },
-    {ETHERTYPE_RSN_PREAUTH,	"802.11i Pre-Authentication"    },
-    {ETHERTYPE_MPLS,		"MPLS label switched packet"	},
-    {ETHERTYPE_MPLS_MULTI,	"MPLS multicast label switched packet" },
-    {ETHERTYPE_3C_NBP_DGRAM,	"3Com NBP Datagram"		},
-    {ETHERTYPE_DEC,		"DEC proto"			},
-    {ETHERTYPE_DNA_DL,		"DEC DNA Dump/Load"		},
-    {ETHERTYPE_DNA_RC,		"DEC DNA Remote Console"	},
-    {ETHERTYPE_DNA_RT,		"DEC DNA Routing"		},
-    {ETHERTYPE_LAT,		"DEC LAT"			},
-    {ETHERTYPE_DEC_DIAG,	"DEC Diagnostics"		},
-    {ETHERTYPE_DEC_CUST,	"DEC Customer use"		},
-    {ETHERTYPE_DEC_SCA,		"DEC LAVC/SCA"			},
-    {ETHERTYPE_DEC_LAST,	"DEC LAST"			},
-    {ETHERTYPE_ETHBRIDGE,	"Transparent Ethernet bridging" },
-    {ETHERTYPE_CGMP,		"Cisco Group Management Protocol" },
-    {ETHERTYPE_MAC_CONTROL,	"MAC Control"			},
-    {ETHERTYPE_SLOW_PROTOCOLS,	"Slow Protocols"		},
-    {ETHERTYPE_RTMAC,		"Real-Time Media Access Control" },
-    {ETHERTYPE_RTCFG,		"Real-Time Configuration Protocol" },
-    {ETHERTYPE_CDMA2000_A10_UBS,"CDMA2000 A10 Unstructured byte stream" },
-    {ETHERTYPE_PROFINET,	"PROFInet"			},
-    {ETHERTYPE_AOE,		"ATA over Ethernet"             },
+    {ETHERTYPE_VLAN,				"802.1Q Virtual LAN"		},
+    {ETHERTYPE_EAPOL,				"802.1X Authentication"         },
+    {ETHERTYPE_RSN_PREAUTH,			"802.11i Pre-Authentication"    },
+    {ETHERTYPE_MPLS,				"MPLS label switched packet"	},
+    {ETHERTYPE_MPLS_MULTI,			"MPLS multicast label switched packet" },
+    {ETHERTYPE_3C_NBP_DGRAM,		"3Com NBP Datagram"		},
+    {ETHERTYPE_DEC,					"DEC proto"			},
+    {ETHERTYPE_DNA_DL,				"DEC DNA Dump/Load"		},
+    {ETHERTYPE_DNA_RC,				"DEC DNA Remote Console"	},
+    {ETHERTYPE_DNA_RT,				"DEC DNA Routing"		},
+    {ETHERTYPE_LAT,					"DEC LAT"			},
+    {ETHERTYPE_DEC_DIAG,			"DEC Diagnostics"		},
+    {ETHERTYPE_DEC_CUST,			"DEC Customer use"		},
+    {ETHERTYPE_DEC_SCA,				"DEC LAVC/SCA"			},
+    {ETHERTYPE_DEC_LAST,			"DEC LAST"			},
+    {ETHERTYPE_ETHBRIDGE,			"Transparent Ethernet bridging" },
+    {ETHERTYPE_CGMP,				"Cisco Group Management Protocol" },
+    {ETHERTYPE_MAC_CONTROL,			"MAC Control"			},
+    {ETHERTYPE_SLOW_PROTOCOLS,		"Slow Protocols"		},
+    {ETHERTYPE_RTMAC,				"Real-Time Media Access Control" },
+    {ETHERTYPE_RTCFG,				"Real-Time Configuration Protocol" },
+    {ETHERTYPE_CDMA2000_A10_UBS,	"CDMA2000 A10 Unstructured byte stream" },
+    {ETHERTYPE_PROFINET,			"PROFInet"			},
+    {ETHERTYPE_AOE,					"ATA over Ethernet"             },
+    {ETHERTYPE_CSM_ENCAPS,			"CSM_ENCAPS Protocol"             },
     {ETHERTYPE_IEEE802_OUI_EXTENDED, "IEEE 802a OUI Extended Ethertype" },
-    {ETHERTYPE_LLDP,	"802.1 Link Layer Discovery Protocol (LLDP)" },
+    {ETHERTYPE_IEC61850_GOOSE,		"IEC 61850/GOOSE" },
+    {ETHERTYPE_IEC61850_GSE,		"IEC 61850/GSE management services" },
+    {ETHERTYPE_IEC61850_SV,			"IEC 61850/SV (Sampled Value Transmission" },
+    {ETHERTYPE_TIPC,				"Transparent Inter Process Communication" },
+    {ETHERTYPE_LLDP,				"802.1 Link Layer Discovery Protocol (LLDP)" },
+    {ETHERTYPE_3GPP2,                           "CDMA2000 A10 3GPP2 Packet" },
     /*
      * NDISWAN on Windows translates Ethernet frames from higher-level
      * protocols into PPP frames to hand to the PPP driver, and translates
@@ -155,10 +163,10 @@ ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_etype,
 		packet_info *pinfo, proto_tree *tree, proto_tree *fh_tree,
 		int etype_id, int trailer_id, int fcs_len)
 {
-	char			*description;
+	const char		*description;
 	tvbuff_t		*next_tvb;
 	guint			length_before;
-	volatile gboolean	dissector_found;
+	volatile gboolean	dissector_found = FALSE;
 	const char		*saved_proto;
 
 	/* Add the Ethernet type to the protocol tree */
