@@ -4,7 +4,7 @@
  *   Copyright 2003  Ronnie Sahlberg, exchange first/last matching and 
  *                                    tap listener and misc updates
  *
- * $Id: packet-fc.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-fc.c 17829 2006-04-06 08:33:08Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -44,19 +44,16 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include <epan/prefs.h>
 #include <epan/reassemble.h>
 #include <epan/conversation.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 #include "packet-fc.h"
 #include "packet-fclctl.h"
 #include "packet-fcbls.h"
 #include <epan/tap.h>
+#include <epan/emem.h>
 
 #define FC_HEADER_SIZE         24
 #define FC_RCTL_EISL           0x50
@@ -137,6 +134,11 @@ static dissector_handle_t data_handle;
 
 static int fc_tap = -1;
 
+typedef struct _fc_conv_data_t {
+    se_tree_t *exchanges;
+} fc_conv_data_t;
+
+
 /* Reassembly stuff */
 static gboolean fc_reassemble = TRUE;
 static guint32  fc_max_frame_size = 1024;
@@ -151,14 +153,6 @@ typedef struct _fcseq_conv_data {
 } fcseq_conv_data_t;
 
 GHashTable *fcseq_req_hash = NULL;
-GMemChunk *fcseq_req_keys = NULL;
-GMemChunk *fcseq_req_vals = NULL;
-guint32 fcseq_init_count = 25;
-
-static GHashTable *fc_exchange_unmatched = NULL;
-static GHashTable *fc_exchange_matched = NULL;
-static GMemChunk *fc_exchange_vals = NULL;
-static guint32 fc_exchange_init_count = 200;
 
 /*
  * Hash Functions
@@ -183,112 +177,15 @@ fcseq_hash (gconstpointer v)
     return val;
 }
 
-static guint
-fc_exchange_hash_unmatched(gconstpointer v)
-{
-    const fc_exchange_data *fced=(const fc_exchange_data *)v;
-
-    return fced->oxid;
-}
-static gint
-fc_exchange_equal_unmatched(gconstpointer v1, gconstpointer v2)
-{
-    const fc_exchange_data *fced1=(const fc_exchange_data *)v1;
-    const fc_exchange_data *fced2=(const fc_exchange_data *)v2;
-
-    /* oxid must match */
-    if(fced1->oxid!=fced2->oxid){
-        return 0;
-    }
-    /* compare s_id, d_id and treat the fc address
-       s_id==00.00.00 as a wildcard matching anything */
-    if( ((fced1->s_id.data[0]!=0)||(fced1->s_id.data[1]!=0)||(fced1->s_id.data[2]!=0)) && CMP_ADDRESS(&fced1->s_id, &fced2->s_id) ){
-        return 0;
-    }
-    if(CMP_ADDRESS(&fced1->d_id, &fced2->d_id)){
-        return 0;
-    }
-
-    return 1;
-}
-
-static guint
-fc_exchange_hash_matched(gconstpointer v)
-{
-    const fc_exchange_data *fced=(const fc_exchange_data *)v;
-
-    return fced->oxid;
-}
-static gint
-fc_exchange_equal_matched(gconstpointer v1, gconstpointer v2)
-{
-    const fc_exchange_data *fced1=(const fc_exchange_data *)v1;
-    const fc_exchange_data *fced2=(const fc_exchange_data *)v2;
-    guint32 fef1, fef2, lef1, lef2;
-
-    /* oxid must match */
-    if(fced1->oxid!=fced2->oxid){
-        return 0;
-    }
-    fef1=fced1->first_exchange_frame;
-    fef2=fced2->first_exchange_frame;
-    lef1=fced1->last_exchange_frame;
-    lef2=fced2->last_exchange_frame;
-    if(!fef1)fef1=fef2;
-    if(!fef2)fef2=fef1;
-    if(!lef1)lef1=lef2;
-    if(!lef2)lef2=lef1;
-
-    if(fef1!=fef2){
-        return 0;
-    }
-    if(lef1!=lef2){
-        return 0;
-    }
-
-    return 1;
-}
-
 static void
 fc_exchange_init_protocol(void)
 {
-    if(fc_exchange_vals){
-        g_mem_chunk_destroy(fc_exchange_vals);
-        fc_exchange_vals=NULL;
-    }
-    if(fc_exchange_unmatched){
-        g_hash_table_destroy(fc_exchange_unmatched);
-        fc_exchange_unmatched=NULL;
-    }
-    if(fc_exchange_matched){
-        g_hash_table_destroy(fc_exchange_matched);
-        fc_exchange_matched=NULL;
-    }
-
-    fc_exchange_unmatched=g_hash_table_new(fc_exchange_hash_unmatched, fc_exchange_equal_unmatched);
-    fc_exchange_matched=g_hash_table_new(fc_exchange_hash_matched, fc_exchange_equal_matched);
-    fc_exchange_vals=g_mem_chunk_new("fc_exchange_vals", sizeof(fc_exchange_data), fc_exchange_init_count*sizeof(fc_exchange_data), G_ALLOC_AND_FREE);
-
     fragment_table_init(&fc_fragment_table);
 
-    if (fcseq_req_keys)
-        g_mem_chunk_destroy (fcseq_req_keys);
-    if (fcseq_req_vals)
-        g_mem_chunk_destroy (fcseq_req_vals);
     if (fcseq_req_hash)
         g_hash_table_destroy(fcseq_req_hash);
     
     fcseq_req_hash = g_hash_table_new(fcseq_hash, fcseq_equal);
-    fcseq_req_keys = g_mem_chunk_new ("fcseq_req_keys",
-                                      sizeof(fcseq_conv_key_t),
-                                      fcseq_init_count *
-                                      sizeof(fcseq_conv_key_t),
-                                      G_ALLOC_AND_FREE);
-    fcseq_req_vals = g_mem_chunk_new ("fcseq_req_vals",
-                                      sizeof(fcseq_conv_data_t),
-                                      fcseq_init_count *
-                                      sizeof(fcseq_conv_data_t),
-                                      G_ALLOC_AND_FREE);
 }
 
 
@@ -318,6 +215,7 @@ static const value_string fc_ftype_vals [] = {
     {FC_FTYPE_VDO,       "Video Data"},
     {FC_FTYPE_LINKCTL,   "Link Ctl"},
     {FC_FTYPE_SBCCS,     "SBCCS"},
+    {FC_FTYPE_OHMS,      "OHMS(Cisco MDS)"},
     {0, NULL},
 };
 
@@ -361,150 +259,7 @@ static const value_string fc_iu_val[] = {
 
 static void fc_defragment_init(void)
 {
-  fragment_table_init(&fc_fragment_table);
-}
-
-
-static gchar *
-fctl_to_str (const guint8 *fctl, gchar *str, gboolean is_ack)
-{
-    int stroff = 0;
-    guint8 tmp = 0;
-
-    if (str == NULL)
-        return (str);
-    
-    if (fctl[2] & 0x80) {
-        strcpy (str, "Exchange Responder, ");
-        stroff += 20;
-    }
-    else {
-        strcpy (str, "Exchange Originator, ");
-        stroff += 21;
-    }
-
-    if (fctl[2] & 0x40) {
-        strcpy (&str[stroff], "Seq Recipient, ");
-        stroff += 15;
-    }
-    else {
-        strcpy (&str[stroff], "Seq Initiator, ");
-        stroff += 15;
-    }
-
-    if (fctl[2] & 0x20) {
-        strcpy (&str[stroff], "Exchg First, ");
-        stroff += 13;
-    }
-
-    if (fctl[2] & 0x10) {
-        strcpy (&str[stroff], "Exchg Last, ");
-        stroff += 12;
-    }
-
-    if (fctl[2] & 0x8) {
-        strcpy (&str[stroff], "Seq Last, ");
-        stroff += 10;
-    }
-
-    if (fctl[2] & 0x2) {
-        strcpy (&str[stroff], "Priority, ");
-        stroff += 10;
-    }
-    else {
-        strcpy (&str[stroff], "CS_CTL, ");
-        stroff += 8;
-    }
-
-    if (fctl[2] & 0x1) {
-        strcpy (&str[stroff], "Transfer Seq Initiative, ");
-        stroff += 25;
-    }
-
-    if (fctl[1] & 0x30) {
-        strcpy (&str[stroff], "ACK_0 Reqd, ");
-        stroff += 12;
-    }
-    else if (fctl[1] & 0x10) {
-        strcpy (&str[stroff], "ACK_1 Reqd, ");
-        stroff += 12;
-    }
-
-    if (fctl[1] & 0x2) {
-        strcpy (&str[stroff], "Rexmitted Seq, ");
-        stroff += 15;
-    }
-
-    tmp = fctl[0] & 0xC0;
-    switch (tmp) {
-    case 0:
-        strcpy (&str[stroff], "Last Data Frame - No Info, ");
-        stroff += 27;
-        break;
-    case 1:
-        strcpy (&str[stroff], "Last Data Frame - Seq Imm, ");
-        stroff += 27;
-        break;
-    case 2:
-        strcpy (&str[stroff], "Last Data Frame - Seq Soon, ");
-        stroff += 28;
-        break;
-    case 3:
-        strcpy (&str[stroff], "Last Data Frame - Seq Delyd, ");
-        stroff += 29;
-        break;
-    }
-
-    tmp = fctl[0] & 0x30;
-    switch (tmp) {
-    case 0:
-        if (is_ack) {
-            strcpy (&str[stroff], "ABTS - Cont, ");
-            stroff += 13;
-        }
-        else {
-            strcpy (&str[stroff], "ABTS - Abort/MS, ");
-            stroff += 17;
-        }
-        break;
-    case 0x10:
-        if (is_ack) {
-            strcpy (&str[stroff], "ABTS - AbortABTS - Abort, ");
-            stroff += 14;
-        }
-        else {
-            strcpy (&str[stroff], "ABTS - Abort/SS, ");
-            stroff += 17;
-        }
-        break;
-    case 0x20:
-        if (is_ack) {
-            strcpy (&str[stroff], "ABTS - Stop, ");
-            stroff += 13;
-        }
-        else {
-            strcpy (&str[stroff], "ABTS - Process/IB, ");
-            stroff += 19;
-        }
-        break;
-    case 0x30:
-        if (is_ack) {
-            strcpy (&str[stroff], "ABTS - Imm Seq Retx, ");
-            stroff += 21;
-        }
-        else {
-            strcpy (&str[stroff], "ABTS - Discard/MS/Imm Retx, ");
-            stroff += 28;
-        }
-        break;
-    }
-
-    if (fctl[0] & 0x8) {
-        strcpy (&str[stroff], "Rel Offset = 1");
-        stroff += 14;
-    }
-
-    return (str);
+  fragment_table_init (&fc_fragment_table);
 }
 
 /* BA_ACC & BA_RJT are decoded in this file itself instead of a traditional
@@ -593,34 +348,31 @@ fc_get_ftype (guint8 r_ctl, guint8 type)
         case FC_TYPE_SB_FROM_CU:
         case FC_TYPE_SB_TO_CU:
             return FC_FTYPE_SBCCS;
+        case FC_TYPE_VENDOR:
+             return FC_FTYPE_OHMS;
         default:
             return FC_FTYPE_UNDEF;
         }
-        break;
     case FC_RCTL_ELS:
         if (((r_ctl & 0x0F) == 0x2) || ((r_ctl & 0x0F) == 0x3))
             return FC_FTYPE_ELS;
+        else if (type == FC_TYPE_ELS) 
+            return FC_FTYPE_OHMS;
         else
-            return FC_FTYPE_UNDEF;
-        break;
+             return FC_FTYPE_UNDEF;
     case FC_RCTL_LINK_DATA:
         return FC_FTYPE_LINKDATA;
-        break;
     case FC_RCTL_VIDEO:
         return FC_FTYPE_VDO;
-        break;
     case FC_RCTL_BLS:
         if (type == 0)
             return FC_FTYPE_BLS;
         else
             return FC_FTYPE_UNDEF;
-        break;
     case FC_RCTL_LINK_CTL:
         return FC_FTYPE_LINKCTL;
-        break;
     default:
         return FC_FTYPE_UNDEF;
-        break;
     }
 }
 
@@ -694,47 +446,111 @@ static const true_false_string tfs_fc_fctl_rel_offset = {
 
 /* code to dissect the  F_CTL bitmask */
 static void
-dissect_fc_fctl(packet_info *pinfo _U_, proto_tree *parent_tree, tvbuff_t *tvb, int offset, gboolean is_ack, guint32 fctl)
+dissect_fc_fctl(packet_info *pinfo _U_, proto_tree *parent_tree, tvbuff_t *tvb, int offset)
 {
-	proto_item *item;
-	proto_tree *tree;
-	gchar str[256];
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+	guint32 flags;
 
-        item=proto_tree_add_uint(parent_tree, hf_fc_fctl, tvb, offset, 3, fctl);
-	tree=proto_item_add_subtree(item, ett_fctl);
+	flags = tvb_get_guint8 (tvb, offset);
+	flags = (flags<<8) | tvb_get_guint8 (tvb, offset+1);
+	flags = (flags<<8) | tvb_get_guint8 (tvb, offset+2);
 
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_responder, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_seq_recipient, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_first, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_last, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_seq_last, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_priority, tvb, offset, 3, fctl);
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_transfer_seq_initiative, tvb, offset, 3, fctl);
-
-	proto_tree_add_uint(tree, hf_fc_fctl_last_data_frame, tvb, offset, 3, fctl);
-
-	proto_tree_add_uint(tree, hf_fc_fctl_ack_0_1, tvb, offset, 3, fctl);
-
-
-	proto_tree_add_boolean(tree, hf_fc_fctl_rexmitted_seq, tvb, offset, 3, fctl);
-
-	if(is_ack){
-		proto_tree_add_uint(tree, hf_fc_fctl_abts_ack, tvb, offset, 3, fctl);
-	} else {
-		proto_tree_add_uint(tree, hf_fc_fctl_abts_ack, tvb, offset, 3, fctl);
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_fc_fctl, tvb, offset, 3, flags);
+		tree=proto_item_add_subtree(item, ett_fctl);
 	}
 
-	proto_tree_add_boolean(tree, hf_fc_fctl_rel_offset, tvb, offset, 3, fctl);
+	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_responder, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_EXCHANGE_RESPONDER){
+		proto_item_append_text(item, " Exchange Responder");
+		if (flags & (~( FC_FCTL_EXCHANGE_RESPONDER )))
+			proto_item_append_text(item, ",");
+	} else {
+		proto_item_append_text(item, " Exchange Originator");
+		if (flags & (~( FC_FCTL_EXCHANGE_RESPONDER )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_EXCHANGE_RESPONDER ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_seq_recipient, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_SEQ_RECIPIENT){
+		proto_item_append_text(item, " Seq Recipient");
+		if (flags & (~( FC_FCTL_SEQ_RECIPIENT )))
+			proto_item_append_text(item, ",");
+	} else {
+		proto_item_append_text(item, " Seq Initiator");
+		if (flags & (~( FC_FCTL_SEQ_RECIPIENT )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_SEQ_RECIPIENT ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_first, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_EXCHANGE_FIRST){
+		proto_item_append_text(item, " Exchg First");
+		if (flags & (~( FC_FCTL_EXCHANGE_FIRST )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_EXCHANGE_FIRST ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_exchange_last, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_EXCHANGE_LAST){
+		proto_item_append_text(item, " Exchg Last");
+		if (flags & (~( FC_FCTL_EXCHANGE_LAST )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_EXCHANGE_LAST ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_seq_last, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_SEQ_LAST){
+		proto_item_append_text(item, " Seq Last");
+		if (flags & (~( FC_FCTL_SEQ_LAST )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_SEQ_LAST ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_priority, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_PRIORITY){
+		proto_item_append_text(item, " Priority");
+		if (flags & (~( FC_FCTL_PRIORITY )))
+			proto_item_append_text(item, ",");
+	} else {
+		proto_item_append_text(item, " CS_CTL");
+		if (flags & (~( FC_FCTL_PRIORITY )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_PRIORITY ));
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_transfer_seq_initiative, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_TRANSFER_SEQ_INITIATIVE){
+		proto_item_append_text(item, " Transfer Seq Initiative");
+		if (flags & (~( FC_FCTL_TRANSFER_SEQ_INITIATIVE )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_TRANSFER_SEQ_INITIATIVE ));
+
+	proto_tree_add_uint(tree, hf_fc_fctl_last_data_frame, tvb, offset, 3, flags);
+
+	proto_tree_add_uint(tree, hf_fc_fctl_ack_0_1, tvb, offset, 3, flags);
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_rexmitted_seq, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_REXMITTED_SEQ){
+		proto_item_append_text(item, " Rexmitted Seq");
+		if (flags & (~( FC_FCTL_REXMITTED_SEQ )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_REXMITTED_SEQ ));
+
+	proto_tree_add_uint(tree, hf_fc_fctl_abts_ack, tvb, offset, 3, flags);
+
+	proto_tree_add_boolean(tree, hf_fc_fctl_rel_offset, tvb, offset, 3, flags);
+	if (flags&FC_FCTL_REL_OFFSET){
+		proto_item_append_text(item, " Rel Offset");
+		if (flags & (~( FC_FCTL_REL_OFFSET )))
+			proto_item_append_text(item, ",");
+	}
+	flags&=(~( FC_FCTL_REL_OFFSET ));
 
-	fctl_to_str( ((guint8 *)&fctl), str, is_ack);
-	proto_item_append_text(item, "  %s", str);
 }
 
 static const value_string fc_bls_proto_val[] = {
@@ -756,7 +572,7 @@ static const value_string fc_els_proto_val[] = {
 
 /* Code to actually dissect the packets */
 static void
-dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean is_ifcp)
 {
    /* Set up structures needed to add the protocol subtree and manage it */
     proto_item *ti=NULL;
@@ -777,6 +593,7 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
     static fc_hdr fchdr;
     fc_exchange_data *fc_ex=NULL;
+    fc_conv_data_t *fc_conv_data=NULL;
 
     conversation_t *conversation;
     fcseq_conv_data_t *cdata;
@@ -798,11 +615,24 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         offset += 8;
         fchdr.r_ctl = tvb_get_guint8 (tvb, offset);
     }
-    
-    SET_ADDRESS (&pinfo->dst, AT_FC, 3, tvb_get_ptr(tvb,offset+1,3));
-    SET_ADDRESS (&fchdr.d_id, AT_FC, 3, tvb_get_ptr(tvb,offset+1,3));
-    SET_ADDRESS (&pinfo->src, AT_FC, 3, tvb_get_ptr(tvb,offset+5,3));
-    SET_ADDRESS (&fchdr.s_id, AT_FC, 3, tvb_get_ptr(tvb,offset+5,3));
+
+    /* Each fc endpoint pair gets its own TCP session in iFCP but
+     * the src/dst ids are undefined(==semi-random) in the FC header.
+     * This means we can no track conversations for FC over iFCP by using
+     * the FC src/dst addresses.
+     * For iFCP: Do not update the pinfo src/dst struct and let it remain 
+     * being tcpip src/dst so that request/response matching in the FCP layer
+     * will use ip addresses instead and still work.
+     */
+    if(!is_ifcp){    
+	SET_ADDRESS (&pinfo->dst, AT_FC, 3, tvb_get_ptr(tvb,offset+1,3));
+	SET_ADDRESS (&pinfo->src, AT_FC, 3, tvb_get_ptr(tvb,offset+5,3));
+	pinfo->srcport=0;
+	pinfo->destport=0;
+    }
+    SET_ADDRESS(&fchdr.d_id, pinfo->dst.type, pinfo->dst.len, pinfo->dst.data);
+    SET_ADDRESS(&fchdr.s_id, pinfo->src.type, pinfo->src.len, pinfo->src.data);
+
     fchdr.cs_ctl = tvb_get_guint8 (tvb, offset+4);
     fchdr.type  = tvb_get_guint8 (tvb, offset+8);
     fchdr.fctl=tvb_get_ntoh24(tvb,offset+9);
@@ -817,6 +647,78 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     pinfo->ptype = PT_EXCHG;
     pinfo->r_ctl = fchdr.r_ctl;
 
+    /* set up a conversation and conversation data */
+    /* TODO treat the fc address  s_id==00.00.00 as a wildcard matching anything */
+    conversation=find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
+                                   pinfo->ptype, pinfo->srcport,
+                                   pinfo->destport, 0);
+    if(!conversation){
+        conversation=conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst,
+                                      pinfo->ptype, pinfo->srcport,
+                                      pinfo->destport, 0);
+    }        
+    fc_conv_data=conversation_get_proto_data(conversation, proto_fc);
+    if(!fc_conv_data){
+        fc_conv_data=se_alloc(sizeof(fc_conv_data_t));
+        fc_conv_data->exchanges=se_tree_create_non_persistent(SE_TREE_TYPE_RED_BLACK, "FC Exchanges");
+        conversation_add_proto_data(conversation, proto_fc, fc_conv_data);
+    }
+
+    /* set up the exchange data */
+    /* XXX we should come up with a way to handle when the 16bit oxid wraps
+     * so that large traces will work
+     */
+    fc_ex=(fc_exchange_data *)se_tree_lookup32(fc_conv_data->exchanges, fchdr.oxid);
+    if(!fc_ex){
+        fc_ex=se_alloc(sizeof(fc_exchange_data));
+        fc_ex->first_exchange_frame=0;
+        fc_ex->last_exchange_frame=0;
+        fc_ex->lun=0xffff;
+        fc_ex->scsi_opcode=0xffff;
+        fc_ex->fc_time=pinfo->fd->abs_ts;
+	se_tree_insert32(fc_conv_data->exchanges, fchdr.oxid, fc_ex);
+    }
+    /* populate the exchange struct */
+    if(!pinfo->fd->flags.visited){
+        if(fchdr.fctl&FC_FCTL_EXCHANGE_FIRST){
+	    fc_ex->first_exchange_frame=pinfo->fd->num;
+            fc_ex->fc_time = pinfo->fd->abs_ts;
+        }
+        if(fchdr.fctl&FC_FCTL_EXCHANGE_LAST){
+            fc_ex->last_exchange_frame=pinfo->fd->num;
+        }
+    }
+
+
+    /* In the interest of speed, if "tree" is NULL, don't do any work not
+       necessary to generate protocol tree items. */
+    if (tree) {
+        ti = proto_tree_add_protocol_format (tree, proto_fc, tvb, offset,
+                                             FC_HEADER_SIZE, "Fibre Channel");
+        fc_tree = proto_item_add_subtree (ti, ett_fc);
+    }
+
+
+    /* put some nice exchange data in the tree */
+    if(!(fchdr.fctl&FC_FCTL_EXCHANGE_FIRST)){
+        proto_item *it;
+        it=proto_tree_add_uint(fc_tree, hf_fc_exchange_first_frame, tvb, 0, 0, fc_ex->first_exchange_frame);
+        PROTO_ITEM_SET_GENERATED(it);
+        if(fchdr.fctl&FC_FCTL_EXCHANGE_LAST){
+            nstime_t delta_ts;
+            nstime_delta(&delta_ts, &pinfo->fd->abs_ts, &fc_ex->fc_time);
+            it=proto_tree_add_time(ti, hf_fc_time, tvb, 0, 0, &delta_ts);
+            PROTO_ITEM_SET_GENERATED(it);
+        }
+    }
+    if(!(fchdr.fctl&FC_FCTL_EXCHANGE_LAST)){
+        proto_item *it;
+        it=proto_tree_add_uint(fc_tree, hf_fc_exchange_last_frame, tvb, 0, 0, fc_ex->last_exchange_frame);
+        PROTO_ITEM_SET_GENERATED(it);
+    }
+
+    fchdr.fced=fc_ex;
+
     is_ack = ((fchdr.r_ctl == 0xC0) || (fchdr.r_ctl == 0xC1));
 
     /* There are two ways to determine if this is the first frame of a
@@ -830,15 +732,14 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     
     is_lastframe_inseq = ((pinfo->sof_eof & PINFO_EOF_LAST_FRAME) == PINFO_EOF_LAST_FRAME);
 
-    if ((pinfo->sof_eof & PINFO_SOF_SOFF) == PINFO_SOF_SOFF) {
-        is_lastframe_inseq = fchdr.fctl & FC_FCTL_SEQ_LAST;
-    }
+    is_lastframe_inseq |= fchdr.fctl & FC_FCTL_SEQ_LAST;
     is_valid_frame = ((pinfo->sof_eof & 0x40) == 0x40);
 
     ftype = fc_get_ftype (fchdr.r_ctl, fchdr.type);
-    
+
     if (check_col (pinfo->cinfo, COL_INFO)) {
-        col_add_str (pinfo->cinfo, COL_INFO, match_strval (ftype, fc_ftype_vals));
+         col_add_str (pinfo->cinfo, COL_INFO, val_to_str (ftype, fc_ftype_vals,
+                                                          "Unknown Type (0x%x)"));
 
         if (ftype == FC_FTYPE_LINKCTL)
             col_append_fstr (pinfo->cinfo, COL_INFO, ", %s",
@@ -847,92 +748,11 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                                           "LCTL 0x%x"));
     }
     
-    /* In the interest of speed, if "tree" is NULL, don't do any work not
-       necessary to generate protocol tree items. */
-    if (tree) {
-        ti = proto_tree_add_protocol_format (tree, proto_fc, tvb, offset,
-                                             FC_HEADER_SIZE, "Fibre Channel");
-        fc_tree = proto_item_add_subtree (ti, ett_fc);
-    }
-
     /* Highlight EISL header, if present */
     if (eisl_offset != -1) {
         proto_tree_add_item (fc_tree, hf_fc_eisl, tvb, eisl_offset, 8, 0);
     }
 
-    /* match first exchange with last exchange */
-    if(fchdr.fctl&FC_FCTL_EXCHANGE_FIRST){
-        if(!pinfo->fd->flags.visited){
-            fc_exchange_data fced, *old_fced;
-
-            /* first check if we already have seen this exchange and it
-               is still open/unmatched. 
-            */
-            fced.oxid=fchdr.oxid;
-            SET_ADDRESS(&fced.s_id, fchdr.s_id.type, fchdr.s_id.len, fchdr.s_id.data);
-            SET_ADDRESS(&fced.d_id, fchdr.d_id.type, fchdr.d_id.len, fchdr.d_id.data);
-            old_fced=g_hash_table_lookup(fc_exchange_unmatched, &fced);
-            if(old_fced){
-                g_hash_table_remove(fc_exchange_unmatched, old_fced);
-            }
-            old_fced=g_mem_chunk_alloc(fc_exchange_vals);
-            old_fced->oxid=fchdr.oxid;
-            COPY_ADDRESS(&old_fced->s_id, &fchdr.s_id);
-            COPY_ADDRESS(&old_fced->d_id, &fchdr.d_id);
-	    old_fced->first_exchange_frame=pinfo->fd->num;
-            old_fced->fc_time.nsecs = pinfo->fd->abs_usecs*1000;
-            old_fced->fc_time.secs = pinfo->fd->abs_secs;
-            g_hash_table_insert(fc_exchange_unmatched, old_fced, old_fced);
-            fc_ex=old_fced;
-        } else {
-            fc_exchange_data fced, *old_fced;
-            fced.oxid=fchdr.oxid;
-            fced.first_exchange_frame=pinfo->fd->num;
-            fced.last_exchange_frame=0;
-            old_fced=g_hash_table_lookup(fc_exchange_matched, &fced);
-            fc_ex=old_fced;
-        }
-    }
-    if(fchdr.fctl&FC_FCTL_EXCHANGE_LAST){
-        if(!pinfo->fd->flags.visited){
-            fc_exchange_data fced, *old_fced;
-
-            fced.oxid=fchdr.oxid;
-            SET_ADDRESS(&fced.s_id, fchdr.d_id.type, fchdr.d_id.len, fchdr.d_id.data);
-            SET_ADDRESS(&fced.d_id, fchdr.s_id.type, fchdr.s_id.len, fchdr.s_id.data);
-            old_fced=g_hash_table_lookup(fc_exchange_unmatched, &fced);
-            if(old_fced){
-                g_hash_table_remove(fc_exchange_unmatched, old_fced);
-                old_fced->last_exchange_frame=pinfo->fd->num;
-                g_hash_table_insert(fc_exchange_matched, old_fced, old_fced);
-            }
-            fc_ex=old_fced;
-        } else {
-            fc_exchange_data fced, *old_fced;
-            fced.oxid=fchdr.oxid;
-            fced.first_exchange_frame=0;
-            fced.last_exchange_frame=pinfo->fd->num;
-            old_fced=g_hash_table_lookup(fc_exchange_matched, &fced);
-            fc_ex=old_fced;
-        }
-    }
-    if(fc_ex){
-        if(fchdr.fctl&FC_FCTL_EXCHANGE_FIRST){
-            proto_tree_add_uint(fc_tree, hf_fc_exchange_last_frame, tvb, 0, 0, fc_ex->last_exchange_frame);
-        }
-        if(fchdr.fctl&FC_FCTL_EXCHANGE_LAST){
-            nstime_t delta_time;
-            proto_tree_add_uint(fc_tree, hf_fc_exchange_first_frame, tvb, 0, 0, fc_ex->first_exchange_frame);
-            delta_time.secs = pinfo->fd->abs_secs - fc_ex->fc_time.secs;
-            delta_time.nsecs = pinfo->fd->abs_usecs*1000 - fc_ex->fc_time.nsecs;
-            if (delta_time.nsecs<0){
-                delta_time.nsecs+=1000000000;
-                delta_time.secs--;
-            }
-            proto_tree_add_time(ti, hf_fc_time, tvb, 0, 0, &delta_time);
-        }
-    }
-    fchdr.fced=fc_ex;
 
     switch (fchdr.r_ctl & 0xF0) {
 
@@ -1026,7 +846,7 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                                     fchdr.r_ctl & 0x0F);
         break;
     }
-        
+  
     proto_tree_add_uint_hidden (fc_tree, hf_fc_ftype, tvb, offset, 1,
                            ftype); 
 
@@ -1063,7 +883,7 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
 
-    dissect_fc_fctl(pinfo, fc_tree, tvb, offset+9, is_ack, fchdr.fctl);
+    dissect_fc_fctl(pinfo, fc_tree, tvb, offset+9);
 
 
     proto_tree_add_item (fc_tree, hf_fc_seqid, tvb, offset+12, 1, FALSE);
@@ -1134,12 +954,27 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         is_exchg_resp = ((tvb_get_guint8 (tvb, offset+20) & 0x80) == 0x80);
     }
 
+    if (tvb_reported_length (tvb) <= FC_HEADER_SIZE)
+        THROW(ReportedBoundsError);
+
     frag_size = tvb_reported_length (tvb)-FC_HEADER_SIZE;
 
-    /* If there is an MDS header, we need to subtract the MDS trailer size */
+    /* If there is an MDS header, we need to subtract the MDS trailer size
+     * Link Ctl, BLS & OHMS are all (encap header + FC Header + encap trailer)
+     * and are never fragmented and so we ignore the frag_size assertion for
+     *  these frames.
+     */
     if ((pinfo->ethertype == ETHERTYPE_UNK) || (pinfo->ethertype == ETHERTYPE_FCFT)) {
+         if ((frag_size <= MDSHDR_TRAILER_SIZE) ||
+             ((frag_size == MDSHDR_TRAILER_SIZE) && (ftype != FC_FTYPE_LINKCTL) &&
+              (ftype != FC_FTYPE_BLS) && (ftype != FC_FTYPE_OHMS)))
+	    THROW(ReportedBoundsError);
         frag_size -= MDSHDR_TRAILER_SIZE;
     } else if (pinfo->ethertype == ETHERTYPE_BRDWALK) {
+         if ((frag_size <= 8) ||
+             ((frag_size == MDSHDR_TRAILER_SIZE) && (ftype != FC_FTYPE_LINKCTL) &&
+              (ftype != FC_FTYPE_BLS) && (ftype != FC_FTYPE_OHMS)))
+              THROW(ReportedBoundsError);
         frag_size -= 8;         /* 4 byte of FC CRC +
                                    4 bytes of error+EOF = 8 bytes  */
     }
@@ -1155,6 +990,7 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * present, if we're configured to reassemble.
      */
     if ((ftype != FC_FTYPE_LINKCTL) && (ftype != FC_FTYPE_BLS) &&
+        (ftype != FC_FTYPE_OHMS) &&
         (!is_lastframe_inseq || !is_1frame_inseq) && fc_reassemble &&
         tvb_bytes_exist(tvb, FC_HEADER_SIZE, frag_size) && tree) {
         /* Add this to the list of fragments */
@@ -1165,15 +1001,6 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
          * SEQ_CNT of the first frame in sequence and use this value to
          * determine the actual offset into a frame.
          */
-        conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
-                                          pinfo->ptype, pinfo->oxid,
-                                          pinfo->rxid, NO_PORT2);
-        if (!conversation) {
-            conversation = conversation_new (pinfo->fd->num, &pinfo->src, &pinfo->dst,
-                                             pinfo->ptype, pinfo->oxid,
-                                             pinfo->rxid, NO_PORT2);
-        }
-        
         ckey.conv_idx = conversation->index;
         
         cdata = (fcseq_conv_data_t *)g_hash_table_lookup (fcseq_req_hash,
@@ -1188,10 +1015,10 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 cdata->seq_cnt = fchdr.seqcnt;
             }
             else {
-                req_key = g_mem_chunk_alloc (fcseq_req_keys);
+                req_key = se_alloc (sizeof(fcseq_conv_key_t));
                 req_key->conv_idx = conversation->index;
                 
-                cdata = g_mem_chunk_alloc (fcseq_req_vals);
+                cdata = se_alloc (sizeof(fcseq_conv_data_t));
                 cdata->seq_cnt = fchdr.seqcnt;
                 
                 g_hash_table_insert (fcseq_req_hash, req_key, cdata);
@@ -1204,38 +1031,51 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         else {
             real_seqcnt = fchdr.seqcnt;
         }
-        
-        frag_id = ((pinfo->oxid << 16) ^ seq_id) | is_exchg_resp ;
 
-        /* We assume that all frames are of the same max size */
-        fcfrag_head = fragment_add (tvb, FC_HEADER_SIZE, pinfo, frag_id,
-                                    fc_fragment_table,
-                                    real_seqcnt * fc_max_frame_size,
-                                    frag_size,
-                                    !is_lastframe_inseq);
+        /* Verify that this is a valid fragment */
+        if (is_lastframe_inseq && !is_1frame_inseq && !real_seqcnt) {
+             /* This is a frame that purports to be the last frame in a
+              * sequence, is not the first frame, but has a seqcnt that is
+              * 0. This is a bogus frame, don't attempt to reassemble it.
+              */
+             next_tvb = tvb_new_subset (tvb, next_offset, -1, -1);
+             if (check_col (pinfo->cinfo, COL_INFO)) {
+                  col_append_str (pinfo->cinfo, COL_INFO, " (Bogus Fragment)");
+             }
+        } else {
         
-        if (fcfrag_head) {
-            next_tvb = tvb_new_real_data (fcfrag_head->data,
-                                          fcfrag_head->datalen,
-                                          fcfrag_head->datalen);
-            tvb_set_child_real_data_tvbuff(tvb, next_tvb);
-            
-            /* Add the defragmented data to the data source list. */
+             frag_id = ((pinfo->oxid << 16) ^ seq_id) | is_exchg_resp ;
+
+             /* We assume that all frames are of the same max size */
+             fcfrag_head = fragment_add (tvb, FC_HEADER_SIZE, pinfo, frag_id,
+                                         fc_fragment_table,
+                                         real_seqcnt * fc_max_frame_size,
+                                         frag_size,
+                                         !is_lastframe_inseq);
+             
+             if (fcfrag_head) {
+                  next_tvb = tvb_new_real_data (fcfrag_head->data,
+                                                fcfrag_head->datalen,
+                                                fcfrag_head->datalen);
+                  tvb_set_child_real_data_tvbuff(tvb, next_tvb);
+                  
+                  /* Add the defragmented data to the data source list. */
             add_new_data_source(pinfo, next_tvb, "Reassembled FC");
             
             if (tree) {
-                proto_tree_add_boolean_hidden (fc_tree, hf_fc_reassembled,
-                                               tvb, offset+9, 1, 1);
+                 proto_tree_add_boolean_hidden (fc_tree, hf_fc_reassembled,
+                                                tvb, offset+9, 1, 1);
             }
-        }
-        else {
-            if (tree) {
-                proto_tree_add_boolean_hidden (fc_tree, hf_fc_reassembled,
-                                               tvb, offset+9, 1, 0);
+             }
+             else {
+                  if (tree) {
+                       proto_tree_add_boolean_hidden (fc_tree, hf_fc_reassembled,
+                                                      tvb, offset+9, 1, 0);
             }
-            next_tvb = tvb_new_subset (tvb, next_offset, -1, -1);
-            call_dissector (data_handle, next_tvb, pinfo, tree);
-            return;
+                  next_tvb = tvb_new_subset (tvb, next_offset, -1, -1);
+                  call_dissector (data_handle, next_tvb, pinfo, tree);
+                  return;
+             }
         }
     } else {
         if (tree) {
@@ -1246,9 +1086,19 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
     if ((ftype != FC_FTYPE_LINKCTL) && (ftype != FC_FTYPE_BLS)) {
-        if (!dissector_try_port (fcftype_dissector_table, ftype, next_tvb,
-                                 pinfo, tree)) {
+	/* If relative offset is used, only dissect the pdu with
+	 * offset 0 (param) */
+	if( (fchdr.fctl&FC_FCTL_REL_OFFSET) && param ){
             call_dissector (data_handle, next_tvb, pinfo, tree);
+	} else {
+	    void *saved_private_data;
+	    saved_private_data = pinfo->private_data;
+	    pinfo->private_data = &fchdr;
+	    if (!dissector_try_port (fcftype_dissector_table, ftype, 
+				next_tvb, pinfo, tree)) {
+	        call_dissector (data_handle, next_tvb, pinfo, tree);
+            }
+	    pinfo->private_data = saved_private_data;
         }
     } else if (ftype == FC_FTYPE_BLS) {
         if ((fchdr.r_ctl & 0x0F) == FC_BLS_BAACC) {
@@ -1261,6 +1111,16 @@ dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     tap_queue_packet(fc_tap, pinfo, &fchdr);
 }
 
+static void
+dissect_fc (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+    dissect_fc_helper (tvb, pinfo, tree, FALSE);
+}
+static void
+dissect_fc_ifcp (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+    dissect_fc_helper (tvb, pinfo, tree, TRUE);
+}
 
 /* Register the protocol with Ethereal */
 
@@ -1415,12 +1275,16 @@ proto_register_fc(void)
     /* Register the protocol name and description */
     proto_fc = proto_register_protocol ("Fibre Channel", "FC", "fc");
     register_dissector ("fc", dissect_fc, proto_fc);
+    register_dissector ("fc_ifcp", dissect_fc_ifcp, proto_fc);
     fc_tap = register_tap("fc");
 
     /* Required function calls to register the header fields and subtrees used */
     proto_register_field_array(proto_fc, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
+    /* subdissectors called through this table will find the fchdr structure
+     * through pinfo->private_data
+     */
     fcftype_dissector_table = register_dissector_table ("fc.ftype",
                                                         "FC Frame Type",
                                                         FT_UINT8, BASE_HEX);
