@@ -3,7 +3,7 @@
  * This decoder is for FC-SB3 version 1.4
  * Copyright 2003, Dinesh G Dutt <ddutt@cisco.com>
  *
- * $Id: packet-fcsb3.c 13243 2005-02-02 20:07:03Z etxrab $
+ * $Id: packet-fcsb3.c 16306 2005-10-25 10:38:36Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,14 +48,10 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/prefs.h>
 #include <epan/packet.h>
 #include <epan/conversation.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 #include "packet-fc.h"
 #include "packet-fcsb3.h"
 
@@ -64,24 +60,18 @@ static int proto_fc_sbccs             = -1;
 static int hf_sbccs_chid              = -1;
 static int hf_sbccs_cuid              = -1;
 static int hf_sbccs_devaddr           = -1;
-static int hf_sbccs_iui               = -1;
-static int hf_sbccs_dhflags           = -1;
 static int hf_sbccs_ccw               = -1;
 static int hf_sbccs_token             = -1;
 static int hf_sbccs_dib_iucnt         = -1;
 static int hf_sbccs_dib_datacnt       = -1;
 static int hf_sbccs_dib_ccw_cmd       = -1;
-static int hf_sbccs_dib_ccw_flags     = -1;
 static int hf_sbccs_dib_ccw_cnt       = -1;
-static int hf_sbccs_dib_statusflags   = -1;
-static int hf_sbccs_dib_status        = -1;
 static int hf_sbccs_dib_residualcnt   = -1;
 static int hf_sbccs_dib_qtuf          = -1;
 static int hf_sbccs_dib_qtu           = -1;
 static int hf_sbccs_dib_dtuf          = -1;
 static int hf_sbccs_dib_dtu           = -1;
 static int hf_sbccs_dib_ctlfn         = -1;
-static int hf_sbccs_dib_ctlparam      = -1;
 static int hf_sbccs_lrc               = -1;
 static int hf_sbccs_dib_iupacing      = -1;
 static int hf_sbccs_dev_xcp_code      = -1;
@@ -92,12 +82,60 @@ static int hf_sbccs_dib_lprcode       = -1;
 static int hf_sbccs_dib_tin_imgid_cnt = -1;
 static int hf_sbccs_dib_lrjcode       = -1;
 static int hf_sbccs_dib_ioprio        = -1;
-static int hf_sbccs_dib_cmdflags      = -1;
 static int hf_sbccs_dib_linkctlfn     = -1;
-static int hf_sbccs_dib_linkctlinfo   = -1;
+static int hf_sbccs_iui = -1;
+static int hf_sbccs_iui_as = -1;
+static int hf_sbccs_iui_es = -1;
+static int hf_sbccs_iui_val = -1;
+static int hf_sbccs_dhflags = -1;
+static int hf_sbccs_dhflags_end = -1;
+static int hf_sbccs_dhflags_chaining = -1;
+static int hf_sbccs_dhflags_earlyend = -1;
+static int hf_sbccs_dhflags_nocrc = -1;
+static int hf_sbccs_dib_ccw_flags = -1;
+static int hf_sbccs_dib_ccw_flags_cd = -1;
+static int hf_sbccs_dib_ccw_flags_cc = -1;
+static int hf_sbccs_dib_ccw_flags_sli = -1;
+static int hf_sbccs_dib_ccw_flags_crr = -1;
+static int hf_sbccs_dib_cmdflags = -1;
+static int hf_sbccs_dib_cmdflags_du = -1;
+static int hf_sbccs_dib_cmdflags_coc = -1;
+static int hf_sbccs_dib_cmdflags_syr = -1;
+static int hf_sbccs_dib_cmdflags_rex = -1;
+static int hf_sbccs_dib_cmdflags_sss = -1;
+static int hf_sbccs_dib_statusflags = -1;
+static int hf_sbccs_dib_statusflags_ffc = -1;
+static int hf_sbccs_dib_statusflags_ci = -1;
+static int hf_sbccs_dib_statusflags_cr = -1;
+static int hf_sbccs_dib_statusflags_lri = -1;
+static int hf_sbccs_dib_statusflags_rv = -1;
+static int hf_sbccs_dib_status = -1;
+static int hf_sbccs_dib_status_attention = -1;
+static int hf_sbccs_dib_status_modifier = -1;
+static int hf_sbccs_dib_status_cue = -1;
+static int hf_sbccs_dib_status_busy = -1;
+static int hf_sbccs_dib_status_channelend = -1;
+static int hf_sbccs_dib_status_deviceend = -1;
+static int hf_sbccs_dib_status_unit_check = -1;
+static int hf_sbccs_dib_status_unit_exception = -1;
+static int hf_sbccs_dib_ctlparam = -1;
+static int hf_sbccs_dib_ctlparam_rc = -1;
+static int hf_sbccs_dib_ctlparam_ru = -1;
+static int hf_sbccs_dib_ctlparam_ro = -1;
+static int hf_sbccs_dib_linkctlinfo = -1;
+static int hf_sbccs_dib_linkctlinfo_ctcconn = -1;
+static int hf_sbccs_dib_linkctlinfo_ecrcg = -1;
 
 /* Initialize the subtree pointers */
 static gint ett_fc_sbccs = -1;
+static gint ett_sbccs_iui = -1;
+static gint ett_sbccs_dhflags = -1;
+static gint ett_sbccs_dib_ccw_flags = -1;
+static gint ett_sbccs_dib_cmdflags = -1;
+static gint ett_sbccs_dib_statusflags = -1;
+static gint ett_sbccs_dib_status = -1;
+static gint ett_sbccs_dib_ctlparam = -1;
+static gint ett_sbccs_dib_linkctlinfo = -1;
 
 static dissector_handle_t data_handle;
 
@@ -228,228 +266,480 @@ static const value_string fc_sbccs_dib_lrj_errcode_val[] = {
     {0x0, NULL},
 };
 
-static gchar *get_iui_string (guint8 iui, gchar *buffer)
+static const true_false_string tfs_sbccs_iui_as = {
+	"AS is set",
+	"as is NOT set"
+};
+static const true_false_string tfs_sbccs_iui_es = {
+	"ES is set",
+	"es is NOT set"
+};
+
+static void
+dissect_iui_flags (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint16 flags)
 {
-    guint pos = 0;
-    buffer[0] = '\0';
-
-    if (iui & 0x10) {
-        strcpy (&buffer[pos], "AS, ");
-        pos += 4;
-    }
-
-    if (iui & 0x8) {
-        strcpy (&buffer[pos], "ES, ");
-        pos += 4;
-    }
-
-    strcpy (&buffer[pos], val_to_str (iui & 0x7, fc_sbccs_iu_val, "0x%x"));
-
-    return (buffer);
-}
-
-static gchar *get_dhflags_string (guint8 dhflags, gchar *buffer)
-{
-    guint pos = 0;
-    buffer[0] = '\0';
-
-    if (dhflags & 0x80) {
-        strcpy (&buffer[pos], "End, ");
-        pos += 5;
-    }
-
-    if (dhflags & 0x10) {
-        strcpy (&buffer[pos], "Chaining, ");
-        pos += 10;
-    }
-
-    if (dhflags & 0x8) {
-        strcpy (&buffer[pos], "Early End, ");
-        pos += 11;
-    }
-
-    if (dhflags & 0x4) {
-        strcpy (&buffer[pos], "No CRC");
-    }
-
-    return (buffer);
-}
-
-static gchar *get_ccw_flags_string (guint8 ccw_flags, gchar *buffer)
-{
-    guint pos = 0;
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
     
-    buffer[0] = '\0';
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_iui, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_iui);
+	}
 
-    if (ccw_flags & 0x80) {
-        strcpy (&buffer[pos], "CD, ");
-        pos += 4;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_iui_as, tvb, offset, 1, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  AS");
+	}
+	flags&=(~( 0x10 ));
 
-    if (ccw_flags & 0x40) {
-        strcpy (&buffer[pos], "CC, ");
-        pos += 4;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_iui_es, tvb, offset, 1, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  ES");
+	}
+	flags&=(~( 0x08 ));
 
-    if (ccw_flags & 0x20) {
-        strcpy (&buffer[pos], "SLI, ");
-        pos += 5;
-    }
-
-    if (ccw_flags & 0x8) {
-        strcpy (&buffer[pos], "CRR");
-        pos += 5;
-    }
-
-    return (buffer);
+        proto_tree_add_item (tree, hf_sbccs_iui_val, tvb, offset, 1, 0);
+	proto_item_append_text(item, val_to_str (flags & 0x7, fc_sbccs_iu_val, "0x%x"));
+	flags&=(~( 0x07 ));
 }
 
-static gchar *get_cmd_flag_string (guint8 cmd_flag, gchar *buffer)
+static const true_false_string tfs_sbccs_linkctlinfo_ctcconn = {
+	"CTC Conn supported",
+	"Ctc conn NOT supported"
+};
+static const true_false_string tfs_sbccs_linkctlinfo_ecrcg = {
+	"Enhanced CRC Generation supported",
+	"Enhanced crc generation NOT supported"
+};
+
+static void
+dissect_linkctlinfo (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint16 flags)
 {
-    guint pos = 0;
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_linkctlinfo, 
+				tvb, offset, 2, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_linkctlinfo);
+	}
 
-    buffer[0] = '\0';
+	proto_tree_add_boolean(tree, hf_sbccs_dib_linkctlinfo_ctcconn, tvb, offset, 2, flags);
+	if (flags&0x80){
+		proto_item_append_text(item, "  CTC Conn");
+	}
+	flags&=(~( 0x80 ));
 
-    if (cmd_flag & 0x10) {
-        strcpy (&buffer[pos], "DU, ");
-        pos += 4;
-    }
-
-    if (cmd_flag & 0x8) {
-        strcpy (&buffer[pos], "COC, ");
-        pos += 4;
-    }
-
-    if (cmd_flag & 0x4) {
-        strcpy (&buffer[pos], "SYR, ");
-        pos += 5;
-    }
-
-    if (cmd_flag & 0x2) {
-        strcpy (&buffer[pos], "REX, ");
-        pos += 5;
-    }
-
-    if (cmd_flag & 0x1) {
-        strcpy (&buffer[pos], "SSS");
-        pos += 5;
-    }
-
-    return (buffer);
+	proto_tree_add_boolean(tree, hf_sbccs_dib_linkctlinfo_ecrcg, tvb, offset, 2, flags);
+	if (flags&0x01){
+		proto_item_append_text(item, "  Enhanced CRC Gen");
+	}
+	flags&=(~( 0x01 ));
 }
 
-static gchar *get_status_flag_string (guint8 status_flag, gchar *buffer)
+
+static const true_false_string tfs_sbccs_dhflags_end = {
+	"END bit is set",
+	"end bit is NOT set"
+};
+static const true_false_string tfs_sbccs_dhflags_chaining = {
+	"CHAINING bit is set",
+	"chaining bit is NOT set"
+};
+static const true_false_string tfs_sbccs_dhflags_earlyend = {
+	"EARLYEND bit is set",
+	"earlyend bit is NOT set"
+};
+static const true_false_string tfs_sbccs_dhflags_nocrc = {
+	"NOCRC bit is set",
+	"nocrc bit is NOT set"
+};
+
+static void
+dissect_dh_flags (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint16 flags)
 {
-    guint pos = 0;
-    guint8 ffc = (status_flag & 0xD0) >> 5;
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dhflags, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dhflags);
+	}
 
-    buffer[0] = '\0';
+	proto_tree_add_boolean(tree, hf_sbccs_dhflags_end, tvb, offset, 1, flags);
+	if (flags&0x80){
+		proto_item_append_text(item, "  End");
+	}
+	flags&=(~( 0x80 ));
 
-    switch (ffc) {
-    case 0:
-        break;                  /* to avoid the catch clause below */
-    case 1:
-        strcpy (&buffer[pos], "FFC:Queuing Information Valid, ");
-        pos += 31;
-        break;
-    case 2:
-        strcpy (&buffer[pos], "FFC:Resetting Event, ");
-        pos += 21;
-        break;
-    default:
-        strcpy (&buffer[pos], "Reserved");
-        break;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_dhflags_chaining, tvb, offset, 1, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  Chaining");
+	}
+	flags&=(~( 0x10 ));
 
-    if (status_flag & 10) {
-        strcpy (&buffer[pos], "CI, ");
-        pos += 4;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_dhflags_earlyend, tvb, offset, 1, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  Early End");
+	}
+	flags&=(~( 0x08 ));
 
-    if (status_flag & 0x4) {
-        strcpy (&buffer[pos], "CR, ");
-        pos += 4;
-    }
-
-    if (status_flag & 0x2) {
-        strcpy (&buffer[pos], "LRI, ");
-        pos += 5;
-    }
-
-    if (status_flag & 0x1) {
-        strcpy (&buffer[pos], "RV");
-    }
-
-    return (buffer);
+	proto_tree_add_boolean(tree, hf_sbccs_dhflags_nocrc, tvb, offset, 1, flags);
+	if (flags&0x04){
+		proto_item_append_text(item, "  No CRC");
+	}
+	flags&=(~( 0x04 ));
 }
 
-static gchar *get_status_string (guint8 status, gchar *buffer)
+static const true_false_string tfs_sbccs_ccwflags_cd = {
+	"CD is set",
+	"cd is NOT set"
+};
+static const true_false_string tfs_sbccs_ccwflags_cc = {
+	"CC is set",
+	"cc is NOT set"
+};
+static const true_false_string tfs_sbccs_ccwflags_sli = {
+	"SLI is set",
+	"sli is NOT set"
+};
+static const true_false_string tfs_sbccs_ccwflags_crr = {
+	"CRR is set",
+	"crr is NOT set"
+};
+
+static void
+dissect_ccw_flags (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint8 flags)
 {
-    guint pos = 0;
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_ccw_flags, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_ccw_flags);
+	}
 
-    buffer[0] = '\0';
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ccw_flags_cd, tvb, offset, 1, flags);
+	if (flags&0x80){
+		proto_item_append_text(item, "  CD");
+	}
+	flags&=(~( 0x80 ));
 
-    if (status & 0x80) {
-        strcpy (&buffer[pos], "Attention, ");
-        pos += 11;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ccw_flags_cc, tvb, offset, 1, flags);
+	if (flags&0x40){
+		proto_item_append_text(item, "  CC");
+	}
+	flags&=(~( 0x40 ));
 
-    if (status & 0x40) {
-        strcpy (&buffer[pos], "Status Modifier, ");
-        pos += 17;
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ccw_flags_sli, tvb, offset, 1, flags);
+	if (flags&0x20){
+		proto_item_append_text(item, "  SLI");
+	}
+	flags&=(~( 0x20 ));
 
-    if (status & 0x20) {
-        strcpy (&buffer[pos], "Control-Unit End, ");
-        pos += 18;
-    }
-
-    if (status & 0x10) {
-        strcpy (&buffer[pos], "Busy, ");
-        pos += 6;
-    }
-
-    if (status & 0x8) {
-        strcpy (&buffer[pos], "Channel End, ");
-        pos += 12;
-    }
-
-    if (status & 0x4) {
-        strcpy (&buffer[pos], "Device End, ");
-        pos += 12;
-    }
-
-    if (status & 0x2) {
-        strcpy (&buffer[pos], "Unit Check, ");
-        pos += 12;
-    }
-
-    if (status & 0x1) {
-        strcpy (&buffer[pos], "Unit Exception");
-    }
-
-    return (buffer);
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ccw_flags_crr, tvb, offset, 1, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  CRR");
+	}
+	flags&=(~( 0x08 ));
 }
 
-static gchar *get_sel_rst_param_string (guint8 ctlparam, gchar *buffer)
+static const true_false_string tfs_sbccs_cmdflags_du = {
+	"DU is set",
+	"du is NOT set"
+};
+static const true_false_string tfs_sbccs_cmdflags_coc = {
+	"COC is set",
+	"coc is NOT set"
+};
+static const true_false_string tfs_sbccs_cmdflags_syr = {
+	"SYR is set",
+	"syr is NOT set"
+};
+static const true_false_string tfs_sbccs_cmdflags_rex = {
+	"REX is set",
+	"rex is NOT set"
+};
+static const true_false_string tfs_sbccs_cmdflags_sss = {
+	"SSS is set",
+	"sss is NOT set"
+};
+
+static void
+dissect_cmd_flags (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint8 flags)
 {
-    guint pos = 0;
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_cmdflags, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_cmdflags);
+	}
 
-    buffer[0] = '\0';
+	proto_tree_add_boolean(tree, hf_sbccs_dib_cmdflags_du, tvb, offset, 1, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  DU");
+	}
+	flags&=(~( 0x10 ));
 
-    if (ctlparam & 0x80) {
-        strcpy (&buffer[pos], "RC, ");
-        pos += 4;
-    }
-    if (ctlparam & 0x10) {
-        strcpy (&buffer[pos], "RU, ");
-        pos += 4;
-    }
-    if (ctlparam & 0x8) {
-        strcpy (&buffer[pos], "RO");
-    }
+	proto_tree_add_boolean(tree, hf_sbccs_dib_cmdflags_coc, tvb, offset, 1, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  COC");
+	}
+	flags&=(~( 0x08 ));
 
-    return (buffer);
+	proto_tree_add_boolean(tree, hf_sbccs_dib_cmdflags_syr, tvb, offset, 1, flags);
+	if (flags&0x04){
+		proto_item_append_text(item, "  SYR");
+	}
+	flags&=(~( 0x04 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_cmdflags_rex, tvb, offset, 1, flags);
+	if (flags&0x02){
+		proto_item_append_text(item, "  REX");
+	}
+	flags&=(~( 0x02 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_cmdflags_sss, tvb, offset, 1, flags);
+	if (flags&0x01){
+		proto_item_append_text(item, "  SSS");
+	}
+	flags&=(~( 0x01 ));
+}
+
+static const value_string status_ffc_val[] = {
+	{ 0, "" },
+	{ 1, "FFC:Queuing Information Valid" },
+	{ 2, "FFC:Resetting Event" },
+	{ 0, NULL }
+};
+
+static const true_false_string tfs_sbccs_statusflags_ci = {
+	"CI is set",
+	"ci is NOT set"
+};
+static const true_false_string tfs_sbccs_statusflags_cr = {
+	"CR is set",
+	"cr is NOT set"
+};
+static const true_false_string tfs_sbccs_statusflags_lri = {
+	"LRI is set",
+	"lri is NOT set"
+};
+static const true_false_string tfs_sbccs_statusflags_rv = {
+	"RV is set",
+	"rv is NOT set"
+};
+
+static void
+dissect_status_flags (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint8 flags)
+{
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_statusflags, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_statusflags);
+	}
+
+
+        proto_tree_add_item (tree, hf_sbccs_dib_statusflags_ffc, tvb, offset, 1, 0);
+	proto_item_append_text(item, val_to_str ((flags>>5)&0x07, status_ffc_val, "Reserved:0x%x"));
+	flags&=(~( 0xE0 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_statusflags_ci, tvb, offset, 1, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  CI");
+	}
+	flags&=(~( 0x10 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_statusflags_cr, tvb, offset, 1, flags);
+	if (flags&0x04){
+		proto_item_append_text(item, "  CR");
+	}
+	flags&=(~( 0x04 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_statusflags_lri, tvb, offset, 1, flags);
+	if (flags&0x02){
+		proto_item_append_text(item, "  LRI");
+	}
+	flags&=(~( 0x02 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_statusflags_rv, tvb, offset, 1, flags);
+	if (flags&0x01){
+		proto_item_append_text(item, "  RV");
+	}
+	flags&=(~( 0x01 ));
+
+}
+
+static const true_false_string tfs_sbccs_status_attention = {
+	"ATTENTION is set",
+	"attention is NOT set"
+};
+static const true_false_string tfs_sbccs_status_modifier = {
+	"STATUS MODIFIER is set",
+	"status modifier is NOT set"
+};
+static const true_false_string tfs_sbccs_status_cue = {
+	"CONTROL-UNIT END is set",
+	"control-unit end is NOT set"
+};
+static const true_false_string tfs_sbccs_status_busy = {
+	"BUSY is set",
+	"busy is NOT set"
+};
+static const true_false_string tfs_sbccs_status_channelend = {
+	"CHANNEL-END is set",
+	"channel-end is NOT set"
+};
+static const true_false_string tfs_sbccs_status_deviceend = {
+	"DEVICE-END is set",
+	"device-end is NOT set"
+};
+static const true_false_string tfs_sbccs_status_unitcheck = {
+	"UNIT CHECK is set",
+	"unit check is NOT set"
+};
+static const true_false_string tfs_sbccs_status_unitexception = {
+	"UNIT EXCEPTION is set",
+	"unit exception is NOT set"
+};
+
+static void
+dissect_status (packet_info *pinfo, proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint8 flags)
+{
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_status, 
+				tvb, offset, 1, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_status);
+	}
+
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_attention, tvb, offset, 1, flags);
+	if (flags&0x80){
+		proto_item_append_text(item, "  Attention");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Attention");
+		}
+	}
+	flags&=(~( 0x80 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_modifier, tvb, offset, 1, flags);
+	if (flags&0x40){
+		proto_item_append_text(item, "  Status Modifier");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Status Modifier");
+		}
+	}
+	flags&=(~( 0x40 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_cue, tvb, offset, 1, flags);
+	if (flags&0x20){
+		proto_item_append_text(item, "  Control-Unit End");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Control-Unit End");
+		}
+	}
+	flags&=(~( 0x20 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_busy, tvb, offset, 1, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  Busy");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Busy");
+		}
+	}
+	flags&=(~( 0x10 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_channelend, tvb, offset, 1, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  Channel End");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Channel End");
+		}
+	}
+	flags&=(~( 0x08 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_deviceend, tvb, offset, 1, flags);
+	if (flags&0x04){
+		proto_item_append_text(item, "  Device End");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Device End");
+		}
+	}
+	flags&=(~( 0x04 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_unit_check, tvb, offset, 1, flags);
+	if (flags&0x02){
+		proto_item_append_text(item, "  Unit Check");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Unit Check");
+		}
+	}
+	flags&=(~( 0x02 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_status_unit_exception, tvb, offset, 1, flags);
+	if (flags&0x01){
+		proto_item_append_text(item, "  Unit Exception");
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_str(pinfo->cinfo, COL_INFO, "  Unit Exception");
+		}
+	}
+	flags&=(~( 0x01 ));
+
+}
+
+static const true_false_string tfs_sbccs_ctlparam_rc = {
+	"RC is SET",
+	"rc is NOT set"
+};
+static const true_false_string tfs_sbccs_ctlparam_ru = {
+	"RU is SET",
+	"ru is NOT set"
+};
+static const true_false_string tfs_sbccs_ctlparam_ro = {
+	"RO is SET",
+	"ro is NOT set"
+};
+
+static void
+dissect_sel_rst_param (proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint32 flags)
+{
+	proto_item *item=NULL;
+	proto_tree *tree=NULL;
+    
+	if(parent_tree){
+		item=proto_tree_add_uint(parent_tree, hf_sbccs_dib_ctlparam, 
+				tvb, offset, 3, flags);
+		tree=proto_item_add_subtree(item, ett_sbccs_dib_ctlparam);
+	}
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ctlparam_rc, tvb, offset, 3, flags);
+	if (flags&0x80){
+		proto_item_append_text(item, "  RC");
+	}
+	flags&=(~( 0x80 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ctlparam_ru, tvb, offset, 3, flags);
+	if (flags&0x10){
+		proto_item_append_text(item, "  RU");
+	}
+	flags&=(~( 0x10 ));
+
+	proto_tree_add_boolean(tree, hf_sbccs_dib_ctlparam_ro, tvb, offset, 3, flags);
+	if (flags&0x08){
+		proto_item_append_text(item, "  RO");
+	}
+	flags&=(~( 0x08 ));
 }
 
 static void get_fc_sbccs_conv_data (tvbuff_t *tvb, guint offset,
@@ -472,7 +762,6 @@ dissect_fc_sbccs_sb3_iu_hdr (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
     proto_item *subti;
     proto_tree *sb3hdr_tree;
     proto_tree *iuhdr_tree;
-    gchar buffer[256];
     guint8 iui, dhflags;
     guint type;
     
@@ -501,13 +790,10 @@ dissect_fc_sbccs_sb3_iu_hdr (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
         offset += FC_SBCCS_SB3_HDR_SIZE;
 
         iui = tvb_get_guint8 (tvb, offset);
-        proto_tree_add_uint_format (iuhdr_tree, hf_sbccs_iui, tvb, offset, 1,
-                                    iui, "Information Unit Identifier: 0x%x (%s)",
-                                    iui, get_iui_string (iui, buffer));
+	dissect_iui_flags(iuhdr_tree, tvb, offset, iui);
+
         dhflags = tvb_get_guint8 (tvb, offset+1);
-        proto_tree_add_uint_format (iuhdr_tree, hf_sbccs_dhflags, tvb, offset, 1,
-                                    dhflags, "DH Flags: 0x%x (%s)",
-                                    dhflags, get_dhflags_string (dhflags, buffer));
+	dissect_dh_flags(iuhdr_tree, tvb, offset+1, dhflags);
         
         proto_tree_add_item (iuhdr_tree, hf_sbccs_ccw, tvb, offset+2, 2, 0);
         proto_tree_add_item (iuhdr_tree, hf_sbccs_token, tvb, offset+5, 3, 0);
@@ -529,7 +815,6 @@ static void dissect_fc_sbccs_dib_cmd_hdr (tvbuff_t *tvb, packet_info *pinfo,
                                           proto_tree *tree, guint offset)
 {
     guint8 flags;
-    gchar buffer[64];
 
     if (check_col (pinfo->cinfo, COL_INFO)) {
         col_append_fstr (pinfo->cinfo, COL_INFO,
@@ -542,17 +827,14 @@ static void dissect_fc_sbccs_dib_cmd_hdr (tvbuff_t *tvb, packet_info *pinfo,
         proto_tree_add_item (tree, hf_sbccs_dib_ccw_cmd, tvb, offset, 1, 0);
 
         flags = tvb_get_guint8 (tvb, offset+1);
-        proto_tree_add_uint_format (tree, hf_sbccs_dib_ccw_flags, tvb,
-                                    offset+1, 1, flags,
-                                    "CCW Control Flags: 0x%x(%s)", flags,
-                                    get_ccw_flags_string (flags, buffer));
+	dissect_ccw_flags(tree, tvb, offset+1, flags);
+
         proto_tree_add_item (tree, hf_sbccs_dib_ccw_cnt, tvb, offset+2, 2, 0);
         proto_tree_add_item (tree, hf_sbccs_dib_ioprio, tvb, offset+5, 1, 0);
 
         flags = tvb_get_guint8 (tvb, offset+7);
-        proto_tree_add_uint_format (tree, hf_sbccs_dib_cmdflags, tvb, offset+7,
-                                    1, flags, "Command Flags: 0x%x(%s)", flags,
-                                    get_cmd_flag_string (flags, buffer));
+	dissect_cmd_flags(tree, tvb, offset+7, flags);
+
         proto_tree_add_item (tree, hf_sbccs_dib_iucnt, tvb, offset+9, 1, 0);
         proto_tree_add_item (tree, hf_sbccs_dib_datacnt, tvb, offset+10, 2, 0);
         proto_tree_add_item (tree, hf_sbccs_lrc, tvb, offset+12, 4, 0);
@@ -565,30 +847,18 @@ static void dissect_fc_sbccs_dib_status_hdr (tvbuff_t *tvb, packet_info *pinfo,
 {
     guint8 flags;
     gboolean rv_valid, qparam_valid;
-    gchar buffer[128];
     tvbuff_t *next_tvb;
     guint16 supp_status_cnt = 0;
 
-    if (check_col (pinfo->cinfo, COL_INFO)) {
-        col_append_fstr (pinfo->cinfo, COL_INFO,
-                         ": %s",
-                         get_status_string (tvb_get_guint8 (tvb, offset+1),
-                                            buffer));
-    }
-    
     if (tree) {
         flags = tvb_get_guint8 (tvb, offset);
         rv_valid = flags & 0x1; /* if residual count is valid */
-        qparam_valid = (((flags & 0xD0) >> 5) == 0x1); /* From the FFC field */
-        proto_tree_add_uint_format (tree, hf_sbccs_dib_statusflags, tvb, offset,
-                                    1, flags, "Status Flags: 0x%x(%s)",
-                                    flags, get_status_flag_string (flags,
-                                                                   buffer));
+        qparam_valid = (((flags & 0xE0) >> 5) == 0x1); /* From the FFC field */
+	dissect_status_flags(tree, tvb, offset, flags);
         
         flags = tvb_get_guint8 (tvb, offset+1);
-        proto_tree_add_uint_format (tree, hf_sbccs_dib_status, tvb, offset+1,
-                                    1, flags, "Status: 0x%x(%s)", flags,
-                                    get_status_string (flags, buffer));
+	dissect_status(pinfo, tree, tvb, offset+1, flags);
+
         if (rv_valid) {
             proto_tree_add_item (tree, hf_sbccs_dib_residualcnt, tvb, offset+2,
                                  2, 0);
@@ -623,7 +893,6 @@ static void dissect_fc_sbccs_dib_ctl_hdr (tvbuff_t *tvb, packet_info *pinfo,
                                           proto_tree *tree, guint offset)
 {
     guint8 ctlfn;
-    gchar buffer[128];
 
     ctlfn = tvb_get_guint8 (tvb, offset);
     if (check_col (pinfo->cinfo, COL_INFO)) {
@@ -639,13 +908,7 @@ static void dissect_fc_sbccs_dib_ctl_hdr (tvbuff_t *tvb, packet_info *pinfo,
         /* Control Function Parameter is to be interpreted in some cases */
         switch (ctlfn) {
         case FC_SBCCS_CTL_FN_SEL_RST:
-            proto_tree_add_uint_format (tree, hf_sbccs_dib_ctlparam, tvb,
-                                        offset+1, 3,
-                                        tvb_get_ntoh24 (tvb, offset+1),
-                                        "Control Parameter: 0x%x(%s)",
-                                        tvb_get_ntoh24 (tvb, offset+1),
-                                        get_sel_rst_param_string (ctlfn,
-                                                                  buffer));
+            dissect_sel_rst_param(tree, tvb, offset+1, tvb_get_ntoh24 (tvb, offset+1));
             break;
         case FC_SBCCS_CTL_FN_DEV_XCP:
             proto_tree_add_item (tree, hf_sbccs_dev_xcp_code, tvb, offset+1,
@@ -678,7 +941,6 @@ static void dissect_fc_sbccs_dib_link_hdr (tvbuff_t *tvb, packet_info *pinfo,
 {
     guint8 link_ctl;
     guint16 ctl_info;
-    gchar buffer[128];
     guint link_payload_len, i;
 
     if (check_col (pinfo->cinfo, COL_INFO)) {
@@ -697,18 +959,7 @@ static void dissect_fc_sbccs_dib_link_hdr (tvbuff_t *tvb, packet_info *pinfo,
         switch (link_ctl) {
         case FC_SBCCS_LINK_CTL_FN_ELP:
         case FC_SBCCS_LINK_CTL_FN_LPE:
-            buffer[0] = '\0';
-            if (ctl_info & 0x1) {
-                strcpy (buffer, "Enhanced CRC Gen, ");
-            }
-            if (ctl_info & 0x80) {
-                strcpy (&buffer[18], "CTC Conn");
-            }
-
-            proto_tree_add_uint_format (tree, hf_sbccs_dib_linkctlinfo, tvb,
-                                        offset+2, 2, ctl_info,
-                                        "Link Control Info: 0x%x(%s)", ctl_info,
-                                        buffer);
+            dissect_linkctlinfo(tree, tvb, offset+2, ctl_info);
             break;
         case FC_SBCCS_LINK_CTL_FN_LPR:
             proto_tree_add_item (tree, hf_sbccs_dib_lprcode, tvb, offset+2, 1,
@@ -889,23 +1140,14 @@ proto_register_fcsbccs (void)
         { &hf_sbccs_dib_ccw_cmd,
           {"CCW Command", "sbccs.ccwcmd", FT_UINT8, BASE_HEX,
            VALS (fc_sbccs_dib_cmd_val), 0x0, "", HFILL}},
-        { &hf_sbccs_dib_ccw_flags,
-          {"CCW Control Flags", "sbccs.ccwflags", FT_UINT8, BASE_HEX, NULL,
-           0x0, "", HFILL}},
         { &hf_sbccs_dib_ccw_cnt,
           {"CCW Count", "sbccs.ccwcnt", FT_UINT16, BASE_DEC, NULL, 0x0, "",
            HFILL}},
         { &hf_sbccs_dib_ioprio,
           {"I/O Priority", "sbccs.ioprio", FT_UINT8, BASE_DEC, NULL, 0x0,
            "", HFILL}},
-        { &hf_sbccs_dib_cmdflags,
-          {"Command Flags", "sbccs.cmdflags", FT_UINT8, BASE_HEX, NULL, 0x0,
-           "", HFILL}},
-        { &hf_sbccs_dib_statusflags,
-          {"Status Flags", "sbccs.statusflags", FT_UINT8, BASE_HEX, NULL,
-           0x0, "", HFILL}},
         { &hf_sbccs_dib_status,
-          {"Status", "sbccs.status", FT_UINT8, BASE_DEC, NULL, 0x0, "",
+          {"Status", "sbccs.status", FT_UINT8, BASE_HEX, NULL, 0x0, "",
            HFILL}},
         { &hf_sbccs_dib_residualcnt,
           {"Residual Count", "sbccs.residualcnt", FT_UINT8, BASE_DEC,
@@ -928,15 +1170,9 @@ proto_register_fcsbccs (void)
         { &hf_sbccs_dib_ctlfn,
           {"Control Function", "sbccs.ctlfn", FT_UINT8, BASE_HEX,
            VALS (fc_sbccs_dib_ctl_fn_val), 0x0, "", HFILL}},
-        { &hf_sbccs_dib_ctlparam,
-          {"Control Parameters", "sbccs.ctlparam", FT_UINT24, BASE_HEX,
-           NULL, 0x0, "", HFILL}},
         { &hf_sbccs_dib_linkctlfn,
           {"Link Control Function", "sbccs.linkctlfn", FT_UINT8, BASE_HEX,
            VALS (fc_sbccs_dib_link_ctl_fn_val), 0x0, "", HFILL}},
-        { &hf_sbccs_dib_linkctlinfo,
-          {"Link Control Information", "sbccs.linkctlinfo", FT_UINT16,
-           BASE_HEX, NULL, 0x0, "", HFILL}},
         { &hf_sbccs_dib_ctccntr,
           {"CTC Counter", "sbccs.ctccntr", FT_UINT16, BASE_DEC, NULL, 0x0,
            "", HFILL}},
@@ -961,12 +1197,137 @@ proto_register_fcsbccs (void)
         { &hf_sbccs_dib_lrjcode,
           {"LRJ Reaspn Code", "sbccs.lrjcode", FT_UINT8, BASE_HEX,
            VALS (fc_sbccs_dib_lrj_errcode_val), 0x7F, "", HFILL}},
+        { &hf_sbccs_iui_as,
+          {"AS", "sbccs.iui.as", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_iui_as), 0x10, "", HFILL}},
+        { &hf_sbccs_iui_es,
+          {"ES", "sbccs.iui.es", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_iui_es), 0x08, "", HFILL}},
+        { &hf_sbccs_iui_val,
+          {"Val", "sbccs.iui.val", FT_UINT8, BASE_HEX,
+           VALS(fc_sbccs_iu_val), 0x07, "", HFILL}},
+        { &hf_sbccs_dhflags_end,
+          {"End", "sbccs.dhflags.end", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_dhflags_end), 0x80, "", HFILL}},
+        { &hf_sbccs_dhflags_chaining,
+          {"Chaining", "sbccs.dhflags.chaining", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_dhflags_chaining), 0x10, "", HFILL}},
+        { &hf_sbccs_dhflags_earlyend,
+          {"Early End", "sbccs.dhflags.earlyend", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_dhflags_earlyend), 0x08, "", HFILL}},
+        { &hf_sbccs_dhflags_nocrc,
+          {"No CRC", "sbccs.dhflags.nocrc", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_dhflags_nocrc), 0x04, "", HFILL}},
+        { &hf_sbccs_dib_ccw_flags,
+          {"CCW Control Flags", "sbccs.ccwflags", FT_UINT8, BASE_HEX, NULL,
+           0x0, "", HFILL}},
+        { &hf_sbccs_dib_ccw_flags_cd,
+          {"CD", "sbccs.ccwflags.cd", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_ccwflags_cd), 0x80, "", HFILL}},
+        { &hf_sbccs_dib_ccw_flags_cc,
+          {"CC", "sbccs.ccwflags.cc", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_ccwflags_cc), 0x40, "", HFILL}},
+        { &hf_sbccs_dib_ccw_flags_sli,
+          {"SLI", "sbccs.ccwflags.sli", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_ccwflags_sli), 0x20, "", HFILL}},
+        { &hf_sbccs_dib_ccw_flags_crr,
+          {"CRR", "sbccs.ccwflags.crr", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_ccwflags_crr), 0x08, "", HFILL}},
+        { &hf_sbccs_dib_cmdflags,
+          {"Command Flags", "sbccs.cmdflags", FT_UINT8, BASE_HEX, NULL, 0x0,
+           "", HFILL}},
+        { &hf_sbccs_dib_cmdflags_du,
+          {"DU", "sbccs.cmdflags.du", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_cmdflags_du), 0x10, "", HFILL}},
+        { &hf_sbccs_dib_cmdflags_coc,
+          {"COC", "sbccs.cmdflags.coc", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_cmdflags_coc), 0x08, "", HFILL}},
+        { &hf_sbccs_dib_cmdflags_syr,
+          {"SYR", "sbccs.cmdflags.syr", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_cmdflags_syr), 0x04, "", HFILL}},
+        { &hf_sbccs_dib_cmdflags_rex,
+          {"REX", "sbccs.cmdflags.rex", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_cmdflags_rex), 0x02, "", HFILL}},
+        { &hf_sbccs_dib_cmdflags_sss,
+          {"SSS", "sbccs.cmdflags.sss", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_cmdflags_sss), 0x01, "", HFILL}},
+        { &hf_sbccs_dib_statusflags,
+          {"Status Flags", "sbccs.statusflags", FT_UINT8, BASE_HEX,
+           NULL, 0, "", HFILL}},
+        { &hf_sbccs_dib_statusflags_ffc,
+          {"FFC", "sbccs.statusflags.ffc", FT_UINT8, BASE_HEX,
+           VALS(status_ffc_val), 0xE0, "", HFILL}},
+        { &hf_sbccs_dib_statusflags_ci,
+          {"CI", "sbccs.statusflags.ci", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_statusflags_ci), 0x10, "", HFILL}},
+        { &hf_sbccs_dib_statusflags_cr,
+          {"CR", "sbccs.statusflags.cr", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_statusflags_cr), 0x04, "", HFILL}},
+        { &hf_sbccs_dib_statusflags_lri,
+          {"LRI", "sbccs.statusflags.lri", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_statusflags_lri), 0x02, "", HFILL}},
+        { &hf_sbccs_dib_statusflags_rv,
+          {"RV", "sbccs.statusflags.rv", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_statusflags_rv), 0x01, "", HFILL}},
+        { &hf_sbccs_dib_status_attention,
+          {"Attention", "sbccs.status.attention", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_attention), 0x80, "", HFILL}},
+        { &hf_sbccs_dib_status_modifier,
+          {"Status Modifier", "sbccs.status.modifier", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_modifier), 0x40, "", HFILL}},
+        { &hf_sbccs_dib_status_cue,
+          {"Control-Unit End", "sbccs.status.cue", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_cue), 0x20, "", HFILL}},
+        { &hf_sbccs_dib_status_busy,
+          {"Busy", "sbccs.status.busy", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_busy), 0x10, "", HFILL}},
+        { &hf_sbccs_dib_status_channelend,
+          {"Channel End", "sbccs.status.channel_end", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_channelend), 0x08, "", HFILL}},
+        { &hf_sbccs_dib_status_deviceend,
+          {"Device End", "sbccs.status.device_end", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_deviceend), 0x04, "", HFILL}},
+        { &hf_sbccs_dib_status_unit_check,
+          {"Unit Check", "sbccs.status.unit_check", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_unitcheck), 0x02, "", HFILL}},
+        { &hf_sbccs_dib_status_unit_exception,
+          {"Unit Exception", "sbccs.status.unitexception", FT_BOOLEAN, 8,
+           TFS(&tfs_sbccs_status_unitexception), 0x01, "", HFILL}},
+        { &hf_sbccs_dib_ctlparam,
+          {"Control Parameters", "sbccs.ctlparam", FT_UINT24, BASE_HEX,
+           NULL, 0x0, "", HFILL}},
+        { &hf_sbccs_dib_ctlparam_rc,
+          {"RC", "sbccs.ctlparam.rc", FT_BOOLEAN, 24,
+           TFS(&tfs_sbccs_ctlparam_rc), 0x80, "", HFILL}},
+        { &hf_sbccs_dib_ctlparam_ru,
+          {"RU", "sbccs.ctlparam.ru", FT_BOOLEAN, 24,
+           TFS(&tfs_sbccs_ctlparam_ru), 0x10, "", HFILL}},
+        { &hf_sbccs_dib_ctlparam_ro,
+          {"RO", "sbccs.ctlparam.ro", FT_BOOLEAN, 24,
+           TFS(&tfs_sbccs_ctlparam_ro), 0x08, "", HFILL}},
+        { &hf_sbccs_dib_linkctlinfo,
+          {"Link Control Information", "sbccs.linkctlinfo", FT_UINT16,
+           BASE_HEX, NULL, 0x0, "", HFILL}},
+        { &hf_sbccs_dib_linkctlinfo_ctcconn,
+          {"CTC Conn", "sbccs.linkctlinfo.ctc_conn", FT_BOOLEAN, 16,
+           TFS(&tfs_sbccs_linkctlinfo_ctcconn), 0x80, "", HFILL}},
+        { &hf_sbccs_dib_linkctlinfo_ecrcg,
+          {"Enhanced CRC Generation", "sbccs.linkctlinfo.ecrcg", FT_BOOLEAN, 16,
+           TFS(&tfs_sbccs_linkctlinfo_ecrcg), 0x01, "", HFILL}},
     };
 
 
     /* Setup protocol subtree array */
     static gint *ett[] = {
         &ett_fc_sbccs,
+        &ett_sbccs_iui,
+        &ett_sbccs_dhflags,
+        &ett_sbccs_dib_ccw_flags,
+        &ett_sbccs_dib_cmdflags,
+        &ett_sbccs_dib_statusflags,
+        &ett_sbccs_dib_status,
+        &ett_sbccs_dib_ctlparam,
+        &ett_sbccs_dib_linkctlinfo,
     };
 
     /* Register the protocol name and description */
