@@ -3,7 +3,7 @@
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  * Copyright 2001, Juan Toledo <toledo@users.sourceforge.net> (Passive FTP)
  *
- * $Id: packet-ftp.c 13243 2005-02-02 20:07:03Z etxrab $
+ * $Id: packet-ftp.c 15381 2005-08-17 08:50:04Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,6 +39,7 @@
 #include <epan/packet.h>
 #include <epan/strutil.h>
 #include <epan/conversation.h>
+#include <epan/emem.h>
 
 static int proto_ftp = -1;
 static int proto_ftp_data = -1;
@@ -176,7 +177,7 @@ parse_port_pasv(const guchar *line, int linelen, guint32 *ftp_ip,
 	/*
 	 * Copy the rest of the line into a null-terminated buffer.
 	 */
-	args = g_malloc(linelen + 1);
+	args = ep_alloc(linelen + 1);
 	memcpy(args, line, linelen);
 	args[linelen] = '\0';
 	p = args;
@@ -219,7 +220,6 @@ parse_port_pasv(const guchar *line, int linelen, guint32 *ftp_ip,
 			p++;
 	}
 
-	g_free(args);
 	return ret;
 }
 
