@@ -2,7 +2,7 @@
  * Routines for gnutella dissection
  * Copyright 2001, B. Johannessen <bob@havoq.com>
  *
- * $Id: packet-gnutella.c 13211 2005-01-31 00:28:02Z guy $
+ * $Id: packet-gnutella.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,6 +35,7 @@
 
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-gnutella.h"
 #include "packet-tcp.h"
 
@@ -351,7 +352,7 @@ static void dissect_gnutella_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
 	proto_tree *gnutella_query_tree;
 	guint8 payload_descriptor;
 	guint32 size = 0;
-	char *payload_descriptor_text;
+	const char *payload_descriptor_text;
 
 	if (tree) {
 		ti = proto_tree_add_item(tree,
