@@ -1,7 +1,7 @@
 /* packet-gsm_sms_ud.c
  * Routines for GSM SMS TP-UD (GSM 03.40) dissection
  *
- * $Id: packet-gsm_sms_ud.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-gsm_sms_ud.c 16039 2005-09-28 16:16:37Z guy $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -70,6 +70,7 @@
 #include <epan/packet.h>
 
 #include <epan/prefs.h>
+#include <epan/emem.h>
 #include <epan/reassemble.h>
 
 static void dissect_gsm_sms_ud(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
@@ -632,7 +633,7 @@ proto_reg_handoff_gsm_sms_ud(void)
 			proto_gsm_sms_ud);
 			
 	wsp_handle = find_dissector("wsp-cl");
-	g_assert(wsp_handle);
+	DISSECTOR_ASSERT(wsp_handle);
 }
 
 
