@@ -4,7 +4,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@samba.org> Added a few 
  *		   bits and pieces ...
  *
- * $Id: packet-gssapi.c 14720 2005-06-21 09:38:59Z sahlberg $
+ * $Id: packet-gssapi.c 17802 2006-04-04 07:53:39Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -80,8 +80,8 @@ gssapi_oid_hash(gconstpointer k)
 }
 
 void
-gssapi_init_oid(char *oid, int proto, int ett, dissector_handle_t handle,
-		dissector_handle_t wrap_handle, gchar *comment)
+gssapi_init_oid(const char *oid, int proto, int ett, dissector_handle_t handle,
+		dissector_handle_t wrap_handle, const gchar *comment)
 {
 	char *key = g_strdup(oid);
 	gssapi_oid_value *value = g_malloc(sizeof(*value));
@@ -101,9 +101,12 @@ gssapi_init_oid(char *oid, int proto, int ett, dissector_handle_t handle,
  * an argument.
  */
 gssapi_oid_value *
-gssapi_lookup_oid_str(gchar *oid_key)
+gssapi_lookup_oid_str(const char *oid_key)
 {
 	gssapi_oid_value *value;
+	if(!oid_key){
+		return NULL;
+	}
 	value = g_hash_table_lookup(gssapi_oids, oid_key);
 	return value;
 }
@@ -124,7 +127,7 @@ dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	gboolean pc, ind_field;
 	gint32 tag;
 	guint32 len1;
-	gchar oid[MAX_OID_STR_LEN];
+	const char *oid;
 
 	start_offset=0;
 	offset=start_offset;
@@ -234,7 +237,7 @@ dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 		/* Read oid */
 		oid_start_offset=offset;
-		offset=dissect_ber_object_identifier(FALSE, pinfo, subtree, tvb, offset, hf_gssapi_oid, oid);
+		offset=dissect_ber_object_identifier_str(FALSE, pinfo, subtree, tvb, offset, hf_gssapi_oid, &oid);
 
 		/*
 		 * Hand off to subdissector.
@@ -433,6 +436,8 @@ static dcerpc_auth_subdissector_fns gssapi_auth_fns = {
 void
 proto_reg_handoff_gssapi(void)
 {
+	dissector_handle_t gssapi_handle;
+
 	ntlmssp_handle = find_dissector("ntlmssp");
 
 	register_dcerpc_auth_subdissector(DCE_C_AUTHN_LEVEL_CONNECT,
@@ -444,4 +449,7 @@ proto_reg_handoff_gssapi(void)
 	register_dcerpc_auth_subdissector(DCE_C_AUTHN_LEVEL_PKT_PRIVACY,
 					  DCE_C_RPC_AUTHN_PROTOCOL_SPNEGO,
 					  &gssapi_auth_fns);
+
+	gssapi_handle = create_dissector_handle(dissect_gssapi,	proto_gssapi);
+	dissector_add_string("dns.tsig.mac", "gss.microsoft.com", gssapi_handle);
 }
