@@ -9,7 +9,7 @@
  * Copyright 2002, Tim Potter <tpot@samba.org>
  * Copyright 1999, Andrew Tridgell <tridge@samba.org>
  *
- * $Id: packet-http.c 15051 2005-07-25 01:22:03Z ulfl $
+ * $Id: packet-http.c 17769 2006-03-30 10:16:35Z jake $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,6 +47,7 @@
 #include <epan/req_resp_hdrs.h>
 #include "packet-http.h"
 #include <epan/prefs.h>
+#include <epan/expert.h>
 
 typedef enum _http_type {
 	HTTP_REQUEST,
@@ -90,6 +91,7 @@ static int hf_http_server = -1;
 static int hf_http_location = -1;
 static int hf_http_set_cookie = -1;
 static int hf_http_last_modified = -1;
+static int hf_http_x_forwarded_for = -1;
 
 static gint ett_http = -1;
 static gint ett_http_ntlmssp = -1;
@@ -106,14 +108,14 @@ static dissector_handle_t http_handle;
  * desegmentation of HTTP headers
  * (when we are over TCP or another protocol providing the desegmentation API)
  */
-static gboolean http_desegment_headers = FALSE;
+static gboolean http_desegment_headers = TRUE;
 
 /*
  * desegmentation of HTTP bodies
  * (when we are over TCP or another protocol providing the desegmentation API)
  * TODO let the user filter on content-type the bodies he wants desegmented
  */
-static gboolean http_desegment_body = FALSE;
+static gboolean http_desegment_body = TRUE;
 
 /*
  * De-chunking of content-encoding: chunk entity bodies.
@@ -197,6 +199,7 @@ static dissector_handle_t gssapi_handle=NULL;
 static const value_string vals_status_code[] = {
 	{ 100, "Continue" },
 	{ 101, "Switching Protocols" },
+	{ 102, "Processing" },
 	{ 199, "Informational - Others" },
 	
 	{ 200, "OK"},
@@ -206,14 +209,16 @@ static const value_string vals_status_code[] = {
 	{ 204, "No Content"},
 	{ 205, "Reset Content"},
 	{ 206, "Partial Content"},
+	{ 207, "Multi-Status"},
 	{ 299, "Success - Others"},
 	
 	{ 300, "Multiple Choices"},
 	{ 301, "Moved Permanently"},
-	{ 302, "Moved Temporarily"},
+	{ 302, "Found"},
 	{ 303, "See Other"},
 	{ 304, "Not Modified"},
 	{ 305, "Use Proxy"},
+	{ 307, "Temporary Redirect"},
 	{ 399, "Redirection - Others"},
 	
 	{ 400, "Bad Request"},
@@ -230,8 +235,13 @@ static const value_string vals_status_code[] = {
 	{ 411, "Length Required"},
 	{ 412, "Precondition Failed"},
 	{ 413, "Request Entity Too Large"},
-	{ 414, "Request-URI Too Large"},
+	{ 414, "Request-URI Too Long"},
 	{ 415, "Unsupported Media Type"},
+	{ 416, "Requested Range Not Satisfiable"},
+	{ 417, "Expectation Failed"},
+	{ 422, "Unprocessable Entity"},
+	{ 423, "Locked"},
+	{ 424, "Failed Dependency"},
 	{ 499, "Client Error - Others"},
 	
 	{ 500, "Internal Server Error"},
@@ -240,6 +250,7 @@ static const value_string vals_status_code[] = {
 	{ 503, "Service Unavailable"},
 	{ 504, "Gateway Time-out"},
 	{ 505, "HTTP Version not supported"},
+	{ 507, "Insufficient Storage"},
 	{ 599, "Server Error - Others"},
 	
 	{ 0, 	NULL}
@@ -460,7 +471,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
     proto_tree *tree)
 {
 	http_proto_t	proto;
-	char		*proto_tag;
+	const char	*proto_tag;
 	proto_tree	*http_tree = NULL;
 	proto_item	*ti = NULL;
 	const guchar	*line;
@@ -755,6 +766,9 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				    tvb_format_text(tvb, offset,
 				      next_offset - offset));
 			}
+			expert_add_info_format(pinfo, hdr_item, PI_SEQUENCE, PI_CHAT, 
+				"%s",
+				tvb_format_text(tvb, offset, next_offset - offset));
 			if (reqresp_dissector) {
 				if (tree) req_tree = proto_item_add_subtree(hdr_item, ett_http_request);
 				else req_tree = NULL;
@@ -919,7 +933,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		if (headers.transfer_encoding != NULL &&
 		    strcasecmp(headers.transfer_encoding, "identity") != 0) {
 			if (http_dechunk_body &&
-			    (strcasecmp(headers.transfer_encoding, "chunked")
+			    (strncasecmp(headers.transfer_encoding, "chunked", 7)
 			    == 0)) {
 
 				chunks_decoded = chunked_encoding_dissector(
@@ -1143,7 +1157,7 @@ basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
 	tokenlen = get_token_len(line, lineend, &next_token);
 	if (tokenlen == 0)
 		return;
-	stat_info->request_uri = (gchar*) ep_tvb_get_string(tvb, offset, tokenlen);
+	stat_info->request_uri = (gchar*) tvb_get_ephemeral_string(tvb, offset, tokenlen);
 	proto_tree_add_string(tree, hf_http_request_uri, tvb, offset, tokenlen,
 	    stat_info->request_uri);
 	offset += next_token - line;
@@ -1234,7 +1248,7 @@ chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 			break;
 		}
 
-		chunk_string = ep_tvb_get_string(tvb, offset, linelen);
+		chunk_string = tvb_get_ephemeral_string(tvb, offset, linelen);
 
 		if (chunk_string == NULL) {
 			/* Can't get the chunk size line */
@@ -1326,6 +1340,13 @@ chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 
 
 			if (chunk_size > 0) {
+				/*
+				 * XXX - just use "proto_tree_add_text()"?
+				 * This means that, in Tethereal, you get
+				 * the entire chunk dumped out in hex,
+				 * in addition to whatever dissection is
+				 * done on the reassembled data.
+				 */
 				call_dissector(data_handle, data_tvb, pinfo,
 				    chunk_subtree);
 			}
@@ -1555,9 +1576,9 @@ is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
  * Process headers.
  */
 typedef struct {
-	char	*name;
-	gint	*hf;
-	int	special;
+	const char	*name;
+	gint		*hf;
+	int		special;
 } header_info;
 
 #define HDR_NO_SPECIAL		0
@@ -1592,6 +1613,7 @@ static const header_info headers[] = {
 	{ "Location", &hf_http_location, HDR_NO_SPECIAL },
 	{ "Set-Cookie", &hf_http_set_cookie, HDR_NO_SPECIAL },
 	{ "Last-Modified", &hf_http_last_modified, HDR_NO_SPECIAL },
+	{ "X-Forwarded-For", &hf_http_x_forwarded_for, HDR_NO_SPECIAL },
 };
 
 static void
@@ -1859,7 +1881,7 @@ dissect_http_udp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	dissect_http_message(tvb, 0, pinfo, tree);
 }
 
-void reinit_http(void) {
+static void reinit_http(void) {
 	if ( http_alternate_tcp_port != alternate_tcp_port ) {
 		
 		if (alternate_tcp_port)
@@ -1995,6 +2017,10 @@ proto_register_http(void)
 	      { "Last-Modified",	"http.last_modified",
 	        FT_STRING, BASE_NONE, NULL, 0x0,
 	        "HTTP Last Modified", HFILL }},
+	    { &hf_http_x_forwarded_for,
+	      { "X-Forwarded-For",	"http.x_forwarded_for",
+	        FT_STRING, BASE_NONE, NULL, 0x0,
+	        "HTTP X-Forwarded-For", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_http,
