@@ -2,7 +2,7 @@
  * Routines for IAPP dissection
  * Copyright 2002, Alfred Arnold <aarnold@elsa.de>
  *
- * $Id: packet-iapp.c 14786 2005-06-26 19:56:52Z guy $
+ * $Id: packet-iapp.c 17545 2006-03-09 11:34:08Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include "oui.h"
+#include <epan/oui.h>
 
 /* Initialize the protocol and registered fields */
 static int proto_iapp = -1;
@@ -184,27 +184,22 @@ static value_string iapp_auth_type_vals[] = {
 static void dissect_caps(proto_item *pitem, tvbuff_t *tvb, int offset)
 {
 	proto_tree *captree;
-	int bit, val, z, thisbit;
+	int bit, val, thisbit;
 	const gchar *strval;
-	gchar bitval[20];
+	gchar bitval[4+1+4+1];	/* "xxxx xxxx\0" */
 
 	captree = proto_item_add_subtree(pitem, ett_iapp_cap);
 	val = tvb_get_guint8(tvb, offset + 3);
 
-	bitval[8] = '\0';
 	for (bit = 7; bit >= 0; bit--)
 	{
-		strval = match_strval(1 << bit, iapp_cap_vals);
+		thisbit = 1 << bit;
+		strval = match_strval(thisbit, iapp_cap_vals);
 		if (strval)
 		{
-			thisbit = (val & (1 << bit)) ? 1 : 0;
-			for (z = 0; z < 7; z++)
-				if (z == 7 - bit)
-					bitval[z] = thisbit + '0';
-				else
-					bitval[z] = '.';
+			other_decode_bitfield_value(bitval, val, thisbit, 8);
 			proto_tree_add_text(captree, tvb, offset + 3, 1, "%s %s: %s",
-				bitval, strval, thisbit ? "Yes" : "No");
+				bitval,	strval, val & thisbit ? "Yes" : "No");
 		}
 	}
 }
@@ -322,6 +317,7 @@ append_pduval_str(proto_item *ti, int type, int len, tvbuff_t *tvb, int offset,
 						if (!first)
 							proto_item_append_text(ti, " ");
 						proto_item_append_text(ti, "%s", strval);
+						first=0;
 					}
 				}
 			proto_item_append_text(ti, ")");
