@@ -1,7 +1,7 @@
 /* packet-icmpv6.c
  * Routines for ICMPv6 packet disassembly
  *
- * $Id: packet-icmpv6.c 12458 2004-10-31 23:36:21Z guy $
+ * $Id: packet-icmpv6.c 17762 2006-03-29 22:04:37Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -11,6 +11,8 @@
  *
  * HMIPv6 support added by Martti Kuparinen <martti.kuparinen@iki.fi>
  *
+ * FMIPv6 support added by Martin Andre <andre@clarinet.u-strasbg.fr>
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -41,16 +43,13 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include "packet-ipv6.h"
 #include "packet-dns.h"
 #include <epan/in_cksum.h>
 #include <epan/addr_resolv.h>
 #include <epan/ipproto.h>
+#include <epan/emem.h>
 
 #ifndef offsetof
 #define	offsetof(type, member)	((size_t)(&((type *)0)->member))
@@ -68,6 +67,10 @@
  * and
  *
  *	draft-ietf-mobileip-hmipv6-05.txt
+ * 
+ * and
+ * 
+ * 	rfc4068.txt
  */
 
 static int proto_icmpv6 = -1;
@@ -113,6 +116,60 @@ static const value_string names_router_pref[] = {
         { ND_RA_FLAG_RTPREF_MEDIUM,	"Medium" },
         { ND_RA_FLAG_RTPREF_LOW,	"Low" },
         { ND_RA_FLAG_RTPREF_RSV,	"Reserved" },
+	{ 0, NULL}
+};
+
+static const value_string names_fmip6_prrtadv_code[] = {
+	{ FMIP6_PRRTADV_MNTUP,	"MN should use AP-ID, AR-info tuple" },
+	{ FMIP6_PRRTADV_NI_HOVER,	"Network Initiated Handover trigger" },
+	{ FMIP6_PRRTADV_NORTINFO,	"No new router information" },
+	{ FMIP6_PRRTADV_LIMRTINFO,	"Limited new router information" },
+	{ FMIP6_PRRTADV_UNSOL,	"Unsolicited" },
+	{ 0,			NULL }
+};
+
+static const value_string names_fmip6_hi_code[] = {
+	{ FMIP6_HI_PCOA,	"FBU sent from previous link" },
+	{ FMIP6_HI_NOTPCOA,	"FBU sent from new link" },
+	{ 0,			NULL }
+};
+
+static const value_string names_fmip6_hack_code[] = {
+	{ FMIP6_HACK_VALID,	"Handover Accepted, NCoA valid" },
+	{ FMIP6_HACK_INVALID,	"Handover Accepted, NCoA not valid" },
+	{ FMIP6_HACK_INUSE,	"Handover Accepted, NCoA in use" },
+	{ FMIP6_HACK_ASSIGNED,	"Handover Accepted, NCoA assigned" },
+	{ FMIP6_HACK_NOTASSIGNED,	"Handover Accepted, NCoA not assigned" },
+	{ FMIP6_HACK_NOTACCEPTED,	"Handover Not Accepted, reason unspecified" },
+	{ FMIP6_HACK_PROHIBITED,	"Administratively prohibited" },
+	{ FMIP6_HACK_INSUFFICIENT,	"Insufficient resources" },
+	{ 0,			NULL }
+};
+
+static const value_string names_fmip6_ip_addr_opt_code[] = {
+	{ FMIP6_OPT_IP_ADDRESS_OPTCODE_PCOA,	"Old Care-of Address" },
+	{ FMIP6_OPT_IP_ADDRESS_OPTCODE_NCOA,	"New Care-of Address" },
+	{ FMIP6_OPT_IP_ADDRESS_OPTCODE_NAR,	"NAR's IP address" },
+	{ 0,			NULL }
+};
+
+static const value_string names_fmip6_lla_opt_code[] = {
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_WILDCARD,	"Wildcard" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_NAP,	"Link-layer Address of the New Access Point" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_MN,	"Link-layer Address of the MN" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_NAR,	"Link-layer Address of the NAR" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_SRC,	"Link-layer Address of the source" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_CURROUTER,	"The AP belongs to the current interface of the router" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_NOPREFIX,	"No prefix information available" },
+	{ FMIP6_OPT_LINK_LAYER_ADDRESS_OPTCODE_NOSUPPORT,	"No fast handovers support available" },
+	{ 0,			NULL }
+};
+
+static const value_string names_fmip6_naack_opt_status[] = {
+	{ FMIP6_OPT_NEIGHBOR_ADV_ACK_STATUS_INVALID,	"New CoA is invalid" },
+	{ FMIP6_OPT_NEIGHBOR_ADV_ACK_STATUS_INVALID_NEW,	"New CoA is invalid, use the supplied CoA" },
+	{ FMIP6_OPT_NEIGHBOR_ADV_ACK_STATUS_UNRECOGNIZED,	"LLA is unrecognized" },
+	{ 0,			NULL }
 };
 
 static void
@@ -142,13 +199,13 @@ dissect_contained_icmpv6(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 }
 
 static void
-dissect_icmpv6opt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
+dissect_icmpv6ndopt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
     proto_tree *icmp6opt_tree, *field_tree;
     proto_item *ti, *tf;
     struct nd_opt_hdr nd_opt_hdr, *opt;
     int len;
-    char *typename;
+    const char *typename;
     static const guint8 nd_redirect_reserved[6] = {0, 0, 0, 0, 0, 0};
     guint8 nd_redirect_res[6];
 
@@ -200,6 +257,9 @@ again:
     case ND_OPT_MAP:
 	typename = "HMIPv6 MAP option";
 	break;
+    case FMIP6_OPT_NEIGHBOR_ADV_ACK:
+	typename = "Neighbor Advertisement Acknowledgment";
+	break;
     default:
 	typename = "Unknown";
 	break;
@@ -217,23 +277,13 @@ again:
     case ND_OPT_SOURCE_LINKADDR:
     case ND_OPT_TARGET_LINKADDR:
       {
-	int len, i, p;
-	const guint8 *a;
-	char *t;
+	int len, p;
 
 	p = offset + sizeof(*opt);
 	len = (opt->nd_opt_len << 3) - sizeof(*opt);
-	a = tvb_get_ptr(tvb, p, len);
-	t = g_malloc(len * 3);
-	memset(t, 0, len * 3);
-	for (i = 0; i < len; i++) {
-	    if (i)
-		t[i * 3 - 1] = ':';
-	    sprintf(&t[i * 3], "%02x", a[i]);
-	}
 	proto_tree_add_text(icmp6opt_tree, tvb,
-	    offset + sizeof(*opt), len, "Link-layer address: %s", t);
-	g_free(t);
+	    offset + sizeof(*opt), len, "Link-layer address: %s",
+	    bytestring_to_str(tvb_get_ptr(tvb, p, len), len, ':'));
 	break;
       }
     case ND_OPT_PREFIX_INFORMATION:
@@ -426,12 +476,161 @@ again:
 	}
 	break;
       }
+
+    case FMIP6_OPT_NEIGHBOR_ADV_ACK:
+      {
+	struct fmip6_opt_neighbor_advertisement_ack fmip6_opt_neighbor_advertisement_ack, *opt_naack;
+	struct e_in6_addr in6;
+
+	opt_naack = &fmip6_opt_neighbor_advertisement_ack;
+	tvb_memcpy(tvb, (guint8 *)opt_naack, offset, sizeof *opt_naack);
+
+	proto_tree_add_text(icmp6opt_tree, tvb,
+			offset + offsetof(struct fmip6_opt_neighbor_advertisement_ack, fmip6_opt_optcode),
+			1, "Option-Code: %u",
+			opt_naack->fmip6_opt_optcode);
+
+	proto_tree_add_text(icmp6opt_tree, tvb,
+			offset + offsetof(struct fmip6_opt_neighbor_advertisement_ack, fmip6_opt_status),
+			1, "Status: %s",
+			val_to_str(opt_naack->fmip6_opt_status, names_fmip6_naack_opt_status, "Unknown"));
+
+	if (opt_naack->fmip6_opt_len == 3) 
+	{
+		tvb_memcpy(tvb, (guint8 *)&in6, offset + sizeof(*opt_naack), 16);
+		proto_tree_add_text(icmp6opt_tree, tvb,
+			offset + sizeof(*opt_naack), 
+			16, "New Care-of Address: %s",
+			ip6_to_str(&in6));
+	}
+
+	break;
+      }
     }
 
     offset += (opt->nd_opt_len << 3);
     goto again;
 }
 
+static void
+dissect_icmpv6fmip6opt(tvbuff_t *tvb, int offset, proto_tree *tree)
+{
+	proto_tree *icmp6opt_tree;
+	proto_item *ti;
+	struct fmip6_opt_hdr fmip6_opt_hdr, *opt;
+	int len;
+	char *typename;
+
+	if (!tree)
+		return;
+
+again:
+	if ((int)tvb_reported_length(tvb) <= offset)
+		return; /* No more options left */
+
+	opt = &fmip6_opt_hdr;
+	tvb_memcpy(tvb, (guint8 *)opt, offset, sizeof *opt);
+	len = opt->fmip6_opt_len << 3;
+
+	/* !!! specify length */
+	ti = proto_tree_add_text(tree, tvb, offset, len, "ICMPv6 options");
+	icmp6opt_tree = proto_item_add_subtree(ti, ett_icmpv6opt);
+
+	if (len == 0) {
+		proto_tree_add_text(icmp6opt_tree, tvb,
+				offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_len), 1,
+				"Invalid option length: %u",
+				opt->fmip6_opt_len);
+		return; /* we must not try to decode this */
+	}
+
+	switch (opt->fmip6_opt_type) {
+		case FMIP6_OPT_IP_ADDRESS:
+			typename = "IP Address";
+			break;
+		case FMIP6_OPT_NEW_ROUTER_PREFIX_INFO:
+			typename = "New Router Prefix Information";
+			break;
+		case FMIP6_OPT_LINK_LAYER_ADDRESS:
+			typename = "Link-layer Address";
+			break;
+		default:
+			typename = "Unknown";
+			break;
+	}
+
+	proto_tree_add_text(icmp6opt_tree, tvb,
+			offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_type), 1,
+			"Type: %u (%s)", opt->fmip6_opt_type, typename);
+	proto_tree_add_text(icmp6opt_tree, tvb,
+			offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_len), 1,
+			"Length: %u bytes (%u)", opt->fmip6_opt_len << 3, opt->fmip6_opt_len);
+
+	/* decode... */
+	switch (opt->fmip6_opt_type) {
+		case FMIP6_OPT_IP_ADDRESS:
+			{
+				struct fmip6_opt_ip_address fmip6_opt_ip_address, *opt_ip;
+
+				opt_ip = &fmip6_opt_ip_address;
+				tvb_memcpy(tvb, (guint8 *)opt_ip, offset, sizeof *opt_ip);
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_optcode), 1, "Option-Code: %s",
+						val_to_str(opt->fmip6_opt_optcode, names_fmip6_ip_addr_opt_code, "Unknown"));
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_ip_address, fmip6_opt_prefix_len),
+						1, "Prefix length: %u", opt_ip->fmip6_opt_prefix_len);
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_ip_address, fmip6_opt_ip6_address), 
+						16, "IPv6 Address: %s",
+						ip6_to_str(&opt_ip->fmip6_opt_ip6_address));
+				break;
+			}
+		case FMIP6_OPT_NEW_ROUTER_PREFIX_INFO:
+			{
+				struct fmip6_opt_new_router_prefix_info fmip6_opt_new_router_prefix_info, *opt_nr;
+
+				opt_nr = &fmip6_opt_new_router_prefix_info;
+				tvb_memcpy(tvb, (guint8 *)opt_nr, offset, sizeof *opt_nr);
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_optcode), 1, "Option-Code: %u",
+						opt->fmip6_opt_optcode);
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_new_router_prefix_info, fmip6_opt_prefix_len),
+						1, "Prefix length: %u", opt_nr->fmip6_opt_prefix_len);
+
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_new_router_prefix_info, fmip6_opt_prefix), 
+						16, "Prefix: %s",
+						ip6_to_str(&opt_nr->fmip6_opt_prefix));
+				break;
+			}
+			break;
+		case FMIP6_OPT_LINK_LAYER_ADDRESS:
+			{
+				int len, p;
+
+				p = offset + sizeof(*opt);
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + offsetof(struct fmip6_opt_hdr, fmip6_opt_optcode), 1, "Option-Code: %s",
+						val_to_str(opt->fmip6_opt_optcode, names_fmip6_lla_opt_code, "Unknown"));
+				len = (opt->fmip6_opt_len << 3) - sizeof(*opt);
+				proto_tree_add_text(icmp6opt_tree, tvb,
+						offset + sizeof(*opt), len, "Link-layer address: %s",
+						bytestring_to_str(tvb_get_ptr(tvb, p, len), len, ':'));
+				break;
+			}
+	}
+
+	offset += (opt->fmip6_opt_len << 3);
+	goto again;
+}
+
 /*
  * draft-ietf-ipngwg-icmp-name-lookups-07.txt
  * Note that the packet format was changed several times in the past.
@@ -478,13 +677,12 @@ bitrange0(guint32 v, int s, char *buf, int buflen)
 				break;
 		}
 		if (i == 1)
-			l = snprintf(p, ep - p, ",%d", s + off);
+			l = g_snprintf(p, ep - p, ",%d", s + off);
 		else {
-			l = snprintf(p, ep - p, ",%d-%d", s + off,
+			l = g_snprintf(p, ep - p, ",%d-%d", s + off,
 			    s + off + i - 1);
 		}
-		if (l == -1 || l > ep - p) {
-			buf[0] = '\0';
+		if (l == -1 || l >= ep - p) {
 			return NULL;
 		}
 		v >>= i; off += i;
@@ -518,14 +716,14 @@ static void
 dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
     proto_tree *field_tree;
-	proto_item *tf;
+    proto_item *tf;
     struct icmp6_nodeinfo icmp6_nodeinfo, *ni;
     int off;
     unsigned int j;
     int i, n, l, p;
     guint16 flags;
-    char dname[MAXDNAME];
-    guint8 ipaddr[4];
+    char *dname;
+    guint32 ipaddr;
 
     ni = &icmp6_nodeinfo;
     tvb_memcpy(tvb, (guint8 *)ni, offset, sizeof *ni);
@@ -625,7 +823,7 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	    p = offset + sizeof *ni;
 	    for (i = 0; i < n; i++) {
 		struct e_in6_addr e_in6_addr;
-		tvb_memcpy(tvb, (guint8 *)&e_in6_addr, p, sizeof e_in6_addr);
+		tvb_get_ipv6(tvb, p, &e_in6_addr);
 		proto_tree_add_text(field_tree, tvb,
 		    p, sizeof(struct e_in6_addr),
 		    "%s", ip6_to_str(&e_in6_addr));
@@ -635,7 +833,7 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	    break;
 	case ICMP6_NI_SUBJ_FQDN:
 	    l = get_dns_name(tvb, offset + sizeof(*ni),
-	    	offset + sizeof(*ni), dname, sizeof(dname));
+	    	offset + sizeof(*ni), &dname);
 	    if (tvb_bytes_exist(tvb, offset + sizeof(*ni) + l, 1) &&
 	        tvb_get_guint8(tvb, offset + sizeof(*ni) + l) == 0) {
 		l++;
@@ -655,9 +853,9 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	    field_tree = proto_item_add_subtree(tf, ett_nodeinfo_subject4);
 	    p = offset + sizeof *ni;
 	    for (i = 0; i < n; i++) {
-		tvb_memcpy(tvb, ipaddr, p, 4);
+		ipaddr = tvb_get_ipv4(tvb, p);
 		proto_tree_add_text(field_tree, tvb,
-		    p, sizeof(guint32), "%s", ip_to_str(ipaddr));
+		    p, sizeof(guint32), "%s", ip_to_str((guint8 *)&ipaddr));
 		p += sizeof(guint32);
 	    }
 	    off = tvb_length_remaining(tvb, offset);
@@ -715,7 +913,7 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	    while (j < tvb_reported_length(tvb)) {
 		l = get_dns_name(tvb, j,
 		   offset + sizeof (*ni) + sizeof(guint32),
-		   dname,sizeof(dname));
+		   &dname);
 		if (tvb_bytes_exist(tvb, j + l, 1) &&
 		    tvb_get_guint8(tvb, j + l) == 0) {
 		    l++;
@@ -740,7 +938,7 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 		struct e_in6_addr e_in6_addr;
 		gint32 ttl;
 		ttl = (gint32)tvb_get_ntohl(tvb, p);
-		tvb_memcpy(tvb, (guint8 *)&e_in6_addr, p + sizeof ttl, sizeof e_in6_addr);
+		tvb_get_ipv6(tvb, p + sizeof ttl, &e_in6_addr);
 		proto_tree_add_text(field_tree, tvb,
 		    p, sizeof(struct e_in6_addr) + sizeof(gint32),
 		    "%s (TTL %d)", ip6_to_str(&e_in6_addr), ttl);
@@ -756,9 +954,10 @@ dissect_nodeinfo(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 	    field_tree = proto_item_add_subtree(tf, ett_nodeinfo_node4);
 	    p = offset + sizeof *ni;
 	    for (i = 0; i < n; i++) {
-		tvb_memcpy(tvb, ipaddr, sizeof(gint32) + p, 4);
+		ipaddr = tvb_get_ipv4(tvb, sizeof(gint32) + p);
 		proto_tree_add_text(field_tree, tvb,
-		    p, sizeof(guint32), "%s (TTL %d)", ip_to_str(ipaddr), tvb_get_ntohl(tvb, p));
+		    p, sizeof(guint32), "%s (TTL %d)",
+		    ip_to_str((guint8 *)&ipaddr), tvb_get_ntohl(tvb, p));
 		p += sizeof(gint32) + sizeof(guint32);
 	    }
 	    off = tvb_length_remaining(tvb, offset);
@@ -775,7 +974,7 @@ static void
 dissect_rrenum(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree)
 {
     proto_tree *field_tree, *opt_tree;
-	proto_item *tf;
+    proto_item *tf;
     struct icmp6_router_renum icmp6_router_renum, *rr;
     struct rr_pco_match rr_pco_match, *match;
     struct rr_pco_use rr_pco_use, *use;
@@ -967,7 +1166,7 @@ dissect_mldrv2( tvbuff_t *tvb, guint32 offset, guint16 count, proto_tree *tree )
     localOffset += 2;
     recordSize = 4 + 16 + (16 * sourceNb) + (auxDataLen * 4);
 
-    tvb_memcpy(tvb, (guint8 *)&addr, localOffset, sizeof(addr) );
+    tvb_get_ipv6(tvb, localOffset, &addr);
     tf = proto_tree_add_text( tree, tvb, offset, recordSize, 
 #ifdef INET6
 			      "%s: %s (%s)", val_to_str(recordType, mldrv2ModesNames,"Unknown mode"),
@@ -986,7 +1185,7 @@ dissect_mldrv2( tvbuff_t *tvb, guint32 offset, guint16 count, proto_tree *tree )
     localOffset += 16;
 
     for( ; sourceNb; sourceNb--, localOffset += 16 ) {
-      tvb_memcpy(tvb, (guint8 *)&addr, localOffset, sizeof(addr) );
+      tvb_get_ipv6(tvb, localOffset, &addr);
       proto_tree_add_text( sub_tree, tvb, localOffset, 16, 
 #ifdef INET6
 			   "Source Address: %s (%s)", get_hostname6(&addr), ip6_to_str(&addr) );
@@ -1003,7 +1202,7 @@ dissect_mldqv2(tvbuff_t *tvb, guint32 offset, guint16 count, proto_tree *tree)
   struct e_in6_addr addr;
 
   for ( ; count; count--, offset += 16) {
-      tvb_memcpy(tvb, (guint8 *)&addr, offset, sizeof(addr));
+      tvb_get_ipv6(tvb, offset, &addr);
       proto_tree_add_text(tree, tvb, offset, 16, 
 			  "Source Address: %s (%s)", get_hostname6(&addr), ip6_to_str(&addr));
   }
@@ -1016,8 +1215,8 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     proto_item *ti, *tf = NULL;
     struct icmp6_hdr icmp6_hdr, *dp;
     struct icmp6_nodeinfo *ni = NULL;
-    char *codename, *typename;
-    char *colcodename, *coltypename;
+    const char *codename, *typename;
+    const char *colcodename, *coltypename;
     int len;
     guint length, reported_length;
     vec_t cksum_vec[4];
@@ -1208,18 +1407,43 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	codename = "Should always be zero";
 	colcodename = NULL;
 	break;
+		case ICMP6_EXPERIMENTAL_MOBILITY:
+			typename = coltypename ="Experimental Mobility";
+			switch (dp->icmp6_data8[0]) {
+				case FMIP6_SUBTYPE_RTSOLPR:
+					typename = coltypename ="RtSolPr (ICMPv6 Experimental Mobility)";
+					codename = "Should always be zero";
+					colcodename = NULL;
+					break;
+				case FMIP6_SUBTYPE_PRRTADV:
+					typename = coltypename ="PrRtAdv (ICMPv6 Experimental Mobility)";
+					codename = val_to_str(dp->icmp6_code, names_fmip6_prrtadv_code, "Unknown");
+					colcodename = NULL;
+					break;
+				case FMIP6_SUBTYPE_HI:
+					typename = coltypename ="HI (ICMPv6 Experimental Mobility)";
+					codename = val_to_str(dp->icmp6_code, names_fmip6_hi_code, "Unknown");
+					colcodename = NULL;
+					break;
+				case FMIP6_SUBTYPE_HACK:
+					typename = coltypename ="HAck (ICMPv6 Experimental Mobility)";
+					codename = val_to_str(dp->icmp6_code, names_fmip6_hack_code, "Unknown");
+					colcodename = NULL;
+					break;
+			}
+			break;
     }
 
     if (check_col(pinfo->cinfo, COL_INFO)) {
 	char typebuf[256], codebuf[256];
 
 	if (coltypename && strcmp(coltypename, "Unknown") == 0) {
-	    snprintf(typebuf, sizeof(typebuf), "Unknown (0x%02x)",
+	    g_snprintf(typebuf, sizeof(typebuf), "Unknown (0x%02x)",
 		dp->icmp6_type);
 	    coltypename = typebuf;
 	}
 	if (colcodename && strcmp(colcodename, "Unknown") == 0) {
-	    snprintf(codebuf, sizeof(codebuf), "Unknown (0x%02x)",
+	    g_snprintf(codebuf, sizeof(codebuf), "Unknown (0x%02x)",
 		dp->icmp6_code);
 	    colcodename = codebuf;
 	}
@@ -1274,7 +1498,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			tvb,
 			offset + offsetof(struct icmp6_hdr, icmp6_cksum), 2,
 			cksum,
-			"Checksum: 0x%04x (correct)", cksum);
+			"Checksum: 0x%04x [correct]", cksum);
 	    } else {
 		proto_tree_add_boolean_hidden(icmp6_tree, hf_icmpv6_checksum_bad,
 			tvb,
@@ -1284,7 +1508,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			tvb,
 			offset + offsetof(struct icmp6_hdr, icmp6_cksum), 2,
 			cksum,
-			"Checksum: 0x%04x (incorrect, should be 0x%04x)",
+			"Checksum: 0x%04x [incorrect, should be 0x%04x]",
 			cksum, in_cksum_shouldbe(cksum, computed_cksum));
 	    }
 	} else {
@@ -1386,7 +1610,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		ip6_to_str((const struct e_in6_addr *)(tvb_get_ptr(tvb, offset + sizeof *dp, sizeof (struct e_in6_addr)))));
 	    break;
 	case ND_ROUTER_SOLICIT:
-	    dissect_icmpv6opt(tvb, offset + sizeof(*dp), pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + sizeof(*dp), pinfo, icmp6_tree);
 	    break;
 	case ICMP6_MLDV2_REPORT: {
 	  guint16 nbRecords;
@@ -1434,7 +1658,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    proto_tree_add_text(icmp6_tree, tvb,
 		offset + offsetof(struct nd_router_advert, nd_ra_retransmit), 4,
 		"Retrans time: %u", pntohl(&ra->nd_ra_retransmit));
-	    dissect_icmpv6opt(tvb, offset + sizeof(struct nd_router_advert), pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + sizeof(struct nd_router_advert), pinfo, icmp6_tree);
 	    break;
 	  }
 	case ND_NEIGHBOR_SOLICIT:
@@ -1453,7 +1677,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 #endif
 			ip6_to_str(&ns->nd_ns_target));
 
-	    dissect_icmpv6opt(tvb, offset + sizeof(*ns), pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + sizeof(*ns), pinfo, icmp6_tree);
 	    break;
 	  }
 	case ND_NEIGHBOR_ADVERT:
@@ -1488,7 +1712,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 #endif
 			ip6_to_str(&na_target));
 
-	    dissect_icmpv6opt(tvb, offset + sizeof(struct nd_neighbor_advert), pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + sizeof(struct nd_neighbor_advert), pinfo, icmp6_tree);
 	    break;
 	  }
 	case ND_REDIRECT:
@@ -1517,7 +1741,7 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 #endif
 			ip6_to_str(&rd->nd_rd_dst));
 
-	    dissect_icmpv6opt(tvb, offset + sizeof(*rd), pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + sizeof(*rd), pinfo, icmp6_tree);
 	    break;
 	  }
 	case ICMP6_ROUTER_RENUMBERING:
@@ -1594,7 +1818,76 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset + 7, 1, "Reserved: %d",
 		tvb_get_guint8(tvb, offset + 7));
 	    /* Show all options */
-	    dissect_icmpv6opt(tvb, offset + 8, pinfo, icmp6_tree);
+	    dissect_icmpv6ndopt(tvb, offset + 8, pinfo, icmp6_tree);
+	    break;
+			case ICMP6_EXPERIMENTAL_MOBILITY:
+		switch (dp->icmp6_data8[0]) {
+			case FMIP6_SUBTYPE_RTSOLPR:
+				{
+					struct fmip6_rtsolpr *rtsolpr;
+					rtsolpr = (struct fmip6_rtsolpr*) dp;
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 4, 1,
+							"Subtype: Router Solicitation for Proxy Advertisement");
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 6, 2,
+							"Identifier: %d", pntohs(&rtsolpr->fmip6_rtsolpr_id));
+					dissect_icmpv6fmip6opt(tvb, offset + sizeof(*dp), icmp6_tree);
+					break;
+				}
+			case FMIP6_SUBTYPE_PRRTADV:
+				{
+					struct fmip6_prrtadv *prrtadv;
+					prrtadv = (struct fmip6_prrtadv*) dp;
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 4, 1,
+							"Subtype: Proxy Router Advertisement");
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 6, 2,
+							"Identifier: %d", pntohs(&prrtadv->fmip6_prrtadv_id));
+					dissect_icmpv6fmip6opt(tvb, offset + sizeof(*dp), icmp6_tree);
+					break;
+				}
+			case FMIP6_SUBTYPE_HI:
+				{
+					struct fmip6_hi *hi;
+					int flagoff;
+					guint8 hi_flags;
+					hi = (struct fmip6_hi*) dp;
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 4, 1,
+							"Subtype: Handover Initiate");
+
+					flagoff = offset + offsetof(struct fmip6_hi, fmip6_hi_flags_reserved);
+					hi_flags = tvb_get_guint8(tvb, flagoff);
+					tf = proto_tree_add_text(icmp6_tree, tvb, flagoff, 1, "Flags: 0x%02x", hi_flags);
+					field_tree = proto_item_add_subtree(tf, ett_icmpv6flag);
+					proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+							decode_boolean_bitfield(hi_flags,
+								FMIP_HI_FLAG_ASSIGNED, 8, "Assigned", "Not assigned"));
+					proto_tree_add_text(field_tree, tvb, flagoff, 1, "%s",
+							decode_boolean_bitfield(hi_flags,
+								FMIP_HI_FLAG_BUFFER, 8, "Buffered", "Not buffered"));
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 6, 2,
+							"Identifier: %d", pntohs(&hi->fmip6_hi_id));
+					dissect_icmpv6fmip6opt(tvb, offset + sizeof(*dp), icmp6_tree);
+					break;
+				}
+			case FMIP6_SUBTYPE_HACK:
+				{
+					struct fmip6_hack *hack;
+					hack = (struct fmip6_hack*) dp;
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 4, 1,
+							"Subtype: Handover Acknowledge");
+					proto_tree_add_text(icmp6_tree, tvb,
+							offset + 6, 2,
+							"Identifier: %d", pntohs(&hack->fmip6_hack_id));
+					dissect_icmpv6fmip6opt(tvb, offset + sizeof(*dp), icmp6_tree);
+					break;
+				}
+		}
 	    break;
 	default:
 	    next_tvb = tvb_new_subset(tvb, offset + sizeof(*dp), -1, -1);
@@ -1652,7 +1945,6 @@ proto_reg_handoff_icmpv6(void)
 
   icmpv6_handle = create_dissector_handle(dissect_icmpv6, proto_icmpv6);
   dissector_add("ip.proto", IP_PROTO_ICMPV6, icmpv6_handle);
-  dissector_add("wtap_encap", WTAP_ENCAP_RAW_ICMPV6, icmpv6_handle);
 
   /*
    * Get a handle for the IPv6 dissector.
