@@ -1,7 +1,7 @@
 /* packet-igmp.c   2001 Ronnie Sahlberg <See AUTHORS for email>
  * Routines for IGMP packet disassembly
  *
- * $Id: packet-igmp.c 12130 2004-09-29 00:52:45Z guy $
+ * $Id: packet-igmp.c 16956 2006-01-05 13:01:08Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,10 +27,10 @@
 	RFC1054	Version 1
 	RFC1112	Version 1	(same as RFC1054 as far as we are concerned)
 	RFC2236	Version 2
-	draft-ietf-idmr-igmp-v3-07	Version 3
+	RFC3376	Version 3
 
 	Size in bytes for each packet
-	type	RFC988	RFC1054	RFC2236 RFC????  DVMRP  MRDISC  MSNIP  IGAP
+	type	RFC988	RFC1054	RFC2236 RFC3376  DVMRP  MRDISC  MSNIP  IGAP
 	        v0      v1      v2      v3       v1/v3
 	0x01      20
 	0x02      20
@@ -103,13 +103,10 @@
 #include <string.h>
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include <epan/ipproto.h>
 #include <epan/in_cksum.h>
+#include "packet-igmp.h"
 #include "packet-dvmrp.h"
 #include "packet-pim.h"
 #include "packet-mrdisc.h"
@@ -129,7 +126,7 @@ static int hf_access_key = -1;
 static int hf_max_resp = -1;
 static int hf_max_resp_exp = -1;
 static int hf_max_resp_mant = -1;
-static int hf_supress = -1;
+static int hf_suppress = -1;
 static int hf_qrv = -1;
 static int hf_qqic = -1;
 static int hf_num_src = -1;
@@ -246,8 +243,8 @@ static const value_string vs_reply_code[] = {
 };
 
 static const true_false_string tfs_s = {
-	"SUPRESS router side processing",
-	"Do not supress router side processing"
+	"SUPPRESS router side processing",
+	"Do not suppress router side processing"
 };
 
 #define IGMP_V3_MODE_IS_INCLUDE		1
@@ -340,13 +337,13 @@ void igmp_checksum(proto_tree *tree, tvbuff_t *tvb, int hf_index,
 		if (cksum == 0) {
 			proto_tree_add_uint_format(tree, hf_index,
 			    tvb, 2, 2, hdrcksum,
-			    "Header checksum: 0x%04x (correct)", hdrcksum);
+			    "Header checksum: 0x%04x [correct]", hdrcksum);
 		} else {
 			proto_tree_add_boolean_hidden(tree, hf_index_bad,
 			    tvb, 2, 2, TRUE);
 			proto_tree_add_uint_format(tree, hf_index,
 			    tvb, 2, 2, hdrcksum,
-			    "Header checksum: 0x%04x (incorrect, should be 0x%04x)", hdrcksum,in_cksum_shouldbe(hdrcksum,cksum));
+			    "Header checksum: 0x%04x [incorrect, should be 0x%04x]", hdrcksum,in_cksum_shouldbe(hdrcksum,cksum));
 		}
 	} else
 		proto_tree_add_uint(tree, hf_index, tvb, 2, 2, hdrcksum);
@@ -431,7 +428,7 @@ dissect_v3_sqrv_bits(tvbuff_t *tvb, proto_tree *parent_tree, int offset)
 	tree = proto_item_add_subtree(item, ett_sqrv_bits);
 
 	/* S flag */
-	proto_tree_add_boolean(tree, hf_supress, tvb, offset, 1, bits);
+	proto_tree_add_boolean(tree, hf_suppress, tvb, offset, 1, bits);
 	/* QRV */
 	proto_tree_add_uint(tree, hf_qrv, tvb, offset, 1, bits);
 	offset += 1;
@@ -449,10 +446,10 @@ dissect_v3_group_record(tvbuff_t *tvb, proto_tree *parent_tree, int offset)
 	guint16 num;
 	guint32 ip;
 
-	tvb_memcpy(tvb, (guint8 *)&ip, offset+4, 4);
+	ip = tvb_get_ipv4(tvb, offset+4);
 	item = proto_tree_add_text(parent_tree, tvb, offset, -1,
 		"Group Record : %s  %s",
-			ip_to_str((gchar*)&ip),
+			ip_to_str((guint8*)&ip),
 			val_to_str(tvb_get_guint8(tvb, offset), vs_record_type,"")
 		);
 	tree = proto_item_add_subtree(item, ett_group_record);
@@ -649,7 +646,7 @@ dissect_igmp_v0(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int type, i
 static int
 dissect_igmp_mtrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int type, int offset)
 {
-	char *typestr, *blocks = NULL;
+	const char *typestr, *blocks = NULL;
 	char buf[20];
 
 	/* All multicast traceroute packets (Query, Request and
@@ -662,7 +659,7 @@ dissect_igmp_mtrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int typ
 	 */
 	if (type == IGMP_TRACEROUTE_RESPONSE) {
 		int i = (tvb_reported_length_remaining(tvb, offset) - IGMP_TRACEROUTE_HDR_LEN) / IGMP_TRACEROUTE_RSP_LEN;
-		snprintf(buf, sizeof buf, ", %d block%s", i, plurality(i, "", "s"));
+		g_snprintf(buf, sizeof buf, ", %d block%s", i, plurality(i, "", "s"));
 		typestr = "Traceroute Response";
 		blocks = buf;
 	} else if (tvb_reported_length_remaining(tvb, offset) == IGMP_TRACEROUTE_HDR_LEN)
@@ -948,9 +945,9 @@ proto_register_igmp(void)
 			{ "Max Resp Time", "igmp.max_resp", FT_UINT8, BASE_DEC,
 			  NULL, 0, "Max Response Time", HFILL }},
 
-		{ &hf_supress,
+		{ &hf_suppress,
 			{ "S", "igmp.s", FT_BOOLEAN, 8,
-			  TFS(&tfs_s), IGMP_V3_S, "Supress Router Side Processing", HFILL }},
+			  TFS(&tfs_s), IGMP_V3_S, "Suppress Router Side Processing", HFILL }},
 
 		{ &hf_qrv,
 			{ "QRV", "igmp.qrv", FT_UINT8, BASE_DEC,
