@@ -3,7 +3,7 @@
  * Routines for image/gif media dissection
  * Copyright 2003, 2004, Olivier Biot.
  *
- * $Id: packet-image-gif.c 12954 2005-01-05 10:05:03Z guy $
+ * $Id: packet-image-gif.c 16373 2005-11-01 18:18:46Z obiot $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -45,10 +45,6 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 
 /* General-purpose debug logger.
@@ -59,9 +55,8 @@
  */
 #if (defined(DEBUG_image_gif) || defined(DEBUG_image))
 #define DebugLog(x) \
-	printf("%s:%u: ", __FILE__, __LINE__); \
-	printf x; \
-	fflush(stdout)
+	g_print("%s:%u: ", __FILE__, __LINE__); \
+	g_print x
 #else
 #define DebugLog(x) ;
 #endif
@@ -168,7 +163,7 @@ dissect_gif(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 	guint8 color_resolution;
 	guint8 image_bpp;
 	guint tvb_len = tvb_reported_length(tvb);
-	char *str = tvb_get_string(tvb, 0, 6);
+	char *str = tvb_get_ephemeral_string(tvb, 0, 6);
 	guint8 version;
 
 	/* Check whether we're processing a GIF object */
@@ -181,7 +176,6 @@ dissect_gif(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 	} else {
 		/* Not a GIF image! */
 		version = GIF_ERROR;
-		g_free(str);
 		return;
 	}
 	/* Add summary to INFO column if it is enabled */
@@ -196,7 +190,6 @@ dissect_gif(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_gif, tvb, 0, -1, TRUE);
 		proto_item_append_text(ti, ", Version: %s", str);
-		g_free(str);
 		gif_tree = proto_item_add_subtree(ti, ett_gif);
 		/* GIF signature */
 		ti = proto_tree_add_item(gif_tree, hf_version, tvb, 0, 6, TRUE);
@@ -440,8 +433,6 @@ dissect_gif(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 			}
 		} /* while */
 	}
-	else
-		g_free(str);
 }
 
 
@@ -616,7 +607,7 @@ proto_register_gif(void)
 		{ &hf_image,
 			{	"Image",
 				IMG_GIF ".image",
-				FT_NONE, BASE_NONE, NULL, 0x80,
+				FT_NONE, BASE_NONE, NULL, 0x00,
 				"Image.",
 				HFILL
 			}
