@@ -3,7 +3,7 @@
  * Routines for JFIF image/jpeg media dissection
  * Copyright 2004, Olivier Biot.
  *
- * $Id: packet-image-jfif.c 12955 2005-01-05 10:13:24Z guy $
+ * $Id: packet-image-jfif.c 16373 2005-11-01 18:18:46Z obiot $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -48,10 +48,6 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 
 /* General-purpose debug logger.
@@ -62,9 +58,8 @@
  */
 #if (defined(DEBUG_image_jfif) || defined(DEBUG_image))
 #define DebugLog(x) \
-	printf("%s:%u: ", __FILE__, __LINE__); \
-	printf x; \
-	fflush(stdout)
+	g_print("%s:%u: ", __FILE__, __LINE__); \
+	g_print x
 #else
 #define DebugLog(x) ;
 #endif
@@ -514,10 +509,9 @@ process_app0_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 
 	proto_tree_add_item(subtree, hf_len, tvb, 2, 2, FALSE);
 
-	str = tvb_get_stringz(tvb, 4, &str_size);
+	str = tvb_get_ephemeral_stringz(tvb, 4, &str_size);
 	ti = proto_tree_add_item(subtree, hf_identifier, tvb, 4, str_size, FALSE);
 	if (strcmp(str, "JFIF") == 0) {
-		g_free(str);
 		/* Version */
 		ti = proto_tree_add_none_format(subtree, hf_version,
 				tvb, 9, 2, "Version: %u.%u",
@@ -555,7 +549,6 @@ process_app0_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 			}
 		}
 	} else if (strcmp(str, "JFXX") == 0) {
-		g_free(str);
 		proto_tree_add_item(subtree, hf_extension_code,
 				tvb, 9, 1, FALSE);
 		{
@@ -572,7 +565,6 @@ process_app0_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 			}
 		}
 	} else { /* Unknown */
-		g_free(str);
 		proto_item_append_text(ti, " (unknown identifier)");
 		offset = 4 + str_size;
 
@@ -611,7 +603,7 @@ process_app1_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 	proto_tree_add_item(subtree, hf_len, tvb, offset, 2, FALSE);
 	offset += 2;
 
-	str = tvb_get_stringz(tvb, offset, &str_size);
+	str = tvb_get_ephemeral_stringz(tvb, offset, &str_size);
 	ti = proto_tree_add_item(subtree, hf_identifier, tvb, offset, str_size, FALSE);
 	offset += str_size;
 	if (strcmp(str, "Exif") == 0) {
@@ -623,8 +615,6 @@ process_app1_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 		guint32 val_32;
 		guint16 num_fields;
 
-		g_free(str);
-
 		offset++; /* Skip a byte supposed to be 0x00 */
 
 		tiff_start = offset;
@@ -738,7 +728,6 @@ process_app1_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 				break;
 		}
 	} else {
-		g_free(str);
 		proto_tree_add_text(subtree, tvb, offset, -1,
 				"Remaining segment data (%u bytes)", len - 2 - str_size);
 		proto_item_append_text(ti, " (Unknown identifier)");
@@ -770,13 +759,11 @@ process_app2_segment(proto_tree *tree, tvbuff_t *tvb, guint32 len,
 
 	proto_tree_add_item(subtree, hf_len, tvb, 2, 2, FALSE);
 
-	str = tvb_get_stringz(tvb, 4, &str_size);
+	str = tvb_get_ephemeral_stringz(tvb, 4, &str_size);
 	ti = proto_tree_add_item(subtree, hf_identifier, tvb, 4, str_size, FALSE);
 	if (strcmp(str, "FPXR") == 0) {
-		g_free(str);
 		proto_tree_add_text(tree, tvb, 0, -1, "Exif FlashPix APP2 application marker");
 	} else {
-		g_free(str);
 		proto_tree_add_text(subtree, tvb, 4 + str_size, -1,
 				"Remaining segment data (%u bytes)", len - 2 - str_size);
 		proto_item_append_text(ti, " (Unknown identifier)");
@@ -820,7 +807,7 @@ dissect_jfif(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 	 * Process the remaining markers and marker segments
 	 */
 	while (offset < tvb_len) {
-		char *str;
+		const char *str;
 		marker = tvb_get_ntohs(tvb, offset);
 		str = match_strval(marker, vals_marker);
 		if (str) { /* Known marker */
