@@ -6,7 +6,7 @@
  * Portions Copyright (c) 2000-2002 by Gilbert Ramirez.
  * Portions Copyright (c) Novell, Inc. 2002-2003
  *
- * $Id: packet-ipx.c 13243 2005-02-02 20:07:03Z etxrab $
+ * $Id: packet-ipx.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,18 +37,17 @@
 #include <epan/packet.h>
 #include "packet-ipx.h"
 #include <epan/addr_resolv.h>
-#include "etypes.h"
-#include "ppptypes.h"
-#include "llcsaps.h"
-#include "aftypes.h"
-#include "arcnet_pids.h"
+#include <epan/etypes.h>
+#include <epan/ppptypes.h>
+#include <epan/llcsaps.h>
+#include <epan/aftypes.h>
+#include <epan/arcnet_pids.h>
 #include <epan/conversation.h>
 #include <epan/tap.h>
+#include <epan/emem.h>
 
 static int ipx_tap = -1;
 
-#define SPX_PACKET_INIT_COUNT	200
-
 /* The information in this module (IPX, SPX, NCP) comes from:
 	NetWare LAN Analysis, Second Edition
 	Laura A. Chappell and Dan E. Hakes
@@ -434,12 +433,9 @@ typedef struct {
 } spx_rexmit_info;
 
 static GHashTable *spx_hash = NULL;
-static GMemChunk *spx_hash_keys = NULL;
-static GMemChunk *spx_hash_values = NULL;
-static GMemChunk *spx_rexmit_infos = NULL;
 
 /* Hash Functions */
-gint
+static gint
 spx_equal(gconstpointer v, gconstpointer v2)
 {
 	const spx_hash_key	*val1 = (const spx_hash_key*)v;
@@ -453,7 +449,7 @@ spx_equal(gconstpointer v, gconstpointer v2)
 	return 0;
 }
 
-guint
+static guint
 spx_hash_func(gconstpointer v)
 {
 	const spx_hash_key	*spx_key = (const spx_hash_key*)v;
@@ -468,26 +464,8 @@ spx_init_protocol(void)
 
 	if (spx_hash)
 		g_hash_table_destroy(spx_hash);
-	if (spx_hash_keys)
-		g_mem_chunk_destroy(spx_hash_keys);
-	if (spx_hash_values)
-		g_mem_chunk_destroy(spx_hash_values);
-	if (spx_rexmit_infos)
-		g_mem_chunk_destroy(spx_rexmit_infos);
 
 	spx_hash = g_hash_table_new(spx_hash_func, spx_equal);
-	spx_hash_keys = g_mem_chunk_new("spx_hash_keys",
-			sizeof(spx_hash_key),
-			SPX_PACKET_INIT_COUNT * sizeof(spx_hash_key),
-			G_ALLOC_ONLY);
-	spx_hash_values = g_mem_chunk_new("spx_hash_values",
-			sizeof(spx_hash_value),
-			SPX_PACKET_INIT_COUNT * sizeof(spx_hash_value),
-			G_ALLOC_ONLY);
-	spx_rexmit_infos = g_mem_chunk_new("spx_rexmit_infos",
-			sizeof(spx_rexmit_infos),
-			SPX_PACKET_INIT_COUNT * sizeof(spx_rexmit_infos),
-			G_ALLOC_ONLY);
 }
 
 /* After the sequential run, we don't need the spx hash table, or
@@ -502,31 +480,21 @@ spx_postseq_cleanup(void)
 		g_hash_table_destroy(spx_hash);
 		spx_hash = NULL;
 	}
-	if (spx_hash_keys) {
-		g_mem_chunk_destroy(spx_hash_keys);
-		spx_hash_keys = NULL;
-	}
-	if (spx_hash_values) {
-		g_mem_chunk_destroy(spx_hash_values);
-		spx_hash_values = NULL;
-	}
-	/* Don't free the spx_rexmit_infos, as they're
-	 * needed during random-access processing of the proto_tree.*/
 }
 
-spx_hash_value*
+static spx_hash_value*
 spx_hash_insert(conversation_t *conversation, guint32 spx_src, guint16 spx_seq)
 {
 	spx_hash_key		*key;
 	spx_hash_value		*value;
 
 	/* Now remember the packet, so we can find it if we later. */
-	key = g_mem_chunk_alloc(spx_hash_keys);
+	key = se_alloc(sizeof(spx_hash_key));
 	key->conversation = conversation;
 	key->spx_src = spx_src;
 	key->spx_seq = spx_seq;
 
-	value = g_mem_chunk_alloc(spx_hash_values);
+	value = se_alloc(sizeof(spx_hash_value));
 	value->spx_ack = 0;
 	value->spx_all = 0;
 	value->num = 0;
@@ -537,7 +505,7 @@ spx_hash_insert(conversation_t *conversation, guint32 spx_src, guint16 spx_seq)
 }
 
 /* Returns the spx_hash_value*, or NULL if not found. */
-spx_hash_value*
+static spx_hash_value*
 spx_hash_lookup(conversation_t *conversation, guint32 spx_src, guint32 spx_seq)
 {
 	spx_hash_key		key;
@@ -766,7 +734,7 @@ dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				 * Found in the hash table.  Mark this frame as
 				 * a retransmission.
 				 */
-				spx_rexmit_info = g_mem_chunk_alloc(spx_rexmit_infos);
+				spx_rexmit_info = se_alloc(sizeof(spx_rexmit_info));
 				spx_rexmit_info->num = pkt_value->num;
 				p_add_proto_data(pinfo->fd, proto_spx,
 				    spx_rexmit_info);
@@ -897,7 +865,7 @@ dissect_ipxrip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		cursor;
 	int		available_length;
 
-	static char	*rip_type[3] = { "Request", "Response", "Unknown" };
+	static const char	*rip_type[3] = { "Request", "Response", "Unknown" };
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "IPX RIP");
@@ -1238,7 +1206,7 @@ dissect_ipxsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16		server_port;
 	guint16		intermediate_network;
 
-	static char	*sap_type[4] = { "General Query", "General Response",
+	static const char	*sap_type[4] = { "General Query", "General Response",
 		"Nearest Query", "Nearest Response" };
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
