@@ -1,7 +1,7 @@
 /* packet-isis-lsp.c
  * Routines for decoding isis lsp packets and their CLVs
  *
- * $Id: packet-isis-lsp.c 14880 2005-07-08 22:11:13Z ulfl $
+ * $Id: packet-isis-lsp.c 16286 2005-10-22 05:27:48Z guy $
  * Stuart Stanley <stuarts@mxmail.net>
  *
  * Ethereal - Network traffic analyzer
@@ -39,6 +39,7 @@
 #include "packet-isis-clv.h"
 #include "packet-isis-lsp.h"
 #include <epan/addr_resolv.h>
+#include <epan/addr_and_mask.h>
 
 /* lsp packets */
 static int hf_isis_lsp_pdu_length = -1;
@@ -423,7 +424,7 @@ static void
 dissect_lsp_mt_id(tvbuff_t *tvb, proto_tree *tree, int offset)
 {
 	int  mt_block, mt_id;
-	char mt_desc[60];
+	char *mt_desc="";
 
 	/* fetch two bytes */
 	mt_block = tvb_get_ntohs(tvb, offset);
@@ -435,22 +436,22 @@ dissect_lsp_mt_id(tvbuff_t *tvb, proto_tree *tree, int offset)
 	/*mask out the lower 12 bits */
 	switch(mt_id) {
 	case 0:
-		strcpy(mt_desc,"'standard' topology");
+		mt_desc="'standard' topology";
 		break;
 	case 1:
-		strcpy(mt_desc,"IPv4 In-Band Management purposes");
+		mt_desc="IPv4 In-Band Management purposes";
 		break;
 	case 2:
-		strcpy(mt_desc,"IPv6 routing topology");
+		mt_desc="IPv6 routing topology";
 		break;
 	case 3:
-		strcpy(mt_desc,"IPv4 multicast routing topology");
+		mt_desc="IPv4 multicast routing topology";
 		break;
 	case 4:
-		strcpy(mt_desc,"IPv6 multicast routing topology");
+		mt_desc="IPv6 multicast routing topology";
 		break;
 	default:
-		strcpy(mt_desc,((mt_block & 0x0fff) < 3996) ? "Reserved for IETF Consensus" : "Development, Experimental and Proprietary features");
+		mt_desc=((mt_block & 0x0fff) < 3996) ? "Reserved for IETF Consensus" : "Development, Experimental and Proprietary features";
 	}
 
 	proto_tree_add_text ( tree, tvb, offset, 2 ,
@@ -488,7 +489,7 @@ dissect_lsp_mt_id(tvbuff_t *tvb, proto_tree *tree, int offset)
  */
 static void
 dissect_metric(tvbuff_t *tvb, proto_tree *tree,	int offset, guint8 value,
-	char *pstr, int force_supported )
+	const char *pstr, int force_supported )
 {
 	int s;
 
@@ -542,7 +543,7 @@ dissect_lsp_ip_reachability_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
 		 * Gotta build a sub-tree for all our pieces
 		 */
 		if ( tree ) {
-			tvb_memcpy(tvb, (guint8 *)&src, offset+4, 4);
+			src = tvb_get_ipv4(tvb, offset+4);
 			mask = tvb_get_ntohl(tvb, offset+8);
 
 			/* find out if the mask matches one of 33 possible prefix lengths */
@@ -689,26 +690,25 @@ dissect_lsp_ext_ip_reachability_clv(tvbuff_t *tvb, proto_tree *tree,
 	proto_tree *subtree = NULL;
 	proto_tree *subtree2 = NULL;
 	guint8     ctrl_info;
-	guint8     bit_length, byte_length;
+	guint      bit_length;
+	int        byte_length;
 	guint8     prefix [4];
 	guint32    metric;
-	guint8     len,i;
-	guint8     subclvs_len;
-	guint8     clv_code, clv_len;
+	guint      len,i;
+	guint      subclvs_len;
+	guint      clv_code, clv_len;
 
 	if (!tree) return;
 
 	while (length > 0) {
-		memset (prefix, 0, 4);
 		ctrl_info = tvb_get_guint8(tvb, offset+4);
 		bit_length = ctrl_info & 0x3f;
-		byte_length = (bit_length + 7) / 8;
-		if (byte_length > sizeof(prefix)) {
+		byte_length = ipv4_addr_and_mask(tvb, offset+5, prefix, bit_length);
+		if (byte_length == -1) {
 			 isis_dissect_unknown(tvb, tree, offset,
-			 	"IPv4 prefix has an invalid length: %d bytes", byte_length );
+			 	"IPv4 prefix has an invalid length: %d bits", bit_length );
       			return;
     		}
- 		tvb_memcpy (tvb, prefix, offset+5, byte_length);
 		metric = tvb_get_ntohl(tvb, offset);
 		subclvs_len = 0;
 		if ((ctrl_info & 0x40) != 0)
@@ -792,26 +792,25 @@ dissect_lsp_ipv6_reachability_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
 	proto_tree        *subtree = NULL;
 	proto_tree        *subtree2 = NULL;
 	guint8            ctrl_info;
-	guint8            bit_length, byte_length;
+	guint             bit_length;
+	int               byte_length;
 	struct e_in6_addr prefix;
 	guint32           metric;
-	guint8            len,i;
-	guint8            subclvs_len;
-	guint8            clv_code, clv_len;
+	guint             len,i;
+	guint             subclvs_len;
+	guint             clv_code, clv_len;
 
 	if (!tree) return;
 
 	while (length > 0) {
-  	memset (prefix.s6_addr, 0, 16);
 		ctrl_info = tvb_get_guint8(tvb, offset+4);
 		bit_length = tvb_get_guint8(tvb, offset+5);
-		byte_length = (bit_length + 7) / 8;
-    		if (byte_length > sizeof(prefix)) {
+		byte_length = ipv6_addr_and_mask(tvb, offset+6, &prefix, bit_length);
+		if (byte_length == -1) {
 			isis_dissect_unknown(tvb, tree, offset,
-				"IPv6 prefix has an invalid length: %d bytes", byte_length );
+				"IPv6 prefix has an invalid length: %d bits", bit_length );
       			return;
     		}
- 		tvb_memcpy (tvb, prefix.s6_addr, offset+6, byte_length);
 		metric = tvb_get_ntohl(tvb, offset);
 		subclvs_len = 0;
 		if ((ctrl_info & 0x20) != 0)
@@ -1431,9 +1430,9 @@ dissect_lsp_ext_is_reachability_clv(tvbuff_t *tvb, proto_tree *tree,
 {
 	proto_item *ti;
 	proto_tree *ntree = NULL;
-	guint8     subclvs_len;
-	guint8     len, i;
-	guint8     clv_code, clv_len;
+	guint      subclvs_len;
+	guint      len, i;
+	guint      clv_code, clv_len;
 
 	if (!tree) return;
 
