@@ -3,7 +3,7 @@
  * Enterasys Networks Home: http://www.enterasys.com/
  * Copyright 2003, Joshua Craig Douglas <jdouglas@enterasys.com>
  *
- * $Id: packet-ismp.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-ismp.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,13 +34,9 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include <epan/strutil.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 
 
 /* Initialize the protocol and registered fields */
@@ -361,6 +357,7 @@ dissect_ismp_edp(tvbuff_t *tvb, packet_info *pinfo, int offset, proto_tree *ismp
 		offset += 2;
 		if (num_neighbors > 0)
 		{
+			tvb_ensure_bytes_exist(tvb, offset, num_neighbors*10);
 			if (tvb_reported_length_remaining(tvb, offset) >= (num_neighbors *10))
 			{
 				neighbors_ptr = tvb_get_ptr( tvb, offset, (num_neighbors*10) );
@@ -442,6 +439,7 @@ dissect_ismp_edp(tvbuff_t *tvb, packet_info *pinfo, int offset, proto_tree *ismp
 				
 				if (tvb_reported_length_remaining(tvb, offset) >= tuple_length)
 				{
+					tvb_ensure_bytes_exist(tvb, offset, tuple_length);
 					switch (tuple_type)
 					{						
 						case EDP_TUPLE_HOLD:
