@@ -4,7 +4,7 @@
  * 
  * Copyright 2004, Takeshi Nakashima <T.Nakashima@jp.yokogawa.com>
  *
- * $Id: packet-kink.c 15051 2005-07-25 01:22:03Z ulfl $
+ * $Id: packet-kink.c 15158 2005-07-30 16:34:38Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -810,7 +810,7 @@ dissect_payload_kink_error(packet_info *pinfo, tvbuff_t *tvb, int offset, proto_
   guint16 payload_length;
   guint32 error_code;
   int start_payload_offset = 0; /* Keep the begining of the payload offset */
-  char *char_error_code[] = {
+  const char *char_error_code[] = {
     "KINK_OK",
     "KINK_PROTOERR",
     "KINK_INVDOI",
