@@ -2,7 +2,7 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-llc.c 12966 2005-01-06 08:21:25Z guy $
+ * $Id: packet-llc.c 16302 2005-10-25 06:00:53Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,13 +29,13 @@
 
 #include <glib.h>
 #include <epan/packet.h>
-#include "oui.h"
+#include <epan/oui.h>
 #include <epan/xdlc.h>
-#include "etypes.h"
-#include "llcsaps.h"
-#include "bridged_pids.h"
-#include "ppptypes.h"
-#include "arcnet_pids.h"
+#include <epan/etypes.h>
+#include <epan/llcsaps.h>
+#include <epan/bridged_pids.h>
+#include <epan/ppptypes.h>
+#include <epan/arcnet_pids.h>
 #include "packet-fc.h"
 #include "packet-ip.h"
 #include "packet-ipx.h"
@@ -73,8 +73,14 @@ static int hf_llc_type = -1;
 static int hf_llc_oui = -1;
 static int hf_llc_pid = -1;
 
+static int proto_basicxid = -1;
+static int hf_llc_xid_format = -1;
+static int hf_llc_xid_types = -1;
+static int hf_llc_xid_wsize = -1;
+
 static gint ett_llc = -1;
 static gint ett_llc_ctrl = -1;
+static gint ett_llc_basicxid = -1;
 
 static dissector_table_t subdissector_table;
 static dissector_table_t xid_subdissector_table;
@@ -145,6 +151,7 @@ const value_string sap_vals[] = {
 	{ SAP_NESTAR,         "Nestar" },
 	{ SAP_PROWAY_ASLM,    "PROWAY (IEC955) Active Station List Maintenance" },
 	{ SAP_ARP,            "ARP" },	/* XXX - hand to "dissect_arp()"? */
+	{ SAP_HPJD,           "HP JetDirect Printer" },
 	{ SAP_SNAP,           "SNAP" },
 	{ SAP_VINES1,         "Banyan Vines" },
 	{ SAP_VINES2,         "Banyan Vines" },
@@ -162,27 +169,54 @@ const value_string sap_vals[] = {
 /*
  * See
  *
+ * http://standards.ieee.org/regauth/oui/oui.txt
+ *
  * http://www.cisco.com/univercd/cc/td/doc/product/lan/trsrb/vlan.htm
  *
  * for the PIDs for VTP and DRiP that go with an OUI of OUI_CISCO.
  */
 const value_string oui_vals[] = {
 	{ OUI_ENCAP_ETHER, "Encapsulated Ethernet" },
+	{ OUI_APPLE_ATALK, "Apple (AppleTalk)" },
+	{ OUI_ATM_FORUM,   "ATM Forum" },
+	/* RFC 2427, RFC 2684 */
+	{ OUI_BRIDGED,     "Frame Relay or ATM bridged frames" },
+	{ OUI_CABLE_BPDU,  "DOCSIS Spanning Tree" }, /* DOCSIS spanning tree BPDU */
+	{ OUI_SIEMENS,     "Siemens AG" },
+	{ OUI_XEROX,       "Xerox" },
 /*
 http://www.cisco.com/univercd/cc/td/doc/product/software/ios113ed/113ed_cr/ibm_r/brprt1/brsrb.htm
 */
 	{ OUI_CISCO,       "Cisco" },
 	{ OUI_CISCO_90,    "Cisco IOS 9.0 Compatible" },
-	{ OUI_BRIDGED,     "Frame Relay or ATM bridged frames" },
-				/* RFC 2427, RFC 2684 */
-	{ OUI_ATM_FORUM,   "ATM Forum" },
-	{ OUI_CABLE_BPDU,  "DOCSIS Spanning Tree" }, /* DOCSIS spanning tree BPDU */
-	{ OUI_APPLE_ATALK, "Apple (AppleTalk)" },
-	{ OUI_NORTEL,      "Nortel Networks SONMP" },
+	{ OUI_CISCOWL,     "Cisco Wireless (Aironet) L2" },
+	{ OUI_EXTREME,     "Extreme Networks" },
 	{ OUI_HP,          "Hewlett-Packard" },
+	{ OUI_NORTEL,      "Nortel Networks SONMP" },
 	{ 0,               NULL }
 };
 
+const value_string format_vals[] = {
+	{ 0x81,        "LLC basic format" },
+	{ 0,           NULL }
+};
+
+/*
+ * Mask to extract the type from XID frame.
+ */
+#define	TYPES_MASK	0x1F
+
+const value_string type_vals[] = {
+	{ 1,           "Type 1 LLC (Class I LLC)" },
+	{ 2,           "Type 2 LLC" },
+	{ 3,           "Type 1 and Type 2 LLCs (Class II LLC)" },
+	{ 4,           "Type 3 LLC" },
+	{ 5,           "Type 1 and Type 3 LLCs (Class III LLC)" },
+	{ 6,           "Type 2 and Type 3 LLCs" },
+	{ 7,           "Type 1 and Type 2 and Type 3 LLCs (Class IV LLC)" },
+	{ 0,           NULL }
+};
+
 /*
  * Hash table for translating OUIs to a dissector table/field info pair;
  * the dissector table maps PID values to dissectors, and the field
@@ -199,7 +233,7 @@ static GHashTable *oui_info_table = NULL;
  * Add an entry for a new OUI.
  */
 void
-llc_add_oui(guint32 oui, const char *table_name, char *table_ui_name,
+llc_add_oui(guint32 oui, const char *table_name, const char *table_ui_name,
     hf_register_info *hf_item)
 {
 	oui_info_t *new_info;
@@ -226,8 +260,6 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 	int		is_snap;
 	guint16		control;
 	int		llc_header_len;
-	guint32		oui;
-	guint16		etype;
 
 	if (!BYTES_ARE_IN_FRAME(offset, len, 2)) {
 		ld->other++;
@@ -244,74 +276,84 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 	 */
 	control = get_xdlc_control(pd, offset+2, pd[offset+1] & SSAP_CR_BIT);
 	llc_header_len += XDLC_CONTROL_LEN(control, TRUE);
-	if (is_snap)
-		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of protocol ID */
 	if (!BYTES_ARE_IN_FRAME(offset, len, llc_header_len)) {
 		ld->other++;
 		return;
 	}
 
-	if (is_snap) {
-		oui = pd[offset+3] << 16 | pd[offset+4] << 8 | pd[offset+5];
-		if (XDLC_IS_INFORMATION(control)) {
-			etype = pntohs(&pd[offset+6]);
-			switch (oui) {
-
-			case OUI_ENCAP_ETHER:
-			case OUI_CISCO_90:
-			case OUI_APPLE_ATALK:
-				/* No, I have no idea why Apple used
-				   one of their own OUIs, rather than
-				   OUI_ENCAP_ETHER, and an Ethernet
-				   packet type as protocol ID, for
-				   AppleTalk data packets - but used
-				   OUI_ENCAP_ETHER and an Ethernet
-				   packet type for AARP packets. */
-				capture_ethertype(etype, pd, offset+8, len,
-				    ld);
-				break;
-			case OUI_CISCO:
-				capture_ethertype(etype, pd, offset + 8, len,
-				    ld);
-				break;
-			default:
-				ld->other++;
-				break;
-			}
-		}
+	if (!XDLC_IS_INFORMATION(control)) {
+		ld->other++;
+		return;
 	}
+	if (is_snap)
+		capture_snap(pd, offset+3, len, ld);
 	else {
 		/* non-SNAP */
-		if (XDLC_IS_INFORMATION(control)) {
-			switch (pd[offset]) {
+		switch (pd[offset]) {
 
-			case SAP_IP:
-				capture_ip(pd, offset + llc_header_len, len,
-				    ld);
-				break;
+		case SAP_IP:
+			capture_ip(pd, offset + llc_header_len, len, ld);
+			break;
 
-			case SAP_NETWARE1:
-			case SAP_NETWARE2:
-				capture_ipx(ld);
-				break;
+		case SAP_NETWARE1:
+		case SAP_NETWARE2:
+			capture_ipx(ld);
+			break;
 
-			case SAP_NETBIOS:
-				capture_netbios(ld);
-				break;
+		case SAP_NETBIOS:
+			capture_netbios(ld);
+			break;
 
-			case SAP_VINES1:
-			case SAP_VINES2:
-				capture_vines(ld);
-				break;
+		case SAP_VINES1:
+		case SAP_VINES2:
+			capture_vines(ld);
+			break;
 
-			default:
-				ld->other++;
-				break;
-			}
+		default:
+			ld->other++;
+			break;
 		}
 	}
 }
 
+void
+capture_snap(const guchar *pd, int offset, int len, packet_counts *ld)
+{
+	guint32		oui;
+	guint16		etype;
+
+	if (!BYTES_ARE_IN_FRAME(offset, len, 5)) {
+		ld->other++;
+		return;
+	}
+
+	oui = pd[offset] << 16 | pd[offset+1] << 8 | pd[offset+2];
+	etype = pntohs(&pd[offset+3]);
+	switch (oui) {
+
+	case OUI_ENCAP_ETHER:
+	case OUI_CISCO_90:
+	case OUI_APPLE_ATALK:
+		/* No, I have no idea why Apple used
+		   one of their own OUIs, rather than
+		   OUI_ENCAP_ETHER, and an Ethernet
+		   packet type as protocol ID, for
+		   AppleTalk data packets - but used
+		   OUI_ENCAP_ETHER and an Ethernet
+		   packet type for AARP packets. */
+		capture_ethertype(etype, pd, offset+5, len, ld);
+		break;
+
+	case OUI_CISCO:
+		capture_ethertype(etype, pd, offset+5, len, ld);
+		break;
+
+	default:
+		ld->other++;
+		break;
+	}
+}
+
 /* Used only for U frames */
 static const xdlc_cf_items llc_cf_items = {
 	NULL,
@@ -338,6 +380,54 @@ static const xdlc_cf_items llc_cf_items_ext = {
 	&hf_llc_ftype_s_u_ext
 };
 
+static void
+dissect_basicxid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	proto_tree	*xid_tree = NULL;
+	proto_item	*ti = NULL;
+	guint8		format, types, wsize;
+
+	if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "XID");
+	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_clear(pinfo->cinfo, COL_INFO);
+	}
+
+	format = tvb_get_guint8(tvb, 0);
+	if (tree) {
+		ti = proto_tree_add_item(tree, proto_basicxid, tvb, 0, -1, FALSE);
+		xid_tree = proto_item_add_subtree(ti, ett_llc_basicxid);
+		proto_tree_add_uint(xid_tree, hf_llc_xid_format, tvb, 0,
+			1, format);
+	} else
+		xid_tree = NULL;
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO,
+		    "Basic Format");
+	}
+
+	types = tvb_get_guint8(tvb, 1);
+	if (tree) {
+		proto_tree_add_uint(xid_tree, hf_llc_xid_types, tvb, 1,
+			1, types & TYPES_MASK);
+	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO,
+		    "; %s", val_to_str(types & TYPES_MASK, type_vals, "0x%02x")
+		);
+	}
+	wsize = tvb_get_guint8(tvb, 2);
+	if (tree) {
+		proto_tree_add_uint(xid_tree, hf_llc_xid_wsize, tvb, 2,
+			1, (wsize & 0xFE) >> 1);
+	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO,
+		    "; Window Size %d", (wsize & 0xFE) >> 1);
+	}
+}
+
 static void
 dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
@@ -346,7 +436,7 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	int		is_snap;
 	guint16		control;
 	int		llc_header_len;
-	guint8		dsap, ssap;
+	guint8		dsap, ssap, format;
 	tvbuff_t	*next_tvb;
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
@@ -404,10 +494,10 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		if (check_col(pinfo->cinfo, COL_INFO)) {
 			col_append_fstr(pinfo->cinfo, COL_INFO,
 			    "; DSAP %s %s, SSAP %s %s",
-			    val_to_str(dsap & SAP_MASK, sap_vals, "%02x"),
+			    val_to_str(dsap & SAP_MASK, sap_vals, "0x%02x"),
 			    dsap & DSAP_GI_BIT ?
 			      "Group" : "Individual",
-			    val_to_str(ssap & SAP_MASK, sap_vals, "%02x"),
+			    val_to_str(ssap & SAP_MASK, sap_vals, "0x%02x"),
 			    ssap & SSAP_CR_BIT ?
 			      "Response" : "Command"
 			);
@@ -430,13 +520,22 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    == (XDLC_XID|XDLC_U)) {
 				/*
 				 * Non-SNAP XID frame.
+				 * Test for LLC basic format first
+				 */
+				format = tvb_get_guint8(next_tvb, 0);
+				if (format == 0x81) {
+				    dissect_basicxid(next_tvb, pinfo, tree);
+				} else {
+				/*
 				 * Try the XID LLC subdissector table
 				 * with the DSAP.
 				 */
-				if (!dissector_try_port(xid_subdissector_table,
-				    dsap, next_tvb, pinfo, tree)) {
-					call_dissector(data_handle, next_tvb,
-					    pinfo, tree);
+				    if (!dissector_try_port(
+					xid_subdissector_table, dsap, next_tvb,
+					pinfo, tree)) {
+					    call_dissector(data_handle,
+						next_tvb, pinfo, tree);
+				    }
 				}
 			} else {
 				call_dissector(data_handle, next_tvb, pinfo,
@@ -461,6 +560,9 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	dissector_table_t subdissector_table;
 	int		hf;
 
+	/*
+	 * XXX - what about non-UI frames?
+	 */
 	oui =	tvb_get_ntoh24(tvb, offset);
 	etype = tvb_get_ntohs(tvb, offset+3);
 
@@ -614,19 +716,19 @@ proto_register_llc(void)
 	static hf_register_info hf[] = {
 		{ &hf_llc_dsap,
 		{ "DSAP",	"llc.dsap", FT_UINT8, BASE_HEX,
-			VALS(sap_vals), 0x0, "", HFILL }},
+			VALS(sap_vals), 0x0, "DSAP - 7 Most Significant Bits only", HFILL }},
 
 		{ &hf_llc_dsap_ig,
 		{ "IG Bit",	"llc.dsap.ig", FT_BOOLEAN, BASE_HEX,
-			&ig_bit, 0x0, "Individual/Group", HFILL }},
+			&ig_bit, 0x0, "Individual/Group - Least Significant Bit only", HFILL }},
 
 		{ &hf_llc_ssap,
 		{ "SSAP", "llc.ssap", FT_UINT8, BASE_HEX,
-			VALS(sap_vals), 0x0, "", HFILL }},
+			VALS(sap_vals), 0x0, "SSAP - 7 Most Significant Bits only", HFILL }},
 
 		{ &hf_llc_ssap_cr,
 		{ "CR Bit", "llc.ssap.cr", FT_BOOLEAN, BASE_HEX,
-			&cr_bit, 0x0, "Command/Response", HFILL }},
+			&cr_bit, 0x0, "Command/Response - Least Significant Bit only", HFILL }},
 
 		{ &hf_llc_ctrl,
 		{ "Control", "llc.control", FT_UINT16, BASE_HEX,
@@ -691,7 +793,7 @@ proto_register_llc(void)
 
 		{ &hf_llc_pid,
 		{ "Protocol ID", "llc.pid", FT_UINT16, BASE_HEX,
-			NULL, 0x0, "", HFILL }}
+			NULL, 0x0, "", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_llc,
@@ -711,6 +813,33 @@ proto_register_llc(void)
 	register_dissector("llc", dissect_llc, proto_llc);
 }
 
+void
+proto_register_basicxid(void)
+{
+	static hf_register_info hf[] = {
+		{ &hf_llc_xid_format,
+		{ "XID Format", "llc.xid.format", FT_UINT8, BASE_HEX,
+			VALS(format_vals), 0x0, "", HFILL }},
+
+		{ &hf_llc_xid_types,
+		{ "LLC Types/Classes", "llc.xid.types", FT_UINT8, BASE_HEX,
+			VALS(type_vals), 0x0, "", HFILL }},
+
+		{ &hf_llc_xid_wsize,
+		{ "Receive Window Size", "llc.xid.wsize", FT_UINT8, BASE_DEC,
+			NULL, 0x0, "", HFILL }},
+	};
+	static gint *ett[] = {
+		&ett_llc_basicxid,
+	};
+
+	proto_basicxid = proto_register_protocol("Logical-Link Control Basic Format XID", "Basic Format XID", "basicxid");
+	proto_register_field_array(proto_basicxid, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
+
+	register_dissector("basicxid", dissect_basicxid, proto_basicxid);
+}
+
 static void
 register_hf(gpointer key _U_, gpointer value, gpointer user_data _U_)
 {
