@@ -3,7 +3,7 @@
  * Routines for LWAPP encapsulated packet disassembly
  * draft-ohara-capwap-lwapp-N (the current draft is 0)
  *
- * $Id: packet-lwapp.c 14883 2005-07-09 00:47:57Z guy $
+ * $Id: packet-lwapp.c 15264 2005-08-08 18:50:39Z ulfl $
  *
  * Copyright (c) 2003 by David Frascone <dave@frascone.com>
  *
@@ -43,10 +43,6 @@
 #include <epan/prefs.h>
 
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #define LWAPP_FLAGS_T 0x04
 #define LWAPP_FLAGS_F 0x02
 #define LWAPP_FLAGS_FT 0x01
