@@ -8,7 +8,7 @@
  * Copyright 2001, 2002, Jeff Morriss <jeff.morriss[AT]ulticom.com>,
  * updated by Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-m2pa.c 12842 2004-12-27 13:17:20Z ulfl $
+ * $Id: packet-m2pa.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,13 +38,13 @@
 
 #include <epan/packet.h>
 #include <epan/prefs.h>
-#include "sctpppids.h"
+#include <epan/sctpppids.h>
 
 #define NETWORK_BYTE_ORDER          FALSE
 #define SCTP_PORT_M2PA              3565
 
-static int global_sctp_port       = SCTP_PORT_M2PA;
-static int sctp_port              = 0;
+static guint global_sctp_port       = SCTP_PORT_M2PA;
+static guint sctp_port              = 0;
 
 void proto_reg_handoff_m2pa(void);
 
