@@ -6,7 +6,7 @@
  *
  * Copyright 2002, Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-m2ua.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: packet-m2ua.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 
 #include <epan/packet.h>
 #include <epan/prefs.h>
-#include "sctpppids.h"
+#include <epan/sctpppids.h>
 
 #define SCTP_PORT_M2UA                  2904
 #define NETWORK_BYTE_ORDER              FALSE
