@@ -4,7 +4,7 @@
  *
  * Official home page: http://openlito.sourceforge.net/
  *
- * $Id: packet-manolito.c 14884 2005-07-09 00:53:17Z guy $
+ * $Id: packet-manolito.c 15758 2005-09-11 21:25:37Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,6 +36,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 
 /* Initialize the protocol and registered fields */
 static int proto_manolito = -1;
@@ -66,7 +67,7 @@ dissect_manolito(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* Set up structures needed to add the protocol subtree and manage it */
 	proto_item *ti;
 	proto_tree *manolito_tree;
-	char* packet_type = 0;
+	const char* packet_type = 0;
 
 	/* Make entries in Protocol column and Info column on summary display */
 	if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
@@ -79,8 +80,8 @@ dissect_manolito(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* MANOLITO packet header (network byte order) */
 	header.checksum = tvb_get_ntohl(tvb, 0);
 	header.seqno = tvb_get_ntohl(tvb, 4);
-	tvb_memcpy(tvb, (guint8 *)&header.src, 8, 4);
-	tvb_memcpy(tvb, (guint8 *)&header.dest, 12, 4);
+	header.src = tvb_get_ipv4(tvb, 8);
+	header.dest = tvb_get_ipv4(tvb, 12);
 	if (tvb_reported_length(tvb) == 19)
 	{
 		header.options = (tvb_get_ntohs(tvb, 16) << 8) +
@@ -125,7 +126,7 @@ dissect_manolito(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			guint8* data;            /* payload */
 			int start;               /* field starting location */
 			char field_name_str[3];  /* printable name */
-			char* longname;          /* human-friendly field name */
+			const char* longname;    /* human-friendly field name */
 
 			start = offset;
 
@@ -191,7 +192,7 @@ dissect_manolito(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			 * widen it past 8 bits, so there shouldn't
 			 * be an overflow.
 			 */
-			data = g_malloc((guint)length + 1);
+			data = ep_alloc((guint)length + 1);
 			tvb_memcpy(tvb, data, ++offset, length);
 			offset += length; 
 
@@ -226,7 +227,6 @@ dissect_manolito(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				proto_tree_add_text(manolito_tree, tvb, start,
 					offset - start, "unknown type %d", dtype);
 			}
-			g_free(data);
 
 		} while(offset < tvb_reported_length(tvb));
 
