@@ -3,7 +3,7 @@
  * Copyright 2001, Tom Uijldert <tom.uijldert@cmg.nl>
  * Copyright 2004, Olivier Biot
  *
- * $Id: packet-mmse.c 15000 2005-07-22 23:04:52Z sahlberg $
+ * $Id: packet-mmse.c 16373 2005-11-01 18:18:46Z obiot $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -62,9 +62,8 @@
  */
 #ifdef DEBUG_mmse
 #define DebugLog(x) \
-	printf("%s:%u: ", __FILE__, __LINE__); \
-	printf x; \
-	fflush(stdout)
+	g_print("%s:%u: ", __FILE__, __LINE__); \
+	g_print x
 #else
 #define DebugLog(x) ;
 #endif
@@ -535,10 +534,10 @@ get_encoded_strval(tvbuff_t *tvb, guint offset, char **strval)
     if (field < 32) {
 	length = get_value_length(tvb, offset, &count);
 	if (length < 2) {
-	    *strval = g_strdup("");
+	    *strval = "";
 	} else {
 	    /* \todo	Something with "Char-set", skip for now	*/
-	    *strval = (char *)tvb_get_string(tvb, offset + count + 1, length - 1);
+	    *strval = (char *)tvb_get_ephemeral_string(tvb, offset + count + 1, length - 1);
 	}
 	return count + length;
     } else
