@@ -1,7 +1,7 @@
 /* packet-mount.c
  * Routines for mount dissection
  *
- * $Id: packet-mount.c 15005 2005-07-23 00:22:02Z sahlberg $
+ * $Id: packet-mount.c 16340 2005-10-27 09:48:19Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -104,6 +104,23 @@ static gint ett_mount_statvfs_flag = -1;
 static char group_name_list[MAX_GROUP_NAME_LIST];
 static int  group_names_len;
 
+/* RFC 1813, Page 107 */
+static const value_string mount3_mountstat3[] =
+{
+	{	0,	"OK" },
+	{	1,	"ERR_PERM" },
+	{	2,	"ERR_NOENT" },
+	{	5,	"ERR_IO" },
+	{	13,	"ERR_ACCESS" },
+	{	20,	"ERR_NOTDIR" },
+	{	22,	"ERR_INVAL" },
+	{	63,	"ERR_NAMETOOLONG" },
+	{	10004,	"ERR_NOTSUPP" },
+	{	10006,	"ERR_SERVERFAULT" },
+	{	0,	NULL }
+};
+
+
 /* RFC 1094, Page 24 */
 /* This function dissects fhstatus for v1 and v2 of the mount protocol.
  * Formally, hf_mount3_status only define the status codes returned by version
@@ -126,6 +143,12 @@ dissect_fhstatus(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree
 		break;
 		default:
 			/* void */
+			if (check_col(pinfo->cinfo, COL_INFO)) {
+				col_append_fstr(
+					pinfo->cinfo, COL_INFO, " Error:%s",
+					val_to_str(status, mount3_mountstat3,
+					    "Unknown (0x%08X)"));
+			}
 		break;
 	}
 
@@ -137,6 +160,8 @@ static int
 dissect_mount_dirpath_call(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		proto_tree *tree)
 {
+	char *mountpoint=NULL;
+
 	if((!pinfo->fd->flags.visited) && nfs_file_name_snooping){
 		rpc_call_info_value *civ=pinfo->private_data;
 
@@ -165,11 +190,12 @@ dissect_mount_dirpath_call(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 	}
 
-	if ( tree )
-	{
-		offset = dissect_rpc_string(tvb,tree,hf_mount_path,offset,NULL);
+	offset = dissect_rpc_string(tvb,tree,hf_mount_path,offset,&mountpoint);
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO," %s", mountpoint);
 	}
 
+
 	return offset;
 }
 
@@ -243,7 +269,7 @@ dissect_group(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tre
 			MAX_GROUP_NAME_LIST-5-group_names_len,
 			group_name_list+group_names_len);
 		if((group_names_len>=(MAX_GROUP_NAME_LIST-5))||(str_len<0)){
-			strcpy(group_name_list+(MAX_GROUP_NAME_LIST-5),"...");
+			g_snprintf(group_name_list+(MAX_GROUP_NAME_LIST-5), 5, "...");
 			group_names_len=MAX_GROUP_NAME_LIST;
 		} else {
 			group_names_len+=str_len;
@@ -499,30 +525,22 @@ dissect_mount_pathconf_reply(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	return offset;
 }
 
-/* RFC 1813, Page 107 */
-static const value_string mount3_mountstat3[] =
-{
-	{	0,	"OK" },
-	{	1,	"ERR_PERM" },
-	{	2,	"ERR_NOENT" },
-	{	5,	"ERR_IO" },
-	{	13,	"ERR_ACCESS" },
-	{	20,	"ERR_NOTDIR" },
-	{	22,	"ERR_INVAL" },
-	{	63,	"ERR_NAMETOOLONG" },
-	{	10004,	"ERR_NOTSUPP" },
-	{	10006,	"ERR_SERVERFAULT" },
-	{	0,	NULL }
-};
-
 
 /* RFC 1813, Page 107 */
 static int
-dissect_mountstat3(tvbuff_t *tvb, proto_tree *tree, int offset, int hfindex, guint32 *status)
+dissect_mountstat3(packet_info *pinfo, tvbuff_t *tvb, proto_tree *tree, int offset, int hfindex, guint32 *status)
 {
 	guint32 mountstat3;
 
 	mountstat3 = tvb_get_ntohl(tvb, offset);
+	if(mountstat3){
+		if (check_col(pinfo->cinfo, COL_INFO)) {
+			col_append_fstr(
+				pinfo->cinfo, COL_INFO, " Error:%s",
+				val_to_str(mountstat3, mount3_mountstat3,
+				    "Unknown (0x%08X)"));
+		}
+	}
 
 	offset = dissect_rpc_uint32(tvb,tree,hfindex,offset);
 	*status = mountstat3;
@@ -538,7 +556,7 @@ dissect_mount3_mnt_reply(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 	guint32 auth_flavor;
 	guint32 auth_flavor_i;
 
-	offset = dissect_mountstat3(tvb,tree,offset,hf_mount3_status,&status);
+	offset = dissect_mountstat3(pinfo,tvb,tree,offset,hf_mount3_status,&status);
 
 	switch (status) {
 		case 0:
