@@ -3,7 +3,10 @@
  * Author: Carlos Pignataro <cpignata@cisco.com>
  * Copyright 2005, cisco Systems, Inc.
  *
- * $Id$
+ * (c) Copyright 2006, _FF_ Francesco Fondelli <francesco.fondelli@gmail.com>  
+ *                     added MPLS OAM support, ITU-T Y.1711
+ *
+ * $Id: packet-mpls.h 17779 2006-03-31 05:09:37Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +36,7 @@ enum {
     LABEL_ROUTER_ALERT,
     LABEL_IP6_EXPLICIT_NULL,
     LABEL_IMPLICIT_NULL,
-
+    LABEL_OAM_ALERT = 14,
     LABEL_MAX_RESERVED = 15
 };
 
