@@ -4,7 +4,7 @@
  *
  * Copyright 2001, Heikki Vatiainen <hessu@cs.tut.fi>
  *
- * $Id: packet-msdp.c 14608 2005-06-10 18:28:22Z ulfl $
+ * $Id: packet-msdp.c 16956 2006-01-05 13:01:08Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,6 +36,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 
 /* MSDP message types. The messages are TLV (Type-Length-Value) encoded */
 enum { MSDP_SA     = 1,
@@ -57,7 +58,7 @@ static const value_string msdp_types[] = {
         { MSDP_KEEP_ALIVE,        "KeepAlive"                    },
         { MSDP_NOTIFICATION,      "Notification"                 },
 
-        { MSDP_TRACE_IN_PROGRESS, "MSDP traceroute in progresss" },
+        { MSDP_TRACE_IN_PROGRESS, "MSDP traceroute in progress"  },
         { MSDP_TRACE_REPLY,       "MSDP traceroute reply"        },
         { 0, NULL },
 };
@@ -246,7 +247,6 @@ static void dissect_msdp_sa(tvbuff_t *tvb, packet_info *pinfo,
     proto_tree *tree, int *offset, int length)
 {
         guint8 entries;
-        guint32 rp_addr;
 
         if (length < 1)
                 return;
@@ -260,7 +260,6 @@ static void dissect_msdp_sa(tvbuff_t *tvb, packet_info *pinfo,
                 length = 0;
                 return;
         }
-        tvb_memcpy(tvb, (guint8 *)&rp_addr, *offset, 4);
         proto_tree_add_item(tree, hf_msdp_sa_rp_addr, tvb, *offset, 4, FALSE);
         *offset += 4;
         length -= 4;
@@ -343,7 +342,7 @@ static void add_notification_data_ipv4addr(tvbuff_t *tvb, proto_tree *tree, int
 
         proto_tree_add_item(tree, hf_msdp_not_res, tvb, *offset, 3, FALSE);
         *offset += 3;
-        tvb_memcpy(tvb, (guint8 *)&ipaddr, *offset, 4);
+        ipaddr = tvb_get_ipv4(tvb, *offset);
         proto_tree_add_ipv4_format(tree, hf_msdp_not_ipv4, tvb, *offset, 4, ipaddr,
                                    "%s: %s", addrtype, ip_to_str((guint8 *)&ipaddr));
         *offset += 4;
