@@ -5,11 +5,16 @@
  *   ANSI T1.111.4-1996
  *   ITU-T Q.704 7/1996
  *   GF 001-9001 (Chinese ITU variant)
+ *   JT-Q704 and NTT-Q704 (Japan)
+ *
+ *   Note that the division of the Japan SLS into the SLC and A/B bit (for
+ *   management messages) is not done.
  *
  * Copyright 2001, Michael Tuexen <tuexen [AT] fh-muenster.de>
- * Updated for ANSI and Chinese ITU support by Jeff Morriss <jeff.morriss[AT]ulticom.com>
+ * Updated for ANSI Chinese ITU, and Japan support by
+ *  Jeff Morriss <jeff.morriss[AT]ulticom.com>
  *
- * $Id: packet-mtp3.c 13517 2005-02-25 20:33:28Z tuexen $
+ * $Id: packet-mtp3.c 17815 2006-04-05 16:03:36Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,13 +44,10 @@
 #include <string.h>
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-#include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include <epan/tap.h>
 #include <epan/prefs.h>
+#include <epan/emem.h>
 
 /* Initialize the protocol and registered fields */
 static int proto_mtp3  = -1;
@@ -78,6 +80,13 @@ static int hf_mtp3_itu_sls = -1;
 static int hf_mtp3_ansi_5_bit_sls = -1;
 static int hf_mtp3_ansi_8_bit_sls = -1;
 static int hf_mtp3_chinese_itu_sls = -1;
+static int hf_mtp3_japan_dpc = -1;
+static int hf_mtp3_japan_opc = -1;
+static int hf_mtp3_japan_pc = -1;
+static int hf_mtp3_japan_4_bit_sls = -1;
+static int hf_mtp3_japan_4_bit_sls_spare = -1;
+static int hf_mtp3_japan_5_bit_sls = -1;
+static int hf_mtp3_japan_5_bit_sls_spare = -1;
 
 /* Initialize the subtree pointers */
 static gint ett_mtp3 = -1;
@@ -94,50 +103,69 @@ typedef enum {
   ITU_PC_STRUCTURE_4_3_4_3 = 3
 } ITU_PC_Structure_Type;
 
+typedef enum {
+  JAPAN_PC_STRUCTURE_NONE    = 1,
+  JAPAN_PC_STRUCTURE_7_4_5   = 2,
+  JAPAN_PC_STRUCTURE_3_4_4_5 = 3
+} JAPAN_PC_Structure_Type;
+
 gint itu_pc_structure = ITU_PC_STRUCTURE_NONE;
+gint japan_pc_structure = JAPAN_PC_STRUCTURE_NONE;
 
 #include <packet-mtp3.h>
 gint mtp3_standard = ITU_STANDARD;
 
 static gboolean mtp3_use_ansi_5_bit_sls = FALSE;
-static gint mtp3_net_addr_fmt = MTP3_NET_ADDR_FMT_DASHED;
+static gboolean mtp3_use_japan_5_bit_sls = FALSE;
+static gint mtp3_addr_fmt = MTP3_ADDR_FMT_DASHED;
 static mtp3_addr_pc_t mtp3_addr_dpc, mtp3_addr_opc;
 
 #define SIO_LENGTH                1
+#define SLS_LENGTH                1
+#define SIO_OFFSET                0
+#define ROUTING_LABEL_OFFSET      (SIO_OFFSET + SIO_LENGTH)
 
 #define ITU_ROUTING_LABEL_LENGTH  4
 #define ITU_HEADER_LENGTH         (SIO_LENGTH + ITU_ROUTING_LABEL_LENGTH)
-#define ITU_SLS_LENGTH            1
 
-#define SIO_OFFSET                0
-#define ITU_SLS_OFFSET            (SIO_OFFSET + ITU_HEADER_LENGTH - ITU_SLS_LENGTH)
-#define ITU_ROUTING_LABEL_OFFSET  (SIO_OFFSET + SIO_LENGTH)
+#define ITU_SLS_OFFSET            (SIO_OFFSET + ITU_HEADER_LENGTH - SLS_LENGTH)
 #define ITU_MTP_PAYLOAD_OFFSET    (SIO_OFFSET + ITU_HEADER_LENGTH)
 
-#define ANSI_SLS_LENGTH           1
-#define ANSI_ROUTING_LABEL_LENGTH (ANSI_PC_LENGTH + ANSI_PC_LENGTH + ANSI_SLS_LENGTH)
+#define ANSI_ROUTING_LABEL_LENGTH (ANSI_PC_LENGTH + ANSI_PC_LENGTH + SLS_LENGTH)
 #define ANSI_HEADER_LENGTH        (SIO_LENGTH + ANSI_ROUTING_LABEL_LENGTH)
 
-#define ANSI_ROUTING_LABEL_OFFSET (SIO_OFFSET + SIO_LENGTH)
-#define ANSI_DPC_OFFSET           ANSI_ROUTING_LABEL_OFFSET
+#define ANSI_DPC_OFFSET           ROUTING_LABEL_OFFSET
 #define ANSI_OPC_OFFSET           (ANSI_DPC_OFFSET + ANSI_PC_LENGTH)
 #define ANSI_SLS_OFFSET           (ANSI_OPC_OFFSET + ANSI_PC_LENGTH)
 #define ANSI_MTP_PAYLOAD_OFFSET   (SIO_OFFSET + ANSI_HEADER_LENGTH)
 
-#define SERVICE_INDICATOR_MASK         0x0F
-#define SPARE_MASK                     0x30
-#define ANSI_PRIORITY_MASK             SPARE_MASK
-#define NETWORK_INDICATOR_MASK         0xC0
-#define ITU_DPC_MASK                   0x00003FFF
-#define ITU_OPC_MASK                   0x0FFFC000
-#define ITU_SLS_MASK                   0xF0000000
+#define JAPAN_SLS_SPARE_LENGTH           1
+#define JAPAN_ROUTING_LABEL_LENGTH (JAPAN_PC_LENGTH + JAPAN_PC_LENGTH + JAPAN_SLS_SPARE_LENGTH)
+#define JAPAN_HEADER_LENGTH        (SIO_LENGTH + JAPAN_ROUTING_LABEL_LENGTH)
 
-#define ANSI_NETWORK_MASK              0x0000FF
-#define ANSI_CLUSTER_MASK              0x00FF00
-#define ANSI_MEMBER_MASK               0xFF0000
-#define ANSI_5BIT_SLS_MASK             0x1F
-#define ANSI_8BIT_SLS_MASK             0xFF
-#define CHINESE_ITU_SLS_MASK           0xF
+#define JAPAN_OPC_OFFSET           (ROUTING_LABEL_OFFSET + JAPAN_PC_LENGTH)
+#define JAPAN_SLS_OFFSET           (JAPAN_OPC_OFFSET + JAPAN_PC_LENGTH)
+#define JAPAN_SPARE_OFFSET         (ROUTING_LABEL_OFFSET + JAPAN_ROUTING_LABEL_LENGTH)
+#define JAPAN_MTP_PAYLOAD_OFFSET   (SIO_OFFSET + JAPAN_HEADER_LENGTH)
+
+#define SERVICE_INDICATOR_MASK     0x0F
+#define SPARE_MASK                 0x30
+#define ANSI_PRIORITY_MASK         SPARE_MASK
+#define NETWORK_INDICATOR_MASK     0xC0
+#define ITU_DPC_MASK               0x00003FFF
+#define ITU_OPC_MASK               0x0FFFC000
+#define ITU_SLS_MASK               0xF0000000
+
+#define ANSI_NETWORK_MASK          0x0000FF
+#define ANSI_CLUSTER_MASK          0x00FF00
+#define ANSI_MEMBER_MASK           0xFF0000
+#define ANSI_5BIT_SLS_MASK         0x1F
+#define ANSI_8BIT_SLS_MASK         0xFF
+#define CHINESE_ITU_SLS_MASK       0xF
+#define JAPAN_4_BIT_SLS_MASK       0xF
+#define JAPAN_4_BIT_SLS_SPARE_MASK 0xF0
+#define JAPAN_5_BIT_SLS_MASK       0x1F
+#define JAPAN_5_BIT_SLS_SPARE_MASK 0xE0
 
 /* the higher values are taken from the M3UA RFC */
 static const value_string mtp3_service_indicator_code_vals[] = {
@@ -194,33 +222,50 @@ static dissector_handle_t data_handle;
  */
 
 void 
-mtp3_pc_to_str_buf(const guint32 pc, gchar *buf)
+mtp3_pc_to_str_buf(const guint32 pc, gchar *buf, int buf_len)
 {
   switch (mtp3_standard)
   {
     case ITU_STANDARD:
       switch (itu_pc_structure) {
         case ITU_PC_STRUCTURE_NONE:
-          sprintf(buf, "%u", pc);
+          g_snprintf(buf, buf_len, "%u", pc);
           break;
         case ITU_PC_STRUCTURE_3_8_3:
           /* this format is used in international ITU networks */
-          sprintf(buf, "%u-%u-%u", (pc & 0x3800)>>11, (pc & 0x7f8) >> 3, (pc & 0x07) >> 0);
+          g_snprintf(buf, buf_len, "%u-%u-%u", (pc & 0x3800)>>11, (pc & 0x7f8) >> 3, (pc & 0x07) >> 0);
           break;
         case ITU_PC_STRUCTURE_4_3_4_3:
           /* this format is used in some national ITU networks, the German one for example. */
-          sprintf(buf, "%u-%u-%u-%u", (pc & 0x3c00) >>10, (pc & 0x0380) >> 7, (pc & 0x0078) >> 3, (pc & 0x0007) >> 0);
+          g_snprintf(buf, buf_len, "%u-%u-%u-%u", (pc & 0x3c00) >>10, (pc & 0x0380) >> 7, (pc & 0x0078) >> 3, (pc & 0x0007) >> 0);
           break;
         default:
-          g_assert_not_reached();
+          DISSECTOR_ASSERT_NOT_REACHED();
       }
       break;
     case ANSI_STANDARD:
     case CHINESE_ITU_STANDARD:
-      sprintf(buf, "%u-%u-%u", (pc & ANSI_NETWORK_MASK), (pc & ANSI_CLUSTER_MASK) >> 8, (pc & ANSI_MEMBER_MASK) >> 16);
+      g_snprintf(buf, buf_len, "%u-%u-%u", (pc & ANSI_NETWORK_MASK), (pc & ANSI_CLUSTER_MASK) >> 8, (pc & ANSI_MEMBER_MASK) >> 16);
+      break;
+    case JAPAN_STANDARD:
+      switch (japan_pc_structure) {
+        case JAPAN_PC_STRUCTURE_NONE:
+	  g_snprintf(buf, buf_len, "%u", pc);
+	  break;
+        case JAPAN_PC_STRUCTURE_7_4_5:
+	  /* This format is specified by NTT */
+	  g_snprintf(buf, buf_len, "%u-%u-%u", (pc & 0xfe00)>>9, (pc & 0x1e0)>>5, (pc & 0x1f));
+	  break;
+        case JAPAN_PC_STRUCTURE_3_4_4_5:
+	  /* Where does this format come from? */
+	  g_snprintf(buf, buf_len, "%u-%u-%u-%u", (pc & 0xe000)>>13, (pc & 0x1e00)>>9, (pc & 0x1e0)>>5, (pc & 0x1f));
+	  break;
+        default:
+          DISSECTOR_ASSERT_NOT_REACHED();
+      }
       break;
     default:
-      g_assert_not_reached();
+      DISSECTOR_ASSERT_NOT_REACHED();
   }
 }
 
@@ -229,9 +274,10 @@ mtp3_pc_to_str_buf(const guint32 pc, gchar *buf)
 gchar *
 mtp3_pc_to_str(const guint32 pc)
 {
-  static gchar str[MAX_STRUCTURED_PC_LENGTH];
+  gchar *str;
   
-  mtp3_pc_to_str_buf(pc, str);
+  str=ep_alloc(MAX_STRUCTURED_PC_LENGTH);
+  mtp3_pc_to_str_buf(pc, str, MAX_STRUCTURED_PC_LENGTH);
   return str;
 }
 
@@ -240,6 +286,8 @@ mtp3_pc_structured(void)
 {
   if ((mtp3_standard == ITU_STANDARD) && (itu_pc_structure == ITU_PC_STRUCTURE_NONE))
     return FALSE;
+  else if ((mtp3_standard == JAPAN_STANDARD) && (japan_pc_structure == JAPAN_PC_STRUCTURE_NONE))
+    return FALSE;
   else
     return TRUE;
 }
@@ -251,60 +299,73 @@ mtp3_pc_structured(void)
 void
 mtp3_addr_to_str_buf(
   const guint8          *data,
-  gchar                 *buf)
+  gchar                 *buf,
+  int                   buf_len)
 {
   const mtp3_addr_pc_t  *addr_pc_p = (const mtp3_addr_pc_t *)data;
 
-  switch (mtp3_net_addr_fmt)
+  switch (mtp3_addr_fmt)
   {
-  case MTP3_NET_ADDR_FMT_DEC:
+  case MTP3_ADDR_FMT_DEC:
     switch (addr_pc_p->type)
     {
     case ITU_STANDARD:
-      sprintf(buf, "%u", addr_pc_p->pc & ITU_PC_MASK);
+      g_snprintf(buf, buf_len, "%u", addr_pc_p->pc & ITU_PC_MASK);
+      break;
+    case JAPAN_STANDARD:
+      g_snprintf(buf, buf_len, "%u", addr_pc_p->pc & JAPAN_PC_MASK);
       break;
     default:
       /* assuming 24-bit */
-      sprintf(buf, "%u", addr_pc_p->pc & ANSI_PC_MASK);
+      g_snprintf(buf, buf_len, "%u", addr_pc_p->pc & ANSI_PC_MASK);
       break;
     }
     break;
 
-  case MTP3_NET_ADDR_FMT_HEX:
+  case MTP3_ADDR_FMT_HEX:
     switch (addr_pc_p->type)
     {
     case ITU_STANDARD:
-      sprintf(buf, "%x", addr_pc_p->pc & ITU_PC_MASK);
+      g_snprintf(buf, buf_len, "%x", addr_pc_p->pc & ITU_PC_MASK);
+      break;
+    case JAPAN_STANDARD:
+      g_snprintf(buf, buf_len, "%x", addr_pc_p->pc & JAPAN_PC_MASK);
       break;
     default:
       /* assuming 24-bit */
-      sprintf(buf, "%x", addr_pc_p->pc & ANSI_PC_MASK);
+      g_snprintf(buf, buf_len, "%x", addr_pc_p->pc & ANSI_PC_MASK);
       break;
     }
     break;
 
-  case MTP3_NET_ADDR_FMT_NI_DEC:
+  case MTP3_ADDR_FMT_NI_DEC:
     switch (addr_pc_p->type)
     {
     case ITU_STANDARD:
-      sprintf(buf, "%u:%u", addr_pc_p->ni, addr_pc_p->pc & ITU_PC_MASK);
+      g_snprintf(buf, buf_len, "%u:%u", addr_pc_p->ni, addr_pc_p->pc & ITU_PC_MASK);
+      break;
+    case JAPAN_STANDARD:
+      g_snprintf(buf, buf_len, "%u:%u", addr_pc_p->ni, addr_pc_p->pc & JAPAN_PC_MASK);
       break;
     default:
       /* assuming 24-bit */
-      sprintf(buf, "%u:%u", addr_pc_p->ni, addr_pc_p->pc & ANSI_PC_MASK);
+      g_snprintf(buf, buf_len, "%u:%u", addr_pc_p->ni, addr_pc_p->pc & ANSI_PC_MASK);
       break;
     }
     break;
 
-  case MTP3_NET_ADDR_FMT_NI_HEX:
+  case MTP3_ADDR_FMT_NI_HEX:
     switch (addr_pc_p->type)
     {
     case ITU_STANDARD:
-      sprintf(buf, "%u:%x", addr_pc_p->ni, addr_pc_p->pc & ITU_PC_MASK);
+      g_snprintf(buf, buf_len, "%u:%x", addr_pc_p->ni, addr_pc_p->pc & ITU_PC_MASK);
+      break;
+    case JAPAN_STANDARD:
+      g_snprintf(buf, buf_len, "%u:%x", addr_pc_p->ni, addr_pc_p->pc & JAPAN_PC_MASK);
       break;
     default:
       /* assuming 24-bit */
-      sprintf(buf, "%u:%x", addr_pc_p->ni, addr_pc_p->pc & ANSI_PC_MASK);
+      g_snprintf(buf, buf_len, "%u:%x", addr_pc_p->ni, addr_pc_p->pc & ANSI_PC_MASK);
       break;
     }
     break;
@@ -312,12 +373,30 @@ mtp3_addr_to_str_buf(
   default:
     /* FALLTHRU */
 
-  case MTP3_NET_ADDR_FMT_DASHED:
-    mtp3_pc_to_str_buf(addr_pc_p->pc, buf);
+  case MTP3_ADDR_FMT_DASHED:
+    mtp3_pc_to_str_buf(addr_pc_p->pc, buf, buf_len);
     break;
   }
 }
 
+guint32 mtp3_pc_hash(const guint8* data) {
+	const mtp3_addr_pc_t  *addr_pc_p = (const mtp3_addr_pc_t *)data;
+	guint32 pc;
+	
+	switch (addr_pc_p->type)
+	{
+		case ITU_STANDARD:
+			pc = (addr_pc_p->pc & ITU_PC_MASK) | ((addr_pc_p->ni % 4) << 14) ;
+			break;
+		default:
+			/* assuming 24-bit */
+			pc = (addr_pc_p->pc & ANSI_PC_MASK) | ((addr_pc_p->ni) << 24) ;
+			break;
+	}
+	
+	return pc;
+}
+
 static void
 dissect_mtp3_sio(tvbuff_t *tvb, packet_info *pinfo, proto_tree *mtp3_tree)
 {
@@ -340,6 +419,7 @@ dissect_mtp3_sio(tvbuff_t *tvb, packet_info *pinfo, proto_tree *mtp3_tree)
     break;
   case ITU_STANDARD:
   case CHINESE_ITU_STANDARD:
+  case JAPAN_STANDARD:
     proto_tree_add_uint(sio_tree, hf_mtp3_itu_spare, tvb, SIO_OFFSET, SIO_LENGTH, sio);
     break;
   }
@@ -362,26 +442,26 @@ dissect_mtp3_routing_label(tvbuff_t *tvb, packet_info *pinfo, proto_tree *mtp3_t
 
   switch (mtp3_standard) {
   case ITU_STANDARD:
-    label_item = proto_tree_add_text(mtp3_tree, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, "Routing label");
+    label_item = proto_tree_add_text(mtp3_tree, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, "Routing label");
     label_tree = proto_item_add_subtree(label_item, ett_mtp3_label);
 
-    label = tvb_get_letohl(tvb, ITU_ROUTING_LABEL_OFFSET);
+    label = tvb_get_letohl(tvb, ROUTING_LABEL_OFFSET);
 
     opc = (label & ITU_OPC_MASK) >> 14;
     dpc =  label & ITU_DPC_MASK;
 
-    proto_tree_add_uint_hidden(label_tree, hf_mtp3_itu_pc, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, opc);
-    proto_tree_add_uint_hidden(label_tree, hf_mtp3_itu_pc, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, dpc);
+    proto_tree_add_uint_hidden(label_tree, hf_mtp3_itu_pc, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, opc);
+    proto_tree_add_uint_hidden(label_tree, hf_mtp3_itu_pc, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, dpc);
 
-    label_dpc_item = proto_tree_add_uint(label_tree, hf_mtp3_itu_dpc, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
+    label_dpc_item = proto_tree_add_uint(label_tree, hf_mtp3_itu_dpc, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
     if (mtp3_pc_structured())
       proto_item_append_text(label_dpc_item, " (%s)", mtp3_pc_to_str(dpc));
 
-    label_opc_item = proto_tree_add_uint(label_tree, hf_mtp3_itu_opc, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
+    label_opc_item = proto_tree_add_uint(label_tree, hf_mtp3_itu_opc, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
     if (mtp3_pc_structured())
       proto_item_append_text(label_opc_item, " (%s)", mtp3_pc_to_str(opc));
 
-    proto_tree_add_uint(label_tree, hf_mtp3_itu_sls, tvb, ITU_ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
+    proto_tree_add_uint(label_tree, hf_mtp3_itu_sls, tvb, ROUTING_LABEL_OFFSET, ITU_ROUTING_LABEL_LENGTH, label);
     break;
 
   case ANSI_STANDARD:
@@ -396,7 +476,7 @@ dissect_mtp3_routing_label(tvbuff_t *tvb, packet_info *pinfo, proto_tree *mtp3_t
     }
 
     /* Create the Routing Label Tree */
-    label_item = proto_tree_add_text(mtp3_tree, tvb, ANSI_ROUTING_LABEL_OFFSET, ANSI_ROUTING_LABEL_LENGTH, "Routing label");
+    label_item = proto_tree_add_text(mtp3_tree, tvb, ROUTING_LABEL_OFFSET, ANSI_ROUTING_LABEL_LENGTH, "Routing label");
     label_tree = proto_item_add_subtree(label_item, ett_mtp3_label);
 
     /* create the DPC tree */
@@ -428,22 +508,53 @@ dissect_mtp3_routing_label(tvbuff_t *tvb, packet_info *pinfo, proto_tree *mtp3_t
     /* SLS */
     if (mtp3_standard == ANSI_STANDARD) {
       if (mtp3_use_ansi_5_bit_sls)
-        proto_tree_add_item(label_tree, hf_mtp3_ansi_5_bit_sls, tvb, ANSI_SLS_OFFSET, ANSI_SLS_LENGTH, TRUE);
+        proto_tree_add_item(label_tree, hf_mtp3_ansi_5_bit_sls, tvb, ANSI_SLS_OFFSET, SLS_LENGTH, TRUE);
       else
-        proto_tree_add_item(label_tree, hf_mtp3_ansi_8_bit_sls, tvb, ANSI_SLS_OFFSET, ANSI_SLS_LENGTH, TRUE);
+        proto_tree_add_item(label_tree, hf_mtp3_ansi_8_bit_sls, tvb, ANSI_SLS_OFFSET, SLS_LENGTH, TRUE);
     } else /* CHINESE_ITU_STANDARD */ {
-      proto_tree_add_item(label_tree, hf_mtp3_chinese_itu_sls, tvb, ANSI_SLS_OFFSET, ITU_SLS_LENGTH, FALSE);
+      proto_tree_add_item(label_tree, hf_mtp3_chinese_itu_sls, tvb, ANSI_SLS_OFFSET, SLS_LENGTH, FALSE);
     }
     break;
+
+  case JAPAN_STANDARD:
+    label_item = proto_tree_add_text(mtp3_tree, tvb, ROUTING_LABEL_OFFSET, JAPAN_ROUTING_LABEL_LENGTH, "Routing label");
+    label_tree = proto_item_add_subtree(label_item, ett_mtp3_label);
+
+    label_dpc_item = proto_tree_add_item(label_tree, hf_mtp3_japan_dpc, tvb, ROUTING_LABEL_OFFSET, JAPAN_PC_LENGTH, TRUE);
+    if (mtp3_pc_structured()) {
+      dpc = tvb_get_letohs(tvb, ROUTING_LABEL_OFFSET);
+      proto_item_append_text(label_dpc_item, " (%s)", mtp3_pc_to_str(dpc));
+    }
+
+    label_opc_item = proto_tree_add_item(label_tree, hf_mtp3_japan_opc, tvb, JAPAN_OPC_OFFSET, JAPAN_PC_LENGTH, TRUE);
+    if (mtp3_pc_structured()) {
+      opc = tvb_get_letohs(tvb, JAPAN_OPC_OFFSET);
+      proto_item_append_text(label_opc_item, " (%s)", mtp3_pc_to_str(opc));
+    }
+
+    proto_tree_add_item_hidden(label_tree, hf_mtp3_japan_pc, tvb, ROUTING_LABEL_OFFSET, JAPAN_PC_LENGTH, TRUE);
+    proto_tree_add_item_hidden(label_tree, hf_mtp3_japan_pc, tvb, JAPAN_OPC_OFFSET, JAPAN_PC_LENGTH, TRUE);
+
+    if (mtp3_use_japan_5_bit_sls) {
+	proto_tree_add_item(label_tree, hf_mtp3_japan_5_bit_sls, tvb, JAPAN_SLS_OFFSET, JAPAN_SLS_SPARE_LENGTH, TRUE);
+	proto_tree_add_item(label_tree, hf_mtp3_japan_5_bit_sls_spare, tvb, JAPAN_SLS_OFFSET, JAPAN_SLS_SPARE_LENGTH, TRUE);
+    } else {
+	proto_tree_add_item(label_tree, hf_mtp3_japan_4_bit_sls, tvb, JAPAN_SLS_OFFSET, JAPAN_SLS_SPARE_LENGTH, TRUE);
+	proto_tree_add_item(label_tree, hf_mtp3_japan_4_bit_sls_spare, tvb, JAPAN_SLS_OFFSET, JAPAN_SLS_SPARE_LENGTH, TRUE);
+    }
+
+    break;
+  default:
+    DISSECTOR_ASSERT_NOT_REACHED();
   }
 
   mtp3_addr_opc.type = mtp3_standard;
   mtp3_addr_opc.pc = opc;
-  SET_ADDRESS(&pinfo->net_src, AT_SS7PC, sizeof(mtp3_addr_opc), (guint8 *) &mtp3_addr_opc);
+  SET_ADDRESS(&pinfo->src, AT_SS7PC, sizeof(mtp3_addr_opc), (guint8 *) &mtp3_addr_opc);
 
   mtp3_addr_dpc.type = mtp3_standard;
   mtp3_addr_dpc.pc = dpc;
-  SET_ADDRESS(&pinfo->net_dst, AT_SS7PC, sizeof(mtp3_addr_dpc), (guint8 *) &mtp3_addr_dpc);
+  SET_ADDRESS(&pinfo->dst, AT_SS7PC, sizeof(mtp3_addr_dpc), (guint8 *) &mtp3_addr_dpc);
 }
 
 static void
@@ -464,6 +575,11 @@ dissect_mtp3_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   case CHINESE_ITU_STANDARD:
     payload_tvb = tvb_new_subset(tvb, ANSI_MTP_PAYLOAD_OFFSET, -1, -1);
     break;
+  case JAPAN_STANDARD:
+    payload_tvb = tvb_new_subset(tvb, JAPAN_MTP_PAYLOAD_OFFSET, -1, -1);
+    break;
+  default:
+    DISSECTOR_ASSERT_NOT_REACHED();
   }
 
   if (check_col(pinfo->cinfo, COL_INFO))
@@ -495,6 +611,9 @@ dissect_mtp3(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       case CHINESE_ITU_STANDARD:
         col_set_str(pinfo->cinfo, COL_PROTOCOL, "MTP3 (Chin. ITU)");
         break;
+      case JAPAN_STANDARD:
+        col_set_str(pinfo->cinfo, COL_PROTOCOL, "MTP3 (Japan)");
+        break;
     };      
 
   if (tree) {
@@ -507,6 +626,9 @@ dissect_mtp3(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     case CHINESE_ITU_STANDARD:
       mtp3_item = proto_tree_add_item(tree, proto_mtp3, tvb, 0, ANSI_HEADER_LENGTH, TRUE);
       break;
+    case JAPAN_STANDARD:
+      mtp3_item = proto_tree_add_item(tree, proto_mtp3, tvb, 0, JAPAN_HEADER_LENGTH, TRUE);
+      break;
     }
     mtp3_tree = proto_item_add_subtree(mtp3_item, ett_mtp3);
 
@@ -536,30 +658,37 @@ proto_register_mtp3(void)
 
   /* Setup list of header fields  See Section 1.6.1 for details*/
   static hf_register_info hf[] = {
-    { &hf_mtp3_service_indicator, { "Service indicator",        "mtp3.service_indicator", FT_UINT8,  BASE_HEX,  VALS(mtp3_service_indicator_code_vals), SERVICE_INDICATOR_MASK, "", HFILL }},
-    { &hf_mtp3_network_indicator, { "Network indicator",        "mtp3.network_indicator", FT_UINT8,  BASE_HEX,  VALS(network_indicator_vals),           NETWORK_INDICATOR_MASK, "", HFILL }},
-    { &hf_mtp3_itu_spare,         { "Spare",                    "mtp3.spare",             FT_UINT8,  BASE_HEX,  NULL,                                   SPARE_MASK,             "", HFILL }},
-    { &hf_mtp3_ansi_priority,     { "Priority",                 "mtp3.priority",          FT_UINT8,  BASE_HEX,  NULL,                                   ANSI_PRIORITY_MASK,     "", HFILL }},
-    { &hf_mtp3_itu_opc,           { "OPC",                      "mtp3.opc",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_OPC_MASK,           "", HFILL }},
-    { &hf_mtp3_itu_pc,            { "PC",                       "mtp3.pc",                FT_UINT32, BASE_DEC,  NULL,                                   0x0,                    "", HFILL }},
-    { &hf_mtp3_24bit_pc,          { "PC",                       "mtp3.pc",                FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,           "", HFILL }},
-    { &hf_mtp3_24bit_opc,         { "OPC",                      "mtp3.opc",               FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,           "", HFILL }},
-    { &hf_mtp3_ansi_opc,          { "DPC",                      "mtp3.ansi_opc",          FT_STRING, BASE_NONE, NULL,                                   0x0,                    "", HFILL }},
-    { &hf_mtp3_chinese_opc,       { "DPC",                      "mtp3.chinese_opc",       FT_STRING, BASE_NONE, NULL,                                   0x0,                    "", HFILL }},
-    { &hf_mtp3_opc_network,       { "OPC Network",              "mtp3.opc.network",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_NETWORK_MASK,      "", HFILL }},
-    { &hf_mtp3_opc_cluster,       { "OPC Cluster",              "mtp3.opc.cluster",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_CLUSTER_MASK,      "", HFILL }},
-    { &hf_mtp3_opc_member,        { "OPC Member",               "mtp3.opc.member",        FT_UINT24, BASE_DEC,  NULL,                                   ANSI_MEMBER_MASK,       "", HFILL }},
-    { &hf_mtp3_itu_dpc,           { "DPC",                      "mtp3.dpc",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_DPC_MASK,           "", HFILL }},
-    { &hf_mtp3_24bit_dpc,         { "DPC",                      "mtp3.dpc",               FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,           "", HFILL }},
-    { &hf_mtp3_ansi_dpc,          { "DPC",                      "mtp3.ansi_dpc",          FT_STRING, BASE_NONE, NULL,                                   0x0,                    "", HFILL }},
-    { &hf_mtp3_chinese_dpc,       { "DPC",                      "mtp3.chinese_dpc",       FT_STRING, BASE_NONE, NULL,                                   0x0,                    "", HFILL }},
-    { &hf_mtp3_dpc_network,       { "DPC Network",              "mtp3.dpc.network",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_NETWORK_MASK,      "", HFILL }},
-    { &hf_mtp3_dpc_cluster,       { "DPC Cluster",              "mtp3.dpc.cluster",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_CLUSTER_MASK,      "", HFILL }},
-    { &hf_mtp3_dpc_member,        { "DPC Member",               "mtp3.dpc.member",        FT_UINT24, BASE_DEC,  NULL,                                   ANSI_MEMBER_MASK,       "", HFILL }},
-    { &hf_mtp3_itu_sls,           { "Signalling Link Selector", "mtp3.sls",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_SLS_MASK,           "", HFILL }},
-    { &hf_mtp3_ansi_5_bit_sls,    { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   ANSI_5BIT_SLS_MASK,     "", HFILL }},
-    { &hf_mtp3_ansi_8_bit_sls,    { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   ANSI_8BIT_SLS_MASK,     "", HFILL }},
-    { &hf_mtp3_chinese_itu_sls,   { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   CHINESE_ITU_SLS_MASK,   "", HFILL }}
+    { &hf_mtp3_service_indicator,     { "Service indicator",        "mtp3.service_indicator", FT_UINT8,  BASE_HEX,  VALS(mtp3_service_indicator_code_vals), SERVICE_INDICATOR_MASK,     "", HFILL }},
+    { &hf_mtp3_network_indicator,     { "Network indicator",        "mtp3.network_indicator", FT_UINT8,  BASE_HEX,  VALS(network_indicator_vals),           NETWORK_INDICATOR_MASK,     "", HFILL }},
+    { &hf_mtp3_itu_spare,             { "Spare",                    "mtp3.spare",             FT_UINT8,  BASE_HEX,  NULL,                                   SPARE_MASK,                 "", HFILL }},
+    { &hf_mtp3_ansi_priority,         { "Priority",                 "mtp3.priority",          FT_UINT8,  BASE_HEX,  NULL,                                   ANSI_PRIORITY_MASK,         "", HFILL }},
+    { &hf_mtp3_itu_opc,               { "OPC",                      "mtp3.opc",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_OPC_MASK,               "", HFILL }},
+    { &hf_mtp3_itu_pc,                { "PC",                       "mtp3.pc",                FT_UINT32, BASE_DEC,  NULL,                                   0x0,                        "", HFILL }},
+    { &hf_mtp3_24bit_pc,              { "PC",                       "mtp3.pc",                FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,               "", HFILL }},
+    { &hf_mtp3_24bit_opc,             { "OPC",                      "mtp3.opc",               FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,               "", HFILL }},
+    { &hf_mtp3_ansi_opc,              { "DPC",                      "mtp3.ansi_opc",          FT_STRING, BASE_NONE, NULL,                                   0x0,                        "", HFILL }},
+    { &hf_mtp3_chinese_opc,           { "DPC",                      "mtp3.chinese_opc",       FT_STRING, BASE_NONE, NULL,                                   0x0,                        "", HFILL }},
+    { &hf_mtp3_opc_network,           { "OPC Network",              "mtp3.opc.network",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_NETWORK_MASK,          "", HFILL }},
+    { &hf_mtp3_opc_cluster,           { "OPC Cluster",              "mtp3.opc.cluster",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_CLUSTER_MASK,          "", HFILL }},
+    { &hf_mtp3_opc_member,            { "OPC Member",               "mtp3.opc.member",        FT_UINT24, BASE_DEC,  NULL,                                   ANSI_MEMBER_MASK,           "", HFILL }},
+    { &hf_mtp3_japan_opc,             { "OPC",                      "mtp3.opc",               FT_UINT16, BASE_DEC,  NULL,                                   JAPAN_PC_MASK,              "", HFILL }},
+    { &hf_mtp3_japan_pc,              { "PC",                       "mtp3.pc",                FT_UINT16, BASE_DEC,  NULL,                                   JAPAN_PC_MASK,              "", HFILL }},
+    { &hf_mtp3_itu_dpc,               { "DPC",                      "mtp3.dpc",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_DPC_MASK,               "", HFILL }},
+    { &hf_mtp3_24bit_dpc,             { "DPC",                      "mtp3.dpc",               FT_UINT32, BASE_DEC,  NULL,                                   ANSI_PC_MASK,               "", HFILL }},
+    { &hf_mtp3_ansi_dpc,              { "DPC",                      "mtp3.ansi_dpc",          FT_STRING, BASE_NONE, NULL,                                   0x0,                        "", HFILL }},
+    { &hf_mtp3_chinese_dpc,           { "DPC",                      "mtp3.chinese_dpc",       FT_STRING, BASE_NONE, NULL,                                   0x0,                        "", HFILL }},
+    { &hf_mtp3_dpc_network,           { "DPC Network",              "mtp3.dpc.network",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_NETWORK_MASK,          "", HFILL }},
+    { &hf_mtp3_dpc_cluster,           { "DPC Cluster",              "mtp3.dpc.cluster",       FT_UINT24, BASE_DEC,  NULL,                                   ANSI_CLUSTER_MASK,          "", HFILL }},
+    { &hf_mtp3_dpc_member,            { "DPC Member",               "mtp3.dpc.member",        FT_UINT24, BASE_DEC,  NULL,                                   ANSI_MEMBER_MASK,           "", HFILL }},
+    { &hf_mtp3_japan_dpc,             { "DPC",                      "mtp3.dpc",               FT_UINT16, BASE_DEC,  NULL,                                   JAPAN_PC_MASK,              "", HFILL }},
+    { &hf_mtp3_itu_sls,               { "Signalling Link Selector", "mtp3.sls",               FT_UINT32, BASE_DEC,  NULL,                                   ITU_SLS_MASK,               "", HFILL }},
+    { &hf_mtp3_japan_4_bit_sls,       { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   JAPAN_4_BIT_SLS_MASK,       "", HFILL }},
+    { &hf_mtp3_japan_4_bit_sls_spare, { "SLS Spare",                "mtp3.sls_spare",         FT_UINT8,  BASE_HEX,  NULL,                                   JAPAN_4_BIT_SLS_SPARE_MASK, "", HFILL }},
+    { &hf_mtp3_japan_5_bit_sls,       { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   JAPAN_5_BIT_SLS_MASK,       "", HFILL }},
+    { &hf_mtp3_japan_5_bit_sls_spare, { "SLS Spare",                "mtp3.sls_spare",         FT_UINT8,  BASE_HEX,  NULL,                                   JAPAN_5_BIT_SLS_SPARE_MASK, "", HFILL }},
+    { &hf_mtp3_ansi_5_bit_sls,        { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   ANSI_5BIT_SLS_MASK,         "", HFILL }},
+    { &hf_mtp3_ansi_8_bit_sls,        { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   ANSI_8BIT_SLS_MASK,         "", HFILL }},
+    { &hf_mtp3_chinese_itu_sls,       { "Signalling Link Selector", "mtp3.sls",               FT_UINT8,  BASE_DEC,  NULL,                                   CHINESE_ITU_SLS_MASK,       "", HFILL }}
   };
 
   /* Setup protocol subtree array */
@@ -575,15 +704,16 @@ proto_register_mtp3(void)
     { "itu",         "ITU",         ITU_STANDARD },
     { "ansi",        "ANSI",        ANSI_STANDARD },
     { "chinese-itu", "Chinese ITU", CHINESE_ITU_STANDARD },
+    { "japan",       "Japan",       JAPAN_STANDARD },
     { NULL, NULL, 0 }
   };
 
-  static enum_val_t mtp3_net_addr_fmt_str_e[] = {
-    { "decimal",        "Decimal",              MTP3_NET_ADDR_FMT_DEC },
-    { "hexadecimal",    "Hexadecimal",          MTP3_NET_ADDR_FMT_HEX },
-    { "ni-decimal",     "NI-Decimal",           MTP3_NET_ADDR_FMT_NI_DEC },
-    { "ni-hexadecimal", "NI-Hexadecimal",       MTP3_NET_ADDR_FMT_NI_HEX },
-    { "dashed",         "Dashed",               MTP3_NET_ADDR_FMT_DASHED },
+  static enum_val_t mtp3_addr_fmt_str_e[] = {
+    { "decimal",        "Decimal",              MTP3_ADDR_FMT_DEC },
+    { "hexadecimal",    "Hexadecimal",          MTP3_ADDR_FMT_HEX },
+    { "ni-decimal",     "NI-Decimal",           MTP3_ADDR_FMT_NI_DEC },
+    { "ni-hexadecimal", "NI-Hexadecimal",       MTP3_ADDR_FMT_NI_HEX },
+    { "dashed",         "Dashed",               MTP3_ADDR_FMT_DASHED },
     { NULL,             NULL,                   0 }
   };
 
@@ -593,6 +723,13 @@ proto_register_mtp3(void)
     { "4-3-4-3",      "4-3-4-3",      ITU_PC_STRUCTURE_4_3_4_3 },
     { NULL,           NULL,           0 }
   };
+
+  static enum_val_t japan_pc_structures[] = {
+    { "unstructured", "Unstructured", JAPAN_PC_STRUCTURE_NONE},
+    { "7-4-5",        "7-4-5",        JAPAN_PC_STRUCTURE_7_4_5 },
+    { "3-4-4-5",      "3-4-4-5",      JAPAN_PC_STRUCTURE_3_4_4_5 },
+    { NULL,           NULL,           0 }
+  };
   
  /* Register the protocol name and description */
   proto_mtp3 = proto_register_protocol("Message Transfer Part Level 3",
@@ -619,14 +756,23 @@ proto_register_mtp3(void)
                                  "The structure of the pointcodes in ITU networks",
                                  &itu_pc_structure, itu_pc_structures, FALSE);
 
+  prefs_register_enum_preference(mtp3_module, "japan_pc_structure", "Japan Pointcode structure",
+                                 "The structure of the pointcodes in Japan networks",
+                                 &japan_pc_structure, japan_pc_structures, FALSE);
+
   prefs_register_bool_preference(mtp3_module, "ansi_5_bit_sls",
                                  "Use 5-bit SLS (ANSI only)",
                                  "Use 5-bit (instead of 8-bit) SLS in ANSI MTP3 packets",
                                  &mtp3_use_ansi_5_bit_sls);
 
-  prefs_register_enum_preference(mtp3_module, "net_addr_format", "Network Address Format",
-                                 "Format for point code in the network address columns",
-                                 &mtp3_net_addr_fmt, mtp3_net_addr_fmt_str_e, FALSE);
+  prefs_register_bool_preference(mtp3_module, "japan_5_bit_sls",
+                                 "Use 5-bit SLS (Japan only)",
+                                 "Use 5-bit (instead of 4-bit) SLS in Japan MTP3 packets",
+                                 &mtp3_use_japan_5_bit_sls);
+
+  prefs_register_enum_preference(mtp3_module, "addr_format", "Address Format",
+                                 "Format for point code in the address columns",
+                                 &mtp3_addr_fmt, mtp3_addr_fmt_str_e, FALSE);
 }
 
 void
