@@ -3,7 +3,7 @@
  * Copyright 2004, Anders Broman.
  * Copyright 2004, Olivier Biot.
  *
- * $Id: packet-multipart.c 13127 2005-01-20 01:26:34Z guy $
+ * $Id: packet-multipart.c 15686 2005-09-05 16:30:31Z etxrab $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -66,10 +66,7 @@
 #include <epan/prefs.h>
 #include <glib.h>
 #include <ctype.h>
-
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
+#include <epan/emem.h>
 
 #include <epan/packet.h>
 
@@ -90,8 +87,8 @@ static gint ett_multipart_body = -1;
 
 
 typedef struct {
-        char *name;
-        char *compact_name;
+        const char *name;
+        const char *compact_name;
 } multipart_header_t;
 
 static const multipart_header_t multipart_headers[] = {
@@ -559,11 +556,10 @@ process_body_part(proto_tree *tree, tvbuff_t *tvb, const guint8 *boundary,
 	while (line_len > 0)
 	{
 		gint colon_offset;
-		char *hdr_str = tvb_get_string(tvb, offset, next_offset - offset);
+		char *hdr_str = tvb_get_ephemeral_string(tvb, offset, next_offset - offset);
 		char *header_str;
 
 		header_str = unfold_and_compact_mime_header(hdr_str, &colon_offset);
-		g_free(hdr_str);
 		if (colon_offset <= 0) {
 			if (tree) {
 				proto_tree_add_text(subtree, tvb, offset, next_offset - offset,
@@ -604,7 +600,7 @@ process_body_part(proto_tree *tree, tvbuff_t *tvb, const guint8 *boundary,
 
 							if (semicolon_offset > 0) {
 								value_str[semicolon_offset] = '\0';
-								parameters = value_str + semicolon_offset + 1;
+								parameters = ep_strdup(value_str + semicolon_offset + 1);
 							} else {
 								parameters = NULL;
 							}
@@ -666,17 +662,21 @@ process_body_part(proto_tree *tree, tvbuff_t *tvb, const guint8 *boundary,
 				dissected = dissector_try_string(media_type_dissector_table,
 						content_type_str, tmp_tvb, pinfo, subtree);
 			}
+			if (! dissected) {
+				const char *save_match_string = pinfo->match_string;
+				pinfo->match_string = content_type_str;
+				call_dissector(media_handle, tmp_tvb, pinfo, subtree);
+				pinfo->match_string = save_match_string;
+			}
 			pinfo->private_data = save_private_data;
 			g_free(content_type_str);
 			content_type_str = NULL;
 			parameters = NULL; /* Shares same memory as content_type_str */
-			if (! dissected) {
-				call_dissector(media_handle, tmp_tvb, pinfo, tree);
-			}
 		} else {
 			call_dissector(data_handle, tmp_tvb, pinfo, subtree);
 		}
 		if (tree) {
+			proto_item_set_len(ti, boundary_start - start);
 			if (*last_boundary == TRUE) {
 				proto_tree_add_text(tree, tvb,
 						boundary_start, boundary_line_len,
