@@ -3,7 +3,7 @@
  * to when it had only NBNS)
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-nbns.c 12925 2005-01-02 09:40:26Z guy $
+ * $Id: packet-nbns.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,6 +33,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-dns.h"
 #include "packet-netbios.h"
 #include "packet-tcp.h"
@@ -213,8 +214,10 @@ static const value_string rcode_vals[] = {
 #define	NB_FLAGS_G		(1<<(15-0))	/* group name */
 
 /* Values for the "NAME_FLAGS" field of a NODE_NAME entry in T_NBSTAT
- * RR data.  From RFC 1001 and 1002, except for NAME_FLAGS_ONT_H_NODE,
- * which was discovered by looking at packet traces. */
+ * RR data.  From RFC 1001 and 1002; as I remember, the "NAME_FLAGS"
+ * field doesn't include any special values for H-mode nodes, even
+ * though one can register them (if so, perhaps that was done to
+ * avoid surprising clients that don't know about H-mode nodes). */
 #define	NAME_FLAGS_PRM		(1<<(15-6))	/* name is permanent node name */
 
 #define	NAME_FLAGS_ACT		(1<<(15-5))	/* name is active */
@@ -230,7 +233,7 @@ static const value_string rcode_vals[] = {
 
 #define	NAME_FLAGS_G		(1<<(15-0))	/* group name */
 
-static char *
+static const char *
 nbns_type_name (int type)
 {
 	switch (type) {
@@ -269,20 +272,23 @@ add_rr_to_tree(proto_item *trr, int rr_type, tvbuff_t *tvb, int offset,
 
 static int
 get_nbns_name(tvbuff_t *tvb, int offset, int nbns_data_offset,
-    char *name_ret, int *name_type_ret)
+    char *name_ret, int name_ret_len, int *name_type_ret)
 {
 	int name_len;
-	char name[MAXDNAME];
-	char nbname[NBNAME_BUF_LEN];
-	char *pname, *pnbname, cname, cnbname;
+	char *name;
+	char *nbname;
+	char *pname, cname, cnbname;
 	int name_type;
+	char *pname_ret;
+	size_t index = 0;
 
-	name_len = get_dns_name(tvb, offset, nbns_data_offset, name,
-	    sizeof(name));
+	nbname=ep_alloc(NBNAME_BUF_LEN);
+	name_len = get_dns_name(tvb, offset, nbns_data_offset, &name);
 
 	/* OK, now undo the first-level encoding. */
 	pname = &name[0];
-	pnbname = &nbname[0];
+	pname_ret=name_ret;
+
 	for (;;) {
 		/* Every two characters of the first level-encoded name
 		 * turn into one character in the decoded name. */
@@ -293,8 +299,7 @@ get_nbns_name(tvbuff_t *tvb, int offset, int nbns_data_offset,
 			break;		/* scope ID follows */
 		if (cname < 'A' || cname > 'Z') {
 			/* Not legal. */
-			strcpy(nbname,
-			    "Illegal NetBIOS name (character not between A and Z in first-level encoding)");
+			nbname="Illegal NetBIOS name (1st character not between A and Z in first-level encoding)";
 			goto bad;
 		}
 		cname -= 'A';
@@ -305,14 +310,12 @@ get_nbns_name(tvbuff_t *tvb, int offset, int nbns_data_offset,
 		if (cname == '\0' || cname == '.') {
 			/* No more characters in the name - but we're in
 			 * the middle of a pair.  Not legal. */
-			strcpy(nbname,
-			    "Illegal NetBIOS name (odd number of bytes)");
+			nbname="Illegal NetBIOS name (odd number of bytes)";
 			goto bad;
 		}
 		if (cname < 'A' || cname > 'Z') {
 			/* Not legal. */
-			strcpy(nbname,
-			    "Illegal NetBIOS name (character not between A and Z in first-level encoding)");
+			nbname="Illegal NetBIOS name (2nd character not between A and Z in first-level encoding)";
 			goto bad;
 		}
 		cname -= 'A';
@@ -320,33 +323,29 @@ get_nbns_name(tvbuff_t *tvb, int offset, int nbns_data_offset,
 		pname++;
 
 		/* Do we have room to store the character? */
-		if (pnbname < &nbname[NETBIOS_NAME_LEN]) {
+		if (index < NETBIOS_NAME_LEN) {
 			/* Yes - store the character. */
-			*pnbname = cnbname;
+			nbname[index++] = cnbname;
 		}
-
-		/* We bump the pointer even if it's past the end of the
-		   name, so we keep track of how long the name is. */
-		pnbname++;
 	}
 
 	/* NetBIOS names are supposed to be exactly 16 bytes long. */
-	if (pnbname - nbname != NETBIOS_NAME_LEN) {
+	if (index != NETBIOS_NAME_LEN) {
 		/* It's not. */
-		sprintf(nbname, "Illegal NetBIOS name (%ld bytes long)",
-		    (long)(pnbname - nbname));
+		g_snprintf(nbname, NBNAME_BUF_LEN, "Illegal NetBIOS name (%lu bytes long)",
+		    (unsigned long)index);
 		goto bad;
 	}
 
 	/* This one is; make its name printable. */
-	name_type = process_netbios_name(nbname, name_ret);
-	name_ret += strlen(name_ret);
-	sprintf(name_ret, "<%02x>", name_type);
-	name_ret += 4;
+	name_type = process_netbios_name(nbname, name_ret, name_ret_len);
+	pname_ret += MIN(strlen(name_ret), name_ret_len);
+	pname_ret += MIN(name_ret_len-(pname_ret-name_ret),
+		g_snprintf(pname_ret, name_ret_len-(pname_ret-name_ret), "<%02x>", name_type));
 	if (cname == '.') {
 		/* We have a scope ID, starting at "pname"; append that to
 		 * the decoded host name. */
-		strcpy(name_ret, pname);
+		g_snprintf(pname_ret, name_ret_len-(pname_ret-name_ret), "%s", pname);
 	}
 	if (name_type_ret != NULL)
 		*name_type_ret = name_type;
@@ -355,7 +354,9 @@ get_nbns_name(tvbuff_t *tvb, int offset, int nbns_data_offset,
 bad:
 	if (name_type_ret != NULL)
 		*name_type_ret = -1;
-	strcpy (name_ret, nbname);
+	/* This is only valid because nbname is always assigned an error string
+	 * before jumping to bad: Otherwise nbname wouldn't be \0 terminated */
+	g_snprintf(pname_ret, name_ret_len-(pname_ret-name_ret), "%s", nbname);
 	return name_len;
 }
 
@@ -369,8 +370,8 @@ get_nbns_name_type_class(tvbuff_t *tvb, int offset, int nbns_data_offset,
 	int type;
 	int class;
 
-	name_len = get_nbns_name(tvb, offset, nbns_data_offset, name_ret,
-	   name_type_ret);
+	name_len = get_nbns_name(tvb, offset, nbns_data_offset, name_ret, 
+			*name_len_ret, name_type_ret);
 	offset += name_len;
 
 	type = tvb_get_ntohs(tvb, offset);
@@ -387,7 +388,7 @@ get_nbns_name_type_class(tvbuff_t *tvb, int offset, int nbns_data_offset,
 
 static void
 add_name_and_type(proto_tree *tree, tvbuff_t *tvb, int offset, int len,
-    char *tag, char *name, int name_type)
+    const char *tag, const char *name, int name_type)
 {
 	if (name_type != -1) {
 		proto_tree_add_text(tree, tvb, offset, len, "%s: %s (%s)",
@@ -398,24 +399,28 @@ add_name_and_type(proto_tree *tree, tvbuff_t *tvb, int offset, int len,
 	}
 }
 
+#define MAX_NAME_LEN (NETBIOS_NAME_LEN - 1)*4 + MAXDNAME + 64
+
 static int
 dissect_nbns_query(tvbuff_t *tvb, int offset, int nbns_data_offset,
     column_info *cinfo, proto_tree *nbns_tree)
 {
 	int len;
-	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
+	char *name;
 	int name_len;
 	int name_type;
 	int type;
 	int class;
-	char *type_name;
+	const char *type_name;
 	int data_offset;
 	int data_start;
 	proto_tree *q_tree;
 	proto_item *tq;
 
+	name=ep_alloc(MAX_NAME_LEN);
 	data_start = data_offset = offset;
 
+	name_len=MAX_NAME_LEN;
 	len = get_nbns_name_type_class(tvb, offset, nbns_data_offset, name,
 	    &name_len, &name_type, &type, &class);
 	data_offset += len;
@@ -449,13 +454,15 @@ static void
 nbns_add_nbns_flags(column_info *cinfo, proto_tree *nbns_tree, tvbuff_t *tvb, int offset,
 		    gushort flags, int is_wack)
 {
-	char buf[128+1];
+	char *buf;
 	guint16 opcode;
 	proto_tree *field_tree;
 	proto_item *tf;
 
+#define MAX_BUF_SIZE (128+1)
+	buf=ep_alloc(MAX_BUF_SIZE);
 	opcode = (guint16) ((flags & F_OPCODE) >> OPCODE_SHIFT);
-	strcpy(buf, val_to_str(opcode, opcode_vals, "Unknown operation"));
+	g_snprintf(buf, MAX_BUF_SIZE, "%s", val_to_str(opcode, opcode_vals, "Unknown operation"));
 	if (flags & F_RESPONSE && !is_wack) {
 		strcat(buf, " response");
 		strcat(buf, ", ");
@@ -497,7 +504,7 @@ nbns_add_nbns_flags(column_info *cinfo, proto_tree *nbns_tree, tvbuff_t *tvb, in
 static void
 nbns_add_nb_flags(proto_tree *rr_tree, tvbuff_t *tvb, int offset, gushort flags)
 {
-	char buf[128+1];
+	char *buf;
 	proto_tree *field_tree;
 	proto_item *tf;
 	static const value_string nb_flags_ont_vals[] = {
@@ -508,7 +515,8 @@ nbns_add_nb_flags(proto_tree *rr_tree, tvbuff_t *tvb, int offset, gushort flags)
 		  { 0,                   NULL     }
 	};
 
-	strcpy(buf, val_to_str(flags & NB_FLAGS_ONT, nb_flags_ont_vals,
+	buf=ep_alloc(MAX_BUF_SIZE);
+	g_snprintf(buf, MAX_BUF_SIZE, "%s", val_to_str(flags & NB_FLAGS_ONT, nb_flags_ont_vals,
 	    "Unknown"));
 	strcat(buf, ", ");
 	if (flags & NB_FLAGS_G)
@@ -532,7 +540,7 @@ static void
 nbns_add_name_flags(proto_tree *rr_tree, tvbuff_t *tvb, int offset,
     gushort flags)
 {
-	char buf[128+1];
+	char *buf;
 	proto_item *field_tree;
 	proto_item *tf;
 	static const value_string name_flags_ont_vals[] = {
@@ -542,7 +550,8 @@ nbns_add_name_flags(proto_tree *rr_tree, tvbuff_t *tvb, int offset,
 		  { 0,                     NULL     }
 	};
 
-	strcpy(buf, val_to_str(flags & NAME_FLAGS_ONT, name_flags_ont_vals,
+	buf=ep_alloc(MAX_BUF_SIZE);
+	g_snprintf(buf, MAX_BUF_SIZE, "%s", val_to_str(flags & NAME_FLAGS_ONT, name_flags_ont_vals,
 	    "Unknown"));
 	strcat(buf, ", ");
 	if (flags & NAME_FLAGS_G)
@@ -595,13 +604,13 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
     column_info *cinfo, proto_tree *nbns_tree, int opcode)
 {
 	int len;
-	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME + 64];
+	char *name;
 	int name_len;
 	int name_type;
 	int type;
 	int class;
-	char *class_name;
-	char *type_name;
+	const char *class_name;
+	const char *type_name;
 	int data_offset;
 	int cur_offset;
 	int data_start;
@@ -610,14 +619,19 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 	gushort flags;
 	proto_tree *rr_tree;
 	proto_item *trr;
-	char name_str[(NETBIOS_NAME_LEN - 1)*4 + 1];
+	char *name_str;
 	guint num_names;
-	char nbname[16+4+1];	/* 4 for [<last char>] */
+	char *nbname;	
 	gushort name_flags;
 
 	data_start = data_offset = offset;
 	cur_offset = offset;
 
+	name=ep_alloc(MAX_NAME_LEN);
+	name_str=ep_alloc(MAX_NAME_LEN);
+	nbname=ep_alloc(16+4+1);	/* 4 for [<last char>] */
+
+	name_len=MAX_NAME_LEN;
 	len = get_nbns_name_type_class(tvb, offset, nbns_data_offset, name,
 	    &name_len, &name_type, &type, &class);
 	data_offset += len;
@@ -725,7 +739,7 @@ dissect_nbns_answer(tvbuff_t *tvb, int offset, int nbns_data_offset,
 			tvb_memcpy(tvb, (guint8 *)nbname, cur_offset,
 			    NETBIOS_NAME_LEN);
 			name_type = process_netbios_name(nbname,
-			    name_str);
+			    name_str, name_len);
 			proto_tree_add_text(rr_tree, tvb, cur_offset,
 			    NETBIOS_NAME_LEN, "Name: %s<%02x> (%s)",
 			    name_str, name_type,
@@ -987,7 +1001,7 @@ dissect_query_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
 static int
 dissect_answer_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
     int count, column_info *cinfo, proto_tree *nbns_tree, int opcode,
-    char *name)
+    const char *name)
 {
 	int start_off, add_off;
 	proto_tree *qatree = NULL;
@@ -1189,10 +1203,12 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		{ 0x00,	NULL }
 	};
 
-	char name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
+	char *name;
 	int name_type;
 	int len;
 
+	name=ep_alloc(MAX_NAME_LEN);
+
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBDS");
 	if (check_col(pinfo->cinfo, COL_INFO))
@@ -1206,7 +1222,7 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	header.flags.node_type = (flags & 12) >> 2;
 
 	header.dgm_id = tvb_get_ntohs(tvb, offset+2);
-	tvb_memcpy(tvb, (guint8 *)&header.src_ip, offset+4, 4);
+	header.src_ip = tvb_get_ipv4(tvb, offset+4);
 	header.src_port = tvb_get_ntohs(tvb, offset+8);
 
 	switch (header.msg_type) {
@@ -1278,7 +1294,7 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset += 4;
 
 		/* Source name */
-		len = get_nbns_name(tvb, offset, offset, name, &name_type);
+		len = get_nbns_name(tvb, offset, offset, name, MAX_NAME_LEN, &name_type);
 
 		if (tree) {
 			add_name_and_type(nbdgm_tree, tvb, offset, len,
@@ -1287,7 +1303,7 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		offset += len;
 
 		/* Destination name */
-		len = get_nbns_name(tvb, offset, offset, name, &name_type);
+		len = get_nbns_name(tvb, offset, offset, name, MAX_NAME_LEN, &name_type);
 
 		if (tree) {
 			add_name_and_type(nbdgm_tree, tvb, offset, len,
@@ -1320,7 +1336,7 @@ dissect_nbdgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	case NBDS_POS_QUERY_RESPONSE:
 	case NBDS_NEG_QUERY_RESPONSE:
 		/* Destination name */
-		len = get_nbns_name(tvb, offset, offset, name, &name_type);
+		len = get_nbns_name(tvb, offset, offset, name, MAX_NAME_LEN, &name_type);
 
 		if (tree) {
 			add_name_and_type(nbdgm_tree, tvb, offset, len,
@@ -1393,12 +1409,14 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	volatile int	length;
 	int		length_remaining;
 	int		len;
-	char		name[(NETBIOS_NAME_LEN - 1)*4 + MAXDNAME];
+	char		*name;
 	int		name_type;
 	gint		reported_len;
 	tvbuff_t	*next_tvb;
 	const char	*saved_proto;
 
+	name=ep_alloc(MAX_NAME_LEN);
+
 	/* Desegmentation */
 	length_remaining = tvb_length_remaining(tvb, offset);
 
@@ -1436,7 +1454,14 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	 * somewhere in the middle of a segment.
 	 */
 	if(!pinfo->fd->flags.visited){
-		if((length+4)>tvb_reported_length_remaining(tvb, offset)){
+		/* 'Only' SMB is transported ontop of this  so make sure
+		 * there is an SMB header there ...
+		 */
+		if( ((length+4)>tvb_reported_length_remaining(tvb, offset))
+		  &&(tvb_length_remaining(tvb, offset)>=8)
+		  &&(tvb_get_guint8(tvb,offset+5)=='S')
+		  &&(tvb_get_guint8(tvb,offset+6)=='M')
+		  &&(tvb_get_guint8(tvb,offset+7)=='B') ){
 			pinfo->want_pdu_tracking=2;
 			pinfo->bytes_until_next_pdu=(length+4)-tvb_reported_length_remaining(tvb, offset);
 		}
@@ -1499,7 +1524,7 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	switch (msg_type) {
 
 	case SESSION_REQUEST:
-	  len = get_nbns_name(tvb, offset, offset, name, &name_type);
+	  len = get_nbns_name(tvb, offset, offset, name, MAX_NAME_LEN, &name_type);
 	  if (tree)
 	    add_name_and_type(nbss_tree, tvb, offset, len,
 				"Called name", name, name_type);
@@ -1508,7 +1533,7 @@ dissect_nbss_packet(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	  if (check_col(pinfo->cinfo, COL_INFO))
 		  col_append_fstr(pinfo->cinfo, COL_INFO, ", to %s ", name);
 
-	  len = get_nbns_name(tvb, offset, offset, name, &name_type);
+	  len = get_nbns_name(tvb, offset, offset, name, MAX_NAME_LEN, &name_type);
 
 	  if (tree)
 	    add_name_and_type(nbss_tree, tvb, offset, len,
