@@ -5,7 +5,7 @@
  *
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-netbios.c 17065 2006-01-20 21:18:18Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include "llcsaps.h"
+#include <epan/llcsaps.h>
 #include <epan/reassemble.h>
 #include <epan/prefs.h>
 #include "packet-netbios.h"
@@ -281,7 +281,7 @@ void capture_netbios(packet_counts *ld)
 
 
 int
-process_netbios_name(const guchar *name_ptr, char *name_ret)
+process_netbios_name(const guchar *name_ptr, char *name_ret, int name_ret_len)
 {
 	int i;
 	int name_type = *(name_ptr + NETBIOS_NAME_LEN - 1);
@@ -290,15 +290,20 @@ process_netbios_name(const guchar *name_ptr, char *name_ret)
 
 	for (i = 0; i < NETBIOS_NAME_LEN - 1; i++) {
 		name_char = *name_ptr++;
-		if (name_char >= ' ' && name_char <= '~')
-			*name_ret++ = name_char;
-		else {
+		if (name_char >= ' ' && name_char <= '~') {
+			if (--name_ret_len > 0)
+				*name_ret++ = name_char;
+		} else {
 			/* It's not printable; show it as <XX>, where
 			   XX is the value in hex. */
-			*name_ret++ = '<';
-			*name_ret++ = hex_digits[(name_char >> 4)];
-			*name_ret++ = hex_digits[(name_char & 0x0F)];
-			*name_ret++ = '>';
+			if (--name_ret_len > 0)
+				*name_ret++ = '<';
+			if (--name_ret_len > 0)
+				*name_ret++ = hex_digits[(name_char >> 4)];
+			if (--name_ret_len > 0)
+				*name_ret++ = hex_digits[(name_char & 0x0F)];
+			if (--name_ret_len > 0)
+				*name_ret++ = '>';
 		}
 	}
 	*name_ret = '\0';
@@ -319,25 +324,25 @@ process_netbios_name(const guchar *name_ptr, char *name_ret)
 }
 
 
-int get_netbios_name( tvbuff_t *tvb, int offset, char *name_ret)
+int get_netbios_name( tvbuff_t *tvb, int offset, char *name_ret, int name_ret_len)
 
 {/*  Extract the name string and name type.  Return the name string in  */
  /* name_ret and return the name_type. */
 
-	return process_netbios_name( tvb_get_ptr( tvb, offset, NETBIOS_NAME_LEN ), name_ret);
+	return process_netbios_name( tvb_get_ptr( tvb, offset, NETBIOS_NAME_LEN ), name_ret, name_ret_len);
 }
 
 
 /*
  * Get a string describing the type of a NetBIOS name.
  */
-char *
+const char *
 netbios_name_type_descr(int name_type)
 {
 	return val_to_str(name_type, nb_name_type_vals, "Unknown");
 }
 
-void netbios_add_name(char* label, tvbuff_t *tvb, int offset,
+void netbios_add_name(const char* label, tvbuff_t *tvb, int offset,
     proto_tree *tree)
 
 {/* add a name field display tree. Display the name and station type in sub-tree */
@@ -346,10 +351,10 @@ void netbios_add_name(char* label, tvbuff_t *tvb, int offset,
 	proto_item *tf;
 	char  name_str[(NETBIOS_NAME_LEN - 1)*4 + 1];
 	int   name_type;
-	char  *name_type_str;
+	const char  *name_type_str;
 
 					/* decode the name field */
-	name_type = get_netbios_name( tvb, offset, name_str);
+	name_type = get_netbios_name( tvb, offset, name_str, (NETBIOS_NAME_LEN - 1)*4 + 1);
 	name_type_str = netbios_name_type_descr(name_type);
 	tf = proto_tree_add_text( tree, tvb, offset, NETBIOS_NAME_LEN,
 	    	"%s: %s<%02x> (%s)", label, name_str, name_type, name_type_str);
@@ -544,7 +549,7 @@ static void nb_data2(int hf, tvbuff_t *tvb, int offset, proto_tree *tree)
 }
 
 
-static void nb_resync_indicator( tvbuff_t *tvb, int offset, proto_tree *tree, char *cmd_str)
+static void nb_resync_indicator( tvbuff_t *tvb, int offset, proto_tree *tree, const char *cmd_str)
 {
 	guint16 resync_indicator = tvb_get_letohs( tvb, offset + NB_DATA2);
 
@@ -1090,7 +1095,7 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_tree		*netb_tree = NULL;
 	proto_item		*ti;
 	guint16			hdr_len, command;
-	char			*command_name;
+	const char		*command_name;
 	char 			name[(NETBIOS_NAME_LEN - 1)*4 + 1];
 	int			name_type;
 	guint16			session_id;
@@ -1130,7 +1135,7 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		command_name = val_to_str(command, cmd_vals, "Unknown (0x%02x)");
                 switch ( command ) {
                 case NB_NAME_QUERY:
-                        name_type = get_netbios_name( tvb, offset + 12, name);
+                        name_type = get_netbios_name( tvb, offset + 12, name, (NETBIOS_NAME_LEN - 1)*4 + 1);
                         col_add_fstr( pinfo->cinfo, COL_INFO, "%s for %s<%02x>",
                             command_name, name, name_type);
                         break;
@@ -1138,7 +1143,7 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 case NB_NAME_RESP:
                 case NB_ADD_NAME:
                 case NB_ADD_GROUP:
-                        name_type = get_netbios_name( tvb, offset + 28, name);
+                        name_type = get_netbios_name( tvb, offset + 28, name, (NETBIOS_NAME_LEN - 1)*4 + 1);
                         col_add_fstr( pinfo->cinfo, COL_INFO, "%s - %s<%02x>",
                             command_name, name, name_type);
                         break;
@@ -1211,10 +1216,12 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 						    "Reassembled NetBIOS");
 						/* Show all fragments. */
 						if (tree) {
+							proto_item *frag_tree_item;
+
 							show_fragment_seq_tree(fd_head,
 							    &netbios_frag_items,
 							    netb_tree, pinfo,
-							    next_tvb);
+							    next_tvb, &frag_tree_item);
 						}
 					} else {
 						next_tvb = tvb_new_subset(tvb,
