@@ -47,7 +47,7 @@
  ** http://www.cisco.com/univercd/cc/td/doc/cisintwk/intsolns/netflsol/nfwhite.htm
  **
  ** $Yahoo: //depot/fumerola/packet-netflow/packet-netflow.c#14 $
- ** $Id: packet-netflow.c 12115 2004-09-27 22:55:15Z guy $
+ ** $Id: packet-netflow.c 17698 2006-03-22 17:19:56Z etxrab $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -158,6 +158,7 @@ static int      ett_netflow = -1;
 static int      ett_unixtime = -1;
 static int      ett_flow = -1;
 static int      ett_template = -1;
+static int      ett_field = -1;
 static int      ett_dataflowset = -1;
 
 /*
@@ -244,6 +245,36 @@ static int      hf_cflow_sampling_interval = -1;
 static int      hf_cflow_sampling_algorithm = -1;
 static int      hf_cflow_flow_active_timeout = -1;
 static int      hf_cflow_flow_inactive_timeout = -1;
+static int      hf_cflow_mpls_top_label_type = -1;
+static int      hf_cflow_mpls_pe_addr = -1;
+const value_string special_mpls_top_label_type[] = {
+	{0,	"Unknown"},
+	{1,	"TE-MIDPT"},
+	{2,	"ATOM"},
+	{3,	"VPN"},
+	{4,	"BGP"},
+	{5,	"LDP"},
+
+	{0,	NULL }
+};
+
+void
+proto_tree_add_mpls_label(proto_tree * pdutree, tvbuff_t * tvb, int offset, int length, int level)
+{
+	if( length == 3) {
+		guint8 b0 = tvb_get_guint8(tvb, offset);
+		guint8 b1 = tvb_get_guint8(tvb, offset + 1);
+		guint8 b2 = tvb_get_guint8(tvb, offset + 2);
+		proto_tree_add_text(pdutree, tvb, offset, length,
+			"MPLS-Label%d: %u exp-bits: %u %s", level,
+			((b0<<12)+(b1<<4)+(b2>>4)), 
+			((b2>>1)&0x7), 
+			((b2&0x1)?"top-of-stack":""));
+	} else {
+		proto_tree_add_text(pdutree, tvb, offset, length,
+			"MPLS-Label%d: bad lengh %d", level, length);
+	}
+}
 
 void		proto_reg_handoff_netflow(void);
 
@@ -264,7 +295,7 @@ static void	dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb,
 static int	dissect_v9_options(proto_tree * pdutree, tvbuff_t * tvb,
 			       int offset);
 static int	dissect_v9_template(proto_tree * pdutree, tvbuff_t * tvb,
-				    int offset);
+				    int offset, int len);
 static void	v9_template_add(struct v9_template * template);
 static struct v9_template *v9_template_get(guint16 id, guint32 src_addr,
 					   guint32 src_id);
@@ -357,10 +388,12 @@ dissect_netflow(tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	if (check_col(pinfo->cinfo, COL_INFO)) {
 		if (ver == 9) {
 			col_add_fstr(pinfo->cinfo, COL_INFO,
-			    "total: %u (v%u) FlowSets", pdus, ver);
+			    "total: %u (v%u) record%s", pdus, ver,
+                            plurality(pdus, "", "s"));
 		} else {
 			col_add_fstr(pinfo->cinfo, COL_INFO,
-			    "total: %u (v%u) flows", pdus, ver);
+			    "total: %u (v%u) flow%s", pdus, ver,
+                            plurality(pdus, "", "s"));
 		}
 	}
 
@@ -376,10 +409,18 @@ dissect_netflow(tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	offset += 4;
 
 	ts.secs = tvb_get_ntohl(tvb, offset);
-	ts.nsecs = tvb_get_ntohl(tvb, offset + 4);
-	timeitem = proto_tree_add_time(netflow_tree,
-				       hf_cflow_timestamp, tvb, offset,
-				       8, &ts);
+	if (ver != 9) {
+	  ts.nsecs = tvb_get_ntohl(tvb, offset + 4);
+	  timeitem = proto_tree_add_time(netflow_tree,
+					 hf_cflow_timestamp, tvb, offset,
+					 8, &ts);
+        } else {
+	  ts.nsecs = 0;
+	  timeitem = proto_tree_add_time(netflow_tree,
+					 hf_cflow_timestamp, tvb, offset,
+					 4, &ts);
+        }
+
 	timetree = proto_item_add_subtree(timeitem, ett_unixtime);
 
 	proto_tree_add_item(timetree, hf_cflow_unix_secs, tvb,
@@ -497,7 +538,7 @@ dissect_netflow(tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 
 		if (ver == 9) {
 			pduitem = proto_tree_add_text(netflow_tree, tvb,
-			    offset, pdusize, "FlowSet %u/%u", x, pdus);
+			    offset, pdusize, "FlowSet %u", x);
 		} else {
 			pduitem = proto_tree_add_text(netflow_tree, tvb,
 			    offset, pdusize, "pdu %u/%u", x, pdus);
@@ -551,14 +592,17 @@ static int
 flow_process_timeperiod(proto_tree * pdutree, tvbuff_t * tvb, int offset)
 {
 	nstime_t        ts;
+	guint32         msec;
 
-	ts.secs = tvb_get_ntohl(tvb, offset) / 1000;
-	ts.nsecs = ((tvb_get_ntohl(tvb, offset) % 1000) * 1000000);
+	msec = tvb_get_ntohl(tvb, offset);
+	ts.secs = msec / 1000;
+	ts.nsecs = (msec % 1000) * 1000000;
 	proto_tree_add_time(pdutree, hf_cflow_timestart, tvb, offset, 4, &ts);
 	offset += 4;
 
-	ts.secs = tvb_get_ntohl(tvb, offset) / 1000;
-	ts.nsecs = ((tvb_get_ntohl(tvb, offset) % 1000) * 1000000);
+	msec = tvb_get_ntohl(tvb, offset);
+	ts.secs = msec / 1000;
+	ts.nsecs = (msec % 1000) * 1000000;
 	proto_tree_add_time(pdutree, hf_cflow_timeend, tvb, offset, 4, &ts);
 	offset += 4;
 
@@ -830,7 +874,7 @@ dissect_v9_flowset(proto_tree * pdutree, tvbuff_t * tvb, int offset, int ver)
 		    offset, 2, FALSE);
 		offset += 2;
 
-		dissect_v9_template(pdutree, tvb, offset);
+		dissect_v9_template(pdutree, tvb, offset, length - 4);
 	} else if (flowset_id == 1) {
 		/* Options */
 		proto_tree_add_item(pdutree, hf_cflow_options_flowset_id, tvb,
@@ -923,10 +967,9 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 	int i;
 
 	for (i = 0; i < template->count; i++) {
-		guint32	ipv4addr;
-		guint8	ipv6addr[16];
 		guint16 type, length;
 		nstime_t ts;
+		guint32 msec;
 
 		type = template->entries[i].type;
 		length = template->entries[i].length;
@@ -993,15 +1036,11 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 
 		case 8: /* source IP */
 			if (length == 4) {
-				tvb_memcpy(tvb, (guint8 *)&ipv4addr, offset,
-				    sizeof(ipv4addr));
-				proto_tree_add_ipv4(pdutree, hf_cflow_srcaddr,
-				    tvb, offset, length, ipv4addr);
+				proto_tree_add_item(pdutree, hf_cflow_srcaddr,
+				    tvb, offset, length, FALSE);
 			} else if (length == 16) {
-				tvb_memcpy(tvb, ipv6addr, offset,
-				    sizeof(ipv6addr));
-				proto_tree_add_ipv6(pdutree, hf_cflow_srcaddr_v6,
-				    tvb, offset, length, ipv6addr);
+				proto_tree_add_item(pdutree, hf_cflow_srcaddr_v6,
+				    tvb, offset, length, FALSE);
 			} else {
 				proto_tree_add_text(pdutree,
 				    tvb, offset, length,
@@ -1026,15 +1065,11 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 
 		case 12: /* dest IP */
 			if (length == 4) {
-				tvb_memcpy(tvb, (guint8 *)&ipv4addr, offset,
-				    sizeof(ipv4addr));
-				proto_tree_add_ipv4(pdutree, hf_cflow_dstaddr,
-				    tvb, offset, length, ipv4addr);
+				proto_tree_add_item(pdutree, hf_cflow_dstaddr,
+				    tvb, offset, length, FALSE);
 			} else if (length == 16) {
-				tvb_memcpy(tvb, ipv6addr, offset,
-				    sizeof(ipv6addr));
-				proto_tree_add_ipv6(pdutree, hf_cflow_dstaddr_v6,
-				    tvb, offset, length, ipv6addr);
+				proto_tree_add_item(pdutree, hf_cflow_dstaddr_v6,
+				    tvb, offset, length, FALSE);
 			} else {
 				proto_tree_add_text(pdutree,
 				    tvb, offset, length,
@@ -1054,15 +1089,11 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 
 		case 15: /* nexthop IP */
 			if (length == 4) {
-				tvb_memcpy(tvb, (guint8 *)&ipv4addr, offset,
-				    sizeof(ipv4addr));
-				proto_tree_add_ipv4(pdutree, hf_cflow_nexthop,
-				    tvb, offset, length, ipv4addr);
+				proto_tree_add_item(pdutree, hf_cflow_nexthop,
+				    tvb, offset, length, FALSE);
 			} else if (length == 16) {
-				tvb_memcpy(tvb, ipv6addr, offset,
-				    sizeof(ipv6addr));
-				proto_tree_add_ipv6(pdutree, hf_cflow_nexthop_v6,
-				    tvb, offset, length, ipv6addr);
+				proto_tree_add_item(pdutree, hf_cflow_nexthop_v6,
+				    tvb, offset, length, FALSE);
 			} else {
 				proto_tree_add_text(pdutree,
 				    tvb, offset, length,
@@ -1082,15 +1113,11 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 
 		case 18: /* BGP nexthop IP */
 			if (length == 4) {
-				tvb_memcpy(tvb, (guint8 *)&ipv4addr, offset,
-				    sizeof(ipv4addr));
-				proto_tree_add_ipv4(pdutree, hf_cflow_bgpnexthop,
-				    tvb, offset, length, ipv4addr);
+				proto_tree_add_item(pdutree, hf_cflow_bgpnexthop,
+				    tvb, offset, length, FALSE);
 			} else if (length == 16) {
-				tvb_memcpy(tvb, ipv6addr, offset,
-				    sizeof(ipv6addr));
-				proto_tree_add_ipv6(pdutree, hf_cflow_bgpnexthop_v6,
-				    tvb, offset, length, ipv6addr);
+				proto_tree_add_item(pdutree, hf_cflow_bgpnexthop_v6,
+				    tvb, offset, length, FALSE);
 			} else {
 				proto_tree_add_text(pdutree,
 				    tvb, offset, length,
@@ -1109,15 +1136,17 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 			break;
 
 		case 21: /* last switched */
-			ts.secs = tvb_get_ntohl(tvb, offset) / 1000;
-			ts.nsecs = 0;
+		        msec = tvb_get_ntohl(tvb, offset);
+			ts.secs = msec / 1000;
+			ts.nsecs = (msec % 1000) * 1000000;
 			proto_tree_add_time(pdutree, hf_cflow_timeend,
 			    tvb, offset, length, &ts);
 			break;
 
 		case 22: /* first switched */
-			ts.secs = tvb_get_ntohl(tvb, offset) / 1000;
-			ts.nsecs = 0;
+		        msec = tvb_get_ntohl(tvb, offset);
+			ts.secs = msec / 1000;
+			ts.nsecs = (msec % 1000) * 1000000;
 			proto_tree_add_time(pdutree, hf_cflow_timestart,
 			    tvb, offset, length, &ts);
 			break;
@@ -1157,6 +1186,26 @@ dissect_v9_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset,
 			    tvb, offset, length, FALSE);
 			break;
 
+		case 46: /* top MPLS label type*/ 
+			proto_tree_add_item(pdutree, hf_cflow_mpls_top_label_type,
+			    tvb, offset, length, FALSE);
+			break;
+		case 47: /* top MPLS label PE address*/ 
+			proto_tree_add_item(pdutree, hf_cflow_mpls_pe_addr,
+			    tvb, offset, length, FALSE);
+			break;
+		case 70: /* MPLS label1*/ 
+			proto_tree_add_mpls_label(pdutree, tvb, offset, length, 1);
+			break;
+		case 71: /* MPLS label2*/ 
+			proto_tree_add_mpls_label(pdutree, tvb, offset, length, 2);
+			break;
+		case 72: /* MPLS label3*/ 
+			proto_tree_add_mpls_label(pdutree, tvb, offset, length, 3);
+			break;
+		case 73: /* MPLS label4*/ 
+			proto_tree_add_mpls_label(pdutree, tvb, offset, length, 4);
+			break;
 		default:
 			proto_tree_add_text(pdutree, tvb, offset, length,
 			    "Type %u", type);
@@ -1232,53 +1281,66 @@ dissect_v9_options(proto_tree * pdutree, tvbuff_t * tvb, int offset)
 }
 
 static int
-dissect_v9_template(proto_tree * pdutree, tvbuff_t * tvb, int offset)
+dissect_v9_template(proto_tree * pdutree, tvbuff_t * tvb, int offset, int len)
 {
 	struct v9_template template;
 	proto_tree *template_tree;
 	proto_item *template_item;
+	proto_tree *field_tree;
+	proto_item *field_item;
 	guint16 id, count;
+	int remaining = len;
 	gint32 i;
 
-	id = tvb_get_ntohs(tvb, offset);
-	proto_tree_add_item(pdutree, hf_cflow_template_id, tvb,
-	    offset, 2, FALSE);
-	offset += 2;
+	while (remaining > 0) {
 
-	count = tvb_get_ntohs(tvb, offset);
-	proto_tree_add_item(pdutree, hf_cflow_template_field_count, tvb,
-	    offset, 2, FALSE);
-	offset += 2;
+	  id = tvb_get_ntohs(tvb, offset);
+	  count = tvb_get_ntohs(tvb, offset + 2);
 
-	/* Cache template */
-	memset(&template, 0, sizeof(template));
-	template.id = id;
-	template.count = count;
-	template.source_addr = 0;	/* XXX */
-	template.source_id = 0;		/* XXX */
-	template.option_template = 0;   /* Data template */
-	template.entries = g_malloc(count * sizeof(struct v9_template_entry));
-	tvb_memcpy(tvb, (guint8 *)template.entries, offset,
-	    count * sizeof(struct v9_template_entry));
-	v9_template_add(&template);
+	  template_item = proto_tree_add_text(pdutree, tvb, offset, 
+	      4 + sizeof(struct v9_template_entry) * count, 
+	      "Template (Id = %u, Count = %u)", id, count);
+	  template_tree = proto_item_add_subtree(template_item, ett_template);
 
-	for (i = 1; i <= count; i++) {
-		guint16 type, length;
+	  proto_tree_add_item(template_tree, hf_cflow_template_id, tvb,
+	      offset, 2, FALSE);
+	  offset += 2;
 
-		type = tvb_get_ntohs(tvb, offset);
-		length = tvb_get_ntohs(tvb, offset + 2);
+	  proto_tree_add_item(template_tree, hf_cflow_template_field_count, 
+	      tvb, offset, 2, FALSE);
+	  offset += 2;
 
-		template_item = proto_tree_add_text(pdutree, tvb,
-		    offset, 4, "Field (%u/%u)", i, count);
-		template_tree = proto_item_add_subtree(template_item, ett_template);
+	  /* Cache template */
+	  memset(&template, 0, sizeof(template));
+	  template.id = id;
+	  template.count = count;
+	  template.source_addr = 0;	/* XXX */
+	  template.source_id = 0;		/* XXX */
+	  template.option_template = 0;   /* Data template */
+	  template.entries = g_malloc(count * sizeof(struct v9_template_entry));
+	  tvb_memcpy(tvb, (guint8 *)template.entries, offset,
+	      count * sizeof(struct v9_template_entry));
+	  v9_template_add(&template);
 
-		proto_tree_add_item(template_tree,
-		    hf_cflow_template_field_type, tvb, offset, 2, FALSE);
-		offset += 2;
+	  for (i = 1; i <= count; i++) {
+	    guint16 type, length;
 
-		proto_tree_add_item(template_tree,
-		    hf_cflow_template_field_length, tvb, offset, 2, FALSE);
-		offset += 2;
+	    type = tvb_get_ntohs(tvb, offset);
+	    length = tvb_get_ntohs(tvb, offset + 2);
+
+	    field_item = proto_tree_add_text(template_tree, tvb,
+		offset, 4, "Field (%u/%u)", i, count);
+	    field_tree = proto_item_add_subtree(field_item, ett_field);
+
+	    proto_tree_add_item(field_tree,
+		hf_cflow_template_field_type, tvb, offset, 2, FALSE);
+	    offset += 2;
+
+	    proto_tree_add_item(field_tree,
+		hf_cflow_template_field_length, tvb, offset, 2, FALSE);
+	    offset += 2;
+	  }
+	  remaining -= 4 + sizeof(struct v9_template_entry) * count;
 	}
 
 	return (0);
@@ -1307,6 +1369,8 @@ static value_string v9_template_types[] = {
 	{ 20, "MUL_DOCTETS" },
 	{ 21, "LAST_SWITCHED" },
 	{ 22, "FIRST_SWITCHED" },
+	{ 23, "OUT_BYTES" },
+	{ 24, "OUT_PKTS" },
 	{ 27, "IPV6_SRC_ADDR" },
 	{ 28, "IPV6_DST_ADDR" },
 	{ 29, "IPV6_SRC_MASK" },
@@ -1320,10 +1384,15 @@ static value_string v9_template_types[] = {
 	{ 37, "FLOW_INACTIVE_TIMEOUT" },
 	{ 38, "ENGINE_TYPE" },
 	{ 39, "ENGINE_ID" },
-	{ 24, "OUT_PKTS" },
 	{ 40, "TOTAL_BYTES_EXP" },
 	{ 41, "TOTAL_PKTS_EXP" },
 	{ 42, "TOTAL_FLOWS_EXP" },
+	{ 46, "MPLS_TOP_LABEL_TYPE" },
+	{ 47, "MPLS_TOP_LABEL_ADDR" },
+	{ 48, "FLOW_SAMPLER_ID" },
+	{ 49, "FLOW_SAMPLER_MODE" },
+	{ 50, "FLOW_SAMPLER_RANDOM_INTERVAL" },
+	{ 55, "DST_TOS" },
 	{ 56, "SRC_MAC" },
 	{ 57, "DST_MAC" },
 	{ 58, "SRC_VLAN" },
@@ -1340,9 +1409,9 @@ static value_string v9_template_types[] = {
 	{ 74, "MPLS_LABEL_5" },
 	{ 75, "MPLS_LABEL_6" },
 	{ 76, "MPLS_LABEL_7" },
-	{ 71, "MPLS_LABEL_8" },
-	{ 72, "MPLS_LABEL_9" },
-	{ 72, "MPLS_LABEL_10" },
+	{ 77, "MPLS_LABEL_8" },
+	{ 78, "MPLS_LABEL_9" },
+	{ 79, "MPLS_LABEL_10" },
 	{ 0, NULL },
 };
 
@@ -1407,12 +1476,12 @@ dissect_pdu(proto_tree * pdutree, tvbuff_t * tvb, int offset, int ver)
 	/*
 	 * memcpy so we can use the values later to calculate a prefix 
 	 */
-	tvb_memcpy(tvb, (guint8 *) & srcaddr, offset, 4);
+	srcaddr = tvb_get_ipv4(tvb, offset);
 	proto_tree_add_ipv4(pdutree, hf_cflow_srcaddr, tvb, offset, 4,
 			    srcaddr);
 	offset += 4;
 
-	tvb_memcpy(tvb, (guint8 *) & dstaddr, offset, 4);
+	dstaddr = tvb_get_ipv4(tvb, offset);
 	proto_tree_add_ipv4(pdutree, hf_cflow_dstaddr, tvb, offset, 4,
 			    dstaddr);
 	offset += 4;
@@ -1878,6 +1947,16 @@ proto_register_netflow(void)
 		  FT_UINT32, BASE_DEC, NULL, 0x0,
 		  "Packets exported", HFILL}
 		 },
+		{&hf_cflow_mpls_top_label_type, 
+		 {"TopLabelType", "cflow.toplabeltype",
+		  FT_UINT8, BASE_DEC, VALS(special_mpls_top_label_type), 0x0,
+		  "Top MPLS label Type", HFILL}
+		 },
+		{&hf_cflow_mpls_pe_addr, 
+		 {"TopLabelAddr", "cflow.toplabeladdr",
+		  FT_IPv4, BASE_NONE, NULL, 0x0,
+		  "Top MPLS label PE address", HFILL}
+		 },
 		{&hf_cflow_flows_exp,
 		 {"FlowsExp", "cflow.flowsexp",
 		  FT_UINT32, BASE_DEC, NULL, 0x0,
@@ -1893,6 +1972,7 @@ proto_register_netflow(void)
 		&ett_unixtime,
 		&ett_flow,
 		&ett_template,
+		&ett_field,
 		&ett_dataflowset
 	};
 
