@@ -2,7 +2,7 @@
  * Routines for Network Service Over IP dissection
  * Copyright 2000, Susanne Edlund <susanne.edlund@ericsson.com>
  *
- * $Id: packet-nsip.c 14884 2005-07-09 00:53:17Z guy $
+ * $Id: packet-nsip.c 17696 2006-03-22 17:11:31Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,6 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include <epan/ipv6-utils.h>
 #include <prefs.h>
 
 #define NSIP_DEBUG 0
@@ -272,7 +271,7 @@ get_value_length(nsip_ie_t *ie, build_info_t *bi) {
   else {
     length_len++;
     length <<= 8;
-    length |= tvb_get_guint8(bi->tvb, bi->offset);
+    length |= tvb_get_guint8(bi->tvb, bi->offset+1);
   }
   ie->value_length = length;
   ie->total_length += length_len + length;
@@ -393,8 +392,7 @@ decode_ip_element(nsip_ip_element_info_t *element, build_info_t *bi, proto_tree
     /* IP address */
     switch (element->version) {
     case NSIP_IP_VERSION_4:
-      tvb_memcpy(bi->tvb, (guint8 *)&ip4_addr, bi->offset, 
-		 element->address_length);
+      ip4_addr = tvb_get_ipv4(bi->tvb, bi->offset);
       proto_tree_add_item(field_tree, hf_nsip_ip_address_ipv4, 
 			  bi->tvb, bi->offset, element->address_length, 
 			  NSIP_LITTLE_ENDIAN); 
@@ -403,8 +401,7 @@ decode_ip_element(nsip_ip_element_info_t *element, build_info_t *bi, proto_tree
 
       break;
     case NSIP_IP_VERSION_6:
-      tvb_memcpy(bi->tvb, (guint8 *)&ip6_addr, bi->offset, 
-		 sizeof(struct e_in6_addr));
+      tvb_get_ipv6(bi->tvb, bi->offset, &ip6_addr);
       proto_tree_add_item(field_tree, hf_nsip_ip_address_ipv6, bi->tvb, 
 			  bi->offset, element->address_length, 
 			  NSIP_LITTLE_ENDIAN); 
@@ -530,8 +527,7 @@ decode_iei_ip_address(nsip_ie_t *ie, build_info_t *bi, int ie_start_offset) {
   switch (address_type) {
   case NSIP_IP_ADDRESS_TYPE_IPV4:
     ie->total_length = 2 + ipv4_element.address_length;
-    tvb_memcpy(bi->tvb, (guint8 *)&ip4_addr, bi->offset, 
-	       ipv4_element.address_length);
+    ip4_addr = tvb_get_ipv4(bi->tvb, bi->offset);
     if (bi->nsip_tree) {
       proto_tree_add_ipv4(bi->nsip_tree, hf_nsip_ip_address_ipv4, 
 			  bi->tvb, ie_start_offset, ie->total_length, 
@@ -540,8 +536,7 @@ decode_iei_ip_address(nsip_ie_t *ie, build_info_t *bi, int ie_start_offset) {
     break;
   case NSIP_IP_ADDRESS_TYPE_IPV6:
     ie->total_length = 2 + ipv6_element.address_length;
-    tvb_memcpy(bi->tvb, (guint8 *)&ip6_addr, bi->offset, 
-	       sizeof(struct e_in6_addr));
+    tvb_get_ipv6(bi->tvb, bi->offset, &ip6_addr);
     if (bi->nsip_tree) {
       proto_tree_add_ipv6(bi->nsip_tree, hf_nsip_ip_address_ipv4, 
 			  bi->tvb, ie_start_offset, ie->total_length, 
@@ -816,10 +811,7 @@ decode_pdu_sns_changeweight(build_info_t *bi) {
 
 static void 
 decode_pdu_sns_config(build_info_t *bi) {
-  int use520 = 1; 
-  
-  if (use520) {
-    /* According to v. 5.2.0 */
+
     nsip_ie_t ies[] = { 
       { 0, NSIP_IE_PRESENCE_M, NSIP_IE_FORMAT_V, 0, 1 }, /* End flag */
       { NSIP_IE_NSEI, NSIP_IE_PRESENCE_M, NSIP_IE_FORMAT_TLV, 0, 4 },
@@ -830,21 +822,6 @@ decode_pdu_sns_config(build_info_t *bi) {
     };
     decode_iei_end_flag(ies, bi, bi->offset);
     decode_pdu_general(&ies[1], 3, bi);
-  }
-  else { /* According to v. 5.3.0 */
-    nsip_ie_t ies[] = { 
-      { NSIP_IE_NSEI, NSIP_IE_PRESENCE_M, NSIP_IE_FORMAT_TLV, 0, 4 },
-      { 0, NSIP_IE_PRESENCE_M, NSIP_IE_FORMAT_V, 0, 1 }, /* End flag */
-      { NSIP_IE_NSEI, NSIP_IE_PRESENCE_M, NSIP_IE_FORMAT_TLV, 0, 4 },
-      { NSIP_IE_IP4_ELEMENTS, NSIP_IE_PRESENCE_C, NSIP_IE_FORMAT_TLV, 0, 0 },
-      /* Unknown length */
-      { NSIP_IE_IP6_ELEMENTS, NSIP_IE_PRESENCE_C, NSIP_IE_FORMAT_TLV, 0, 0 },
-      /* Unknown length */
-    };
-    decode_pdu_general(ies, 1, bi);
-    decode_iei_end_flag(&ies[1], bi, bi->offset);
-    decode_pdu_general(&ies[2], 3, bi);
-   }
 }
 
 static void 
