@@ -2,7 +2,7 @@
 * Routines for the disassembly of the "Nortel Networks / SynOptics Network Management Protocol"
 * (c) Copyright Giles Scott <giles.scott1 [AT] arubanetworks.com>
 *
-* $Id: packet-nt-sonmp.c 12601 2004-11-25 20:38:10Z sahlberg $
+* $Id: packet-nt-sonmp.c 16956 2006-01-05 13:01:08Z jmayer $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 /* #include "strutil.h" */
-#include "nlpid.h"
+#include <epan/nlpid.h>
 
 /* Although this protocol is propietary it is documented in the SynOptics MIB's
 * So I'm not giving anything away :-)
@@ -171,11 +171,11 @@ static value_string sonmp_backplane_val[] =
 	{4, "ethernet and FDDI"},
 	{5, "ethernet, tokenring and FDDI"},
 	{6, "ethernet and tokenring with redundant power"},
-	{7, "ethernet, tokenring, FDDI with redunadant power"},
+	{7, "ethernet, tokenring, FDDI with redundant power"},
 	{8, "token ring"},
 	{9, "ethernet, tokenring and fast ethernet"},
 	{10, "ethernet and fast ethernet"},
-	{11, "ethernet, tokenring, fast ethernet with redunant power"},
+	{11, "ethernet, tokenring, fast ethernet with redundant power"},
 	{12, "ethernet, fast ethernet and gigabit ethernet"},
 	{0, NULL}
 };
@@ -211,7 +211,7 @@ static gint ett_sonmp = -1;
 static void 
 dissect_sonmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	char *hello_type;
+	const char *hello_type;
 	proto_tree *sonmp_tree = NULL;
 	proto_item *ti;
 	
