@@ -5,7 +5,7 @@
  * ISO 10589 ISIS (Intradomain Routing Information Exchange Protocol)
  * ISO  9542 ESIS (End System To Intermediate System Routing Exchange Protocol)
  *
- * $Id: packet-osi-options.c 14325 2005-05-07 00:14:11Z lroland $
+ * $Id: packet-osi-options.c 15844 2005-09-17 00:02:31Z guy $
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
  * Ethereal - Network traffic analyzer
@@ -35,7 +35,7 @@
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "nlpid.h"
+#include <epan/nlpid.h>
 #include "packet-osi.h"
 #include "packet-isis.h"
 #include "packet-isis-clv.h"
@@ -277,7 +277,7 @@ static void
 dissect_option_rfd( const guchar error, const guchar field, int offset,
                           guchar len, tvbuff_t *tvb, proto_tree *tree ) {
   guchar error_class = 0;
-  char   *format_string[] =
+  const char   *format_string[] =
              { "Reason for discard {General}        : %s, in field %u",
                "Reason for discard {Address}        : %s, in field %u",
                "Reason for discard {Source Routing}: %s, in field %u",
