@@ -1,8 +1,9 @@
 /* packet-pagp.c
  * Routines for PAgP (Port Aggregation Protocol - aka FEC) dissection
- * Copyright 2004, Mark C. Brown <mbrown@nosila.net>
+ * Original Author Mark C. Brown <mbrown@hp.com>
+ * Copyright (C) 2004 Hewlett-Packard Development Company, L.P.
  *
- * $Id: packet-pagp.c 12157 2004-09-30 19:53:27Z guy $
+ * $Id: packet-pagp.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,9 +34,9 @@
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "etypes.h"
-#include "llcsaps.h"
-#include "ppptypes.h"
+#include <epan/etypes.h>
+#include <epan/llcsaps.h>
+#include <epan/ppptypes.h>
 #include <epan/address.h>
 #include <epan/addr_resolv.h>
 
@@ -390,16 +391,14 @@ dissect_pagp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                                                                                 
 		switch (tlv) {
 		   case PAGP_TLV_DEVICE_NAME:
-			ch = tvb_get_string(tvb, offset+4, len-4);
+			ch = tvb_get_ephemeral_string(tvb, offset+4, len-4);
 			proto_tree_add_string(tlv_tree, hf_pagp_tlv_device_name,
 			   tvb, offset+4, len-4, ch);
-			g_free(ch);
 			break;
 		   case PAGP_TLV_PORT_NAME:
-			ch = tvb_get_string(tvb, offset+4, len-4);
+			ch = tvb_get_ephemeral_string(tvb, offset+4, len-4);
 			proto_tree_add_string(tlv_tree, hf_pagp_tlv_port_name,
 			   tvb, offset+4, len-4, ch);
-			g_free(ch);
 			break;
 		   case PAGP_TLV_AGPORT_MAC:
 			p_sys = tvb_get_ptr(tvb, offset+4, 6);
@@ -541,6 +540,16 @@ proto_register_pagp(void)
 	FT_ETHER,	BASE_NONE,	NULL,	0x0,
       	"Source MAC on frames for this aggregate", HFILL }},
 
+    { &hf_pagp_flush_local_device_id,
+      { "Flush Local Device ID",	"pagp.flushlocaldevid",
+	FT_ETHER,	BASE_NONE,	NULL,	0x0,
+      	"Flush local device ID", HFILL }},
+
+    { &hf_pagp_flush_partner_device_id,
+      { "Flush Partner Device ID",	"pagp.flushpartnerdevid",
+	FT_ETHER,	BASE_NONE,	NULL,	0x0,
+      	"Flush remote device ID", HFILL }},
+
     { &hf_pagp_flush_transaction_id,
       { "Transaction ID",		"pagp.transid",
 	FT_UINT32,	BASE_HEX,	NULL,	0x0,
