@@ -1,7 +1,7 @@
 /* packet-pgm.c
  * Routines for PGM packet disassembly, RFC 3208
  *
- * $Id: packet-pgm.c 12835 2004-12-26 00:22:16Z guy $
+ * $Id: packet-pgm.c 17065 2006-01-20 21:18:18Z jmayer $
  *
  * Copyright (c) 2000 by Talarian Corp
  *
@@ -42,6 +42,7 @@
 #include <epan/prefs.h>
 
 #include <epan/proto.h>
+#include <epan/emem.h>
 
 /*
  * Flag to control whether to check the PGM checksum.
@@ -346,10 +347,10 @@ typedef struct {
 #define DEFAULT_UDP_ENCAP_UCAST_PORT 3055
 #define DEFAULT_UDP_ENCAP_MCAST_PORT 3056
 
-static int udp_encap_ucast_port = 0;
-static int udp_encap_mcast_port = 0;
-static int old_encap_ucast_port = 0;
-static int old_encap_mcast_port = 0;
+static guint udp_encap_ucast_port = 0;
+static guint udp_encap_mcast_port = 0;
+static guint old_encap_ucast_port = 0;
+static guint old_encap_mcast_port = 0;
 
 static int proto_pgm = -1;
 static int ett_pgm = -1;
@@ -484,71 +485,58 @@ static dissector_handle_t data_handle;
  * was the URL for the PGM draft.
  */
 
-static char *
+static const char *
 optsstr(nchar_t opts)
 {
-	static char msg[256];
-	char *p = msg, *str;
-
+	char *msg;
+	size_t returned_length, index = 0;
+	const int MAX_STR_LEN = 256;
+	msg=ep_alloc(MAX_STR_LEN);
 	if (opts == 0)
 		return("");
 
 	if (opts & PGM_OPT){
-		sprintf(p, "Present");
-		p += strlen("Present");
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "Present");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
 	if (opts & PGM_OPT_NETSIG){
-		if (p != msg)
-			str = ",NetSig";
-		else
-			str = "NetSig";
-		sprintf(p, str);
-		p += strlen(str);
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "%sNetSig", (!index)?"":",");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
 	if (opts & PGM_OPT_VAR_PKTLEN){
-		if (p != msg)
-			str = ",VarLen";
-		else
-			str = "VarLen";
-		sprintf(p, str);
-		p += strlen(str);
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "%sVarLen", (!index)?"":",");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
 	if (opts & PGM_OPT_PARITY){
-		if (p != msg)
-			str = ",Parity";
-		else
-			str = "Parity";
-		sprintf(p, str);
-		p += strlen(str);
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "%sParity", (!index)?"":",");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
-	if (p == msg) {
-		sprintf(p, "0x%x", opts);
+	if (!index) {
+		g_snprintf(&msg[index], MAX_STR_LEN-index, "0x%x", opts);
 	}
 	return(msg);
 }
-static char *
+static const char *
 paritystr(nchar_t parity)
 {
-	static char msg[256];
-	char *p = msg, *str;
+	char *msg;
+	size_t returned_length, index = 0;
+	const int MAX_STR_LEN = 256;
 
+	msg=ep_alloc(MAX_STR_LEN);
 	if (parity == 0)
 		return("");
 
 	if (parity & PGM_OPT_PARITY_PRM_PRO){
-		sprintf(p, "Pro-active");
-		p += strlen("Pro-active");
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "Pro-active");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
 	if (parity & PGM_OPT_PARITY_PRM_OND){
-		if (p != msg)
-			str = ",On-demand";
-		else
-			str = "On-demand";
-		sprintf(p, str);
-		p += strlen(str);
+		returned_length = g_snprintf(&msg[index], MAX_STR_LEN-index, "%sOn-demand", (!index)?"":",");
+		index += MIN(returned_length, MAX_STR_LEN-index);
 	}
-	if (p == msg) {
-		sprintf(p, "0x%x", parity);
+	if (!index) {
+		g_snprintf(&msg[index], MAX_STR_LEN-index, "0x%x", parity);
 	}
 	return(msg);
 }
@@ -752,7 +740,7 @@ dissect_pgmopts(tvbuff_t *tvb, int offset, proto_tree *tree,
 		case PGM_OPT_NAK_LIST:{
 			pgm_opt_nak_list_t optdata;
 			nlong_t naklist[PGM_MAX_NAK_LIST_SZ+1];
-			char nakbuf[8192], *ptr;
+			char *nakbuf, *ptr;
 			int i, j, naks, soffset = 0;
 
 			opt_tree = proto_item_add_subtree(tf, ett_pgm_opts_naklist);
@@ -782,15 +770,17 @@ dissect_pgmopts(tvbuff_t *tvb, int offset, proto_tree *tree,
 			optdata.len -= sizeof(pgm_opt_nak_list_t);
 			tvb_memcpy(tvb, (guint8 *)naklist, offset+4, optdata.len);
 			naks = (optdata.len/sizeof(nlong_t));
+			nakbuf=ep_alloc(8192);
+			nakbuf[0]=0;
 			ptr = nakbuf;
 			j = 0;
 			/*
 			 * Print out 8 per line
 			 */
 			for (i=0; i < naks; i++) {
-				sprintf(nakbuf+soffset, "0x%lx ",
-				    (unsigned long)g_ntohl(naklist[i]));
-				soffset = strlen(nakbuf);
+				soffset += MIN(8192-soffset,
+					g_snprintf(nakbuf+soffset, 8192-soffset, "0x%lx ",
+						(unsigned long)g_ntohl(naklist[i])));
 				if ((++j % 8) == 0) {
 					if (firsttime) {
 						proto_tree_add_bytes_format(opt_tree,
@@ -1130,7 +1120,7 @@ static const value_string poll_subtype_vals[] = {
 /* separated into a stand alone routine to other protocol dissectors */
 /* can call to it, ie. socks	*/
 
-void
+static void
 decode_pgm_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	proto_tree *tree, pgm_type *pgmhdr)
 {
@@ -1158,7 +1148,7 @@ decode_pgm_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
   call_dissector(data_handle,next_tvb, pinfo, tree);
 
 }
-int
+static int
 total_size(tvbuff_t *tvb, pgm_type *hdr)
 {
 	int bytes = sizeof(pgm_type);
@@ -1352,12 +1342,12 @@ dissect_pgm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			computed_cksum = in_cksum(&cksum_vec[0], 1);
 			if (computed_cksum == 0) {
 				proto_tree_add_uint_format(pgm_tree, hf_pgm_main_cksum, tvb,
-					offset+6, 2, pgmhdr.cksum, "Checksum: 0x%04x (correct)", pgmhdr.cksum);
+					offset+6, 2, pgmhdr.cksum, "Checksum: 0x%04x [correct]", pgmhdr.cksum);
 			} else {
 				proto_tree_add_boolean_hidden(pgm_tree, hf_pgm_main_cksum_bad, tvb,
 				    offset+6, 2, TRUE);
 				proto_tree_add_uint_format(pgm_tree, hf_pgm_main_cksum, tvb,
-				    offset+6, 2, pgmhdr.cksum, "Checksum: 0x%04x (incorrect, should be 0x%04x)",
+				    offset+6, 2, pgmhdr.cksum, "Checksum: 0x%04x [incorrect, should be 0x%04x]",
 					pgmhdr.cksum, in_cksum_shouldbe(pgmhdr.cksum, computed_cksum));
 			}
 		} else {
