@@ -2,7 +2,7 @@
  * Routines for PIM disassembly
  * (c) Copyright Jun-ichiro itojun Hagino <itojun@itojun.org>
  *
- * $Id: packet-pim.c 12130 2004-09-29 00:52:45Z guy $
+ * $Id: packet-pim.c 17038 2006-01-16 07:59:44Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,15 +31,13 @@
 #include <string.h>
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/ipproto.h>
 #include <epan/afn.h>
 #include "packet-ipv6.h"
 #include <epan/in_cksum.h>
+#include "packet-pim.h"
 
 #define PIM_TYPE(x)	((x) & 0x0f)
 #define PIM_VER(x)	(((x) & 0xf0) >> 4)
@@ -73,14 +71,14 @@ dissect_pimv1_addr(tvbuff_t *tvb, int offset) {
 
     flags_masklen = tvb_get_ntohs(tvb, offset);
     if (flags_masklen & 0x0180) {
-	(void)snprintf(buf, sizeof(buf),
+	g_snprintf(buf, sizeof(buf),
 	    "(%s%s%s) ",
 	    flags_masklen & 0x0100 ? "S" : "",
 	    flags_masklen & 0x0080 ? "W" : "",
 	    flags_masklen & 0x0040 ? "R" : "");
     } else
 	buf[0] = '\0';
-    (void)snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%s/%u",
+	g_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%s/%u",
 	ip_to_str(tvb_get_ptr(tvb, offset + 2, 4)), flags_masklen & 0x3f);
 
     return buf;
@@ -196,12 +194,12 @@ dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	    if (computed_cksum == 0) {
 		proto_tree_add_uint_format(pim_tree, hf_pim_cksum, tvb,
 			    offset, 2, pim_cksum,
-			    "Checksum: 0x%04x (correct)",
+			    "Checksum: 0x%04x [correct]",
 			    pim_cksum);
 	    } else {
 		proto_tree_add_uint_format(pim_tree, hf_pim_cksum, tvb,
 			    offset, 2, pim_cksum,
-			    "Checksum: 0x%04x (incorrect, should be 0x%04x)",
+			    "Checksum: 0x%04x [incorrect, should be 0x%04x]",
 			    pim_cksum, in_cksum_shouldbe(pim_cksum, computed_cksum));
 	    }
 	} else {
@@ -378,6 +376,10 @@ dissect_pimv1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	    offset += 1;
 
 	    for (i = 0; i < ngroup; i++) {
+		/*
+		 * XXX - does the group address have the length "adr_len"
+		 * and the group mask the length "mask_len"?
+		 */
 		tigroup = proto_tree_add_text(pimopt_tree, tvb, offset, 4,
 		    "Group %d: %s", i,
 		    ip_to_str(tvb_get_ptr(tvb, offset, 4)));
@@ -515,13 +517,13 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
 	switch (af) {
 	case AFNUM_INET:
 	    len = 4;
-	    (void)snprintf(buf, sizeof(buf), "%s",
+	    g_snprintf(buf, sizeof(buf), "%s",
 	        ip_to_str(tvb_get_ptr(tvb, offset + 2, len)));
 	    break;
 
 	case AFNUM_INET6:
 	    len = 16;
-	    (void)snprintf(buf, sizeof(buf), "%s",
+	    g_snprintf(buf, sizeof(buf), "%s",
 		ip6_to_str((const struct e_in6_addr *)tvb_get_ptr(tvb, offset + 2, len)));
 	    break;
 	}
@@ -534,13 +536,13 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
 	switch (af) {
 	case AFNUM_INET:
 	    len = 4;
-	    (void)snprintf(buf, sizeof(buf), "%s/%u",
+	    g_snprintf(buf, sizeof(buf), "%s/%u",
 		ip_to_str(tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 
 	case AFNUM_INET6:
 	    len = 16;
-	    (void)snprintf(buf, sizeof(buf), "%s/%u",
+	    g_snprintf(buf, sizeof(buf), "%s/%u",
 		ip6_to_str((const struct e_in6_addr *)tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 	}
@@ -554,18 +556,18 @@ dissect_pim_addr(tvbuff_t *tvb, int offset, enum pimv2_addrtype at,
 	switch (af) {
 	case AFNUM_INET:
 	    len = 4;
-	    (void)snprintf(buf, sizeof(buf), "%s/%u",
+	    g_snprintf(buf, sizeof(buf), "%s/%u",
 		ip_to_str(tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 
 	case AFNUM_INET6:
 	    len = 16;
-	    (void)snprintf(buf, sizeof(buf), "%s/%u",
+	    g_snprintf(buf, sizeof(buf), "%s/%u",
 		ip6_to_str((const struct e_in6_addr *)tvb_get_ptr(tvb, offset + 4, len)), mask_len);
 	    break;
 	}
 	if (flags) {
-	    (void)snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+	    g_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
 		" (%s%s%s)",
 		flags & 0x04 ? "S" : "",
 		flags & 0x02 ? "W" : "",
@@ -607,7 +609,7 @@ dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
     guint16 pim_cksum, computed_cksum;
     vec_t cksum_vec[4];
     guint32 phdr[2];
-    char *typestr;
+    const char *typestr;
     proto_tree *pim_tree = NULL;
     proto_item *ti;
     proto_tree *pimopt_tree = NULL;
@@ -705,19 +707,19 @@ dissect_pim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	    default:
 	    	/* PIM is available for IPv4 and IPv6 right now */
 	    	computed_cksum = 0;	/* squelch GCC complaints */
-		g_assert_not_reached();
+		DISSECTOR_ASSERT_NOT_REACHED();
 		break;
 	    }
 
 	    if (computed_cksum == 0) {
 		proto_tree_add_uint_format(pim_tree, hf_pim_cksum, tvb,
 			offset + 2, 2, pim_cksum,
-			"Checksum: 0x%04x (correct)",
+			"Checksum: 0x%04x [correct]",
 			pim_cksum);
 	    } else {
 		proto_tree_add_uint_format(pim_tree, hf_pim_cksum, tvb,
 			offset + 2, 2, pim_cksum,
-			"Checksum: 0x%04x (incorrect, should be 0x%04x)",
+			"Checksum: 0x%04x [incorrect, should be 0x%04x]",
 			pim_cksum, in_cksum_shouldbe(pim_cksum, computed_cksum));
 	    }
 	} else {
