@@ -2,7 +2,7 @@
  *  packet-radiotap.c
  *	Decode packets with a Radiotap header
  *
- * $Id: packet-radiotap.c 12429 2004-10-29 02:40:01Z guy $
+ * $Id: packet-radiotap.c 17085 2006-01-23 23:21:02Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,13 +48,14 @@ struct ieee80211_radiotap_header {
 				 * it_version, it_pad,
 				 * it_len, and data fields.
 				 */
-    guint32   it_present;       /* A bitmap telling which
-				 * fields are present. Set bit 31
-				 * (0x80000000) to extend the
-				 * bitmap by another 32 bits.
-				 * Additional extensions are made
-				 * by setting bit 31.
-				 */
+#define MAX_PRESENT 1
+    guint32   it_present[MAX_PRESENT];	/* A bitmap telling which
+					 * fields are present. Set bit 31
+					 * (0x80000000) to extend the
+					 * bitmap by another 32 bits.
+					 * Additional extensions are made
+					 * by setting bit 31.
+					 */
 };
 
 enum ieee80211_radiotap_type {
@@ -119,46 +120,122 @@ enum ieee80211_radiotap_type {
 #define	IEEE80211_RADIOTAP_F_FRAG	0x08	/* sent/received
 						 * with fragmentation
 						 */
+#define	IEEE80211_RADIOTAP_F_FCS	0x10	/* frame includes FCS */
+#define	IEEE80211_RADIOTAP_F_DATAPAD	0x20	/* frame has padding between
+						 * 802.11 header and payload
+						 * (to 32-bit boundary)
+						 */
 
 /* protocol */
 static int proto_radiotap = -1;
 
 static int hf_radiotap_version = -1;
+static int hf_radiotap_pad = -1;
 static int hf_radiotap_length = -1;
+static int hf_radiotap_present1 = -1;
 static int hf_radiotap_mactime = -1;
 static int hf_radiotap_channel_frequency = -1;
 static int hf_radiotap_channel_flags = -1;
 static int hf_radiotap_datarate = -1;
 static int hf_radiotap_antenna = -1;
-static int hf_radiotap_antsignal = -1;
-static int hf_radiotap_antnoise = -1;
+static int hf_radiotap_dbm_antsignal = -1;
+static int hf_radiotap_db_antsignal = -1;
+static int hf_radiotap_dbm_antnoise = -1;
+static int hf_radiotap_db_antnoise = -1;
 static int hf_radiotap_txpower = -1;
 static int hf_radiotap_preamble = -1;
+static int hf_radiotap_fcs = -1;
+static int hf_radiotap_datapad = -1;
 
 static gint ett_radiotap = -1;
+static gint ett_radiotap_present = -1;
 
 static dissector_handle_t ieee80211_handle;
+static dissector_handle_t ieee80211_datapad_handle;
 
 static void
 dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
+#define BITNO_32(x) (((x) >> 16) ? 16 + BITNO_16((x) >> 16) : BITNO_16((x)))
+#define BITNO_16(x) (((x) >> 8) ? 8 + BITNO_8((x) >> 8) : BITNO_8((x)))
+#define BITNO_8(x) (((x) >> 4) ? 4 + BITNO_4((x) >> 4) : BITNO_4((x)))
+#define BITNO_4(x) (((x) >> 2) ? 2 + BITNO_2((x) >> 2) : BITNO_2((x)))
+#define BITNO_2(x) (((x) & 2) ? 1 : 0)
+#define BIT(n)	(1 << n)
+
 void
 capture_radiotap(const guchar *pd, int offset, int len, packet_counts *ld)
 {
     const struct ieee80211_radiotap_header *hdr;
+    guint16 it_len;
+    guint32 present;
+    guint8 rflags;
 
     if(!BYTES_ARE_IN_FRAME(offset, len, (int)sizeof(*hdr))) {
         ld->other ++;
         return;
     }
-    hdr = (const struct ieee80211_radiotap_header *)pd;
-    if(!BYTES_ARE_IN_FRAME(offset, len, hdr->it_len)) {
+    hdr = (const struct ieee80211_radiotap_header *)&pd[offset];
+    it_len = pletohs(&hdr->it_len);
+    if(!BYTES_ARE_IN_FRAME(offset, len, it_len)) {
         ld->other ++;
         return;
     }
 
+    if(it_len > len) {
+        /* Header length is bigger than total packet length */
+        ld->other ++;
+        return;
+    }
+
+    if(it_len < sizeof(*hdr)) {
+        /* Header length is shorter than fixed-length portion of header */
+        ld->other ++;
+        return;
+    }
+
+    present = pletohl(&hdr->it_present);
+    offset += sizeof(*hdr);
+    it_len -= sizeof(*hdr);
+
+    rflags = 0;
+
+    /*
+     * IEEE80211_RADIOTAP_TSFT is the lowest-order bit.
+     */
+    if (present & BIT(IEEE80211_RADIOTAP_TSFT)) {
+    	if (it_len < 8) {
+	    /* No room in header for this field. */
+	    ld->other ++;
+	    return;
+	}
+	/* That field is present, and it's 8 bits long. */
+	offset += 8;
+	it_len -= 8;
+    }
+
+    /*
+     * IEEE80211_RADIOTAP_FLAGS is the next bit.
+     */
+    if (present & BIT(IEEE80211_RADIOTAP_FLAGS)) {
+    	if (it_len < 1) {
+	    /* No room in header for this field. */
+	    ld->other ++;
+	    return;
+	}
+	/* That field is present; fetch it. */
+	if(!BYTES_ARE_IN_FRAME(offset, len, 1)) {
+	    ld->other ++;
+	    return;
+	}
+	rflags = pd[offset];
+    }
+
     /* 802.11 header follows */
-    capture_ieee80211(pd, offset + hdr->it_len, len, ld);
+    if (rflags & IEEE80211_RADIOTAP_F_DATAPAD)
+	capture_ieee80211_datapad(pd, offset + it_len, len, ld);
+    else
+	capture_ieee80211(pd, offset + it_len, len, ld);
 }
 
 void
@@ -183,18 +260,38 @@ proto_register_radiotap(void)
     { 0, NULL },
   };
 
+  static const value_string truefalse_type[] = {
+    { 0, "False" },
+    { 1, "True" },
+    { 0, NULL },
+  };
+
   static hf_register_info hf[] = {
     { &hf_radiotap_version,
       { "Header revision", "radiotap.version",
-	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
+	FT_UINT8, BASE_DEC, NULL, 0x0, "", HFILL } },
+    { &hf_radiotap_pad,
+      { "Header pad", "radiotap.pad",
+	FT_UINT8, BASE_DEC, NULL, 0x0, "", HFILL } },
     { &hf_radiotap_length,
        { "Header length", "radiotap.length",
-	 FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
+	 FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL } },
+    { &hf_radiotap_present1,
+       { "Present elements", "radiotap.present",
+	 FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL } },
 
     { &hf_radiotap_preamble,
       { "Preamble", "radiotap.flags.preamble",
 	FT_UINT32, BASE_DEC, VALS(preamble_type), 0x0, "", HFILL } },
 
+    /* XXX for debugging */
+    { &hf_radiotap_fcs,
+      { "FCS", "radiotap.flags.fcs",
+	FT_UINT32, BASE_DEC, VALS(truefalse_type), 0x0, "", HFILL } },
+    { &hf_radiotap_datapad,
+      { "DATAPAD", "radiotap.flags.datapad",
+	FT_UINT32, BASE_DEC, VALS(truefalse_type), 0x0, "", HFILL } },
+
     { &hf_radiotap_mactime,
        { "MAC timestamp", "radiotap.mactime",
 	 FT_UINT64, BASE_DEC, NULL, 0x0, "", HFILL } },
@@ -203,25 +300,32 @@ proto_register_radiotap(void)
 	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
     { &hf_radiotap_channel_flags,
       { "Channel type", "radiotap.channel.flags",
-	FT_UINT32, BASE_HEX, VALS(phy_type), 0x0, "", HFILL } },
+	FT_UINT16, BASE_HEX, VALS(phy_type), 0x0, "", HFILL } },
     { &hf_radiotap_datarate,
       { "Data rate", "radiotap.datarate",
 	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
     { &hf_radiotap_antenna,
       { "Antenna", "radiotap.antenna",
 	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
-    { &hf_radiotap_antsignal,
-      { "SSI Signal", "radiotap.antsignal",
+    { &hf_radiotap_dbm_antsignal,
+      { "SSI Signal (dBm)", "radiotap.dbm_antsignal",
 	FT_INT32, BASE_DEC, NULL, 0x0, "", HFILL } },
-    { &hf_radiotap_antnoise,
-      { "SSI Noise", "radiotap.antnoise",
+    { &hf_radiotap_db_antsignal,
+      { "SSI Signal (dB)", "radiotap.db_antsignal",
+	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
+    { &hf_radiotap_dbm_antnoise,
+      { "SSI Noise (dBm)", "radiotap.dbm_antnoise",
 	FT_INT32, BASE_DEC, NULL, 0x0, "", HFILL } },
+    { &hf_radiotap_db_antnoise,
+      { "SSI Noise (dB)", "radiotap.db_antnoise",
+	FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL } },
     { &hf_radiotap_txpower,
       { "Transmit power", "radiotap.txpower",
 	FT_INT32, BASE_DEC, NULL, 0x0, "", HFILL } },
   };
   static gint *ett[] = {
-    &ett_radiotap
+    &ett_radiotap,
+    &ett_radiotap_present
   };
 
   proto_radiotap = proto_register_protocol("IEEE 802.11 Radiotap Capture header", "802.11 Radiotap", "radiotap");
@@ -237,6 +341,7 @@ proto_register_radiotap(void)
 static int
 ieee80211_mhz2ieee(int freq, int flags)
 {
+#define IS_CHAN_IN_PUBLIC_SAFETY_BAND(_c) ((_c) > 4940 && (_c) < 4990)
     if (flags & IEEE80211_CHAN_2GHZ) {		/* 2GHz band */
 	if (freq == 2484)
 	    return 14;
@@ -245,33 +350,42 @@ ieee80211_mhz2ieee(int freq, int flags)
 	else
 	    return 15 + ((freq - 2512) / 20);
     } else if (flags & IEEE80211_CHAN_5GHZ) {	/* 5Ghz band */
-	return (freq - 5000) / 5;
+	if (IS_CHAN_IN_PUBLIC_SAFETY_BAND(freq))
+	    return ((freq * 10) + (((freq % 5) == 2) ? 5 : 0) - 49400) / 5;
+	if (freq <= 5000)
+	    return (freq - 4000) / 5;
+	else
+	    return (freq - 5000) / 5;
     } else {					/* either, guess */
 	if (freq == 2484)
 	    return 14;
 	if (freq < 2484)
 	    return (freq - 2407) / 5;
-	if (freq < 5000)
-	    return 15 + ((freq - 2512) / 20);
+	if (freq < 5000) {
+	    if (IS_CHAN_IN_PUBLIC_SAFETY_BAND(freq))
+		return ((freq * 10) + (((freq % 5) == 2) ? 5 : 0) - 49400)/5;
+	    else if (freq > 4900)
+		return (freq - 4000) / 5;
+	    else
+		return 15 + ((freq - 2512) / 20);
+	}
 	return (freq - 5000) / 5;
     }
+#undef IS_CHAN_IN_PUBLIC_SAFETY_BAND
 }
 
 static void
 dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-#define BITNO_32(x) (((x) >> 16) ? 16 + BITNO_16((x) >> 16) : BITNO_16((x)))
-#define BITNO_16(x) (((x) >> 8) ? 8 + BITNO_8((x) >> 8) : BITNO_8((x)))
-#define BITNO_8(x) (((x) >> 4) ? 4 + BITNO_4((x) >> 4) : BITNO_4((x)))
-#define BITNO_4(x) (((x) >> 2) ? 2 + BITNO_2((x) >> 2) : BITNO_2((x)))
-#define BITNO_2(x) (((x) & 2) ? 1 : 0)
-#define BIT(n)	(1 << n)
     proto_tree *radiotap_tree = NULL;
-    proto_item *ti;
+    proto_tree *pt, *present_tree;
+    proto_item *ti = NULL;
     int offset;
-    guint32 version;
+    guint32 version, pad;
     guint32 length;
     guint32 rate, freq, flags;
+    gint8 dbm;
+    guint8 db, rflags;
     guint32 present, next_present;
     int bit;
 
@@ -282,21 +396,48 @@ dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     offset = 0;
 
     version = tvb_get_guint8(tvb, offset);
+    pad = tvb_get_guint8(tvb, offset+1);
     length = tvb_get_letohs(tvb, offset+2);
     present = tvb_get_letohl(tvb, offset+4);
-    offset+=sizeof(struct ieee80211_radiotap_header);
 
     if(check_col(pinfo->cinfo, COL_INFO))
-	col_add_fstr(pinfo->cinfo, COL_INFO, "Radiotap Capture v%x, Length %d",
+	col_add_fstr(pinfo->cinfo, COL_INFO, "Radiotap Capture v%u, Length %u",
 		version, length);
 
     /* Dissect the packet */
     if (tree) {
 	ti = proto_tree_add_protocol_format(tree, proto_radiotap,
-	      tvb, 0, length, "Radiotap Header");
+	    tvb, 0, length, "Radiotap Header v%u, Length %u", version, length);
 	radiotap_tree = proto_item_add_subtree(ti, ett_radiotap);
+	proto_tree_add_uint(radiotap_tree, hf_radiotap_version,
+	    tvb, offset, 1, version);
+	proto_tree_add_uint(radiotap_tree, hf_radiotap_pad,
+	    tvb, offset + 1, 1, pad);
+	ti = proto_tree_add_uint(radiotap_tree, hf_radiotap_length,
+	    tvb, offset + 2, 2, length);
     }
+    /*
+     * FIXME: This only works if there is exactly 1 it_present
+     *        field in the header
+     */
+    if (length < sizeof(struct ieee80211_radiotap_header)) {
+	/*
+	 * Radiotap header is shorter than the fixed-length portion
+	 * plus one "present" bitset.
+	 */
+	if (tree)
+	    proto_item_append_text(ti, " (bogus - minimum length is 8)");
+	return;
+    }
+    if (tree) {
+	pt = proto_tree_add_uint_format(radiotap_tree, hf_radiotap_present1,
+	    tvb, offset + 4, 4, present, "Present flags (0x%08x)", present);
+	present_tree = proto_item_add_subtree(pt, ett_radiotap_present);
+    }
+    offset += sizeof(struct ieee80211_radiotap_header);
+    length -= sizeof(struct ieee80211_radiotap_header);
 
+    rflags = 0;
     for (; present; present = next_present) {
 	/* clear the least significant bit that is set */
 	next_present = present & (present - 1);
@@ -306,48 +447,25 @@ dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	switch (bit) {
 	case IEEE80211_RADIOTAP_FLAGS:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    if (tree) {
 		proto_tree_add_uint(radiotap_tree, hf_radiotap_preamble,
-			tvb, 0, 0, (tvb_get_guint8(tvb, offset) &
-				IEEE80211_RADIOTAP_F_SHORTPRE) != 0);
+			tvb, 0, 0, (rflags&IEEE80211_RADIOTAP_F_SHORTPRE) != 0);
+		proto_tree_add_uint(radiotap_tree, hf_radiotap_fcs,
+			tvb, 0, 0, (rflags&IEEE80211_RADIOTAP_F_FCS) != 0);
+		proto_tree_add_uint(radiotap_tree, hf_radiotap_datapad,
+			tvb, 0, 0, (rflags&IEEE80211_RADIOTAP_F_DATAPAD) != 0);
 	    }
 	    offset++;
+	    length--;
 	    /* XXX CFP, WEP, FRAG */
-#if 0
-      capability = tvb_get_letohs (tvb, offset);
-
-      cap_item = proto_tree_add_uint_format (tree, ff_capture,
-					     tvb, offset, 2,
-					     capability,
-					     "Capability Information: 0x%04X",
-					     capability);
-      cap_tree = proto_item_add_subtree (cap_item, ett_cap_tree);
-      proto_tree_add_boolean (cap_tree, ff_cf_ess, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_cf_ibss, tvb, offset, 2,
-			      capability);
-      if (ESS_SET (capability) != 0)	/* This is an AP */
-	proto_tree_add_uint (cap_tree, ff_cf_ap_poll, tvb, offset, 2,
-			     capability);
-
-      else			/* This is a STA */
-	proto_tree_add_uint (cap_tree, ff_cf_sta_poll, tvb, offset, 2,
-			     capability);
-      proto_tree_add_boolean (cap_tree, ff_cf_privacy, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_cf_preamble, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_cf_pbcc, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_cf_agility, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_short_slot_time, tvb, offset, 2,
-			      capability);
-      proto_tree_add_boolean (cap_tree, ff_dsss_ofdm, tvb, offset, 2,
-			      capability);
-#endif
 	    break;
 	case IEEE80211_RADIOTAP_RATE:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    rate = tvb_get_guint8(tvb, offset) & 0x7f;
 	    if (check_col(pinfo->cinfo, COL_TX_RATE)) {
 		col_add_fstr(pinfo->cinfo, COL_TX_RATE, "%d.%d",
@@ -359,44 +477,96 @@ dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			"Data Rate: %d.%d Mb/s", rate / 2, rate & 1 ? 5 : 0);
 	    }
 	    offset++;
+	    length--;
 	    break;
 	case IEEE80211_RADIOTAP_DBM_ANTSIGNAL:
-	case IEEE80211_RADIOTAP_DB_ANTSIGNAL:
-	    /* XXX distinguish units */
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
+	    dbm = (gint8) tvb_get_guint8(tvb, offset);
 	    if (check_col(pinfo->cinfo, COL_RSSI)) {
-		col_add_fstr(pinfo->cinfo, COL_RSSI, "%d",
-		    tvb_get_guint8(tvb, offset));
+		col_add_fstr(pinfo->cinfo, COL_RSSI, "%d dBm", dbm);
 	    }
 	    if (tree) {
-		proto_tree_add_int(radiotap_tree, hf_radiotap_antsignal,
-				   tvb, offset, 1, tvb_get_guint8(tvb, offset));
+		proto_tree_add_int_format(radiotap_tree,
+					  hf_radiotap_dbm_antsignal,
+					  tvb, offset, 1, dbm,
+					  "SSI Signal: %d dBm", dbm);
 	    }
 	    offset++;
+	    length--;
+	    break;
+	case IEEE80211_RADIOTAP_DB_ANTSIGNAL:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
+	    db = tvb_get_guint8(tvb, offset);
+	    if (check_col(pinfo->cinfo, COL_RSSI)) {
+		col_add_fstr(pinfo->cinfo, COL_RSSI, "%u dB", db);
+	    }
+	    if (tree) {
+		proto_tree_add_uint_format(radiotap_tree,
+					   hf_radiotap_db_antsignal,
+					   tvb, offset, 1, db,
+					   "SSI Signal: %u dB", db);
+	    }
+	    offset++;
+	    length--;
 	    break;
 	case IEEE80211_RADIOTAP_DBM_ANTNOISE:
-	case IEEE80211_RADIOTAP_DB_ANTNOISE:
-	    /* XXX distinguish units */
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
+	    dbm = (gint8) tvb_get_guint8(tvb, offset);
 	    if (tree) {
-		proto_tree_add_int(radiotap_tree, hf_radiotap_antnoise,
-				   tvb, offset, 1, tvb_get_guint8(tvb, offset));
+		proto_tree_add_int_format(radiotap_tree,
+					  hf_radiotap_dbm_antnoise,
+					  tvb, offset, 1, dbm,
+					  "SSI Noise: %d dBm", dbm);
 	    }
 	    offset++;
+	    length--;
+	    break;
+	case IEEE80211_RADIOTAP_DB_ANTNOISE:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
+	    db = tvb_get_guint8(tvb, offset);
+	    if (tree) {
+		proto_tree_add_uint_format(radiotap_tree,
+					   hf_radiotap_db_antnoise,
+					   tvb, offset, 1, db,
+					   "SSI Noise: %u dB", db);
+	    }
+	    offset++;
+	    length--;
 	    break;
 	case IEEE80211_RADIOTAP_ANTENNA:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    if (tree) {
 		proto_tree_add_uint(radiotap_tree, hf_radiotap_antenna,
 				   tvb, offset, 1, tvb_get_guint8(tvb, offset));
 	    }
 	    offset++;
+	    length--;
 	    break;
 	case IEEE80211_RADIOTAP_DBM_TX_POWER:
+	    if (length < 1)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    if (tree) {
 		proto_tree_add_int(radiotap_tree, hf_radiotap_txpower,
 				   tvb, offset, 1, tvb_get_guint8(tvb, offset));
 	    }
 	    offset++;
+	    length--;
 	    break;
 	case IEEE80211_RADIOTAP_CHANNEL:
+	    if (length < 4)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    if (tree) {
 		freq = tvb_get_letohs(tvb, offset);
 		flags = tvb_get_letohs(tvb, offset+2);
@@ -407,22 +577,31 @@ dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			tvb, offset+2, 2, flags);
 	    }
 	    offset+=4;
+	    length-=4;
 	    break;
 	case IEEE80211_RADIOTAP_FHSS:
 	case IEEE80211_RADIOTAP_LOCK_QUALITY:
 	case IEEE80211_RADIOTAP_TX_ATTENUATION:
 	case IEEE80211_RADIOTAP_DB_TX_ATTENUATION:
+	    if (length < 2)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 #if 0
 	    tvb_get_letohs(tvb, offset);
 #endif
 	    offset+=2;
+	    length-=2;
 	    break;
 	case IEEE80211_RADIOTAP_TSFT:
+	    if (length < 8)
+		break;
+	    rflags = tvb_get_guint8(tvb, offset);
 	    if (tree) {
-		proto_tree_add_item(radiotap_tree, hf_radiotap_mactime,
-				    tvb, offset, 8, FALSE);
+		proto_tree_add_uint64(radiotap_tree, hf_radiotap_mactime,
+				tvb, offset, 8, tvb_get_letoh64(tvb, offset));
 	    }
 	    offset+=8;
+	    length-=8;
 	    break;
 	default:
 	    /*
@@ -434,15 +613,12 @@ dissect_radiotap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
     }
 
+    if (rflags & IEEE80211_RADIOTAP_F_FCS)
+	pinfo->pseudo_header->ieee_802_11.fcs_len = 4;
     /* dissect the 802.11 header next */
-    call_dissector(ieee80211_handle,
+    call_dissector((rflags & IEEE80211_RADIOTAP_F_DATAPAD) ?
+	ieee80211_datapad_handle : ieee80211_handle,
 	tvb_new_subset(tvb, length, -1, -1), pinfo, tree);
-#undef BITNO_32
-#undef BITNO_16
-#undef BITNO_8
-#undef BITNO_4
-#undef BITNO_2
-#undef BIT
 }
 
 void
@@ -452,6 +628,7 @@ proto_reg_handoff_radiotap(void)
 
     /* handle for 802.11 dissector */
     ieee80211_handle = find_dissector("wlan");
+    ieee80211_datapad_handle = find_dissector("wlan_datapad");
 
     radiotap_handle = create_dissector_handle(dissect_radiotap, proto_radiotap);
 
