@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 14583 2005-06-07 23:25:46Z lego $
+ * $Id: packet-rtp.c 17231 2006-02-09 19:22:15Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,6 +48,11 @@
  *    version 1, as documented in
  *
  *	ftp://gaia.cs.umass.edu/pub/hgschulz/rtp/draft-ietf-avt-rtp-04.txt
+ *
+ * It also dissects PacketCable CCC-encapsulated RTP data, as described in
+ * chapter 5 of the PacketCable Electronic Surveillance Specification:
+ *
+ *   http://www.packetcable.com/downloads/specs/PKT-SP-ESP1.5-I01-050128.pdf
  */
 
 
@@ -62,14 +67,19 @@
 #include <string.h>
 
 #include "packet-rtp.h"
-#include "rtp_pt.h"
+#include <epan/rtp_pt.h>
+#include "packet-ntp.h"
 #include <epan/conversation.h>
 #include <epan/tap.h>
 
 #include <epan/prefs.h>
+#include <epan/emem.h>
 
 static dissector_handle_t rtp_handle;
 static dissector_handle_t stun_handle;
+static dissector_handle_t t38_handle;
+
+static dissector_handle_t pkt_ccc_handle;
 
 static int rtp_tap = -1;
 
@@ -109,12 +119,26 @@ static gint ett_hdr_ext   = -1;
 static gint ett_rtp_setup = -1;
 
 
+/* PacketCable CCC header fields */
+static int proto_pkt_ccc       = -1;
+static int hf_pkt_ccc_id       = -1;
+static int hf_pkt_ccc_ts       = -1;
+
+/* PacketCable CCC field defining a sub tree */
+static gint ett_pkt_ccc = -1;
+
+/* PacketCable CCC port preference */
+static gboolean global_pkt_ccc_udp_port = 0;
+
+
 #define RTP0_INVALID 0
 #define RTP0_STUN    1
+#define RTP0_T38     2
 
 static enum_val_t rtp_version0_types[] = {
 	{ "invalid", "Invalid RTP packets", RTP0_INVALID },
 	{ "stun", "STUN packets", RTP0_STUN },
+	{ "t38", "T.38 packets", RTP0_T38 },
 	{ NULL, NULL, 0 }
 };
 static guint global_rtp_version0_type = 0;
@@ -134,9 +158,6 @@ static gboolean global_rtp_show_setup_info = TRUE;
 /* Try heuristic RTP decode */
 static gboolean global_rtp_heur = FALSE;
 
-/* Memory chunk for storing conversation and per-packet info */
-static GMemChunk *rtp_conversations = NULL;
-
 /*
  * Fields in the first octet of the RTP header.
  */
@@ -236,18 +257,10 @@ const value_string rtp_payload_type_short_vals[] =
        { 0,            NULL },
 };
 
-static void
-free_hash_item( gpointer key _U_ , gpointer value, gpointer user_data _U_ )
-{
-        g_free(value);
-		g_free(key);
-}
-
 void
 rtp_free_hash_dyn_payload(GHashTable *rtp_dyn_payload)
 {
 	if (rtp_dyn_payload == NULL) return;
-	g_hash_table_foreach(rtp_dyn_payload, (GHFunc)free_hash_item, NULL);
 	g_hash_table_destroy(rtp_dyn_payload);
 	rtp_dyn_payload = NULL;
 }
@@ -256,7 +269,7 @@ rtp_free_hash_dyn_payload(GHashTable *rtp_dyn_payload)
 void rtp_add_address(packet_info *pinfo,
                      address *addr, int port,
                      int other_port,
-                     gchar *setup_method, guint32 setup_frame_number, GHashTable *rtp_dyn_payload)
+                     const gchar *setup_method, guint32 setup_frame_number, GHashTable *rtp_dyn_payload)
 {
 	address null_addr;
 	conversation_t* p_conv;
@@ -303,7 +316,7 @@ void rtp_add_address(packet_info *pinfo,
 	 */
 	if ( ! p_conv_data ) {
 		/* Create conversation data */
-		p_conv_data = g_mem_chunk_alloc(rtp_conversations);
+		p_conv_data = se_alloc(sizeof(struct _rtp_conversation_info));
 		p_conv_data->rtp_dyn_payload = NULL;
 
 		conversation_add_proto_data(p_conv, proto_rtp, p_conv_data);
@@ -321,19 +334,6 @@ void rtp_add_address(packet_info *pinfo,
 	p_conv_data->rtp_dyn_payload = rtp_dyn_payload;
 }
 
-static void rtp_init( void )
-{
-	/* (Re)allocate mem chunk for conversations */
-	if (rtp_conversations)
-	{
-		g_mem_chunk_destroy(rtp_conversations);
-	}
-	rtp_conversations = g_mem_chunk_new("rtp_conversations",
-	                                    sizeof(struct _rtp_conversation_info),
-	                                    20 * sizeof(struct _rtp_conversation_info),
-	                                    G_ALLOC_ONLY);
-}
-
 static gboolean
 dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
@@ -341,7 +341,7 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
  	unsigned int version;
 	unsigned int payload_type;
  	unsigned int offset = 0;
-   
+
 	/* This is a heuristic dissector, which means we get all the UDP
 	 * traffic not sent to a known dissector and not claimed by
 	 * a heuristic dissector called before us!
@@ -360,6 +360,10 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 			call_dissector(stun_handle, tvb, pinfo, tree);
 			return TRUE;
 
+		case RTP0_T38:
+			call_dissector(t38_handle, tvb, pinfo, tree);
+			return TRUE;
+
 		case RTP0_INVALID:
 		default:
 			return FALSE; /* Unknown or unsupported version */
@@ -461,6 +465,10 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 			call_dissector(stun_handle, tvb, pinfo, tree);
 			return;
 
+		case RTP0_T38:
+			call_dissector(t38_handle, tvb, pinfo, tree);
+			return;
+
 		case RTP0_INVALID:
 		default:
 			; /* Unknown or unsupported version (let it fall through */
@@ -515,6 +523,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	rtp_info->info_timestamp = timestamp;
 	rtp_info->info_sync_src = sync_src;
 	rtp_info->info_setup_frame_num = 0;
+	rtp_info->info_payload_type_str = NULL;
 
 	/*
 	 * Do we have all the data?
@@ -560,8 +569,10 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	if ( (payload_type>95) && (payload_type<128) ) {
 		/* Use existing packet info if available */
 		p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
-		if (p_conv_data && p_conv_data->rtp_dyn_payload)
+		if (p_conv_data && p_conv_data->rtp_dyn_payload){
 			payload_type_str = g_hash_table_lookup(p_conv_data->rtp_dyn_payload, &payload_type);
+			rtp_info->info_payload_type_str = payload_type_str;
+		}
 	}
 
 	if ( check_col( pinfo->cinfo, COL_INFO) ) {
@@ -601,7 +612,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		    1, octet2 );
 
 		item = proto_tree_add_uint_format( rtp_tree, hf_rtp_payload_type, tvb,
-		    offset, 1, octet2, "Payload type: %s (%u)", 
+		    offset, 1, octet2, "Payload type: %s (%u)",
 			payload_type_str ? payload_type_str : val_to_str( payload_type, rtp_payload_type_vals,"Unknown"),
 			payload_type);
 
@@ -777,8 +788,9 @@ static void get_conv_info(packet_info *pinfo, struct _rtp_info *rtp_info)
 
 			if (p_conv_data) {
 				/* Save this conversation info into packet info */
-				p_conv_packet_data = g_mem_chunk_alloc(rtp_conversations);
-				strcpy(p_conv_packet_data->method, p_conv_data->method);
+				p_conv_packet_data = se_alloc(sizeof(struct _rtp_conversation_info));
+				g_snprintf(p_conv_packet_data->method, MAX_RTP_SETUP_METHOD_SIZE, "%s", p_conv_data->method);
+				p_conv_packet_data->method[MAX_RTP_SETUP_METHOD_SIZE]=0;
 				p_conv_packet_data->frame_number = p_conv_data->frame_number;
 				p_conv_packet_data->rtp_dyn_payload = p_conv_data->rtp_dyn_payload;
 				p_add_proto_data(pinfo->fd, proto_rtp, p_conv_packet_data);
@@ -822,6 +834,97 @@ static void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 }
 
+/* Dissect PacketCable CCC header */
+
+static void
+dissect_pkt_ccc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	proto_item *ti            = NULL;
+	proto_tree *pkt_ccc_tree      = NULL;
+ 	const guint8 *ptime = tvb_get_ptr(tvb, 4, 8);
+
+	if ( tree ) {
+		ti = proto_tree_add_item(tree, proto_pkt_ccc, tvb, 0, 12, FALSE);
+		pkt_ccc_tree = proto_item_add_subtree(ti, ett_pkt_ccc);
+
+		proto_tree_add_item(pkt_ccc_tree, hf_pkt_ccc_id, tvb, 0, 4, FALSE);
+		proto_tree_add_bytes_format(pkt_ccc_tree, hf_pkt_ccc_ts, tvb,
+		    4, 8, "NTP timestamp: %s", ntp_fmt_ts(ptime));
+	}
+
+	dissect_rtp(tvb, pinfo, tree);
+}
+
+
+/* Register PacketCable CCC */
+
+void
+proto_register_pkt_ccc(void)
+{
+	static hf_register_info hf[] =
+	{
+		{
+			&hf_pkt_ccc_id,
+			{
+				"PacketCable CCC Identifier",
+				"pkt_ccc.ccc_id",
+				FT_UINT32,
+				BASE_DEC,
+				NULL,
+				0x0,
+				"CCC_ID", HFILL
+			}
+		},
+		{
+			&hf_pkt_ccc_ts,
+			{
+				"PacketCable CCC Timestamp",
+				"pkt_ccc.ts",
+				FT_BYTES,
+				BASE_NONE,
+				NULL,
+				0x0,
+				"Timestamp", HFILL
+			}
+		},
+
+	};
+
+	static gint *ett[] =
+	{
+		&ett_pkt_ccc,
+	};
+
+	module_t *pkt_ccc_module;
+
+
+	proto_pkt_ccc = proto_register_protocol("PacketCable Call Content Connection",
+	    "PKT CCC", "pkt_ccc");
+	proto_register_field_array(proto_pkt_ccc, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
+
+	register_dissector("pkt_ccc", dissect_pkt_ccc, proto_pkt_ccc);
+
+	pkt_ccc_module = prefs_register_protocol(proto_pkt_ccc, NULL);
+
+	prefs_register_uint_preference(pkt_ccc_module, "udp_port",
+	                               "UDP port",
+	                               "Decode packets on this UDP port as PacketCable CCC",
+	                               10, &global_pkt_ccc_udp_port);
+}
+
+void
+proto_reg_handoff_pkt_ccc(void)
+{
+	/*
+	 * Register this dissector as one that can be selected by a
+	 * UDP port number.
+	 */
+	pkt_ccc_handle = find_dissector("pkt_ccc");
+	dissector_add_handle("udp.port", pkt_ccc_handle);
+}
+
+/* Register RTP */
 
 void
 proto_register_rtp(void)
@@ -1101,10 +1204,9 @@ proto_register_rtp(void)
 
 	prefs_register_enum_preference(rtp_module, "version0_type",
 	                               "Treat RTP version 0 packets as",
-	                               "If an RTP version 0 packet is encountered, it can be treated as an invalid packet or a STUN packet",
+	                               "If an RTP version 0 packet is encountered, it can be treated as an invalid packet, a STUN packet, or a T.38 packet",
 	                               &global_rtp_version0_type,
 	                               rtp_version0_types, FALSE);
-	register_init_routine( &rtp_init );
 }
 
 void
@@ -1112,6 +1214,7 @@ proto_reg_handoff_rtp(void)
 {
 	data_handle = find_dissector("data");
 	stun_handle = find_dissector("stun");
+	t38_handle = find_dissector("t38");
 	/*
 	 * Register this dissector as one that can be selected by a
 	 * UDP port number.
