@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-rtsp.c 13699 2005-03-10 20:23:04Z gerald $
+ * $Id: packet-rtsp.c 16048 2005-09-28 22:15:38Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -46,6 +46,7 @@
 #include <epan/conversation.h>
 #include <epan/strutil.h>
 #include "packet-e164.h"
+#include <epan/emem.h>
 
 static int proto_rtsp		= -1;
 
@@ -53,33 +54,37 @@ static gint ett_rtsp		= -1;
 static gint ett_rtspframe	= -1;
 static gint ett_rtsp_method 	= -1;
 
+static int hf_rtsp_request	= -1;
+static int hf_rtsp_response	= -1;
+static int hf_rtsp_content_type	= -1;
+static int hf_rtsp_content_length	= -1;
 static int hf_rtsp_method	= -1;
 static int hf_rtsp_url		= -1;
 static int hf_rtsp_status	= -1;
 static int hf_rtsp_session	= -1;
+static int hf_rtsp_transport	= -1;
+static int hf_rtsp_rdtfeaturelevel	= -1;
 static int hf_rtsp_X_Vig_Msisdn	= -1;
 
 static dissector_handle_t sdp_handle;
 static dissector_handle_t rtp_handle;
 static dissector_handle_t rtcp_handle;
+static dissector_handle_t rdt_handle;
 
 void proto_reg_handoff_rtsp(void);
 
-static GMemChunk *rtsp_vals = NULL;
-#define rtsp_hash_init_count 20
-
 /*
  * desegmentation of RTSP headers
  * (when we are over TCP or another protocol providing the desegmentation API)
  */
-static gboolean rtsp_desegment_headers = FALSE;
+static gboolean rtsp_desegment_headers = TRUE;
 
 /*
  * desegmentation of RTSP bodies
  * (when we are over TCP or another protocol providing the desegmentation API)
  * TODO let the user filter on content-type the bodies he wants desegmented
  */
-static gboolean rtsp_desegment_body = FALSE;
+static gboolean rtsp_desegment_body = TRUE;
 
 /* http://www.iana.org/assignments/port-numberslists two rtsp ports */
 #define TCP_PORT_RTSP			554
@@ -253,15 +258,17 @@ dissect_rtspinterleaved(tvbuff_t *tvb, int offset, packet_info *pinfo,
 }
 
 static void process_rtsp_request(tvbuff_t *tvb, int offset, const guchar *data,
-	size_t linelen, proto_tree *tree);
+                                 size_t linelen, size_t next_line_offset,
+                                 proto_tree *tree);
 
 static void process_rtsp_reply(tvbuff_t *tvb, int offset, const guchar *data,
-	size_t linelen, proto_tree *tree);
+                               size_t linelen, size_t next_line_offset,
+                               proto_tree *tree);
 
 typedef enum {
 	RTSP_REQUEST,
 	RTSP_REPLY,
-	NOT_RTSP
+	RTSP_NOT_FIRST_LINE
 } rtsp_type_t;
 
 static const char *rtsp_methods[] = {
@@ -303,12 +310,15 @@ is_rtsp_request_or_reply(const guchar *line, size_t linelen, rtsp_type_t *type)
 		size_t len = strlen(rtsp_methods[ii]);
 		if (linelen >= len &&
 		    strncasecmp(rtsp_methods[ii], line, len) == 0 &&
-		    (len == linelen || isspace(line[len]))) {
+		    (len == linelen || isspace(line[len])))
+		{
 			*type = RTSP_REQUEST;
 			return TRUE;
 		}
 	}
-	*type = NOT_RTSP;
+
+	/* Wasn't a request or a response */
+	*type = RTSP_NOT_FIRST_LINE;
 	return FALSE;
 }
 
@@ -342,12 +352,14 @@ static const char rtsp_transport[] = "Transport:";
 static const char rtsp_sps[] = "server_port=";
 static const char rtsp_cps[] = "client_port=";
 static const char rtsp_rtp[] = "rtp/";
+static const char rtsp_rdt_feature_level[] = "RDTFeatureLevel";
 static const char rtsp_real_rdt[] = "x-real-rdt/";
+static const char rtsp_real_tng[] = "x-pn-tng/"; /* synonym for x-real-rdt */
 static const char rtsp_inter[] = "interleaved=";
 
 static void
 rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
-	size_t line_len)
+                         size_t line_len, gint rdt_feature_level)
 {
 	conversation_t	*conv;
 	guchar		buf[256];
@@ -357,10 +369,10 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 	guint		c_data_port, c_mon_port;
 	guint		s_data_port, s_mon_port;
 
-	if (line_len > sizeof(buf) - 1) {
-		/*
-		 * Don't overflow the buffer.
-		 */
+	/* Copy line into buf */
+	if (line_len > sizeof(buf) - 1)
+	{
+		/* Don't overflow the buffer. */
 		line_len = sizeof(buf) - 1;
 	}
 	memcpy(buf, line_begin, line_len);
@@ -370,15 +382,17 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 	tmp = buf + STRLEN_CONST(rtsp_transport);
 	while (*tmp && isspace(*tmp))
 		tmp++;
-	
+
 	/* Work out which transport type is here */
 	if (strncasecmp(tmp, rtsp_rtp, strlen(rtsp_rtp)) == 0)
 		rtp_transport = TRUE;
 	else
-	if (strncasecmp(tmp, rtsp_real_rdt, strlen(rtsp_real_rdt)) == 0)
+	if (strncasecmp(tmp, rtsp_real_rdt, strlen(rtsp_real_rdt)) == 0 ||
+	    strncasecmp(tmp, rtsp_real_tng, strlen(rtsp_real_tng)) == 0)
 		rdt_transport = TRUE;
 	else
 	{
+		/* Give up on unknown transport types */
 		return;
 	}
 	
@@ -403,14 +417,15 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 			return;
 		}
 	}
+	
+	
+	/* Deal with RTSP TCP-interleaved conversations. */
 	if (!c_data_port) {
 		rtsp_conversation_data_t	*data;
 		guint				s_data_chan, s_mon_chan;
 		int				i;
 
-		/*
-		 * Deal with RTSP TCP-interleaved conversations.
-		 */
+		/* Search tranport line for interleaved string */
 		if ((tmp = strstr(buf, rtsp_inter)) == NULL) {
 			/*
 			 * No interleaved or server_port - probably a
@@ -418,26 +433,41 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 			 */
 			return;
 		}
+		
+		/* Move tmp to beyone interleaved string */
 		tmp += strlen(rtsp_inter);
+		/* Look for channel number(s) */
 		i = sscanf(tmp, "%u-%u", &s_data_chan, &s_mon_chan);
-		if (i < 1) {
-			g_warning("Frame %u: rtsp: bad interleaved",
-				pinfo->fd->num);
+		if (i < 1)
+		{
+			g_warning("Frame %u: rtsp: bad interleaved", pinfo->fd->num);
 			return;
 		}
+		
+		/* At least data channel present, look for conversation (presumably TCP) */
 		conv = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype,
-			pinfo->srcport, pinfo->destport, 0);
-		if (!conv) {
+		                         pinfo->srcport, pinfo->destport, 0);
+
+		/* Create new conversation if necessary */
+		if (!conv)
+		{
 			conv = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst,
-				pinfo->ptype, pinfo->srcport, pinfo->destport,
-				0);
-		}
-		data = conversation_get_proto_data(conv, proto_rtsp);
-		if (!data) {
-			data = g_mem_chunk_alloc(rtsp_vals);
-			conversation_add_proto_data(conv, proto_rtsp, data);
+			                        pinfo->ptype, pinfo->srcport, pinfo->destport,
+			                        0);
 		}
 		
+		/* Look for previous data */
+		data = conversation_get_proto_data(conv, proto_rtsp);
+
+		/* Create new data if necessary */
+		if (!data)
+		{
+			data = se_alloc(sizeof(rtsp_conversation_data_t));
+			conversation_add_proto_data(conv, proto_rtsp, data);
+		}
+
+		/* Now set the dissector handle of the interleaved channel
+		   according to the transport protocol used */
 		if (rtp_transport)
 		{
 			if (s_data_chan < RTSP_MAX_INTERLEAVED) {
@@ -449,6 +479,13 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 					rtcp_handle;
 			}
 		}
+		else if (rdt_transport)
+		{
+			if (s_data_chan < RTSP_MAX_INTERLEAVED) {
+				data->interleaved[s_data_chan].dissector =
+					rdt_handle;
+			}
+		}
 		return;
 	}
 
@@ -461,20 +498,23 @@ rtsp_create_conversation(packet_info *pinfo, const guchar *line_begin,
 	 */
 	if (rtp_transport)
 	{
+		/* There is always data for RTP */
 		rtp_add_address(pinfo, &pinfo->dst, c_data_port, s_data_port,
-						"RTSP", pinfo->fd->num, 0);
+		                "RTSP", pinfo->fd->num, NULL);
 	
-		if (!c_mon_port)
-			return;
-	
-		rtcp_add_address(pinfo, &pinfo->dst, c_mon_port, s_mon_port,
-						 "RTSP", pinfo->fd->num);
+		/* RTCP only if indicated */
+		if (c_mon_port)
+		{
+			rtcp_add_address(pinfo, &pinfo->dst, c_mon_port, s_mon_port,
+			                 "RTSP", pinfo->fd->num);
+		}
 	}
 	else
 	if (rdt_transport)
 	{
+		/* Real Data Transport */
 		rdt_add_address(pinfo, &pinfo->dst, c_data_port, s_data_port,
-						"RTSP", pinfo->fd->num);
+		                "RTSP", rdt_feature_level);
 	}
 }
 
@@ -538,6 +578,7 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	int			value_offset;
 	int			value_len;
 	e164_info_t		e164_info;
+	gint		rdt_feature_level = 0;
 
 	/*
 	 * Is this a request or response?
@@ -673,8 +714,7 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		/*
 		 * OK, does it look like an RTSP request or response?
 		 */
-		is_request_or_reply = is_rtsp_request_or_reply(line, linelen,
-		    &rtsp_type);
+		is_request_or_reply = is_rtsp_request_or_reply(line, linelen, &rtsp_type);
 		if (is_request_or_reply)
 			goto is_rtsp;
 
@@ -796,8 +836,8 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			 * request or reply.
 			 */
 			proto_tree_add_text(rtsp_tree, tvb, offset,
-			    next_offset - offset, "%s",
-			    tvb_format_text(tvb, offset, next_offset - offset));
+			                    next_offset - offset, "%s",
+			                    tvb_format_text(tvb, offset, next_offset - offset));
 			offset = next_offset;
 			break;
 		}
@@ -808,29 +848,36 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 */
 		saw_req_resp_or_header = TRUE;
 		if (rtsp_tree) {
-			ti = proto_tree_add_text(rtsp_tree, tvb, offset,
-				next_offset - offset, "%s",
-				tvb_format_text(tvb, offset, next_offset - offset));
 
-			sub_tree = proto_item_add_subtree(ti, ett_rtsp_method);
-
-			switch (rtsp_type) {
-
-			case RTSP_REQUEST:
-				process_rtsp_request(tvb, offset, line, linelen,
-				    sub_tree);
-				break;
-
-			case RTSP_REPLY:
-				process_rtsp_reply(tvb, offset, line, linelen,
-				    sub_tree);
-				break;
-
-			case NOT_RTSP:
-				break;
+			switch (rtsp_type)
+			{
+				case RTSP_REQUEST:
+					process_rtsp_request(tvb, offset, line, linelen, next_offset, rtsp_tree);
+					break;
+	
+				case RTSP_REPLY:
+					process_rtsp_reply(tvb, offset, line, linelen, next_offset, rtsp_tree);
+					break;
+	
+				case RTSP_NOT_FIRST_LINE:
+					/* Drop through, it may well be a header line */
+					break;
 			}
 		}
-		if (is_header) {
+
+		if (is_header)
+		{
+			/* We know that colon_offset must be set */
+			
+			/* Skip whitespace after the colon. */
+			value_offset = colon_offset + 1;
+			while ((value_offset < line_end_offset) &&
+				   ((c = tvb_get_guint8(tvb, value_offset)) == ' ' || c == '\t'))
+			{
+				value_offset++;
+			}
+			value_len = line_end_offset - value_offset;
+
 			/*
 			 * Process some headers specially.
 			 */
@@ -838,15 +885,27 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	( (size_t)linelen > STRLEN_CONST(header) && \
 	 strncasecmp(line, (header), STRLEN_CONST(header)) == 0)
 
-			if (HDR_MATCHES(rtsp_transport)) {
+			if (HDR_MATCHES(rtsp_transport))
+			{
+				proto_tree_add_string(rtsp_tree, hf_rtsp_transport, tvb,
+				                      offset, linelen,
+				                      tvb_format_text(tvb, value_offset,
+				                                      value_len));
+
 				/*
 				 * Based on the port numbers specified
 				 * in the Transport: header, set up
 				 * a conversation that will be dissected
 				 * with the appropriate dissector.
 				 */
-				rtsp_create_conversation(pinfo, line, linelen);
-			} else if (HDR_MATCHES(rtsp_content_type)) {
+				rtsp_create_conversation(pinfo, line, linelen, rdt_feature_level);
+			} else if (HDR_MATCHES(rtsp_content_type))
+			{
+				proto_tree_add_string(rtsp_tree, hf_rtsp_content_type,
+				                      tvb, offset, linelen,
+				                      tvb_format_text(tvb, value_offset,
+				                                      value_len));
+
 				/*
 				 * If the Content-Type: header says this
 				 * is SDP, dissect the payload as SDP.
@@ -863,84 +922,83 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				 * HTTP).
 				 */
 				if (is_content_sdp(line, linelen))
+				{
 					is_sdp = TRUE;
-			} else if (HDR_MATCHES(rtsp_content_length)) {
+				}
+			} else if (HDR_MATCHES(rtsp_content_length))
+			{
+				proto_tree_add_uint(rtsp_tree, hf_rtsp_content_length,
+				                    tvb, offset, linelen,
+				                    atoi(tvb_format_text(tvb, value_offset,
+				                                         value_len)));
+
 				/*
 				 * Only the amount specified by the
 				 * Content-Length: header should be treated
 				 * as payload.
 				 */
-				content_length = rtsp_get_content_length(line,
-				    linelen);
+				content_length = rtsp_get_content_length(line, linelen);
 
-			} else if (HDR_MATCHES(rtsp_Session)) {
-				/*
-				 * Extract the session string
-
-				 */
-
-				if ( colon_offset != -1 ){
-					/*
-					* Skip whitespace after the colon.
-					* (Code from SIP dissector )
-					*/
-					value_offset = colon_offset + 1;
-					while (value_offset < line_end_offset
-						&& ((c = tvb_get_guint8(tvb,
-							value_offset)) == ' '
-						|| c == '\t'))
-						value_offset++;
-					/*
-					 * Put the value into the protocol tree
-					*/
-					value_len = line_end_offset - value_offset;
-					proto_tree_add_string(sub_tree, hf_rtsp_session,tvb,
-						value_offset, value_len ,
-						tvb_format_text(tvb, value_offset, value_len));
-				}
+			} else if (HDR_MATCHES(rtsp_Session))
+			{
+				/* Put the value into the protocol tree */
+				proto_tree_add_string(rtsp_tree, hf_rtsp_session, tvb,
+				                      offset, linelen,
+				                      tvb_format_text(tvb, value_offset, value_len));
 
 			} else if (HDR_MATCHES(rtsp_X_Vig_Msisdn)) {
 				/*
 				 * Extract the X_Vig_Msisdn string
 				 */
-				if ( colon_offset != -1 ){
-					/*
-					 * Skip whitespace after the colon.
-					 * (Code from SIP dissector )
-					 */
-					value_offset = colon_offset + 1;
-					while (value_offset < line_end_offset
-						&& ((c = tvb_get_guint8(tvb,
-						    value_offset)) == ' '
-						  || c == '\t'))
-						value_offset++;
-					/*
-					 * Put the value into the protocol tree
-					 */
-					value_len = line_end_offset - value_offset;
-					proto_tree_add_string(sub_tree, hf_rtsp_X_Vig_Msisdn,tvb,
-						value_offset, value_len ,
-						tvb_format_text(tvb, value_offset, value_len));
+				if (colon_offset != -1)
+				{
+					/* Put the value into the protocol tree */
+					ti = proto_tree_add_string(rtsp_tree, hf_rtsp_X_Vig_Msisdn,tvb,
+					                           offset, linelen ,
+					                           tvb_format_text(tvb, value_offset, value_len));
+					sub_tree = proto_item_add_subtree(ti, ett_rtsp_method);
 
 					e164_info.e164_number_type = CALLING_PARTY_NUMBER;
 					e164_info.nature_of_address = 0;
 
-					e164_info.E164_number_str = tvb_get_string(tvb, value_offset,
-						value_len);
+					e164_info.E164_number_str = tvb_get_ephemeral_string(tvb, value_offset,
+					                                              value_len);
 					e164_info.E164_number_length = value_len;
 					dissect_e164_number(tvb, sub_tree, value_offset,
-						value_len, e164_info);
-					g_free(e164_info.E164_number_str);
-
-
+					                    value_len, e164_info);
 				}
+			} else if (HDR_MATCHES(rtsp_rdt_feature_level))
+			{
+				rdt_feature_level = atoi(tvb_format_text(tvb, value_offset,
+				                                         value_len));
+				proto_tree_add_uint(rtsp_tree, hf_rtsp_rdtfeaturelevel,
+				                    tvb, offset, linelen,
+				                    atoi(tvb_format_text(tvb, value_offset,
+				                                         value_len)));
+			}
+			else
+			{
+				/* Default case for headers. Show line as text */
+				proto_tree_add_text(rtsp_tree, tvb, offset,
+				                    next_offset - offset, "%s",
+				                    tvb_format_text(tvb, offset, next_offset - offset));
 			}
-
 		}
+		else if (rtsp_type == RTSP_NOT_FIRST_LINE)
+		{
+			/* Catch-all for all other lines... Show line as text.
+			   TODO: should these be shown as errors? */
+			proto_tree_add_text(rtsp_tree, tvb, offset,
+			                    next_offset - offset, "%s",
+			                    tvb_format_text(tvb, offset, next_offset - offset));
+		}
+		
 		offset = next_offset;
 	}
 
 	/*
+	 * Have now read all of the lines of this message.
+	 *
 	 * If a content length was supplied, the amount of data to be
 	 * processed as RTSP payload is the minimum of the content
 	 * length and the amount of data remaining in the frame.
@@ -1046,8 +1104,10 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 static void
 process_rtsp_request(tvbuff_t *tvb, int offset, const guchar *data,
-	size_t linelen, proto_tree *tree)
+                     size_t linelen, size_t next_line_offset, proto_tree *tree)
 {
+	proto_tree		*sub_tree = NULL;
+	proto_item		*ti = NULL;
 	const guchar	*lineend = data + linelen;
 	unsigned		ii;
 	const guchar	*url;
@@ -1068,52 +1128,81 @@ process_rtsp_request(tvbuff_t *tvb, int offset, const guchar *data,
 		 * RTSP_REQUEST, so we know one of the request methods
 		 * matched, so we "can't get here".
 		 */
-		g_assert_not_reached();
+		DISSECTOR_ASSERT_NOT_REACHED();
 	}
 
-	/* Method name */
-	proto_tree_add_string(tree, hf_rtsp_method, tvb, offset,
-		strlen(rtsp_methods[ii]), rtsp_methods[ii]);
+	/* Add a tree for this request */
+	ti = proto_tree_add_string(tree, hf_rtsp_request, tvb, offset,
+	                          next_line_offset - offset,
+	                          tvb_format_text(tvb, offset, next_line_offset - offset));
+	sub_tree = proto_item_add_subtree(ti, ett_rtsp_method);
 
+	
+	/* Add method name to tree */
+	proto_tree_add_string(sub_tree, hf_rtsp_method, tvb, offset,
+	                      strlen(rtsp_methods[ii]), rtsp_methods[ii]);
 
 	/* URL */
 	url = data;
+	/* Skip method name again */
 	while (url < lineend && !isspace(*url))
 		url++;
+	/* Skip spaces */
 	while (url < lineend && isspace(*url))
 		url++;
+	/* URL starts here */
 	url_start = url;
+	/* Scan to end of URL */
 	while (url < lineend && !isspace(*url))
 		url++;
-	tmp_url = g_malloc(url - url_start + 1);
+	/* Create a URL-sized buffer and copy contents */
+	tmp_url = ep_alloc(url - url_start + 1);
 	memcpy(tmp_url, url_start, url - url_start);
-	tmp_url[url - url_start] = 0;
-	proto_tree_add_string(tree, hf_rtsp_url, tvb,
-		offset + (url_start - data), url - url_start, tmp_url);
-	g_free(tmp_url);
+	tmp_url[url - url_start] = '\0';
+	
+	/* Add URL to tree */
+	proto_tree_add_string(sub_tree, hf_rtsp_url, tvb,
+	                      offset + (url_start - data), url - url_start, tmp_url);
 }
 
+/* Read first line of a reply message */
 static void
 process_rtsp_reply(tvbuff_t *tvb, int offset, const guchar *data,
-	size_t linelen, proto_tree *tree)
+	size_t linelen, size_t next_line_offset, proto_tree *tree)
 {
+	proto_tree		*sub_tree = NULL;
+	proto_item		*ti = NULL;
 	const guchar	*lineend = data + linelen;
 	const guchar	*status = data;
 	const guchar	*status_start;
 	unsigned int	status_i;
 
+	/* Add a tree for this request */
+	ti = proto_tree_add_string(tree, hf_rtsp_response, tvb, offset,
+	                           next_line_offset - offset,
+	                           tvb_format_text(tvb, offset, next_line_offset - offset));
+	sub_tree = proto_item_add_subtree(ti, ett_rtsp_method);
+
+
 	/* status code */
+	
+	/* Skip protocol/version */
 	while (status < lineend && !isspace(*status))
 		status++;
+	/* Skip spaces */
 	while (status < lineend && isspace(*status))
 		status++;
+
+	/* Actual code number now */
 	status_start = status;
 	status_i = 0;
 	while (status < lineend && isdigit(*status))
 		status_i = status_i * 10 + *status++ - '0';
-	proto_tree_add_uint(tree, hf_rtsp_status, tvb,
-		offset + (status_start - data),
-		status - status_start, status_i);
+	
+	/* Add field to tree */
+	proto_tree_add_uint(sub_tree, hf_rtsp_status, tvb,
+	                    offset + (status_start - data),
+	                    status - status_start, status_i);
 }
 
 static void
@@ -1139,21 +1228,6 @@ dissect_rtsp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 }
 
-static void
-rtsp_init(void)
-{
-/* Routine to initialize rtsp protocol before each capture or filter pass. */
-/* Release any memory if needed.  Then setup the memory chunks.		*/
-
-  	if (rtsp_vals)
-   		g_mem_chunk_destroy(rtsp_vals);
-
-  	rtsp_vals = g_mem_chunk_new("rtsp_vals",
-		sizeof(rtsp_conversation_data_t),
-		rtsp_hash_init_count * sizeof(rtsp_conversation_data_t),
-		G_ALLOC_AND_FREE);
-}
-
 void
 proto_register_rtsp(void)
 {
@@ -1163,9 +1237,21 @@ proto_register_rtsp(void)
 		&ett_rtsp_method,
 	};
 	static hf_register_info hf[] = {
+		{ &hf_rtsp_request,
+			{ "Request", "rtsp.request", FT_STRING, BASE_NONE, NULL, 0,
+			"", HFILL }},
+		{ &hf_rtsp_response,
+			{ "Response", "rtsp.response", FT_STRING, BASE_NONE, NULL, 0,
+			"", HFILL }},
 		{ &hf_rtsp_method,
 			{ "Method", "rtsp.method", FT_STRING, BASE_NONE, NULL, 0,
 			"", HFILL }},
+		{ &hf_rtsp_content_type,
+			{ "Content-type", "rtsp.content-type", FT_STRING, BASE_NONE, NULL, 0,
+			"", HFILL }},
+		{ &hf_rtsp_content_length,
+			{ "Content-length", "rtsp.content-length", FT_UINT32, BASE_DEC, NULL, 0,
+			"", HFILL }},
 		{ &hf_rtsp_url,
 			{ "URL", "rtsp.url", FT_STRING, BASE_NONE, NULL, 0,
 			"", HFILL }},
@@ -1175,6 +1261,12 @@ proto_register_rtsp(void)
 		{ &hf_rtsp_session,
 			{ "Session", "rtsp.session", FT_STRING, BASE_NONE, NULL, 0,
 			"", HFILL }},
+		{ &hf_rtsp_transport,
+			{ "Transport", "rtsp.transport", FT_STRING, BASE_NONE, NULL, 0,
+			"", HFILL }},
+		{ &hf_rtsp_rdtfeaturelevel,
+			{ "RDTFeatureLevel", "rtsp.rdt-feature-level", FT_UINT32, BASE_DEC, NULL, 0,
+			"", HFILL }},
 		{ &hf_rtsp_X_Vig_Msisdn,
 			{ "X-Vig-Msisdn", "X_Vig_Msisdn", FT_STRING, BASE_NONE, NULL, 0,
 			"", HFILL }},
@@ -1213,7 +1305,6 @@ proto_register_rtsp(void)
 	    "of a request spanning multiple TCP segments",
 	    &rtsp_desegment_body);
 
-	register_init_routine(rtsp_init);	/* register re-init routine */
 }
 
 void
@@ -1242,4 +1333,5 @@ proto_reg_handoff_rtsp(void)
 	sdp_handle = find_dissector("sdp");
 	rtp_handle = find_dissector("rtp");
 	rtcp_handle = find_dissector("rtcp");
+	rdt_handle = find_dissector("rdt");
 }
