@@ -1,8 +1,8 @@
 /* packet-sctp.h
  *
- * Defintion of SCTP specific structures used b tap listeners.
+ * Defintion of SCTP specific structures used by tap listeners.
  *
- * $Id: packet-sctp.h 13294 2005-02-04 22:39:48Z tuexen $
+ * $Id: packet-sctp.h 14278 2005-05-02 20:45:22Z tuexen $
  * Copyright 2004 Michael Tuexen <tuexen [AT] fh-muenster.de>
 
  * Ethereal - Network traffic analyzer
@@ -36,6 +36,12 @@ struct _sctp_info {
   gboolean crc32c_calculated;
   gboolean crc32c_correct;
   gboolean checksum_zero;
+  gboolean vtag_reflected;
+  /* FIXME: do we need the ports and addresses to be here? */
+  guint16 sport;
+  guint16 dport;
+  address ip_src;
+  address ip_dst;
   guint32 verification_tag;
   guint32 number_of_tvbs;
   tvbuff_t *tvb[MAXIMUM_NUMBER_OF_TVBS];
