@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-sdp.c 13699 2005-03-10 20:23:04Z gerald $
+ * $Id: packet-sdp.c 17056 2006-01-19 23:01:24Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,9 +40,6 @@
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#ifdef NEED_INET_ATON_H
-# include <epan/inet_aton.h>
-#endif
 
 #ifdef HAVE_WINSOCK2_H
 #include <winsock2.h>		/* needed to define AF_ values on Windows */
@@ -56,12 +53,13 @@
 #include <epan/packet.h>
 #include <epan/conversation.h>
 #include <epan/strutil.h>
+#include <epan/emem.h>
 
 #include "tap.h"
 #include "packet-sdp.h"
 
 #include "packet-rtp.h"
-#include "rtp_pt.h"
+#include <epan/rtp_pt.h>
 #include "packet-rtcp.h"
 
 #include "packet-t38.h"
@@ -72,7 +70,6 @@ static dissector_handle_t rtcp_handle=NULL;
 static dissector_handle_t t38_handle=NULL;
 
 static int sdp_tap = -1;
-static sdp_packet_info *sdp_pi;
 
 static int proto_sdp = -1;
 
@@ -96,6 +93,8 @@ static int hf_media = -1;
 static int hf_media_title = -1;
 static int hf_unknown = -1;
 static int hf_invalid = -1;
+static int hf_ipbcp_version = -1;
+static int hf_ipbcp_type = -1;
 
 /* hf_owner subfields*/
 static int hf_owner_username = -1;
@@ -163,14 +162,21 @@ static int ett_sdp_media_attribute = -1;
 
 
 #define SDP_MAX_RTP_CHANNELS 4
+#define SDP_MAX_RTP_PAYLOAD_TYPES 20
+
+typedef struct {
+	gint32 pt[SDP_MAX_RTP_PAYLOAD_TYPES];
+	gint8 pt_count;
+	GHashTable *rtp_dyn_payload;
+} transport_media_pt_t;
 
 typedef struct {
 	char *connection_address;
 	char *connection_type;
 	char *media_port[SDP_MAX_RTP_CHANNELS];
 	char *media_proto[SDP_MAX_RTP_CHANNELS];
+	transport_media_pt_t media[SDP_MAX_RTP_CHANNELS];
 	gint8 media_count;
-	guint32 rtp_event_pt;
 } transport_info_t;
 
 /* static functions */
@@ -219,24 +225,25 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean 	is_ipv4_addr=FALSE;
 	gboolean	is_ipv6_addr=FALSE;
     guint32 	ipaddr[4];
-	gint		n;
+	gint		n,i;
+	sdp_packet_info *sdp_pi;
 
     /* Initialise packet info for passing to tap */
-    sdp_pi = g_malloc(sizeof(sdp_packet_info));
+    sdp_pi = ep_alloc(sizeof(sdp_packet_info));
     sdp_pi->summary_str[0] = '\0';
 
 	/* Initialise RTP channel info */
 	transport_info.connection_address=NULL;
 	transport_info.connection_type=NULL;
-	transport_info.rtp_event_pt=0;
 	for (n=0; n < SDP_MAX_RTP_CHANNELS; n++)
 	{
 	    transport_info.media_port[n]=NULL;
 	    transport_info.media_proto[n]=NULL;
+		transport_info.media[n].pt_count = 0;
+		transport_info.media[n].rtp_dyn_payload = g_hash_table_new( g_int_hash, g_int_equal);
 	}
 	transport_info.media_count = 0;
 
-
 	/*
 	 * As RFC 2327 says, "SDP is purely a format for session
 	 * description - it does not incorporate a transport protocol,
@@ -355,7 +362,7 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		tokenoffset = 2;
 		if (hf == hf_unknown)
 			tokenoffset = 0;
-		string = tvb_get_string(tvb, offset + tokenoffset,
+		string = tvb_get_ephemeral_string(tvb, offset + tokenoffset,
 		    linelen - tokenoffset);
 		sub_ti = proto_tree_add_string_format(sdp_tree,hf,tvb, offset,
 					       linelen, string,
@@ -363,7 +370,6 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					       proto_registrar_get_name(hf),
 					       format_text(string,
 					         linelen - tokenoffset));
-		g_free(string);
 		call_sdp_subdissector(tvb_new_subset(tvb,offset+tokenoffset,
 						     linelen-tokenoffset,
 						     linelen-tokenoffset),
@@ -382,7 +388,6 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	{
 	    if(transport_info.media_port[n]!=NULL) {
 		    port = atol(transport_info.media_port[n]);
-		    g_free(transport_info.media_port[n]);
 	    }
 	    if(transport_info.media_proto[n]!=NULL) {
 		    /* Check if media protocol is RTP */
@@ -390,7 +395,6 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		    /* Check if media protocol is T38 */
 		    is_t38 = ( (strcmp(transport_info.media_proto[n],"UDPTL")==0) || (strcmp(transport_info.media_proto[n],"udptl")==0) );
 
-		    g_free(transport_info.media_proto[n]);
 	    }
 	    if(transport_info.connection_address!=NULL) {
 		    if(transport_info.connection_type!=NULL) {
@@ -412,12 +416,13 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				}
 		    }
 	    }
+		set_rtp = FALSE;
 	    /* Add rtp and rtcp conversation, if available (overrides t38 if conversation already set) */
 	    if((!pinfo->fd->flags.visited) && port!=0 && is_rtp && (is_ipv4_addr || is_ipv6_addr)){
 		    src_addr.data=(char *)&ipaddr;
 		    if(rtp_handle){
 				rtp_add_address(pinfo, &src_addr, port, 0,
-	                "SDP", pinfo->fd->num, transport_info.rtp_event_pt);
+	                "SDP", pinfo->fd->num, transport_info.media[n].rtp_dyn_payload);
 				set_rtp = TRUE;
 		    }
 		    if(rtcp_handle){
@@ -425,23 +430,42 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				rtcp_add_address(pinfo, &src_addr, port, 0,
 	                 "SDP", pinfo->fd->num);
 		    }
-	    }
+		} 
 			
 	    /* Add t38 conversation, if available and only if no rtp */
 	    if((!pinfo->fd->flags.visited) && port!=0 && !set_rtp && is_t38 && is_ipv4_addr){
-                    src_addr.data=(char *)&ipaddr;
-                    if(t38_handle){
-                                t38_add_address(pinfo, &src_addr, port, 0, "SDP", pinfo->fd->num);
-                    }
+			src_addr.data=(char *)&ipaddr;
+            if(t38_handle){
+				t38_add_address(pinfo, &src_addr, port, 0, "SDP", pinfo->fd->num);
+            }	
 	    }
+
+		/* Create the RTP summary str for the Voip Call analysis */
+		for (i = 0; i < transport_info.media[n].pt_count; i++)
+		{
+			/* if the payload type is dynamic (96 to 127), check the hash table to add the desc in the SDP summary */
+			if ( (transport_info.media[n].pt[i] >=96) && (transport_info.media[n].pt[i] <=127) ) {
+				gchar *str_dyn_pt = g_hash_table_lookup(transport_info.media[n].rtp_dyn_payload, &transport_info.media[n].pt[i]);
+				if (str_dyn_pt)
+					g_snprintf(sdp_pi->summary_str, 50, "%s %s", sdp_pi->summary_str, str_dyn_pt);
+				else
+					g_snprintf(sdp_pi->summary_str, 50, "%s %d", sdp_pi->summary_str, transport_info.media[n].pt[i]);
+			} else 
+				g_snprintf(sdp_pi->summary_str, 50, "%s %s", sdp_pi->summary_str, val_to_str(transport_info.media[n].pt[i], rtp_payload_type_short_vals, "%u"));
+		}
+
+		/* Free the hash table if we did't assigned it to a conv use it */
+		if (set_rtp == FALSE) 
+			rtp_free_hash_dyn_payload(transport_info.media[n].rtp_dyn_payload);
+
+		/* Create the T38 summary str for the Voip Call analysis */
+		if (is_t38) g_snprintf(sdp_pi->summary_str, 50, "%s t38", sdp_pi->summary_str);	
 	}
 
-	/* Free up 'connection info' strings */
-	if(transport_info.connection_address) {
-		g_free(transport_info.connection_address);
-	}
-	if(transport_info.connection_type!=NULL) {
-		g_free(transport_info.connection_type);
+	/* Free the remainded hash tables not used */
+	for (n = transport_info.media_count; n < SDP_MAX_RTP_CHANNELS; n++)
+	{
+		rtp_free_hash_dyn_payload(transport_info.media[n].rtp_dyn_payload);
 	}
 
 
@@ -450,6 +474,7 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		proto_tree_add_text(sdp_tree, tvb, offset, datalen,
 		    "Data (%d bytes)", datalen);
 	}
+
     /* Report this packet to the tap */
     tap_queue_packet(sdp_tap, pinfo, sdp_pi);
 }
@@ -578,7 +603,7 @@ dissect_sdp_connection_info(tvbuff_t *tvb, proto_item* ti,
     return;
   tokenlen = next_offset - offset;
   /* Save connection address type */
-  transport_info->connection_type = tvb_get_string(tvb, offset, tokenlen);
+  transport_info->connection_type = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
 
   proto_tree_add_item(sdp_connection_info_tree,
@@ -593,11 +618,11 @@ dissect_sdp_connection_info(tvbuff_t *tvb, proto_item* ti,
     tokenlen = -1;	/* end of tvbuff */
     /* Save connection address */
     transport_info->connection_address =
-        tvb_get_string(tvb, offset, tvb_length_remaining(tvb, offset));
+        tvb_get_ephemeral_string(tvb, offset, tvb_length_remaining(tvb, offset));
   } else {
     tokenlen = next_offset - offset;
     /* Save connection address */
-    transport_info->connection_address = tvb_get_string(tvb, offset, tokenlen);
+    transport_info->connection_address = tvb_get_ephemeral_string(tvb, offset, tokenlen);
   }
 
   proto_tree_add_item(sdp_connection_info_tree,
@@ -783,7 +808,8 @@ static void dissect_sdp_encryption_key(tvbuff_t *tvb, proto_item * ti){
 static void dissect_sdp_session_attribute(tvbuff_t *tvb, proto_item * ti){
   proto_tree *sdp_session_attribute_tree;
   gint offset, next_offset, tokenlen;
-
+  guint8 *field_name;
+  
   offset = 0;
   next_offset = 0;
   tokenlen = 0;
@@ -802,18 +828,51 @@ static void dissect_sdp_session_attribute(tvbuff_t *tvb, proto_item * ti){
 		      hf_session_attribute_field,
 		      tvb, offset, tokenlen, FALSE);
 
+  field_name = tvb_get_ephemeral_string(tvb, offset, tokenlen);
+  
   offset = next_offset + 1;
-  proto_tree_add_item(sdp_session_attribute_tree,
-		      hf_session_attribute_value,
-		      tvb, offset, -1, FALSE);
 
+  if (strcmp(field_name, "ipbcp") == 0) {
+	  
+	  offset = tvb_pbrk_guint8(tvb,offset,-1,"0123456789");
+	  
+	  if (offset == -1)
+		  return;
+	  
+	  next_offset = tvb_find_guint8(tvb,offset,-1,' ');
+	  
+	  if (next_offset == -1)
+		  return;
+	  
+	  tokenlen = next_offset - offset;
+	  
+	  proto_tree_add_item(sdp_session_attribute_tree,hf_ipbcp_version,tvb,offset,tokenlen,FALSE);
+
+	  offset = tvb_pbrk_guint8(tvb,offset,-1,"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
+
+	  if (offset == -1)
+		  return;
+
+	  tokenlen = tvb_find_line_end(tvb,offset,-1, &next_offset, FALSE);
+	  
+	  if (tokenlen == -1) 
+		  return;
+	  
+	  proto_tree_add_item(sdp_session_attribute_tree,hf_ipbcp_type,tvb,offset,tokenlen,FALSE);
+	  
+  } else {
+	  proto_tree_add_item(sdp_session_attribute_tree,
+						  hf_session_attribute_value,
+						  tvb, offset, -1, FALSE);
+
+  }
 }
 
 static void
 dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 		  transport_info_t *transport_info){
   proto_tree *sdp_media_tree;
-  gint offset, next_offset, tokenlen;
+  gint offset, next_offset, tokenlen, index;
   guint8 *media_format;
 
   offset = 0;
@@ -843,7 +902,7 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
   if(next_offset != -1){
     tokenlen = next_offset - offset;
     /* Save port info */
-    transport_info->media_port[transport_info->media_count] = tvb_get_string(tvb, offset, tokenlen);
+    transport_info->media_port[transport_info->media_count] = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
     proto_tree_add_item(sdp_media_tree, hf_media_port, tvb,
 			offset, tokenlen, FALSE);
@@ -862,7 +921,7 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
       return;
     tokenlen = next_offset - offset;
     /* Save port info */
-    transport_info->media_port[transport_info->media_count] = tvb_get_string(tvb, offset, tokenlen);
+    transport_info->media_port[transport_info->media_count] = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
     /* XXX Remember Port */
     proto_tree_add_item(sdp_media_tree, hf_media_port, tvb,
@@ -877,7 +936,7 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 
   tokenlen = next_offset - offset;
   /* Save port protocol */
-  transport_info->media_proto[transport_info->media_count] = tvb_get_string(tvb, offset, tokenlen);
+  transport_info->media_proto[transport_info->media_count] = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
   /* XXX Remember Protocol */
   proto_tree_add_item(sdp_media_tree, hf_media_proto, tvb,
@@ -897,11 +956,13 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
     
     if (strcmp(transport_info->media_proto[transport_info->media_count],
                "RTP/AVP") == 0) {
-      media_format = tvb_get_string(tvb, offset, tokenlen);
+      media_format = tvb_get_ephemeral_string(tvb, offset, tokenlen);
       proto_tree_add_string(sdp_media_tree, hf_media_format, tvb, offset,
                              tokenlen, val_to_str(atol(media_format), rtp_payload_type_vals, "%u"));
-      g_snprintf(sdp_pi->summary_str, 50, "%s %s", sdp_pi->summary_str, val_to_str(atol(media_format), rtp_payload_type_short_vals, "%u"));
-      g_free(media_format);
+	  index = transport_info->media[transport_info->media_count].pt_count;
+	  transport_info->media[transport_info->media_count].pt[index] = atol(media_format);
+	  if (index < (SDP_MAX_RTP_PAYLOAD_TYPES-1))
+		  transport_info->media[transport_info->media_count].pt_count++;
     } else {
       proto_tree_add_item(sdp_media_tree, hf_media_format, tvb,
 			  offset, tokenlen, FALSE);
@@ -925,7 +986,7 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 
 static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti, transport_info_t *transport_info){
   proto_tree *sdp_media_attribute_tree;
-  gint offset, next_offset, tokenlen;
+  gint offset, next_offset, tokenlen, n;
   guint8 *field_name;
   guint8 *payload_type;
   guint8 *encoding_name;
@@ -948,31 +1009,31 @@ static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti, transpor
 		      hf_media_attribute_field,
 		      tvb, offset, tokenlen, FALSE);
 
-  field_name = tvb_get_string(tvb, offset, tokenlen);
+  field_name = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
   offset = next_offset + 1;
   proto_tree_add_item(sdp_media_attribute_tree,
 		      hf_media_attribute_value,
 		      tvb, offset, -1, FALSE);
 
-  /* decode the rtpmap to see if it is DynamicPayload for RTP-Events RFC2833 to dissect them automatic */
+  /* decode the rtpmap to see if it is DynamicPayload to dissect them automatic */
   if (strcmp(field_name, "rtpmap") == 0) {
+	gint *key;
+
 	next_offset = tvb_find_guint8(tvb,offset,-1,' ');
-	g_free(field_name);
 
     if(next_offset == -1)
 		return;
 
     tokenlen = next_offset - offset;
 
-	payload_type = tvb_get_string(tvb, offset, tokenlen);
+    payload_type = tvb_get_ephemeral_string(tvb, offset, tokenlen);
 
     offset = next_offset + 1;
 
     next_offset = tvb_find_guint8(tvb,offset,-1,'/');
 
     if(next_offset == -1){
-		g_free(payload_type);
         return;
     }
 
@@ -980,12 +1041,43 @@ static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti, transpor
 
     encoding_name = tvb_get_string(tvb, offset, tokenlen);
 
-    if (strcmp(encoding_name, "telephone-event") == 0)
-		transport_info->rtp_event_pt = atol(payload_type);
+	key=g_malloc( sizeof(gint) );
+	*key=atol(payload_type);
 
-    g_free(payload_type);
-    g_free(encoding_name);
-  } else g_free(field_name);
+	/* As per RFC2327 it is possible to have multiple Media Descriptions ("m=") For example:
+
+			a=rtpmap:101 G726-32/8000
+			m=audio 49170 RTP/AVP 0 97
+			a=rtpmap:97 telephone-event/8000
+			m=audio 49172 RTP/AVP 97 101
+			a=rtpmap:97 G726-24/8000
+
+	The Media attributes ("a="s) after the "m=" only apply for that "m=". 
+	If there is an "a=" before the first "m=", that attribute apply for all the session	(all the "m="s).
+	*/
+
+	/* so, if this "a=" appear before any "m=", we add it to all the dynamic hash tables */ 
+	if (transport_info->media_count == 0) {
+		for (n=0; n < SDP_MAX_RTP_CHANNELS; n++) {
+			if (n==0)
+				g_hash_table_insert(transport_info->media[n].rtp_dyn_payload, key, encoding_name);
+			else {	/* we create a new key and encoding_name to assign to the other hash tables */
+				gint *key2;
+				key2=g_malloc( sizeof(gint) );
+				*key2=atol(payload_type);
+				g_hash_table_insert(transport_info->media[n].rtp_dyn_payload, key2, encoding_name);
+			}
+		}
+
+	/* if the "a=" is after an "m=", only apply to this "m=" */
+	} else 
+		/* in case there is an overflow in SDP_MAX_RTP_CHANNELS, we keep always the last "m=" */
+		if (transport_info->media_count == SDP_MAX_RTP_CHANNELS-1)
+			g_hash_table_insert(transport_info->media[ transport_info->media_count ].rtp_dyn_payload, key, encoding_name);		
+		else
+			g_hash_table_insert(transport_info->media[ transport_info->media_count-1 ].rtp_dyn_payload, key, encoding_name);
+
+  }
 }
 
 void
@@ -1192,7 +1284,14 @@ proto_register_sdp(void)
       { "Media Attribute Value",
 	"sdp.media_attribute.value",FT_STRING, BASE_NONE, NULL, 0x0,
 	"Media Attribute Value", HFILL }},
-
+    { &hf_ipbcp_version,
+      { "IPBCP Protocol Version",
+	"ipbcp.version",FT_STRING, BASE_NONE, NULL, 0x0,
+	"IPBCP Protocol Version", HFILL }},
+    { &hf_ipbcp_type,
+      { "IPBCP Command Type",
+	"ipbcp.command",FT_STRING, BASE_NONE, NULL, 0x0,
+	"IPBCP Command Type", HFILL }},
   };
   static gint *ett[] = {
     &ett_sdp,
