@@ -1,8 +1,8 @@
 /* packet-ses.c
 *
-* Routine to dissect ISO 8327-1 OSI Session Protocol packets
+* Routine to dissect ITU-T Rec. X.225 (1995 E)/ISO 8327-1 OSI Session Protocol packets
 *
-* $Id: packet-ses.c 14786 2005-06-26 19:56:52Z guy $
+* $Id: packet-ses.c 17726 2006-03-25 16:32:46Z gal $
 *
 * Yuriy Sidelnikov <YSidelnikov@hotmail.com>
 *
@@ -37,6 +37,7 @@
 
 #include "packet-ses.h"
 #include "packet-frame.h"
+#include "packet-ber.h"
 #include <epan/prefs.h>
 
 #include <epan/strutil.h>
@@ -146,6 +147,9 @@ static int hf_calling_session_selector = -1;
 /* called session selector */
 static int hf_called_session_selector = -1;
 
+/* activity id */
+static int hf_activity_identifier = -1;
+
 /* serial number */
 static int hf_serial_number = -1;
 
@@ -163,37 +167,37 @@ static int hf_large_second_initial_serial_number = -1;
 
 const value_string ses_vals[] =
 {
-  {SES_CONNECTION_REQUEST,  "Connection request PDU" },
-  {SES_CONNECTION_ACCEPT,    "Connection accept PDU"   },
-  {SES_EXCEPTION_REPORT,    "Exception report PDU"   },
-  {SES_DATA_TRANSFER,  "Data transfer PDU" },
-  {SES_PLEASE_TOKENS,    "Please tokens PDU"   },
-  {SES_EXPEDITED,    "Expedited PDU"   },
-  {SES_PREPARE,    "Prepare PDU"   },
-  {SES_NOT_FINISHED,    "Not finished PDU"   },
-  {SES_FINISH,    "Finish PDU"   },
-  {SES_DISCONNECT,    "Disconnect PDU"   },
-  {SES_REFUSE,    "Refuse PDU"   },
-  {SES_CONNECTION_DATA_OVERFLOW,    "Data overflow PDU"   },
-  {SES_OVERFLOW_ACCEPT,    "Overflow accept PDU"   },
-  {SES_GIVE_TOKENS_CONFIRM,    "Tokens confirm PDU"   },
-  {SES_GIVE_TOKENS_ACK,    "Give tokens ACK PDU"   },
-  {SES_ABORT,    "Abort PDU"   },
-  {SES_ABORT_ACCEPT,    "Abort accept PDU"   },
-  {SES_ACTIVITY_RESUME,    "Activity resume PDU"   },
-  {SES_TYPED_DATA,    "Typed data PDU"   },
-  {SES_RESYNCHRONIZE_ACK,    "Resynchronize ACK PDU"   },
-  {SES_MAJOR_SYNC_POINT,    "Session major sync point PDU"   },
-  {SES_MAJOR_SYNC_ACK,    "Session major sync ACK PDU"   },
-  {SES_ACTIVITY_START,    "Activity start PDU"   },
-  {SES_EXCEPTION_DATA,    "Exception data PDU"   },
-  {SES_MINOR_SYNC_POINT,    "Minor sync point PDU"   },
-  {SES_MINOR_SYNC_ACK,    "Minor sync ACK PDU"   },
-  {SES_RESYNCHRONIZE,    "Resynchronize PDU"   },
-  {SES_ACTIVITY_DISCARD,    "Activity discard PDU"   },
-  {SES_ACTIVITY_DISCARD_ACK,    "Activity discard ACK PDU"   },
-  {SES_CAPABILITY,    "Capability PDU"   },
-  {SES_CAPABILITY_DATA_ACK,    "Capability data ACK PDU"   },
+  {SES_CONNECTION_REQUEST,			"CONNECT (CN) SPDU" },				/* 13 */
+  {SES_CONNECTION_ACCEPT,			"ACCEPT (AC) SPDU" },				/* 14 */
+  {SES_EXCEPTION_REPORT,			"EXCEPTION REPORT (ER) SPDU"   },	/*  0 */
+  {SES_DATA_TRANSFER,				"DATA TRANSFER (DT) SPDU" },		/*  1 */
+  {SES_PLEASE_TOKENS,				"PLEASE TOKENS (PT) SPDU"   },		/*  2 */
+  {SES_EXPEDITED,					"EXPEDITED (EX) SPDU"   },			/*  5 */
+  {SES_PREPARE,						"PREPARE (PR) SPDU"   },			/*  7 */
+  {SES_NOT_FINISHED,				"NOT FINISHED (NF) SPDU"   },		/*  8 */
+  {SES_FINISH,						"FINISH (FN) SPDU"   },				/*  9 */
+  {SES_DISCONNECT,					"DISCONNECT (DN) SPDU"   },			/* 10 */
+  {SES_REFUSE,						"REFUSE (RF) SPDU"   },				/* 12 */
+  {SES_CONNECTION_DATA_OVERFLOW,	"CONNECT DATA OVERFLOW (CDO) SPDU"},/* 15 */
+  {SES_OVERFLOW_ACCEPT,				"OVERFLOW ACCEPT (OA) SPDU"   },	/* 16 */
+  {SES_GIVE_TOKENS_CONFIRM,			"GIVE TOKENS CONFIRM (GTC) SPDU"},	/* 21 */
+  {SES_GIVE_TOKENS_ACK,				"GIVE TOKENS ACK (GTA) SPDU"   },	/* 22 */
+  {SES_ABORT,						"ABORT (AB) SPDU"   },				/* 25 */
+  {SES_ABORT_ACCEPT,				"ABORT ACCEPT (AA) SPDU"   },		/* 26 */
+  {SES_ACTIVITY_RESUME,				"ACTIVITY RESUME (AR) SPDU"   },	/* 29 */
+  {SES_TYPED_DATA,					"TYPED DATA (TD) SPDU"   },			/* 33 */
+  {SES_RESYNCHRONIZE_ACK,			"RESYNCHRONIZE ACK (RA) SPDU"   },	/* 34 */
+  {SES_MAJOR_SYNC_POINT,			"MAJOR SYNC POINT (MAP) SPDU"   },	/* 41 */
+  {SES_MAJOR_SYNC_ACK,				"MAJOR SYNC ACK (MAA) SPDU"   },	/* 42 */
+  {SES_ACTIVITY_START,				"ACTIVITY START (AS) SPDU"   },		/* 45 */
+  {SES_EXCEPTION_DATA,				"EXCEPTION DATA (ED) SPDU"   },		/* 48 */
+  {SES_MINOR_SYNC_POINT,			"MINOR SYNC POINT (MIP) SPDU"   },	/* 49 */
+  {SES_MINOR_SYNC_ACK,				"MINOR SYNC ACK (MIA) SPDU"   },	/* 50 */
+  {SES_RESYNCHRONIZE,				"RESYNCHRONIZE (RS) SPDU"   },		/* 53 */
+  {SES_ACTIVITY_DISCARD,			"ACTIVITY DISCARD (AD) SPDU"   },	/* 57 */
+  {SES_ACTIVITY_DISCARD_ACK,		"ACTIVITY DISCARD ACK (ADA) SPDU" },/* 58 */
+  {SES_CAPABILITY,					"CAPABILITY DATA (CD) SPDU"   },	/* 61 */
+  {SES_CAPABILITY_DATA_ACK,			"CAPABILITY DATA ACK (CDA) SPDU" }, /* 62 */
   {0,             NULL           }
 };
 
@@ -224,6 +228,7 @@ static const value_string param_vals[] =
   {EnclosureItem, "Enclosure Item"},
   {Token_Setting_Item, "Token Setting Item"},
   {Resync_Type, "Resync Type"},
+  {Activity_Identifier, "Activity Identifier"},
   {Serial_Number, "Serial Number"},
   {Linking_Information, "Linking Information"},
   {Reflect_Parameter, "Reflect Parameter"},
@@ -650,6 +655,16 @@ dissect_parameter(tvbuff_t *tvb, int offset, proto_tree *tree,
 		}
 		break;
 
+	case Activity_Identifier:
+		if (param_len == 0)
+			break;
+		if (tree)
+		{
+			dissect_ber_integer(FALSE, pinfo, param_tree, tvb, offset,
+			    hf_activity_identifier, NULL);
+		}
+		break;
+
 	case Serial_Number:
 		if (param_len == 0)
 			break;
@@ -903,6 +918,8 @@ dissect_parameters(tvbuff_t *tvb, int offset, guint16 len, proto_tree *tree,
 			switch(param_type)
 			{
 			case Extended_User_Data:
+				call_pres_dissector(tvb, offset, param_len,
+				    pinfo, tree, param_tree, session);
 				break;
 
 			case User_Data:
@@ -958,6 +975,7 @@ dissect_spdu(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	type = tvb_get_guint8(tvb, offset);
 	session.spdu_type = type;
 	session.abort_type = SESSION_NO_ABORT;
+	session.ros_op = 0;
 
 	if (tokens) {
 	  	if (check_col(pinfo->cinfo, COL_INFO))
@@ -1561,6 +1579,18 @@ proto_register_ses(void)
 				HFILL
 			}
 		},
+		{
+			&hf_activity_identifier,
+			{
+				"Activity Identifier",
+				"ses.activity_identifier",
+				FT_UINT32, BASE_DEC,
+				NULL,
+				0x0,
+				"Activity Identifier",
+				HFILL
+			}
+		},
 		{
 			&hf_serial_number,
 			{
@@ -1759,6 +1789,17 @@ dissect_ses_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		return FALSE;  /* no, it isn't a session PDU */
 	}
 
+	/* some Siemens SIMATIC protocols also use COTP, and shouldn't be 
+	 * misinterpreted as SES.
+	 * the starter in this case is fixed to 0x32 (SES_MINOR_SYNC_ACK for SES), 
+	 * so if the parameter type is unknown, it's probably SIMATIC */
+	if(type == 0x32 && tvb_bytes_exist(tvb, 0, 3)) {
+		type = tvb_get_guint8(tvb, offset+2);
+		if (match_strval(type, param_vals) == NULL) {
+			return FALSE; /* it's probably a SIMATIC protocol */
+		}
+	}
+
 	/*  OK,let's check SPDU length  */
 	/*  get length of SPDU */
 	len = get_item_len(tvb, offset+1, &len_len);
