@@ -1,8 +1,8 @@
 /* packet-sigcomp.c
  * Routines for Signaling Compression (SigComp) dissection.
- * Copyright 2004, Anders Broman <anders.broman@ericsson.com>
+ * Copyright 2004-2005, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-sigcomp.c 14667 2005-06-16 21:16:23Z etxrab $
+ * $Id: packet-sigcomp.c 17510 2006-03-07 20:51:05Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -24,6 +24,7 @@
  * References:
  * http://www.ietf.org/rfc/rfc3320.txt?number=3320
  * http://www.ietf.org/rfc/rfc3321.txt?number=3321
+ * http://www.ietf.org/rfc/rfc4077.txt?number=4077
  * Useful links :
  * http://www.ietf.org/internet-drafts/draft-ietf-rohc-sigcomp-impl-guide-03.txt
  * http://www.ietf.org/internet-drafts/draft-ietf-rohc-sigcomp-sip-01.txt
@@ -39,13 +40,10 @@
 #include <math.h>
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include "prefs.h"
 #include "strutil.h"
+#include <epan/emem.h>
 #include <epan/sigcomp-udvm.h>
 #include <epan/sigcomp_state_hdlr.h>
 
@@ -103,6 +101,14 @@ static int hf_udvm_uncompressed						= -1;
 static int hf_udvm_offset							= -1;
 static int hf_udvm_addr_offset						= -1;
 static int hf_udvm_start_value						= -1;
+static int hf_sigcomp_nack_ver						= -1;
+static int hf_sigcomp_nack_reason_code				= -1;
+static int hf_sigcomp_nack_failed_op_code			= -1;
+static int hf_sigcomp_nack_pc						= -1;
+static int hf_sigcomp_nack_sha1						= -1;
+static int hf_sigcomp_nack_state_id					= -1;
+static int hf_sigcomp_nack_memory_size				= -1;
+static int hf_sigcomp_nack_cycles_per_bit			= -1;				
 
 /* Initialize the subtree pointers */
 static gint ett_sigcomp				= -1;
@@ -111,18 +117,22 @@ static gint ett_sigcomp_udvm_exe	= -1;
 static gint ett_raw_text			= -1;
 
 static dissector_handle_t sip_handle;
-/* set the tcp port */
+/* set the udp ports */
 static guint SigCompUDPPort1 = 5555;
 static guint SigCompUDPPort2 = 6666;
 
-/* Default preference wether to display the bytecode in UDVM operands or not */
+/* set the tcp ports */
+static guint SigCompTCPPort1 = 5555;
+static guint SigCompTCPPort2 = 6666;
+
+/* Default preference whether to display the bytecode in UDVM operands or not */
 static gboolean display_udvm_bytecode = FALSE;
-/* Default preference wether to dissect the UDVM code or not */
+/* Default preference whether to dissect the UDVM code or not */
 static gboolean dissect_udvm_code = TRUE;
 static gboolean display_raw_txt = FALSE;
-/* Default preference wether to decompress the message or not */
+/* Default preference whether to decompress the message or not */
 static gboolean decompress = TRUE;
-/* Default preference wether to print debug info at execution of UDVM 
+/* Default preference whether to print debug info at execution of UDVM 
  * 0 = No printout
  * 1 = details level 1
  * 2 = details level 2
@@ -258,6 +268,42 @@ static const value_string display_lit_bytecode_vals[] = {
 	{ 0,	NULL }
 };
 
+#define SIGCOMP_NACK_STATE_NOT_FOUND              1
+#define SIGCOMP_NACK_CYCLES_EXHAUSTED             2
+#define SIGCOMP_NACK_BYTECODES_TOO_LARGE         18  
+#define SIGCOMP_NACK_ID_NOT_UNIQUE               21  
+#define SIGCOMP_NACK_STATE_TOO_SHORT             23  
+
+static const value_string sigcomp_nack_reason_code_vals[] = {
+	{ 0x01,	"STATE_NOT_FOUND" },			/*1  State ID (6 - 20 bytes) */
+	{ 0x02,	"CYCLES_EXHAUSTED" },			/*2  Cycles Per Bit (1 byte) */
+	{ 0x03,	"USER_REQUESTED" },
+	{ 0x04,	"SEGFAULT" },
+	{ 0x05,	"TOO_MANY_STATE_REQUESTS" },
+	{ 0x06,	"INVALID_STATE_ID_LENGTH" },
+	{ 0x07,	"INVALID_STATE_PRIORITY" },
+	{ 0x08,	"OUTPUT_OVERFLOW" },
+	{ 0x09,	"STACK_UNDERFLOW" },
+	{ 0x10,	"BAD_INPUT_BITORDER" },
+	{ 0x11,	"DIV_BY_ZERO" },
+	{ 0x12,	"SWITCH_VALUE_TOO_HIGH" },
+	{ 0x13,	"TOO_MANY_BITS_REQUESTED" },
+	{ 0x14,	"INVALID_OPERAND" },
+	{ 0x15,	"HUFFMAN_NO_MATCH" },
+	{ 0x16,	"MESSAGE_TOO_SHORT" },
+	{ 0x17,	"INVALID_CODE_LOCATION" },
+	{ 0x18,	"BYTECODES_TOO_LARGE" },		/*18  Memory size (2 bytes) */
+	{ 0x19,	"INVALID_OPCODE" },
+	{ 0x20,	"INVALID_STATE_PROBE" },
+	{ 0x21,	"ID_NOT_UNIQUE" },				/*21  State ID (6 - 20 bytes) */
+	{ 0x22,	"MULTILOAD_OVERWRITTEN" },
+	{ 0x23,	"STATE_TOO_SHORT" },			/*23  State ID (6 - 20 bytes) */
+	{ 0x24,	"INTERNAL_ERROR" },
+	{ 0x25,	"FRAMING_ERROR" },
+	{ 0,	NULL }
+};
+
+
 static void dissect_udvm_bytecode(tvbuff_t *udvm_tvb, proto_tree *sigcomp_udvm_tree, guint destination);
 
 static int dissect_udvm_multitype_operand(tvbuff_t *udvm_tvb, proto_tree *sigcomp_udvm_tree, 
@@ -270,25 +316,224 @@ static int dissect_udvm_literal_operand(tvbuff_t *udvm_tvb, proto_tree *sigcomp_
 static int dissect_udvm_reference_operand(tvbuff_t *udvm_tvb, proto_tree *sigcomp_udvm_tree, 
 							   gint offset, gint *start_offset, guint16 *value);
 static void tvb_raw_text_add(tvbuff_t *tvb, proto_tree *tree);
+
+static int dissect_sigcomp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+
+static proto_tree *top_tree;
+
 /* Initialize the state handler
  *
  */
-void
+static void
 sigcomp_init_protocol(void)
 {
 	sigcomp_init_udvm();
 } 
+/* Sigcomp over TCP record marking used 
+ * RFC 3320
+ * 4.2.2.  Record Marking
+ *
+ * For a stream-based transport, the dispatcher delimits messages by
+ * parsing the compressed data stream for instances of 0xFF and taking
+ * the following actions:
+ * Occurs in data stream:     Action:
+ *
+ *   0xFF 00                    one 0xFF byte in the data stream
+ *   0xFF 01                    same, but the next byte is quoted (could
+ *                              be another 0xFF)
+ *      :                                           :
+ *   0xFF 7F                    same, but the next 127 bytes are quoted
+ *   0xFF 80 to 0xFF FE         (reserved for future standardization)
+ *   0xFF FF                    end of SigComp message			 
+ *   :
+ *   In UDVM version 0x01, any occurrence of the combinations 0xFF80 to
+ *   0xFFFE that are not protected by quoting causes decompression
+ *   failure; the decompressor SHOULD close the stream-based transport in
+ *   this case.
+ */
 
+/*
+ * TODO: Reassembly, handle more than one message in a tcp segment.
+ */
+
+static int
+dissect_sigcomp_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	proto_item	*ti;
+	proto_tree	*sigcomp_tree;
+	tvbuff_t	*unescaped_tvb;
+
+	guint8		*buff;
+	int			offset = 0;
+	int			length;
+	guint8		octet;
+	guint16		data;
+	int			i;
+	int			n;
+	gboolean	end_off_message;
+
+	top_tree = tree;
+
+	/* Is this SIGCOMP ? */
+	data = tvb_get_ntohs(tvb, offset);
+	if(data == 0xffff){
+		/* delimiter */
+		offset = offset + 2;
+		octet = tvb_get_guint8(tvb,offset);
+	}else{
+		octet = tvb_get_guint8(tvb,offset);
+	}
+	if ((octet  & 0xf8) != 0xf8)
+	 return offset;
+
+	/* Make entries in Protocol column and Info column on summary display */
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SIGCOMP");
+
+	if (check_col(pinfo->cinfo, COL_INFO)) 
+		col_clear(pinfo->cinfo, COL_INFO);
+
+	length = tvb_length_remaining(tvb,offset);
+
+try_again:
+	/* create display subtree for the protocol */
+	ti = proto_tree_add_item(tree, proto_sigcomp, tvb, 0, -1, FALSE);
+	sigcomp_tree = proto_item_add_subtree(ti, ett_sigcomp);
+	i=0;
+	end_off_message = FALSE;
+	buff = g_malloc(length-offset);
+	if (udvm_print_detail_level>2)
+		proto_tree_add_text(sigcomp_tree, tvb, offset, -1,"Starting to remove escape digits");
+	while ((offset < length) && (end_off_message == FALSE)){
+		octet = tvb_get_guint8(tvb,offset);
+		if ( octet == 0xff ){
+			if ( offset +1 >= length ){
+				/* if the tvb is short dont check for the second escape digit */
+				offset++;
+				continue;
+			}
+			if (udvm_print_detail_level>2)
+				proto_tree_add_text(sigcomp_tree, tvb, offset, 2,
+					"              Escape digit found (0xFF)");
+			octet = tvb_get_guint8(tvb, offset+1);
+			if ( octet == 0){
+				buff[i] = 0xff;
+				offset = offset +2;
+				i++;
+				continue;
+			}
+			if ((octet > 0x7f) && (octet < 0xff )){
+				if (udvm_print_detail_level>2)
+					proto_tree_add_text(sigcomp_tree, tvb, offset, 2,
+						"              Illegal escape code");
+				offset = offset + tvb_length_remaining(tvb,offset);
+				return offset;
+			}
+			if ( octet == 0xff){
+				if (udvm_print_detail_level>2)
+					proto_tree_add_text(sigcomp_tree, tvb, offset, 2,
+						"              End of SigComp message indication found (0xFFFF)");
+				end_off_message = TRUE;
+				offset = offset+2;
+				continue;
+			}
+			buff[i] = 0xff;
+			if (udvm_print_detail_level>2)
+				proto_tree_add_text(sigcomp_tree, tvb, offset, 1,
+							"              Addr: %u tvb value(0x%0x) ", i, buff[i]);
+			i++;
+			offset = offset+2;
+			if (udvm_print_detail_level>2)
+			proto_tree_add_text(sigcomp_tree, tvb, offset, octet,
+						"              Copying %u bytes literally",octet);
+			if( offset+octet >= length)
+				/* if the tvb is short dont copy further than the end */
+				octet = length - offset;
+			for ( n=0; n < octet; n++ ){
+				buff[i] = tvb_get_guint8(tvb, offset);
+				if (udvm_print_detail_level>2)
+					proto_tree_add_text(sigcomp_tree, tvb, offset, 1,
+								"                  Addr: %u tvb value(0x%0x) ", i, buff[i]);
+				i++;
+				offset++;
+			}
+			continue;
+		}
+		buff[i] = octet;
+		if (udvm_print_detail_level>2)
+			proto_tree_add_text(sigcomp_tree, tvb, offset, 1,
+						"              Addr: %u tvb value(0x%0x) ", i, buff[i]);
+
+		i++;
+		offset++;
+	}
+	unescaped_tvb = tvb_new_real_data(buff,i,i);
+	/* Arrange that the allocated packet data copy be freed when the
+	 * tvbuff is freed. 
+	 */
+	tvb_set_free_cb( unescaped_tvb, g_free );
+	/* Add the tvbuff to the list of tvbuffs to which the tvbuff we
+	 * were handed refers, so it'll get cleaned up when that tvbuff
+	 * is cleaned up. 
+	 */
+	tvb_set_child_real_data_tvbuff( tvb, unescaped_tvb );
+	add_new_data_source(pinfo, unescaped_tvb, "Unescaped Data handed to the SigComp dissector");
+
+	proto_tree_add_text(sigcomp_tree, unescaped_tvb, 0, -1,"Data handed to the Sigcomp dissector");
+	if (end_off_message == TRUE){
+		dissect_sigcomp_common(unescaped_tvb, pinfo, sigcomp_tree);
+	}else{
+		proto_tree_add_text(sigcomp_tree, unescaped_tvb, 0, -1,"TCP Fragment, no end mark found");
+	}
+	if ( offset < length){
+		goto try_again;
+	}
+
+	return offset;
+}
 /* Code to actually dissect the packets */
 static int
 dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	proto_item	*ti;
+	proto_tree	*sigcomp_tree;
+	gint		offset = 0;
+	gint8		octet;
+
+	/* If we got called from SIP this might be over TCP */
+	if ( pinfo->ptype == PT_TCP )
+		return dissect_sigcomp_tcp(tvb, pinfo, tree);
+
+	/* Is this a SigComp message or not ? */
+	octet = tvb_get_guint8(tvb, offset);
+	if ((octet  & 0xf8) != 0xf8)
+	 return 0;
+
+	/* Make entries in Protocol column and Info column on summary display */
+	if (check_col(pinfo->cinfo, COL_PROTOCOL))
+		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SIGCOMP");
+
+	if (check_col(pinfo->cinfo, COL_INFO)) 
+		col_clear(pinfo->cinfo, COL_INFO);
+
+	top_tree = tree;
+
+	/* create display subtree for the protocol */
+	ti = proto_tree_add_item(tree, proto_sigcomp, tvb, 0, -1, FALSE);
+	sigcomp_tree = proto_item_add_subtree(ti, ett_sigcomp);
+
+	return dissect_sigcomp_common(tvb, pinfo, sigcomp_tree);
+}
+/* Code to actually dissect the packets */
+static int
+dissect_sigcomp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *sigcomp_tree)
 {
 
 /* Set up structures needed to add the protocol subtree and manage it */
-	tvbuff_t	 *udvm_tvb, *msg_tvb, *udvm2_tvb;
+	tvbuff_t	*udvm_tvb, *msg_tvb, *udvm2_tvb;
 	tvbuff_t	*decomp_tvb = NULL;
-	proto_item *ti, *udvm_bytecode_item, *udvm_exe_item;
-	proto_tree *sigcomp_tree, *sigcomp_udvm_tree, *sigcomp_udvm_exe_tree;
+	proto_item	*udvm_bytecode_item, *udvm_exe_item;
+	proto_tree	*sigcomp_udvm_tree, *sigcomp_udvm_exe_tree;
 	gint		offset = 0;
 	gint		bytecode_offset;
 	guint16		partial_state_len;
@@ -310,21 +555,7 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint16		result_code;
 	gchar		*partial_state_str;
 
-/* Is this a SigComp message or not ? */
-	octet = tvb_get_guint8(tvb, offset);
-	if ((octet  & 0xf8) != 0xf8)
-	 return 0;
 
-/* Make entries in Protocol column and Info column on summary display */
-	if (check_col(pinfo->cinfo, COL_PROTOCOL))
-		col_set_str(pinfo->cinfo, COL_PROTOCOL, "SIGCOMP");
-
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-		col_clear(pinfo->cinfo, COL_INFO);
-
-/* create display subtree for the protocol */
-	ti = proto_tree_add_item(tree, proto_sigcomp, tvb, 0, -1, FALSE);
-	sigcomp_tree = proto_item_add_subtree(ti, ett_sigcomp);
 
 /* add an item to the subtree, see section 1.6 for more information */
 	octet = tvb_get_guint8(tvb, offset);
@@ -356,6 +587,38 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
  *                                     |                               |
  *                                     +---+---+---+---+---+---+---+---+
  *
+ * RFC 4077:
+ * The format of the NACK message and the use of the fields within it
+ * are shown in Figure 1.
+ *
+ *                     0   1   2   3   4   5   6   7
+ *                  +---+---+---+---+---+---+---+---+
+ *                  | 1   1   1   1   1 | T |   0   |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |                               |
+ *                  :    returned feedback item     :
+ *                  |                               |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |         code_len = 0          |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  | code_len = 0  |  version = 1  |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |          Reason Code          |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |  OPCODE of failed instruction |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |   PC of failed instruction    |
+ *                  |                               |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |                               |
+ *                  : SHA-1 Hash of failed message  :
+ *                  |                               |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  |                               |
+ *                  :         Error Details         :
+ *                  |                               |
+ *                  +---+---+---+---+---+---+---+---+
+ *                  Figure 1: SigComp NACK Message Format
  */
 
 	proto_tree_add_item(sigcomp_tree,hf_sigcomp_t_bit, tvb, offset, 1, FALSE);
@@ -417,12 +680,12 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		proto_tree_add_string(sigcomp_tree,hf_sigcomp_partial_state,
 			tvb, offset, partial_state_len, partial_state_str);
 		offset = offset + partial_state_len;
+		msg_len = tvb_reported_length_remaining(tvb, offset);
+
 		if(msg_len>0)
-			proto_tree_add_text(sigcomp_tree, tvb, offset, -1, "Remaining SigComp message %u bytes",
-				tvb_reported_length_remaining(tvb, offset));
+			proto_tree_add_text(sigcomp_tree, tvb, offset, -1, "Remaining SigComp message %u bytes", msg_len);
 
 		if ( decompress ) {
-			msg_len = tvb_reported_length_remaining(tvb, offset);
 			msg_tvb = tvb_new_subset(tvb, offset, msg_len, msg_len);
 			/*
 			 * buff					= Where "state" will be stored
@@ -465,7 +728,7 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				return tvb_length(tvb);
 			}
 
-			udvm_tvb = tvb_new_real_data(buff,state_length+128,state_length+128);
+			udvm_tvb = tvb_new_real_data(buff,state_length+state_address,state_length+state_address);
 			/* Arrange that the allocated packet data copy be freed when the
 			 * tvbuff is freed. 
 			 */
@@ -477,7 +740,7 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			tvb_set_child_real_data_tvbuff( tvb, udvm_tvb );
 
 
-			udvm2_tvb = tvb_new_subset(udvm_tvb, 128, state_length, state_length);
+			udvm2_tvb = tvb_new_subset(udvm_tvb, state_address, state_length, state_length);
 			/* TODO Check if buff needs to be free'd */
 			udvm_exe_item = proto_tree_add_text(sigcomp_tree, udvm2_tvb, 0, state_length, 
 				"UDVM execution trace");
@@ -486,18 +749,18 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			decomp_tvb = decompress_sigcomp_message(udvm2_tvb, msg_tvb, pinfo,
 						   sigcomp_udvm_exe_tree, state_address, 
 						   udvm_print_detail_level, hf_sigcomp_partial_state,
-						   offset, state_length, partial_state_len);
+						   offset, state_length, partial_state_len, state_instruction);
 		
 
 			if ( decomp_tvb ){
 				proto_tree_add_text(sigcomp_tree, decomp_tvb, 0, -1,"SigComp message Decompressed WOHO!!");
 				if ( display_raw_txt )
-					tvb_raw_text_add(decomp_tvb, tree);
+					tvb_raw_text_add(decomp_tvb, top_tree);
 				if (check_col(pinfo->cinfo, COL_PROTOCOL)){
 					col_append_str(pinfo->cinfo, COL_PROTOCOL, "/");
 					col_set_fence(pinfo->cinfo,COL_PROTOCOL);
 				}
-				call_dissector(sip_handle, decomp_tvb, pinfo, tree);
+				call_dissector(sip_handle, decomp_tvb, pinfo, top_tree);
 			}
 		}/* if decompress */
 
@@ -526,51 +789,85 @@ dissect_sigcomp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			offset = offset + len;
 		}
 		len = tvb_get_ntohs(tvb, offset) >> 4;
-		octet =  tvb_get_guint8(tvb, (offset + 1));
-		destination = (octet & 0x0f);
-		if ( destination != 0 )
-			destination = 64 + ( destination * 64 );
-		proto_tree_add_uint(sigcomp_tree,hf_sigcomp_code_len, tvb, offset, 2, len);
-		proto_tree_add_item(sigcomp_tree,hf_sigcomp_destination, tvb, (offset+ 1), 1, FALSE);
-		offset = offset +2;
+		if (len == 0){
+			offset++;
+			/* NACK MESSAGE */
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_ver, tvb, offset, 1, FALSE);
+			offset++;
+			octet = tvb_get_guint8(tvb, offset);
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_reason_code, tvb, offset, 1, FALSE);
+			offset++;
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_failed_op_code, tvb, offset, 1, FALSE);
+			offset++;
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_pc, tvb, offset, 2, FALSE);
+			offset = offset +2;
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_sha1, tvb, offset, 20, FALSE);
+			offset = offset +20;
+			switch ( octet){
+			case SIGCOMP_NACK_STATE_NOT_FOUND:
+			case SIGCOMP_NACK_ID_NOT_UNIQUE:
+			case SIGCOMP_NACK_STATE_TOO_SHORT:
+				/* State ID (6 - 20 bytes) */
+				proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_state_id, tvb, offset, -1, FALSE);
+				break;
+			case SIGCOMP_NACK_CYCLES_EXHAUSTED:
+				/* Cycles Per Bit (1 byte) */
+				proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_cycles_per_bit, tvb, offset, 1, FALSE);
+				break;
+			case SIGCOMP_NACK_BYTECODES_TOO_LARGE:
+				/* Memory size (2 bytes) */
+				proto_tree_add_item(sigcomp_tree,hf_sigcomp_nack_memory_size, tvb, offset, 2, FALSE);
+				break;
+			default:
+				break;
+			}
+		}else{
+			octet = tvb_get_guint8(tvb, (offset + 1));
+			destination = (octet & 0x0f);
+			if ( destination != 0 )
+				destination = 64 + ( destination * 64 );
+			proto_tree_add_uint(sigcomp_tree,hf_sigcomp_code_len, tvb, offset, 2, len);
+			proto_tree_add_item(sigcomp_tree,hf_sigcomp_destination, tvb, (offset+ 1), 1, FALSE);
+			offset = offset +2;
 
-		bytecode_len = len;
-		bytecode_offset = offset;
-		udvm_bytecode_item = proto_tree_add_text(sigcomp_tree, tvb, bytecode_offset, bytecode_len, 
-			"Uploaded UDVM bytecode %u (0x%x) bytes", bytecode_len, bytecode_len);
-		sigcomp_udvm_tree = proto_item_add_subtree( udvm_bytecode_item, ett_sigcomp_udvm);
+			bytecode_len = len;
+			bytecode_offset = offset;
+			udvm_bytecode_item = proto_tree_add_text(sigcomp_tree, tvb, bytecode_offset, bytecode_len, 
+				"Uploaded UDVM bytecode %u (0x%x) bytes", bytecode_len, bytecode_len);
+			sigcomp_udvm_tree = proto_item_add_subtree( udvm_bytecode_item, ett_sigcomp_udvm);
 
-		udvm_tvb = tvb_new_subset(tvb, offset, len, len);
-		if ( dissect_udvm_code )
-			dissect_udvm_bytecode(udvm_tvb, sigcomp_udvm_tree, destination); 
+			udvm_tvb = tvb_new_subset(tvb, offset, len, len);
+			if ( dissect_udvm_code )
+				dissect_udvm_bytecode(udvm_tvb, sigcomp_udvm_tree, destination); 
 
-		offset = offset + len;
-		msg_len = tvb_reported_length_remaining(tvb, offset);
-		if(msg_len>0)
-			proto_tree_add_text(sigcomp_tree, tvb, offset, -1, "Remaining SigComp message %u bytes",
-				tvb_reported_length_remaining(tvb, offset));
-		if ( decompress ){
+			offset = offset + len;
+			msg_len = tvb_reported_length_remaining(tvb, offset);
+			if(msg_len>0)
+				proto_tree_add_text(sigcomp_tree, tvb, offset, -1, "Remaining SigComp message %u bytes",
+					tvb_reported_length_remaining(tvb, offset));
+			if ( decompress ){
 
-			msg_tvb = tvb_new_subset(tvb, offset, msg_len, msg_len);
+				msg_tvb = tvb_new_subset(tvb, offset, msg_len, msg_len);
 	
-			udvm_exe_item = proto_tree_add_text(sigcomp_tree, tvb, bytecode_offset, bytecode_len, 
-				"UDVM execution trace");
-			sigcomp_udvm_exe_tree = proto_item_add_subtree( udvm_exe_item, ett_sigcomp_udvm_exe);
-			decomp_tvb = decompress_sigcomp_message(udvm_tvb, msg_tvb, pinfo,
+				udvm_exe_item = proto_tree_add_text(sigcomp_tree, tvb, bytecode_offset, bytecode_len, 
+					"UDVM execution trace");
+				sigcomp_udvm_exe_tree = proto_item_add_subtree( udvm_exe_item, ett_sigcomp_udvm_exe);
+				decomp_tvb = decompress_sigcomp_message(udvm_tvb, msg_tvb, pinfo,
 						   sigcomp_udvm_exe_tree, destination, 
 						   udvm_print_detail_level, hf_sigcomp_partial_state,
-						   offset, 0, 0);
-			if ( decomp_tvb ){
-				proto_tree_add_text(sigcomp_tree, decomp_tvb, 0, -1,"SigComp message Decompressed WOHO!!");
-				if ( display_raw_txt )
-					tvb_raw_text_add(decomp_tvb, tree);
-				if (check_col(pinfo->cinfo, COL_PROTOCOL)){
-					col_append_str(pinfo->cinfo, COL_PROTOCOL, "/");
-					col_set_fence(pinfo->cinfo,COL_PROTOCOL);
+						   offset, 0, 0, destination);
+				if ( decomp_tvb ){
+					proto_tree_add_text(sigcomp_tree, decomp_tvb, 0, -1,"SigComp message Decompressed WOHO!!");
+					if ( display_raw_txt )
+						tvb_raw_text_add(decomp_tvb, top_tree);
+					if (check_col(pinfo->cinfo, COL_PROTOCOL)){
+						col_append_str(pinfo->cinfo, COL_PROTOCOL, "/");
+						col_set_fence(pinfo->cinfo,COL_PROTOCOL);
+					}
+					call_dissector(sip_handle, decomp_tvb, pinfo, top_tree);
 				}
-				call_dissector(sip_handle, decomp_tvb, pinfo, tree);
-			}
-		} /* if decompress */
+			} /* if decompress */
+		}/*if len==0 */
 
 	}
 	return tvb_length(tvb);
@@ -1902,25 +2199,34 @@ void
 proto_reg_handoff_sigcomp(void)
 {
 	static dissector_handle_t sigcomp_handle;
+	static dissector_handle_t sigcomp_tcp_handle;
 	static int Initialized=FALSE;
 	static int udp_port1 = 5555;
 	static int udp_port2 = 6666;
+	static int tcp_port1 = 5555;
+	static int tcp_port2 = 6666;
 
 	if (!Initialized) {
-		sigcomp_handle = new_create_dissector_handle(dissect_sigcomp,
-			proto_sigcomp);
+		sigcomp_handle = new_create_dissector_handle(dissect_sigcomp,proto_sigcomp);
+		sigcomp_tcp_handle = new_create_dissector_handle(dissect_sigcomp_tcp,proto_sigcomp);
 		Initialized=TRUE;
 	}else{
 		dissector_delete("udp.port", udp_port1, sigcomp_handle);
 		dissector_delete("udp.port", udp_port2, sigcomp_handle);
+		dissector_delete("tcp.port", tcp_port1, sigcomp_tcp_handle);
+		dissector_delete("tcp.port", tcp_port2, sigcomp_tcp_handle);
 	}
 
 	udp_port1 = SigCompUDPPort1;
 	udp_port2 = SigCompUDPPort2;
+	tcp_port1 = SigCompTCPPort1;
+	tcp_port2 = SigCompTCPPort2;
 
 
 	dissector_add("udp.port", SigCompUDPPort1, sigcomp_handle);
 	dissector_add("udp.port", SigCompUDPPort2, sigcomp_handle);
+	dissector_add("tcp.port", SigCompTCPPort1, sigcomp_tcp_handle);
+	dissector_add("tcp.port", SigCompTCPPort2, sigcomp_tcp_handle);
 
 	sip_handle = find_dissector("sip");
 
@@ -2193,6 +2499,46 @@ proto_register_sigcomp(void)
 			FT_UINT16, BASE_DEC, NULL, 0x0,          
 			"Offset", HFILL }
 		},
+		{ &hf_sigcomp_nack_ver,
+			{ "NACK Version", "sigcomp.nack.ver",
+			FT_UINT8, BASE_DEC, NULL, 0x0f,          
+			"NACK Version", HFILL }
+		},
+		{ &hf_sigcomp_nack_reason_code,
+			{ "Reason Code", "sigcomp.nack.reason",
+			FT_UINT8, BASE_DEC, VALS(sigcomp_nack_reason_code_vals), 0x0,          
+			"NACK Reason Code", HFILL }
+		},
+		{ &hf_sigcomp_nack_failed_op_code,
+			{ "OPCODE of failed instruction", "sigcomp.nack.failed_op_code",
+			FT_UINT8, BASE_DEC, VALS(udvm_instruction_code_vals), 0x0,          
+			"NACK OPCODE of failed instruction", HFILL }
+		},
+		{ &hf_sigcomp_nack_pc,
+			{ "PC of failed instruction", "sigcomp.nack.pc",
+			FT_UINT16, BASE_DEC, NULL, 0x0,          
+			"NACK PC of failed instruction", HFILL }
+		},
+		{ &hf_sigcomp_nack_sha1,
+			{ "SHA-1 Hash of failed message", "sigcomp.nack.sha1",
+			FT_BYTES, BASE_HEX, NULL, 0x0,          
+			"NACK SHA-1 Hash of failed message", HFILL }
+		},
+		{ &hf_sigcomp_nack_state_id,
+			{ "State ID (6 - 20 bytes)", "sigcomp.nack.tate_id",
+			FT_BYTES, BASE_HEX, NULL, 0x0,          
+			"NACK State ID (6 - 20 bytes)", HFILL }
+		},
+		{ &hf_sigcomp_nack_cycles_per_bit,
+			{ "Cycles Per Bit ", "sigcomp.nack.cycles_per_bit",
+			FT_UINT8, BASE_DEC, NULL, 0x0,          
+			"NACK Cycles Per Bit", HFILL }
+		},
+		{ &hf_sigcomp_nack_memory_size,
+			{ "Memory size", "sigcomp.memory_size",
+			FT_UINT16, BASE_DEC, NULL, 0x0,          
+			"Memory size", HFILL }
+		},
 	};
 
 /* Setup protocol subtree array */
@@ -2243,26 +2589,37 @@ proto_register_sigcomp(void)
 								   "Set UDP port 2 for SigComp messages",
 								   10,
 								   &SigCompUDPPort2);
+	prefs_register_uint_preference(sigcomp_module, "tcp.port",
+								   "Sigcomp TCP Port 1",
+								   "Set TCP port 1 for SigComp messages",
+								   10,
+								   &SigCompTCPPort1);
+
+	prefs_register_uint_preference(sigcomp_module, "tcp.port2",
+								   "Sigcomp TCP Port 2",
+								   "Set TCP port 2 for SigComp messages",
+								   10,
+								   &SigCompTCPPort2);
 	prefs_register_bool_preference(sigcomp_module, "display.udvm.code",
 								   "Dissect the UDVM code",
-								   "Preference wether to Dissect the UDVM code or not",
+								   "Preference whether to Dissect the UDVM code or not",
 								   &dissect_udvm_code);
 
 	prefs_register_bool_preference(sigcomp_module, "display.bytecode",
 								   "Display the bytecode of operands",
-								   "preference wether to display the bytecode in UDVM operands or not",
+								   "preference whether to display the bytecode in UDVM operands or not",
 								   &display_udvm_bytecode);
 	prefs_register_bool_preference(sigcomp_module, "decomp.msg",
 								   "Decompress message",
-								   "preference wether to decompress message or not",
+								   "preference whether to decompress message or not",
 								   &decompress);
 	prefs_register_bool_preference(sigcomp_module, "display.decomp.msg.as.txt",
 								   "Displays the decompressed message as text",
-								   "preference wether to display the decompressed message as raw text or not",
+								   "preference whether to display the decompressed message as raw text or not",
 								   &display_raw_txt);
     prefs_register_enum_preference(sigcomp_module, "show.udvm.execution",
       "Level of detail of UDVM execution",
-      "0 = UDVM executes silently, then incrising detail about execution of UDVM instructions, Warning! CPU intense at high detail",
+      "0 = UDVM executes silently, then increasing detail about execution of UDVM instructions, Warning! CPU intense at high detail",
       &udvm_print_detail_level, udvm_detail_vals, FALSE);
 
 	register_init_routine(&sigcomp_init_protocol);
