@@ -2,7 +2,7 @@
  * Routines for SMB mailslot packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-smb-mailslot.c 14129 2005-04-18 18:57:37Z gerald $
+ * $Id: packet-smb-mailslot.c 17620 2006-03-14 05:30:26Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,7 +34,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include "smb.h"
+#include <epan/dissectors/packet-smb.h>
 #include "packet-smb-mailslot.h"
 #include "packet-smb-browse.h"
 #include "packet-smb-pipe.h"
@@ -116,7 +116,7 @@ dissect_mailslot_smb(tvbuff_t *mshdr_tvb, tvbuff_t *setup_tvb,
 	}
 
 	smb_info = pinfo->private_data;
-	if (smb_info != NULL && smb_info->sip != NULL && smb_info->sip->extra_info_type == SMB_EI_TRI)
+	if (smb_info->sip != NULL && smb_info->sip->extra_info_type == SMB_EI_TRI)
 		tri = smb_info->sip->extra_info;
 	else
 		tri = NULL;
@@ -140,7 +140,11 @@ dissect_mailslot_smb(tvbuff_t *mshdr_tvb, tvbuff_t *setup_tvb,
 				tri->trans_subcmd = trans_subcmd;
 		}
 	} else {
-		trans_subcmd = tri->trans_subcmd;
+		if(!tri){
+			return FALSE;
+		} else {
+			trans_subcmd = tri->trans_subcmd;
+		}
 	}
 
 	/* Only do these ones if we have them. For fragmented SMB Transactions
