@@ -2,7 +2,7 @@
  * Routines for Subnetwork Dependent Convergence Protocol (SNDCP) dissection
  * Copyright 2000, Christian Falckenberg <christian.falckenberg@nortelnetworks.com>
  *
- * $Id: packet-sndcp.c 15036 2005-07-24 10:32:37Z lego $
+ * $Id: packet-sndcp.c 17518 2006-03-08 09:43:06Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,10 +33,6 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include <epan/reassemble.h>
 
@@ -199,7 +195,7 @@ dissect_sndcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint8         addr_field, comp_field, npdu_field1, nsapi, dcomp=0, pcomp=0;
   guint16        offset=0, npdu=0, segment=0, npdu_field2;
   tvbuff_t	*next_tvb, *npdu_tvb;
-  guint32        len;
+  gint           len;
   gboolean       first, more_frags, unack;
 
   /* Set up structures needed to add the protocol subtree and manage it 
@@ -286,8 +282,8 @@ dissect_sndcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	npdu_field_item = proto_tree_add_text(sndcp_tree, tvb, offset,1, "Acknowledged mode, N-PDU %d", npdu_field1 );
 	npdu_field_tree = proto_item_add_subtree(npdu_field_item, ett_sndcp_npdu_field);
 	proto_tree_add_uint(npdu_field_tree, hf_sndcp_npdu1, tvb, offset, 1, npdu_field1 );
-	offset++;
       }
+      offset++;
     }
   }
 
@@ -328,6 +324,10 @@ dissect_sndcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     gboolean        save_fragmented = pinfo->fragmented;
     
     len = tvb_length_remaining(tvb, offset);
+    if(len<=0){
+        return;
+    }
+
     pinfo->fragmented = TRUE;
 
     if (unack) 
