@@ -2,7 +2,7 @@
  * Routines for socks versions 4 &5  packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-socks.c 14608 2005-06-10 18:28:22Z ulfl $
+ * $Id: packet-socks.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -77,6 +77,7 @@
 #include <epan/addr_resolv.h>
 #include <epan/conversation.h>
 
+#include <epan/emem.h>
 #include "packet-tcp.h"
 #include "packet-udp.h"
 #include <epan/strutil.h>
@@ -181,7 +182,7 @@ typedef struct {
 
 
 
-static char *address_type_table[] = {
+static const char *address_type_table[] = {
 	"Unknown",
 	"IPv4",
 	"Unknown",
@@ -226,17 +227,12 @@ static const value_string cmd_strings[] = {
 	{0, NULL}
 };
 
-#define socks_hash_init_count 20
-#define socks_hash_val_length (sizeof(socks_hash_entry_t))
-
-static GMemChunk *socks_vals = NULL;
-
 
 /************************* Support routines ***************************/
 
 
 static int display_string(tvbuff_t *tvb, int offset,
-	proto_tree *tree, char *label){
+	proto_tree *tree, const char *label){
 
 /* display a string with a length, characters encoding */
 /* they are displayed under a tree with the name in Label variable */
@@ -269,7 +265,7 @@ static int display_string(tvbuff_t *tvb, int offset,
 
 
 
-static char *get_auth_method_name( guint Number){
+static const char *get_auth_method_name( guint Number){
 
 /* return the name of the authenication method */
 
@@ -287,7 +283,7 @@ static char *get_auth_method_name( guint Number){
 }
 
 
-static char *get_command_name( guint Number){
+static const char *get_command_name( guint Number){
 
 /* return the name of the command as a string */
 
@@ -346,8 +342,7 @@ static int get_address_v5(tvbuff_t *tvb, int offset,
 	if ( a_type == 1){ 		/* IPv4 address */
 
 	   	if ( hash_info)
-	   		tvb_memcpy(tvb, (guint8 *)&hash_info->dst_addr,
-	   		    offset, 4);
+	   		hash_info->dst_addr = tvb_get_ipv4(tvb, offset);
 		offset += 4;
 	}
 
@@ -556,7 +551,7 @@ display_socks_v5(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 	unsigned int i, command;
 	guint temp;
-	char *AuthMethodStr;
+	const char *AuthMethodStr;
 	guint8 auth_status;
 
 	proto_tree_add_item( tree, hf_socks_ver, tvb, offset, 1, FALSE);
@@ -676,7 +671,7 @@ state_machine_v4( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 			hash_info->port =  tvb_get_ntohs(tvb, offset + 2);
 						/* get remote address	*/
 
-		tvb_memcpy(tvb, (guint8 *)&hash_info->dst_addr, offset + 4, 4);
+		hash_info->dst_addr = tvb_get_ipv4(tvb, offset + 4);
 
 						/* save the packet pointer */
 		hash_info->connect_row = get_packet_ptr;
@@ -946,6 +941,9 @@ static void call_next_dissector(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	guint32 *ptr;
 	struct tcpinfo *tcpinfo = pinfo->private_data;
 	guint16 save_can_desegment;
+	struct tcp_analysis *tcpd=NULL;
+
+	tcpd=get_tcp_conversation_data(pinfo);
 
  	if (( hash_info->command  == PING_COMMAND) ||
  	    ( hash_info->command  == TRACERT_COMMAND))
@@ -972,7 +970,7 @@ static void call_next_dissector(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		pinfo->can_desegment = pinfo->saved_can_desegment;
 		dissect_tcp_payload(tvb, pinfo, offset, tcpinfo->seq,
 		    tcpinfo->nxtseq, pinfo->srcport, pinfo->destport,
-		    tree, socks_tree);
+		    tree, socks_tree, tcpd);
 		pinfo->can_desegment = save_can_desegment;
 
 		CLEANUP_CALL_AND_POP;
@@ -1009,7 +1007,7 @@ dissect_socks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 	}
 	hash_info = conversation_get_proto_data(conversation,proto_socks);
 	if ( !hash_info){
-    		hash_info = g_mem_chunk_alloc(socks_vals);
+    		hash_info = se_alloc(sizeof(socks_hash_entry_t));
 		hash_info->start_done_row = G_MAXINT;
     		hash_info->state = None;
 		hash_info->port = 0;
@@ -1115,21 +1113,6 @@ dissect_socks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
 
 
-static void socks_reinit( void){
-
-/* Do the cleanup work when a new pass through the packet list is	*/
-/* performed. Reset the highest row seen counter and re-initialize the	*/
-/* conversation memory chunks.						*/
-
-  	if (socks_vals)
-    		g_mem_chunk_destroy(socks_vals);
-
-  	socks_vals = g_mem_chunk_new("socks_vals", socks_hash_val_length,
-		socks_hash_init_count * socks_hash_val_length,
-		G_ALLOC_AND_FREE);
-}
-
-
 void
 proto_register_socks( void){
 
@@ -1208,8 +1191,6 @@ proto_register_socks( void){
 	proto_register_field_array(proto_socks, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
-	register_init_routine( &socks_reinit);	/* register re-init routine */
-
 	socks_udp_handle = create_dissector_handle(socks_udp_dissector,
 	    proto_socks);
 	socks_handle = create_dissector_handle(dissect_socks, proto_socks);
