@@ -9,7 +9,7 @@
  *       In particular I have not had an opportunity to see how it
  *       responds to SRVLOC over TCP.
  *
- * $Id: packet-srvloc.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-srvloc.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,9 +48,13 @@
 #include <epan/packet.h>
 #include <epan/strutil.h>
 #include <epan/prefs.h>
+#include <epan/emem.h>
 #include "packet-tcp.h"
 #include <epan/reassemble.h>
+#include <epan/expert.h>
 
+static proto_item *expert_item = NULL;
+static guint16 expert_status = 0;
 static gboolean srvloc_desegment = TRUE;
 static int proto_srvloc = -1;
 static int hf_srvloc_version = -1;
@@ -272,7 +276,7 @@ static const value_string srvloc_errs[] = {
 
 /* Define Error Codes for Version 2 */
 
-#define LANGUAGE_NOT_SUPPORTED 	1 
+#define LANGUAGE_NOT_SUPPORTED 	1
 #define PARSE_ERROR 		2
 #define INVALID_REGISTRATION	3
 #define SCOPE_NOT_SUPPORTED	4
@@ -381,10 +385,13 @@ static int
 dissect_authblk_v2(tvbuff_t *tvb, int offset, proto_tree *tree)
 {
     guint16 length;
-    
+    nstime_t ts;
+
     proto_tree_add_item(tree, hf_srvloc_authblkv2_bsd, tvb, offset, 2, FALSE);
     proto_tree_add_item(tree, hf_srvloc_authblkv2_len, tvb, offset+2, 2, FALSE);
-    proto_tree_add_item(tree, hf_srvloc_authblkv2_timestamp, tvb, offset+4, 4, FALSE);
+    ts.nsecs = 0;
+    ts.secs = tvb_get_ntohl(tvb, offset + 4);
+    proto_tree_add_time(tree, hf_srvloc_authblkv2_timestamp, tvb, offset+4, 4, &ts);
     length = tvb_get_ntohs(tvb, offset + 8);
     proto_tree_add_uint(tree, hf_srvloc_authblkv2_slpspilen, tvb, offset + 8, 2, length);
     offset += 10;
@@ -411,10 +418,9 @@ add_v1_string(proto_tree *tree, int hf, tvbuff_t *tvb, int offset, int length,
         switch (encoding) {
 
         case CHARSET_ISO_10646_UCS_2:
-                unicode_str = tvb_fake_unicode(tvb, offset, length/2, FALSE);
+                unicode_str = tvb_get_ephemeral_faked_unicode(tvb, offset, length/2, FALSE);
                 proto_tree_add_string(tree, hf, tvb, offset, length,
                                     unicode_str);
-                g_free(unicode_str);
                 break;
 
         default:
@@ -423,15 +429,37 @@ add_v1_string(proto_tree *tree, int hf, tvbuff_t *tvb, int offset, int length,
         }
 }
 
+/*
+ * XXX - is this trying to guess the byte order?
+ *
+ *	http://www.iana.org/assignments/character-sets
+ *
+ * says of ISO-10646-UCS-2, which has the code 1000 (this routine is used
+ * with CHARSET_ISO_10646_UCS_2, which is #defined to be 1000):
+ *
+ *	this needs to specify network byte order: the standard
+ *	does not specify (it is a 16-bit integer space)
+ *
+ * Does that mean that in SRVLOC, ISO-10646-UCS-2 is always big-endian?
+ * If so, can we just use "tvb_get_ephemeral_faked_unicode()" and be
+ * done with it?
+ *
+ * XXX - this is also used with CHARSET_UTF_8.  Is that a cut-and-pasteo?
+ */
 static guint8*
 unicode_to_bytes(tvbuff_t *tvb, int offset, int length, gboolean endianness)
 {
   const char	*ascii_text = tvb_get_ptr(tvb, offset, length);
-  guint8	i, j=0;
+  int	i, j=0;
   guint8	c_char, c_char1;
-  static guint8 byte_array[255];
+  guint8	*byte_array;
+
+  /* XXX - Is this the correct behavior? */
+  if (length < 1)
+    return "";
 
   if (endianness) {
+      byte_array = ep_alloc(length*2 + 1);
       for (i = length; i > 0; i--) {
         c_char = ascii_text[i];
         if (c_char != 0) {
@@ -450,6 +478,7 @@ unicode_to_bytes(tvbuff_t *tvb, int offset, int length, gboolean endianness)
   }
   else
   {
+      byte_array = ep_alloc(length + 1);
       for (i = 0; i < length; i++) {
         c_char = ascii_text[i];
         if (c_char != 0) {
@@ -465,11 +494,11 @@ unicode_to_bytes(tvbuff_t *tvb, int offset, int length, gboolean endianness)
 
 /*
  * Format of x-x-x-xxxxxxxx. Each of these entries represents the service binding to UDP, TCP, or IPX.
- * The first digit is the protocol family: 2 for TCP/UPD, 6 for IPX. 
- * The second digit is the socket type: 1 for socket stream (TCP), 2 for datagram (UDP and IPX). 
- * The third is the protocol: 6 for TCP, 17 for UDP, and 1000 for IPX. 
+ * The first digit is the protocol family: 2 for TCP/UPD, 6 for IPX.
+ * The second digit is the socket type: 1 for socket stream (TCP), 2 for datagram (UDP and IPX).
+ * The third is the protocol: 6 for TCP, 17 for UDP, and 1000 for IPX.
  * Last is the IP address, in hex, of the interface that is registered (or, in the case of IPX, an IPX network number).
-*/              
+*/
 static void
 attr_list(proto_tree *tree, int hf, tvbuff_t *tvb, int offset, int length,
     guint16 encoding)
@@ -505,12 +534,10 @@ attr_list(proto_tree *tree, int hf, tvbuff_t *tvb, int offset, int length,
 
     case CHARSET_ISO_10646_UCS_2:
 
-        tmp = tvb_fake_unicode(tvb, offset, length/2, FALSE);
+        tmp = tvb_get_ephemeral_faked_unicode(tvb, offset, length/2, FALSE);
         type_len = strcspn(tmp, "=");
-        g_free(tmp);
-        attr_type = tvb_fake_unicode(tvb, offset+2, type_len-1, FALSE);
+        attr_type = tvb_get_ephemeral_faked_unicode(tvb, offset+2, type_len-1, FALSE);
         proto_tree_add_string(tree, hf, tvb, offset, type_len*2, attr_type);
-        g_free(attr_type);
         i=1;
         for (foffset = offset + ((type_len*2)+2); foffset<length; foffset += 2) {
 
@@ -765,6 +792,10 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				val_to_str(encoding, charsets, "Unknown"));
 	    proto_tree_add_text(srvloc_tree, tvb, offset + 10, 2, "Transaction ID: %u",
 				tvb_get_ntohs(tvb, offset + 10));
+        /* added echo of XID to info colomn by Greg Morris 0ct 14, 2005 */
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_append_fstr(pinfo->cinfo, COL_INFO, ", V1 Transaction ID - %u", tvb_get_ntohs(tvb, offset + 10));
+
 	    offset += 12;
 
 	    switch (function) {
@@ -780,9 +811,13 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 add_v1_string(srvloc_tree, hf_srvloc_srvreq_predicate, tvb, offset, length, encoding);
                 offset += length;
 		break;
-		
+
             case SRVRPLY:
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 count = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_srvrply_urlcount, tvb, offset, 2, count);
@@ -830,7 +865,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             case SRVACK:
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
             break;
 
@@ -858,7 +897,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             case ATTRRPLY:
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 length = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_attrrply_attrlistlen, tvb, offset, 2, length);
@@ -872,7 +915,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             case DAADVERT:
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 length = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_daadvert_urllen, tvb, offset, 2, length);
@@ -897,8 +944,8 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 if (0xFFFF == length) {
                     proto_tree_add_uint(srvloc_tree, hf_srvloc_srvtypereq_nameauthlistlenall, tvb, offset, 2, length);
                     offset += 2;
-                } 
-                else 
+                }
+                else
                 {
                     proto_tree_add_uint(srvloc_tree, hf_srvloc_srvtypereq_nameauthlistlen, tvb, offset, 2, length);
                     offset += 2;
@@ -913,7 +960,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             case SRVTYPERPLY:
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 count = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_text(srvloc_tree, tvb, offset, 2, "Service Type Count: %u",
@@ -930,7 +981,8 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             default:
-                proto_tree_add_text(srvloc_tree, tvb, offset, -1, "Unknown Function Type");
+                expert_item = proto_tree_add_text(srvloc_tree, tvb, offset, -1, "Unknown Function Type");
+                expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Unknown Function Type: %d", function);
 	    };
 	}
 	else { /* Version 2 */
@@ -953,6 +1005,8 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				next_ext_off);
 	    proto_tree_add_uint(srvloc_tree, hf_srvloc_xid, tvb, offset + 10, 2,
 				tvb_get_ntohs(tvb, offset + 10));
+        if (check_col(pinfo->cinfo, COL_INFO))
+                col_append_fstr(pinfo->cinfo, COL_INFO, ", V2 XID - %u", tvb_get_ntohs(tvb, offset + 10));
 	    lang_tag_len = tvb_get_ntohs(tvb, offset + 12);
 	    proto_tree_add_uint(srvloc_tree, hf_srvloc_langtaglen, tvb, offset + 12, 2,	lang_tag_len);
 	    proto_tree_add_item(srvloc_tree, hf_srvloc_langtag, tvb, offset + 14, lang_tag_len,	TRUE);
@@ -986,9 +1040,13 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 proto_tree_add_item(srvloc_tree, hf_srvloc_srvreq_slpspi, tvb, offset, length, TRUE);
 		offset += length;
 		break;
-		
+
             case SRVRPLY: /* RFC2608 8.2 */
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 count = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_srvrply_urlcount, tvb, offset, 2, count);
@@ -1038,9 +1096,13 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 proto_tree_add_item(srvloc_tree, hf_srvloc_srvdereg_taglist, tvb, offset, length, TRUE);
                 offset += length;
 		break;
-            
+
             case SRVACK: /* RFC2608 8.4 */
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
             break;
 
@@ -1073,7 +1135,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
             case ATTRRPLY: /* RFC2608 10.4 */
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 length = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_attrrply_attrlistlen, tvb, offset, 2, length);
@@ -1081,7 +1147,7 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                     offset += 2;
                     attr_list(srvloc_tree, hf_srvloc_attrrply_attrlist, tvb, offset, length, CHARSET_UTF_8);
                     offset += length;
-            		count = tvb_get_guint8(tvb, offset); 
+            		count = tvb_get_guint8(tvb, offset);
             		proto_tree_add_uint(srvloc_tree, hf_srvloc_attrrply_attrauthcount, tvb, offset, 1, count);
             		offset += 1;
             		while (count > 0) {
@@ -1090,9 +1156,13 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             		}
                 }
             break;
-            
+
             case DAADVERT: /* RCC 2608 8.5 */
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
 		ts.nsecs = 0;
 		ts.secs = tvb_get_ntohl(tvb, offset);
@@ -1119,7 +1189,7 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 offset += 2;
                 proto_tree_add_item(srvloc_tree, hf_srvloc_daadvert_slpspi, tvb, offset, length, TRUE);
                 offset += length;
-		count = tvb_get_guint8(tvb, offset); 
+		count = tvb_get_guint8(tvb, offset);
 		proto_tree_add_uint(srvloc_tree, hf_srvloc_daadvert_authcount, tvb, offset, 1, count);
 		offset += 1;
 		while (count > 0) {
@@ -1152,7 +1222,11 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             break;
 
 	    case SRVTYPERPLY: /* rfc2608 10.2 */
-                proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_item = proto_tree_add_item(srvloc_tree, hf_srvloc_error_v2, tvb, offset, 2, FALSE);
+                expert_status = tvb_get_ntohs(tvb, offset);
+                if (expert_status!=0) {
+                    expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Error: %s", val_to_str(expert_status, srvloc_errs_v2, "Unknown SRVLOC Error (0x%02x)"));
+                }
                 offset += 2;
                 length = tvb_get_ntohs(tvb, offset);
                 proto_tree_add_uint(srvloc_tree, hf_srvloc_srvtyperply_srvtypelistlen, tvb, offset, 2, length);
@@ -1187,7 +1261,8 @@ dissect_srvloc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		break;
 
             default:
-                proto_tree_add_text(srvloc_tree, tvb, offset, -1, "Unknown Function Type");
+                expert_item = proto_tree_add_text(srvloc_tree, tvb, offset, -1, "Unknown Function Type");
+                expert_add_info_format(pinfo, expert_item, PI_RESPONSE_CODE, PI_ERROR, "Unknown Function Type: %d", function);
 	    };
 	};
     }
@@ -1218,7 +1293,7 @@ dissect_srvloc_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
     if (check_col(pinfo->cinfo, COL_INFO))
         col_clear(pinfo->cinfo, COL_INFO);
-	
+
     if (tree) {
         ti = proto_tree_add_item(tree, proto_srvloc, tvb, 0, -1, FALSE);
         srvloc_tree = proto_item_add_subtree(ti, ett_srvloc);
@@ -1261,23 +1336,23 @@ proto_register_srvloc(void)
             "", HFILL }
         },
         {&hf_srvloc_xid,
-            {"XID", "srvloc.xid", 
-            FT_UINT24, BASE_DEC, NULL, 0x0, 
+            {"XID", "srvloc.xid",
+            FT_UINT24, BASE_DEC, NULL, 0x0,
             "Transaction ID", HFILL }
         },
         {&hf_srvloc_langtag,
-            {"Lang Tag", "srvloc.langtag", 
-            FT_STRING, BASE_NONE, NULL, 0x0, 
+            {"Lang Tag", "srvloc.langtag",
+            FT_STRING, BASE_NONE, NULL, 0x0,
             "", HFILL }
         },
         {&hf_srvloc_langtaglen,
-            {"Lang Tag Len", "srvloc.langtaglen", 
-            FT_UINT16, BASE_DEC, NULL, 0x0, 
+            {"Lang Tag Len", "srvloc.langtaglen",
+            FT_UINT16, BASE_DEC, NULL, 0x0,
             "", HFILL }
         },
         {&hf_srvloc_nextextoff,
-            {"Next Extension Offset", "srvloc.nextextoff", 
-            FT_UINT24, BASE_DEC, NULL, 0x0, 
+            {"Next Extension Offset", "srvloc.nextextoff",
+            FT_UINT24, BASE_DEC, NULL, 0x0,
             "", HFILL }
         },
 
@@ -1311,8 +1386,8 @@ proto_register_srvloc(void)
         },
 
         {&hf_srvloc_pktlen,
-            {"Packet Length", "srvloc.pktlen", 
-            FT_UINT24, BASE_DEC, NULL, 0x0, 
+            {"Packet Length", "srvloc.pktlen",
+            FT_UINT24, BASE_DEC, NULL, 0x0,
             "", HFILL }
         },
 
@@ -1349,8 +1424,8 @@ proto_register_srvloc(void)
 	    TFS(&tfs_srvloc_flags_fresh), FLAG_F, "Is this a new registration?", HFILL }},
 
         {&hf_srvloc_flags_v2,
-            {"Flags", "srvloc.flags_v2", 
-            FT_UINT16, BASE_HEX, NULL, 0x0, 
+            {"Flags", "srvloc.flags_v2",
+            FT_UINT16, BASE_HEX, NULL, 0x0,
              "", HFILL }
          },
 
