@@ -3,7 +3,7 @@
  *
  * Huagang XIE <huagang@intruvert.com>
  *
- * $Id: packet-ssh.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: packet-ssh.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,6 +37,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/conversation.h>
+#include <epan/emem.h>
 
 #include "packet-tcp.h"
 #include <epan/reassemble.h>
@@ -93,8 +94,6 @@ struct ssh_flow_data {
 	guint	rsp_counter;
 	guint 	version;
 };
-static GMemChunk *ssh_this_data=NULL;
-static GMemChunk *ssh_global_data = NULL;
 
 static int proto_ssh = -1;
 static int hf_ssh_packet_length= -1;
@@ -160,6 +159,7 @@ static const value_string ssh1_msg_vals[] = {
 	{SSH_SMSG_PUBLIC_KEY,"Public Key"},
 	{SSH_CMSG_SESSION_KEY,"Session Key"},
 	{SSH_CMSG_USER,"User"},
+	{0, NULL}
 };
 
 
@@ -186,22 +186,6 @@ static int ssh_dissect_encrypted_packet(tvbuff_t *tvb, packet_info *pinfo,
 proto_item * ssh_proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
     gint start, gint length, gboolean little_endian);
 
-static void
-ssh_init_protocol(void)
-{
-  	if (ssh_global_data)
-    		g_mem_chunk_destroy(ssh_global_data);
-  	if (ssh_this_data)
-    		g_mem_chunk_destroy(ssh_this_data);
-
-  	ssh_global_data = g_mem_chunk_new("ssh_global_datas",
-				      sizeof(struct ssh_flow_data),
-				      100* sizeof(struct ssh_flow_data), G_ALLOC_AND_FREE);
-  	ssh_this_data = g_mem_chunk_new("ssh_pku_data",
-				      sizeof(struct ssh_pdu_data),
-				      100* sizeof(struct ssh_pdu_data), G_ALLOC_AND_FREE);
-
-}
 
 static void
 dissect_ssh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -238,7 +222,7 @@ dissect_ssh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	global_data = conversation_get_proto_data(conversation,proto_ssh);
 	if(!global_data ) {
-		global_data = g_mem_chunk_alloc(ssh_global_data);
+		global_data = se_alloc(sizeof(struct ssh_flow_data));
 		global_data->req_counter=0;
 		global_data->rsp_counter=0;
 		global_data->version=SSH_VERSION_UNKNOWN;
@@ -251,7 +235,7 @@ dissect_ssh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (pinfo->destport == pinfo->match_port) {
 	  	is_response=FALSE;
 		if(!this_data) {
-			this_data = g_mem_chunk_alloc(ssh_this_data);
+			this_data = se_alloc(sizeof(struct ssh_pdu_data));
 			this_data->counter = global_data->req_counter++;
 			p_add_proto_data(pinfo->fd, proto_ssh, this_data);
 			is_newdata = TRUE;
@@ -259,7 +243,7 @@ dissect_ssh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}else {
 		is_response=TRUE;
 		if(!this_data) {
-			this_data = g_mem_chunk_alloc(ssh_global_data);
+			this_data = se_alloc(sizeof(struct ssh_flow_data));
 			this_data->counter = global_data->rsp_counter++;
 			p_add_proto_data(pinfo->fd, proto_ssh, this_data);
 			is_newdata = TRUE;
@@ -919,7 +903,6 @@ proto_register_ssh(void)
 				       "SSH", "ssh");
  	 proto_register_field_array(proto_ssh, hf, array_length(hf));
   	proto_register_subtree_array(ett, array_length(ett));
-	register_init_routine(&ssh_init_protocol);
 
   	ssh_module = prefs_register_protocol(proto_ssh, NULL);
 	prefs_register_bool_preference(ssh_module, "desegment_buffers",
