@@ -5,7 +5,7 @@
  * Full Tacacs+ parsing with decryption by
  *   Emanuele Caratti <wiz@iol.it>
  *
- * $Id: packet-tacacs.c 12116 2004-09-27 23:29:22Z guy $
+ * $Id: packet-tacacs.c 17056 2006-01-19 23:01:24Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,16 +38,36 @@
 #endif
 
 #include <stdio.h>
-
 #include <string.h>
+
+#include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>		/* needed to define AF_ values on Windows */
+#endif
+
+#ifdef NEED_INET_V6DEFS_H
+# include "inet_v6defs.h"
+#endif
+
 #include <glib.h>
 #include <epan/packet.h>
 
 #include <epan/prefs.h>
 #include <epan/crypt-md5.h>
+#include <epan/emem.h>
 #include "packet-tacacs.h"
 
-static void md5_xor( guint8 *data, char *key, int data_len, guint8 *session_id, guint8 version, guint8 seq_no );
+static void md5_xor( guint8 *data, const char *key, int data_len, guint8 *session_id, guint8 version, guint8 seq_no );
 
 static int proto_tacacs = -1;
 static int hf_tacacs_version = -1;
@@ -66,7 +86,7 @@ static int hf_tacacs_result3 = -1;
 
 static gint ett_tacacs = -1;
 
-static char *tacplus_opt_key;
+static const char *tacplus_opt_key;
 static GSList *tacplus_keys = NULL;
 
 #define VERSION_TACACS	0x00
@@ -321,7 +341,7 @@ typedef struct _tacplus_key_entry {
 } tacplus_key_entry;
 
 static gint 
-tacplus_decrypted_tvb_setup( tvbuff_t *tvb, tvbuff_t **dst_tvb, packet_info *pinfo, guint32 len, guint8 version, char *key )
+tacplus_decrypted_tvb_setup( tvbuff_t *tvb, tvbuff_t **dst_tvb, packet_info *pinfo, guint32 len, guint8 version, const char *key )
 {
 	guint8	*buff;
 	guint8 session_id[4];
@@ -552,10 +572,9 @@ dissect_tacplus_body_authen_req_cont( tvbuff_t *tvb, proto_tree *tree )
 	val=tvb_get_ntohs( tvb, AUTHEN_C_USER_LEN_OFF ); 
 	proto_tree_add_text( tree, tvb, AUTHEN_C_USER_LEN_OFF, 2 , "User length: %d", val );
 	if( val ){
-		buff=tvb_get_string( tvb, var_off, val );
+		buff=tvb_get_ephemeral_string( tvb, var_off, val );
 		proto_tree_add_text( tree, tvb, var_off, val, "User: %s", buff );
 		var_off+=val;
-		g_free(buff);
 	}
 
 	val=tvb_get_ntohs( tvb, AUTHEN_C_DATA_LEN_OFF ); 
@@ -590,10 +609,9 @@ dissect_tacplus_body_authen_rep( tvbuff_t *tvb, proto_tree *tree )
 	proto_tree_add_text( tree, tvb, AUTHEN_R_SRV_MSG_LEN_OFF, 2 ,
 				"Server message length: %d", val );
 	if( val ) {
-		buff=tvb_get_string(tvb, var_off, val );
+		buff=tvb_get_ephemeral_string(tvb, var_off, val );
 		proto_tree_add_text(tree, tvb, var_off, val, "Server message: %s", buff );
 		var_off+=val;
-		g_free(buff);
 	}
 
 	val=tvb_get_ntohs(tvb, AUTHEN_R_DATA_LEN_OFF );
@@ -715,11 +733,10 @@ dissect_tacplus_body_acct_rep( tvbuff_t* tvb, proto_tree *tree )
 	proto_tree_add_text( tree, tvb, ACCT_R_SRV_MSG_LEN_OFF, 2 ,
 				"Server message length: %d", val );
 	if( val ) {
-		buff=tvb_get_string( tvb, var_off, val );
+		buff=tvb_get_ephemeral_string( tvb, var_off, val );
 		proto_tree_add_text( tree, tvb, var_off,
 				val, "Server message: %s", buff );
 		var_off+=val;
-		g_free(buff);
 	}
 
 	/*  Data */
@@ -727,10 +744,9 @@ dissect_tacplus_body_acct_rep( tvbuff_t* tvb, proto_tree *tree )
 	proto_tree_add_text( tree, tvb, ACCT_R_DATA_LEN_OFF, 2 ,
 				"Data length: %d", val );
 	if( val ) {
-		buff= tvb_get_string( tvb, var_off, val );
+		buff= tvb_get_ephemeral_string( tvb, var_off, val );
 		proto_tree_add_text( tree, tvb, var_off,
 				val, "Data: %s", buff );
-		g_free(buff);
 	}
 }
 
@@ -789,8 +805,8 @@ tacplus_print_key_entry( gpointer data, gpointer user_data )
 static int
 cmp_conv_address( gconstpointer p1, gconstpointer p2 )
 {
-	tacplus_key_entry *a1=(tacplus_key_entry*)p1;
-	tacplus_key_entry *a2=(tacplus_key_entry*)p2;
+	const tacplus_key_entry *a1=p1;
+	const tacplus_key_entry *a2=p2;
 	gint32	ret;
 	/*
 	printf("p1=>");
@@ -810,7 +826,7 @@ cmp_conv_address( gconstpointer p1, gconstpointer p2 )
 	return ret;
 }
 
-static char*
+static const char*
 find_key( address *srv, address *cln )
 {
 	tacplus_key_entry data;
@@ -827,17 +843,18 @@ find_key( address *srv, address *cln )
 
 	return (tacplus_keys?NULL:tacplus_opt_key);
 }
-#define AF_INET 2
-int inet_pton(int , const char*, void*);
 
 static void
-mkipv4_address( address **addr, char *str_addr )
+mkipv4_address( address **addr, const char *str_addr )
 {
+	char *addr_data;
+
 	*addr=g_malloc( sizeof(address) );
+	addr_data=g_malloc( 4 );
+	inet_pton( AF_INET, str_addr, addr_data );
 	(*addr)->type=AT_IPv4;
 	(*addr)->len=4;
-	(*addr)->data=g_malloc( 4 );
-	inet_pton( AF_INET, (const char*)str_addr, (void*)(*addr)->data );
+	(*addr)->data=addr_data;
 }
 static void
 parse_tuple( char *key_from_option )
@@ -873,9 +890,9 @@ free_tacplus_keys( gpointer data, gpointer user_data _U_ )
 
 static 
 void
-parse_tacplus_keys( char *keys_from_option )
+parse_tacplus_keys( const char *keys_from_option )
 {
-	char *s1,*s;
+	char *key_copy,*s,*s1;
 
 	/* Drop old keys */
 	if( tacplus_keys ) {
@@ -888,16 +905,15 @@ parse_tacplus_keys( char *keys_from_option )
 		/* option not in client/server=key format */
 		return ;
 	}
-	s=strdup(keys_from_option);
-	s1=s;
-	keys_from_option = s;
-	while(keys_from_option){
-		if( (s=strchr( keys_from_option, ' ' )) != NULL )
-			*s++='\0';
-		parse_tuple( keys_from_option );
-		keys_from_option=s;
+	key_copy=strdup(keys_from_option);
+	s=key_copy;
+	while(s){
+		if( (s1=strchr( s, ' ' )) != NULL )
+			*s1++='\0';
+		parse_tuple( s );
+		s=s1;
 	}
-	g_free( s1 );
+	g_free( key_copy );
 #ifdef DEB_TACPLUS
 	g_slist_foreach( tacplus_keys, tacplus_print_key_entry, GINT_TO_POINTER(1) );
 #endif
@@ -915,7 +931,7 @@ dissect_tacplus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	proto_item	*tmp_pi;
 	guint32		len;
 	gboolean	request=( pinfo->destport == TCP_PORT_TACACS );
-	char		*key=NULL;
+	const char	*key=NULL;
 
 	if( request ) {
 		key=find_key( &pinfo->dst, &pinfo->src );
@@ -1002,7 +1018,7 @@ dissect_tacplus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 }
 
-void
+static void
 tacplus_pref_cb(void)
 {
 	parse_tacplus_keys( tacplus_opt_key );
@@ -1092,7 +1108,7 @@ proto_reg_handoff_tacplus(void)
 #define MD5_LEN 16
 
 static void
-md5_xor( guint8 *data, char *key, int data_len, guint8 *session_id, guint8 version, guint8 seq_no )
+md5_xor( guint8 *data, const char *key, int data_len, guint8 *session_id, guint8 version, guint8 seq_no )
 {
 	int i,j,md5_len;
 	md5_byte_t *md5_buff;
@@ -1103,7 +1119,7 @@ md5_xor( guint8 *data, char *key, int data_len, guint8 *session_id, guint8 versi
 	md5_len = 4 /* sizeof(session_id) */ + strlen(key)
 			+ sizeof(version) + sizeof(seq_no);
 	
-	md5_buff = (md5_byte_t*)g_malloc(md5_len+MD5_LEN);
+	md5_buff = (md5_byte_t*)ep_alloc(md5_len+MD5_LEN);
 
 
 	mdp = md5_buff;
@@ -1133,5 +1149,4 @@ md5_xor( guint8 *data, char *key, int data_len, guint8 *session_id, guint8 versi
 		md5_append(&mdcontext, md5_buff, md5_len);
 		md5_finish(&mdcontext,hash);
 	}
-	g_free( md5_buff );
 }
