@@ -2,7 +2,7 @@
  * Routines for Telnet packet dissection; see RFC 854 and RFC 855
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-telnet.c 14692 2005-06-18 15:41:26Z jmayer $
+ * $Id: packet-telnet.c 17487 2006-03-07 04:33:46Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,6 +37,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/strutil.h>
+#include <epan/emem.h>
 #include "packet-kerberos.h"
 
 static int proto_telnet = -1;
@@ -123,7 +124,7 @@ typedef enum {
 
 /* Member of table of IP or TCP options. */
 typedef struct tn_opt {
-  char  *name;			/* name of option */
+  const char  *name;		/* name of option */
   gint  *subtree_index;		/* pointer to subtree index for option */
   tn_opt_len_type len_type;	/* type of option length field */
   int	optlen;			/* value length should be (minimum if VARIABLE) */
@@ -417,9 +418,8 @@ dissect_comport_subopt(packet_info *pinfo _U_, const char *optname, tvbuff_t *tv
     if (len == 0) {
         proto_tree_add_text(tree, tvb, offset, 1, "%s Requests Signature",source);
     } else {
-        guint8 *sig = tvb_get_string(tvb, offset + 1, len);
+        guint8 *sig = tvb_get_ephemeral_string(tvb, offset + 1, len);
         proto_tree_add_text(tree, tvb, offset, 1 + len, "%s Signature: %s",source, sig);
-	g_free(sig);
     }
     break;
 
@@ -708,9 +708,6 @@ unescape_and_tvbuffify_telnet_option(packet_info *pinfo, tvbuff_t *tvb, int offs
 		return NULL;
 
 	spos=tvb_get_ptr(tvb, offset, len);
-	/* XXX we never g_free() this one.  This is done automagically
-	   when the parent tvb is destroyed?
-        */
 	buf=g_malloc(len);
 	dpos=buf;
 	skip=0;
@@ -727,6 +724,7 @@ unescape_and_tvbuffify_telnet_option(packet_info *pinfo, tvbuff_t *tvb, int offs
 		l--;
 	}
 	krb5_tvb = tvb_new_real_data(buf, len-skip, len-skip); 
+	tvb_set_free_cb(krb5_tvb, g_free);
 	tvb_set_child_real_data_tvbuff(tvb, krb5_tvb);
 	add_new_data_source(pinfo, krb5_tvb, "Unpacked Telnet Uption");
 
@@ -801,7 +799,7 @@ static void
 dissect_authentication_subopt(packet_info *pinfo, const char *optname _U_, tvbuff_t *tvb, int offset, int len, proto_tree *tree)
 {
 	guint8 acmd;
-	char name[256];
+	char *name;
 
 /* XXX here we should really split it up in a conversation struct keeping
        track of what method we actually use and not just assume it is always
@@ -827,10 +825,11 @@ dissect_authentication_subopt(packet_info *pinfo, const char *optname _U_, tvbuf
 		break;
 	case TN_AC_NAME:
 		if(len<255){
+			name=ep_alloc(256);
 			tvb_memcpy(tvb, name, offset, len);
 			name[len]=0;
 		} else {
-			strcpy(name, "<...name too long...>");
+			name="<...name too long...>";
 		}
 		proto_tree_add_string(tree, hf_telnet_auth_name, tvb, offset, len, name);
 		break;
@@ -1281,7 +1280,7 @@ telnet_sub_option(packet_info *pinfo, proto_tree *telnet_tree, tvbuff_t *tvb, in
 
 static int
 telnet_will_wont_do_dont(proto_tree *telnet_tree, tvbuff_t *tvb,
-			int start_offset, char *type)
+			int start_offset, const char *type)
 {
   int offset = start_offset;
   guint8 opt_byte;
@@ -1289,7 +1288,7 @@ telnet_will_wont_do_dont(proto_tree *telnet_tree, tvbuff_t *tvb,
 
   offset += 2;	/* skip IAC and WILL,WONT,DO,DONT} */
   opt_byte = tvb_get_guint8(tvb, offset);
-  if (opt_byte > NOPTIONS)
+  if (opt_byte >= NOPTIONS)
     opt = "<unknown option>";
   else
     opt = options[opt_byte].name;
