@@ -7,7 +7,7 @@
  * Routine to dissect RFC 1006 TPKT packet containing OSI TP PDU
  * Copyright 2001, Martin Thomas <Martin_A_Thomas@yahoo.com>
  *
- * $Id: packet-tpkt.c 14909 2005-07-12 21:09:34Z ulfl $
+ * $Id: packet-tpkt.c 16016 2005-09-27 05:39:58Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -133,7 +133,7 @@ dissect_tpkt_encap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	 * which case we'd have to zero the buffer out explicitly
 	 * anyway.
 	 */
-	if (tpkt_desegment && check_col(pinfo->cinfo, COL_INFO))
+	if (desegment && check_col(pinfo->cinfo, COL_INFO))
 		col_add_str(pinfo->cinfo, COL_INFO, "");
 
 	while (tvb_reported_length_remaining(tvb, offset) != 0) {
@@ -232,7 +232,7 @@ dissect_tpkt_encap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		 * information without getting TPKT stuff in the middle;
 		 * why the second?
 		 */
-		if (!tpkt_desegment && !pinfo->fragmented
+		if (!desegment && !pinfo->fragmented
 		    && check_col(pinfo->cinfo, COL_INFO)) {
 			col_add_fstr(pinfo->cinfo, COL_INFO,
 			    "TPKT Data length = %u", data_len);
