@@ -2,7 +2,7 @@
  * Routines for Token-Ring Media Access Control
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-trmac.c 14786 2005-06-26 19:56:52Z guy $
+ * $Id: packet-trmac.c 15222 2005-08-05 13:10:58Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -103,9 +103,11 @@ sv_text(tvbuff_t *tvb, int svoff, proto_tree *tree)
 	int	sv_length = tvb_get_guint8(tvb, svoff+0);
 	guint16	beacon_type, ring;
 
-	char *beacon[] = {"Recovery mode set", "Signal loss error",
+	const char *beacon[] = {
+		"Recovery mode set", "Signal loss error",
 		"Streaming signal not Claim Token MAC frame",
-		"Streaming signal, Claim Token MAC frame"};
+		"Streaming signal, Claim Token MAC frame"
+	};
 
 	proto_tree	*sv_tree;
 	proto_item	*ti;
