@@ -2,7 +2,7 @@
  * Routines for Unlicensed Mobile Access(UMA) dissection
  * Copyright 2005, Anders Broman <anders.broman[at]ericsson.com>
  *
- * $Id: packet-uma.c 15034 2005-07-24 04:24:30Z sahlberg $
+ * $Id: packet-uma.c 17127 2006-01-30 05:55:46Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -24,7 +24,7 @@
  *
  * References:
  * http://www.umatechnology.org/ 
- * UMA Protocols (Stage 3) R1.0.3 (2005-02-26)
+ * UMA Protocols (Stage 3) R1.0.4 (5/16/2005)
  * 
  * http://www.3gpp.org/specs/numbering.htm 
  * 3GPP TS 24.008 V6.2.0 (2003-09)
@@ -62,16 +62,13 @@
 #include <glib.h>
 #include <epan/conversation.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 #include "prefs.h"
 #include "packet-bssap.h"
 #include "packet-gsm_a.h"
 #include "packet-rtp.h"
 #include "packet-rtcp.h"
+#include "packet-e212.h"
 
 static dissector_handle_t uma_tcp_handle = NULL;
 static dissector_handle_t uma_udp_handle = NULL;
@@ -108,9 +105,14 @@ static int hf_uma_urr_mnc				= -1;
 static int hf_uma_urr_lac				= -1;
 static int hf_uma_urr_gci				= -1;
 static int hf_uma_urr_tura				= -1;
+static int hf_uma_urr_gc				= -1;
+static int hf_uma_urr_uc				= -1;
 static int hf_uma_urr_tlra				= -1;
 static int hf_uma_urr_rrs				= -1;
 static int hf_uma_urr_location_estimate = -1;
+static int hf_uma_urr_sign_of_latitude	= -1;
+static int hf_uma_urr_degrees_of_latitude =-1;
+static int hf_uma_urr_degrees_of_longitude =-1;
 static int hf_uma_urr_IP_Address_type	= -1;
 static int hf_uma_urr_FQDN				= -1;
 static int hf_uma_urr_sgw_ipv4			= -1;
@@ -122,7 +124,6 @@ static int hf_uma_urr_DTM				= -1;
 static int hf_uma_urr_GPRS				= -1;
 static int hf_uma_urr_NMO				= -1;
 static int hf_uma_urr_ECMC				= -1;
-static int hf_uma_urr_EC				= -1;
 static int hf_uma_urr_T3212_timer		= -1;
 static int hf_uma_urr_RAC				= -1;
 static int hf_uma_urr_ap_location		= -1;
@@ -149,11 +150,9 @@ static int hf_uma_urr_algorithm_id		= -1;
 static int hf_uma_urr_GPRS_resumption	= -1;
 static int hf_uma_urr_ULQI				= -1;
 static int hf_uma_urr_TU3920_timer		= -1;
-static int hf_uma_urr_rate				= -1;
-static int hf_uma_urr_precedence		= -1;
-static int hf_uma_urr_a_bit				= -1;
-static int hf_uma_urr_t_bit				= -1;
-static int hf_uma_urr_c_r_bit			= -1;
+static int hf_uma_urr_peak_tpt_cls		= -1;
+static int hf_uma_urr_radio_pri			= -1;
+static int hf_uma_urr_rlc_mode			= -1;
 static int hf_uma_urr_URLCcause			= -1;
 static int hf_uma_urr_udr				= -1;
 static int hf_uma_urr_TU4001_timer		= -1;
@@ -188,6 +187,8 @@ static int hf_uma_urr_uma_service_zone_str = -1;
 static int hf_uma_urr_suti				= -1;
 static int hf_uma_urr_uma_mps			= -1;
 static int hf_uma_urr_num_of_plms		= -1;
+static int hf_uma_urr_cbs				= -1;
+static int hf_uma_urr_num_of_cbs_frms	= -1;
 static int hf_uma_urr_unc_ipv4			= -1;
 static int hf_uma_unc_FQDN				= -1;
 static int hf_uma_urr_GPRS_user_data_transport_ipv4 = -1;
@@ -216,6 +217,7 @@ static guint gbl_umaTcpPort1 = 14001;
  * Protocol Discriminator (PD)
  */
 static const value_string uma_pd_vals[] = {
+	{ 0,		"URR_C"},
 	{ 1,		"URR"},
 	{ 2,		"URLC"}, 
 	{ 0,	NULL }
@@ -234,6 +236,7 @@ static const value_string uma_urr_msg_type_vals[] = {
 	{ 20,		"URR DEREGISTER"},
 	{ 21,		"URR REGISTER UPDATE UPLINK"},
 	{ 22,		"URR REGISTER UPDATE DOWNLINK"},
+	{ 23,		"URR CELL BROADCAST INFO"},
 	{ 32,		"URR CIPHERING MODE COMMAND"},
 	{ 33,		"URR CIPHERING MODE COMPLETE"},
 	{ 48,		"URR ACTIVATE CHANNEL"},
@@ -290,9 +293,9 @@ static const value_string uma_urr_IE_type_vals[] = {
 	{ 1,		"Mobile Identity"},
 	{ 2,		"UMA Release Indicator"},
 	{ 3,		"Radio Identity"},
-	{ 4,		"Cell Identity"},
+	{ 4,		"GERAN Cell Identity"},
 	{ 5,		"Location Area Identification"},
-	{ 6,		"GSM Coverage Indicator"},
+	{ 6,		"GERAN/UTRAN Coverage Indicator"},
 	{ 7,		"UMA Classmark"},
 	{ 8,		"Geographical Location"},
 	{ 9,		"UNC SGW IP Address"},
@@ -303,7 +306,7 @@ static const value_string uma_urr_IE_type_vals[] = {
 	{ 14,		"UMA Control Channel Description"},
 	{ 15,		"Cell Identifier List"},
 	{ 16,		"TU3907 Timer"},
-	{ 17,		"GSM RR State"},
+	{ 17,		"GSM RR/UTRAN RRC State"},
 	{ 18,		"Routing Area Identification"},
 	{ 19,		"UMA Band"},
 	{ 20,		"URR State"},
@@ -357,6 +360,9 @@ static const value_string uma_urr_IE_type_vals[] = {
 	{ 68,		"Registration indicators"},
 	{ 69,		"UMA PLMN List"},
 	{ 70,		"Received Signal Level List"},
+	{ 71,		"Required UMA Services"},
+	{ 72,		"Broadcast Container"},
+	{ 73,		"3G Cell Identity"},
 	{ 96,		"MS Radio Identity"},
 	{ 97,		"UNC IP Address"},
 	{ 98,		"UNC Fully Qualified Domain/Host Name"},
@@ -403,14 +409,19 @@ static const value_string tura_vals[] = {
 	{ 0,		"No radio"},
 	{ 1,		"Bluetooth"},
 	{ 2,		"WLAN 802.11"},
-	{ 3,		"Unspecified"},
+	{ 15,		"Unspecified"},
 	{ 0,	NULL }
 };
-
-/*TLRA, Type of Licensed Radio (octet 3)*/
-static const value_string tlra_vals[] = {
-	{ 0,		"None"},
-	{ 1,		"GERAN"},
+/* GC, GERAN Capable (octet 3) */
+static const value_string gc_vals[] = {
+	{ 0,		"The MS is not GERAN capable."},
+	{ 1,		"The MS is GERAN capable."},
+	{ 0,	NULL }
+};
+/* UC, UTRAN Capable (octet 3) */
+static const value_string uc_vals[] = {
+	{ 0,		"The MS is not UTRAN  capable."},
+	{ 1,		"The MS is UTRAN  capable."},
 	{ 0,	NULL }
 };
 /*RRS, RTP Redundancy Support (octet 4)*/
@@ -432,6 +443,13 @@ static const value_string type_of_shape_vals[] = {
 };
 
 
+/* 3GPP TS 23.032 7.3.1 */
+static const value_string sign_of_latitude_vals[] = {
+	{ 0,		"North"},
+	{ 1,		"South"},
+	{ 0,	NULL }
+};
+
 /*IP address type number value (octet 3)*/
 static const value_string IP_address_type_vals[] = {
 	{ 0x21,		"IPv4 address"},
@@ -528,6 +546,10 @@ static const value_string Three_GECS_vals[] = {
 static const value_string GRS_GSM_RR_State_vals[] = {
 	{ 0,		"GSM RR is in IDLE state"},
 	{ 1,		"GSM RR is in DEDICATED state"},
+	{ 2,		"UTRAN RRC is in IDLE STATE"},
+	{ 3,		"UTRAN RRC is in CELL_DCH STATE"},
+	{ 4,		"UTRAN RRC is in CELL_FACH STATE"},
+	{ 7,		"Unknown"},
 	{ 0,	NULL }
 };
 /* UMA Band (4 bit field) */
@@ -626,7 +648,7 @@ static const value_string ULQI_vals[] = {
 	{ 0,	NULL }
 };
 
-static const value_string precedence_vals[] = {
+static const value_string radio_pri_vals[] = {
 	{ 0,		"Radio priority 1"},
 	{ 1,		"Radio priority 2"},
 	{ 2,		"Radio priority 3"},
@@ -634,21 +656,13 @@ static const value_string precedence_vals[] = {
 	{ 3,		"Radio Priority Unknown"},
 	{ 0,	NULL }
 };
-static const value_string a_bit_vals[] = {
-	{ 0,		"Radio interface uses RLC/MAC ARQ functionality"},
-	{ 1,		"Radio interface uses RLC/MAC-UNITDATA functionality"},
-	{ 0,	NULL }
-};
-static const value_string t_bit_vals[] = {
-	{ 0,		"The SDU contains signalling (e.g. related to GMM)"},
-	{ 1,		"The SDU contains data"},
-	{ 0,	NULL }
-};
-static const value_string c_r_bit_vals[] = {
-	{ 0,		"The SDU contains a LLC ACK or SACK command/response frame type "},
-	{ 1,		"The SDU does not contain a LLC ACK or SACK command/response frame type"},
+
+static const value_string rlc_mode_vals[] = {
+	{ 0,		"RLC acknowledged mode"},
+	{ 1,		"RLC unacknowledged mode"},
 	{ 0,	NULL }
 };
+
 /*URLC Cause (octet 3) */
 static const value_string URLC_cause_vals[] = {
 	{ 0,		"success"},
@@ -713,6 +727,7 @@ static const value_string ICMI_vals[] = {
 	{ 1,		"The initial codec mode is defined by the Start Mode field"},
 	{ 0,	NULL }
 };
+
 /* MPS, Manual PLMN Selection indicator (octet 3) */
 static const value_string mps_vals[] = {
 	{ 0,		"The MS is in Automatic PLMN selection mode."},
@@ -720,6 +735,14 @@ static const value_string mps_vals[] = {
 	{ 2,		"The MS is in Manual PLMN selection mode and tries to register; no PLMN list is needed."},
 	{ 0,	NULL }
 };
+
+/* CBS Cell Broadcast Service (octet 3) */
+static const value_string cbs_vals[] = {
+	{ 0,		"CBS is not required by the Mobile station"},
+	{ 1,		"CBS is required by the mobile station"},
+	{ 0,	NULL }
+};
+
 /* LBLI, Location Black List indicator (octet 3) */
 static const value_string LBLI_vals[] = {
 	{ 0,		"MCC"},
@@ -775,8 +798,9 @@ static const value_string RI_vals[] = {
 /* Window Size (octet 3 to octet n) */
 
 static const value_string window_size_vals[] = {
-	{ 0,		"Window size 1 (single redundancy)"},
-	{ 1,		"Window size 2 (double redundancy)"},
+	{ 0,		"Window size 1, No redundancy"},
+	{ 1,		"Window size 2 (single redundancy)"},
+	{ 2,		"Window size 3 (double redundancy)"},
 	{ 0,	NULL }
 };
 
@@ -793,39 +817,7 @@ static const value_string suti_vals[] = {
 	{ 0,	NULL }
 };
 	/* Code to actually dissect the packets */
-static int
-dissect_mcc_mnc(tvbuff_t *tvb, proto_tree *urr_ie_tree, int offset){
 
-	int			start_offset;	
-	guint8		octet;
-	guint16		mcc, mnc;
-	guint8		mcc1, mcc2, mcc3, mnc1, mnc2, mnc3;
-
-	start_offset = offset;
-	/* Mobile country code MCC */
-	octet = tvb_get_guint8(tvb,offset);
-	mcc1 = octet & 0x0f;
-	mcc2 = octet >> 4;
-	offset++;
-	octet = tvb_get_guint8(tvb,offset);
-	mcc3 = octet & 0x0f;
-	/* MNC, Mobile network code (octet 3 bits 5 to 8, octet 4)  */
-	mnc3 = octet >> 4;
-	offset++;
-	octet = tvb_get_guint8(tvb,offset);
-	mnc1 = octet & 0x0f;
-	mnc2 = octet >> 4;
-
-	mcc = 100 * mcc1 + 10 * mcc2 + mcc3;
-	mnc = 10 * mnc1 + mnc2;
-	if (mnc3 != 0xf) {
-		mnc += 10 * mnc + mnc3;
-	}
-	proto_tree_add_uint(urr_ie_tree, hf_uma_urr_mcc , tvb, start_offset, 2, mcc );
-	proto_tree_add_uint(urr_ie_tree, hf_uma_urr_mnc , tvb, start_offset + 1, 2, mnc );
-	offset++;
-	return offset;
-}
 static int
 dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 {
@@ -877,12 +869,13 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 	}
 
 	switch(ie_value){
+	/* 11.2.1 Mobile Identity */
 	case 1:			
 	/* Mobile Identity 
 	 * The rest of the IE is coded as in [TS 24.008] not including IEI and
 	 * length, if present.(10.5.1.4)
 	 */
-		de_mid(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_mid(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 
 	case 2:			
@@ -911,7 +904,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		 * The rest of the IE is coded as in [TS 24.008] not including IEI and
 		 * length, if present.
 		 */
-		de_lai(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_lai(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 6:			
 		/* GSM Coverage Indicator */
@@ -920,7 +913,8 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 	case 7:			
 		/* UMA Classmark */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_tura, tvb, ie_offset, 1, FALSE);
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_tlra, tvb, ie_offset, 1, FALSE);
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_gc, tvb, ie_offset, 1, FALSE);
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_uc, tvb, ie_offset, 1, FALSE);
 		/* UMA Protocols (Stage 3) R1.0.3 */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_rrs, tvb, ie_offset, 1, FALSE);
 		break;
@@ -929,8 +923,21 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		 * The Location Estimate field is composed of 1 or more octets with an internal structure 
 		 * according to section 7 in [23.032].
 		 */
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_location_estimate, tvb, ie_offset, ie_len, FALSE);
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_location_estimate, tvb, ie_offset, 1, FALSE);
 		/* TODO:Add further dissecton here ? */
+		octet = tvb_get_guint8(tvb,ie_offset);
+		switch (octet){
+		case 0: /* Ellipsoid Point */
+			ie_offset++;
+			proto_tree_add_item(urr_ie_tree, hf_uma_urr_sign_of_latitude, tvb, ie_offset, 1, FALSE);
+			proto_tree_add_item(urr_ie_tree, hf_uma_urr_degrees_of_latitude, tvb, ie_offset, 3, FALSE);
+			ie_offset = ie_offset + 3;
+			proto_tree_add_item(urr_ie_tree, hf_uma_urr_degrees_of_longitude, tvb, ie_offset, 3, FALSE);
+			break;
+		default:
+			break;
+		}
+
 		break;
 	case 9:			
 		/* UNC SGW IP Address 
@@ -942,14 +949,14 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		if ( octet == 0x57 ){ /* IPv6 */
 
 		}else{ /* All other values shall be interpreted as Ipv4 address in this version of the protocol.*/
-			tvb_memcpy(tvb, (guint8 *)&sgw_ipv4_address, ie_offset, 4);
+			sgw_ipv4_address = tvb_get_ipv4(tvb, ie_offset);
 			proto_tree_add_ipv4(urr_ie_tree, hf_uma_urr_sgw_ipv4, tvb, ie_offset, 4, sgw_ipv4_address);
 
 		}
 		break;		 
 	case 10:		/* UNC SGW Fully Qualified Domain/Host Name */
 		if ( ie_len > 0){
-			string = ep_tvb_get_string(tvb, ie_offset, ie_len);
+			string = tvb_get_ephemeral_string(tvb, ie_offset, ie_len);
 			proto_tree_add_string(urr_ie_tree, hf_uma_urr_FQDN, tvb, ie_offset, ie_len, string);
 		}else{
 			proto_tree_add_text(urr_ie_tree,tvb,offset,1,"FQDN not present");
@@ -965,12 +972,11 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		/* UMA Cell Description 
 		 * The rest of the IE is coded as in [TS 44.018], Cell Description IE, not including IEI and length, if present
 		 */
-		de_rr_cell_dsc(tvb, urr_ie_tree, ie_offset, ie_len, NULL);
+		de_rr_cell_dsc(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 14:		
 		/* UMA Control Channel Description 
 		 */
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_EC, tvb, ie_offset, 1, FALSE);
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_ECMC, tvb, ie_offset, 1, FALSE);
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_NMO, tvb, ie_offset, 1, FALSE);
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_GPRS, tvb, ie_offset, 1, FALSE);
@@ -992,12 +998,13 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_3GECS, tvb, ie_offset, 1, FALSE);
 		ie_offset++;
 		proto_tree_add_text(urr_ie_tree,tvb,ie_offset,2,"Access Control Class N");
+		/* These fields are specified and described in 3GPP TS 44.018 and 3GPP TS 22.011. */
 		break;
 	case 15:		
 		/* Cell Identifier List 
 		 * The rest of the IE is coded as in [TS 48.008], not including IEI and length, if present
 		 */
-		be_cell_id_list(tvb, urr_ie_tree, offset, ie_len, NULL);
+		be_cell_id_list(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 16:		/* TU3907 Timer */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_TU3907_timer, tvb, ie_offset, 2, FALSE);
@@ -1007,7 +1014,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		break;
 	case 18:		/* Routing Area Identification */
 		/* The rest of the IE is coded as in [TS 24.008] not including IEI and length, if present.*/
-		de_gmm_rai(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_gmm_rai(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 19:		/* UMA Band */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_UMA_band, tvb, ie_offset, 1, FALSE);
@@ -1044,32 +1051,32 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		/* Channel Mode 
 		 * The rest of the IE is coded as in [TS 44.018], not including IEI and length, if present
 		 */
-		de_rr_ch_mode(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_rr_ch_mode(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 28:		
 		/* Mobile Station Classmark 2 
 		 * The rest of the IE is coded as in [TS 24.008], not including IEI and length, if present
 		 */
-		de_ms_cm_2(tvb, urr_ie_tree, ie_offset, ie_len, NULL);
+		de_ms_cm_2(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 29:		
 		/* RR Cause
 		 * The rest of the IE is coded as in [TS 44.018], not including IEI and length, if present
 		 */
-		de_rr_cause(tvb, urr_ie_tree, ie_offset, 1, NULL);
+		de_rr_cause(tvb, urr_ie_tree, ie_offset, 1, NULL, 0);
 		break;
 	case 30:		
 		/* Cipher Mode Setting
 		 * Note: The coding of fields SC and algorithm identifier is defined in [44.018] 
 		 * as part of the Cipher Mode Setting IE.
 		 */
-		de_rr_cip_mode_set(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_rr_cip_mode_set(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 31:		
 		/* GPRS Resumption 
 		 * If the target RAT is GERAN, the rest of the IE is coded as HANDOVER COMMAND message in [TS 44.018]
 		 */
-		dtap_rr_ho_cmd(tvb, urr_ie_tree, offset, ie_len);
+		dtap_rr_ho_cmd(tvb, urr_ie_tree, ie_offset, ie_len);
 		break;
 	case 32:		
 		/* Handover From UMAN Command 
@@ -1091,41 +1098,36 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		 * [TS 44.018]:10.5.2.41a
 		 * The TLLI is encoded as a binary number with a length of 4 octets. TLLI is defined in 3GPP TS 23.003
 		 */
-		de_rr_tlli(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_rr_tlli(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 35:		
 		/* Packet Flow Identifier 
 		 * The rest of the IE is coded as in [TS 24.008], not including IEI and length, if present.
 		 */
-		de_sm_pflow_id(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_sm_pflow_id(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 36:		
 		/* Suspension Cause 
 		 * The rest of the IE is coded as in [TS 44.018], not including IEI and length, if present.
 		 */		
-		de_rr_sus_cau(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_rr_sus_cau(tvb, urr_ie_tree, ie_offset, ie_len, NULL, 0);
 		break;
 	case 37:		/* TU3920 Timer */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_TU3920_timer, tvb, ie_offset, 2, FALSE);
 		break;
+		/* 11.2.38 QoS */
 	case 38:		
 		/* QoS 
-		 * The rest of the IE is coded as QoS Profile IE in [TS 48.018], not including IEI and length, if present.
-		 * octet 3-4	Peak bit rate provided by the network, coded as the Bucket Leak Rate "R value" part, 
-		 * see sub-clause 11.3.4 (note)
-		 * NOTE:	The bit rate 0 (zero) shall mean "best effort" in this IE.
-		 * The R field is the binary encoding of the rate information expressed in 100 bits/s increments, 
-		 * starting from 0 x 100 bits/s until 65 535 x 100 bits/s (6 Mbps).
+		 * PEAK_THROUGHPUT_CLASS (octet 3, bits 1-4)
+		 * This field is coded as PEAK_THROUGHPUT_CLASS field in
+		 * the Channel Request Description information
+		 * element specified in [TS 44.060]
 		 */
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_rate, tvb, ie_offset, 1, FALSE);
-		ie_offset = ie_offset + 2;
-		/*
-		 * octet 5	SPARE	C/R	T	A	Precedence 
-		 */
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_precedence, tvb, ie_offset, 1, FALSE);
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_a_bit, tvb, ie_offset, 1, FALSE);
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_t_bit, tvb, ie_offset, 1, FALSE);
-		proto_tree_add_item(urr_ie_tree, hf_uma_urr_c_r_bit, tvb, ie_offset, 1, FALSE);
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_peak_tpt_cls, tvb, ie_offset, 1, FALSE);
+		/* RADIO_PRIORITY (octet 3, bits 5-6) */
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_radio_pri, tvb, ie_offset, 1, FALSE);
+		/* RLC_MODE (octet 3, bit 7) */
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_rlc_mode, tvb, ie_offset, 1, FALSE);
 		break;
 	case 39:		/* URLC Cause */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_URLCcause, tvb, ie_offset, 1, FALSE);
@@ -1188,6 +1190,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		 */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_payload_type, tvb, ie_offset, 1, FALSE);
 		break;
+	/* 11.2.55 Multirate Configuration */
 	case 55:		
 		/* Multi-rate Configuration 
 		 * The rest of the IE is coded as in [TS 44.018], not including IEI and length, if present
@@ -1265,6 +1268,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		/* UMA Service Zone string, 1st character */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_uma_service_zone_str, tvb, ie_offset, str_len, FALSE);
 		break;
+	/* 11.2.63 RTP Redundancy Configuration */
 	case 63:
 		/* RTP Redundancy Configuration */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_window_size, tvb, ie_offset, 1, FALSE);
@@ -1280,7 +1284,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		/* Classmark Enquiry Mask 
 		 * The rest of the IE is the Classmark Enquiry Mask coded as in [TS 44.018], not including IEI and length, if present
 		 */
-		de_rr_cm_enq_mask(tvb, urr_ie_tree, offset, ie_len, NULL);
+		de_rr_cm_enq_mask(tvb, urr_ie_tree, offset, ie_len, NULL, 0);
 		break;
 	case 66:
 		/* UTRAN Cell Identifier List 
@@ -1290,7 +1294,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		octet = tvb_get_guint8(tvb,ie_offset);
 		ie_offset++;
 		if ( octet == 0 ){
-			ie_offset = dissect_mcc_mnc(tvb, urr_ie_tree, ie_offset);
+			ie_offset = dissect_e212_mcc_mnc(tvb, urr_ie_tree, ie_offset);
 			proto_tree_add_item(urr_ie_tree, hf_uma_urr_lac, tvb, ie_offset, 2, FALSE);
 			ie_offset = ie_offset + 2;
 			/* The octets 9-12 are coded as shown in 3GPP TS 25.331, Table 'Cell identity'. 
@@ -1317,9 +1321,25 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		/* TODO insert while loop here */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_lac, tvb, ie_offset, 2, FALSE);
 		break;
+	/* 11.2.70 Received Signal Level List */
 	case 70:
 		/* Received Signal Level List */
 		break;
+	/* 11.2.71 Required UMA Services */
+	case 71:
+		/* CBS Cell Broadcast Service (octet 3) */
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_cbs, tvb, ie_offset, 1, FALSE);
+		break;
+	/* 11.2.72 Broadcast Container */
+	case 72:
+		octet = tvb_get_guint8(tvb,ie_offset);
+		proto_tree_add_item(urr_ie_tree, hf_uma_urr_num_of_cbs_frms , tvb, ie_offset, 1, FALSE);
+		proto_tree_add_text(urr_ie_tree, tvb, ie_offset + 1, ie_len-1,"CBS Frames - Not decoded");
+		break;
+	/* 11.2.73 3G Cell Identity */
+	case 73:
+		/* The rest of the IE is coded as in [TS 25.331] not including IEI and length, if present. */
+		break;
 	case 96:		/* MS Radio Identity */
 		proto_tree_add_item(urr_ie_tree, hf_uma_urr_radio_type_of_id, tvb, ie_offset, 1, FALSE);
 		octet = tvb_get_guint8(tvb,ie_offset);
@@ -1343,7 +1363,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		if ( octet == 0x57 ){ /* IPv6 */
 
 		}else{ /* All other values shall be interpreted as Ipv4 address in this version of the protocol.*/
-			tvb_memcpy(tvb, (guint8 *)&unc_ipv4_address, ie_offset, 4);
+			unc_ipv4_address = tvb_get_ipv4(tvb, ie_offset);
 			proto_tree_add_ipv4(urr_ie_tree, hf_uma_urr_unc_ipv4, tvb, ie_offset, 4, unc_ipv4_address);
 			rtp_ipv4_address = unc_ipv4_address;
 
@@ -1351,7 +1371,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		break;
 	case 98:		/* UNC Fully Qualified Domain/Host Name */
 		if ( ie_len > 0){
-			string = ep_tvb_get_string(tvb, ie_offset, ie_len);
+			string = tvb_get_ephemeral_string(tvb, ie_offset, ie_len);
 			proto_tree_add_string(urr_ie_tree, hf_uma_unc_FQDN, tvb, ie_offset, ie_len, string);
 		}else{
 			proto_tree_add_text(urr_ie_tree,tvb,offset,1,"UNC FQDN not present");
@@ -1367,7 +1387,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		if ( octet == 0x57 ){ /* IPv6 */
 
 		}else{ /* All other values shall be interpreted as Ipv4 address in this version of the protocol.*/
-			tvb_memcpy(tvb, (guint8 *)&GPRS_user_data_ipv4_address, ie_offset, 4);
+			GPRS_user_data_ipv4_address = tvb_get_ipv4(tvb, ie_offset);
 			proto_tree_add_ipv4(urr_ie_tree, hf_uma_urr_GPRS_user_data_transport_ipv4, tvb, ie_offset, 4, GPRS_user_data_ipv4_address);
 
 		}
@@ -1516,6 +1536,7 @@ dissect_uma(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			
 		proto_tree_add_item(uma_tree, hf_uma_pd, tvb, offset, 1, FALSE);
 		switch  ( pd ){
+		case 0: /* URR_C */
 		case 1: /* URR */
 			offset++;
 			octet = tvb_get_guint8(tvb,offset);
@@ -1775,13 +1796,18 @@ proto_register_uma(void)
 		},
 		{ &hf_uma_urr_tura,
 			{ "TURA, Type of Unlicensed Radio","uma.urr.tura",
-			FT_UINT8,BASE_DEC,  VALS(tura_vals), 0x3,          
+			FT_UINT8,BASE_DEC,  VALS(tura_vals), 0xf,          
 			"TURA, Type of Unlicensed Radio", HFILL }
 		},
-		{ &hf_uma_urr_tlra,
-			{ "TLRA, Type of Licensed Radio","uma.urr.tlra",
-			FT_UINT8,BASE_DEC, VALS(tlra_vals), 0xc,          
-			"TLRA, Type of Licensed Radio", HFILL }
+		{ &hf_uma_urr_gc,
+			{ "GC, GERAN Capable","uma.urr.gc",
+			FT_UINT8,BASE_DEC,  VALS(gc_vals), 0x10,          
+			"GC, GERAN Capable", HFILL }
+		},
+		{ &hf_uma_urr_uc,
+			{ "UC, UTRAN Capable","uma.urr.uc",
+			FT_UINT8,BASE_DEC,  VALS(uc_vals), 0x20,          
+			"GC, GERAN Capable", HFILL }
 		},
 		{ &hf_uma_urr_rrs,
 			{ "RTP Redundancy Support(RRS)","uma.urr.rrs",
@@ -1793,6 +1819,21 @@ proto_register_uma(void)
 			FT_UINT8,BASE_DEC, VALS(type_of_shape_vals), 0xf,          
 			"Location estimate", HFILL }
 		},
+		{ &hf_uma_urr_sign_of_latitude,
+			{ "ign of latitude","uma.urr.sign_of_latitude",
+			FT_UINT8,BASE_DEC, VALS(sign_of_latitude_vals), 0x80,          
+			"Sign of latitude", HFILL }
+		},
+		{ &hf_uma_urr_degrees_of_latitude,
+			{ "Degrees of latitude","uma.urr.sign_of_latitude",
+			FT_UINT32,BASE_DEC, NULL, 0x3fffff,          
+			"Degrees of latitude", HFILL }
+		},
+		{ &hf_uma_urr_degrees_of_longitude,
+			{ "Degrees of longitude","uma.urr.sign_of_longitude",
+			FT_UINT32,BASE_DEC, NULL, 0xffffff,          
+			"Degrees of longitude", HFILL }
+		},
 		{ &hf_uma_urr_IP_Address_type,
 			{ "IP address type number value","uma.urr.ip_type",
 			FT_UINT8,BASE_DEC,  VALS(IP_address_type_vals), 0x0,          
@@ -1818,11 +1859,6 @@ proto_register_uma(void)
 			FT_UINT8,BASE_DEC,  VALS(discovery_reject_cause_vals), 0x0,          
 			"Discovery Reject Cause", HFILL }
 		},
-		{ &hf_uma_urr_EC,
-			{ "EC Emergency Call allowed","uma.urr.EC",
-			FT_UINT8,BASE_DEC,  VALS(EC_vals), 0x1,          
-			"EC Emergency Call allowed", HFILL }
-		},
 		{ &hf_uma_urr_ECMC,
 			{ "ECMC, Early Classmark Sending Control","uma.urr.is_rej_cau",
 			FT_UINT8,BASE_DEC,  VALS(ECMC_vals), 0x2,          
@@ -1910,7 +1946,7 @@ proto_register_uma(void)
 		},
 		{ &hf_uma_urr_GSM_RR_state,
 			{ "GSM RR State value","uma.urr.gsmrrstate",
-			FT_UINT8,BASE_DEC,  VALS(GRS_GSM_RR_State_vals), 0x1,          
+			FT_UINT8,BASE_DEC,  VALS(GRS_GSM_RR_State_vals), 0x7,          
 			"GSM RR State value", HFILL }
 		},
 		{ &hf_uma_urr_UMA_band,
@@ -1983,30 +2019,20 @@ proto_register_uma(void)
 			FT_UINT16,BASE_DEC,  NULL, 0x0,          
 			"TU3920 Timer value(seconds)", HFILL }
 		},
-		{ &hf_uma_urr_rate,
-			{ "Peak bit rate provided by the network(100 bits/s)","uma.urr.tu3920",
-			FT_UINT16,BASE_DEC,  NULL, 0x0,          
-			"Peak bit rate provided by the network(100 bits/s)", HFILL }
+		{ &hf_uma_urr_peak_tpt_cls,
+			{ "PEAK_THROUGHPUT_CLASS","uma.urr.peak_tpt_cls",
+			FT_UINT8,BASE_DEC,  NULL, 0x0f,          
+			"PEAK_THROUGHPUT_CLASS", HFILL }
 		},
-		{ &hf_uma_urr_precedence,
-			{ "Precedence","uma.urr.precedence",
-			FT_UINT8,BASE_DEC,  VALS(precedence_vals), 0x7,          
-			"Precedence", HFILL }
+		{ &hf_uma_urr_radio_pri,
+			{ "Radio Priority","uma.urr.radio_pri",
+			FT_UINT8,BASE_DEC,  VALS(radio_pri_vals), 0x30,          
+			"RADIO_PRIORITY", HFILL }
 		},
-		{ &hf_uma_urr_a_bit,
-			{ "A bit","uma.urr.a_bit",
-			FT_UINT8,BASE_DEC,  VALS(a_bit_vals), 0x8,          
-			"A bit", HFILL }
-		},
-		{ &hf_uma_urr_t_bit,
-			{ "T bit","uma.urr.t_bit",
-			FT_UINT8,BASE_DEC,  VALS(t_bit_vals), 0x10,          
-			"T bit", HFILL }
-		},
-		{ &hf_uma_urr_c_r_bit,
-			{ "C/R bit","uma.urr.c_r_bit",
-			FT_UINT8,BASE_DEC,  VALS(c_r_bit_vals), 0x20,          
-			"C/R bit", HFILL }
+		{ &hf_uma_urr_rlc_mode,
+			{ "RLC mode","uma.urr.rrlc_mode",
+			FT_UINT8,BASE_DEC,  VALS(rlc_mode_vals), 0x80,          
+			"RLC mode", HFILL }
 		},
 		{ &hf_uma_urr_URLCcause,
 			{ "URLC Cause","uma.urr.URLCcause",
@@ -2168,6 +2194,16 @@ proto_register_uma(void)
 			FT_UINT8,BASE_DEC,  NULL, 0x0,          
 			"Number of PLMN:s", HFILL }
 		},
+		{ &hf_uma_urr_cbs,
+			{ "CBS Cell Broadcast Service","uma.urr.cbs",
+			FT_UINT8,BASE_DEC,  VALS(cbs_vals), 0x01,          
+			"CBS Cell Broadcast Service", HFILL }
+		},
+		{ &hf_uma_urr_num_of_cbs_frms,
+			{ "Number of CBS Frames","uma.urr.num_of_cbs_frms",
+			FT_UINT8,BASE_DEC,  NULL, 0x0,          
+			"Number of CBS Frames", HFILL }
+		},
 		{ &hf_uma_urr_ms_radio_id,
 			{ "MS Radio Identity","uma.urr.ms_radio_id",
 			FT_ETHER, BASE_DEC, NULL, 0x00,          
