@@ -1,7 +1,7 @@
 /* packet-vines.c
  * Routines for Banyan VINES protocol packet disassembly
  *
- * $Id: packet-vines.c 13925 2005-03-26 23:02:24Z gerald $
+ * $Id: packet-vines.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Don Lafontaine <lafont02@cn.ca>
  *
@@ -33,11 +33,11 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include "packet-vines.h"
-#include "etypes.h"
-#include "ppptypes.h"
+#include <epan/etypes.h>
+#include <epan/ppptypes.h>
 #include <epan/ipproto.h>
-#include "arcnet_pids.h"
-#include "llcsaps.h"
+#include <epan/arcnet_pids.h>
+#include <epan/llcsaps.h>
 
 #define UDP_PORT_VINES	573
 
@@ -106,7 +106,7 @@ dissect_vines_frp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	guint8   vines_frp_ctrl;
 	proto_tree *vines_frp_tree;
 	proto_item *ti;
-	gchar	frp_flags_str[32];
+	gchar	*frp_flags_str="";
 	tvbuff_t *next_tvb;
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
@@ -129,23 +129,23 @@ dissect_vines_frp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		switch (vines_frp_ctrl) {
 
 		case 0:
-			strcpy(frp_flags_str, "middle");
+			frp_flags_str= "middle";
 			break;
 
 		case 1:
-			strcpy(frp_flags_str, "first");
+			frp_flags_str= "first";
 			break;
 
 		case 2:
-			strcpy(frp_flags_str, "last");
+			frp_flags_str= "last";
 			break;
 
 		case 3:
-			strcpy(frp_flags_str, "only");
+			frp_flags_str= "only";
 			break;
 
 		default:
-			strcpy(frp_flags_str, "please report: unknown");
+			frp_flags_str= "please report: unknown";
 			break;
 		}
 
@@ -1026,11 +1026,11 @@ static const value_string vines_rtp_info_type_vals[] = {
 };
 
 static void rtp_show_machine_type(proto_tree *tree, tvbuff_t *tvb, int offset,
-    char *tag);
+    const char *tag);
 static void rtp_show_flags(proto_tree *tree, tvbuff_t *tvb, int offset,
-    char *tag);
+    const char *tag);
 static int srtp_show_machine_info(proto_tree *tree, tvbuff_t *tvb, int offset,
-    char *tag);
+    const char *tag);
 static int rtp_show_gateway_info(proto_tree *tree, tvbuff_t *tvb, int offset,
     guint8 link_addr_length, guint8 source_route_length);
 
@@ -1404,7 +1404,7 @@ dissect_vines_rtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 }
 
 static void
-rtp_show_machine_type(proto_tree *tree, tvbuff_t *tvb, int offset, char *tag)
+rtp_show_machine_type(proto_tree *tree, tvbuff_t *tvb, int offset, const char *tag)
 {
 	guint8 machine_type;
 	proto_item *ti;
@@ -1432,7 +1432,7 @@ rtp_show_machine_type(proto_tree *tree, tvbuff_t *tvb, int offset, char *tag)
 }
 
 static void
-rtp_show_flags(proto_tree *tree, tvbuff_t *tvb, int offset, char *tag)
+rtp_show_flags(proto_tree *tree, tvbuff_t *tvb, int offset, const char *tag)
 {
 	guint8 flags;
 	proto_item *ti;
@@ -1461,7 +1461,7 @@ rtp_show_flags(proto_tree *tree, tvbuff_t *tvb, int offset, char *tag)
 }
 
 static int
-srtp_show_machine_info(proto_tree *tree, tvbuff_t *tvb, int offset, char *tag)
+srtp_show_machine_info(proto_tree *tree, tvbuff_t *tvb, int offset, const char *tag)
 {
 	guint16 metric;
 	guint8 node_type;
