@@ -2,7 +2,7 @@
  * Routines for Web Cache Coordination Protocol dissection
  * Jerry Talkington <jtalkington@users.sourceforge.net>
  *
- * $Id: packet-wccp.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-wccp.c 17046 2006-01-17 10:56:06Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,12 +31,9 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/strutil.h>
+#include <epan/emem.h>
 #include "packet-wccp.h"
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 static int proto_wccp = -1;
 static int hf_wccp_message_type = -1;	/* the message type */
 static int hf_wccp_version = -1;	/* protocol version */
@@ -273,7 +270,7 @@ dissect_wccp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    "Number of Web Caches: %u", cache_count);
 			offset += 4;
 			for (i = 0; i < cache_count; i++) {
-				tvb_memcpy(tvb, (guint8 *)&ipaddr, offset, 4);
+				ipaddr = tvb_get_ipv4(tvb, offset);
 				proto_tree_add_ipv4_format(wccp_tree,
 				    hf_cache_ip, tvb, offset, 4,
 				    ipaddr,
@@ -381,21 +378,14 @@ wccp_bucket_info(guint8 bucket_info, proto_tree *bucket_tree, guint32 start,
 static gchar *
 bucket_name(guint8 bucket)
 {
-	static gchar str[4][10+1];
-	static gchar *cur;
+	gchar *cur=NULL;
 
-	if (cur == &str[0][0])
-		cur = &str[1][0];
-	else if (cur == &str[1][0])
-		cur = &str[2][0];
-	else if (cur == &str[2][0])
-		cur = &str[3][0];
-	else
-		cur = &str[0][0];
-	if (bucket == 0xff)
-		strcpy(cur, "Unassigned");
-	else
-		sprintf(cur, "%u", bucket);
+	if (bucket == 0xff) {
+		cur="Unassigned";
+	} else {
+		cur=ep_alloc(10+1);
+		g_snprintf(cur, 10+1, "%u", bucket);
+	}
 	return cur;
 }
 
@@ -972,21 +962,13 @@ dissect_wccp2_router_assignment_element(tvbuff_t *tvb, int offset,
 static gchar *
 assignment_bucket_name(guint8 bucket)
 {
-	static gchar str[4][10+1];
-	static gchar *cur;
+	gchar *cur=NULL;
 
-	if (cur == &str[0][0])
-		cur = &str[1][0];
-	else if (cur == &str[1][0])
-		cur = &str[2][0];
-	else if (cur == &str[2][0])
-		cur = &str[3][0];
-	else
-		cur = &str[0][0];
-	if (bucket == 0xff)
-		strcpy(cur, "Unassigned");
-	else {
-		sprintf(cur, "%u%s", bucket >> 1,
+	if (bucket == 0xff) {
+		cur="Unassigned";
+	} else {
+		cur=ep_alloc(10+1);
+		g_snprintf(cur, 10+1, "%u%s", bucket >> 1,
 		    (bucket & 0x01) ? " (Alt)" : "");
 	}
 	return cur;
@@ -1209,10 +1191,10 @@ dissect_32_bit_capability_flags(tvbuff_t *tvb, int curr_offset,
 	proto_item *tm;
 	proto_tree *method_tree;
 	int i;
-	char flags_string[128+1];
+	char *flags_string;
 	char *p;
-	int space_left;
-	char buf[1025];
+	size_t returned_length, str_index = 0;
+	char *buf;
 
 	if (capability_val_len != 4) {
 		proto_tree_add_text(element_tree, tvb,
@@ -1222,31 +1204,39 @@ dissect_32_bit_capability_flags(tvbuff_t *tvb, int curr_offset,
 	}
 
 	capability_val = tvb_get_ntohl(tvb, curr_offset + 4);
-	flags_string[0] = '\0';
-	p = &flags_string[0];
-	space_left = sizeof flags_string;
+#define FLAGS_STRING_LEN (128+1)
+	flags_string=ep_alloc(FLAGS_STRING_LEN);
+	flags_string[0] = 0;
 	for (i = 0; flags[i].short_name != NULL; i++) {
 		if (capability_val & flags[i].value) {
-			if (p != &flags_string[0]) {
-				snprintf(p, space_left, ",");
-				p = &flags_string[strlen(flags_string)];
+			if (str_index != 0) {
+				returned_length = g_snprintf(&flags_string[str_index],
+					FLAGS_STRING_LEN-str_index, ",");
+				str_index += MIN(returned_length, FLAGS_STRING_LEN-str_index);
 			}
-			snprintf(p, space_left, "%s", flags[i].short_name);
-			p = &flags_string[strlen(flags_string)];
+			returned_length = g_snprintf(&flags_string[str_index],
+				FLAGS_STRING_LEN-str_index, "%s", flags[i].short_name);
+			str_index += MIN(returned_length, FLAGS_STRING_LEN-str_index);
 		}
 	}
 	tm = proto_tree_add_text(element_tree, tvb, curr_offset+4, 4,
 	    "Value: 0x%08X (%s)", capability_val, flags_string);
+
 	method_tree = proto_item_add_subtree(tm, ett);
+#define BUF_SIZE 1024
+	buf=ep_alloc(BUF_SIZE);
+	buf[0]=0;
 	for (i = 0; flags[i].long_name != NULL; i++) {
 		p = decode_bitfield_value(buf, capability_val,
 		      flags[i].value, 32);
-		strcpy(p, flags[i].long_name);
-		strcat(p, ": ");
-		if (capability_val & flags[i].value)
-		    strcat(p, "Supported");
-		else
-		    strcat(p, "Not supported");
+		str_index = MIN(p-buf, BUF_SIZE);
+		returned_length = g_snprintf(&flags_string[str_index], BUF_SIZE-str_index,
+			"%s: %s", flags[i].long_name,
+				(capability_val & flags[i].value)?
+					"Supported":
+					"Not supported"
+				);
+		str_index += MIN(returned_length, BUF_SIZE-str_index);
 		proto_tree_add_text(method_tree, tvb, curr_offset+4, 4,
 		    "%s", buf);
 	}
