@@ -1,7 +1,7 @@
 /* packet-windows-common.c
  * Routines for dissecting various Windows data types
  *
- * $Id: packet-windows-common.c 14788 2005-06-26 21:11:22Z guy $
+ * $Id: packet-windows-common.c 17038 2006-01-16 07:59:44Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,10 +29,11 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-dcerpc.h"
 #include "packet-smb-sidsnooping.h"
 #include "packet-windows-common.h"
-#include "smb.h"	/* for "sid_name_snooping" */
+#include <epan/dissectors/packet-smb.h>	/* for "sid_name_snooping" */
 
 static int hf_nt_sec_desc_revision = -1;
 static int hf_nt_sec_desc_type_owner_defaulted = -1;
@@ -1225,12 +1226,6 @@ dissect_nt_64bit_time(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_date)
 	return offset;
 }
 
-static void
-free_g_string(void *arg)
-{
-	g_string_free(arg, TRUE);
-}
-
 /* Dissect a NT SID.  Label it with 'name' and return a string version of
    the SID in the 'sid_str' parameter which must be freed by the caller.
    hf_sid can be -1 if the caller doesnt care what name is used and then 
@@ -1254,10 +1249,14 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
 	int na_offset;
         guint auth = 0;   /* FIXME: What if it is larger than 32-bits */
 	int i;
-	GString *gstr;
-	char sid_string[245];
+#define MAX_STR_LEN 256
+	char *str;
+	char *sid_string;
 	char *sid_name;
+	size_t returned_length;
+	unsigned long str_index;;
 
+	sid_string=ep_alloc(MAX_STR_LEN);
 	if(hf_sid==-1){
 		hf_sid=hf_nt_sid;
 	}
@@ -1287,9 +1286,9 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
 
           sa_offset = offset;
 
-          gstr = g_string_new("");
-
-	  CLEANUP_PUSH(free_g_string, gstr);
+          str = ep_alloc(MAX_STR_LEN);
+          str_index = 0;
+          str[str_index]=0;
 
 	  /* sub authorities, leave RID to last */
 	  for(i=0; i < (num_auth > 4?(num_auth - 1):num_auth); i++){
@@ -1302,8 +1301,10 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
 	     * and IA-64 runs little-endian, as does x86-64), we can (?)
 	     * assume that non le byte encodings will be "uncommon"?
 	     */
-             g_string_sprintfa(gstr, (i>0 ? "-%u" : "%u"),
-                  tvb_get_letohl(tvb, offset));
+
+             returned_length = g_snprintf(&str[str_index], MAX_STR_LEN-str_index,
+                (i>0 ? "-%u" : "%u"), tvb_get_letohl(tvb, offset));
+             str_index += MIN(returned_length, MAX_STR_LEN-str_index);
              offset+=4;
 	  }
 
@@ -1313,10 +1314,10 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
             rid_present=TRUE;
             rid_offset=offset;
 	    offset+=4;
-            g_snprintf(sid_string, sizeof(sid_string), "S-1-%u-%s-%u", auth, gstr->str, rid);
+            g_snprintf(sid_string, MAX_STR_LEN, "S-1-%u-%s-%u", auth, str, rid);
 	  } else {
             rid_present=FALSE;
-            g_snprintf(sid_string, sizeof(sid_string), "S-1-%u-%s", auth, gstr->str);
+            g_snprintf(sid_string, MAX_STR_LEN, "S-1-%u-%s", auth, str);
           }
 
           sid_name=NULL;
@@ -1336,7 +1337,7 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
           proto_tree_add_item(tree, hf_nt_sid_revision, tvb, rev_offset, 1, TRUE);
           proto_tree_add_item(tree, hf_nt_sid_num_auth, tvb, na_offset, 1, TRUE);
           proto_tree_add_text(tree, tvb, na_offset+1, 6, "Authority: %u", auth);
-          proto_tree_add_text(tree, tvb, sa_offset, num_auth * 4, "Sub-authorities: %s", gstr->str);
+          proto_tree_add_text(tree, tvb, sa_offset, num_auth * 4, "Sub-authorities: %s", str);
 
           if(rid_present){
             proto_tree_add_text(tree, tvb, rid_offset, 4, "RID: %u", rid);
@@ -1349,8 +1350,6 @@ dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
               *sid_str = g_strdup(sid_string);
             }
           }
-
-	  CLEANUP_CALL_AND_POP;
 	}
 
 
@@ -1655,11 +1654,15 @@ dissect_nt_access_mask(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 
 static int hf_nt_access_mask = -1;
 
+#define ACE_TYPE_ACCESS_ALLOWED		0
+#define ACE_TYPE_ACCESS_DENIED		1
+#define ACE_TYPE_SYSTEM_AUDIT		2
+#define ACE_TYPE_SYSTEM_ALARM		3
 static const value_string ace_type_vals[] = {
-  { 0, "Access Allowed"},
-  { 1, "Access Denied"},
-  { 2, "System Audit"},
-  { 3, "System Alarm"},
+  { ACE_TYPE_ACCESS_ALLOWED,	"Access Allowed"},
+  { ACE_TYPE_ACCESS_DENIED,	"Access Denied"},
+  { ACE_TYPE_SYSTEM_AUDIT,	"System Audit"},
+  { ACE_TYPE_SYSTEM_ALARM,	"System Alarm"},
   { 0, NULL}
 };
 static const true_false_string tfs_ace_flags_object_inherit = {
@@ -1705,7 +1708,7 @@ dissect_nt_v2_ace_flags(tvbuff_t *tvb, int offset, proto_tree *parent_tree,
 	proto_item *item = NULL;
 	proto_tree *tree = NULL;
 	guint8 mask;
-	char *sep = " ";
+	const char *sep = " ";
 
 	mask = tvb_get_guint8(tvb, offset);
 
@@ -1760,8 +1763,8 @@ dissect_nt_v2_ace(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	proto_item *item = NULL;
 	proto_tree *tree = NULL;
 	int old_offset = offset;
-	guint16 size;
 	char *sid_str = NULL;
+	guint16 size;
 	guint8 type;
 	guint8 flags;
 	guint32 perms = 0;
@@ -1793,22 +1796,36 @@ dissect_nt_v2_ace(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	}
 	proto_tree_add_uint(tree, hf_nt_ace_size, tvb, offset, 2, size);
 	offset += 2;
+	
+	/* some ACE types we not yet handle store other things than access mask
+	 * and SID in here.
+	 * sometimes things that are not related at all to access control.
+	 *    naughty naughty.    -- ronnie
+	 */
+	switch(type){
+	case ACE_TYPE_ACCESS_ALLOWED:
+	case ACE_TYPE_ACCESS_DENIED:
+	case ACE_TYPE_SYSTEM_AUDIT:
+	case ACE_TYPE_SYSTEM_ALARM:
 
-	/* access mask */
-	offset = dissect_nt_access_mask(
-		tvb, offset, pinfo, tree, drep, 
-		hf_nt_access_mask, ami, &perms);
+		/* access mask */
+		offset = dissect_nt_access_mask(
+			tvb, offset, pinfo, tree, drep, 
+			hf_nt_access_mask, ami, &perms);
 
-	/* SID */
-	offset = dissect_nt_sid(tvb, offset, tree, "ACE", &sid_str, -1);
+		/* SID */
+		offset = dissect_nt_sid(tvb, offset, tree, "ACE", &sid_str, -1);
 
-	if (item)
-		proto_item_append_text(
-			item, "%s, flags 0x%02x, %s, mask 0x%08x", sid_str, flags,
-			val_to_str(type, ace_type_vals, "Unknown ACE type (0x%02x)"),
-			perms);
-
-	g_free(sid_str);
+		if (item)
+			proto_item_append_text(
+				item, "%s, flags 0x%02x, %s, mask 0x%08x", sid_str, flags,
+				val_to_str(type, ace_type_vals, "Unknown ACE type (0x%02x)"),
+				perms);
+		if(sid_str){
+			g_free(sid_str);
+		}
+		break;
+	};
 
 	proto_item_set_len(item, offset-old_offset);
 
@@ -1820,7 +1837,7 @@ dissect_nt_v2_ace(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 static int
 dissect_nt_acl(tvbuff_t *tvb, int offset, packet_info *pinfo,
-	       proto_tree *parent_tree, guint8 *drep, char *name,
+	       proto_tree *parent_tree, guint8 *drep, const char *name,
 	       struct access_mask_info *ami)
 {
 	proto_item *item = NULL;
@@ -2009,23 +2026,23 @@ dissect_nt_sec_desc_type(tvbuff_t *tvb, int offset, proto_tree *parent_tree)
 
 int
 dissect_nt_sec_desc(tvbuff_t *tvb, int offset, packet_info *pinfo,
-		    proto_tree *parent_tree, guint8 *drep, int len, 
+		    proto_tree *parent_tree, guint8 *drep,
+		    gboolean len_supplied, int len, 
 		    struct access_mask_info *ami)
 {
 	proto_item *item = NULL;
 	proto_tree *tree = NULL;
 	guint16 revision;
-	int old_offset = offset;
+	int start_offset = offset;
+	int item_offset, end_offset;
 	guint32 owner_sid_offset;
 	guint32 group_sid_offset;
 	guint32 sacl_offset;
 	guint32 dacl_offset;
 
-	if(parent_tree){
-		item = proto_tree_add_text(parent_tree, tvb, offset, len,
-					   "NT Security Descriptor");
-		tree = proto_item_add_subtree(item, ett_nt_sec_desc);
-	}
+	item = proto_tree_add_text(parent_tree, tvb, offset, -1,
+				   "NT Security Descriptor");
+	tree = proto_item_add_subtree(item, ett_nt_sec_desc);
 
 	/* revision */
 	revision = tvb_get_letohs(tvb, offset);
@@ -2033,7 +2050,6 @@ dissect_nt_sec_desc(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		tvb, offset, 2, revision);
 	offset += 2;
 
-
 	switch(revision){
 	case 1:  /* only version we will ever see of this structure?*/
 	  /* type */
@@ -2041,52 +2057,120 @@ dissect_nt_sec_desc(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 	  /* offset to owner sid */
 	  owner_sid_offset = tvb_get_letohl(tvb, offset);
-	  proto_tree_add_text(tree, tvb, offset, 4, "Offset to owner SID: %u", owner_sid_offset);
+	  if(owner_sid_offset != 0 && owner_sid_offset < 20){
+	    /* Bogus value - points into fixed portion of descriptor */
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to owner SID: %u (bogus, must be >= 20)", owner_sid_offset);
+	    owner_sid_offset = 0;
+	  } else
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to owner SID: %u", owner_sid_offset);
 	  offset += 4;
 
 	  /* offset to group sid */
 	  group_sid_offset = tvb_get_letohl(tvb, offset);
-	  proto_tree_add_text(tree, tvb, offset, 4, "Offset to group SID: %u", group_sid_offset);
+	  if(group_sid_offset != 0 && group_sid_offset < 20){
+	    /* Bogus value - points into fixed portion of descriptor */
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to group SID: %u (bogus, must be >= 20)", group_sid_offset);
+	    group_sid_offset = 0;
+	  } else
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to group SID: %u", group_sid_offset);
 	  offset += 4;
 
 	  /* offset to sacl */
 	  sacl_offset = tvb_get_letohl(tvb, offset);
-	  proto_tree_add_text(tree, tvb, offset, 4, "Offset to SACL: %u", sacl_offset);
+	  if(sacl_offset != 0 && sacl_offset < 20){
+	    /* Bogus value - points into fixed portion of descriptor */
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to SACL: %u (bogus, must be >= 20)", sacl_offset);
+	    sacl_offset = 0;
+	  } else
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to SACL: %u", sacl_offset);
 	  offset += 4;
 
 	  /* offset to dacl */
 	  dacl_offset = tvb_get_letohl(tvb, offset);
-	  proto_tree_add_text(tree, tvb, offset, 4, "Offset to DACL: %u", dacl_offset);
+	  if(dacl_offset != 0 && dacl_offset < 20){
+	    /* Bogus value - points into fixed portion of descriptor */
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to DACL: %u (bogus, must be >= 20)", dacl_offset);
+	    dacl_offset = 0;
+	  } else
+	    proto_tree_add_text(tree, tvb, offset, 4, "Offset to DACL: %u", dacl_offset);
 	  offset += 4;
 
+	  end_offset = offset;
+
 	  /*owner SID*/
 	  if(owner_sid_offset){
-	    if (len == -1)
-	      offset = dissect_nt_sid(tvb, offset, tree, "Owner", NULL, -1);
-	    else
-	      dissect_nt_sid(
-		      tvb, old_offset+owner_sid_offset, tree, "Owner", NULL, -1);
+	    item_offset = start_offset+owner_sid_offset;
+	    if (item_offset < start_offset) {
+	      /*
+	       * Overflow - throw an exception.
+	       */
+	      THROW(ReportedBoundsError);
+	    }
+	    offset = dissect_nt_sid(tvb, item_offset, tree, "Owner", NULL, -1);
+	    if (offset > end_offset)
+	      end_offset = offset;
 	  }
 
 	  /*group SID*/
 	  if(group_sid_offset){
-	    dissect_nt_sid(
-		    tvb, old_offset+group_sid_offset, tree, "Group", NULL, -1);
+	    item_offset = start_offset+group_sid_offset;
+	    if (item_offset < start_offset) {
+	      /*
+	       * Overflow - throw an exception.
+	       */
+	      THROW(ReportedBoundsError);
+	    }
+	    offset = dissect_nt_sid(tvb, item_offset, tree, "Group", NULL, -1);
+	    if (offset > end_offset)
+	      end_offset = offset;
 	  }
 
 	  /* sacl */
 	  if(sacl_offset){
-	    dissect_nt_acl(tvb, old_offset+sacl_offset, pinfo, tree, 
-			   drep, "System (SACL)", ami);
+	    item_offset = start_offset+sacl_offset;
+	    if (item_offset < start_offset) {
+	      /*
+	       * Overflow - throw an exception.
+	       */
+	      THROW(ReportedBoundsError);
+	    }
+	    offset = dissect_nt_acl(tvb, item_offset, pinfo, tree, 
+				    drep, "System (SACL)", ami);
+	    if (offset > end_offset)
+	      end_offset = offset;
 	  }
 
 	  /* dacl */
 	  if(dacl_offset){
-	    dissect_nt_acl(tvb, old_offset+dacl_offset, pinfo, tree, 
-			   drep, "User (DACL)", ami);
+	    item_offset = start_offset+dacl_offset;
+	    if (item_offset < start_offset) {
+	      /*
+	       * Overflow - throw an exception.
+	       */
+	      THROW(ReportedBoundsError);
+	    }
+	    offset = dissect_nt_acl(tvb, item_offset, pinfo, tree, 
+				    drep, "User (DACL)", ami);
+	    if (offset > end_offset)
+	      end_offset = offset;
 	  }
+	  break;
 
+	default:
+	  end_offset = offset;
+	  break;
 	}
+	if (len_supplied) {
+	  /* Make sure the length isn't too large (so that we get an
+	     overflow) */
+	  tvb_ensure_bytes_exist(tvb, start_offset, len);
+	} else {
+	  /* The length of the security descriptor is the difference
+	     between the starting offset and the offset past the last
+	     item in the descriptor. */
+	  len = end_offset - start_offset;
+	}
+	proto_item_set_len(item, len);
 
 	return offset+len;
 }
