@@ -2,7 +2,7 @@
  *
  * Routines to dissect WTLS component of WAP traffic.
  *
- * $Id: packet-wtls.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-wtls.c 15264 2005-08-08 18:50:39Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,10 +34,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
@@ -462,8 +458,8 @@ dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint coun
 	guint public_key = 0;
 	guint signature = 0;
 	char valStr[1024];
-        char *valBulk = NULL;
-        char *valMac = NULL;
+        const char *valBulk = NULL;
+        const char *valMac = NULL;
 
 	proto_item *ti;
 	proto_item *cli_key_item;
@@ -804,22 +800,22 @@ dissect_wtls_handshake(proto_tree *tree, tvbuff_t *tvb, guint offset, guint coun
                                {
                                        if (valMac != NULL)
                                        {
-                                               snprintf(valStr,1024,"%s, %s",valBulk,valMac);
+                                               g_snprintf(valStr,1024,"%s, %s",valBulk,valMac);
                                        }
                                        else
                                        {
-                                               snprintf(valStr,1024,"%s, Unknown MAC (0x%02x)",valBulk,tvb_get_guint8 (tvb, offset));
+                                               g_snprintf(valStr,1024,"%s, Unknown MAC (0x%02x)",valBulk,tvb_get_guint8 (tvb, offset));
                                        }
                                }
                                else
                                {
                                        if (valMac != NULL)
                                        {
-                                               snprintf(valStr,1024,"Unknown Bulk (0x%02x), %s",value,valMac);
+                                               g_snprintf(valStr,1024,"Unknown Bulk (0x%02x), %s",value,valMac);
                                        }
                                        else
                                        {
-                                               snprintf(valStr,1024,"Unknown Bulk (0x%02x), Unknown MAC (0x%02x)",value,
+                                               g_snprintf(valStr,1024,"Unknown Bulk (0x%02x), Unknown MAC (0x%02x)",value,
                                                                tvb_get_guint8 (tvb, offset));
                                        }
                                 }
