@@ -2,7 +2,7 @@
  *
  * Routines to dissect WTP component of WAP traffic.
  *
- * $Id: packet-wtp.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-wtp.c 17046 2006-01-17 10:56:06Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,14 +34,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "reassemble.h"
+#include <epan/reassemble.h>
+#include <epan/emem.h>
 #include "packet-wap.h"
 #include "packet-wtp.h"
 #include "packet-wsp.h"
@@ -310,8 +307,9 @@ wtp_handle_tpi(proto_tree *tree, tvbuff_t *tvb)
 static void
 dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-    static GString *szInfo = NULL;
+    char *szInfo;
     int		offCur		= 0; /* current offset from start of WTP data */
+    size_t		returned_length, str_index = 0;
 
     unsigned char  b0;
 
@@ -328,7 +326,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     proto_tree		*wtp_tree = NULL;
 
     char		pdut;
-    char		clsTransaction 	= ' ';
+    char		clsTransaction = 3;
     int			numMissing = 0;		/* Number of missing packets in a negative ack */
     int			i;
     tvbuff_t		*wsp_tvb = NULL;
@@ -337,8 +335,8 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     int			dataOffset;
     gint		dataLen;
 
-    if (szInfo == NULL)
-    	szInfo = g_string_sized_new(32);
+#define SZINFO_SIZE 256
+    szInfo=ep_alloc(SZINFO_SIZE);
 
     b0 = tvb_get_guint8 (tvb, offCur + 0);
     /* Discover Concatenated PDUs */
@@ -396,12 +394,14 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 #ifdef DEBUG
 	printf("WTP packet %u: tree = %p, pdu = %s (%u) length: %u\n",
 			pinfo->fd->num, tree,
-			match_strval(pdut, vals_wtp_pdu_type), pdut, tvb_length(tvb));
+			val_to_str(pdut, vals_wtp_pdu_type, "Unknown PDU type 0x%x"),
+			pdut, tvb_length(tvb));
 #endif
 
     /* Develop the string to put in the Info column */
-    g_string_sprintf(szInfo, "WTP %s",
+    returned_length =  g_snprintf(szInfo, SZINFO_SIZE, "WTP %s",
 		    val_to_str(pdut, vals_wtp_pdu_type, "Unknown PDU type 0x%x"));
+    str_index += MIN(returned_length, SZINFO_SIZE-str_index);
 
     switch (pdut) {
 	case INVOKE:
@@ -409,7 +409,9 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    TID = tvb_get_ntohs(tvb, offCur + 1);
 	    psn = 0;
 	    clsTransaction = transaction_class(tvb_get_guint8(tvb, offCur + 3));
-	    g_string_sprintfa(szInfo, " Class %d", clsTransaction);
+	    returned_length = g_snprintf(&szInfo[str_index], SZINFO_SIZE-str_index,
+		" Class %d", clsTransaction);
+            str_index += MIN(returned_length, SZINFO_SIZE-str_index);
 	    cbHeader = 4;
 	    break;
 
@@ -418,8 +420,11 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    fTTR = transmission_trailer(b0);
 	    TID = tvb_get_ntohs(tvb, offCur + 1);
 	    psn = tvb_get_guint8(tvb, offCur + 3);
-	    if (psn != 0)
-		g_string_sprintfa(szInfo, " (%u)", psn);
+	    if (psn != 0) {
+		returned_length = g_snprintf(&szInfo[str_index], SZINFO_SIZE-str_index,
+			" (%u)", psn);
+                str_index += MIN(returned_length, SZINFO_SIZE-str_index);
+	    }
 	    cbHeader = 4;
 	    break;
 
@@ -448,7 +453,8 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    break;
     };
     if (fRID) {
-	g_string_append( szInfo, " R" );
+	returned_length = g_snprintf(&szInfo[str_index], SZINFO_SIZE-str_index, " R" );
+        str_index += MIN(returned_length, SZINFO_SIZE-str_index);
     };
     /* In the interest of speed, if "tree" is NULL, don't do any work not
        necessary to generate protocol tree items. */
@@ -495,7 +501,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				", PDU: Invoke (%u)"
 				", Transaction Class: %s (%u)",
 				INVOKE,
-				match_strval(clsTransaction, vals_transaction_classes),
+				val_to_str(clsTransaction, vals_transaction_classes, "Undefined"),
 				clsTransaction);
 		break;
 
@@ -532,7 +538,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					", Reason: %s (%u)",
 					ABORT,
 					PROVIDER,
-					match_strval(reason, vals_abort_reason_provider), 
+					val_to_str(reason, vals_abort_reason_provider, "Undefined"),
 					reason);
 		}
 		else if (abortType == USER)
@@ -545,7 +551,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					", Reason: %s (%u)",
 					ABORT,
 					PROVIDER,
-					match_strval(reason, vals_wsp_reason_codes), 
+					val_to_str(reason, vals_wsp_reason_codes, "Undefined"),
 					reason);
 		}
 		break;
@@ -728,7 +734,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					if (check_col(pinfo->cinfo, COL_INFO)) {
 						col_append_fstr(pinfo->cinfo, COL_INFO,
 								"%s (WTP payload reassembled in packet %u)",
-								szInfo->str, fd_wtp->reassembled_in);
+								szInfo, fd_wtp->reassembled_in);
 					}
 					if (tree) {
 						proto_tree_add_text(wtp_tree, tvb, dataOffset, -1,
@@ -740,7 +746,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				if (check_col(pinfo->cinfo, COL_INFO)) {
 					col_append_fstr(pinfo->cinfo, COL_INFO,
 								"%s (Unreassembled fragment %u)",
-								szInfo->str, psn);
+								szInfo, psn);
 				}
 				if (tree) {
 					proto_tree_add_text(wtp_tree, tvb, dataOffset, -1,
@@ -761,14 +767,14 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		{
 			/* Nothing to hand to subdissector */
 			if (check_col(pinfo->cinfo, COL_INFO))
-				col_append_str(pinfo->cinfo, COL_INFO, szInfo->str);
+				col_append_str(pinfo->cinfo, COL_INFO, szInfo);
 		}
 	}
 	else
 	{
 		/* Nothing to hand to subdissector */
 		if (check_col(pinfo->cinfo, COL_INFO))
-			col_append_str(pinfo->cinfo, COL_INFO, szInfo->str);
+			col_append_str(pinfo->cinfo, COL_INFO, szInfo);
 	}
 }
 
