@@ -1,7 +1,7 @@
 /* filesystem.c
  * Filesystem utility routines
  *
- * $Id: filesystem.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: filesystem.c 17537 2006-03-08 22:13:21Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -41,19 +41,16 @@
 #include <sys/stat.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
+#ifdef _WIN32
 #include <windows.h>
-#endif
-
-#ifdef HAVE_DIRECT_H
-#include <direct.h>		/* to declare "mkdir()" on Windows */
-#endif
-
-#ifndef _WIN32
+#include <tchar.h>
+#include "epan/strutil.h"
+#else
 #include <pwd.h>
 #endif
 
 #include "filesystem.h"
+#include <wiretap/file_util.h>
 
 /*
  * Given a pathname, return a pointer to the last pathname separator
@@ -184,7 +181,7 @@ test_for_directory(const char *path)
 {
 	struct stat statb;
 
-	if (stat(path, &statb) < 0)
+	if (eth_stat(path, &statb) < 0)
 		return errno;
 
 	if (S_ISDIR(statb.st_mode))
@@ -198,7 +195,7 @@ test_for_fifo(const char *path)
 {
 	struct stat statb;
 
-	if (stat(path, &statb) < 0)
+	if (eth_stat(path, &statb) < 0)
 		return errno;
 
 	if (S_ISFIFO(statb.st_mode))
@@ -207,9 +204,221 @@ test_for_fifo(const char *path)
 		return 0;
 }
 
+static char *progfile_dir;
+
 /*
- * Get the directory in which Ethereal's global configuration and data
- * files are stored.
+ * Get the pathname of the directory from which the executable came,
+ * and save it for future use.
+ */
+void
+init_progfile_dir(const char *arg0
+#ifdef _WIN32
+	_U_
+#endif
+)
+{
+	char *dir_end;
+	char *path;
+#ifdef _WIN32
+	TCHAR prog_pathname_w[_MAX_PATH+2];
+	size_t progfile_dir_len;
+        char *prog_pathname;
+
+	/*
+	 * Attempt to get the full pathname of the currently running
+	 * program.
+	 */
+	if (GetModuleFileName(NULL, prog_pathname_w, sizeof prog_pathname_w) != 0) {
+                /*
+                 * XXX - Should we use g_utf16_to_utf8(), as in
+                 * getenv_utf8()?
+                 */
+                prog_pathname = utf_16to8(prog_pathname_w);
+		/*
+		 * We got it; strip off the last component, which would be
+		 * the file name of the executable, giving us the pathname
+		 * of the directory where the executable resies
+		 *
+		 * First, find the last "\\" in the directory, as that
+		 * marks the end of the directory pathname.
+		 *
+		 * XXX - Can the pathname be something such as
+		 * "C:ethereal.exe"?  Or is it always a full pathname
+		 * beginning with "\\" after the drive letter?
+		 */
+		dir_end = strrchr(prog_pathname, '\\');
+		if (dir_end != NULL) {
+			/*
+			 * Found it - now figure out how long the program
+			 * directory pathname will be.
+			 */
+			progfile_dir_len = (dir_end - prog_pathname);
+
+			/*
+			 * Allocate a buffer for the program directory
+			 * pathname, and construct it.
+			 */
+			path = g_malloc(progfile_dir_len + 1);
+			strncpy(path, prog_pathname, progfile_dir_len);
+			path[progfile_dir_len] = '\0';
+			progfile_dir = path;
+		}
+	}
+#else
+	char *prog_pathname;
+	char *curdir;
+	long path_max;
+	char *path_start, *path_end;
+	size_t path_component_len;
+
+	/*
+	 * Try to figure out the directory in which the currently running
+	 * program resides, given the argv[0] it was started with.  That
+	 * might be the absolute path of the program, or a path relative
+	 * to the current directory of the process that started it, or
+	 * just a name for the program if it was started from the command
+	 * line and was searched for in $PATH.  It's not guaranteed to be
+	 * any of those, however, so there are no guarantees....
+	 */
+	if (arg0[0] == '/') {
+		/*
+		 * It's an absolute path.
+		 */
+		prog_pathname = g_strdup(arg0);
+	} else if (strchr(arg0, '/') != NULL) {
+		/*
+		 * It's a relative path, with a directory in it.
+		 * Get the current directory, and combine it
+		 * with that directory.
+		 */
+		path_max = pathconf(".", _PC_PATH_MAX);
+		if (path_max == -1) {
+			/*
+			 * We have no idea how big a buffer to
+			 * allocate for the current directory.
+			 */
+			return;
+		}
+		curdir = g_malloc(path_max);
+		if (getcwd(curdir, sizeof curdir) == NULL) {
+			/*
+			 * It failed - give up, and just stick
+			 * with DATAFILE_DIR.
+			 */
+			g_free(curdir);
+			return;
+		}
+		path = g_malloc(strlen(curdir) + 1 + strlen(arg0) + 1);
+		strcpy(path, curdir);
+		strcat(path, "/");
+		strcat(path, arg0);
+		g_free(curdir);
+		prog_pathname = path;
+	} else {
+		/*
+		 * It's just a file name.
+		 * Search the path for a file with that name
+		 * that's executable.
+		 */
+		prog_pathname = NULL;	/* haven't found it yet */
+		path_start = getenv("PATH");
+		if (path_start != NULL) {
+			while (*path_start != '\0') {
+				path_end = strchr(path_start, ':');
+				if (path_end == NULL)
+					path_end = path_start + strlen(path_start);
+				path_component_len = path_end - path_start;
+				path = g_malloc(path_component_len + 1
+				    + strlen(arg0) + 1);
+				memcpy(path, path_start, path_component_len);
+				path[path_component_len] = '\0';
+				strcat(path, "/");
+				strcat(path, arg0);
+				if (access(path, X_OK) == 0) {
+					/*
+					 * Found it!
+					 */
+					prog_pathname = path;
+					break;
+				}
+
+				/*
+				 * That's not it.  If there are more
+				 * path components to test, try them.
+				 */
+				if (*path_end == '\0') {
+					/*
+					 * There's nothing more to try.
+					 */
+					break;
+				}
+				if (*path_end == ':')
+					path_end++;
+				path_start = path_end;
+				g_free(path);
+			}
+		}
+	}
+
+	if (prog_pathname != NULL) {
+		/*
+		 * OK, we have what we think is the pathname
+		 * of the program.
+		 *
+		 * First, find the last "/" in the directory,
+		 * as that marks the end of the directory pathname.
+		 */
+		dir_end = strrchr(prog_pathname, '/');
+		if (dir_end != NULL) {
+			/*
+			 * Found it.  Strip off the last component,
+			 * as that's the path of the program.
+			 */
+			*dir_end = '\0';
+
+			/*
+			 * Is there a "/.libs" at the end?
+			 */
+			dir_end = strrchr(prog_pathname, '/');
+			if (dir_end != NULL) {
+				if (strcmp(dir_end, "/.libs") == 0) {
+					/*
+					 * Yup, it's ".libs".
+					 * Strip that off; it's an
+					 * artifact of libtool.
+					 */
+					*dir_end = '\0';
+				}
+			}
+
+			/*
+			 * OK, we have the path we want.
+			 */
+			progfile_dir = prog_pathname;
+		} else {
+			/*
+			 * This "shouldn't happen"; we apparently
+			 * have no "/" in the pathname.
+			 * Just free up prog_pathname.
+			 */
+			g_free(prog_pathname);
+		}
+	}
+#endif
+}
+
+/*
+ * Get the directory in which the program resides.
+ */
+const char *
+get_progfile_dir(void)
+{
+	return progfile_dir;
+}
+
+/*
+ * Get the directory in which the global configuration and data files are
+ * stored.
  *
  * XXX - if we ever make libethereal a real library, used by multiple
  * applications (more than just Tethereal and versions of Ethereal with
@@ -236,68 +445,25 @@ const char *
 get_datafile_dir(void)
 {
 #ifdef _WIN32
-	char prog_pathname[_MAX_PATH+2];
-	char *dir_end;
-	size_t datafile_dir_len;
-	static char *datafile_dir;
-
 	/*
-	 * Have we already gotten the pathname?
-	 * If so, just return it.
-	 */
-	if (datafile_dir != NULL)
-		return datafile_dir;
-
-	/*
-	 * No, we haven't.
-	 * Start out by assuming it's the default installation directory.
-	 */
-	datafile_dir = "C:\\Program Files\\Ethereal\\";
-
-	/*
-	 * Now we attempt to get the full pathname of the currently running
-	 * program, under the assumption that we're running an installed
-	 * version of the program.  If we fail, we don't change "datafile_dir",
-	 * and thus end up using the default.
+	 * Do we have the pathname of the program?  If so, assume we're
+	 * running an installed version of the program.  If we fail,
+	 * we don't change "datafile_dir", and thus end up using the
+	 * default.
 	 *
 	 * XXX - does NSIS put the installation directory into
 	 * "\HKEY_LOCAL_MACHINE\SOFTWARE\Ethereal\InstallDir"?
 	 * If so, perhaps we should read that from the registry,
 	 * instead.
 	 */
-	if (GetModuleFileName(NULL, prog_pathname, sizeof prog_pathname) != 0) {
-		/*
-		 * If the program is an installed version, the full pathname
-		 * includes the pathname of the directory in which it was
-		 * installed; get that directory's pathname, and construct
-		 * from it the pathname of the directory in which the
-		 * plugins were installed.
-		 *
-		 * First, find the last "\\" in the directory, as that
-		 * marks the end of the directory pathname.
-		 *
-		 * XXX - Can the pathname be something such as
-		 * "C:ethereal.exe"?  Or is it always a full pathname
-		 * beginning with "\\" after the drive letter?
-		 */
-		dir_end = strrchr(prog_pathname, '\\');
-		if (dir_end != NULL) {
-			/*
-			 * Found it - now figure out how long the datafile
-			 * directory pathname will be.
-			 */
-			datafile_dir_len = (dir_end - prog_pathname);
+	if (progfile_dir != NULL)
+		return progfile_dir;
 
-			/*
-			 * Allocate a buffer for the plugin directory
-			 * pathname, and construct it.
-			 */
-			datafile_dir = g_malloc(datafile_dir_len + 1);
-			strncpy(datafile_dir, prog_pathname, datafile_dir_len);
-			datafile_dir[datafile_dir_len] = '\0';
-		}
-	}
-	return datafile_dir;
+	/*
+	 * No, we don't.
+	 * Fall back on the default installation directory.
+	 */
+	return "C:\\Program Files\\Ethereal\\";
 #else
 	/*
 	 * Just use DATAFILE_DIR, as that's what the configure script
@@ -310,8 +476,8 @@ get_datafile_dir(void)
 /*
  * Get the directory in which files that, at least on UNIX, are
  * system files (such as "/etc/ethers") are stored; on Windows,
- * there's no "/etc" directory, so we get them from the Ethereal
- * global configuration and data file directory.
+ * there's no "/etc" directory, so we get them from the global
+ * configuration and data file directory.
  */
 const char *
 get_systemfile_dir(void)
@@ -337,6 +503,51 @@ get_systemfile_dir(void)
 #define PF_DIR ".ethereal"
 #endif
 
+#ifdef WIN32
+/* utf8 version of getenv, needed to get win32 filename paths */
+char *getenv_utf8(const char *varname)
+{
+	char *envvar;
+	wchar_t *envvarw;
+	wchar_t *varnamew;
+
+	envvar = getenv(varname);
+
+	/* since GLib 2.6 we need an utf8 version of the filename */
+#if GLIB_MAJOR_VERSION > 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 6)
+	if (!G_WIN32_HAVE_WIDECHAR_API ()) {
+		/* Windows OT (9x, ME), convert from current code page to utf8 */
+		/* it's the best we can do here ... */
+        envvar = g_locale_to_utf8(envvar, -1, NULL, NULL, NULL);
+		/* XXX - memleak */
+		return envvar;
+	}
+
+	/* Windows NT, 2000, XP, ... */
+	/* using the wide char version of getenv should work under all circumstances */
+
+	/* convert given varname to utf16, needed by _wgetenv */
+	varnamew = g_utf8_to_utf16(varname, -1, NULL, NULL, NULL);
+	if (varnamew == NULL) {
+		return envvar;
+	}
+
+	/* use wide char version of getenv */
+	envvarw = _wgetenv(varnamew);
+	g_free(varnamew);
+	if (envvarw == NULL) {
+		return envvar;
+	}
+
+	/* convert value to utf8 */
+	envvar = g_utf16_to_utf8(envvarw, -1, NULL, NULL, NULL);
+	/* XXX - memleak */
+#endif
+
+	return envvar;
+}
+#endif
+
 /*
  * Get the directory in which personal configuration files reside;
  * in UNIX-compatible systems, it's ".ethereal", under the user's home
@@ -369,32 +580,28 @@ get_persconffile_dir(void)
 	 * Ethereal even if the home directory is an inaccessible
 	 * network drive.
 	 */
-	appdatadir = getenv("APPDATA");
+	appdatadir = getenv_utf8("APPDATA");
 	if (appdatadir != NULL) {
 		/*
 		 * Concatenate %APPDATA% with "\Ethereal".
 		 */
-		pf_dir = g_malloc(strlen(appdatadir) + strlen(PF_DIR) + 2);
-		sprintf(pf_dir, "%s" G_DIR_SEPARATOR_S "%s", appdatadir,
-		    PF_DIR);
+		pf_dir = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s",
+			appdatadir, PF_DIR);
 	} else {
 		/*
 		 * OK, %APPDATA% wasn't set, so use
 		 * %USERPROFILE%\Application Data.
 		 */
-		userprofiledir = getenv("USERPROFILE");
+		userprofiledir = getenv_utf8("USERPROFILE");
 		if (userprofiledir != NULL) {
-			pf_dir = g_malloc(strlen(userprofiledir) +
-			    strlen("Application Data") + strlen(PF_DIR) + 3);
-			sprintf(pf_dir,
+			pf_dir = g_strdup_printf(
 			    "%s" G_DIR_SEPARATOR_S "Application Data" G_DIR_SEPARATOR_S "%s",
 			    userprofiledir, PF_DIR);
 		} else {
 			/*
 			 * Give up and use "C:".
 			 */
-			pf_dir = g_malloc(strlen("C:") + strlen(PF_DIR) + 2);
-			sprintf(pf_dir, "C:" G_DIR_SEPARATOR_S "%s", PF_DIR);
+			pf_dir = g_strdup_printf("C:" G_DIR_SEPARATOR_S "%s", PF_DIR);
 		}
 	}
 #else
@@ -418,8 +625,7 @@ get_persconffile_dir(void)
 		} else
 			homedir = "/tmp";
 	}
-	pf_dir = g_malloc(strlen(homedir) + strlen(PF_DIR) + 2);
-	sprintf(pf_dir, "%s" G_DIR_SEPARATOR_S "%s", homedir, PF_DIR);
+	pf_dir = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", homedir, PF_DIR);
 #endif
 
 	return pf_dir;
@@ -444,7 +650,7 @@ create_persconffile_dir(char **pf_dir_path_return)
 	int ret;
 
 	pf_dir_path = get_persconffile_dir();
-	if (stat(pf_dir_path, &s_buf) != 0 && errno == ENOENT) {
+	if (eth_stat(pf_dir_path, &s_buf) != 0 && errno == ENOENT) {
 #ifdef _WIN32
 		/*
 		 * Does the parent directory of that directory
@@ -462,20 +668,20 @@ create_persconffile_dir(char **pf_dir_path_return)
 		pf_dir_parent_path_len = strlen(pf_dir_parent_path);
 		if (pf_dir_parent_path_len > 0
 		    && pf_dir_parent_path[pf_dir_parent_path_len - 1] != ':'
-		    && stat(pf_dir_parent_path, &s_buf) != 0) {
+		    && eth_stat(pf_dir_parent_path, &s_buf) != 0) {
 			/*
 			 * No, it doesn't exist - make it first.
 			 */
-			ret = mkdir(pf_dir_parent_path);
+			ret = eth_mkdir(pf_dir_parent_path, 0755);
 			if (ret == -1) {
 				*pf_dir_path_return = pf_dir_parent_path;
 				return -1;
 			}
 		}
 		g_free(pf_dir_path_copy);
-		ret = mkdir(pf_dir_path);
+		ret = eth_mkdir(pf_dir_path, 0755);
 #else
-		ret = mkdir(pf_dir_path, 0755);
+		ret = eth_mkdir(pf_dir_path, 0755);
 #endif
 	} else {
 		/*
@@ -512,9 +718,9 @@ get_home_dir(void)
 	 * Is there a chance that it might be set but one or more of
 	 * HOMEDRIVE or HOMEPATH isn't set?
 	 */
-	homedrive = getenv("HOMEDRIVE");
+	homedrive = getenv_utf8("HOMEDRIVE");
 	if (homedrive != NULL) {
-		homepath = getenv("HOMEPATH");
+		homepath = getenv_utf8("HOMEPATH");
 		if (homepath != NULL) {
 			/*
 			 * This is cached, so we don't need to worry about
@@ -573,24 +779,20 @@ get_persconffile_path(const char *filename, gboolean for_writing
 	char *old_path;
 #endif
 
-	path = (gchar *) g_malloc(strlen(get_persconffile_dir()) +
-	    strlen(filename) + 2);
-	sprintf(path, "%s" G_DIR_SEPARATOR_S "%s", get_persconffile_dir(),
+	path = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", get_persconffile_dir(),
 	    filename);
 #ifdef _WIN32
 	if (!for_writing) {
-		if (stat(path, &s_buf) != 0 && errno == ENOENT) {
+		if (eth_stat(path, &s_buf) != 0 && errno == ENOENT) {
 			/*
 			 * OK, it's not in the personal configuration file
 			 * directory; is it in the ".ethereal" subdirectory
 			 * of their home directory?
 			 */
-			old_path = (gchar *) g_malloc(strlen(get_home_dir()) +
-			    strlen(".ethereal") + strlen(filename) + 3);
-			sprintf(old_path,
+			old_path = g_strdup_printf(
 			    "%s" G_DIR_SEPARATOR_S ".ethereal" G_DIR_SEPARATOR_S "%s",
 			    get_home_dir(), filename);
-			if (stat(old_path, &s_buf) == 0) {
+			if (eth_stat(old_path, &s_buf) == 0) {
 				/*
 				 * OK, it exists; return it instead.
 				 */
@@ -611,21 +813,16 @@ get_persconffile_path(const char *filename, gboolean for_writing
 char *
 get_datafile_path(const char *filename)
 {
-	char *path;
 
-	path = (gchar *) g_malloc(strlen(get_datafile_dir()) +
-	    strlen(filename) + 2);
-	sprintf(path, "%s" G_DIR_SEPARATOR_S "%s", get_datafile_dir(),
+	return g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", get_datafile_dir(),
 	    filename);
-
-	return path;
 }
 
 /* Delete a file */
 gboolean
 deletefile(const char *path)
 {
-	return unlink(path) == 0;
+	return eth_unlink(path) == 0;
 }
 
 /*
@@ -634,13 +831,8 @@ deletefile(const char *path)
  */
 char *get_tempfile_path(const char *filename)
 {
-	char *path;
 
-	path = (gchar *) g_malloc(strlen(g_get_tmp_dir()) +
-	    strlen(filename) + 2);
-	sprintf(path, "%s" G_DIR_SEPARATOR_S "%s", g_get_tmp_dir(), filename);
-
-	return path;
+	return g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s", g_get_tmp_dir(), filename);
 }
 
 /*
@@ -684,7 +876,7 @@ file_open_error_message(int err, gboolean for_writing)
 #endif
 
 	default:
-		snprintf(errmsg_errno, sizeof(errmsg_errno),
+		g_snprintf(errmsg_errno, sizeof(errmsg_errno),
 				"The file \"%%s\" could not be %s: %s.",
 				for_writing ? "created" : "opened",
 				strerror(err));
@@ -717,7 +909,7 @@ file_write_error_message(int err)
 #endif
 
 	default:
-		snprintf(errmsg_errno, sizeof(errmsg_errno),
+		g_snprintf(errmsg_errno, sizeof(errmsg_errno),
 		    "An error occurred while writing to the file \"%%s\": %s.",
 		    strerror(err));
 		errmsg = errmsg_errno;
@@ -733,6 +925,7 @@ file_exists(const char *fname)
   struct stat   file_stat;
 
 
+#ifdef _WIN32
   /*
    * This is a bit tricky on win32. The st_ino field is documented as:
    * "The inode, and therefore st_ino, has no meaning in the FAT, ..."
@@ -740,29 +933,50 @@ file_exists(const char *fname)
    * so this is working, but maybe not quite the way expected. ULFL
    */
    file_stat.st_ino = 1;   /* this will make things work if an error occured */
-   stat(fname, &file_stat);
+   eth_stat(fname, &file_stat);
    if (file_stat.st_ino == 0) {
        return TRUE;
    } else {
        return FALSE;
    }
+#else
+   if (eth_stat(fname, &file_stat) != 0 && errno == ENOENT) {
+       return FALSE;
+   } else {
+       return TRUE;
+   }
+#endif
 
 }
 
-
+/*
+ * Check that the from file is not the same as to file
+ * We do it here so we catch all cases ...
+ * Unfortunately, the file requester gives us an absolute file
+ * name and the read file name may be relative (if supplied on
+ * the command line), so we can't just compare paths. From Joerg Mayer.
+ */
 gboolean
 files_identical(const char *fname1, const char *fname2)
 {
     /* Two different implementations, because:
-     * - _fullpath is not available on unix 
-     * - the stat inode will not work as expected on Win32, so two different implementations.
      *
-     * XXX - will _fullpath work with UNC?
+     * - _fullpath is not available on UN*X, so we can't get full
+     *   paths and compare them (which wouldn't work with hard links
+     *   in any case);
+     *
+     * - st_ino isn't filled in with a meaningful value on Windows.
      */
 #ifdef _WIN32
     char full1[MAX_PATH], full2[MAX_PATH];
 
-
+    /*
+     * Get the absolute full paths of the file and compare them.
+     * That won't work if you have hard links, but those aren't
+     * much used on Windows, even though NTFS supports them.
+     *
+     * XXX - will _fullpath work with UNC?
+     */
     if( _fullpath( full1, fname1, MAX_PATH ) == NULL ) {
         return FALSE;
     }
@@ -770,37 +984,24 @@ files_identical(const char *fname1, const char *fname2)
     if( _fullpath( full2, fname2, MAX_PATH ) == NULL ) {
         return FALSE;
     }
-    
+
     if(strcmp(full1, full2) == 0) {
         return TRUE;
     } else {
         return FALSE;
     }
 #else
-  struct stat   infile, outfile;
-
-  /*
-   * Check that the from file is not the same as to file
-   * We do it here so we catch all cases ...
-   * Unfortunately, the file requester gives us an absolute file
-   * name and the read file name may be relative (if supplied on
-   * the command line). From Joerg Mayer.
-   *
-   * This is a bit tricky on win32. The st_ino field is documented as:
-   * "The inode, and therefore st_ino, has no meaning in the FAT, ..."
-   * but it *is* set to zero if stat() returns without an error,
-   * so this is not working, as it only checks if both files existing. ULFL
-   */
-   infile.st_ino = 1;   /* These prevent us from getting equality         */
-   outfile.st_ino = 2;  /* If one or other of the files is not accessible */
-   stat(fname1, &infile);
-   stat(fname2, &outfile);
-   if (infile.st_ino == outfile.st_ino) {
-       return TRUE;
-   } else {
-       return FALSE;
-   }
+  struct stat   filestat1, filestat2;
 
+   /*
+    * Compare st_dev and st_ino.
+    */
+   if (eth_stat(fname1, &filestat1) == -1)
+       return FALSE;	/* can't get info about the first file */
+   if (eth_stat(fname2, &filestat2) == -1)
+       return FALSE;	/* can't get info about the second file */
+   return (filestat1.st_dev == filestat2.st_dev &&
+           filestat1.st_ino == filestat2.st_ino);
 #endif
 }
 
