@@ -1,7 +1,7 @@
 /* frame_data.h
  * Definitions for frame_data structures and routines
  *
- * $Id: frame_data.h,v 1.13 2004/01/31 04:10:05 guy Exp $
+ * $Id: frame_data.h 17093 2006-01-24 19:03:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,22 +27,15 @@
 
 #include "column_info.h"
 #include "tvbuff.h"
+#include <epan/nstime.h>
 
-#if 0
-/* Defined in color.h */
-#ifndef __COLOR_H__
-typedef struct _color_t /* {
-	guint32 pixel;
-	guint16 red;
-	guint16 green;
-	guint16 blue;
-} */ color_t;
-#endif
-#endif
 
 /* XXX - some of this stuff is used only while a packet is being dissected;
    should we keep that stuff in the "packet_info" structure, instead, to
    save memory? */
+/* The frame number is the ordinal number of the frame in the capture, so
+   it's 1-origin.  In various contexts, 0 as a frame number means "frame
+   number unknown". */
 typedef struct _frame_data {
   struct _frame_data *next; /* Next element in list */
   struct _frame_data *prev; /* Previous element in list */
@@ -51,12 +44,9 @@ typedef struct _frame_data {
   guint32      pkt_len;     /* Packet length */
   guint32      cap_len;     /* Amount actually captured */
   guint32      cum_bytes;   /* Cumulative bytes into the capture */
-  gint32       rel_secs;    /* Relative seconds (yes, it can be negative) */
-  gint32       rel_usecs;   /* Relative microseconds (yes, it can be negative) */
-  guint32      abs_secs;    /* Absolute seconds */
-  guint32      abs_usecs;   /* Absolute microseconds */
-  gint32       del_secs;    /* Delta seconds (yes, it can be negative) */
-  gint32       del_usecs;   /* Delta microseconds (yes, it can be negative) */
+  nstime_t     abs_ts;      /* Absolute timestamp */
+  nstime_t     rel_ts;      /* Relative timestamp (yes, it can be negative) */
+  nstime_t     del_ts;      /* Delta timestamp (yes, it can be negative) */
   long         file_off;    /* File offset */
   int          lnk_t;       /* Per-packet encapsulation/data-link type */
   struct {
@@ -80,14 +70,8 @@ typedef struct {
 
 /* Utility routines used by packet*.c */
 
-extern void p_add_proto_data(frame_data *, int, void *);
-extern void *p_get_proto_data(frame_data *, int);
-extern void p_rem_proto_data(frame_data *fd, int proto);
-
-/* An init routine to be called by epan_init */
-extern void frame_data_init(void);
-
-/* A cleanup routine to be called by epan_cleanup */
-extern void frame_data_cleanup(void);
+extern void p_add_proto_data(frame_data *fd, int proto, void *proto_data);
+extern void *p_get_proto_data(frame_data *fd, int proto);
+extern void p_remove_proto_data(frame_data *fd, int proto);
 
 #endif  /* __FRAME_DATA__ */
