@@ -1,5 +1,5 @@
 /*
- * $Id: ftype-bytes.c 15010 2005-07-23 06:21:17Z jmayer $
+ * $Id: ftype-bytes.c 17566 2006-03-10 11:58:22Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,7 +39,6 @@
 
 #define ETHER_LEN	6
 #define IPv6_LEN	16
-#define GUID_LEN	16
 
 static void
 bytes_fvalue_new(fvalue_t *fv)
@@ -76,6 +75,19 @@ bytes_repr_len(fvalue_t *fv, ftrepr_t rtype _U_)
 	return fv->value.bytes->len * 3 - 1;
 }
 
+static int
+oid_repr_len(fvalue_t *fv _U_, ftrepr_t rtype _U_)
+{
+	/* more exact computation will come later */
+	return fv->value.bytes->len * 3 + 16;
+}
+
+static void
+oid_to_repr(fvalue_t *fv, ftrepr_t rtype _U_, char *buf)
+{
+	oid_to_str_buf(fv->value.bytes->data, fv->value.bytes->len, buf, oid_repr_len(fv, rtype));
+}
+
 static void
 bytes_to_repr(fvalue_t *fv, ftrepr_t rtype _U_, char *buf)
 {
@@ -123,12 +135,17 @@ ipv6_fvalue_set(fvalue_t *fv, gpointer value, gboolean already_copied)
 }
 
 static void
-guid_fvalue_set(fvalue_t *fv, gpointer value, gboolean already_copied)
+oid_fvalue_set(fvalue_t *fv, gpointer value, gboolean already_copied)
 {
-	g_assert(!already_copied);
-	common_fvalue_set(fv, value, GUID_LEN);
+	g_assert(already_copied);
+
+	/* Free up the old value, if we have one */
+	bytes_fvalue_free(fv);
+
+	fv->value.bytes = value;
 }
 
+
 static gpointer
 value_get(fvalue_t *fv)
 {
@@ -226,29 +243,51 @@ ipv6_from_unparsed(fvalue_t *fv, char *s, gboolean allow_partial_value _U_, LogF
 	return TRUE;
 }
 
-static gboolean
-guid_from_unparsed(fvalue_t *fv, char *s, gboolean allow_partial_value, LogFunc logfunc)
+static int
+ipv6_repr_len(fvalue_t *fv _U_, ftrepr_t rtype _U_)
 {
+	/*
+	 * 39 characters for "XXXX:XXXX:XXXX:XXXX:XXXX:XXXX:XXXX:XXXX".
+	 */
+	return 39;
+}
+
+static void
+ipv6_to_repr(fvalue_t *fv, ftrepr_t rtype _U_, char *buf)
+{
+	ip6_to_str_buf((struct e_in6_addr *)fv->value.bytes->data, buf);
+}
+
+static gboolean
+oid_from_unparsed(fvalue_t *fv, char *s, gboolean allow_partial_value _U_, LogFunc logfunc)
+{
+	GByteArray	*bytes;
+	gboolean	res;
+
+
 	/*
 	 * Don't log a message if this fails; we'll try looking it
-	 * up as an GUID if it does, and if that fails,
+	 * up as an OID if it does, and if that fails,
 	 * we'll log a message.
 	 */
 	if (bytes_from_unparsed(fv, s, TRUE, NULL)) {
-		if (fv->value.bytes->len > GUID_LEN) {
-			logfunc("\"%s\" contains too many bytes to be a valid Globally Unique Identifier.",
-			    s);
-			return FALSE;
-		}
-		else if (fv->value.bytes->len < GUID_LEN && !allow_partial_value) {
-			logfunc("\"%s\" contains too few bytes to be a valid Globally Unique Identifier.",
-			    s);
-			return FALSE;
-		}
-
 		return TRUE;
 	}
-	return FALSE;
+
+	bytes = g_byte_array_new();
+	res = oid_str_to_bytes(s, bytes);
+	if (!res) {
+		if (logfunc != NULL)
+			logfunc("\"%s\" is not a valid OBJECT IDENTIFIER.", s);
+		g_byte_array_free(bytes, TRUE);
+		return FALSE;
+	}
+
+	/* Free up the old value, if we have one */
+	bytes_fvalue_free(fv);
+	fv->value.bytes = bytes;
+
+	return TRUE;
 }
 
 static guint
@@ -553,8 +592,8 @@ ftype_register_bytes(void)
 		bytes_fvalue_free,		/* free_value */
 		ipv6_from_unparsed,		/* val_from_unparsed */
 		NULL,				/* val_from_string */
-		NULL,				/* val_to_string_repr */
-		NULL,				/* len_string_repr */
+		ipv6_to_repr,			/* val_to_string_repr */
+		ipv6_repr_len,			/* len_string_repr */
 
 		ipv6_fvalue_set,		/* set_value */
 		NULL,				/* set_value_integer */
@@ -580,18 +619,18 @@ ftype_register_bytes(void)
 		slice,
 	};
 
-	static ftype_t guid_type = {
-		"GUID",			/* name */
-		"Globally Unique Identifier",			/* pretty_name */
-		GUID_LEN,			/* wire_size */
+	static ftype_t oid_type = {
+		"OID",			/* name */
+		"OBJECT IDENTIFIER",			/* pretty_name */
+		0,			/* wire_size */
 		bytes_fvalue_new,		/* new_value */
 		bytes_fvalue_free,		/* free_value */
-		guid_from_unparsed,		/* val_from_unparsed */
+		oid_from_unparsed,		/* val_from_unparsed */
 		NULL,				/* val_from_string */
-		bytes_to_repr,		/* val_to_string_repr */
-		bytes_repr_len,		/* len_string_repr */
+		oid_to_repr,			/* val_to_string_repr */
+		oid_repr_len,			/* len_string_repr */
 
-		guid_fvalue_set,		/* set_value */
+		oid_fvalue_set,		/* set_value */
 		NULL,				/* set_value_integer */
 		NULL,				/* set_value_integer64 */
 		NULL,				/* set_value_floating */
@@ -619,5 +658,5 @@ ftype_register_bytes(void)
 	ftype_register(FT_UINT_BYTES, &uint_bytes_type);
 	ftype_register(FT_ETHER, &ether_type);
 	ftype_register(FT_IPv6, &ipv6_type);
-	ftype_register(FT_GUID, &guid_type);
+	ftype_register(FT_OID, &oid_type);
 }
