@@ -1,5 +1,5 @@
 /*
- * $Id: ftype-ipv4.c,v 1.16 2004/02/27 12:00:32 obiot Exp $
+ * $Id: ftype-ipv4.c 16366 2005-10-30 22:55:30Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,7 +28,7 @@
 
 #include <ftypes-int.h>
 #include <epan/ipv4.h>
-#include <epan/resolv.h>
+#include <epan/addr_resolv.h>
 
 
 static void
@@ -126,9 +126,9 @@ static int
 val_repr_len(fvalue_t *fv _U_, ftrepr_t rtype _U_)
 {
 	/*
-	 * 14 characters for "XXX.XXX.XXX.XXX".
+	 * 15 characters for "XXX.XXX.XXX.XXX".
 	 */
-	return 14;
+	return 15;
 }
 
 static void
@@ -199,23 +199,25 @@ ftype_register_ipv4(void)
 {
 
 	static ftype_t ipv4_type = {
-		"FT_IPv4",
-		"IPv4 address",
-		4,
-		NULL,
-		NULL,
+		"FT_IPv4",			/* name */
+		"IPv4 address",			/* pretty_name */
+		4,				/* wire_size */
+		NULL,				/* new_value */
+		NULL,				/* free_value */
 		val_from_unparsed,		/* val_from_unparsed */
 		NULL,				/* val_from_string */
 		val_to_repr,			/* val_to_string_repr */
 		val_repr_len,			/* len_string_repr */
 
-		NULL,
-		set_integer,
-		NULL,
+		NULL,				/* set_value */
+		set_integer,			/* set_value_integer */
+		NULL,				/* set_value_integer64 */
+		NULL,				/* set_value_floating */
 
-		value_get,
-		NULL,
-		NULL,
+		value_get,			/* get_value */
+		NULL,				/* get_value_integer */
+		NULL,				/* get_value_integer64 */
+		NULL,				/* get_value_floating */
 
 		cmp_eq,
 		cmp_ne,
