@@ -1,5 +1,5 @@
 /*
- * $Id: ftype-time.c,v 1.21 2003/08/27 15:23:07 gram Exp $
+ * $Id: ftype-time.c 15407 2005-08-18 12:55:01Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -170,9 +170,15 @@ static gboolean
 relative_val_from_unparsed(fvalue_t *fv, char *s, gboolean allow_partial_value _U_, LogFunc logfunc)
 {
 	char    *curptr, *endptr;
-
+        gboolean negative = FALSE;
+        
 	curptr = s;
 
+        if(*curptr == '-') {
+            negative = TRUE;
+            curptr++;
+        }
+        
 	/*
 	 * If it doesn't begin with ".", it should contain a seconds
 	 * value.
@@ -212,9 +218,10 @@ relative_val_from_unparsed(fvalue_t *fv, char *s, gboolean allow_partial_value _
 		fv->value.time.nsecs = 0;
 	}
 
-	/*
-	 * XXX - what about negative values?
-	 */
+        if(negative) {
+            fv->value.time.secs = -fv->value.time.secs;
+            fv->value.time.nsecs = -fv->value.time.nsecs;
+        }
 	return TRUE;
 
 fail:
@@ -338,23 +345,25 @@ ftype_register_time(void)
 {
 
 	static ftype_t abstime_type = {
-		"FT_ABSOLUTE_TIME",
-		"date/time",
-		0,
-		time_fvalue_new,
-		NULL,
+		"FT_ABSOLUTE_TIME",		/* name */
+		"date/time",			/* pretty_name */
+		0,				/* wire_size */
+		time_fvalue_new,		/* new_value */
+		NULL,				/* free_value */
 		absolute_val_from_unparsed,	/* val_from_unparsed */
 		absolute_val_from_string,	/* val_from_string */
 		absolute_val_to_repr,		/* val_to_string_repr */
 		absolute_val_repr_len,		/* len_string_repr */
 
-		time_fvalue_set,
-		NULL,
-		NULL,
+		time_fvalue_set,		/* set_value */
+		NULL,				/* set_value_integer */
+		NULL,				/* set_value_integer64 */
+		NULL,				/* set_value_floating */
 
-		value_get,
-		NULL,
-		NULL,
+		value_get,			/* get_value */
+		NULL,				/* get_value_integer */
+		NULL,				/* get_value_integer64 */
+		NULL,				/* get_value_floating */
 
 		cmp_eq,
 		cmp_ne,
@@ -362,29 +371,33 @@ ftype_register_time(void)
 		cmp_ge,
 		cmp_lt,
 		cmp_le,
+		NULL,				/* cmp_bitwise_and */
 		NULL,				/* cmp_contains */
+		NULL,				/* cmp_matches */
 
 		NULL,
 		NULL
 	};
 	static ftype_t reltime_type = {
-		"FT_RELATIVE_TIME",
-		"time offset",
-		0,
-		time_fvalue_new,
-		NULL,
+		"FT_RELATIVE_TIME",		/* name */
+		"time offset",			/* pretty_name */
+		0,				/* wire_size */
+		time_fvalue_new,		/* new_value */
+		NULL,				/* free_value */
 		relative_val_from_unparsed,	/* val_from_unparsed */
 		NULL,				/* val_from_string */
 		relative_val_to_repr,		/* val_to_string_repr */
 		relative_val_repr_len,		/* len_string_repr */
 
-		time_fvalue_set,
-		NULL,
-		NULL,
+		time_fvalue_set,		/* set_value */
+		NULL,				/* set_value_integer */
+		NULL,				/* set_value_integer64 */
+		NULL,				/* set_value_floating */
 
-		value_get,
-		NULL,
-		NULL,
+		value_get,			/* get_value */
+		NULL,				/* get_value_integer */
+		NULL,				/* get_value_integer64 */
+		NULL,				/* get_value_floating */
 
 		cmp_eq,
 		cmp_ne,
@@ -392,7 +405,9 @@ ftype_register_time(void)
 		cmp_ge,
 		cmp_lt,
 		cmp_le,
+		NULL,				/* cmp_bitwise_and */
 		NULL,				/* cmp_contains */
+		NULL,				/* cmp_matches */
 
 		NULL,
 		NULL
