@@ -1,5 +1,5 @@
 /*
- * $Id: ftypes.c 15010 2005-07-23 06:21:17Z jmayer $
+ * $Id: ftypes.c 17566 2006-03-10 11:58:22Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,6 +28,8 @@
 #include <glib.h>
 #include "../slab.h"
 
+#include "ftypes.h"
+
 /* Keep track of ftype_t's via their ftenum number */
 static ftype_t* type_list[FT_NUM_TYPES];
 
@@ -42,6 +44,7 @@ ftypes_initialize(void)
 	ftype_register_double();
 	ftype_register_integers();
 	ftype_register_ipv4();
+	ftype_register_guid();
 	ftype_register_none();
 	ftype_register_string();
 	ftype_register_time();
