@@ -1,7 +1,7 @@
 /* ftypes.h
  * Definitions for field types
  *
- * $Id: ftypes.h 12715 2004-12-11 00:13:27Z lroland $
+ * $Id: ftypes.h 17566 2006-03-10 11:58:22Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -60,6 +60,8 @@ enum ftenum {
 	FT_IPXNET,
 	FT_FRAMENUM,	/* a UINT32, but if selected lets you go to frame with that numbe */
 	FT_PCRE,		/* a compiled Perl-Compatible Regular Expression object */
+	FT_GUID,		/* GUID, UUID */
+	FT_OID,			/* OBJECT IDENTIFIER */
 	FT_NUM_TYPES /* last item number plus one */
 };
 
@@ -136,6 +138,7 @@ ftype_can_matches(enum ftenum ftype);
 /* ---------------- FVALUE ----------------- */
 
 #include <epan/ipv4.h>
+#include <epan/guid-utils.h>
 
 #include <epan/tvbuff.h>
 #include <epan/nstime.h>
@@ -154,6 +157,7 @@ typedef struct _fvalue_t {
 		GByteArray	*bytes;
 		GString		*gstring;
 		ipv4_addr	ipv4;
+		e_guid_t	guid;
 		nstime_t	time;
 		tvbuff_t	*tvb;
 #ifdef HAVE_LIBPCRE
@@ -169,7 +173,7 @@ typedef struct _fvalue_t {
 
 typedef void (*FvalueNewFunc)(fvalue_t*);
 typedef void (*FvalueFreeFunc)(fvalue_t*);
-typedef void (*LogFunc)(char*,...);
+typedef void (*LogFunc)(const char*,...);
 
 typedef gboolean (*FvalueFromUnparsed)(fvalue_t*, char*, gboolean, LogFunc);
 typedef gboolean (*FvalueFromString)(fvalue_t*, char*, LogFunc);
