@@ -2,7 +2,7 @@
  * Routines for ISO/OSI network and transport protocol packet disassembly
  * Main entrance point and common functions
  *
- * $Id: osi-utils.c,v 1.14 2003/11/20 19:05:40 guy Exp $
+ * $Id: osi-utils.c 17514 2006-03-07 22:38:51Z gram $
  * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -34,65 +34,69 @@
 #include <glib.h>
 
 #include "osi-utils.h"
+#include "emem.h"
 
+/*
+ * XXX - shouldn't there be a centralized routine for dissecting NSAPs?
+ * See also "dissect_atm_nsap()" in epan/dissectors/packet-arp.c and
+ * "dissect_nsap()" in epan/dissectors/packet-isup.c.
+ */
 gchar *
 print_nsap_net( const guint8 *ad, int length )
 {
-  static gchar  str[MAX_NSAP_LEN * 3 + 50]; /* reserve space for nice layout */
   gchar *cur;
 
-  cur = str;
-  print_nsap_net_buf( ad, length, cur );
+  cur = ep_alloc(MAX_NSAP_LEN * 3 + 50);
+  print_nsap_net_buf( ad, length, cur, MAX_NSAP_LEN * 3 + 50);
   return( cur );
 }
 
 void
-print_nsap_net_buf( const guint8 *ad, int length, gchar *buf )
+print_nsap_net_buf( const guint8 *ad, int length, gchar *buf, int buf_len)
 {
   gchar *cur;
 
   /* to do : NSAP / NET decoding */
 
   if ( (length <= 0 ) || ( length > MAX_NSAP_LEN ) ) {
-    sprintf( buf, "<Invalid length of NSAP>");
+    g_snprintf(buf, buf_len, "<Invalid length of NSAP>");
     return;
   }
   cur = buf;
   if ( ( length == RFC1237_NSAP_LEN ) || ( length == RFC1237_NSAP_LEN + 1 ) ) {
-    print_area_buf( ad, RFC1237_FULLAREA_LEN, cur );
+    print_area_buf(ad, RFC1237_FULLAREA_LEN, cur, buf_len-(cur-buf));
     cur += strlen( cur );
-    print_system_id_buf( ad + RFC1237_FULLAREA_LEN, RFC1237_SYSTEMID_LEN, cur );
+    print_system_id_buf( ad + RFC1237_FULLAREA_LEN, RFC1237_SYSTEMID_LEN, cur, buf_len-(cur-buf));
     cur += strlen( cur );
-    cur += sprintf( cur, "[%02x]",
+    cur += g_snprintf(cur, buf_len-(cur-buf), "[%02x]",
                     ad[ RFC1237_FULLAREA_LEN + RFC1237_SYSTEMID_LEN ] );
     if ( length == RFC1237_NSAP_LEN + 1 ) {
-      cur += sprintf( cur, "-%02x", ad[ length -1 ] );
+      cur += g_snprintf(cur, buf_len-(cur-buf), "-%02x", ad[ length -1 ] );
     }
   }
   else {    /* probably format as standard */
-    print_area_buf( ad, length, buf );
+    print_area_buf( ad, length, buf, buf_len);
   }
 } /* print_nsap */
 
 gchar *
 print_system_id( const guint8 *ad, int length )
 {
-  static gchar  str[MAX_SYSTEMID_LEN * 3 + 5]; /* Don't trust exact matching */
   gchar        *cur;
 
-  cur = str;
-  print_system_id_buf(ad, length, cur );
+  cur = ep_alloc(MAX_SYSTEMID_LEN * 3 + 5);
+  print_system_id_buf(ad, length, cur, MAX_SYSTEMID_LEN * 3 + 5);
   return( cur );
 }
 
 void
-print_system_id_buf( const guint8 *ad, int length, gchar *buf )
+print_system_id_buf( const guint8 *ad, int length, gchar *buf, int buf_len)
 {
   gchar        *cur;
   int           tmp;
 
   if ( ( length <= 0 ) || ( length > MAX_SYSTEMID_LEN ) ) {
-    sprintf( buf, "<Invalid length of SYSTEM ID>");
+    g_snprintf(buf, buf_len, "<Invalid length of SYSTEM ID>");
     return;
   }
 
@@ -100,30 +104,31 @@ print_system_id_buf( const guint8 *ad, int length, gchar *buf )
   if ( ( 6 == length ) || /* System-ID */
        ( 7 == length ) || /* LAN-ID */
        ( 8 == length )) { /* LSP-ID */
-    cur += sprintf(cur, "%02x%02x.%02x%02x.%02x%02x", ad[0], ad[1],
+    cur += g_snprintf(cur, buf_len-(cur-buf), "%02x%02x.%02x%02x.%02x%02x", ad[0], ad[1],
                     ad[2], ad[3], ad[4], ad[5] );
     if ( ( 7 == length ) ||
          ( 8 == length )) {
-        cur += sprintf( cur, ".%02x", ad[6] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), ".%02x", ad[6] );
     }
     if ( 8 == length ) {
-        cur += sprintf( cur, "-%02x", ad[7] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "-%02x", ad[7] );
     }
   }
   else {
     tmp = 0;
     while ( tmp < length / 4 ) { /* 16 / 4 == 4 > four Octets left to print */
-      cur += sprintf( cur, "%02x", ad[tmp++] );
-      cur += sprintf( cur, "%02x", ad[tmp++] );
-      cur += sprintf( cur, "%02x", ad[tmp++] );
-      cur += sprintf( cur, "%02x.", ad[tmp++] );
+      cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+      cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+      cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+      cur += g_snprintf(cur, buf_len-(cur-buf), "%02x.", ad[tmp++] );
     }
     if ( 1 == tmp ) {   /* Special case for Designated IS */
-      sprintf( --cur, ".%02x", ad[tmp] );
+      cur--;
+      g_snprintf(cur, buf_len-(cur-buf), ".%02x", ad[tmp] );
     }
     else {
       for ( ; tmp < length; ) {  /* print the rest without dot */
-        cur += sprintf( cur, "%02x", ad[tmp++] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
       }
     }
   }
@@ -132,16 +137,15 @@ print_system_id_buf( const guint8 *ad, int length, gchar *buf )
 gchar *
 print_area(const guint8 *ad, int length)
 {
-  static gchar  str[MAX_AREA_LEN * 3 + 20]; /* reserve space for nice layout */
   gchar *cur;
 
-  cur = str;
-  print_area_buf( ad, length, cur );
+  cur = ep_alloc(MAX_AREA_LEN * 3 + 20);
+  print_area_buf(ad, length, cur, MAX_AREA_LEN * 3 + 20);
   return cur;
 }
 
 void
-print_area_buf(const guint8 *ad, int length, gchar *buf)
+print_area_buf(const guint8 *ad, int length, gchar *buf, int buf_len)
 {
   gchar *cur;
   int  tmp  = 0;
@@ -150,7 +154,7 @@ print_area_buf(const guint8 *ad, int length, gchar *buf)
    * and take away all these stupid resource consuming local statics
    */
   if (length <= 0 || length > MAX_AREA_LEN) {
-    sprintf( buf, "<Invalid length of AREA>");
+    g_snprintf(buf, buf_len, "<Invalid length of AREA>");
     return;
   }
 
@@ -164,37 +168,39 @@ print_area_buf(const guint8 *ad, int length, gchar *buf)
        )
      ) {    /* AFI is good and length is long enough  */
 
-    if ( length > RFC1237_FULLAREA_LEN + 1 ) {  /* Special Case Designated IS */
-      sprintf( buf, "<Invalid length of AREA for DCC / GOSIP AFI>");
-      return;
-    }
+    /* there used to be a check for (length > RFC1237_FULLAREA_LEN + 1) here,
+     * in order to report an invalied length of AREA for DCC / GOSIP AFI,
+     * but that can *never* be the case because the if() test above explicitly
+     * tests for (length == RFC1237_FULLAREA_LEN) or (length == RFC1237_FULLAREA_LEN + 1)
+     */
 
-    cur += sprintf( cur, "[%02x|%02x:%02x][%02x|%02x:%02x:%02x|%02x:%02x]",
+    cur += g_snprintf(cur, buf_len-(cur-buf), "[%02x|%02x:%02x][%02x|%02x:%02x:%02x|%02x:%02x]",
                     ad[0], ad[1], ad[2], ad[3], ad[4],
                     ad[5], ad[6], ad[7], ad[8] );
-    cur += sprintf( cur, "[%02x:%02x|%02x:%02x]",
+    cur += g_snprintf(cur, buf_len-(cur-buf), "[%02x:%02x|%02x:%02x]",
                     ad[9], ad[10],  ad[11], ad[12] );
     if ( RFC1237_FULLAREA_LEN + 1 == length )
-      sprintf( cur, "-[%02x]", ad[20] );
+      g_snprintf(cur, buf_len-(cur-buf), "-[%02x]", ad[20] );
   }
   else { /* print standard format */
     if ( length == RFC1237_AREA_LEN ) {
-      sprintf( buf, "%02x.%02x%02x", ad[0], ad[1], ad[2] );
+      g_snprintf(buf, buf_len, "%02x.%02x%02x", ad[0], ad[1], ad[2] );
       return;
     }
     if ( 4 < length ) {
       while ( tmp < length / 4 ) {      /* 16/4==4 > four Octets left to print */
-        cur += sprintf( cur, "%02x", ad[tmp++] );
-        cur += sprintf( cur, "%02x", ad[tmp++] );
-        cur += sprintf( cur, "%02x", ad[tmp++] );
-        cur += sprintf( cur, "%02x.", ad[tmp++] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
+        cur += g_snprintf(cur, buf_len-(cur-buf), "%02x.", ad[tmp++] );
       }
       if ( 1 == tmp ) {                     /* Special case for Designated IS */
-        sprintf( --cur, "-%02x", ad[tmp] );
+        cur--;
+        g_snprintf(cur, buf_len-(cur-buf), "-%02x", ad[tmp] );
       }
       else {
         for ( ; tmp < length; ) {  /* print the rest without dot */
-          cur += sprintf( cur, "%02x", ad[tmp++] );
+          cur += g_snprintf(cur, buf_len-(cur-buf), "%02x", ad[tmp++] );
         }
       }
     }
