@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c 13658 2005-03-08 09:43:36Z sahlberg $
+ * $Id: packet.c 17653 2006-03-16 23:18:15Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,7 +43,6 @@
 #include "timestamp.h"
 
 #include "atalk-utils.h"
-#include "ipv6-utils.h"
 #include "sna-utils.h"
 #include "osi-utils.h"
 #include "to_str.h"
@@ -52,8 +51,10 @@
 #include "tvbuff.h"
 #include "plugins.h"
 #include "epan_dissect.h"
+#include "emem.h"
 
 #include <epan/reassemble.h>
+#include <epan/stream.h>
 
 static gint proto_malformed = -1;
 static dissector_handle_t frame_handle = NULL;
@@ -118,9 +119,20 @@ call_init_routine(gpointer routine, gpointer dummy _U_)
 	(*func)();
 }
 
+/*
+ * XXX - for now, these are the same; the "init" routines free whatever
+ * stuff is left over from any previous dissection, and then initialize
+ * their tables.
+ *
+ * We should probably split that into "init" and "cleanup" routines, for
+ * cleanliness' sake.
+ */
 void
 init_dissection(void)
 {
+	/* Reclaim and reinitialize all memory of seasonal scope */
+	se_free_all();
+
 	/* Initialize the table of conversations. */
 	epan_conversation_init();
 
@@ -135,6 +147,15 @@ init_dissection(void)
 	   may free up space for fragments, which they find by using the
 	   data structures that "reassemble_init()" frees. */
 	reassemble_init();
+
+	/* Initialise the stream-handling tables */
+	stream_init();
+}
+
+void
+cleanup_dissection(void)
+{
+	init_dissection();
 }
 
 /* Allow protocols to register a "cleanup" routine to be
@@ -165,31 +186,23 @@ postseq_cleanup_all_protocols(void)
 			&call_postseq_cleanup_routine, NULL);
 }
 
-/* Contains information about data sources. */
-static GMemChunk *data_source_chunk = NULL;
-
 /*
  * Add a new data source to the list of data sources for a frame, given
  * the tvbuff for the data source and its name.
  */
 void
-add_new_data_source(packet_info *pinfo, tvbuff_t *tvb, char *name)
+add_new_data_source(packet_info *pinfo, tvbuff_t *tvb, const char *name)
 {
 	data_source *src;
 
-	if (data_source_chunk == NULL) {
-		data_source_chunk = g_mem_chunk_new("data_source_chunk",
-		    sizeof (data_source), 10 * sizeof (data_source),
-		    G_ALLOC_AND_FREE);
-	}
-	src = g_mem_chunk_alloc(data_source_chunk);
+	src = ep_alloc(sizeof (data_source));
 	src->tvb = tvb;
 	/*
 	 * XXX - if we require this argument to be a string constant,
 	 * we don't need to allocate a buffer for a copy and make a
 	 * copy, and wouldn't need to free the buffer, either.
 	 */
-	src->name = g_strdup_printf("%s (%u bytes)", name, tvb_length(tvb));
+	src->name = ep_strdup_printf("%s (%u bytes)", name, tvb_length(tvb));
 	pinfo->data_src = g_slist_append(pinfo->data_src, src);
 }
 
@@ -199,25 +212,17 @@ add_new_data_source(packet_info *pinfo, tvbuff_t *tvb, char *name)
 void
 free_data_sources(packet_info *pinfo)
 {
-	GSList *src_le;
-	data_source *src;
-
-	for (src_le = pinfo->data_src; src_le != NULL; src_le = src_le->next) {
-	    	src = src_le->data;
-	    	g_free(src->name);
-	    	g_mem_chunk_free(data_source_chunk, src);
-	}
 	g_slist_free(pinfo->data_src);
 	pinfo->data_src = NULL;
 }
 
 /* Allow dissectors to register a "final_registration" routine
- * that is run like the proto_register_XXX() routine, but the end
+ * that is run like the proto_register_XXX() routine, but at the
  * end of the epan_init() function; that is, *after* all other
  * subsystems, like dfilters, have finished initializing. This is
  * useful for dissector registration routines which need to compile
  * display filters. dfilters can't initialize itself until all protocols
- * have registereed themselves. */
+ * have registered themselves. */
 static GSList *final_registration_routines;
 
 void
@@ -291,19 +296,21 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	edt->pi.want_pdu_tracking = 0;
 	edt->pi.p2p_dir = P2P_DIR_UNKNOWN;
 	edt->pi.private_data = NULL;
-        edt->pi.oxid = 0;
-        edt->pi.rxid = 0;
-        edt->pi.r_ctl = 0;
-        edt->pi.src_idx = 0;
-        edt->pi.dst_idx = 0;
-        edt->pi.vsan = 0;
-        edt->pi.dcectxid = 0;
-        edt->pi.dcetransporttype = -1;
+	edt->pi.oxid = 0;
+	edt->pi.rxid = 0;
+	edt->pi.r_ctl = 0;
+	edt->pi.src_idx = 0;
+	edt->pi.dst_idx = 0;
+	edt->pi.vsan = 0;
+	edt->pi.dcectxid = 0;
+	edt->pi.dcetransporttype = -1;
 	edt->pi.decrypt_gssapi_tvb = 0;
 	edt->pi.gssapi_wrap_tvb = NULL;
 	edt->pi.gssapi_encrypted_tvb = NULL;
 	edt->pi.gssapi_decrypted_tvb = NULL;
-        edt->pi.layer_names = NULL;
+	edt->pi.layer_names = NULL;
+	edt->pi.link_number = 0;
+	edt->pi.annex_a_used = MTP2_ANNEX_A_USED_UNKNOWN;
 
 	TRY {
 		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
@@ -322,13 +329,12 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 		g_assert_not_reached();
 	}
 	CATCH(ReportedBoundsError) {
-	  if(proto_malformed != -1){
-		proto_tree_add_protocol_format(edt->tree, proto_malformed, edt->tvb, 0, 0,
-				"[Malformed Frame: Packet Length]" );
-	  }
-	  else {
-	    g_assert_not_reached();
-	  }
+		if(proto_malformed != -1){
+			proto_tree_add_protocol_format(edt->tree, proto_malformed, edt->tvb, 0, 0,
+			                               "[Malformed Frame: Packet Length]" );
+		} else {
+			g_assert_not_reached();
+		}
 	}
 	ENDTRY;
 
@@ -350,6 +356,16 @@ struct dissector_handle {
 	protocol_t	*protocol;
 };
 
+/* This function will return
+ * old style dissector :
+ *   length of the payload or 1 of the payload is empty
+ * new dissector :
+ *   >0  this protocol was successfully dissected and this was this protocol.
+ *   0   this packet did not match this protocol.
+ *
+ * The only time this function will return 0 is if it is a new style dissector
+ * and if the dissector rejected the packet.
+ */
 static int
 call_dissector_through_handle(dissector_handle_t handle, tvbuff_t *tvb,
     packet_info *pinfo, proto_tree *tree)
@@ -364,9 +380,9 @@ call_dissector_through_handle(dissector_handle_t handle, tvbuff_t *tvb,
 		    proto_get_protocol_short_name(handle->protocol);
 	}
 
-	if (handle->is_new)
+	if (handle->is_new) {
 		ret = (*handle->dissector.new)(tvb, pinfo, tree);
-	else {
+	} else {
 		(*handle->dissector.old)(tvb, pinfo, tree);
 		ret = tvb_length(tvb);
 		if (ret == 0) {
@@ -399,7 +415,7 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 	packet_info *volatile pinfo = pinfo_arg;
 	const char *saved_proto;
 	guint16 saved_can_desegment;
-	volatile int ret;
+	volatile int ret = 0;
 	gboolean save_writable;
 	volatile address save_dl_src;
 	volatile address save_dl_dst;
@@ -447,7 +463,7 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 		if (pinfo->layer_names) {
 			if (pinfo->layer_names->len > 0)
 				g_string_append(pinfo->layer_names, ":");
-			g_string_append(pinfo->layer_names, 
+			g_string_append(pinfo->layer_names,
 			    proto_get_protocol_filter_name(proto_get_id(handle->protocol)));
 		}
 	}
@@ -591,7 +607,7 @@ struct dtbl_entry {
 struct dissector_table {
 	GHashTable	*hash_table;
 	GSList		*dissector_handles;
-	char		*ui_name;
+	const char	*ui_name;
 	ftenum_t	type;
 	int		base;
 };
@@ -881,7 +897,7 @@ find_string_dtbl_entry(dissector_table_t sub_dissectors, const gchar *pattern)
 
 /* Add an entry to a string dissector table. */
 void
-dissector_add_string(const char *name, gchar *pattern,
+dissector_add_string(const char *name, const gchar *pattern,
     dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
@@ -912,7 +928,7 @@ dissector_add_string(const char *name, gchar *pattern,
 	dtbl_entry->initial = dtbl_entry->current;
 
 /* do the table insertion */
-    	g_hash_table_insert( sub_dissectors->hash_table, pattern,
+    	g_hash_table_insert( sub_dissectors->hash_table, (gpointer)pattern,
     	    (gpointer)dtbl_entry);
 
 	/*
@@ -1200,7 +1216,7 @@ dissector_all_tables_foreach_func (gpointer key, gpointer value, gpointer user_d
  * Walk all dissector tables calling a user supplied function on each
  * entry.
  */
-void
+static void
 dissector_all_tables_foreach (DATFunc func,
 			      gpointer user_data)
 {
@@ -1236,7 +1252,7 @@ dissector_table_foreach (char *name,
  * function on each entry.
  */
 void
-dissector_table_foreach_handle(char *name,
+dissector_table_foreach_handle(const char *name,
 			       DATFunc_handle func,
 			       gpointer user_data)
 {
@@ -1344,7 +1360,7 @@ dissector_all_tables_foreach_table (DATFunc_table func,
 }
 
 dissector_table_t
-register_dissector_table(const char *name, char *ui_name, ftenum_t type,
+register_dissector_table(const char *name, const char *ui_name, ftenum_t type,
     int base)
 {
 	dissector_table_t	sub_dissectors;
@@ -1392,7 +1408,7 @@ register_dissector_table(const char *name, char *ui_name, ftenum_t type,
 	return sub_dissectors;
 }
 
-char *
+const char *
 get_dissector_table_ui_name(const char *name)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
@@ -1494,17 +1510,17 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 		if (dtbl_entry->protocol != NULL) {
 			pinfo->current_proto =
 			    proto_get_protocol_short_name(dtbl_entry->protocol);
-		}
 
-		/*
-		 * Add the protocol name to the layers; we'll remove it
-		 * if the dissector fails.
-		 */
-		if (pinfo->layer_names) {
-			if (pinfo->layer_names->len > 0)
-				g_string_append(pinfo->layer_names, ":");
-			g_string_append(pinfo->layer_names,
-			    proto_get_protocol_filter_name(proto_get_id(dtbl_entry->protocol)));
+			/*
+			 * Add the protocol name to the layers; we'll remove it
+			 * if the dissector fails.
+			 */
+			if (pinfo->layer_names) {
+				if (pinfo->layer_names->len > 0)
+					g_string_append(pinfo->layer_names, ":");
+				g_string_append(pinfo->layer_names,
+					proto_get_protocol_filter_name(proto_get_id(dtbl_entry->protocol)));
+			}
 		}
 
 		if ((*dtbl_entry->dissector)(tvb, pinfo, tree)) {
@@ -1558,7 +1574,7 @@ static GHashTable *registered_dissectors = NULL;
 
 /* Get the short name of the protocol for a dissector handle, if it has
    a protocol. */
-char *
+const char *
 dissector_handle_get_short_name(dissector_handle_t handle)
 {
 	if (handle->protocol == NULL) {
@@ -1676,7 +1692,9 @@ new_register_dissector(const char *name, new_dissector_t dissector, int proto)
 	    (gpointer) handle);
 }
 
-/* Call a dissector through a handle. */
+/* Call a dissector through a handle and if this fails call the "data"
+ * dissector.
+ */
 int
 call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
     packet_info *pinfo, proto_tree *tree)
@@ -1696,3 +1714,91 @@ call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
 	}
 	return ret;
 }
+
+/* Call a dissector through a handle but if the dissector rejected it
+ * return 0 instead of using the default "data" dissector.
+ */
+int
+call_dissector_only(dissector_handle_t handle, tvbuff_t *tvb,
+    packet_info *pinfo, proto_tree *tree)
+{
+	int ret;
+
+	ret = call_dissector_work(handle, tvb, pinfo, tree);
+	return ret;
+}
+
+/*
+ * Dumps the "layer type"/"decode as" associations to stdout, similar
+ * to the proto_registrar_dump_*() routines.
+ *
+ * There is one record per line. The fields are tab-delimited.
+ *
+ * Field 1 = layer type, e.g. "tcp.port"
+ * Field 2 = selector in decimal
+ * Field 3 = "decode as" name, e.g. "http"
+ */
+
+
+static void
+dissector_dump_decodes_display(gchar *table_name, ftenum_t selector_type _U_,
+    gpointer key, gpointer value, gpointer user_data _U_)
+{
+	guint32 selector = (guint32) key;
+	dissector_table_t sub_dissectors = find_dissector_table(table_name);
+	dtbl_entry_t *dtbl_entry;
+	dissector_handle_t handle;
+	gint proto_id;
+	const gchar *decode_as;
+
+	g_assert(sub_dissectors);
+	switch (sub_dissectors->type) {
+
+		case FT_UINT8:
+		case FT_UINT16:
+		case FT_UINT24:
+		case FT_UINT32:
+			dtbl_entry = value;
+			g_assert(dtbl_entry);
+
+			handle = dtbl_entry->current;
+			g_assert(handle);
+
+			proto_id = dissector_handle_get_protocol_index(handle);
+
+			if (proto_id != -1) {
+				decode_as = proto_get_protocol_filter_name(proto_id);
+				g_assert(decode_as != NULL);
+				printf("%s\t%u\t%s\n", table_name, selector, decode_as);
+			}
+			break;
+
+	default:
+		break;
+	}
+}
+
+void
+dissector_dump_decodes() {
+	dissector_all_tables_foreach(dissector_dump_decodes_display, NULL);
+}
+
+static GPtrArray* post_dissectors = NULL;
+static guint num_of_postdissectors = 0;
+
+void register_postdissector(dissector_handle_t handle) {
+    if (!post_dissectors)
+        post_dissectors = g_ptr_array_new();
+
+    g_ptr_array_add(post_dissectors, handle);
+    num_of_postdissectors++;
+}
+
+extern void call_all_postdissectors(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
+    guint i;
+    for(i=0;i<num_of_postdissectors;i++) {
+        call_dissector((dissector_handle_t) g_ptr_array_index(post_dissectors,i),
+                       tvb,pinfo,tree);
+    }
+}
+
