@@ -1,7 +1,7 @@
 /* packet.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet.h 15043 2005-07-24 19:01:28Z guy $
+ * $Id: packet.h 17089 2006-01-24 00:26:57Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -140,7 +140,7 @@ typedef gboolean (*heur_dissector_t)(tvbuff_t *tvb, packet_info *pinfo,
 
 typedef void (*DATFunc) (gchar *table_name, ftenum_t selector_type,
     gpointer key, gpointer value, gpointer user_data);
-typedef void (*DATFunc_handle) (gchar *table_name, gpointer value,
+typedef void (*DATFunc_handle) (const gchar *table_name, gpointer value,
     gpointer user_data);
 typedef void (*DATFunc_table) (gchar *table_name, const gchar *ui_name,
     gpointer user_data);
@@ -156,7 +156,7 @@ extern void dissector_table_foreach (char *name, DATFunc func,
     gpointer user_data);
 extern void dissector_all_tables_foreach_changed (DATFunc func,
     gpointer user_data);
-extern void dissector_table_foreach_handle(char *name, DATFunc_handle func,
+extern void dissector_table_foreach_handle(const char *name, DATFunc_handle func,
     gpointer user_data);
 extern void dissector_all_tables_foreach_table (DATFunc_table func,
     gpointer user_data);
@@ -406,4 +406,11 @@ extern void ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_ethertype,
  */
 extern void dissector_dump_decodes(void);
 
+/*
+ * post dissectors are to be called by packet-frame.c after every other
+ * dissector has been called.
+ */
+extern void register_postdissector(dissector_handle_t);
+extern void call_all_postdissectors(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+
 #endif /* packet.h */
