@@ -1,7 +1,7 @@
 /* packet_info.h
  * Definitions for packet info structures and routines
  *
- * $Id: packet_info.h 13689 2005-03-10 08:52:59Z sahlberg $
+ * $Id: packet_info.h 15015 2005-07-23 11:41:25Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,6 +33,10 @@
 #define P2P_DIR_SENT	0
 #define P2P_DIR_RECV	1
 
+#define MTP2_ANNEX_A_USED_UNKNOWN -1
+#define MTP2_ANNEX_A_NOT_USED      0
+#define MTP2_ANNEX_A_USED          1
+
 #define PINFO_SOF_FIRST_FRAME   0x1
 #define PINFO_SOF_SOFF          0x2
 #define PINFO_EOF_LAST_FRAME    0x80
@@ -56,7 +60,7 @@ typedef struct _packet_info {
   guint32 ipxptype;		/* IPX packet type, if this is an IPX packet */
   circuit_type ctype;		/* type of circuit, for protocols with a VC identifier */
   guint32 circuit_id;		/* circuit ID, for protocols with a VC identifier */
-  char   *noreassembly_reason;  /* reason why reassembly wasn't done, if any */
+  const char *noreassembly_reason;  /* reason why reassembly wasn't done, if any */
   gboolean fragmented;		/* TRUE if the protocol is only a fragment */
   gboolean in_error_pkt;	/* TRUE if we're inside an {ICMP,CLNP,...} error packet */
   port_type ptype;		/* type of the following two port numbers */
@@ -110,7 +114,10 @@ typedef struct _packet_info {
 
   int     iplen;
   int     iphdrlen;
-  int	  p2p_dir;
+  int	  p2p_dir;              /* Packet was captured as an 
+                                       outbound (P2P_DIR_SENT) 
+                                       inbound (P2P_DIR_RECV) 
+                                       unknown (P2P_DIR_UNKNOWN) */
   guint16 oxid;                 /* next 2 fields reqd to identify fibre */
   guint16 rxid;                 /* channel conversations */
   guint8  r_ctl;                /* R_CTL field in Fibre Channel Protocol */
@@ -132,10 +139,14 @@ typedef struct _packet_info {
   guint16 dcetransportsalt;	/* fid: if transporttype==DCE_CN_TRANSPORT_SMBPIPE */
 
   /* Extra data for handling of decryption of GSSAPI wrapped tvbuffs.
-     Caller sets gssapi_decrypt_tvb if this service is requested.
+     Caller sets decrypt_gssapi_tvb if this service is requested.
      If gssapi_encrypted_tvb is NULL, then the rest of the tvb data following
-     the gssapi blob it self is decrypted othervise the gssapi_encrypted_tvb
+     the gssapi blob itself is decrypted othervise the gssapi_encrypted_tvb
      tvb will be decrypted (DCERPC has the data before the gssapi blob)
+     If, on return, gssapi_data_encrypted is FALSE, the wrapped tvbuff
+     was signed (i.e., an encrypted signature was present, to check
+     whether the data was modified by a man in the middle) but not sealed
+     (i.e., the data itself wasn't encrypted).
   */
 #define DECRYPT_GSSAPI_NORMAL	1
 #define DECRYPT_GSSAPI_DCE	2
@@ -143,12 +154,15 @@ typedef struct _packet_info {
   tvbuff_t *gssapi_wrap_tvb;
   tvbuff_t *gssapi_encrypted_tvb;
   tvbuff_t *gssapi_decrypted_tvb;
+  gboolean gssapi_data_encrypted;
  
   guint32 ppid[MAX_NUMBER_OF_PPIDS]; /* The first NUMBER_OF_PPIDS PPIDS which are present
                                       * in the SCTP packet
                                       */
   void    *private_data;	/* pointer to data passed from one dissector to another */
   GString *layer_names; 	/* layers of each protocol */
+  guint16 link_number;
+  gchar   annex_a_used;
 } packet_info;
 
 #endif /* __PACKET_INFO_H__ */
