@@ -1,7 +1,7 @@
 /* reassemble.c
  * Routines for {fragment,segment} reassembly
  *
- * $Id: reassemble.c 15079 2005-07-26 08:01:16Z jmayer $
+ * $Id: reassemble.c 17074 2006-01-22 19:31:42Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,6 +32,8 @@
 
 #include <epan/reassemble.h>
 
+#include <epan/emem.h>
+
 #include <epan/dissectors/packet-dcerpc.h>
 
 typedef struct _fragment_key {
@@ -48,7 +50,6 @@ typedef struct _dcerpc_fragment_key {
 } dcerpc_fragment_key;
 
 static GMemChunk *fragment_key_chunk = NULL;
-static GMemChunk *dcerpc_fragment_key_chunk = NULL;
 static GMemChunk *fragment_data_chunk = NULL;
 static int fragment_init_count = 200;
 
@@ -144,8 +145,6 @@ typedef struct _reassembled_key {
 	guint32 frame;
 } reassembled_key;
 
-static GMemChunk *reassembled_key_chunk = NULL;
-
 static gint
 reassembled_equal(gconstpointer k1, gconstpointer k2)
 {
@@ -308,28 +307,16 @@ reassemble_init(void)
 {
 	if (fragment_key_chunk != NULL)
 		g_mem_chunk_destroy(fragment_key_chunk);
-	if (dcerpc_fragment_key_chunk != NULL)
-		g_mem_chunk_destroy(dcerpc_fragment_key_chunk);
 	if (fragment_data_chunk != NULL)
 		g_mem_chunk_destroy(fragment_data_chunk);
-	if (reassembled_key_chunk != NULL)
-		g_mem_chunk_destroy(reassembled_key_chunk);
 	fragment_key_chunk = g_mem_chunk_new("fragment_key_chunk",
 	    sizeof(fragment_key),
 	    fragment_init_count * sizeof(fragment_key),
 	    G_ALLOC_AND_FREE);
-	dcerpc_fragment_key_chunk = g_mem_chunk_new("dcerpc_fragment_key_chunk",
-	    sizeof(dcerpc_fragment_key),
-	    fragment_init_count * sizeof(dcerpc_fragment_key),
-	    G_ALLOC_AND_FREE);
 	fragment_data_chunk = g_mem_chunk_new("fragment_data_chunk",
 	    sizeof(fragment_data),
 	    fragment_init_count * sizeof(fragment_data),
 	    G_ALLOC_ONLY);
-	reassembled_key_chunk = g_mem_chunk_new("reassembled_key_chunk",
-	    sizeof(reassembled_key),
-	    fragment_init_count * sizeof(reassembled_key),
-	    G_ALLOC_AND_FREE);
 }
 
 /* This function cleans up the stored state and removes the reassembly data and
@@ -399,6 +386,35 @@ fragment_get(packet_info *pinfo, guint32 id, GHashTable *fragment_table)
 	return fd_head;
 }
 
+/* id *must* be the frame number for this to work! */
+fragment_data *
+fragment_get_reassembled(packet_info *pinfo, guint32 id, GHashTable *reassembled_table)
+{
+	fragment_data *fd_head;
+	reassembled_key key;
+
+	/* create key to search hash with */
+	key.frame = id;
+	key.id = id;
+	fd_head = g_hash_table_lookup(reassembled_table, &key);
+
+	return fd_head;
+}
+
+fragment_data *
+fragment_get_reassembled_id(packet_info *pinfo, guint32 id, GHashTable *reassembled_table)
+{
+	fragment_data *fd_head;
+	reassembled_key key;
+
+	/* create key to search hash with */
+	key.frame = pinfo->fd->num;
+	key.id = id;
+	fd_head = g_hash_table_lookup(reassembled_table, &key);
+
+	return fd_head;
+}
+
 /* This function can be used to explicitely set the total length (if known)
  * for reassembly of a PDU.
  * This is useful for reassembly of PDUs where one may have the total length specified
@@ -476,6 +492,17 @@ fragment_set_partial_reassembly(packet_info *pinfo, guint32 id, GHashTable *frag
 
 	fd_head = g_hash_table_lookup(fragment_table, &key);
 
+	/*
+	 * XXX - why not do all the stuff done early in "fragment_add_work()",
+	 * turning off FD_DEFRAGMENTED and pointing the fragments' data
+	 * pointers to the appropriate part of the already-reassembled
+	 * data, and clearing the data length and "reassembled in" frame
+	 * number, here?  We currently have a hack in the TCP dissector
+	 * not to set the "reassembled in" value if the "partial reassembly"
+	 * flag is set, so that in the first pass through the packets
+	 * we don't falsely set a packet as reassembled in that packet
+	 * if the dissector decided that even more reassembly was needed.
+	 */
 	if(fd_head){
 		fd_head->flags |= FD_PARTIAL_REASSEMBLY;
 	}
@@ -524,7 +551,7 @@ fragment_reassembled(fragment_data *fd_head, packet_info *pinfo,
 		 * This was not fragmented, so there's no fragment
 		 * table; just hash it using the current frame number.
 		 */
-		new_key = g_mem_chunk_alloc(reassembled_key_chunk);
+		new_key = se_alloc(sizeof(reassembled_key));
 		new_key->frame = pinfo->fd->num;
 		new_key->id = id;
 		g_hash_table_insert(reassembled_table, new_key, fd_head);
@@ -533,7 +560,7 @@ fragment_reassembled(fragment_data *fd_head, packet_info *pinfo,
 		 * Hash it with the frame numbers for all the frames.
 		 */
 		for (fd = fd_head->next; fd != NULL; fd = fd->next){
-			new_key = g_mem_chunk_alloc(reassembled_key_chunk);
+			new_key = se_alloc(sizeof(reassembled_key));
 			new_key->frame = fd->frame;
 			new_key->id = id;
 			g_hash_table_insert(reassembled_table, new_key,
@@ -600,6 +627,7 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		fd_head->flags ^= FD_DEFRAGMENTED|FD_PARTIAL_REASSEMBLY;
 		fd_head->flags &= (~FD_TOOLONGFRAGMENT) & (~FD_MULTIPLETAILS);
 		fd_head->datalen=0;
+		fd_head->reassembled_in=0;
 	}
 
 	if (!more_frags) {
@@ -712,7 +740,6 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		fd_head->flags |= FD_TOOLONGFRAGMENT;
 	}
 
-
 	/* we have received an entire packet, defragment it and
          * free all fragments
          */
@@ -791,6 +818,10 @@ fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	fragment_data *fd_item;
 	gboolean already_added=pinfo->fd->flags.visited;
 
+
+    /* dissector shouldn't give us garbage tvb info */
+    DISSECTOR_ASSERT(tvb_bytes_exist(tvb, offset, frag_data_len));
+
 	/* create key to search hash with */
 	key.src = pinfo->src;
 	key.dst = pinfo->dst;
@@ -798,6 +829,21 @@ fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 
 	fd_head = g_hash_table_lookup(fragment_table, &key);
 
+#if 0
+	/* debug output of associated fragments. */
+	/* leave it here for future debugging sessions */
+	if(strcmp(pinfo->current_proto, "DCERPC") == 0) {
+		printf("proto:%s num:%u id:%u offset:%u len:%u more:%u visited:%u\n", 
+			pinfo->current_proto, pinfo->fd->num, id, frag_offset, frag_data_len, more_frags, pinfo->fd->flags.visited);
+		if(fd_head != NULL) {
+			for(fd_item=fd_head->next;fd_item;fd_item=fd_item->next){
+				printf("fd_frame:%u fd_offset:%u len:%u datalen:%u\n", 
+					fd_item->frame, fd_item->offset, fd_item->len, fd_item->datalen);
+			}
+		}
+	}
+#endif
+
 	/*
 	 * "already_added" is true if "pinfo->fd->flags.visited" is true;
 	 * if "pinfo->fd->flags.visited", this isn't the first pass, so
@@ -815,10 +861,14 @@ fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	 * fragment, it's a data structure for the reassembled packet.
 	 * We don't check it because its "frame" member isn't initialized
 	 * to anything, and because it doesn't count in any case.
+	 *
+	 * And as another additional check, make sure the fragment offsets are
+	 * the same, as otherwise we get into trouble if multiple fragments
+	 * are in one PDU.
 	 */
 	if (!already_added && check_already_added && fd_head != NULL) {
 		for(fd_item=fd_head->next;fd_item;fd_item=fd_item->next){
-			if(pinfo->fd->num==fd_item->frame){
+			if(pinfo->fd->num==fd_item->frame && frag_offset==fd_item->offset){
 				already_added=TRUE;
 			}
 		}
@@ -1076,7 +1126,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		/* make sure it doesnt conflict with previous data */
 		dfpos=0;
 		last_fd=NULL;
-		for (fd_i=fd_head->next;fd_i->offset!=fd->offset;fd_i=fd_i->next) {
+		for (fd_i=fd_head->next;fd_i && (fd_i->offset!=fd->offset);fd_i=fd_i->next) {
 		  if (!last_fd || last_fd->offset!=fd_i->offset){
 		    dfpos += fd_i->len;
 		  }
@@ -1151,8 +1201,11 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	 * XXX - what if we didn't capture the entire fragment due
 	 * to a too-short snapshot length?
 	 */
-	fd->data = g_malloc(fd->len);
-	tvb_memcpy(tvb, fd->data, offset, fd->len);
+	/* check len, ther may be a fragment with 0 len, that is actually the tail */
+	if (fd->len) {
+		fd->data = g_malloc(fd->len);
+		tvb_memcpy(tvb, fd->data, offset, fd->len);
+	}
 	LINK_FRAG(fd_head,fd);
 
 
@@ -1323,7 +1376,7 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 }
 
 fragment_data *
-fragment_add_dcerpc(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
+fragment_add_dcerpc_dg(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
                     void *v_act_id,
                     GHashTable *fragment_table, guint32 frag_number,
                     guint32 frag_data_len, gboolean more_frags)
@@ -1373,7 +1426,7 @@ fragment_add_dcerpc(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
                 * addresses, allocating new buffers for the address
                 * data.
                 */
-               new_key = g_mem_chunk_alloc(dcerpc_fragment_key_chunk);
+               new_key = se_alloc(sizeof(dcerpc_fragment_key));
                COPY_ADDRESS(&new_key->src, &key.src);
                COPY_ADDRESS(&new_key->dst, &key.dst);
                new_key->id = key.id;
@@ -1436,6 +1489,8 @@ fragment_add_dcerpc(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
  * of that (empty) list.
  *
  * Otherwise, it returns NULL.
+ *
+ * XXX - Should we simply return NULL for zero-length fragments?
  */
 static fragment_data *
 fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
