@@ -2,13 +2,12 @@
  * Routines for SNA
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: sna-utils.c,v 1.4 2002/08/28 20:40:45 jmayer Exp $
+ * $Id: sna-utils.c 15457 2005-08-20 02:33:33Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
- *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -24,30 +23,47 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#include "sna-utils.h"
+#include <string.h>
 
-/* FID Type 4 */
+#include "packet_info.h"
+#include "pint.h"
+#include "sna-utils.h"
+#include "emem.h"
 
 gchar *
-sna_fid_type_4_addr_to_str(const struct sna_fid_type_4_addr *addrp)
+sna_fid_to_str(const address *addr)
 {
-  static gchar	str[3][14];
-  static gchar	*cur;
+  gchar	*cur;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
-
-  sprintf(cur, "%08X.%04X", addrp->saf, addrp->ef);
+  cur=ep_alloc(14);
+  sna_fid_to_str_buf(addr, cur, 14);
   return cur;
 }
 
+void
+sna_fid_to_str_buf(const address *addr, gchar *buf, int buf_len)
+{
+  struct sna_fid_type_4_addr sna_fid_type_4_addr;
+
+  switch (addr->len) {
+
+  case 1:
+    g_snprintf(buf, buf_len, "%04X", addr->data[0]);
+    break;
+
+  case 2:
+    g_snprintf(buf, buf_len, "%04X", pntohs(&addr->data[0]));
+    break;
+
+  case SNA_FID_TYPE_4_ADDR_LEN:
+    /* FID Type 4 */
+    memcpy(&sna_fid_type_4_addr, addr->data, SNA_FID_TYPE_4_ADDR_LEN);
+    g_snprintf(buf, buf_len, "%08X.%04X", sna_fid_type_4_addr.saf,
+	    sna_fid_type_4_addr.ef);
+    break;
+  }
+}
