@@ -2,7 +2,7 @@
  * API for a counter tree for ethereal
  * 2004, Luis E. G. Ontanon
  *
- * $Id: stats_tree.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: stats_tree.c 17627 2006-03-15 01:41:39Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,6 +31,8 @@
 #include <epan/stats_tree_priv.h>
 #include <string.h>
 
+#include "stats_tree.h"
+
 /*
 TODO: 
    - sort out the sorting issue
@@ -50,7 +52,7 @@ extern void stats_tree_get_strs_from_node(const stat_node* node, guint8* value,
 	if (rate) {
 		*rate = '\0';
 		if (node->st->elapsed > 0.0) {
-			f = ((float)node->counter) / node->st->elapsed;
+			f = ((float)node->counter) / (float)node->st->elapsed;
 			g_snprintf(rate,NUM_BUF_SIZE,"%f",f);
 		}
 	}
@@ -172,7 +174,7 @@ extern void stats_tree_free(stats_tree* st) {
 	
 	g_free(st->filter);
 	g_hash_table_destroy(st->names);
-	g_ptr_array_free(st->parents,FALSE);
+	g_ptr_array_free(st->parents,TRUE);
 	
     for (child = st->root.children; child; child = next ) {
         /* child->next will be gone after free_stat_node, so cache it here */
@@ -210,6 +212,10 @@ static void reset_stat_node(stat_node* node) {
 /* reset the whole stats_tree */
 extern void stats_tree_reset(void* p) {
 	stats_tree* st = p;
+    
+	st->start = -1.0;
+	st->elapsed = 0.0;
+    
 	reset_stat_node(&st->root);
 	
 	if (st->cfg->reset_tree) {
@@ -285,8 +291,7 @@ extern stats_tree* stats_tree_new(stats_tree_cfg* cfg, tree_pres* pr,char* filte
 	
 	st->start = -1.0;
 	st->elapsed = 0.0;
-	st->highest_seen = 0;
-	
+
 	st->root.counter = 0;
 	st->root.name = g_strdup(cfg->name);
 	st->root.st = st;
@@ -304,13 +309,7 @@ extern stats_tree* stats_tree_new(stats_tree_cfg* cfg, tree_pres* pr,char* filte
 /* will be the tap packet cb */
 extern int stats_tree_packet(void* p, packet_info* pinfo, epan_dissect_t *edt, const void *pri) {
 	stats_tree* st = p;
-	float now;
-	
-	if (st->highest_seen >= pinfo->fd->num) return 0;
-	
-	st->highest_seen = pinfo->fd->num;
-	
-	now = (((float)pinfo->fd->rel_secs) + (((float)pinfo->fd->rel_usecs)/1000000) );
+	double now = nstime_to_msec(&pinfo->fd->rel_ts);
 	
 	if (st->start < 0.0) st->start = now;
 	
@@ -322,10 +321,6 @@ extern int stats_tree_packet(void* p, packet_info* pinfo, epan_dissect_t *edt, c
 		return 0;
 }
 
-extern GHashTable* stat_tree_registry(void) {
-	return registry;
-}
-
 extern stats_tree_cfg* stats_tree_get_cfg_by_abbr(guint8* abbr) {
 	return g_hash_table_lookup(registry,abbr);
 }
@@ -346,6 +341,7 @@ static void setup_tree_presentation(gpointer k _U_, gpointer v, gpointer p) {
 	stats_tree_cfg* cfg = v;
 	struct _stats_tree_pres_cbs *d = p;
 	
+	cfg->in_use = FALSE;
 	cfg->setup_node_pr = d->setup_node_pr;
 	cfg->new_tree_pr = d->new_tree_pr;
 	cfg->free_node_pr = d->free_node_pr;
@@ -620,7 +616,7 @@ extern int stats_tree_tick_range(stats_tree* st,
 	}
 	
 	if ( node == NULL ) 
-		return node->id;
+		g_assert_not_reached();
 	
 	for ( child = node->children; child; child = child->next) {
 		floor =  child->rng->floor;
