@@ -1,7 +1,7 @@
 /* strutil.h
  * String utility definitions
  *
- * $Id: strutil.h 12404 2004-10-27 04:45:00Z ulfl $
+ * $Id: strutil.h 17534 2006-03-08 20:55:32Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -99,6 +99,15 @@ gchar*     bytes_to_str_punct(const guint8 *bd, int bd_len, gchar punct);
 gboolean   hex_str_to_bytes(const char *hex_str, GByteArray *bytes,
     gboolean force_separators);
 
+/** Turn a OID string representation (dot notaion) into a byte array.
+ *
+ * @param oid_str The OID string (dot notaion).
+ * @param bytes The GByteArray that will receive the bytes.  This
+ *        must be initialized by the caller.
+ * @return True if the string was converted successfully
+ */
+gboolean   oid_str_to_bytes(const char *oid_str, GByteArray *bytes);
+
 /** Return a XML escaped representation of the unescaped string.
  *  The returned string must be freed when no longer in use. 
  *
@@ -144,4 +153,39 @@ guint8 * convert_string_to_hex(const char *string, size_t *nbytes);
  */
 char * convert_string_case(const char *string, gboolean case_insensitive);
 
+/* g_strlcat() does not exist in GLib 1.2[.x] */
+#if GLIB_MAJOR_VERSION < 2
+gsize g_strlcat(gchar *dst, gchar *src, gsize size);
+#endif
+
+/* g_ascii_isprint() does not exist in GLib 1.2[.x].
+ * assume all codes >=0x20 and <0x80 are ASCII printables.
+ */
+#if GLIB_MAJOR_VERSION < 2
+#define g_ascii_isprint(c)						\
+	(((c<0x20)||(c>=0x80))?FALSE:TRUE)
+#endif
+
+#ifdef _WIN32
+
+/** Given a UTF-8 string, convert it to UTF-16.  This is meant to be used
+ * to convert between GTK+ 2.x (UTF-8) to Windows (UTF-16).
+ *
+ * @param utf8str The string to convert.  May be NULL.
+ * @return The string converted to UTF-16.  If utf8str is NULL, returns
+ * NULL.  The return value should NOT be freed by the caller.
+ */
+wchar_t * utf_8to16(const char *utf8str);
+
+/** Given a UTF-16 string, convert it to UTF-8.  This is meant to be used
+ * to convert between GTK+ 2.x (UTF-8) to Windows (UTF-16).
+ *
+ * @param utf16str The string to convert.  May be NULL.
+ * @return The string converted to UTF-8.  If utf16str is NULL, returns
+ * NULL.  The return value should NOT be freed by the caller.
+ */
+gchar * utf_16to8(const wchar_t *utf16str);
+
+#endif /* _WIN32 */
+
 #endif /* __STRUTIL_H__ */
