@@ -2,7 +2,7 @@
  * T.35 and H.221 tables
  * 2003  Tomas Kukosa
  *
- * $Id: t35.c 12150 2004-09-30 18:12:34Z etxrab $
+ * $Id: t35.c 16655 2005-12-02 19:33:40Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -236,6 +236,7 @@ const value_string T35Extension_vals[] = {
 };
 
 const value_string H221ManufacturerCode_vals[] = {
+	{  0x000b2d00, "Sony"}					,                           /* From captures */
 	{  0x04000042, "Deutsche Telekom AG" },                             /* From Ref. 3 */
 	{  0x04000043, "Deutsche Telekom AG" },                             /* From Ref. 3 */
 	{  0x04000082, "Siemens AG" },                                      /* From Ref. 3 */
@@ -251,6 +252,7 @@ const value_string H221ManufacturerCode_vals[] = {
 	{  0x04000082, "SITK Institut fr Telekommunikation GmbH & Co KG" },/* From Ref. 3 */
 	{  0x0900003D, "Equivalence (OpenH323)" },                          /* From captures */
 	{  0x20000081, "Mediatrix Telecom" },                               /* From Ref. 1 */
+	{  0x3c000000, "Nokia" },											/* From captures */
 	{  0x3d000310, "Swissvoice" },
 	{  0x3d000311, "Swissvoice" },
 	{  0x3d000312, "Swissvoice" },
@@ -268,6 +270,7 @@ const value_string H221ManufacturerCode_vals[] = {
 	{  0x3d00031e, "Swissvoice" },
 	{  0x3d00031f, "Swissvoice" },
 	{  0x82000002, "Ericsson" },                                        /* From captures */
+	{  0x8a000003, "Teldat H. Kruszynski, M. Cichocki Sp. J." } ,		/* By email		 */
 	{  0xa5000001, "Ericsson" },                                        /* From captures */
 	{  0xb4000000, "British Telecommunications" },                      /* From Ref. 2 */
 	{  0xb4000100, "British Telecommunications" },                      /* From Ref. 2 */
