@@ -1,7 +1,7 @@
 /* to_str.c
  * Routines for utilities to convert various other types to strings.
  *
- * $Id: to_str.c 13270 2005-02-03 23:18:23Z jmayer $
+ * $Id: to_str.c 17759 2006-03-29 21:54:22Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,10 +33,6 @@
 # include <sys/types.h>		/* needed for <netinet/in.h> */
 #endif
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #ifdef HAVE_NETINET_IN_H
 # include <netinet/in.h>	/* needed for <arpa/inet.h> on some platforms */
 #endif
@@ -67,24 +63,22 @@
 #include <epan/dissectors/packet-mtp3.h>
 #include <stdio.h>
 #include <time.h>
-
-#define MAX_BYTESTRING_LEN	6
+#include "emem.h"
 
 /* Routine to convert a sequence of bytes to a hex string, one byte/two hex
  * digits at at a time, with a specified punctuation character between
- * the bytes.  The sequence of bytes must be no longer than
- * MAX_BYTESTRING_LEN.
+ * the bytes.
  *
  * If punct is '\0', no punctuation is applied (and thus
  * the resulting string is (len-1) bytes shorter)
  */
-static gchar *
+gchar *
 bytestring_to_str(const guint8 *ad, guint32 len, char punct) {
-  static gchar  str[3][MAX_BYTESTRING_LEN*3];
-  static gchar *cur;
+  gchar *buf;
   gchar        *p;
-  int          i;
+  int          i = (int) len - 1;
   guint32      octet;
+  size_t       buflen;
   /* At least one version of Apple's C compiler/linker is buggy, causing
      a complaint from the linker about the "literal C string section"
      not ending with '\0' if we initialize a 16-element "char" array with
@@ -95,25 +89,24 @@ bytestring_to_str(const guint8 *ad, guint32 len, char punct) {
       { '0', '1', '2', '3', '4', '5', '6', '7',
         '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
 
-  g_assert(len > 0 && len <= MAX_BYTESTRING_LEN);
-  len--;
+  if (punct)
+    buflen=len*3;
+  else
+    buflen=len*2 + 1;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
+  if (buflen < 3 || i < 0) {
+    return "";
   }
-  p = &cur[18];
-  *--p = '\0';
-  i = len;
+
+  buf=ep_alloc(buflen);
+  p = &buf[buflen - 1];
+  *p = '\0';
   for (;;) {
     octet = ad[i];
     *--p = hex_digits[octet&0xF];
     octet >>= 4;
     *--p = hex_digits[octet&0xF];
-    if (i == 0)
+    if (i <= 0)
       break;
     if (punct)
       *--p = punct;
@@ -125,8 +118,8 @@ bytestring_to_str(const guint8 *ad, guint32 len, char punct) {
 /* Wrapper for the most common case of asking
  * for a string using a colon as the hex-digit separator.
  */
-/* XXX FIXME   
-remove this one later when every call has been converted to address_to_str() 
+/* XXX FIXME
+remove this one later when every call has been converted to address_to_str()
 */
 gchar *
 ether_to_str(const guint8 *ad)
@@ -134,62 +127,55 @@ ether_to_str(const guint8 *ad)
 	return bytestring_to_str(ad, 6, ':');
 }
 
-/* 
+/*
  This function is very fast and this function is called a lot.
  XXX update the address_to_str stuff to use this function.
 */
 gchar *
 ip_to_str(const guint8 *ad) {
-  static gchar  str[4][16];
-  static guint32   cur_idx=0;
-  gchar *cur;
+  gchar *buf;
 
-  cur_idx++;
-  if(cur_idx>3){ 
-     cur_idx=0;
-  }
-  cur=&str[cur_idx][0];
-
-  ip_to_str_buf(ad, cur);
-  return cur;
+  buf=ep_alloc(16);
+  ip_to_str_buf(ad, buf);
+  return buf;
 }
 
-/* 
+/*
  This function is very fast and this function is called a lot.
  XXX update the address_to_str stuff to use this function.
 */
 static const char * const fast_strings[] = {
-"0", "1", "2", "3", "4", "5", "6", "7", 
-"8", "9", "10", "11", "12", "13", "14", "15", 
-"16", "17", "18", "19", "20", "21", "22", "23", 
-"24", "25", "26", "27", "28", "29", "30", "31", 
-"32", "33", "34", "35", "36", "37", "38", "39", 
-"40", "41", "42", "43", "44", "45", "46", "47", 
-"48", "49", "50", "51", "52", "53", "54", "55", 
-"56", "57", "58", "59", "60", "61", "62", "63", 
-"64", "65", "66", "67", "68", "69", "70", "71", 
-"72", "73", "74", "75", "76", "77", "78", "79", 
-"80", "81", "82", "83", "84", "85", "86", "87", 
-"88", "89", "90", "91", "92", "93", "94", "95", 
-"96", "97", "98", "99", "100", "101", "102", "103", 
-"104", "105", "106", "107", "108", "109", "110", "111", 
-"112", "113", "114", "115", "116", "117", "118", "119", 
-"120", "121", "122", "123", "124", "125", "126", "127", 
-"128", "129", "130", "131", "132", "133", "134", "135", 
-"136", "137", "138", "139", "140", "141", "142", "143", 
-"144", "145", "146", "147", "148", "149", "150", "151", 
-"152", "153", "154", "155", "156", "157", "158", "159", 
-"160", "161", "162", "163", "164", "165", "166", "167", 
-"168", "169", "170", "171", "172", "173", "174", "175", 
-"176", "177", "178", "179", "180", "181", "182", "183", 
-"184", "185", "186", "187", "188", "189", "190", "191", 
-"192", "193", "194", "195", "196", "197", "198", "199", 
-"200", "201", "202", "203", "204", "205", "206", "207", 
-"208", "209", "210", "211", "212", "213", "214", "215", 
-"216", "217", "218", "219", "220", "221", "222", "223", 
-"224", "225", "226", "227", "228", "229", "230", "231", 
-"232", "233", "234", "235", "236", "237", "238", "239", 
-"240", "241", "242", "243", "244", "245", "246", "247", 
+"0", "1", "2", "3", "4", "5", "6", "7",
+"8", "9", "10", "11", "12", "13", "14", "15",
+"16", "17", "18", "19", "20", "21", "22", "23",
+"24", "25", "26", "27", "28", "29", "30", "31",
+"32", "33", "34", "35", "36", "37", "38", "39",
+"40", "41", "42", "43", "44", "45", "46", "47",
+"48", "49", "50", "51", "52", "53", "54", "55",
+"56", "57", "58", "59", "60", "61", "62", "63",
+"64", "65", "66", "67", "68", "69", "70", "71",
+"72", "73", "74", "75", "76", "77", "78", "79",
+"80", "81", "82", "83", "84", "85", "86", "87",
+"88", "89", "90", "91", "92", "93", "94", "95",
+"96", "97", "98", "99", "100", "101", "102", "103",
+"104", "105", "106", "107", "108", "109", "110", "111",
+"112", "113", "114", "115", "116", "117", "118", "119",
+"120", "121", "122", "123", "124", "125", "126", "127",
+"128", "129", "130", "131", "132", "133", "134", "135",
+"136", "137", "138", "139", "140", "141", "142", "143",
+"144", "145", "146", "147", "148", "149", "150", "151",
+"152", "153", "154", "155", "156", "157", "158", "159",
+"160", "161", "162", "163", "164", "165", "166", "167",
+"168", "169", "170", "171", "172", "173", "174", "175",
+"176", "177", "178", "179", "180", "181", "182", "183",
+"184", "185", "186", "187", "188", "189", "190", "191",
+"192", "193", "194", "195", "196", "197", "198", "199",
+"200", "201", "202", "203", "204", "205", "206", "207",
+"208", "209", "210", "211", "212", "213", "214", "215",
+"216", "217", "218", "219", "220", "221", "222", "223",
+"224", "225", "226", "227", "228", "229", "230", "231",
+"232", "233", "234", "235", "236", "237", "238", "239",
+"240", "241", "242", "243", "244", "245", "246", "247",
 "248", "249", "250", "251", "252", "253", "254", "255"
 };
 void
@@ -228,25 +214,20 @@ ip_to_str_buf(const guint8 *ad, gchar *buf)
 }
 
 
-/* XXX FIXME   
-remove this one later when every call has been converted to address_to_str() 
+/* XXX FIXME
+remove this one later when every call has been converted to address_to_str()
 */
 gchar *
 ip6_to_str(const struct e_in6_addr *ad) {
 #ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46
 #endif
-  static int i=0;
-  static gchar *strp, str[4][INET6_ADDRSTRLEN];
+  static gchar *str;
 
-  i++;
-  if(i>=4){
-    i=0;
-  }
-  strp=str[i];
+  str=ep_alloc(INET6_ADDRSTRLEN+1);
 
-  ip6_to_str_buf(ad, strp);
-  return strp;
+  ip6_to_str_buf(ad, str);
+  return str;
 }
 
 void
@@ -258,28 +239,20 @@ ip6_to_str_buf(const struct e_in6_addr *ad, gchar *buf)
 gchar*
 ipx_addr_to_str(guint32 net, const guint8 *ad)
 {
-	static gchar	str[3][8+1+MAXNAMELEN+1]; /* 8 digits, 1 period, NAME, 1 null */
-	static gchar	*cur;
-	char		*name;
-
-	if (cur == &str[0][0]) {
-		cur = &str[1][0];
-	} else if (cur == &str[1][0]) {
-		cur = &str[2][0];
-	} else {
-		cur = &str[0][0];
-	}
+	gchar	*buf;
+	char	*name;
 
+	buf=ep_alloc(8+1+MAXNAMELEN+1); /* 8 digits, 1 period, NAME, 1 null */
 	name = get_ether_name_if_known(ad);
 
 	if (name) {
-		sprintf(cur, "%s.%s", get_ipxnet_name(net), name);
+		g_snprintf(buf, 8+1+MAXNAMELEN+1, "%s.%s", get_ipxnet_name(net), name);
 	}
 	else {
-		sprintf(cur, "%s.%s", get_ipxnet_name(net),
+		g_snprintf(buf, 8+1+MAXNAMELEN+1, "%s.%s", get_ipxnet_name(net),
 		    bytestring_to_str(ad, 6, '\0'));
 	}
-	return cur;
+	return buf;
 }
 
 gchar*
@@ -292,8 +265,7 @@ ipxnet_to_string(const guint8 *ad)
 gchar *
 ipxnet_to_str_punct(const guint32 ad, char punct)
 {
-  static gchar  str[3][12];
-  static gchar *cur;
+  gchar        *buf;
   gchar        *p;
   int          i;
   guint32      octet;
@@ -309,14 +281,8 @@ ipxnet_to_str_punct(const guint32 ad, char punct)
   static const guint32  octet_mask[4] =
 	  { 0xff000000 , 0x00ff0000, 0x0000ff00, 0x000000ff };
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
-  p = &cur[12];
+  buf=ep_alloc(12);
+  p = &buf[12];
   *--p = '\0';
   i = 3;
   for (;;) {
@@ -336,24 +302,18 @@ ipxnet_to_str_punct(const guint32 ad, char punct)
 gchar *
 vines_addr_to_str(const guint8 *addrp)
 {
-  static gchar	str[3][214];
-  static gchar	*cur;
+  gchar	*buf;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
-  vines_addr_to_str_buf(addrp, cur);
-  return cur;
+  buf=ep_alloc(214);
+
+  vines_addr_to_str_buf(addrp, buf, 214);
+  return buf;
 }
 
 void
-vines_addr_to_str_buf(const guint8 *addrp, gchar *buf)
+vines_addr_to_str_buf(const guint8 *addrp, gchar *buf, int buf_len)
 {
-  sprintf(buf, "%08x.%04x", pntohl(&addrp[0]), pntohs(&addrp[4]));
+  g_snprintf(buf, buf_len, "%08x.%04x", pntohl(&addrp[0]), pntohs(&addrp[4]));
 }
 
 #define	PLURALIZE(n)	(((n) > 1) ? "s" : "")
@@ -362,8 +322,9 @@ vines_addr_to_str_buf(const guint8 *addrp, gchar *buf)
 /*
  * Maximum length of a string showing days/hours/minutes/seconds.
  * (Does not include the terminating '\0'.)
+ * Includes space for a '-' sign for any negative compunents.
  */
-#define TIME_SECS_LEN	(8+1+4+2+2+5+2+2+7+2+2+7)
+#define TIME_SECS_LEN	(10+1+4+2+2+5+2+2+7+2+2+7+4)
 
 /*
  * Convert a value in seconds and fractions of a second to a string,
@@ -371,15 +332,27 @@ vines_addr_to_str_buf(const guint8 *addrp, gchar *buf)
  * into a buffer.
  * "is_nsecs" says that "frac" is microseconds if true and milliseconds
  * if false.
+ * If time is negative, add a '-' to all non-null components.
  */
 static void
-time_secs_to_str_buf(guint32 time, guint32 frac, gboolean is_nsecs,
-			   gchar *buf)
+time_secs_to_str_buf(gint32 time, guint32 frac, gboolean is_nsecs,
+			   gchar *buf, int buf_len)
 {
   static gchar *p;
   int hours, mins, secs;
+  const gchar *msign = "";
   gboolean do_comma = FALSE;
 
+  if(time<0){
+    time= -time;
+    msign="-";
+  }
+
+  if(time<0){	/* We've overflowed. */
+    g_snprintf(buf, buf_len, "Unable to cope with time value %d", time);
+    return;
+  }
+
   secs = time % 60;
   time /= 60;
   mins = time % 60;
@@ -387,81 +360,73 @@ time_secs_to_str_buf(guint32 time, guint32 frac, gboolean is_nsecs,
   hours = time % 24;
   time /= 24;
 
+  /* This would probably be cleaner if we used GStrings instead. */
   p = buf;
   if (time != 0) {
-    sprintf(p, "%u day%s", time, PLURALIZE(time));
-    p += strlen(p);
+    p += g_snprintf(p, buf_len, "%s%u day%s", time?msign:"", time, PLURALIZE(time));
     do_comma = TRUE;
   }
   if (hours != 0) {
-    sprintf(p, "%s%u hour%s", COMMA(do_comma), hours, PLURALIZE(hours));
-    p += strlen(p);
+    p += g_snprintf(p, buf_len-(p-buf), "%s%s%u hour%s", COMMA(do_comma), hours?msign:"", hours, PLURALIZE(hours));
     do_comma = TRUE;
   }
   if (mins != 0) {
-    sprintf(p, "%s%u minute%s", COMMA(do_comma), mins, PLURALIZE(mins));
-    p += strlen(p);
+    p += g_snprintf(p, buf_len-(p-buf), "%s%s%u minute%s", COMMA(do_comma), mins?msign:"", mins, PLURALIZE(mins));
     do_comma = TRUE;
   }
   if (secs != 0 || frac != 0) {
     if (frac != 0) {
       if (is_nsecs)
-        sprintf(p, "%s%u.%09u seconds", COMMA(do_comma), secs, frac);
+        p += g_snprintf(p, buf_len-(p-buf), "%s%s%u.%09u seconds", COMMA(do_comma), msign, secs, frac);
       else
-        sprintf(p, "%s%u.%03u seconds", COMMA(do_comma), secs, frac);
+        p += g_snprintf(p, buf_len-(p-buf), "%s%s%u.%03u seconds", COMMA(do_comma), msign, secs, frac);
     } else
-      sprintf(p, "%s%u second%s", COMMA(do_comma), secs, PLURALIZE(secs));
+      p += g_snprintf(p, buf_len-(p-buf), "%s%s%u second%s", COMMA(do_comma), msign, secs, PLURALIZE(secs));
   }
 }
 
 gchar *
-time_secs_to_str(guint32 time)
+time_secs_to_str(gint32 time)
 {
-  static gchar  str[3][TIME_SECS_LEN+1];
-  static gchar *cur;
+  gchar *buf;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
+  buf=ep_alloc(TIME_SECS_LEN+1);
 
   if (time == 0) {
-    sprintf(cur, "0 time");
-    return cur;
+    g_snprintf(buf, TIME_SECS_LEN+1, "0 time");
+    return buf;
   }
 
-  time_secs_to_str_buf(time, 0, FALSE, cur);
-  return cur;
+  time_secs_to_str_buf(time, 0, FALSE, buf, TIME_SECS_LEN+1);
+  return buf;
 }
 
 gchar *
-time_msecs_to_str(guint32 time)
+time_msecs_to_str(gint32 time)
 {
-  static gchar  str[3][TIME_SECS_LEN+1+3+1];
-  static gchar *cur;
+  gchar *buf;
   int msecs;
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {
-    cur = &str[2][0];
-  } else {
-    cur = &str[0][0];
-  }
+  buf=ep_alloc(TIME_SECS_LEN+1+3+1);
 
   if (time == 0) {
-    sprintf(cur, "0 time");
-    return cur;
+    g_snprintf(buf, TIME_SECS_LEN+1+3+1, "0 time");
+    return buf;
   }
 
-  msecs = time % 1000;
-  time /= 1000;
+  if(time<0){
+    /* oops we got passed a negative time */
+    time= -time;
+    msecs = time % 1000;
+    time /= 1000;
+    time= -time;
+  } else {
+    msecs = time % 1000;
+    time /= 1000;
+  }
 
-  time_secs_to_str_buf(time, msecs, FALSE, cur);
-  return cur;
+  time_secs_to_str_buf(time, msecs, FALSE, buf, TIME_SECS_LEN+1+3+1);
+  return buf;
 }
 
 static const char *mon_names[12] = {
@@ -483,20 +448,14 @@ gchar *
 abs_time_to_str(nstime_t *abs_time)
 {
         struct tm *tmp;
-        static gchar *cur;
-        static char str[3][3+1+2+2+4+1+2+1+2+1+2+1+9+1];
+        gchar *buf;
 
-        if (cur == &str[0][0]) {
-                cur = &str[1][0];
-        } else if (cur == &str[1][0]) {
-                cur = &str[2][0];
-        } else {
-                cur = &str[0][0];
-        }
+	buf=ep_alloc(3+1+2+2+4+1+2+1+2+1+2+1+9+1);
 
         tmp = localtime(&abs_time->secs);
         if (tmp) {
-		sprintf(cur, "%s %2d, %d %02d:%02d:%02d.%09ld",
+		g_snprintf(buf, 3+1+2+2+4+1+2+1+2+1+2+1+9+1,
+		    "%s %2d, %d %02d:%02d:%02d.%09ld",
 		    mon_names[tmp->tm_mon],
 		    tmp->tm_mday,
 		    tmp->tm_year + 1900,
@@ -505,28 +464,22 @@ abs_time_to_str(nstime_t *abs_time)
 		    tmp->tm_sec,
 		    (long)abs_time->nsecs);
         } else
-		strncpy(cur, "Not representable", sizeof(str[0]));
-        return cur;
+		strncpy(buf, "Not representable", 3+1+2+2+4+1+2+1+2+1+2+1+9+1);
+        return buf;
 }
 
 gchar *
 abs_time_secs_to_str(time_t abs_time)
 {
         struct tm *tmp;
-        static gchar *cur;
-        static char str[3][3+1+2+2+4+1+2+1+2+1+2+1];
+        gchar *buf;
 
-        if (cur == &str[0][0]) {
-                cur = &str[1][0];
-        } else if (cur == &str[1][0]) {
-                cur = &str[2][0];
-        } else {
-                cur = &str[0][0];
-        }
+	buf=ep_alloc(3+1+2+2+4+1+2+1+2+1+2+1);
 
         tmp = localtime(&abs_time);
         if (tmp) {
-		sprintf(cur, "%s %2d, %d %02d:%02d:%02d",
+		g_snprintf(buf, 3+1+2+2+4+1+2+1+2+1+2+1,
+		    "%s %2d, %d %02d:%02d:%02d",
 		    mon_names[tmp->tm_mon],
 		    tmp->tm_mday,
 		    tmp->tm_year + 1900,
@@ -534,15 +487,15 @@ abs_time_secs_to_str(time_t abs_time)
 		    tmp->tm_min,
 		    tmp->tm_sec);
         } else
-		strncpy(cur, "Not representable", sizeof(str[0]));
-        return cur;
+		strncpy(buf, "Not representable", 3+1+2+2+4+1+2+1+2+1+2+1);
+        return buf;
 }
 
 void
 display_signed_time(gchar *buf, int buflen, gint32 sec, gint32 frac,
     time_res_t units)
 {
-	char *sign;
+	const char *sign;
 
 	/* If the fractional part of the time stamp is negative,
 	   print its absolute value and, if the seconds part isn't
@@ -556,16 +509,28 @@ display_signed_time(gchar *buf, int buflen, gint32 sec, gint32 frac,
 	}
 	switch (units) {
 
+	case SECS:
+		g_snprintf(buf, buflen, "%s%d", sign, sec);
+		break;
+
+	case DSECS:
+		g_snprintf(buf, buflen, "%s%d.%01d", sign, sec, frac);
+		break;
+
+	case CSECS:
+		g_snprintf(buf, buflen, "%s%d.%02d", sign, sec, frac);
+		break;
+
 	case MSECS:
-		snprintf(buf, buflen, "%s%d.%03d", sign, sec, frac);
+		g_snprintf(buf, buflen, "%s%d.%03d", sign, sec, frac);
 		break;
 
 	case USECS:
-		snprintf(buf, buflen, "%s%d.%06d", sign, sec, frac);
+		g_snprintf(buf, buflen, "%s%d.%06d", sign, sec, frac);
 		break;
 
 	case NSECS:
-		snprintf(buf, buflen, "%s%d.%09d", sign, sec, frac);
+		g_snprintf(buf, buflen, "%s%d.%09d", sign, sec, frac);
 		break;
 	}
 }
@@ -576,21 +541,14 @@ display_signed_time(gchar *buf, int buflen, gint32 sec, gint32 frac,
 gchar *
 rel_time_to_str(nstime_t *rel_time)
 {
-	static gchar *cur;
-	static char str[3][1+TIME_SECS_LEN+1+6+1];
+	gchar *buf;
 	char *p;
-	char *sign;
-	guint32 time;
+	const char *sign;
+	gint32 time;
 	gint32 nsec;
 
-	if (cur == &str[0][0]) {
-		cur = &str[1][0];
-	} else if (cur == &str[1][0]) {
-		cur = &str[2][0];
-	} else {
-		cur = &str[0][0];
-	}
-	p = cur;
+	buf=ep_alloc(1+TIME_SECS_LEN+1+6+1);
+	p = buf;
 
 	/* If the nanoseconds part of the time stamp is negative,
 	   print its absolute value and, if the seconds part isn't
@@ -600,8 +558,8 @@ rel_time_to_str(nstime_t *rel_time)
 	time = rel_time->secs;
 	nsec = rel_time->nsecs;
 	if (time == 0 && nsec == 0) {
-		sprintf(cur, "0.000000000 seconds");
-		return cur;
+		g_snprintf(buf, 1+TIME_SECS_LEN+1+6+1, "0.000000000 seconds");
+		return buf;
 	}
 	if (nsec < 0) {
 		nsec = -nsec;
@@ -615,8 +573,8 @@ rel_time_to_str(nstime_t *rel_time)
 		time = -rel_time->secs;
 	}
 
-	time_secs_to_str_buf(time, nsec, TRUE, p);
-	return cur;
+	time_secs_to_str_buf(time, nsec, TRUE, p, 1+TIME_SECS_LEN+1+6+1);
+	return buf;
 }
 
 #define REL_TIME_SECS_LEN	(1+10+1+9+1)
@@ -627,25 +585,18 @@ rel_time_to_str(nstime_t *rel_time)
 gchar *
 rel_time_to_secs_str(nstime_t *rel_time)
 {
-        static gchar *cur;
-        static char str[3][REL_TIME_SECS_LEN];
+        gchar *buf;
 
-        if (cur == &str[0][0]) {
-                cur = &str[1][0];
-        } else if (cur == &str[1][0]) {
-                cur = &str[2][0];
-        } else {
-                cur = &str[0][0];
-        }
+	buf=ep_alloc(REL_TIME_SECS_LEN);
 
-        display_signed_time(cur, REL_TIME_SECS_LEN, rel_time->secs,
+        display_signed_time(buf, REL_TIME_SECS_LEN, rel_time->secs,
             rel_time->nsecs, NSECS);
-        return cur;
+        return buf;
 }
 
 
-/* XXX FIXME   
-remove this one later when every call has been converted to address_to_str() 
+/* XXX FIXME
+remove this one later when every call has been converted to address_to_str()
 */
 gchar *
 fc_to_str(const guint8 *ad)
@@ -670,10 +621,12 @@ fcwwn_to_str (const guint8 *ad)
 {
     int fmt;
     guint8 oui[6];
-    static gchar ethstr[512];
-    
+    gchar *ethstr;
+
     if (ad == NULL) return NULL;
-    
+
+    ethstr=ep_alloc(512);
+
     fmt = (ad[0] & 0xF0) >> 4;
 
     switch (fmt) {
@@ -681,7 +634,7 @@ fcwwn_to_str (const guint8 *ad)
     case FC_NH_NAA_IEEE:
     case FC_NH_NAA_IEEE_E:
         memcpy (oui, &ad[2], 6);
-        sprintf (ethstr, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x (%s)", ad[0], 
+        g_snprintf (ethstr, 512, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x (%s)", ad[0],
                  ad[1], ad[2], ad[3], ad[4], ad[5], ad[6], ad[7],
                  get_manuf_name (oui));
         break;
@@ -694,13 +647,13 @@ fcwwn_to_str (const guint8 *ad)
         oui[4] = ((ad[4] & 0x0F) << 4) | ((ad[5] & 0xF0) >> 4);
         oui[5] = ((ad[5] & 0x0F) << 4) | ((ad[6] & 0xF0) >> 4);
 
-        sprintf (ethstr, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x (%s)", ad[0],
+        g_snprintf (ethstr, 512, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x (%s)", ad[0],
                  ad[1], ad[2], ad[3], ad[4], ad[5], ad[6], ad[7],
                  get_manuf_name (oui));
         break;
 
     default:
-        sprintf (ethstr, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x", ad[0],
+        g_snprintf (ethstr, 512, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x", ad[0],
                  ad[1], ad[2], ad[3], ad[4], ad[5], ad[6], ad[7]);
         break;
     }
@@ -709,6 +662,7 @@ fcwwn_to_str (const guint8 *ad)
 
 /* Generate, into "buf", a string showing the bits of a bitfield.
    Return a pointer to the character after that string. */
+/*XXX this needs a buf_len check */
 char *
 other_decode_bitfield_value(char *buf, guint32 val, guint32 mask, int width)
 {
@@ -758,9 +712,10 @@ const char *
 decode_boolean_bitfield(guint32 val, guint32 mask, int width,
     const char *truedesc, const char *falsedesc)
 {
-  static char buf[1025];
+  char *buf;
   char *p;
 
+  buf=ep_alloc(1025); /* is this a bit overkill? */
   p = decode_bitfield_value(buf, val, mask, width);
   if (val & mask)
     strcpy(p, truedesc);
@@ -775,17 +730,18 @@ const char *
 decode_numeric_bitfield(guint32 val, guint32 mask, int width,
     const char *fmt)
 {
-  static char buf[1025];
+  char *buf;
   char *p;
   int shift = 0;
 
+  buf=ep_alloc(1025); /* isnt this a bit overkill? */
   /* Compute the number of bits we have to shift the bitfield right
      to extract its value. */
   while ((mask & (1<<shift)) == 0)
     shift++;
 
   p = decode_bitfield_value(buf, val, mask, width);
-  sprintf(p, fmt, (val & mask) >> shift);
+  g_snprintf(p, 1025-(p-buf), fmt, (val & mask) >> shift);
   return buf;
 }
 
@@ -802,33 +758,27 @@ decode_numeric_bitfield(guint32 val, guint32 mask, int width,
    that "col_set_addr()" need know nothing whatsoever about particular
    address types */
 /* convert an address struct into a printable string */
-gchar*	
+gchar*
 address_to_str(const address *addr)
 {
-#ifndef INET6_ADDRSTRLEN
-#define INET6_ADDRSTRLEN 46
-#endif
-  static int i=0;
-  static gchar *strp, str[16][INET6_ADDRSTRLEN];/* IPv6 is the largest one */
+  gchar *str;
 
-  i++;
-  if(i>=16){
-    i=0;
-  }
-  strp=str[i];
-
-  address_to_str_buf(addr, strp);
-  return strp;
+  str=ep_alloc(256);
+  address_to_str_buf(addr, str, 256);
+  return str;
 }
 
 void
-address_to_str_buf(const address *addr, gchar *buf)
+address_to_str_buf(const address *addr, gchar *buf, int buf_len)
 {
   struct atalk_ddp_addr ddp_addr;
 
   switch(addr->type){
+  case AT_NONE:
+    g_snprintf(buf, buf_len, "%s", "");
+    break;
   case AT_ETHER:
-    sprintf(buf, "%02x:%02x:%02x:%02x:%02x:%02x", addr->data[0], addr->data[1], addr->data[2], addr->data[3], addr->data[4], addr->data[5]);
+    g_snprintf(buf, buf_len, "%02x:%02x:%02x:%02x:%02x:%02x", addr->data[0], addr->data[1], addr->data[2], addr->data[3], addr->data[4], addr->data[5]);
     break;
   case AT_IPv4:
     ip_to_str_buf(addr->data, buf);
@@ -837,36 +787,131 @@ address_to_str_buf(const address *addr, gchar *buf)
     inet_ntop(AF_INET6, addr->data, buf, INET6_ADDRSTRLEN);
     break;
   case AT_IPX:
-    sprintf(buf, "%02x%02x%02x%02x.%02x%02x%02x%02x%02x%02x", addr->data[0], addr->data[1], addr->data[2], addr->data[3], addr->data[4], addr->data[5], addr->data[6], addr->data[7], addr->data[8], addr->data[9]);
+    g_snprintf(buf, buf_len, "%02x%02x%02x%02x.%02x%02x%02x%02x%02x%02x", addr->data[0], addr->data[1], addr->data[2], addr->data[3], addr->data[4], addr->data[5], addr->data[6], addr->data[7], addr->data[8], addr->data[9]);
     break;
   case AT_SNA:
-    sna_fid_to_str_buf(addr, buf);
+    sna_fid_to_str_buf(addr, buf, buf_len);
     break;
   case AT_ATALK:
     memcpy(&ddp_addr, addr->data, sizeof ddp_addr);
-    atalk_addr_to_str_buf(&ddp_addr, buf);
+    atalk_addr_to_str_buf(&ddp_addr, buf, buf_len);
     break;
   case AT_VINES:
-    vines_addr_to_str_buf(addr->data, buf);
+    vines_addr_to_str_buf(addr->data, buf, buf_len);
     break;
   case AT_OSI:
-    print_nsap_net_buf(addr->data, addr->len, buf);
+    print_nsap_net_buf(addr->data, addr->len, buf, buf_len);
     break;
   case AT_ARCNET:
-    sprintf(buf, "0x%02X", addr->data[0]);
+    g_snprintf(buf, buf_len, "0x%02X", addr->data[0]);
     break;
   case AT_FC:
-    sprintf(buf, "%02x.%02x.%02x", addr->data[0], addr->data[1], addr->data[2]);
+    g_snprintf(buf, buf_len, "%02x.%02x.%02x", addr->data[0], addr->data[1], addr->data[2]);
     break;
   case AT_SS7PC:
-    mtp3_addr_to_str_buf(addr->data, buf);
+    mtp3_addr_to_str_buf(addr->data, buf, buf_len);
+    break;
+  case AT_STRINGZ:
+    g_snprintf(buf, buf_len, "%s", addr->data);
     break;
   case AT_EUI64:
-    sprintf(buf, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
+    g_snprintf(buf, buf_len, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
             addr->data[0], addr->data[1], addr->data[2], addr->data[3],
             addr->data[4], addr->data[5], addr->data[6], addr->data[7]);
     break;
+  case AT_URI: {
+    int copy_len = addr->len < (buf_len - 1) ? addr->len : (buf_len - 1);
+    memmove(buf, addr->data, copy_len );
+    buf[copy_len] = '\0';
+    }
+    break;
+  case AT_TIPC:
+	  tipc_addr_to_str_buf(addr->data, buf, buf_len);
+	  break;
   default:
     g_assert_not_reached();
   }
 }
+
+gchar* oid_to_str(const guint8 *oid, gint oid_len) {
+  gchar *buf;
+
+  buf=ep_alloc(MAX_OID_STR_LEN);
+  return oid_to_str_buf(oid, oid_len, buf, MAX_OID_STR_LEN);
+}
+
+#define OID_STR_LIMIT (1 + 10 + 4 + 1) /* "." + 10 digits + ".>>>" + '\0' */
+gchar* oid_to_str_buf(const guint8 *oid, gint oid_len, gchar *buf, int buf_len) {
+  gint i;
+  guint8 byte;
+  guint32 subid0, subid;
+  gboolean is_first;
+  gchar *bufp;
+
+  bufp = buf; subid = 0; is_first = TRUE;
+  for (i=0; i<oid_len; i++){
+    byte = oid[i];
+    if ((bufp - buf) > (buf_len - OID_STR_LIMIT)) {
+      bufp += g_snprintf(bufp, buf_len-(bufp-buf), ".>>>");
+      break;
+    }
+    subid <<= 7;
+    subid |= byte & 0x7F;
+    if (byte & 0x80) {
+      continue;
+    }
+    if (is_first) {
+      subid0 = 0;
+      if (subid >= 40) { subid0++; subid-=40; }
+      if (subid >= 40) { subid0++; subid-=40; }
+      bufp += g_snprintf(bufp, buf_len-(bufp-buf), "%u.%u", subid0, subid);
+      is_first = FALSE;
+    } else {
+      bufp += g_snprintf(bufp, buf_len-(bufp-buf), ".%u", subid);
+    }
+    subid = 0;
+  }
+  *bufp = '\0';
+
+  return buf;
+}
+
+gchar* guid_to_str(const e_guid_t *guid) {
+  gchar *buf;
+
+  buf=ep_alloc(GUID_STR_LEN);
+  return guid_to_str_buf(guid, buf, GUID_STR_LEN);
+}
+
+gchar* guid_to_str_buf(const e_guid_t *guid, gchar *buf, int buf_len) {
+  g_snprintf(buf, buf_len, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
+          guid->data1, guid->data2, guid->data3,
+          guid->data4[0], guid->data4[1], guid->data4[2], guid->data4[3], guid->data4[4], guid->data4[5], guid->data4[6], guid->data4[7]);
+  return buf;
+}
+
+void
+tipc_addr_to_str_buf( const guint8 *data, gchar *buf, int buf_len){
+	guint8 zone;
+	guint16 subnetwork;
+	guint16 processor;
+	guint32 tipc_address;
+
+	tipc_address = data[0];
+	tipc_address = (tipc_address << 8) ^ data[1];
+	tipc_address = (tipc_address << 8) ^ data[2];
+	tipc_address = (tipc_address << 8) ^ data[3];
+
+	processor = tipc_address & 0x0fff;
+
+	tipc_address = tipc_address >> 12;
+	subnetwork = tipc_address & 0x0fff;
+
+	tipc_address = tipc_address >> 12;
+	zone = tipc_address & 0xff;
+
+	g_snprintf(buf,buf_len,"%u.%u.%u",zone,subnetwork,processor);
+
+
+}
+
