@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c 15100 2005-07-26 18:32:12Z lego $
+ * $Id: tvbuff.c 17567 2006-03-10 12:57:22Z kukosa $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -49,6 +49,7 @@
 #include "tvbuff.h"
 #include "strutil.h"
 #include "emem.h"
+#include "proto.h"	/* XXX - only used for DISSECTOR_ASSERT, probably a new header file? */
 
 static const guint8*
 ensure_contiguous_no_exception(tvbuff_t *tvb, gint offset, gint length,
@@ -124,7 +125,6 @@ tvb_new(tvbuff_type type)
 	tvbuff_t	*tvb;
 
 	tvb = g_chunk_new(tvbuff_t, tvbuff_mem_chunk);
-	g_assert(tvb);
 
 	tvb_init(tvb, type);
 
@@ -232,7 +232,7 @@ tvb_free_chain(tvbuff_t* tvb)
 void
 tvb_set_free_cb(tvbuff_t* tvb, tvbuff_free_cb_t func)
 {
-	g_assert(tvb->type == TVBUFF_REAL_DATA);
+	DISSECTOR_ASSERT(tvb->type == TVBUFF_REAL_DATA);
 	tvb->free_cb = func;
 }
 
@@ -246,17 +246,17 @@ add_to_used_in_list(tvbuff_t *tvb, tvbuff_t *used_in)
 void
 tvb_set_child_real_data_tvbuff(tvbuff_t* parent, tvbuff_t* child)
 {
-	g_assert(parent->initialized);
-	g_assert(child->initialized);
-	g_assert(child->type == TVBUFF_REAL_DATA);
+	DISSECTOR_ASSERT(parent->initialized);
+	DISSECTOR_ASSERT(child->initialized);
+	DISSECTOR_ASSERT(child->type == TVBUFF_REAL_DATA);
 	add_to_used_in_list(parent, child);
 }
 
 void
 tvb_set_real_data(tvbuff_t* tvb, const guint8* data, guint length, gint reported_length)
 {
-	g_assert(tvb->type == TVBUFF_REAL_DATA);
-	g_assert(!tvb->initialized);
+	DISSECTOR_ASSERT(tvb->type == TVBUFF_REAL_DATA);
+	DISSECTOR_ASSERT(!tvb->initialized);
 
 	if (reported_length < -1) {
 		THROW(ReportedBoundsError);
@@ -315,8 +315,8 @@ static gboolean
 compute_offset_length(tvbuff_t *tvb, gint offset, gint length,
 		guint *offset_ptr, guint *length_ptr, int *exception)
 {
-	g_assert(offset_ptr);
-	g_assert(length_ptr);
+	DISSECTOR_ASSERT(offset_ptr);
+	DISSECTOR_ASSERT(length_ptr);
 
 	/* Compute the offset */
 	if (offset >= 0) {
@@ -381,7 +381,8 @@ check_offset_length_no_exception(tvbuff_t *tvb, gint offset, gint length,
 {
 	guint	end_offset;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (!compute_offset_length(tvb, offset, length, offset_ptr, length_ptr, exception)) {
 		return FALSE;
@@ -423,7 +424,7 @@ check_offset_length_no_exception(tvbuff_t *tvb, gint offset, gint length,
 		return FALSE;
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 }
 
 /* Checks (+/-) offset and length and throws an exception if
@@ -436,7 +437,7 @@ check_offset_length(tvbuff_t *tvb, gint offset, gint length,
 	int exception = 0;
 
 	if (!check_offset_length_no_exception(tvb, offset, length, offset_ptr, length_ptr, &exception)) {
-		g_assert(exception > 0);
+		DISSECTOR_ASSERT(exception > 0);
 		THROW(exception);
 	}
 	return;
@@ -447,8 +448,8 @@ void
 tvb_set_subset(tvbuff_t *tvb, tvbuff_t *backing,
 		gint backing_offset, gint backing_length, gint reported_length)
 {
-	g_assert(tvb->type == TVBUFF_SUBSET);
-	g_assert(!tvb->initialized);
+	DISSECTOR_ASSERT(tvb->type == TVBUFF_SUBSET);
+	DISSECTOR_ASSERT(!tvb->initialized);
 
 	if (reported_length < -1) {
 		THROW(ReportedBoundsError);
@@ -513,7 +514,7 @@ tvb_composite_append(tvbuff_t* tvb, tvbuff_t* member)
 {
 	tvb_comp_t	*composite;
 
-	g_assert(!tvb->initialized);
+	DISSECTOR_ASSERT(!tvb->initialized);
 	composite = &tvb->tvbuffs.composite;
 	composite->tvbs = g_slist_append( composite->tvbs, member );
 	add_to_used_in_list(member, tvb);
@@ -524,7 +525,7 @@ tvb_composite_prepend(tvbuff_t* tvb, tvbuff_t* member)
 {
 	tvb_comp_t	*composite;
 
-	g_assert(!tvb->initialized);
+	DISSECTOR_ASSERT(!tvb->initialized);
 	composite = &tvb->tvbuffs.composite;
 	composite->tvbs = g_slist_prepend( composite->tvbs, member );
 	add_to_used_in_list(member, tvb);
@@ -545,8 +546,8 @@ tvb_composite_finalize(tvbuff_t* tvb)
 	tvb_comp_t	*composite;
 	int		i = 0;
 
-	g_assert(!tvb->initialized);
-	g_assert(tvb->length == 0);
+	DISSECTOR_ASSERT(!tvb->initialized);
+	DISSECTOR_ASSERT(tvb->length == 0);
 
 	composite = &tvb->tvbuffs.composite;
 	num_members = g_slist_length(composite->tvbs);
@@ -555,7 +556,7 @@ tvb_composite_finalize(tvbuff_t* tvb)
 	composite->end_offsets = g_new(guint, num_members);
 
 	for (slist = composite->tvbs; slist != NULL; slist = slist->next) {
-		g_assert((guint) i < num_members);
+		DISSECTOR_ASSERT((guint) i < num_members);
 		member_tvb = slist->data;
 		composite->start_offsets[i] = tvb->length;
 		tvb->length += member_tvb->length;
@@ -571,7 +572,7 @@ tvb_composite_finalize(tvbuff_t* tvb)
 guint
 tvb_length(tvbuff_t* tvb)
 {
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	return tvb->length;
 }
@@ -581,7 +582,7 @@ tvb_length_remaining(tvbuff_t *tvb, gint offset)
 {
 	guint	abs_offset, abs_length;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (compute_offset_length(tvb, offset, -1, &abs_offset, &abs_length, NULL)) {
 		return abs_length;
@@ -597,7 +598,7 @@ tvb_ensure_length_remaining(tvbuff_t *tvb, gint offset)
 	guint	abs_offset, abs_length;
 	int	exception;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (!compute_offset_length(tvb, offset, -1, &abs_offset, &abs_length, &exception)) {
 		THROW(exception);
@@ -626,7 +627,7 @@ tvb_bytes_exist(tvbuff_t *tvb, gint offset, gint length)
 {
 	guint		abs_offset, abs_length;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (!compute_offset_length(tvb, offset, length, &abs_offset, &abs_length, NULL))
 		return FALSE;
@@ -646,7 +647,7 @@ tvb_ensure_bytes_exist(tvbuff_t *tvb, gint offset, gint length)
 {
 	guint		abs_offset, abs_length;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	/*
 	 * -1 doesn't mean "until end of buffer", as that's pointless
@@ -669,7 +670,7 @@ tvb_offset_exists(tvbuff_t *tvb, gint offset)
 {
 	guint		abs_offset, abs_length;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 	if (!compute_offset_length(tvb, offset, -1, &abs_offset, &abs_length, NULL))
 		return FALSE;
 
@@ -684,7 +685,7 @@ tvb_offset_exists(tvbuff_t *tvb, gint offset)
 guint
 tvb_reported_length(tvbuff_t* tvb)
 {
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	return tvb->reported_length;
 }
@@ -694,7 +695,7 @@ tvb_reported_length_remaining(tvbuff_t *tvb, gint offset)
 {
 	guint	abs_offset, abs_length;
 
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (compute_offset_length(tvb, offset, -1, &abs_offset, &abs_length, NULL)) {
 		if (tvb->reported_length >= abs_offset)
@@ -716,7 +717,7 @@ tvb_reported_length_remaining(tvbuff_t *tvb, gint offset)
 void
 tvb_set_reported_length(tvbuff_t* tvb, guint reported_length)
 {
-	g_assert(tvb->initialized);
+	DISSECTOR_ASSERT(tvb->initialized);
 
 	if (reported_length > tvb->reported_length)
 		THROW(ReportedBoundsError);
@@ -743,7 +744,7 @@ first_real_data_ptr(tvbuff_t *tvb)
 			return first_real_data_ptr(member);
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return NULL;
 }
 
@@ -763,7 +764,7 @@ offset_from_real_beginning(tvbuff_t *tvb, int counter)
 			return offset_from_real_beginning(member, counter);
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return 0;
 }
 
@@ -777,7 +778,7 @@ composite_ensure_contiguous_no_exception(tvbuff_t *tvb, guint abs_offset,
 	guint		member_offset, member_length;
 	GSList		*slist;
 
-	g_assert(tvb->type == TVBUFF_COMPOSITE);
+	DISSECTOR_ASSERT(tvb->type == TVBUFF_COMPOSITE);
 
 	/* Maybe the range specified by offset/length
 	 * is contiguous inside one of the member tvbuffs */
@@ -791,7 +792,7 @@ composite_ensure_contiguous_no_exception(tvbuff_t *tvb, guint abs_offset,
 			break;
 		}
 	}
-	g_assert(member_tvb);
+	DISSECTOR_ASSERT(member_tvb);
 
 	if (check_offset_length_no_exception(member_tvb, abs_offset - composite->start_offsets[i],
 				abs_length, &member_offset, &member_length, NULL)) {
@@ -799,7 +800,7 @@ composite_ensure_contiguous_no_exception(tvbuff_t *tvb, guint abs_offset,
 		/*
 		 * The range is, in fact, contiguous within member_tvb.
 		 */
-		g_assert(!tvb->real_data);
+		DISSECTOR_ASSERT(!tvb->real_data);
 		return ensure_contiguous_no_exception(member_tvb, member_offset, member_length, NULL);
 	}
 	else {
@@ -807,7 +808,7 @@ composite_ensure_contiguous_no_exception(tvbuff_t *tvb, guint abs_offset,
 		return tvb->real_data + abs_offset;
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return NULL;
 }
 
@@ -832,7 +833,7 @@ ensure_contiguous_no_exception(tvbuff_t *tvb, gint offset, gint length,
 	else {
 		switch(tvb->type) {
 			case TVBUFF_REAL_DATA:
-				g_assert_not_reached();
+				DISSECTOR_ASSERT_NOT_REACHED();
 			case TVBUFF_SUBSET:
 				return ensure_contiguous_no_exception(tvb->tvbuffs.subset.tvb,
 						abs_offset - tvb->tvbuffs.subset.offset,
@@ -842,7 +843,7 @@ ensure_contiguous_no_exception(tvbuff_t *tvb, gint offset, gint length,
 		}
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return NULL;
 }
 
@@ -854,7 +855,7 @@ ensure_contiguous(tvbuff_t *tvb, gint offset, gint length)
 
 	p = ensure_contiguous_no_exception(tvb, offset, length, &exception);
 	if (p == NULL) {
-		g_assert(exception > 0);
+		DISSECTOR_ASSERT(exception > 0);
 		THROW(exception);
 	}
 	return p;
@@ -909,7 +910,7 @@ composite_memcpy(tvbuff_t *tvb, guint8* target, guint abs_offset, guint abs_leng
 	gboolean	retval;
 	GSList		*slist;
 
-	g_assert(tvb->type == TVBUFF_COMPOSITE);
+	DISSECTOR_ASSERT(tvb->type == TVBUFF_COMPOSITE);
 
 	/* Maybe the range specified by offset/length
 	 * is contiguous inside one of the member tvbuffs */
@@ -923,12 +924,12 @@ composite_memcpy(tvbuff_t *tvb, guint8* target, guint abs_offset, guint abs_leng
 			break;
 		}
 	}
-	g_assert(member_tvb);
+	DISSECTOR_ASSERT(member_tvb);
 
 	if (check_offset_length_no_exception(member_tvb, abs_offset - composite->start_offsets[i],
 				abs_length, &member_offset, &member_length, NULL)) {
 
-		g_assert(!tvb->real_data);
+		DISSECTOR_ASSERT(!tvb->real_data);
 		return tvb_memcpy(member_tvb, target, member_offset, member_length);
 	}
 	else {
@@ -939,7 +940,7 @@ composite_memcpy(tvbuff_t *tvb, guint8* target, guint abs_offset, guint abs_leng
 		 */
 		retval = compute_offset_length(member_tvb, abs_offset - composite->start_offsets[i], -1,
 				&member_offset, &member_length, NULL);
-		g_assert(retval);
+		DISSECTOR_ASSERT(retval);
 
 		tvb_memcpy(member_tvb, target, member_offset, member_length);
 		abs_offset	+= member_length;
@@ -953,7 +954,7 @@ composite_memcpy(tvbuff_t *tvb, guint8* target, guint abs_offset, guint abs_leng
 		return target;
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return NULL;
 }
 
@@ -962,7 +963,7 @@ tvb_memcpy(tvbuff_t *tvb, guint8* target, gint offset, gint length)
 {
 	guint	abs_offset, abs_length;
 
-	g_assert(length >= -1);
+	DISSECTOR_ASSERT(length >= -1);
 	check_offset_length(tvb, offset, length, &abs_offset, &abs_length);
 
 	if (tvb->real_data) {
@@ -971,7 +972,7 @@ tvb_memcpy(tvbuff_t *tvb, guint8* target, gint offset, gint length)
 
 	switch(tvb->type) {
 		case TVBUFF_REAL_DATA:
-			g_assert_not_reached();
+			DISSECTOR_ASSERT_NOT_REACHED();
 
 		case TVBUFF_SUBSET:
 			return tvb_memcpy(tvb->tvbuffs.subset.tvb, target,
@@ -982,7 +983,7 @@ tvb_memcpy(tvbuff_t *tvb, guint8* target, gint offset, gint length)
 			return composite_memcpy(tvb, target, offset, length);
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return NULL;
 }
 
@@ -1386,6 +1387,61 @@ tvb_get_letohieee_double(tvbuff_t *tvb, int offset)
 #endif
 }
 
+/* Fetch an IPv4 address, in network byte order.
+ * We do *not* convert them to host byte order; we leave them in
+ * network byte order. */
+guint32
+tvb_get_ipv4(tvbuff_t *tvb, gint offset)
+{
+	const guint8* ptr;
+	guint32 addr;
+
+	ptr = ensure_contiguous(tvb, offset, sizeof(guint32));
+	memcpy(&addr, ptr, sizeof addr);
+	return addr;
+}
+
+/* Fetch an IPv6 address. */
+void
+tvb_get_ipv6(tvbuff_t *tvb, gint offset, struct e_in6_addr *addr)
+{
+	const guint8* ptr;
+
+	ptr = ensure_contiguous(tvb, offset, sizeof(*addr));
+	memcpy(addr, ptr, sizeof *addr);
+}
+
+/* Fetch a GUID. */
+void
+tvb_get_ntohguid(tvbuff_t *tvb, gint offset, e_guid_t *guid)
+{
+	ensure_contiguous(tvb, offset, sizeof(*guid));
+	guid->data1 = tvb_get_ntohl(tvb, offset);
+	guid->data2 = tvb_get_ntohs(tvb, offset + 4);
+	guid->data3 = tvb_get_ntohs(tvb, offset + 6);
+	tvb_memcpy(tvb, guid->data4, offset + 8, sizeof guid->data4);
+}
+
+void
+tvb_get_letohguid(tvbuff_t *tvb, gint offset, e_guid_t *guid)
+{
+	ensure_contiguous(tvb, offset, sizeof(*guid));
+	guid->data1 = tvb_get_letohl(tvb, offset);
+	guid->data2 = tvb_get_letohs(tvb, offset + 4);
+	guid->data3 = tvb_get_letohs(tvb, offset + 6);
+	tvb_memcpy(tvb, guid->data4, offset + 8, sizeof guid->data4);
+}
+
+void
+tvb_get_guid(tvbuff_t *tvb, gint offset, e_guid_t *guid, gboolean little_endian)
+{
+	if (little_endian) {
+		tvb_get_letohguid(tvb, offset, guid);
+	} else {
+		tvb_get_ntohguid(tvb, offset, guid);
+	}
+}
+
 /* Find first occurence of needle in tvbuff, starting at offset. Searches
  * at most maxlength number of bytes; if maxlength is -1, searches to
  * end of tvbuff.
@@ -1433,7 +1489,7 @@ tvb_find_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 needle)
 
 	switch(tvb->type) {
 		case TVBUFF_REAL_DATA:
-			g_assert_not_reached();
+			DISSECTOR_ASSERT_NOT_REACHED();
 
 		case TVBUFF_SUBSET:
 			return tvb_find_guint8(tvb->tvbuffs.subset.tvb,
@@ -1441,11 +1497,11 @@ tvb_find_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 needle)
 					limit, needle);
 
 		case TVBUFF_COMPOSITE:
-			g_assert_not_reached();
+			DISSECTOR_ASSERT_NOT_REACHED();
 			/* XXX - return composite_find_guint8(tvb, offset, limit, needle); */
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return -1;
 }
 
@@ -1496,7 +1552,7 @@ tvb_pbrk_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 *needles)
 
 	switch(tvb->type) {
 		case TVBUFF_REAL_DATA:
-			g_assert_not_reached();
+			DISSECTOR_ASSERT_NOT_REACHED();
 
 		case TVBUFF_SUBSET:
 			return tvb_pbrk_guint8(tvb->tvbuffs.subset.tvb,
@@ -1504,11 +1560,11 @@ tvb_pbrk_guint8(tvbuff_t *tvb, gint offset, gint maxlength, guint8 *needles)
 					limit, needles);
 
 		case TVBUFF_COMPOSITE:
-			g_assert_not_reached();
+			DISSECTOR_ASSERT_NOT_REACHED();
 			/* XXX - return composite_pbrk_guint8(tvb, offset, limit, needle); */
 	}
 
-	g_assert_not_reached();
+	DISSECTOR_ASSERT_NOT_REACHED();
 	return -1;
 }
 
@@ -1683,6 +1739,41 @@ tvb_fake_unicode(tvbuff_t *tvb, int offset, int len, gboolean little_endian)
 	return buffer;
 }
 
+/* Convert a string from Unicode to ASCII.  At the moment we fake it by
+ * replacing all non-ASCII characters with a '.' )-:   The len parameter is 
+ * the number of guint16's to convert from Unicode.
+ *
+ * This function allocates memory from a buffer with packet lifetime.
+ * You do not have to free this buffer, it will be automatically freed
+ * when ethereal starts decoding the next packet.
+ */
+char *
+tvb_get_ephemeral_faked_unicode(tvbuff_t *tvb, int offset, int len, gboolean little_endian)
+{
+	char *buffer;
+	int i;
+	guint16 character;
+
+	/* Make sure we have enough data before allocating the buffer,
+	   so we don't blow up if the length is huge. */
+	tvb_ensure_bytes_exist(tvb, offset, 2*len);
+
+	/* We know we won't throw an exception, so we don't have to worry
+	   about leaking this buffer. */
+	buffer = ep_alloc(len + 1);
+
+	for (i = 0; i < len; i++) {
+		character = little_endian ? tvb_get_letohs(tvb, offset)
+					  : tvb_get_ntohs(tvb, offset);
+		buffer[i] = character < 256 ? character : '.';
+		offset += 2;
+	}
+
+	buffer[len] = 0;
+
+	return buffer;
+}
+
 /*
  * Format the data in the tvb from offset for length ...
  */
@@ -1767,7 +1858,7 @@ tvb_get_string(tvbuff_t *tvb, gint offset, gint length)
  * after the current packet has been dissected.
  */
 guint8 *
-ep_tvb_get_string(tvbuff_t *tvb, gint offset, gint length)
+tvb_get_ephemeral_string(tvbuff_t *tvb, gint offset, gint length)
 {
 	const guint8 *ptr;
 	guint8 *strbuf = NULL;
@@ -1804,12 +1895,38 @@ tvb_get_stringz(tvbuff_t *tvb, gint offset, gint *lengthp)
 	*lengthp = size;
 	return strptr;
 }
+/*
+ * Given a tvbuff and an offset, with the offset assumed to refer to
+ * a null-terminated string, find the length of that string (and throw
+ * an exception if the tvbuff ends before we find the null), allocate
+ * a buffer big enough to hold the string, copy the string into it,
+ * and return a pointer to the string.  Also return the length of the
+ * string (including the terminating null) through a pointer.
+ *
+ * This function allocates memory from a buffer with packet lifetime.
+ * You do not have to free this buffer, it will be automatically freed
+ * when ethereal starts decoding the next packet.
+ * Do not use this function if you want the allocated memory to be persistent
+ * after the current packet has been dissected.
+ */
+guint8 *
+tvb_get_ephemeral_stringz(tvbuff_t *tvb, gint offset, gint *lengthp)
+{
+	guint size;
+	guint8 *strptr;
+
+	size = tvb_strsize(tvb, offset);
+	strptr = ep_alloc(size);
+	tvb_memcpy(tvb, strptr, offset, size);
+	*lengthp = size;
+	return strptr;
+}
 
 /* Looks for a stringz (NUL-terminated string) in tvbuff and copies
  * no more than bufsize number of bytes, including terminating NUL, to buffer.
  * Returns length of string (not including terminating NUL), or -1 if the string was
  * truncated in the buffer due to not having reached the terminating NUL.
- * In this way, it acts like snprintf().
+ * In this way, it acts like g_snprintf().
  *
  * bufsize MUST be greater than 0.
  *
@@ -1835,7 +1952,7 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer,
 	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
 
 	/* There must at least be room for the terminating NUL. */
-	g_assert(bufsize != 0);
+	DISSECTOR_ASSERT(bufsize != 0);
 
 	/* If there's no room for anything else, just return the NUL. */
 	if (bufsize == 1) {
@@ -1856,13 +1973,13 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer,
 	/* This should not happen because check_offset_length() would
 	 * have already thrown an exception if 'offset' were out-of-bounds.
 	 */
-	g_assert(len != -1);
+	DISSECTOR_ASSERT(len != -1);
 
 	/*
 	 * If we've been passed a negative number, bufsize will
 	 * be huge.
 	 */
-	g_assert(bufsize <= G_MAXINT);
+	DISSECTOR_ASSERT(bufsize <= G_MAXINT);
 
 	if ((guint)len < bufsize) {
 		limit = len;
@@ -1898,7 +2015,7 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer,
  * no more than bufsize number of bytes, including terminating NUL, to buffer.
  * Returns length of string (not including terminating NUL), or -1 if the string was
  * truncated in the buffer due to not having reached the terminating NUL.
- * In this way, it acts like snprintf().
+ * In this way, it acts like g_snprintf().
  *
  * When processing a packet where the remaining number of bytes is less
  * than bufsize, an exception is not thrown if the end of the packet
@@ -2331,6 +2448,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 	err = inflateInit2(strm, wbits);
 	inits_done = 1;
 	if (err != Z_OK) {
+		inflateEnd(strm);
 		g_free(strm);
 		g_free(compr);
 		g_free(strmbuf);
@@ -2358,6 +2476,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 				    bytes_pass);
 
 				if (new_data == NULL) {
+					inflateEnd(strm);
 					g_free(strm);
 					g_free(strmbuf);
 					g_free(compr);
@@ -2391,7 +2510,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 			 * to decompress this fully, so return what we've done
 			 * so far, if any.
 			 */
-
+			inflateEnd(strm);
 			g_free(strm);
 			g_free(strmbuf);
 
@@ -2425,6 +2544,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 			if (*c == Z_DEFLATED) {
 				c++;
 			} else {
+				inflateEnd(strm);
 				g_free(strm);
 				g_free(compr);
 				g_free(strmbuf);
@@ -2469,6 +2589,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 			next = c;
 			strm->next_in = next;
 			if (c - compr > comprlen) {
+				inflateEnd(strm);
 				g_free(strm);
 				g_free(compr);
 				g_free(strmbuf);
@@ -2476,6 +2597,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 			}
 			comprlen -= (c - compr);
 			
+			inflateEnd(strm);
 			err = inflateInit2(strm, wbits);
 			inits_done++;
 		} else if (err == Z_DATA_ERROR && uncompr == NULL &&
@@ -2512,6 +2634,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 				return NULL;
 			}
 		} else {
+			inflateEnd(strm);
 			g_free(strm);
 			g_free(strmbuf);
 			g_free(compr);
