@@ -1,7 +1,7 @@
 /* globals.h
  * Global defines, etc.
  *
- * $Id: globals.h,v 1.33 2004/03/18 19:04:31 obiot Exp $
+ * $Id: globals.h 17267 2006-02-12 03:15:29Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,9 +29,8 @@
 #include <epan/timestamp.h>
 
 extern capture_file cfile;
-extern gchar       *ethereal_path;
-extern gchar       *last_open_dir;
 #ifdef HAVE_LIBPCAP
+/** @todo move this to the gtk dir */
 extern gboolean     auto_scroll_live;
 #endif
 
