@@ -1,6 +1,6 @@
 /* about_dlg.c
  *
- * $Id: about_dlg.c 13459 2005-02-21 19:51:13Z ulfl $
+ * $Id: about_dlg.c 17267 2006-02-12 03:15:29Z guy $
  *
  * Ulf Lamping <ulf.lamping@web.de>
  *
@@ -32,7 +32,7 @@
 #include <epan/filesystem.h>
 #include <epan/plugins.h>
 #include "about_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "compat_macros.h"
 #include "globals.h"
@@ -40,16 +40,11 @@
 #include "text_page.h"
 #endif
 
-#include "svnversion.h"
-
 #include "../image/eicon3d64.xpm"
 #include "gtkglobals.h"
-
-extern GString *comp_info_str, *runtime_info_str;
-
-#ifdef HAVE_PLUGINS
-extern GtkWidget *about_plugins_page_new(void);
-#endif
+#include "version_info.h"
+#include "main.h"
+#include "plugins_dlg.h"
 
 static void about_ethereal_destroy_cb(GtkWidget *, gpointer);
 
@@ -156,18 +151,19 @@ about_ethereal_page_new(void)
 
   /* Construct the message string */
   message = g_strdup_printf(
-       "Version " VERSION
-#ifdef SVNVERSION
-       " (" SVNVERSION ")"
-#endif
-       " (C) 1998-2005 Gerald Combs <gerald@ethereal.com>\n\n"
-       "%s\n\n"
-       "%s\n\n"
-
-       "Ethereal is Open Source Software released under the GNU General Public License.\n\n"
-
+       "Version " VERSION "%s\n"
+       "\n"
+       "%s"
+       "\n"
+       "%s"
+       "\n"
+       "%s"
+       "\n"
+       "Ethereal is Open Source Software released under the GNU General Public License.\n"
+       "\n"
        "Check the man page and http://www.ethereal.com for more information.",
-       comp_info_str->str, runtime_info_str->str);
+       svnversion, get_copyright_info(), comp_info_str->str,
+       runtime_info_str->str);
 
   msg_label = gtk_label_new(message);
   g_free(message);
@@ -207,7 +203,7 @@ about_folders_page_new(void)
   GtkWidget   *table;
   const char *constpath;
   char *path;
-  gchar *titles[] = { "Name", "Folder", "Typical Files"};
+  const gchar *titles[] = { "Name", "Folder", "Typical Files"};
   GtkWidget *scrolledwindow;
 
   scrolledwindow = scrolled_window_new(NULL, NULL);
@@ -246,11 +242,9 @@ about_folders_page_new(void)
       "\"ethers\", \"ipxnets\"");
 
   /* program */
-  path = g_strdup(ethereal_path);
-  path = get_dirname(path);
-  about_folders_row(table, "Program", path,
+  constpath = get_progfile_dir();
+  about_folders_row(table, "Program", constpath,
       "program files");
-  g_free((void *) path);
 
 #ifdef HAVE_PLUGINS
   /* pers plugins */
