@@ -1,7 +1,7 @@
 /* bootp_stat.c
  * boop_stat   2003 Jean-Michel FAYARD
  *
- * $Id: bootp_stat.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: bootp_stat.c 17261 2006-02-12 00:54:56Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,16 +33,15 @@
 #include <epan/packet_info.h>
 #include <epan/epan.h>
 
-#include "tap_menu.h"
 #include "simple_dialog.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "../globals.h"
 #include "compat_macros.h"
+#include "../stat_menu.h"
 #include "../tap_dfilter_dlg.h"
-#include "tap_dfilter_dlg.h"
 
 typedef const char* bootp_info_value_t;
 
@@ -191,16 +190,16 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
 /* When called, this function will create a new instance of gtk2-dhcpstat.
  */
 static void
-dhcpstat_init(char *optarg)
+dhcpstat_init(const char *optarg, void *userdata _U_)
 {
 	dhcpstat_t *sp;
-	char 		*filter=NULL;
+	const char	*filter=NULL;
 	char 		*title=NULL;
 	GString		*error_string;
 	GtkWidget	*message_type_fr;
-    GtkWidget	*vbox;
-    GtkWidget	*bt_close;
-    GtkWidget	*bbox;
+	GtkWidget	*vbox;
+	GtkWidget	*bt_close;
+	GtkWidget	*bbox;
 	
 	if (strncmp (optarg, "bootp,stat,", 11) == 0){
 		filter=optarg+11;
@@ -222,9 +221,9 @@ dhcpstat_init(char *optarg)
 	sp->win = window_new(GTK_WINDOW_TOPLEVEL, title);
 	g_free(title);
 
-    vbox = gtk_vbox_new(FALSE, 3);
+	vbox = gtk_vbox_new(FALSE, 3);
 	gtk_container_add(GTK_CONTAINER(sp->win), vbox);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 
 	/* Status Codes frame */
 	message_type_fr = gtk_frame_new("DHCP Message Type");
@@ -255,20 +254,20 @@ dhcpstat_init(char *optarg)
 	}
 
 	/* Button row. */
-    bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
-    gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
+	bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
-    bt_close = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
-    window_set_cancel_button(sp->win, bt_close, window_cancel_button_cb);
+	bt_close = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+	window_set_cancel_button(sp->win, bt_close, window_cancel_button_cb);
 
-    SIGNAL_CONNECT(sp->win, "delete_event", window_delete_event_cb, NULL);
-    SIGNAL_CONNECT(sp->win, "destroy", win_destroy_cb, sp);
+	SIGNAL_CONNECT(sp->win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(sp->win, "destroy", win_destroy_cb, sp);
 
-    gtk_widget_show_all(sp->win);
+	gtk_widget_show_all(sp->win);
 
-    window_present(sp->win);
+	window_present(sp->win);
 
-    cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 }
 
 static tap_dfilter_dlg dhcp_stat_dlg = {
@@ -281,8 +280,6 @@ static tap_dfilter_dlg dhcp_stat_dlg = {
 void
 register_tap_listener_gtkdhcpstat(void)
 {
-	register_ethereal_tap("bootp,stat", dhcpstat_init);
-
-	register_tap_menu_item("BOOTP-DHCP", REGISTER_TAP_GROUP_NONE,
-	    gtk_tap_dfilter_dlg_cb, NULL, NULL, &(dhcp_stat_dlg));
+	register_dfilter_stat(&dhcp_stat_dlg, "BOOTP-DHCP",
+	    REGISTER_STAT_GROUP_NONE);
 }
