@@ -1,7 +1,7 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c 13320 2005-02-06 21:20:35Z ulfl $
+ * $Id: capture_dlg.c 17792 2006-04-03 03:12:32Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,22 +28,24 @@
 
 #ifdef HAVE_LIBPCAP
 
-#include <pcap.h>
 #include <string.h>
 #include <gtk/gtk.h>
+#include <stdio.h>        /* menu.h needs FILE */
+#include <gtk/menu.h>
 
 #include <epan/packet.h>
 #include "capture.h"
 #include "globals.h"
 #include <epan/addr_resolv.h>
 #include "main.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "capture.h"
 #include "capture_dlg.h"
+#include "capture_errs.h"
 #include "filter_dlg.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "pcap-util.h"
+#include "capture-pcap-util.h"
 #include "capture_ui_utils.h"
 #include <epan/prefs.h>
 #include "ringbuffer.h"
@@ -102,6 +104,16 @@
 
 #define E_CAP_OM_LT_VALUE_KEY       "cap_om_lt_value"
 
+
+/*
+ * Keep a static pointer to the current "Capture Options" window, if
+ * any, so that if somebody tries to do "Capture:Start" while there's
+ * already a "Capture Options" window up, we just pop up the existing
+ * one, rather than creating a new one.
+ */
+static GtkWidget *cap_open_w;
+
+
 static void
 capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te);
 
@@ -111,41 +123,44 @@ select_link_type_cb(GtkWidget *w, gpointer data);
 static void
 capture_prep_adjust_sensitivity(GtkWidget *tb, gpointer parent_w);
 
-static void
-capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w);
-
 static void
 capture_prep_destroy_cb(GtkWidget *win, gpointer user_data);
 
 static void
 capture_prep_interface_changed_cb(GtkWidget *entry, gpointer parent_w);
 
+static void
+capture_dlg_prep(gpointer parent_w);
+
+
+/* stop the currently running capture */
 void
 capture_stop_cb(GtkWidget *w _U_, gpointer d _U_)
 {
     capture_stop(capture_opts);
 }
 
-/*
- * Keep a static pointer to the current "Capture Options" window, if
- * any, so that if somebody tries to do "Capture:Start" while there's
- * already a "Capture Options" window up, we just pop up the existing
- * one, rather than creating a new one.
- */
-static GtkWidget *cap_open_w;
+/* restart (stop - delete old file - start) running capture */
+void
+capture_restart_cb(GtkWidget *w _U_, gpointer d _U_)
+{
+    capture_restart(capture_opts);
+}
 
+/* init the link type list */
+/* (often this list has only one entry and will therefore be disabled) */
 static void
 set_link_type_list(GtkWidget *linktype_om, GtkWidget *entry)
 {
   gchar *entry_text;
   gchar *if_text;
-  gchar *if_name;
+  const gchar *if_name;
   GList *if_list;
   GList *if_entry;
   if_info_t *if_info;
   GList *lt_list;
   int err;
-  char err_buf[PCAP_ERRBUF_SIZE];
+  char err_buf[CAPTURE_PCAP_ERRBUF_SIZE];
   GtkWidget *lt_menu, *lt_menu_item;
   GList *lt_entry;
   data_link_info_t *data_link_info;
@@ -265,13 +280,15 @@ set_link_type_list(GtkWidget *linktype_om, GtkWidget *entry)
 #define TIME_UNIT_HOUR      2
 #define TIME_UNIT_DAY       3
 #define MAX_TIME_UNITS 4
-static char *time_unit_name[MAX_TIME_UNITS] = {
+static const char *time_unit_name[MAX_TIME_UNITS] = {
 	"second(s)",
 	"minute(s)",
 	"hour(s)",
 	"day(s)",
 };
 
+/* create one of the duration options */
+/* (and select the matching unit depending on the given value) */
 static GtkWidget *time_unit_option_menu_new(guint32 value) {
     GtkWidget *unit_om, *menu, *menu_item;
     int i;
@@ -309,6 +326,7 @@ static GtkWidget *time_unit_option_menu_new(guint32 value) {
     return unit_om;
 }
 
+/* convert time value from raw to displayed (e.g. 60s -> 1min) */
 static guint32 time_unit_option_menu_convert_value(
 guint32 value)
 {
@@ -331,6 +349,7 @@ guint32 value)
     return value;
 }
 
+/* get raw value from unit and value fields */
 static guint32 time_unit_option_menu_get_value(
 GtkWidget *unit_om,
 guint32 value)
@@ -363,18 +382,18 @@ guint32 value)
 }
 
 
-#define SIZE_UNIT_BYTES     0
-#define SIZE_UNIT_KILOBYTES 1
-#define SIZE_UNIT_MEGABYTES 2
-#define SIZE_UNIT_GIGABYTES 3
-#define MAX_SIZE_UNITS 4
-static char *size_unit_name[MAX_SIZE_UNITS] = {
-	"byte(s)",
+#define SIZE_UNIT_KILOBYTES 0
+#define SIZE_UNIT_MEGABYTES 1
+#define SIZE_UNIT_GIGABYTES 2
+#define MAX_SIZE_UNITS 3
+static const char *size_unit_name[MAX_SIZE_UNITS] = {
 	"kilobyte(s)",
 	"megabyte(s)",
 	"gigabyte(s)",
 };
 
+/* create one of the size options */
+/* (and select the matching unit depending on the given value) */
 static GtkWidget *size_unit_option_menu_new(guint32 value) {
     GtkWidget *unit_om, *menu, *menu_item;
     int i;
@@ -390,20 +409,15 @@ static GtkWidget *size_unit_option_menu_new(guint32 value) {
     /* the selected menu item can't be changed, once the option_menu
        is created, so set the matching menu item now */
     /* gigabytes */
-    if(value >= 1024 * 1024 * 1024) {
+    if(value >= 1024 * 1024) {
 	    gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_GIGABYTES);
     } else {
         /* megabytes */
-        if(value >= 1024 * 1024) {
+        if(value >= 1024) {
 	        gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_MEGABYTES);
         } else {
             /* kilobytes */
-            if(value >= 1024) {
-	            gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_KILOBYTES);
-            } else {
-                /* bytes */
-	            gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_BYTES);
-            }
+            gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_KILOBYTES);
         }
     }
 
@@ -412,28 +426,25 @@ static GtkWidget *size_unit_option_menu_new(guint32 value) {
     return unit_om;
 }
 
+/* convert size value from raw to displayed (e.g. 1024 Bytes -> 1 KB) */
 static guint32 size_unit_option_menu_set_value(
 guint32 value)
 {
     /* gigabytes */
-    if(value >= 1024 * 1024 * 1024) {
-        return value / (1024 * 1024 * 1024);
-    }
-
-    /* megabytes */
     if(value >= 1024 * 1024) {
         return value / (1024 * 1024);
     }
 
-    /* kilobytes */
+    /* megabytes */
     if(value >= 1024) {
-        return value / 1024;
+        return value / (1024);
     }
 
-    /* bytes */
+    /* kilobytes */
     return value;
 }
 
+/* get raw value from unit and value fields */
 static guint32 size_unit_option_menu_convert_value(
 GtkWidget *unit_om,
 guint32 value)
@@ -447,30 +458,23 @@ guint32 value)
 
 
     switch(unit) {
-    case(SIZE_UNIT_BYTES):
+    case(SIZE_UNIT_KILOBYTES):
         return value;
         break;
-    case(SIZE_UNIT_KILOBYTES):
+    case(SIZE_UNIT_MEGABYTES):
         if(value > G_MAXINT / 1024) {
             return 0;
         } else {
             return value * 1024;
         }
         break;
-    case(SIZE_UNIT_MEGABYTES):
+    case(SIZE_UNIT_GIGABYTES):
         if(value > G_MAXINT / (1024 * 1024)) {
             return 0;
         } else {
             return value * 1024 * 1024;
         }
         break;
-    case(SIZE_UNIT_GIGABYTES):
-        if(value > G_MAXINT / (1024 * 1024 * 1024)) {
-            return 0;
-        } else {
-            return value * 1024 * 1024 * 1024;
-        }
-        break;
     default:
         g_assert_not_reached();
         return 0;
@@ -478,8 +482,9 @@ guint32 value)
 }
 
 
+/* show capture prepare (options) dialog */
 void
-capture_prep(void)
+capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
 {
   GtkWidget     *main_vb,
                 *main_hb, *left_vb, *right_vb,
@@ -518,10 +523,10 @@ capture_prep(void)
   GtkTooltips   *tooltips;
   GtkAdjustment *snap_adj, *ringbuffer_nbf_adj,
 		*stop_packets_adj, *stop_filesize_adj, *stop_duration_adj, *stop_files_adj, *ring_filesize_adj, *file_duration_adj;
-  GList         *if_list, *combo_list, *filter_list;
+  GList         *if_list, *combo_list, *cfilter_list;
   int           err;
   int           row;
-  char          err_str[PCAP_ERRBUF_SIZE];
+  char          err_str[CAPTURE_PCAP_ERRBUF_SIZE];
   gchar         *cant_get_if_list_errstr;
 #ifdef _WIN32
   GtkAdjustment *buffer_size_adj;
@@ -529,6 +534,8 @@ capture_prep(void)
 #endif
   guint32       value;
   gchar         *cap_title;
+  gchar         *if_device;
+
 
   if (cap_open_w != NULL) {
     /* There's already a "Capture Options" dialog box; reactivate it. */
@@ -539,24 +546,12 @@ capture_prep(void)
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (!has_wpcap) {
-	  simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		  "Unable to load WinPcap (wpcap.dll); Ethereal will not be able\n"
-		  "to capture packets.\n\n"
-		  "In order to capture packets, WinPcap must be installed; see\n"
-		  "\n"
-		  "        http://winpcap.polito.it/\n"
-		  "\n"
-		  "or the mirror at\n"
-		  "\n"
-		  "        http://winpcap.mirror.ethereal.com/\n"
-		  "\n"
-		  "or the mirror at\n"
-		  "\n"
-		  "        http://www.mirrors.wiretapped.net/security/packet-capture/winpcap/\n"
-		  "\n"
-		  "for a downloadable version of WinPcap and for instructions\n"
-		  "on how to install WinPcap.");
-	  return;
+    char *detailed_err;
+
+    detailed_err = cant_load_winpcap_err("Ethereal");
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", detailed_err);
+    g_free(detailed_err);
+    return;
   }
 #endif
 
@@ -611,11 +606,15 @@ capture_prep(void)
     /* No interface was specified on the command line or in a previous
        capture, but there is one specified in the preferences file;
        make the one from the preferences file the default */
-    capture_opts->iface	= g_strdup(prefs.capture_device);
+    if_device = g_strdup(prefs.capture_device);
+    capture_opts->iface = g_strdup(get_if_name(if_device));
+    g_free(if_device);
   }
-  if (capture_opts->iface != NULL)
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry), capture_opts->iface);
-  else if (combo_list != NULL) {
+  if (capture_opts->iface != NULL) {
+    if_device = build_capture_combo_name(if_list, capture_opts->iface);
+    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry), if_device);
+    g_free(if_device);
+  } else if (combo_list != NULL) {
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry),
 		       (char *)combo_list->data);
   }
@@ -735,15 +734,15 @@ capture_prep(void)
   /* Create the capture filter combo */
   filter_cm = gtk_combo_new();
 
-  filter_list = OBJECT_GET_DATA(top_level, E_CFILTER_FL_KEY);
+  cfilter_list = OBJECT_GET_DATA(top_level, E_CFILTER_FL_KEY);
   gtk_combo_disable_activate(GTK_COMBO(filter_cm));
   gtk_combo_set_case_sensitive(GTK_COMBO(filter_cm), TRUE);
-  OBJECT_SET_DATA(top_level, E_CFILTER_FL_KEY, filter_list);
+  OBJECT_SET_DATA(top_level, E_CFILTER_FL_KEY, cfilter_list);
   OBJECT_SET_DATA(top_level, E_CFILTER_CM_KEY, filter_cm);
   filter_te = GTK_COMBO(filter_cm)->entry;
 
-  if (filter_list != NULL)
-    gtk_combo_set_popdown_strings(GTK_COMBO(filter_cm), filter_list);
+  if (cfilter_list != NULL)
+    gtk_combo_set_popdown_strings(GTK_COMBO(filter_cm), cfilter_list);
   if (capture_opts->cfilter)
     gtk_entry_set_text(GTK_ENTRY(filter_te), capture_opts->cfilter);
   gtk_tooltips_set_tip(tooltips, filter_te,
@@ -823,7 +822,8 @@ capture_prep(void)
 		capture_opts->has_autostop_filesize);
   SIGNAL_CONNECT(ring_filesize_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
   gtk_tooltips_set_tip(tooltips, ring_filesize_cb,
-    "If the selected file size is exceeded, capturing switches to the next file.",
+    "If the selected file size is exceeded, capturing switches to the next file.\n"
+    "PLEASE NOTE: at least one of the \"Next file every\" options MUST be selected.",
     NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ring_filesize_cb, 0, 1, row, row+1);
 
@@ -849,7 +849,8 @@ capture_prep(void)
   SIGNAL_CONNECT(file_duration_cb, "toggled",
 		 capture_prep_adjust_sensitivity, cap_open_w);
   gtk_tooltips_set_tip(tooltips, file_duration_cb,
-    "If the selected duration is exceeded, capturing switches to the next file.",
+    "If the selected duration is exceeded, capturing switches to the next file.\n"
+    "PLEASE NOTE: at least one of the \"Next file every\" options MUST be selected.",
     NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), file_duration_cb, 0, 1, row, row+1);
 
@@ -1007,7 +1008,7 @@ capture_prep(void)
   sync_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
       "_Update list of packets in real time", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sync_cb),
-		capture_opts->sync_mode);
+		capture_opts->real_time_mode);
   SIGNAL_CONNECT(sync_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
   gtk_tooltips_set_tip(tooltips, sync_cb,
     "Using this option will show the captured packets immediately on the main screen. "
@@ -1063,12 +1064,12 @@ capture_prep(void)
     "Perform transport layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), t_resolv_cb);
 
-  /* Button row: OK and cancel buttons */
-  bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_CANCEL, GTK_STOCK_HELP, NULL);
+  /* Button row: "Start" and "Cancel" buttons */
+  bbox = dlg_button_row_new(ETHEREAL_STOCK_CAPTURE_START, GTK_STOCK_CANCEL, GTK_STOCK_HELP, NULL);
   gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 5);
 
-  ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
-  SIGNAL_CONNECT(ok_bt, "clicked", capture_prep_ok_cb, cap_open_w);
+  ok_bt = OBJECT_GET_DATA(bbox, ETHEREAL_STOCK_CAPTURE_START);
+  SIGNAL_CONNECT(ok_bt, "clicked", capture_start_cb, cap_open_w);
   gtk_tooltips_set_tip(tooltips, ok_bt,
     "Start the capture process.", NULL);
 
@@ -1169,8 +1170,47 @@ capture_prep(void)
   window_present(cap_open_w);
 }
 
+/* everythings prepared, now it's really time to start the capture */
+void
+capture_start_confirmed(void) {
+
+
+    /* init iface, if never used before */
+    /* XXX - would better be doing this in main.c */
+    if(capture_opts->iface == NULL) {
+        gchar *if_device;
+        const gchar *if_name;
+
+        /* did the user ever selected a capture interface before? */
+        if(prefs.capture_device == NULL) {
+            simple_dialog(ESD_TYPE_CONFIRMATION,
+                        ESD_BTN_OK,
+                        PRIMARY_TEXT_START "No capture interface selected!" PRIMARY_TEXT_END "\n\n"
+                        "To select an interface use:\n\n"
+                        "Capture->Options (until Ethereal is stopped)\n"
+                        "Edit->Preferences/Capture (permanent, if saved)");
+            return;
+        }
+        if_device = g_strdup(prefs.capture_device);
+        if_name = get_if_name(if_device);
+        capture_opts->iface = g_strdup(if_name);
+
+        g_free(if_device);
+    }
+
+    /* XXX - we might need to init other pref data as well... */
+    menu_auto_scroll_live_changed(auto_scroll_live);
+
+    if (capture_start(capture_opts)) {
+        /* The capture succeeded, which means the capture filter syntax is
+        valid; add this capture filter to the recent capture filter list. */
+        cfilter_combo_add_recent(capture_opts->cfilter);
+    }
+}
+
+/* user confirmed the "Save capture file..." dialog */
 static void
-capture_prep_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
+capture_start_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
 {
     switch(btn) {
     case(ESD_BTN_SAVE):
@@ -1178,7 +1218,9 @@ capture_prep_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
         file_save_as_cmd(after_save_capture_dialog, data);
         break;
     case(ESD_BTN_DONT_SAVE):
-        capture_prep();
+        /* XXX - unlink old file? */
+        /* start the capture */
+        capture_start_confirmed();
         break;
     case(ESD_BTN_CANCEL):
         break;
@@ -1187,23 +1229,43 @@ capture_prep_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
     }
 }
 
+/* user pressed the "Start" button (in dialog or toolbar) */
 void
-capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
+capture_start_cb(GtkWidget *w _U_, gpointer d _U_)
 {
   gpointer  dialog;
 
+#ifdef _WIN32
+  /* Is WPcap loaded? */
+  if (!has_wpcap) {
+      char * err_msg = cant_load_winpcap_err("Ethereal");
+
+	  simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+          err_msg);
+      g_free(err_msg);
+	  return;
+  }
+#endif
+
+  /* get the values and close the options dialog */
+  if(cap_open_w) {
+    capture_dlg_prep(cap_open_w);
+    window_destroy(GTK_WIDGET(cap_open_w));
+  }
+
   if((cfile.state != FILE_CLOSED) && !cfile.user_saved && prefs.gui_ask_unsaved) {
     /* user didn't saved his current file, ask him */
     dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_SAVE_DONTSAVE_CANCEL,
                 PRIMARY_TEXT_START "Save capture file before starting a new capture?" PRIMARY_TEXT_END "\n\n"
                 "If you start a new capture without saving, your current capture data will\nbe discarded.");
-    simple_dialog_set_cb(dialog, capture_prep_answered_cb, NULL);
+    simple_dialog_set_cb(dialog, capture_start_answered_cb, NULL);
   } else {
     /* unchanged file, just capture a new one */
-    capture_prep();
+    capture_start_confirmed();
   }
 }
 
+/* user selected a link type, convert to internal value */
 static void
 select_link_type_cb(GtkWidget *w, gpointer data)
 {
@@ -1215,6 +1277,7 @@ select_link_type_cb(GtkWidget *w, gpointer data)
     OBJECT_SET_DATA(linktype_om, E_CAP_OM_LT_VALUE_KEY, GINT_TO_POINTER(new_linktype));
 }
 
+/* user pressed "File" button */
 static void
 capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
 {
@@ -1222,8 +1285,9 @@ capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
 }
 
 
+/* convert dialog settings into capture_opts values */
 static void
-capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
+capture_dlg_prep(gpointer parent_w) {
   GtkWidget *if_cb, *snap_cb, *snap_sb, *promisc_cb, *filter_te, *filter_cm,
             *file_te, *multi_files_on_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_cb,
             *linktype_om, *sync_cb, *auto_scroll_cb, *hide_info_cb,
@@ -1239,7 +1303,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
 #endif
   gchar *entry_text;
   gchar *if_text;
-  gchar *if_name;
+  const gchar *if_name;
   const gchar *filter_text;
   const gchar *g_save_file;
   gchar *cf_name;
@@ -1334,6 +1398,8 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   g_assert(filter_text != NULL);
   capture_opts->cfilter = g_strdup(filter_text);
 
+  /* Ethereal always saves to a capture file. */
+  capture_opts->saving_to_file = TRUE;
   g_save_file = gtk_entry_get_text(GTK_ENTRY(file_te));
   if (g_save_file && g_save_file[0]) {
     /* User specified a file to which the capture should be written. */
@@ -1363,7 +1429,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
       time_unit_option_menu_get_value(stop_duration_om, capture_opts->autostop_duration);
   }
 
-  capture_opts->sync_mode =
+  capture_opts->real_time_mode =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sync_cb));
 
   auto_scroll_live =
@@ -1400,8 +1466,20 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   capture_opts->multi_files_on =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(multi_files_on_cb));
 
-  if(capture_opts->sync_mode)
-    capture_opts->multi_files_on = FALSE;
+  capture_opts->has_file_duration =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(file_duration_cb));
+  if (capture_opts->has_file_duration) {
+    capture_opts->file_duration =
+      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(file_duration_sb));
+    capture_opts->file_duration =
+      time_unit_option_menu_get_value(file_duration_om, capture_opts->file_duration);
+  }
+
+  capture_opts->has_autostop_files =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stop_files_cb));
+  if (capture_opts->has_autostop_files)
+    capture_opts->autostop_files =
+      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(stop_files_sb));
 
   if (capture_opts->multi_files_on) {
     capture_opts->has_autostop_filesize =
@@ -1425,10 +1503,10 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
         PRIMARY_TEXT_START "Multiple files: No capture file name given!\n\n" PRIMARY_TEXT_END
         "You must specify a filename if you want to use multiple files.");
       return;
-    } else if (!capture_opts->has_autostop_filesize) {
+    } else if (!capture_opts->has_autostop_filesize && !capture_opts->has_file_duration) {
       simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
         PRIMARY_TEXT_START "Multiple files: No file limit given!\n\n" PRIMARY_TEXT_END
-        "You must specify a file size at which is switched to the next capture file\n"
+        "You must specify a file size or duration at which is switched to the next capture file\n"
         "if you want to use multiple files.");
       g_free(capture_opts->save_file);
       capture_opts->save_file = NULL;
@@ -1449,32 +1527,10 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
         return;
       }
     }
-  }
-
-  capture_opts->has_file_duration =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(file_duration_cb));
-  if (capture_opts->has_file_duration) {
-    capture_opts->file_duration =
-      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(file_duration_sb));
-    capture_opts->file_duration =
-      time_unit_option_menu_get_value(file_duration_om, capture_opts->file_duration);
-  }
-
-  capture_opts->has_autostop_files =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stop_files_cb));
-  if (capture_opts->has_autostop_files)
-    capture_opts->autostop_files =
-      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(stop_files_sb));
-
-  window_destroy(GTK_WIDGET(parent_w));
-
-  if (do_capture(capture_opts)) {
-    /* The capture succeeded, which means the capture filter syntax is
-       valid; add this capture filter to the recent capture filter list. */
-    cfilter_combo_add_recent(capture_opts->cfilter);
-  }
+  } /* multi_files_on */
 }
 
+/* user requested to destroy the dialog */
 static void
 capture_prep_destroy_cb(GtkWidget *win, gpointer user_data _U_)
 {
@@ -1493,6 +1549,7 @@ capture_prep_destroy_cb(GtkWidget *win, gpointer user_data _U_)
   cap_open_w = NULL;
 }
 
+/* user changed the interface entry */
 static void
 capture_prep_interface_changed_cb(GtkWidget *entry, gpointer argp)
 {
@@ -1561,30 +1618,35 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
        mode off if it's on, and make its toggle button, and the spin
        button for the number of ring buffer files (and the spin button's
        label), insensitive. */
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(multi_files_on_cb), FALSE);
-    gtk_widget_set_sensitive(GTK_WIDGET(multi_files_on_cb), FALSE);
+/*    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(multi_files_on_cb), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(multi_files_on_cb), FALSE);*/
 
     /* Auto-scroll mode is meaningful only in "Update list of packets
        in real time" captures, so make its toggle button sensitive. */
     gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), TRUE);
 
-    gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), TRUE);
+    /*gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), TRUE);*/
   } else {
     /* "Update list of packets in real time" captures disabled; that
        means ring buffer mode is OK, so make its toggle button
        sensitive. */
-    gtk_widget_set_sensitive(GTK_WIDGET(multi_files_on_cb), TRUE);
+/*    gtk_widget_set_sensitive(GTK_WIDGET(multi_files_on_cb), TRUE);*/
 
     /* Auto-scroll mode is meaningful only in "Update list of packets
        in real time" captures, so make its toggle button insensitive. */
     gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), FALSE);
 
-    gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), FALSE);
+    /*gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), FALSE);*/
   }
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(multi_files_on_cb))) {
     /* Ring buffer mode enabled. */
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ring_filesize_cb), TRUE);
+
+    /* Force at least one of the "file switch" conditions (we need at least one) */
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ring_filesize_cb)) == FALSE &&
+        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(file_duration_cb)) == FALSE) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ring_filesize_cb), TRUE);
+    }
 
     gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_cb), TRUE);
     gtk_widget_set_sensitive(GTK_WIDGET(ringbuffer_nbf_sb),
