@@ -1,7 +1,7 @@
 /* capture_if_dlg.c
  * Routines for the capture interface dialog
  *
- * $Id: capture_if_dlg.c 13389 2005-02-13 00:10:15Z lroland $
+ * $Id: capture_if_dlg.c 17846 2006-04-10 17:24:14Z jake $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,21 +28,15 @@
 
 #ifdef HAVE_LIBPCAP
 
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
-
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
 
 
-#include <pcap.h>
-
 #include <gtk/gtk.h>
 
 #include "globals.h"
-#include "pcap-util.h"
+#include "capture-pcap-util.h"
 
 #ifdef _WIN32
 #include "capture-wpcap.h"
@@ -52,16 +46,16 @@
 #include "simple_dialog.h"
 #include "capture.h"
 #include "capture_dlg.h"
+#include "capture_if_details_dlg.h"
+#include "capture_errs.h"
 
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 
 #include "wtap.h"
 #include "main.h"
 #include "help_dlg.h"
-
-
-extern gboolean is_capture_in_progress(void);
+#include "toolbar.h"
 
 /*
  * Keep a static pointer to the current "Capture Interfaces" window, if
@@ -95,6 +89,9 @@ typedef struct if_dlg_data_s {
     GtkWidget   *last_lb;
     GtkWidget   *capture_bt;
     GtkWidget   *prepare_bt;
+#ifdef _WIN32
+    GtkWidget   *details_bt;
+#endif
     guint32     last_packets;
     gchar       *device;
 } if_dlg_data_t;
@@ -113,11 +110,16 @@ capture_do_cb(GtkWidget *capture_bt _U_, gpointer if_data)
 
   capture_opts->iface = g_strdup(if_dlg_data->device);
 
-  if (capture_opts->save_file)
+  /* XXX - remove this? */
+  if (capture_opts->save_file) {
     g_free(capture_opts->save_file);
-  capture_opts->save_file = NULL;
+    capture_opts->save_file = NULL;
+  }
 
-  do_capture(capture_opts);
+  /* stop capturing from all interfaces, we are going to do real work now ... */
+  window_destroy(cap_if_w);
+
+  capture_start_cb(NULL, NULL);
 }
 
 
@@ -132,15 +134,31 @@ capture_prepare_cb(GtkWidget *prepare_bt _U_, gpointer if_data)
 
   capture_opts->iface = g_strdup(if_dlg_data->device);
 
+  /* stop capturing from all interfaces, we are going to do real work now ... */
+  window_destroy(cap_if_w);
+
   capture_prep_cb(NULL, NULL);
 }
 
 
+#ifdef _WIN32
+/* capture details button was pressed */
+static void
+capture_details_cb(GtkWidget *details_bt _U_, gpointer if_data)
+{
+  if_dlg_data_t *if_dlg_data = if_data;
+
+
+  capture_if_details_open(if_dlg_data->device);
+}
+#endif
+
+
 /* open a single interface */
-void
+static void
 open_if(gchar *name, if_dlg_data_t *if_dlg_data)
 {
-  gchar       open_err_str[PCAP_ERRBUF_SIZE];
+  gchar       open_err_str[CAPTURE_PCAP_ERRBUF_SIZE];
 
   /*
    * XXX - on systems with BPF, the number of BPF devices limits the
@@ -221,7 +239,7 @@ update_if(if_dlg_data_t *if_dlg_data)
 
 
 /* close a single interface */
-void
+static void
 close_if(if_dlg_data_t *if_dlg_data)
 {
     if(if_dlg_data->pch)
@@ -231,7 +249,7 @@ close_if(if_dlg_data_t *if_dlg_data)
 
 
 /* update all interfaces */
-gboolean
+static gboolean
 update_all(gpointer data)
 {
     GList *curr;
@@ -299,7 +317,7 @@ capture_if_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 void
 capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-  GtkWidget     *main_vb, *bbox, *close_bt, *help_bt;
+  GtkWidget     *main_vb, *main_sw, *bbox, *close_bt, *help_bt;
 
   GtkWidget     *if_tb;
   GtkWidget     *if_lb;
@@ -308,9 +326,10 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
 #endif
   GtkTooltips   *tooltips;
   int           err;
-  char          err_str[PCAP_ERRBUF_SIZE];
+  char          err_str[CAPTURE_PCAP_ERRBUF_SIZE];
   gchar         *cant_get_if_list_errstr;
-  int           row;
+  GtkRequisition requisition;
+  int           row, height;
   if_dlg_data_t *if_dlg_data;
   int           ifs;
   GList         *curr;
@@ -329,24 +348,12 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (!has_wpcap) {
-	  simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		  "Unable to load WinPcap (wpcap.dll); Ethereal will not be able\n"
-		  "to capture packets.\n\n"
-		  "In order to capture packets, WinPcap must be installed; see\n"
-		  "\n"
-		  "        http://winpcap.polito.it/\n"
-		  "\n"
-		  "or the mirror at\n"
-		  "\n"
-		  "        http://winpcap.mirror.ethereal.com/\n"
-		  "\n"
-		  "or the mirror at\n"
-		  "\n"
-		  "        http://www.mirrors.wiretapped.net/security/packet-capture/winpcap/\n"
-		  "\n"
-		  "for a downloadable version of WinPcap and for instructions\n"
-		  "on how to install WinPcap.");
-	  return;
+    char *detailed_err;
+
+    detailed_err = cant_load_winpcap_err("Ethereal");
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", detailed_err);
+    g_free(detailed_err);
+    return;
   }
 #endif
 
@@ -371,17 +378,22 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   gtk_window_add_accel_group(GTK_WINDOW(cap_if_w), accel_group);
 #endif
 
+  main_sw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(main_sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_container_add(GTK_CONTAINER(cap_if_w), main_sw);
+
   main_vb = gtk_vbox_new(FALSE, 0);
   gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
-  gtk_container_add(GTK_CONTAINER(cap_if_w), main_vb);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(main_sw), main_vb);
 
 
   if_tb = gtk_table_new(6,1, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(if_tb), 3);
   gtk_table_set_col_spacings(GTK_TABLE(if_tb), 3);
-  gtk_container_add(GTK_CONTAINER(main_vb), if_tb);
+  gtk_box_pack_start(GTK_BOX(main_vb), if_tb, FALSE, FALSE, 0);
 
   row = 0;
+  height = 0;
 
 #ifndef _WIN32
   /*
@@ -408,10 +420,16 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   stop_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_STOP);
   gtk_tooltips_set_tip(tooltips, stop_bt, 
           "Stop a running capture.", NULL);
+#ifdef _WIN32
+  gtk_table_attach_defaults(GTK_TABLE(if_tb), stop_bt, 5, 8, row, row+1);
+#else
   gtk_table_attach_defaults(GTK_TABLE(if_tb), stop_bt, 5, 7, row, row+1);
+#endif
   SIGNAL_CONNECT(stop_bt, "clicked", capture_stop_cb, NULL);
 
   row++;
+  gtk_widget_size_request(stop_bt, &requisition);
+  height += requisition.height + 15;
 
   for(ifs = 0; (curr = g_list_nth(if_list, ifs)); ifs++) {
       g_string_assign(if_tool_str, "");
@@ -498,11 +516,25 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
           "Open the capture options dialog with this interface selected.", NULL);
       gtk_table_attach_defaults(GTK_TABLE(if_tb), if_dlg_data->prepare_bt, 6, 7, row, row+1);
 
+      /* details button */
+#ifdef _WIN32
+      if_dlg_data->details_bt = gtk_button_new_with_label("Details");
+      SIGNAL_CONNECT(if_dlg_data->details_bt, "clicked", capture_details_cb, if_dlg_data);
+      gtk_tooltips_set_tip(tooltips, if_dlg_data->details_bt, 
+          "Open the capture details dialog of this interface.", NULL);
+      gtk_table_attach_defaults(GTK_TABLE(if_tb), if_dlg_data->details_bt, 7, 8, row, row+1);
+#endif
+
       open_if(if_info->name, if_dlg_data);
 
       if_data = g_list_append(if_data, if_dlg_data);
 
       row++;
+      if (row <= 10) {
+          /* Lets add up 10 rows of interfaces, otherwise the window may become too high */
+          gtk_widget_size_request(GTK_WIDGET(if_dlg_data->prepare_bt), &requisition);
+          height += requisition.height;
+      }
   }
 
   g_string_free(if_tool_str, TRUE);
@@ -524,6 +556,10 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
     SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_CAPTURE_INTERFACES_DIALOG);
   }
 
+  gtk_widget_size_request(GTK_WIDGET(close_bt), &requisition);
+  height += requisition.height + 27;
+  gtk_window_set_default_size(GTK_WINDOW(cap_if_w), -1, height);
+
   gtk_widget_grab_default(close_bt);
 
   SIGNAL_CONNECT(cap_if_w, "delete_event", window_delete_event_cb, NULL);
