@@ -1,7 +1,7 @@
 /* capture_prefs.c
  * Dialog box for capture preferences
  *
- * $Id: capture_prefs.c 13373 2005-02-10 07:23:01Z guy $
+ * $Id: capture_prefs.c 17327 2006-02-17 02:18:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -28,7 +28,6 @@
 
 #ifdef HAVE_LIBPCAP
 
-#include <pcap.h>
 #include <string.h>
 #include <gtk/gtk.h>
 
@@ -37,10 +36,10 @@
 #include "gtkglobals.h"
 #include <epan/prefs.h>
 #include "prefs_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "simple_dialog.h"
-#include "pcap-util.h"
+#include "capture-pcap-util.h"
 #include "capture_ui_utils.h"
 #include "main.h"
 #include "compat_macros.h"
@@ -85,7 +84,7 @@ capture_prefs_show(void)
 	GtkWidget	*ifopts_lb, *ifopts_bt;
 	GList		*if_list, *combo_list;
 	int		err;
-	char		err_str[PCAP_ERRBUF_SIZE];
+	char		err_str[CAPTURE_PCAP_ERRBUF_SIZE];
     int         row = 0;
     GtkTooltips *tooltips = gtk_tooltips_new();
 
@@ -251,9 +250,8 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 				*cur_opts_fr, *ed_opts_fr, *main_vb,
 				*if_descr_lb, *if_hide_lb,
 				*bbox, *ok_bt, *cancel_bt;
-	gchar *cur_titles[] = { "Device", "Description", 
-												"Comment", "Hide?" };
-    int row = 0;
+	const gchar *cur_titles[] = { "Device", "Description", "Comment", "Hide?" };
+	int row = 0;
 
 	GtkWidget *caller = gtk_widget_get_toplevel(w);
 	
@@ -289,7 +287,7 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	/*
 	 * Create current options CList.
 	 */
-	cur_clist = gtk_clist_new_with_titles(IFOPTS_CLIST_COLS, cur_titles);
+	cur_clist = gtk_clist_new_with_titles(IFOPTS_CLIST_COLS, (gchar **) cur_titles);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 1, 230);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 2, 260);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 3, 40);
@@ -696,7 +694,7 @@ ifopts_if_clist_add(void)
 {
 	GList		*if_list;
 	int		err;
-	char		err_str[PCAP_ERRBUF_SIZE];
+	char		err_str[CAPTURE_PCAP_ERRBUF_SIZE];
 	gchar		*cant_get_if_list_errstr;
 	if_info_t	*if_info;
 	guint		i;
