@@ -1,7 +1,7 @@
 /* color_dlg.c
  * Definitions for dialog boxes for color filters
  *
- * $Id: color_dlg.c 13910 2005-03-26 01:09:14Z ulfl $
+ * $Id: color_dlg.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,7 +40,7 @@
 #include <epan/dfilter/dfilter.h>
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dfilter_expr_dlg.h"
 #include "compat_macros.h"
 #include "filter_dlg.h"
@@ -63,7 +63,7 @@ static void remember_selected_row(GtkTreeSelection *sel, gpointer list);
 #endif
 static void color_destroy_cb(GtkButton *button, gpointer user_data);
 static void destroy_edit_dialog_cb(gpointer filter_arg, gpointer dummy);
-static void create_new_color_filter(GtkButton *button, char *filter);
+static void create_new_color_filter(GtkButton *button, const char *filter);
 static void color_new_cb(GtkButton *button, gpointer user_data);
 static void color_props_cb(GtkButton *button, gpointer user_data);
 static void color_delete_cb(GtkWidget *widget, gpointer user_data);
@@ -209,7 +209,7 @@ colorize_dialog_new (char *filter)
   GtkTreeViewColumn *column;
   GtkTreeSelection  *selection;
 #endif
-  gchar *titles[] = { "Name", "String" };
+  const gchar *titles[] = { "Name", "String" };
 
 
   row_selected = -1; /* no row selected */
@@ -317,7 +317,7 @@ colorize_dialog_new (char *filter)
   gtk_box_pack_start (GTK_BOX (list_vbox), scrolledwindow1, TRUE, TRUE, 0);
 
 #if GTK_MAJOR_VERSION < 2
-  color_filters = gtk_clist_new_with_titles(2, titles);
+  color_filters = gtk_clist_new_with_titles(2, (gchar **) titles);
 #else
   /* the list store contains : filter name, filter string, foreground
    * color, background color, pointer to color filter */
@@ -500,7 +500,7 @@ add_filter_to_list(gpointer filter_arg, gpointer list_arg)
   num_of_filters++;
 }
 
-void move_this_row (GtkWidget   *color_filters, 
+static void move_this_row (GtkWidget   *color_filters, 
                      gint         filter_number,
                      gint         amount)            /* only tested with +1(down) and -1(up) */
 {
@@ -689,7 +689,7 @@ struct remember_data
 };
 /* called for each selected row in the tree.
 */
-void remember_this_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer arg)
+static void remember_this_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer arg)
 {
     gint         *path_index;
     color_filter_t *colorf;
@@ -903,7 +903,7 @@ color_import_cb(GtkButton *button, gpointer user_data )
 /* Create a new filter in the list, and pop up an "Edit color filter"
    dialog box to edit it. */
 static void
-create_new_color_filter(GtkButton *button, char *filter)
+create_new_color_filter(GtkButton *button, const char *filter)
 {
   color_filter_t   *colorf;
   GtkStyle         *style;
@@ -1081,7 +1081,7 @@ color_clear_cmd(GtkWidget *widget)
 }
 
 /* clear button: user responded to question */
-void color_clear_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
+static void color_clear_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
 {
     switch(btn) {
     case(ESD_BTN_CLEAR):
