@@ -1,7 +1,7 @@
 /* column_prefs.c
  * Dialog box for column preferences
  *
- * $Id: column_prefs.c,v 1.16 2003/03/10 02:06:33 jmayer Exp $
+ * $Id: column_prefs.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,13 +31,10 @@
 #include "globals.h"
 #include "column_prefs.h"
 #include "gtkglobals.h"
-#include "prefs.h"
-#include "column.h"
+#include <epan/prefs.h>
+#include <epan/column.h>
 #include "compat_macros.h"
-
-#if GTK_MAJOR_VERSION >= 2
-#include "ui_util.h"
-#endif
+#include "gui_utils.h"
 
 static GtkWidget *column_l, *del_bt, *title_te, *fmt_m, *up_bt, *dn_bt;
 static gint       cur_fmt, cur_row;
@@ -67,14 +64,18 @@ void          column_set_arrow_button_sensitivity(GList *);
 /* Called when the 'Columns' preference notebook page is selected. */
 GtkWidget *
 column_prefs_show() {
-  GtkWidget         *main_vb, *top_hb, *list_bb, *new_bt, *column_sc,
-                    *tb, *lb, *menu, *mitem, *arrow_hb;
+  GtkWidget         *main_vb, *top_hb, *new_bt,
+                    *tb, *lb, *menu, *mitem;
+  GtkWidget         *order_fr, *order_vb, *order_lb;
+  GtkWidget         *list_fr, *list_vb, *list_lb, *list_sc;
+  GtkWidget         *edit_fr, *edit_vb;
+  GtkWidget         *props_fr, *props_hb;
   GList             *clp = NULL;
   fmt_data          *cfmt;
   gint               i;
-  gchar             *column_titles[] = {"Title", "Format"};
+  const gchar       *column_titles[] = {"Title", "Format"};
 #if GTK_MAJOR_VERSION < 2
-  gchar             *col_ent[2];
+  const gchar       *col_ent[2];
   gint               row;
 #else
   GtkListStore      *store;
@@ -82,6 +83,8 @@ column_prefs_show() {
   GtkTreeViewColumn *column;
   GtkTreeSelection  *sel;
   GtkTreeIter        iter;
+  GtkTreeIter        first_iter;
+  gint               first_row = TRUE;
 #endif
 
   /* Container for each row of widgets */
@@ -95,63 +98,59 @@ column_prefs_show() {
   gtk_container_add(GTK_CONTAINER(main_vb), top_hb);
   gtk_widget_show(top_hb);
 
-  list_bb = gtk_vbutton_box_new();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (list_bb), GTK_BUTTONBOX_START);
-  gtk_container_add(GTK_CONTAINER(top_hb), list_bb);
-  gtk_widget_show(list_bb);
 
-#if GTK_MAJOR_VERSION < 2
-  new_bt = gtk_button_new_with_label ("Add New");
-#else
-  new_bt = gtk_button_new_from_stock(GTK_STOCK_NEW);
-#endif
+  /* edit frame */
+  edit_fr = gtk_frame_new("Edit");
+  gtk_box_pack_start (GTK_BOX (top_hb), edit_fr, FALSE, FALSE, 0);
+  gtk_widget_show(edit_fr);
+
+  edit_vb = gtk_vbox_new (TRUE, 0);
+  gtk_container_set_border_width  (GTK_CONTAINER (edit_vb), 5);
+  gtk_container_add(GTK_CONTAINER(edit_fr), edit_vb);
+  gtk_widget_show(edit_vb);
+
+  new_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_NEW);
   SIGNAL_CONNECT(new_bt, "clicked", column_list_new_cb, NULL);
-  gtk_container_add(GTK_CONTAINER(list_bb), new_bt);
+  gtk_box_pack_start (GTK_BOX (edit_vb), new_bt, FALSE, FALSE, 5);
+#if GTK_MAJOR_VERSION < 2
+  WIDGET_SET_SIZE(new_bt, 50, 20);
+#endif
   gtk_widget_show(new_bt);
 
-#if GTK_MAJOR_VERSION < 2
-  del_bt = gtk_button_new_with_label ("Delete");
-#else
-  del_bt = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-#endif
+  del_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_DELETE);
   gtk_widget_set_sensitive(del_bt, FALSE);
   SIGNAL_CONNECT(del_bt, "clicked", column_list_delete_cb, NULL);
-  gtk_container_add(GTK_CONTAINER(list_bb), del_bt);
+#if GTK_MAJOR_VERSION < 2
+  WIDGET_SET_SIZE(del_bt, 50, 20);
+#endif
+  gtk_box_pack_start (GTK_BOX (edit_vb), del_bt, FALSE, FALSE, 5);
   gtk_widget_show(del_bt);
 
-  arrow_hb = gtk_hbox_new(TRUE, 3);
-  gtk_container_add(GTK_CONTAINER(list_bb), arrow_hb);
-  gtk_widget_show(arrow_hb);
 
-#if GTK_MAJOR_VERSION < 2
-  up_bt = gtk_button_new_with_label("Up");
-#else
-  up_bt = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+  /* columns list frame */
+  list_fr = gtk_frame_new("Columns");
+  gtk_box_pack_start (GTK_BOX (top_hb), list_fr, TRUE, TRUE, 0);
+  gtk_widget_show(list_fr);
+
+  list_vb = gtk_vbox_new (FALSE, 0);
+  gtk_container_set_border_width  (GTK_CONTAINER (list_vb), 5);
+  gtk_widget_show (list_vb);
+  gtk_container_add(GTK_CONTAINER(list_fr), list_vb);
+
+  list_lb = gtk_label_new (("[First list entry will be displayed left]"));
+  gtk_widget_show (list_lb);
+  gtk_box_pack_start (GTK_BOX (list_vb), list_lb, FALSE, FALSE, 0);
+
+  list_sc = scrolled_window_new(NULL, NULL);
+#if GTK_MAJOR_VERSION >= 2
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(list_sc), 
+                                   GTK_SHADOW_IN);
 #endif
-  gtk_widget_set_sensitive(up_bt, FALSE);
-  SIGNAL_CONNECT(up_bt, "clicked", column_arrow_cb, NULL);
-  gtk_box_pack_start(GTK_BOX(arrow_hb), up_bt, TRUE, TRUE, 0);
-  gtk_widget_show(up_bt);
+  gtk_container_add(GTK_CONTAINER(list_vb), list_sc);
+  gtk_widget_show(list_sc);
 
 #if GTK_MAJOR_VERSION < 2
-  dn_bt = gtk_button_new_with_label("Down");
-#else
-  dn_bt = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
-#endif
-  gtk_widget_set_sensitive(dn_bt, FALSE);
-  SIGNAL_CONNECT(dn_bt, "clicked", column_arrow_cb, NULL);
-  gtk_box_pack_start(GTK_BOX(arrow_hb), dn_bt, TRUE, TRUE, 0);
-  gtk_widget_show(dn_bt);
-
-  column_sc = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(column_sc),
-                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  WIDGET_SET_SIZE(column_sc, 250, 150);
-  gtk_container_add(GTK_CONTAINER(top_hb), column_sc);
-  gtk_widget_show(column_sc);
-
-#if GTK_MAJOR_VERSION < 2
-  column_l = gtk_clist_new_with_titles(2, column_titles);
+  column_l = gtk_clist_new_with_titles(2, (gchar **) column_titles);
   /* XXX - make this match the packet list prefs? */
   gtk_clist_set_selection_mode(GTK_CLIST(column_l), GTK_SELECTION_SINGLE);
   gtk_clist_column_titles_passive(GTK_CLIST(column_l));
@@ -161,7 +160,7 @@ column_prefs_show() {
 
   SIGNAL_CONNECT(column_l, "select-row", column_list_select_cb, NULL);
   SIGNAL_CONNECT(column_l, "unselect-row", column_list_unselect_cb, NULL);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(column_sc),
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(list_sc),
                                         column_l);
 #else
   store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
@@ -183,7 +182,7 @@ column_prefs_show() {
   gtk_tree_selection_set_mode(sel, GTK_SELECTION_SINGLE);
 
   SIGNAL_CONNECT(sel, "changed", column_list_select_cb, NULL);
-  gtk_container_add(GTK_CONTAINER(column_sc), column_l);
+  gtk_container_add(GTK_CONTAINER(list_sc), column_l);
 #endif
   gtk_widget_show(column_l);
 
@@ -193,27 +192,72 @@ column_prefs_show() {
 #if GTK_MAJOR_VERSION < 2
     col_ent[0] = cfmt->title;
     col_ent[1] = col_format_desc(get_column_format_from_str(cfmt->fmt));
-    row = gtk_clist_append(GTK_CLIST(column_l), col_ent);
+    row = gtk_clist_append(GTK_CLIST(column_l), (gchar **) col_ent);
     gtk_clist_set_row_data(GTK_CLIST(column_l), row, clp);
 #else
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, 0, cfmt->title, 1,
                        col_format_desc(get_column_format_from_str(cfmt->fmt)),
                        2, clp, -1);
+    if (first_row) {
+        first_iter = iter;
+        first_row = FALSE;
+    }
 #endif
     clp = clp->next;
   }
 #if GTK_MAJOR_VERSION >= 2
   g_object_unref(G_OBJECT(store));
 #endif
+  
+
+  /* order frame */
+  order_fr = gtk_frame_new("Order");
+  gtk_box_pack_start (GTK_BOX (top_hb), order_fr, FALSE, FALSE, 0);
+  gtk_widget_show(order_fr);
+
+  order_vb = gtk_vbox_new (TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(order_fr), order_vb);
+  gtk_container_set_border_width  (GTK_CONTAINER (order_vb), 5);
+  gtk_widget_show(order_vb);
+
+  up_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_GO_UP);
+  gtk_widget_set_sensitive(up_bt, FALSE);
+  SIGNAL_CONNECT(up_bt, "clicked", column_arrow_cb, NULL);
+  gtk_box_pack_start(GTK_BOX(order_vb), up_bt, FALSE, FALSE, 0);
+#if GTK_MAJOR_VERSION < 2
+  WIDGET_SET_SIZE(up_bt, 50, 20);
+#endif
+  gtk_widget_show(up_bt);
+
+  order_lb = gtk_label_new (("Move\nselected\ncolumn\nup or down"));
+  gtk_widget_show (order_lb);
+  gtk_box_pack_start (GTK_BOX (order_vb), order_lb, FALSE, FALSE, 0);
+
+  dn_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_GO_DOWN);
+  gtk_widget_set_sensitive(dn_bt, FALSE);
+  SIGNAL_CONNECT(dn_bt, "clicked", column_arrow_cb, NULL);
+  gtk_box_pack_start(GTK_BOX(order_vb), dn_bt, FALSE, FALSE, 0);
+#if GTK_MAJOR_VERSION < 2
+  WIDGET_SET_SIZE(dn_bt, 50, 20);
+#endif
+  gtk_widget_show(dn_bt);
+
+
+  /* properties frame */
+  props_fr = gtk_frame_new("Properties");
+  gtk_box_pack_start (GTK_BOX (main_vb), props_fr, FALSE, FALSE, 0);
+  gtk_widget_show(props_fr);
+
   /* Colunm name entry and format selection */
   tb = gtk_table_new(2, 2, FALSE);
-  gtk_container_add(GTK_CONTAINER(main_vb), tb);
+  gtk_container_border_width(GTK_CONTAINER(tb), 5);
+  gtk_container_add(GTK_CONTAINER(props_fr), tb);
   gtk_table_set_row_spacings(GTK_TABLE(tb), 10);
   gtk_table_set_col_spacings(GTK_TABLE(tb), 15);
   gtk_widget_show(tb);
 
-  lb = gtk_label_new("Column title:");
+  lb = gtk_label_new("Title:");
   gtk_misc_set_alignment(GTK_MISC(lb), 1.0, 0.5);
   gtk_table_attach_defaults(GTK_TABLE(tb), lb, 0, 1, 0, 1);
   gtk_widget_show(lb);
@@ -224,15 +268,15 @@ column_prefs_show() {
   gtk_widget_set_sensitive(title_te, FALSE);
   gtk_widget_show(title_te);
 
-  lb = gtk_label_new("Column format:");
+  lb = gtk_label_new("Format:");
   gtk_misc_set_alignment(GTK_MISC(lb), 1.0, 0.5);
   gtk_table_attach_defaults(GTK_TABLE(tb), lb, 0, 1, 1, 2);
   gtk_widget_show(lb);
 
-  top_hb = gtk_hbox_new(FALSE, 5);
-  gtk_table_attach(GTK_TABLE(tb), top_hb, 1, 2, 1, 2, GTK_FILL,
+  props_hb = gtk_hbox_new(FALSE, 5);
+  gtk_table_attach(GTK_TABLE(tb), props_hb, 1, 2, 1, 2, GTK_FILL,
                    GTK_SHRINK, 0, 0);
-  gtk_widget_show(top_hb);
+  gtk_widget_show(props_hb);
 
   fmt_m = gtk_option_menu_new();
   menu  = gtk_menu_new();
@@ -246,9 +290,22 @@ column_prefs_show() {
   cur_fmt = 0;
   gtk_option_menu_set_history(GTK_OPTION_MENU(fmt_m), cur_fmt);
   gtk_widget_set_sensitive(fmt_m, FALSE);
-  gtk_box_pack_start(GTK_BOX(top_hb), fmt_m, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(props_hb), fmt_m, FALSE, FALSE, 0);
   gtk_widget_show(fmt_m);
 
+  lb = gtk_label_new("Unlike all other preferences, you have to \"Save\" "
+      "and restart Ethereal to let column changes take effect!");
+  gtk_misc_set_alignment(GTK_MISC(lb), 0.5, 0.5);
+  gtk_box_pack_start (GTK_BOX (main_vb), lb, FALSE, FALSE, 0);
+  gtk_widget_show(lb);
+
+  /* select the first row */
+#if GTK_MAJOR_VERSION < 2
+  gtk_clist_select_row(GTK_CLIST(column_l), 0, 0);
+#else
+  gtk_tree_selection_select_iter(sel, &first_iter);
+#endif
+
   return(main_vb);
 }
 
@@ -276,12 +333,13 @@ column_list_select_cb(GtkCList *clist,
   gtk_editable_select_region(GTK_EDITABLE(title_te), 0, -1);
   gtk_widget_grab_focus(title_te);
 
-  gtk_option_menu_set_history(GTK_OPTION_MENU(fmt_m), cur_fmt);
-
   gtk_widget_set_sensitive(del_bt, TRUE);
   gtk_widget_set_sensitive(title_te, TRUE);
   gtk_widget_set_sensitive(fmt_m, TRUE);
   column_set_arrow_button_sensitivity(clp);
+
+  /* do this *after* set_sensitive(fmt_m), to have the correct "sensitive" effect */
+  gtk_option_menu_set_history(GTK_OPTION_MENU(fmt_m), cur_fmt);
 }
 
 /* A row was deselected.  Clear the text entry box and disable various widgets. */
@@ -361,9 +419,9 @@ column_list_select_cb(GtkTreeSelection *sel, gpointer  user_data _U_)
 static void
 column_list_new_cb(GtkWidget *w _U_, gpointer data _U_) {
     fmt_data     *cfmt;
-    gchar        *title = "New Column";
+    const gchar  *title = "New Column";
 #if GTK_MAJOR_VERSION < 2
-    gchar        *col_ent[2];
+    const gchar  *col_ent[2];
 #else
     GtkTreeModel *model;
     GtkTreeIter   iter;
@@ -380,7 +438,7 @@ column_list_new_cb(GtkWidget *w _U_, gpointer data _U_) {
 #if GTK_MAJOR_VERSION < 2
     col_ent[0] = title;
     col_ent[1] = col_format_desc(cur_fmt);
-    cur_row = gtk_clist_append(GTK_CLIST(column_l), col_ent);
+    cur_row = gtk_clist_append(GTK_CLIST(column_l), (gchar **) col_ent);
     gtk_clist_set_row_data(GTK_CLIST(column_l), cur_row,
                            g_list_last(prefs.col_list));
 
@@ -625,5 +683,4 @@ column_prefs_destroy(GtkWidget *w) {
     /* Let the list cb know we're about to destroy the widget tree, so it */
     /* doesn't operate on widgets that don't exist. */
     OBJECT_SET_DATA(w, E_COL_CM_KEY, (gpointer)TRUE);
-    gtk_widget_destroy(GTK_WIDGET(w));
 }
