@@ -1,7 +1,7 @@
 /* conversations_fc.c
  * conversations_fc   2003 Ronnie Sahlberg
  *
- * $Id: conversations_fc.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: conversations_fc.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "conversations_table.h"
@@ -53,9 +55,9 @@ fc_conversation_packet(void *pct, packet_info *pinfo, epan_dissect_t *edt _U_, c
 
 
 static void
-fc_conversation_init(char *optarg)
+fc_conversation_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"conv,fc,",8)){
 		filter=optarg+8;
@@ -71,16 +73,16 @@ fc_conversation_init(char *optarg)
 static void
 fc_endpoints_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	fc_conversation_init("conv,fc");
+	fc_conversation_init("conv,fc",NULL);
 }
 
 
 void
 register_tap_listener_fc_conversation(void)
 {
-	register_ethereal_tap("conv,fc", fc_conversation_init);
+	register_stat_cmd_arg("conv,fc", fc_conversation_init, NULL);
 
-	register_tap_menu_item("Fibre Channel", REGISTER_TAP_GROUP_CONVERSATION_LIST,
+	register_stat_menu_item("Fibre Channel", REGISTER_STAT_GROUP_CONVERSATION_LIST,
 	    fc_endpoints_cb, NULL, NULL, NULL);
 
 	register_conversation_table(TRUE, "Fibre Channel", "fc", NULL /*filter*/, fc_conversation_packet);
