@@ -1,7 +1,7 @@
 /* conversations_tcpip.c
  * conversations_tcpip   2003 Ronnie Sahlberg
  *
- * $Id: conversations_tcpip.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: conversations_tcpip.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,10 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/emem.h>
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "conversations_table.h"
@@ -53,9 +56,9 @@ tcpip_conversation_packet(void *pct, packet_info *pinfo, epan_dissect_t *edt _U_
 
 
 static void
-tcpip_conversation_init(char *optarg)
+tcpip_conversation_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"conv,tcp,",9)){
 		filter=optarg+9;
@@ -71,16 +74,16 @@ tcpip_conversation_init(char *optarg)
 static void
 tcpip_conversation_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	tcpip_conversation_init("conv,tcp");
+	tcpip_conversation_init("conv,tcp",NULL);
 }
 
 
 void
 register_tap_listener_tcpip_conversation(void)
 {
-	register_ethereal_tap("conv,tcp", tcpip_conversation_init);
+	register_stat_cmd_arg("conv,tcp", tcpip_conversation_init,NULL);
 
-	register_tap_menu_item("TCP (IPv4 & IPv6)", REGISTER_TAP_GROUP_CONVERSATION_LIST,
+	register_stat_menu_item("TCP (IPv4 & IPv6)", REGISTER_STAT_GROUP_CONVERSATION_LIST,
 	    tcpip_conversation_cb, NULL, NULL, NULL);
 
 	register_conversation_table(FALSE, "TCP", "tcp", NULL /*filter*/, tcpip_conversation_packet);
