@@ -1,7 +1,7 @@
 /* conversations_tr.c
  * conversations_tr   2003 Ronnie Sahlberg
  *
- * $Id: conversations_tr.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: conversations_tr.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "conversations_table.h"
@@ -53,9 +55,9 @@ tr_conversation_packet(void *pct, packet_info *pinfo, epan_dissect_t *edt _U_, c
 
 
 static void
-tr_conversation_init(char *optarg)
+tr_conversation_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"conv,tr,",8)){
 		filter=optarg+8;
@@ -71,16 +73,16 @@ tr_conversation_init(char *optarg)
 static void
 tr_conversation_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	tr_conversation_init("conv,tr");
+	tr_conversation_init("conv,tr",NULL);
 }
 
 
 void
 register_tap_listener_tr_conversation(void)
 {
-	register_ethereal_tap("conv,tr", tr_conversation_init);
+	register_stat_cmd_arg("conv,tr", tr_conversation_init, NULL);
 
-	register_tap_menu_item("Token Ring", REGISTER_TAP_GROUP_CONVERSATION_LIST,
+	register_stat_menu_item("Token Ring", REGISTER_STAT_GROUP_CONVERSATION_LIST,
 	    tr_conversation_cb, NULL, NULL, NULL);
 
     register_conversation_table(TRUE, "Token Ring", "tr", NULL /*filter*/, tr_conversation_packet);
