@@ -1,7 +1,7 @@
 /* conversations_udpip.c
  * conversations_udpip   2003 Ronnie Sahlberg
  *
- * $Id: conversations_udpip.c 13993 2005-04-01 09:08:27Z guy $
+ * $Id: conversations_udpip.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "conversations_table.h"
@@ -53,9 +55,9 @@ udpip_conversation_packet(void *pct, packet_info *pinfo, epan_dissect_t *edt _U_
 
 
 static void
-udpip_conversation_init(char *optarg)
+udpip_conversation_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"conv,udp,",9)){
 		filter=optarg+9;
@@ -71,16 +73,16 @@ udpip_conversation_init(char *optarg)
 static void
 udpip_conversation_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	udpip_conversation_init("conv,udp");
+	udpip_conversation_init("conv,udp",NULL);
 }
 
 
 void
 register_tap_listener_udpip_conversation(void)
 {
-	register_tap_listener_cmd_arg("conv,udp", udpip_conversation_init);
+	register_stat_cmd_arg("conv,udp", udpip_conversation_init, NULL);
 
-	register_tap_menu_item("UDP (IPv4 & IPv6)", REGISTER_TAP_GROUP_CONVERSATION_LIST,
+	register_stat_menu_item("UDP (IPv4 & IPv6)", REGISTER_STAT_GROUP_CONVERSATION_LIST,
 	    udpip_conversation_cb, NULL, NULL, NULL);
 
     register_conversation_table(FALSE, "UDP", "udp", NULL /*filter*/, udpip_conversation_packet);
