@@ -1,7 +1,7 @@
 /* dcerpc_stat.c
  * dcerpc_stat   2002 Ronnie Sahlberg
  *
- * $Id: dcerpc_stat.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: dcerpc_stat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,10 +40,12 @@
 #include <epan/packet_info.h>
 #include <epan/epan.h>
 
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include <epan/dissectors/packet-dcerpc.h>
@@ -58,7 +60,7 @@
 typedef struct _rpcstat_t {
 	GtkWidget *win;
 	srt_stat_table srt_table;
-	char *prog;
+	const char *prog;
 	e_uuid_t uuid;
 	guint16 ver;
 	int num_procedures;
@@ -184,7 +186,7 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
 /* When called, this function will create a new instance of gtk-dcerpcstat.
  */
 static void
-gtk_dcerpcstat_init(char *optarg)
+gtk_dcerpcstat_init(const char *optarg, void* userdata _U_)
 {
 	rpcstat_t *rs;
 	guint32 i, max_procs;
@@ -193,15 +195,15 @@ gtk_dcerpcstat_init(char *optarg)
 	GtkWidget *vbox;
 	GtkWidget *stat_label;
 	GtkWidget *filter_label;
-    GtkWidget *bbox;
-    GtkWidget *close_bt;
+	GtkWidget *bbox;
+	GtkWidget *close_bt;
 	dcerpc_sub_dissector *procs;
 	e_uuid_t uuid;
 	guint d1,d2,d3,d40,d41,d42,d43,d44,d45,d46,d47;
 	int major, minor;
 	guint16 ver;
 	int pos=0;
-        char *filter=NULL;
+        const char *filter=NULL;
         GString *error_string;
 	int hf_opnum;
 
@@ -295,7 +297,7 @@ gtk_dcerpcstat_init(char *optarg)
 
        	for(i=0;i<(max_procs+1);i++){
 		int j;
-		char *proc_name;
+		const char *proc_name;
 
 		proc_name="unknown";
 		for(j=0;procs[j].name;j++){
@@ -323,15 +325,15 @@ gtk_dcerpcstat_init(char *optarg)
 	gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
 	close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
-    window_set_cancel_button(rs->win, close_bt, window_cancel_button_cb);
+	window_set_cancel_button(rs->win, close_bt, window_cancel_button_cb);
 
-    SIGNAL_CONNECT(rs->win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(rs->win, "delete_event", window_delete_event_cb, NULL);
 	SIGNAL_CONNECT(rs->win, "destroy", win_destroy_cb, rs);
 
-    gtk_widget_show_all(rs->win);
-    window_present(rs->win);
+	gtk_widget_show_all(rs->win);
+	window_present(rs->win);
 
-    cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 }
 
 
@@ -369,7 +371,7 @@ dcerpcstat_start_button_clicked(GtkWidget *item _U_, gpointer data _U_)
 		g_string_sprintfa(str, ",%s", filter);
 	}
 
-	gtk_dcerpcstat_init(str->str);
+	gtk_dcerpcstat_init(str->str,NULL);
 	g_string_free(str, TRUE);
 }
 
@@ -431,7 +433,7 @@ static GtkWidget *program_submenu_menu;
 static GtkWidget *program_submenu_item;
 static GtkWidget *program_submenu_label;
 static int program_subitem_index;
-static char *first_menu_name;
+static const char *first_menu_name;
 static void 
 dcerpcstat_add_program_to_menu(dcerpc_uuid_key *k, dcerpc_uuid_value *v)
 {
@@ -548,7 +550,8 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	static construct_args_t args = {
 	  "Service Response Time Statistics Filter",
 	  FALSE,
-	  FALSE
+	  FALSE,
+      FALSE
 	};
 
 	/* if the window is already open, bring it to front and
@@ -675,8 +678,8 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkdcerpcstat(void)
 {
-	register_ethereal_tap("dcerpc,srt,", gtk_dcerpcstat_init);
+	register_stat_cmd_arg("dcerpc,srt,", gtk_dcerpcstat_init,NULL);
 
-	register_tap_menu_item("DCE-RPC...", REGISTER_TAP_GROUP_RESPONSE_TIME,
+	register_stat_menu_item("DCE-RPC...", REGISTER_STAT_GROUP_RESPONSE_TIME,
 	    gtk_dcerpcstat_cb, NULL, NULL, NULL);
 }
