@@ -1,6 +1,6 @@
 /* decode_as_dlg.c
  *
- * $Id: decode_as_dlg.c 15043 2005-07-24 19:01:28Z guy $
+ * $Id: decode_as_dlg.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Routines to modify dissector tables on the fly.
  *
@@ -35,7 +35,7 @@
 #include "simple_dialog.h"
 #include <epan/packet.h>
 #include <epan/ipproto.h>
-#include "ui_util.h"
+#include "gui_utils.h"
 #include <epan/epan_dissect.h>
 #include "compat_macros.h"
 #include "decode_as_dcerpc.h"
@@ -265,7 +265,7 @@ gchar *selector_name,
 const gchar *initial_proto_name, 
 const gchar *current_proto_name)
 {
-    gchar     *text[E_LIST_D_COLUMNS];
+    const gchar     *text[E_LIST_D_COLUMNS];
 #if GTK_MAJOR_VERSION < 2
     GtkCList  *clist;
     gint       row;
@@ -280,12 +280,12 @@ const gchar *current_proto_name)
     store = (GtkListStore *)list_data;
 #endif
 
-    text[E_LIST_D_TABLE] = (char *)table_name;
+    text[E_LIST_D_TABLE] = table_name;
     text[E_LIST_D_SELECTOR] = selector_name;
-    text[E_LIST_D_INITIAL] = (char *)initial_proto_name;
-    text[E_LIST_D_CURRENT] = (char *)current_proto_name;
+    text[E_LIST_D_INITIAL] = initial_proto_name;
+    text[E_LIST_D_CURRENT] = current_proto_name;
 #if GTK_MAJOR_VERSION < 2
-    row = gtk_clist_prepend(clist, text);
+    row = gtk_clist_prepend(clist, (gchar **) text);
 #else
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, E_LIST_D_TABLE, text[E_LIST_D_TABLE],
@@ -523,7 +523,7 @@ void
 decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
 {
     GtkWidget         *main_vb, *bbox, *ok_bt, *clear_bt, *help_bt, *scrolled_window;
-    gchar             *titles[E_LIST_D_COLUMNS] = {
+    const gchar       *titles[E_LIST_D_COLUMNS] = {
         "Table", "Value", "Initial", "Current"
     };
     gint               column;
@@ -555,7 +555,7 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
     {
 	/* Initialize list */
 #if GTK_MAJOR_VERSION < 2
-	list = GTK_CLIST(gtk_clist_new_with_titles(E_LIST_D_COLUMNS, titles));
+	list = GTK_CLIST(gtk_clist_new_with_titles(E_LIST_D_COLUMNS, (gchar **) titles));
 	gtk_clist_column_titles_passive(list);
 	for (column = 0; column < E_LIST_D_COLUMNS; column++)
 	    gtk_clist_set_column_auto_resize(list, column, TRUE);
@@ -1230,9 +1230,9 @@ lookup_handle(GtkTreeModel *model, GtkTreePath *path _U_, GtkTreeIter *iter,
  * to store any information generated by this routine.
  */
 void
-decode_add_to_list (gchar *table_name, const gchar *proto_name, gpointer value, gpointer user_data)
+decode_add_to_list (const gchar *table_name, const gchar *proto_name, gpointer value, gpointer user_data)
 {
-    gchar     *text[E_LIST_S_COLUMNS];
+    const gchar     *text[E_LIST_S_COLUMNS];
 #if GTK_MAJOR_VERSION < 2
     GtkCList  *list;
     gint       row;
@@ -1263,10 +1263,10 @@ decode_add_to_list (gchar *table_name, const gchar *proto_name, gpointer value,
     if (hli.found) return;
 #endif
 
-    text[E_LIST_S_PROTO_NAME] = (char *)proto_name;
+    text[E_LIST_S_PROTO_NAME] = proto_name;
     text[E_LIST_S_TABLE] = table_name;
 #if GTK_MAJOR_VERSION < 2
-    row = gtk_clist_prepend(list, text);
+    row = gtk_clist_prepend(list, (gchar **) text);
     gtk_clist_set_row_data(list, row, value);
 #else
     gtk_list_store_append(store, &iter);
@@ -1278,7 +1278,7 @@ decode_add_to_list (gchar *table_name, const gchar *proto_name, gpointer value,
 }
 
 static void
-decode_proto_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
+decode_proto_add_to_list (const gchar *table_name, gpointer value, gpointer user_data)
 {
     const gchar     *proto_name;
     gint       i;
@@ -1387,7 +1387,7 @@ decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
 void
 decode_list_menu_finish(GtkWidget *list)
 {
-    gchar *text[E_LIST_S_COLUMNS];
+    const gchar *text[E_LIST_S_COLUMNS];
 #if GTK_MAJOR_VERSION < 2
     gint row;
 #else
@@ -1398,7 +1398,7 @@ decode_list_menu_finish(GtkWidget *list)
     text[E_LIST_S_PROTO_NAME] = "(default)";
     text[E_LIST_S_TABLE] = "(none)";
 #if GTK_MAJOR_VERSION < 2
-    row = gtk_clist_prepend(GTK_CLIST(list), text);
+    row = gtk_clist_prepend(GTK_CLIST(list), (gchar **) text);
     gtk_clist_set_row_data(GTK_CLIST(list), row, NULL);
 
     gtk_clist_select_row(GTK_CLIST(list), 0, -1);
@@ -1432,7 +1432,7 @@ decode_list_menu_finish(GtkWidget *list)
  * scrolled window.
  */
 static GtkWidget *
-decode_add_simple_menu (GtkWidget *page, gchar *table_name)
+decode_add_simple_menu (GtkWidget *page, const gchar *table_name)
 {
     GtkWidget *scrolled_window;
     GtkWidget *list;
@@ -1469,15 +1469,15 @@ decode_add_simple_menu (GtkWidget *page, gchar *table_name)
  * routine.
  */
 static GtkWidget *
-decode_add_simple_page (gchar *prompt, gchar *title, gchar *table_name,
+decode_add_simple_page (const gchar *prompt, const gchar *title, const gchar *table_name,
 			guint value)
 {
     GtkWidget	*page, *label, *scrolled_window;
 
     page = gtk_hbox_new(FALSE, 5);
     OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_simple);
-    OBJECT_SET_DATA(page, E_PAGE_TABLE, table_name);
-    OBJECT_SET_DATA(page, E_PAGE_TITLE, title);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, (gchar *) table_name);
+    OBJECT_SET_DATA(page, E_PAGE_TITLE, (gchar *) title);
     OBJECT_SET_DATA(page, E_PAGE_VALUE, GUINT_TO_POINTER(value));
 
     /* Always enabled */
@@ -1520,13 +1520,13 @@ decode_add_simple_page (gchar *prompt, gchar *title, gchar *table_name,
  * this routine.
  */
 static GtkWidget *
-decode_add_tcpudp_page (gchar *prompt, gchar *table_name)
+decode_add_tcpudp_page (const gchar *prompt, const gchar *table_name)
 {
     GtkWidget	*page, *label, *scrolled_window, *optmenu;
 
     page = gtk_hbox_new(FALSE, 5);
     OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_transport);
-    OBJECT_SET_DATA(page, E_PAGE_TABLE, table_name);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, (gchar *) table_name);
     OBJECT_SET_DATA(page, E_PAGE_TITLE, "Transport");
 
     /* Always enabled */
@@ -1548,7 +1548,7 @@ decode_add_tcpudp_page (gchar *prompt, gchar *table_name)
     return(page);
 }
 
-void
+static void
 decode_sctp_list_menu_start(GtkWidget **list_p, GtkWidget **scrolled_win_p)
 {
 #if GTK_MAJOR_VERSION < 2
@@ -1723,13 +1723,13 @@ decode_sctp_add_port_ppid (GtkWidget *page)
 
 
 static GtkWidget *
-decode_add_sctp_page (gchar *prompt, gchar *table_name)
+decode_add_sctp_page (const gchar *prompt, const gchar *table_name)
 {
     GtkWidget	*page, *label, *scrolled_window,  *radio, *vbox, *alignment, *sctpbox, *sctpmenu;
 
     page = gtk_hbox_new(FALSE, 5);
     OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_transport);
-    OBJECT_SET_DATA(page, E_PAGE_TABLE, table_name);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, (gchar *) table_name);
     OBJECT_SET_DATA(page, E_PAGE_TITLE, "Transport");
 
     vbox = gtk_vbox_new(FALSE, 5);
