@@ -1,7 +1,7 @@
 /* dlg_utils.c
  * Utilities to use when constructing dialogs
  *
- * $Id: dlg_utils.c 13530 2005-02-27 13:00:22Z lego $
+ * $Id: dlg_utils.c 15886 2005-09-20 01:53:38Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,7 +34,7 @@
 #include "globals.h"
 
 #include "gtkglobals.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "keys.h"
 #include "compat_macros.h"
@@ -63,7 +63,7 @@ static void file_selection_browse_destroy_cb(GtkWidget *win, GtkWidget* file_te)
 
 /* create a button for the button row (helper for dlg_button_row_new) */
 static GtkWidget *
-dlg_button_new(GtkWidget *hbox, GtkWidget *button_hbox, gchar *stock_id)
+dlg_button_new(GtkWidget *hbox, GtkWidget *button_hbox, const gchar *stock_id)
 {
     GtkWidget *button;
 
@@ -79,7 +79,7 @@ dlg_button_new(GtkWidget *hbox, GtkWidget *button_hbox, gchar *stock_id)
 /* create a button row for a dialog */
 
 /* The purpose of this is, to have one place available, where all button rows 
- * from all dialogs are layouted. This will:
+ * from all dialogs are laid out. This will:
  *
  * a.) keep the button layout more consistent over the different dialogs
  * b.) being able to switch between different button layouts, e.g.:
@@ -87,31 +87,32 @@ dlg_button_new(GtkWidget *hbox, GtkWidget *button_hbox, gchar *stock_id)
  *     GTK2 (e.g. GNOME) "Apply" "Cancel" "OK"
  */
 GtkWidget *
-dlg_button_row_new(gchar *stock_id_first, ...)
+dlg_button_row_new(const gchar *stock_id_first, ...)
 {
     gint        buttons = 0;
     va_list     stock_id_list;
-    gchar       *stock_id = stock_id_first;
+    const gchar *stock_id = stock_id_first;
     GtkWidget   *hbox;
     GtkWidget   *button_hbox;
     GtkWidget   *help_hbox;
     GtkWidget   *button;
 
-    gchar *ok           = NULL;
-    gchar *apply        = NULL;
-    gchar *save         = NULL;
-    gchar *dont_save	= NULL;
-    gchar *cancel       = NULL;
-    gchar *close        = NULL;
-    gchar *clear        = NULL;
-    gchar *stop         = NULL;
-    gchar *create_stat  = NULL;
-    gchar *help         = NULL;
-    gchar *print        = NULL;
-    gchar *find         = NULL;
-    gchar *jump         = NULL;
-    gchar *yes          = NULL;
-    gchar *no           = NULL;
+    const gchar *ok           = NULL;
+    const gchar *apply        = NULL;
+    const gchar *save         = NULL;
+    const gchar *dont_save    = NULL;
+    const gchar *cancel       = NULL;
+    const gchar *close        = NULL;
+    const gchar *clear        = NULL;
+    const gchar *start        = NULL;
+    const gchar *stop         = NULL;
+    const gchar *create_stat  = NULL;
+    const gchar *help         = NULL;
+    const gchar *print        = NULL;
+    const gchar *find         = NULL;
+    const gchar *jump         = NULL;
+    const gchar *yes          = NULL;
+    const gchar *no           = NULL;
 
 
     va_start(stock_id_list, stock_id_first);
@@ -134,6 +135,10 @@ dlg_button_row_new(gchar *stock_id_first, ...)
             close = stock_id;
         } else if (strcmp(stock_id, GTK_STOCK_CLEAR) == 0) {
             clear = stock_id;
+#ifdef HAVE_LIBPCAP
+        } else if (strcmp(stock_id, ETHEREAL_STOCK_CAPTURE_START) == 0) {
+            start = stock_id;
+#endif /* HAVE_LIBPCAP */
         } else if (strcmp(stock_id, GTK_STOCK_STOP) == 0) {
             stop = stock_id;
         } else if (strcmp(stock_id, GTK_STOCK_HELP) == 0) {
@@ -157,10 +162,6 @@ dlg_button_row_new(gchar *stock_id_first, ...)
     }
     va_end(stock_id_list);
 
-    /* we should have at least one button */
-    g_assert(buttons);
-
-
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_widget_show(hbox);
 
@@ -172,6 +173,11 @@ dlg_button_row_new(gchar *stock_id_first, ...)
     gtk_box_pack_end(GTK_BOX(hbox), help_hbox, FALSE, FALSE, 0);
     gtk_widget_show(help_hbox);
 
+    if (buttons == 0) {
+        /* if no buttons wanted, simply do nothing */
+        return hbox;
+    }
+
     if (buttons == 1) {
         /* if only one button, simply put it in the middle (default) */
         dlg_button_new(hbox, button_hbox, stock_id_first);
@@ -239,6 +245,11 @@ dlg_button_row_new(gchar *stock_id_first, ...)
             dlg_button_new(hbox, button_hbox, create_stat);
             return hbox;
         }
+        if (start && cancel) {
+            dlg_button_new(hbox, button_hbox, cancel);
+            dlg_button_new(hbox, button_hbox, start);
+            return hbox;
+        }
     }
     if (buttons == 3) {
         if (ok && save && close) {
@@ -301,6 +312,7 @@ dlg_button_row_new(gchar *stock_id_first, ...)
     if (no      != NULL) dlg_button_new(hbox, button_hbox, no);
     if (save    != NULL) dlg_button_new(hbox, button_hbox, save);
     if (dont_save != NULL) dlg_button_new(hbox, button_hbox, dont_save);
+    if (start   != NULL) dlg_button_new(hbox, button_hbox, start);
     if (stop    != NULL) dlg_button_new(hbox, button_hbox, stop);
     if (close   != NULL) dlg_button_new(hbox, button_hbox, close);
     if (clear   != NULL) dlg_button_new(hbox, button_hbox, clear);
