@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c 13419 2005-02-17 03:05:54Z guy $
+ * $Id: file_dlg.c 17355 2006-02-19 23:51:39Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,7 +38,7 @@
 #include <epan/addr_resolv.h>
 #include "keys.h"
 #include "filter_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
 #include "menu.h"
@@ -63,8 +63,10 @@
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_IO_H
-#include <io.h> /* open/close on win32 */
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+#include <gdk/gdkwin32.h>
+#include <windows.h>
+#include "win32-file-dlg.h"
 #endif
 
 static void file_open_ok_cb(GtkWidget *w, gpointer fs);
@@ -116,18 +118,8 @@ static GtkWidget *cfmark_cb;
 static GtkWidget *ft_om;
 static GtkWidget *range_tb;
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #define PREVIEW_STR_MAX         200
 
-static double
-secs_usecs( guint32 s, guint32 us)
-{
-  return (us / 1000000.0) + (double)s;
-}
-
 
 /* set a new filename for the preview widget */
 static wtap *
@@ -137,9 +129,8 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
     wtap       *wth;
     int         err = 0;
     gchar      *err_info;
-    struct stat cf_stat;
     gchar       string_buff[PREVIEW_STR_MAX];
-    guint64     filesize;
+    gint64      filesize;
 
 
     /* init preview labels */
@@ -181,14 +172,13 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
     }
 
     /* Find the size of the file. */
-    if (fstat(wtap_fd(wth), &cf_stat) < 0) {
+    filesize = wtap_file_size(wth, &err);
+    if (filesize == -1) {
+        gtk_label_set_text(GTK_LABEL(label), "error getting file size");
         wtap_close(wth);
         return NULL;
     }
-
-    /* size */
-    filesize = cf_stat.st_size;
-    g_snprintf(string_buff, PREVIEW_STR_MAX, "%" PRIu64 " bytes", filesize);
+    g_snprintf(string_buff, PREVIEW_STR_MAX, "%" PRId64 " bytes", filesize);
     label = OBJECT_GET_DATA(prev, PREVIEW_SIZE_KEY);
     gtk_label_set_text(GTK_LABEL(label), string_buff);
 
@@ -213,8 +203,8 @@ preview_do(GtkWidget *prev, wtap *wth)
     gchar      *err_info;
     long        data_offset;
     const struct wtap_pkthdr *phdr;
-    double      start_time = 0;	/* seconds, with msec resolution */
-    double      stop_time = 0;	/* seconds, with msec resolution */
+    double      start_time = 0;	/* seconds, with nsec resolution */
+    double      stop_time = 0;	/* seconds, with nsec resolution */
     double      cur_time;
     unsigned int packets = 0;
     gboolean    is_breaked = FALSE;
@@ -225,8 +215,8 @@ preview_do(GtkWidget *prev, wtap *wth)
 
     time(&time_preview);
     while ( (wtap_read(wth, &err, &err_info, &data_offset)) ) {
-        phdr = wtap_phdr(wth);        
-        cur_time = secs_usecs(phdr->ts.tv_sec, phdr->ts.tv_usec);
+        phdr = wtap_phdr(wth);
+        cur_time = nstime_to_sec( (const nstime_t *) &phdr->ts );
         if(packets == 0) {
             start_time 	= cur_time;
             stop_time = cur_time;
@@ -269,24 +259,28 @@ preview_do(GtkWidget *prev, wtap *wth)
     /* first packet */
     ti_time = (long)start_time;
     ti_tm = localtime( &ti_time );
-    g_snprintf(string_buff, PREVIEW_STR_MAX,
-             "%04d-%02d-%02d %02d:%02d:%02d",
-             ti_tm->tm_year + 1900,
-             ti_tm->tm_mon + 1,
-             ti_tm->tm_mday,
-             ti_tm->tm_hour,
-             ti_tm->tm_min,
-             ti_tm->tm_sec);
+	if(ti_tm) {
+		g_snprintf(string_buff, PREVIEW_STR_MAX,
+				 "%04d-%02d-%02d %02d:%02d:%02d",
+				 ti_tm->tm_year + 1900,
+				 ti_tm->tm_mon + 1,
+				 ti_tm->tm_mday,
+				 ti_tm->tm_hour,
+				 ti_tm->tm_min,
+				 ti_tm->tm_sec);
+	} else {
+		g_snprintf(string_buff, PREVIEW_STR_MAX, "?");
+	}
     label = OBJECT_GET_DATA(prev, PREVIEW_FIRST_KEY);
     gtk_label_set_text(GTK_LABEL(label), string_buff);
 
     /* elapsed time */
     elapsed_time = (unsigned int)(stop_time-start_time);
     if(elapsed_time/86400) {
-      g_snprintf(string_buff, PREVIEW_STR_MAX, "%02u days %02u:%02u:%02u", 
+      g_snprintf(string_buff, PREVIEW_STR_MAX, "%02u days %02u:%02u:%02u",
         elapsed_time/86400, elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
     } else {
-      g_snprintf(string_buff, PREVIEW_STR_MAX, "%02u:%02u:%02u", 
+      g_snprintf(string_buff, PREVIEW_STR_MAX, "%02u:%02u:%02u",
         elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
     }
     if(is_breaked) {
@@ -359,7 +353,7 @@ file_open_entry_changed(GtkWidget *w _U_, gpointer file_sel)
 
 /* copied from summary_dlg.c */
 static GtkWidget *
-add_string_to_table_sensitive(GtkWidget *list, guint *row, gchar *title, gchar *value, gboolean sensitive)
+add_string_to_table_sensitive(GtkWidget *list, guint *row, const gchar *title, const gchar *value, gboolean sensitive)
 {
     GtkWidget *label;
     gchar     *indent;
@@ -386,7 +380,7 @@ add_string_to_table_sensitive(GtkWidget *list, guint *row, gchar *title, gchar *
 }
 
 static GtkWidget *
-add_string_to_table(GtkWidget *list, guint *row, gchar *title, gchar *value)
+add_string_to_table(GtkWidget *list, guint *row, const gchar *title, const gchar *value)
 {
     return add_string_to_table_sensitive(list, row, title, value, TRUE);
 }
@@ -430,20 +424,25 @@ preview_new(void)
 static GtkWidget *file_open_w;
 
 /* Open a file */
-void
+static void
 file_open_cmd(GtkWidget *w)
 {
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_open_file(GDK_WINDOW_HWND(top_level->window));
+#else /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
   GtkWidget	*main_hb, *main_vb, *filter_hbox, *filter_bt, *filter_te,
   		*m_resolv_cb, *n_resolv_cb, *t_resolv_cb, *prev;
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
+
   /* No Apply button, and "OK" just sets our text widget, it doesn't
      activate it (i.e., it doesn't cause us to try to open the file). */
   static construct_args_t args = {
   	"Ethereal: Read Filter",
   	FALSE,
-  	FALSE
+  	FALSE,
+    TRUE
   };
 
   if (file_open_w != NULL) {
@@ -455,7 +454,7 @@ file_open_cmd(GtkWidget *w)
   file_open_w = file_selection_new("Ethereal: Open Capture File",
                                    FILE_SELECTION_OPEN);
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
-  /* it's annoying, that the file chooser dialog is already shown here, 
+  /* it's annoying, that the file chooser dialog is already shown here,
      so we cannot use the correct gtk_window_set_default_size() to resize it */
   WIDGET_SET_SIZE(GTK_WINDOW(file_open_w), DEF_WIDTH, DEF_HEIGHT);
 #else
@@ -490,7 +489,7 @@ file_open_cmd(GtkWidget *w)
     break;
   }
 
-  
+
   main_hb = gtk_hbox_new(FALSE, 3);
   file_selection_set_extra_widget(file_open_w, main_hb);
   gtk_widget_show(main_hb);
@@ -574,7 +573,7 @@ file_open_cmd(GtkWidget *w)
   gtk_box_pack_start(GTK_BOX(main_hb), prev, TRUE, TRUE, 0);
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
-  SIGNAL_CONNECT(GTK_FILE_CHOOSER(file_open_w), "selection-changed", 
+  SIGNAL_CONNECT(GTK_FILE_CHOOSER(file_open_w), "selection-changed",
       file_open_entry_changed, file_open_w);
   file_open_entry_changed(file_open_w, file_open_w);
 
@@ -585,8 +584,8 @@ file_open_cmd(GtkWidget *w)
     file_open_ok_cb(file_open_w, file_open_w);
   }
   else window_destroy(file_open_w);
-#else
-  SIGNAL_CONNECT(GTK_FILE_SELECTION(file_open_w)->selection_entry, "changed", 
+#else /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+  SIGNAL_CONNECT(GTK_FILE_SELECTION(file_open_w)->selection_entry, "changed",
       file_open_entry_changed, file_open_w);
 
   /* Connect the ok_button to file_open_ok_cb function and pass along a
@@ -598,14 +597,15 @@ file_open_cmd(GtkWidget *w)
                   E_DFILTER_TE_KEY, OBJECT_GET_DATA(w, E_DFILTER_TE_KEY));
 
   /* Connect the cancel_button to destroy the widget */
-  window_set_cancel_button(file_open_w, 
+  window_set_cancel_button(file_open_w,
       GTK_FILE_SELECTION(file_open_w)->cancel_button, window_cancel_button_cb);
 
   SIGNAL_CONNECT(file_open_w, "delete_event", window_delete_event_cb, NULL);
 
   gtk_widget_show(file_open_w);
   window_present(file_open_w);
-#endif
+#endif /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+#endif /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
 }
 
 static void file_open_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
@@ -757,9 +757,14 @@ file_open_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 static GtkWidget *file_merge_w;
 
 /* Merge existing with another file */
-void
+static void
 file_merge_cmd(GtkWidget *w)
 {
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_merge_file(GDK_WINDOW_HWND(top_level->window));
+  packet_list_freeze();
+  packet_list_thaw();
+#else /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
   GtkWidget	*main_hb, *main_vb, *ft_hb, *ft_lb, *filter_hbox,
 		*filter_bt, *filter_te, *prepend_rb, *chrono_rb,
 		*append_rb, *prev;
@@ -772,7 +777,8 @@ file_merge_cmd(GtkWidget *w)
   static construct_args_t args = {
   	"Ethereal: Read Filter",
   	FALSE,
-  	FALSE
+  	FALSE,
+    TRUE
   };
 
   if (file_merge_w != NULL) {
@@ -787,7 +793,7 @@ file_merge_cmd(GtkWidget *w)
   file_merge_w = file_selection_new("Ethereal: Merge with Capture File",
                                    FILE_SELECTION_OPEN);
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
-  /* it's annoying, that the file chooser dialog is already shown here, 
+  /* it's annoying, that the file chooser dialog is already shown here,
      so we cannot use the correct gtk_window_set_default_size() to resize it */
   WIDGET_SET_SIZE(GTK_WINDOW(file_merge_w), DEF_WIDTH, DEF_HEIGHT);
 #else
@@ -821,7 +827,7 @@ file_merge_cmd(GtkWidget *w)
       file_selection_set_current_folder(file_merge_w, prefs.gui_fileopen_dir);
     break;
   }
-    
+
   main_hb = gtk_hbox_new(FALSE, 3);
   file_selection_set_extra_widget(file_merge_w, main_hb);
   gtk_widget_show(main_hb);
@@ -874,7 +880,7 @@ file_merge_cmd(GtkWidget *w)
 #endif
 
   prepend_rb = RADIO_BUTTON_NEW_WITH_MNEMONIC(NULL, "Prepend packets to existing file", accel_group);
-  gtk_tooltips_set_tip(tooltips, prepend_rb, 
+  gtk_tooltips_set_tip(tooltips, prepend_rb,
       "The resulting file contains the packets from the selected, followed by the packets from the currently loaded file,"
       " the packet timestamps will be ignored.", NULL);
   gtk_box_pack_start(GTK_BOX(main_vb), prepend_rb, FALSE, FALSE, 0);
@@ -888,7 +894,7 @@ file_merge_cmd(GtkWidget *w)
   gtk_widget_show(prepend_rb);
 
   chrono_rb = RADIO_BUTTON_NEW_WITH_MNEMONIC(prepend_rb, "Merge packets chronologically", accel_group);
-  gtk_tooltips_set_tip(tooltips, chrono_rb, 
+  gtk_tooltips_set_tip(tooltips, chrono_rb,
       "The resulting file contains all the packets from the currently loaded and the selected file,"
       " sorted by the packet timestamps.", NULL);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chrono_rb), TRUE);
@@ -902,7 +908,7 @@ file_merge_cmd(GtkWidget *w)
 #endif
 
   append_rb = RADIO_BUTTON_NEW_WITH_MNEMONIC(prepend_rb, "Append packets to existing file", accel_group);
-  gtk_tooltips_set_tip(tooltips, append_rb, 
+  gtk_tooltips_set_tip(tooltips, append_rb,
       "The resulting file contains the packets from the currently loaded, followed by the packets from the selected file,"
       " the packet timestamps will be ignored.", NULL);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(append_rb),
@@ -926,7 +932,7 @@ file_merge_cmd(GtkWidget *w)
   gtk_box_pack_start(GTK_BOX(main_hb), prev, TRUE, TRUE, 0);
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
-  SIGNAL_CONNECT(GTK_FILE_CHOOSER(file_merge_w), "selection-changed", 
+  SIGNAL_CONNECT(GTK_FILE_CHOOSER(file_merge_w), "selection-changed",
       file_open_entry_changed, file_merge_w);
   file_open_entry_changed(file_merge_w, file_merge_w);
 
@@ -937,8 +943,8 @@ file_merge_cmd(GtkWidget *w)
     file_merge_ok_cb(file_merge_w, file_merge_w);
   }
   else window_destroy(file_merge_w);
-#else
-  SIGNAL_CONNECT(GTK_FILE_SELECTION(file_merge_w)->selection_entry, "changed", 
+#else /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+  SIGNAL_CONNECT(GTK_FILE_SELECTION(file_merge_w)->selection_entry, "changed",
       file_open_entry_changed, file_merge_w);
 
   /* Connect the ok_button to file_merge_ok_cb function and pass along a
@@ -950,14 +956,15 @@ file_merge_cmd(GtkWidget *w)
                   E_DFILTER_TE_KEY, OBJECT_GET_DATA(w, E_DFILTER_TE_KEY));
 
   /* Connect the cancel_button to destroy the widget */
-  window_set_cancel_button(file_merge_w, 
+  window_set_cancel_button(file_merge_w,
       GTK_FILE_SELECTION(file_merge_w)->cancel_button, window_cancel_button_cb);
 
   SIGNAL_CONNECT(file_merge_w, "delete_event", window_delete_event_cb, NULL);
 
   gtk_widget_show(file_merge_w);
   window_present(file_merge_w);
-#endif
+#endif /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+#endif /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
 }
 
 static void file_merge_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
@@ -1052,7 +1059,7 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
   }
 
   g_free(cf_name);
-  
+
   if (merge_status != CF_OK) {
     if (rfcode != NULL)
       dfilter_free(rfcode);
@@ -1116,7 +1123,7 @@ file_merge_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 }
 
 
-void file_close_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
+static void file_close_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
 {
     switch(btn) {
     case(ESD_BTN_SAVE):
@@ -1171,20 +1178,8 @@ can_save_with_wiretap(int ft)
 }
 
 
-/* Generate a list of the file types we can save this file as.
-
-   "filetype" is the type it has now.
-
-   "encap" is the encapsulation for its packets (which could be
-   "unknown" or "per-packet").
-
-   "filtered" is TRUE if we're to save only the packets that passed
-   the display filter (in which case we have to save it using Wiretap)
-   and FALSE if we're to save the entire file (in which case, if we're
-   saving it in the type it has already, we can just copy it).
-
-   The same applies for sel_curr, sel_all, sel_m_only, sel_m_range and sel_man_range
-*/
+/* Generate a list of the file types we can save this file as, by
+   checking what Wiretap supports. */
 static void
 set_file_type_list(GtkWidget *option_menu)
 {
@@ -1202,23 +1197,19 @@ set_file_type_list(GtkWidget *option_menu)
   /* Check all file types. */
   index = 0;
   for (ft = 0; ft < WTAP_NUM_FILE_TYPES; ft++) {
-    if (!packet_range_process_all(&range) || ft != cfile.cd_t) {
-      /* not all unfiltered packets or a different file type.  We have to use Wiretap. */
-      if (!can_save_with_wiretap(ft))
-        continue;	/* We can't. */
+    if (can_save_with_wiretap(ft)) {
+      /* OK, we can write it out in this type. */
+      ft_menu_item = gtk_menu_item_new_with_label(wtap_file_type_string(ft));
+      if (ft == filetype) {
+        /* Default to the same format as the file, if it's supported. */
+        item_to_select = index;
+      }
+      SIGNAL_CONNECT(ft_menu_item, "activate", select_file_type_cb,
+                     GINT_TO_POINTER(ft));
+      gtk_menu_append(GTK_MENU(ft_menu), ft_menu_item);
+      gtk_widget_show(ft_menu_item);
+      index++;
     }
-
-    /* OK, we can write it out in this type. */
-    ft_menu_item = gtk_menu_item_new_with_label(wtap_file_type_string(ft));
-    if (ft == filetype) {
-      /* Default to the same format as the file, if it's supported. */
-      item_to_select = index;
-    }
-    SIGNAL_CONNECT(ft_menu_item, "activate", select_file_type_cb,
-                   GINT_TO_POINTER(ft));
-    gtk_menu_append(GTK_MENU(ft_menu), ft_menu_item);
-    gtk_widget_show(ft_menu_item);
-    index++;
   }
 
   gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), ft_menu);
@@ -1229,46 +1220,29 @@ static void
 select_file_type_cb(GtkWidget *w _U_, gpointer data)
 {
   int new_filetype = GPOINTER_TO_INT(data);
+  GtkWidget *compressed_cb;
 
   if (filetype != new_filetype) {
-    /* We can select only the filtered or marked packets to be saved if we can
-       use Wiretap to save the file. */
-    if (range_tb != NULL)
-       range_set_displayed_sensitive(range_tb, can_save_with_wiretap(new_filetype));
     filetype = new_filetype;
-    file_set_save_marked_sensitive();
+    compressed_cb = OBJECT_GET_DATA(file_save_as_w, "compressed");
+    gtk_widget_set_sensitive(compressed_cb, wtap_dump_can_compress(new_filetype));
   }
 }
 
 
 /*
- * Set the "Save only marked packets" toggle button as appropriate for
- * the current output file type and count of marked packets.
- *
- * Called when the "Save As..." dialog box is created and when either
- * the file type or the marked count changes.
+ * Update various dynamic parts of the range controls; called from outside
+ * the file dialog code whenever the packet counts change.
  */
 void
-file_set_save_marked_sensitive(void)
+file_save_update_dynamics(void)
 {
   if (file_save_as_w == NULL) {
     /* We don't currently have a "Save As..." dialog box up. */
     return;
   }
-	
-  /* We can request that only the marked packets be saved only if we
-     can use Wiretap to save the file and if there *are* marked packets. */
-  if (can_save_with_wiretap(filetype) && cfile.marked_count > 0) {
-    range_set_marked_sensitive(range_tb, TRUE);
-  }
-  else {
-    /* Force the "Save only marked packets" toggle to "false", turn
-       off the flag it controls, and update the list of types we can
-       save the file as. */
-    range.process = range_process_all;
-    set_file_type_list(ft_om);
-    range_set_marked_sensitive(range_tb, FALSE);
-  }
+
+  range_update_dynamics(range_tb);
 }
 
 
@@ -1279,13 +1253,16 @@ gpointer            action_after_save_data_g;
 void
 file_save_as_cmd(action_after_save_e action_after_save, gpointer action_after_save_data)
 {
-  GtkWidget     *main_vb, *ft_hb, *ft_lb, *range_fr;
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_save_as_file(GDK_WINDOW_HWND(top_level->window), action_after_save, action_after_save_data);
+#else /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
+  GtkWidget     *main_vb, *ft_hb, *ft_lb, *range_fr, *compressed_cb;
   GtkTooltips   *tooltips;
 
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
-	  
+
   if (file_save_as_w != NULL) {
     /* There's already an "Save Capture File As" dialog box; reactivate it. */
     reactivate_window(file_save_as_w);
@@ -1300,7 +1277,7 @@ file_save_as_cmd(action_after_save_e action_after_save, gpointer action_after_sa
 
   /* Enable tooltips */
   tooltips = gtk_tooltips_new();
-	  
+
   /* build the file selection */
   file_save_as_w = file_selection_new ("Ethereal: Save Capture File As",
                                        FILE_SELECTION_SAVE);
@@ -1314,19 +1291,19 @@ file_save_as_cmd(action_after_save_e action_after_save, gpointer action_after_sa
   accel_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(file_save_as_w), accel_group);
 #endif
-	
+
   /* Container for each row of widgets */
-       
+
   main_vb = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
   file_selection_set_extra_widget(file_save_as_w, main_vb);
-  gtk_widget_show(main_vb);	
-		
+  gtk_widget_show(main_vb);
+
   /*** Packet Range frame ***/
   range_fr = gtk_frame_new("Packet Range");
   gtk_box_pack_start(GTK_BOX(main_vb), range_fr, FALSE, FALSE, 0);
   gtk_widget_show(range_fr);
-  
+
   /* range table */
   range_tb = range_new(&range
 #if GTK_MAJOR_VERSION < 2
@@ -1352,18 +1329,20 @@ file_save_as_cmd(action_after_save_e action_after_save, gpointer action_after_sa
   gtk_box_pack_start(GTK_BOX(ft_hb), ft_om, FALSE, FALSE, 0);
   gtk_widget_show(ft_om);
 
-  /*
-   * Set the sensitivity of the "Save only marked packets" toggle
-   * button
-   *
-   * This has to be done after we create the file type menu option,
-   * as the routine that sets it also sets that menu.
-   */
-  file_set_save_marked_sensitive();
-	
   /* dynamic values in the range frame */
   range_update_dynamics(range_tb);
 
+  /* compressed */
+  compressed_cb = gtk_check_button_new_with_label("Compress with gzip");
+  gtk_container_add(GTK_CONTAINER(ft_hb), compressed_cb);
+  /* XXX - disable output compression for now, as this doesn't work with the
+   * current optimization to simply copy a capture file if it's using the same
+   * encapsulation ... */
+  /* the rest of the implementation is just working fine :-( */
+  /*gtk_widget_show(compressed_cb);*/
+  OBJECT_SET_DATA(file_save_as_w, "compressed", compressed_cb);
+  gtk_widget_set_sensitive(compressed_cb, wtap_dump_can_compress(cfile.cd_t));
+
   SIGNAL_CONNECT(file_save_as_w, "destroy", file_save_as_destroy_cb, NULL);
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
@@ -1372,20 +1351,21 @@ file_save_as_cmd(action_after_save_e action_after_save, gpointer action_after_sa
   } else {
     window_destroy(file_save_as_w);
   }
-#else
+#else /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
   /* Connect the ok_button to file_save_as_ok_cb function and pass along a
      pointer to the file selection box widget */
   SIGNAL_CONNECT(GTK_FILE_SELECTION (file_save_as_w)->ok_button, "clicked",
                  file_save_as_ok_cb, file_save_as_w);
 
-  window_set_cancel_button(file_save_as_w, 
+  window_set_cancel_button(file_save_as_w,
       GTK_FILE_SELECTION(file_save_as_w)->cancel_button, window_cancel_button_cb);
 
   SIGNAL_CONNECT(file_save_as_w, "delete_event", window_delete_event_cb, NULL);
 
   gtk_widget_show(file_save_as_w);
   window_present(file_save_as_w);
-#endif
+#endif /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+#endif /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
 }
 
 void
@@ -1394,10 +1374,15 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   file_save_as_cmd(after_save_no_action, NULL);
 }
 
+
+/* all tests ok, we only have to save the file */
+/* (and probably continue with a pending operation) */
 static void
-file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
+file_save_as_cb(GtkWidget *w _U_, gpointer fs) {
   gchar	*cf_name;
   gchar	*dirname;
+  GtkWidget *compressed_cb;
+
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
   cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
@@ -1405,41 +1390,12 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
   cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
 #endif
 
-  /* Perhaps the user specified a directory instead of a file.
-     Check whether they did. */
-  if (test_for_directory(cf_name) == EISDIR) {
-        /* It's a directory - set the file selection box to display that
-           directory, and leave the selection box displayed. */
-        set_last_open_dir(cf_name);
-        g_free(cf_name);
-        file_selection_set_current_folder(fs, get_last_open_dir());
-        return;
-  }
-
-  /* Check whether the range is valid. */
-  if (!range_check_validity(&range)) {
-    /* The range isn't valid; don't dismiss the open dialog box,
-       just leave it around so that the user can, after they
-       dismiss the alert box popped up for the error, try again. */
-    g_free(cf_name);
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
-    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
-     * as this will prevent the user from closing the now existing error
-     * message, simply close the dialog (this is the best we can do here). */
-    if (file_save_as_w)
-      window_destroy(GTK_WIDGET (fs));
-#else
-    gtk_widget_show(GTK_WIDGET (fs));
-#endif
-    return;
-  }
-
-  /* don't show the dialog while saving */
-  gtk_widget_hide(GTK_WIDGET (fs));
+  compressed_cb = OBJECT_GET_DATA(file_save_as_w, "compressed");
 
   /* Write out the packets (all, or only the ones from the current
      range) to the file with the specified name. */
-  if (cf_save(&cfile, cf_name, &range, filetype) != CF_OK) {
+  if (cf_save(&cfile, cf_name, &range, filetype,
+	  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(compressed_cb))) != CF_OK) {
     /* The write failed; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the error, try again. */
@@ -1484,7 +1440,7 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
       break;
 #ifdef HAVE_LIBPCAP
   case(after_save_capture_dialog):
-      capture_prep();
+      capture_start_confirmed();
       break;
 #endif
   case(after_save_close_file):
@@ -1500,6 +1456,117 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
   action_after_save_g = after_save_no_action;
 }
 
+
+static void file_save_as_exists_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
+{
+    gchar	*cf_name;
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(data)));
+#else
+    cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
+#endif
+
+    switch(btn) {
+    case(ESD_BTN_OK):
+        /* save file */
+        unlink(cf_name);
+        file_save_as_cb(NULL, data);
+        break;
+    case(ESD_BTN_CANCEL):
+        break;
+    default:
+        g_assert_not_reached();
+    }
+}
+
+
+/* user pressed "Save" dialog "Ok" button */
+static void
+file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
+  gchar	*cf_name;
+  gpointer  dialog;
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+  cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
+#else
+  cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+#endif
+
+  /* Perhaps the user specified a directory instead of a file.
+     Check whether they did. */
+  if (test_for_directory(cf_name) == EISDIR) {
+        /* It's a directory - set the file selection box to display that
+           directory, and leave the selection box displayed. */
+        set_last_open_dir(cf_name);
+        g_free(cf_name);
+        file_selection_set_current_folder(fs, get_last_open_dir());
+        return;
+  }
+
+  /* Check whether the range is valid. */
+  if (!range_check_validity(&range)) {
+    /* The range isn't valid; don't dismiss the open dialog box,
+       just leave it around so that the user can, after they
+       dismiss the alert box popped up for the error, try again. */
+    g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#else
+    gtk_widget_show(GTK_WIDGET (fs));
+#endif
+    return;
+  }
+
+  /*
+   * Check that the from file is not the same as to file
+   * We do it here so we catch all cases ...
+   * Unfortunately, the file requester gives us an absolute file
+   * name and the read file name may be relative (if supplied on
+   * the command line). From Joerg Mayer.
+   */
+  if (files_identical(cfile.filename, cf_name)) {
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+      "%sCapture file: \"%s\" identical to loaded file!%s\n\n"
+      "Please choose a different filename.",
+      simple_dialog_primary_start(), cf_name, simple_dialog_primary_end());
+    g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#else
+    gtk_widget_show(GTK_WIDGET (fs));
+#endif
+    return;
+  }
+
+  /* don't show the dialog while saving (or asking) */
+  gtk_widget_hide(GTK_WIDGET (fs));
+
+  /* it the file doesn't exist, simply try to save it */
+  if (!file_exists(cf_name)) {
+    file_save_as_cb(NULL, fs);
+    g_free(cf_name);
+    return;
+  }
+
+  /* the file exists, ask the user to remove it first */
+  dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_OK_CANCEL,
+      PRIMARY_TEXT_START "A file named \"%s\" already exists."
+      PRIMARY_TEXT_END "\n\n"
+      "Do you want to replace it with the capture you are saving?", cf_name);
+  simple_dialog_set_cb(dialog, file_save_as_exists_answered_cb, fs);
+
+  g_free(cf_name);
+}
+
 void
 file_save_as_destroy(void)
 {
@@ -1553,10 +1620,14 @@ color_global_cb(GtkWidget *widget _U_, gpointer data)
 void
 file_color_import_cmd_cb(GtkWidget *w _U_, gpointer data)
 {
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_import_color_file(GDK_WINDOW_HWND(top_level->window));
+#else /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
   GtkWidget	*main_vb, *cfglobal_but;
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
+
   /* No Apply button, and "OK" just sets our text widget, it doesn't
      activate it (i.e., it doesn't cause us to try to open the file). */
 
@@ -1599,7 +1670,7 @@ file_color_import_cmd_cb(GtkWidget *w _U_, gpointer data)
       file_color_import_ok_cb(file_color_import_w, file_color_import_w);
   }
   else window_destroy(file_color_import_w);
-#else
+#else /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
   /* Connect the ok_button to file_open_ok_cb function and pass along a
      pointer to the file selection box widget */
   SIGNAL_CONNECT(GTK_FILE_SELECTION(file_color_import_w)->ok_button, "clicked",
@@ -1608,7 +1679,7 @@ file_color_import_cmd_cb(GtkWidget *w _U_, gpointer data)
   OBJECT_SET_DATA(GTK_FILE_SELECTION(file_color_import_w)->ok_button,
                   ARGUMENT_CL, data);
 
-  window_set_cancel_button(file_color_import_w, 
+  window_set_cancel_button(file_color_import_w,
       GTK_FILE_SELECTION(file_color_import_w)->cancel_button, window_cancel_button_cb);
 
   SIGNAL_CONNECT(file_color_import_w, "delete_event", window_delete_event_cb, NULL);
@@ -1616,7 +1687,8 @@ file_color_import_cmd_cb(GtkWidget *w _U_, gpointer data)
 
   gtk_widget_show(file_color_import_w);
   window_present(file_color_import_w);
-#endif
+#endif /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+#endif /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
 }
 
 static void
@@ -1624,8 +1696,8 @@ file_color_import_ok_cb(GtkWidget *w, gpointer fs) {
   gchar     *cf_name, *s;
   gpointer  argument;
 
-  argument = OBJECT_GET_DATA(w, ARGUMENT_CL);     /* to be passed back into read_other_filters */
-  
+  argument = OBJECT_GET_DATA(w, ARGUMENT_CL);     /* to be passed back into color_filters_import */
+
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
   cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
 #else
@@ -1635,16 +1707,16 @@ file_color_import_ok_cb(GtkWidget *w, gpointer fs) {
      Check whether they did. */
   if (test_for_directory(cf_name) == EISDIR) {
 	/* It's a directory - set the file selection box to display that
-	   directory, don't try to open the directory as a capture file. */
+	   directory, don't try to open the directory as a color filter file. */
         set_last_open_dir(cf_name);
         g_free(cf_name);
         file_selection_set_current_folder(fs, get_last_open_dir());
     	return;
   }
 
-  /* Try to open the capture file. */
+  /* Try to open the color filter file. */
 
-  if (!read_other_filters(cf_name, argument)) {
+  if (!color_filters_import(cf_name, argument)) {
     /* We couldn't open it; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the open error,
@@ -1681,7 +1753,7 @@ static GtkWidget *file_color_export_w;
  * Called when the "Export" dialog box is created and when the marked
  * count changes.
  */
-void
+static void
 color_set_export_marked_sensitive(GtkWidget * cfmark_cb)
 {
   if (file_color_export_w == NULL) {
@@ -1711,6 +1783,9 @@ color_toggle_marked_cb(GtkWidget *widget, gpointer data _U_)
 void
 file_color_export_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
 {
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_color_file(GDK_WINDOW_HWND(top_level->window));
+#else /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
   GtkWidget *main_vb, *cfglobal_but;
 
   if (file_color_export_w != NULL) {
@@ -1752,13 +1827,13 @@ file_color_export_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
       file_color_export_ok_cb(file_color_export_w, file_color_export_w);
   }
   else window_destroy(file_color_export_w);
-#else
+#else /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
   /* Connect the ok_button to file_export_ok_cb function and pass along a
      pointer to the file selection box widget */
   SIGNAL_CONNECT(GTK_FILE_SELECTION (file_color_export_w)->ok_button, "clicked",
                  file_color_export_ok_cb, file_color_export_w);
 
-  window_set_cancel_button(file_color_export_w, 
+  window_set_cancel_button(file_color_export_w,
       GTK_FILE_SELECTION(file_color_export_w)->cancel_button, window_cancel_button_cb);
 
   SIGNAL_CONNECT(file_color_export_w, "delete_event", window_delete_event_cb, NULL);
@@ -1768,7 +1843,8 @@ file_color_export_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
 
   gtk_widget_show(file_color_export_w);
   window_present(file_color_export_w);
-#endif
+#endif /* (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2 */
+#endif /* GTK_MAJOR_VERSION >= 2 && _WIN32 */
 }
 
 static void
@@ -1796,7 +1872,7 @@ file_color_export_ok_cb(GtkWidget *w _U_, gpointer fs) {
   /* Write out the filters (all, or only the ones that are currently
      displayed or marked) to the file with the specified name. */
 
-   if (!write_other_filters(cf_name, color_marked))
+   if (!color_filters_export(cf_name, color_marked))
    {
     /* The write failed; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
