@@ -1,7 +1,7 @@
 /* find_dlg.c
  * Routines for "find frame" window
  *
- * $Id: find_dlg.c 14829 2005-07-01 20:47:22Z ulfl $
+ * $Id: find_dlg.c 17560 2006-03-09 23:13:49Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include <epan/strutil.h>
 #include "globals.h"
 
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "find_dlg.h"
 #include "filter_dlg.h"
 #include "alert_box.h"
@@ -125,7 +125,8 @@ find_frame_cb(GtkWidget *w _U_, gpointer d _U_)
   static construct_args_t args = {
   	"Ethereal: Search Filter",
   	FALSE,
-  	TRUE
+  	TRUE,
+    FALSE
   };
 
   if (find_frame_w != NULL) {
@@ -539,9 +540,9 @@ find_frame_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w)
   const gchar     *filter_text, *string_type;
   search_charset_t scs_type = SCS_ASCII_AND_UNICODE;
   guint8          *bytes = NULL;
-  size_t           nbytes;
+  size_t           nbytes = 0;
   char            *string = NULL;
-  dfilter_t       *sfcode;
+  dfilter_t       *sfcode = NULL;
   gboolean        found_packet;
 
   filter_te = (GtkWidget *)OBJECT_GET_DATA(parent_w, E_FIND_FILT_KEY);
@@ -783,7 +784,7 @@ find_previous_cb(GtkWidget *w , gpointer d)
 
 /* this function jumps to the next packet matching the filter */
 void
-find_previous_next_frame_with_filter(char *filter, gboolean backwards)
+find_previous_next_frame_with_filter(const char *filter, gboolean backwards)
 {
   dfilter_t *sfcode;
   gboolean sbackwards_saved;
