@@ -1,6 +1,6 @@
 /* follow_dlg.c
  *
- * $Id: follow_dlg.c 13366 2005-02-09 23:38:00Z lroland $
+ * $Id: follow_dlg.c 17280 2006-02-13 01:30:03Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -30,9 +30,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_IO_H
-#include <io.h>			/* open/close on win32 */
-#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -42,6 +39,7 @@
 
 #include "isprint.h"
 
+#include "file_util.h"
 #include "color.h"
 #include "colors.h"
 #include "file.h"
@@ -57,16 +55,23 @@
 #include <epan/prefs.h>
 #include <epan/addr_resolv.h>
 #include <epan/charsets.h>
-#include "util.h"
-#include "ui_util.h"
+#include "tempfile.h"
+#include "gui_utils.h"
 #include <epan/epan_dissect.h>
 #include <epan/filesystem.h>
 #include "compat_macros.h"
 #include <epan/ipproto.h>
-#include "tap_menu.h"
 #include "print_mswin.h"
 #include "font_utils.h"
 
+/* This is backwards-compatibility code for old versions of GTK+ (2.2.1 and
+ * earlier).  It defines the new wrap behavior (unknown in earlier versions)
+ * as the old (slightly buggy) wrap behavior.
+ */
+#ifndef GTK_WRAP_WORD_CHAR
+#define GTK_WRAP_WORD_CHAR GTK_WRAP_WORD
+#endif
+
 /* Show Stream */
 typedef enum {
 	FROM_CLIENT,
@@ -204,7 +209,7 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			  "Could not create temporary file %s: %s",
 			  follow_info->data_out_filename, strerror(errno));
-	    close(tmp_fd);
+	    eth_close(tmp_fd);
 	    unlink(follow_info->data_out_filename);
 	    g_free(follow_info);
 	    return;
@@ -296,6 +301,7 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 #else
         text = gtk_text_view_new();
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
 #endif
 	gtk_container_add(GTK_CONTAINER(txt_scrollw), text);
 	follow_info->text = text;
@@ -602,7 +608,7 @@ follow_read_stream(follow_info_t *follow_info,
 
     iplen = (follow_info->is_ipv6) ? 16 : 4;
 
-    data_out_file = fopen(follow_info->data_out_filename, "rb");
+    data_out_file = eth_fopen(follow_info->data_out_filename, "rb");
     if (data_out_file == NULL) {
 	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		      "Could not open temporary file %s: %s", follow_info->data_out_filename,
@@ -932,7 +938,7 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
         print_mswin(print_dest);
 
         /* trash temp file */
-        remove(print_dest);
+        eth_remove(print_dest);
     }
 #endif
     return;
@@ -950,7 +956,7 @@ print_error:
 #ifdef _WIN32
     if (win_printer) {
         /* trash temp file */
-        remove(print_dest);
+        eth_remove(print_dest);
     }
 #endif
 }
@@ -1139,10 +1145,10 @@ follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
     follow_info = OBJECT_GET_DATA(fs, E_FOLLOW_INFO_KEY);
     if (follow_info->show_type == SHOW_RAW) {
         /* Write the data out as raw binary data */
-        fh = fopen(to_name, "wb");
+        fh = eth_fopen(to_name, "wb");
     } else {
         /* Write it out as text */
-        fh = fopen(to_name, "w");
+        fh = eth_fopen(to_name, "w");
     }
     if (fh == NULL) {
         open_failure_alert_box(to_name, errno, TRUE);
