@@ -1,7 +1,7 @@
 /* font_utils.c
  * Utilities to use for font manipulation
  *
- * $Id: font_utils.c 13346 2005-02-07 20:52:59Z guy $
+ * $Id: font_utils.c 17534 2006-03-08 20:55:32Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,8 +35,11 @@
 
 #ifdef _WIN32
 #include <windows.h>
+#include <tchar.h>
+#include <epan/strutil.h>
 #endif
 
+#include "main.h"
 #include "recent.h"
 #include <epan/prefs.h>
 
@@ -534,7 +537,7 @@ user_font_apply(void) {
     }
 
     /* the font(s) seem to be ok */
-    set_plist_font(new_r_font);
+    packet_list_set_font(new_r_font);
     set_ptree_font_all(new_r_font);
     old_r_font = m_r_font;
     old_b_font = m_b_font;
@@ -582,10 +585,10 @@ static int get_windows_font_gtk1(char *fontspec, int fontspec_len)
     HDC h_dc;
     HGDIOBJ h_font;
     TEXTMETRIC tm;
-    char name[NAME_BUFFER_LEN];
+    TCHAR name[NAME_BUFFER_LEN];
     int len, pix_height;
 
-    h_dc = CreateDC("DISPLAY", NULL, NULL, NULL);
+    h_dc = CreateDC(_T("DISPLAY"), NULL, NULL, NULL);
     if (h_dc == NULL) return 1;
     h_font = GetStockObject(DEFAULT_GUI_FONT);
     if (h_font == NULL || !SelectObject(h_dc, h_font)) {
@@ -603,8 +606,8 @@ static int get_windows_font_gtk1(char *fontspec, int fontspec_len)
     }
     pix_height = tm.tmHeight;
     DeleteDC(h_dc);
-    g_snprintf(fontspec, fontspec_len, "-*-%s-*-*-*-*-%i-*-*-*-p-*-iso8859-1", name,
-            pix_height);
+    g_snprintf(fontspec, fontspec_len, "-*-%s-*-*-*-*-%i-*-*-*-p-*-iso8859-1",
+            utf_16to8(name), pix_height);
     return 0;
 }
 
@@ -710,11 +713,7 @@ static void try_to_get_windows_font_gtk2(void)
 #endif /* _WIN32 */
 
 
-void font_init(gboolean capture_child
-#ifndef HAVE_LIBPCAP
-	_U_
-#endif
-)
+void font_init(void)
 {
 #if GTK_MAJOR_VERSION < 2
   gchar *bold_font_name;
@@ -735,18 +734,12 @@ void font_init(gboolean capture_child
   if (m_r_font == NULL || m_b_font == NULL) {
     /* XXX - pop this up as a dialog box? no */
     if (m_r_font == NULL) {
-#ifdef HAVE_LIBPCAP
-      if (!capture_child)
-#endif
 	fprintf(stderr, "ethereal: Warning: font %s not found - defaulting to 6x13 and 6x13bold\n",
 		prefs.gui_font_name1);
     } else {
       gdk_font_unref(m_r_font);
     }
     if (m_b_font == NULL) {
-#ifdef HAVE_LIBPCAP
-      if (!capture_child)
-#endif
 	fprintf(stderr, "ethereal: Warning: font %s not found - defaulting to 6x13 and 6x13bold\n",
 		bold_font_name);
     } else {
@@ -771,18 +764,12 @@ void font_init(gboolean capture_child
   if (m_r_font == NULL || m_b_font == NULL) {
     /* XXX - pop this up as a dialog box? no */
     if (m_r_font == NULL) {
-#ifdef HAVE_LIBPCAP
-      if (!capture_child)
-#endif
 	fprintf(stderr, "ethereal: Warning: font %s not found - defaulting to Monospace 9\n",
 		prefs.gui_font_name2);
     } else {
       pango_font_description_free(m_r_font);
     }
     if (m_b_font == NULL) {
-#ifdef HAVE_LIBPCAP
-      if (!capture_child)
-#endif
         fprintf(stderr, "ethereal: Warning: bold font %s not found - defaulting"
                         " to Monospace 9\n", prefs.gui_font_name2);
     } else {
