@@ -1,7 +1,7 @@
 /* goto_dlg.c
  * Routines for "go to packet" window
  *
- * $Id: goto_dlg.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: goto_dlg.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include "simple_dialog.h"
 #include "dlg_utils.h"
 #include "compat_macros.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "help_dlg.h"
 
 /* Capture callback data keys */
