@@ -2,7 +2,7 @@
  * h225 RAS Service Response Time statistics for ethereal
  * Copyright 2003 Lars Roland
  *
- * $Id: h225_ras_srt.c 13993 2005-04-01 09:08:27Z guy $
+ * $Id: h225_ras_srt.c 17261 2006-02-12 00:54:56Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,23 +39,22 @@
 #include <epan/epan.h>
 #include <epan/value_string.h>
 
-#include "tap_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include <epan/dissectors/packet-h225.h>
 #include "../timestats.h"
-#include "gtk_stat_util.h"
+#include "gui_stat_util.h"
 #include "compat_macros.h"
 #include "../simple_dialog.h"
 #include "dlg_utils.h"
 #include "../file.h"
 #include "../globals.h"
+#include "../stat_menu.h"
 #include "../tap_dfilter_dlg.h"
-#include "tap_dfilter_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 
 
-static void gtk_h225rassrt_init(char *optarg);
+static void gtk_h225rassrt_init(const char *optarg, void *userdata);
 
 static tap_dfilter_dlg h225_rassrt_dlg = {
 	"H.225 RAS Service Response Time",
@@ -268,7 +267,8 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
 }
 
 
-static gchar *titles[]={"RAS-Type",
+static const gchar *titles[]={
+			"RAS-Type",
 			"Measurements",
 			"Min RTT",
 			"Max RTT",
@@ -281,19 +281,18 @@ static gchar *titles[]={"RAS-Type",
 			"Repeated Responses" };
 
 static void
-gtk_h225rassrt_init(char *optarg)
+gtk_h225rassrt_init(const char *optarg, void *userdata _U_)
 {
 	h225rassrt_t *hs;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString *error_string;
-    GtkWidget *bbox;
-    GtkWidget *close_bt;
+	GtkWidget *bbox;
+	GtkWidget *close_bt;
 
 	if(strncmp(optarg,"h225,srt,",9) == 0){
 		filter=optarg+9;
 	} else {
-		filter=g_malloc(1);
-		*filter='\0';
+		filter="";
 	}
 
 	hs=g_malloc(sizeof(h225rassrt_t));
@@ -328,22 +327,20 @@ gtk_h225rassrt_init(char *optarg)
 	gtk_box_pack_end(GTK_BOX(hs->vbox), bbox, FALSE, FALSE, 0);
 
 	close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
-    window_set_cancel_button(hs->win, close_bt, window_cancel_button_cb);
+	window_set_cancel_button(hs->win, close_bt, window_cancel_button_cb);
 
-    SIGNAL_CONNECT(hs->win, "delete_event", window_delete_event_cb, NULL);
-    SIGNAL_CONNECT(hs->win, "destroy", win_destroy_cb, hs);
+	SIGNAL_CONNECT(hs->win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(hs->win, "destroy", win_destroy_cb, hs);
 
 	gtk_widget_show_all(hs->win);
-    window_present(hs->win);
+	window_present(hs->win);
 
-    cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 }
 
 void
 register_tap_listener_gtk_h225rassrt(void)
 {
-	register_tap_listener_cmd_arg("h225,srt", gtk_h225rassrt_init);
-
-	register_tap_menu_item("H.225 RAS...", REGISTER_TAP_GROUP_RESPONSE_TIME,
-	    gtk_tap_dfilter_dlg_cb, NULL, NULL, &(h225_rassrt_dlg));
+	register_dfilter_stat(&h225_rassrt_dlg, "H.225 RAS",
+	    REGISTER_STAT_GROUP_RESPONSE_TIME);
 }
