@@ -1,7 +1,7 @@
 /* hostlist_fc.c   2004 Ian Schorr
  * modified from endpoint_talkers_fc.c   2003 Ronnie Sahlberg
  *
- * $Id: hostlist_fc.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: hostlist_fc.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "hostlist_table.h"
@@ -58,9 +60,9 @@ fc_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, const
 
 
 static void
-gtk_fc_hostlist_init(char *optarg)
+gtk_fc_hostlist_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"hosts,fc,",9)){
 		filter=optarg+9;
@@ -76,16 +78,16 @@ gtk_fc_hostlist_init(char *optarg)
 static void
 gtk_fc_hostlist_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	gtk_fc_hostlist_init("hosts,fc");
+	gtk_fc_hostlist_init("hosts,fc",NULL);
 }
 
 
 void
 register_tap_listener_fc_hostlist(void)
 {
-	register_ethereal_tap("hosts,fc", gtk_fc_hostlist_init);
+	register_stat_cmd_arg("hosts,fc", gtk_fc_hostlist_init,NULL);
 
-	register_tap_menu_item("Fibre Channel", REGISTER_TAP_GROUP_ENDPOINT_LIST,
+	register_stat_menu_item("Fibre Channel", REGISTER_STAT_GROUP_ENDPOINT_LIST,
 	    gtk_fc_hostlist_cb, NULL, NULL, NULL);
 
 	register_hostlist_table(TRUE, "Fibre Channel", "fc", NULL /*filter*/, fc_hostlist_packet);
