@@ -1,7 +1,7 @@
 /* hostlist_fddi.c   2004 Ian Schorr
  * modified from endpoint_talkers_fddi.c   2003 Ronnie Sahlberg
  *
- * $Id: hostlist_fddi.c 12128 2004-09-29 00:06:36Z guy $
+ * $Id: hostlist_fddi.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "hostlist_table.h"
@@ -41,10 +43,10 @@
 
 
 static int
-fddi_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, void *vip)
+fddi_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, const void *vip)
 {
 	hostlist_table *hosts=(hostlist_table *)pit;
-	fddi_hdr *ehdr=vip;
+	const fddi_hdr *ehdr=vip;
 
 	/* Take two "add" passes per packet, adding for each direction, ensures that all
 	packets are counted properly (even if address is sending to itself) 
@@ -58,9 +60,9 @@ fddi_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, voi
 
 
 static void
-gtk_fddi_hostlist_init(char *optarg)
+gtk_fddi_hostlist_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"hosts,fddi,",11)){
 		filter=optarg+11;
@@ -68,7 +70,7 @@ gtk_fddi_hostlist_init(char *optarg)
 		filter=NULL;
 	}
 
-	init_hostlist_table(TRUE, "FDDI Hosts", "fddi", filter, (void *)fddi_hostlist_packet);
+	init_hostlist_table(TRUE, "FDDI Hosts", "fddi", filter, fddi_hostlist_packet);
 
 }
 
@@ -76,18 +78,17 @@ gtk_fddi_hostlist_init(char *optarg)
 static void
 gtk_fddi_hostlist_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	gtk_fddi_hostlist_init("hosts,fddi");
+	gtk_fddi_hostlist_init("hosts,fddi",NULL);
 }
 
 
 void
 register_tap_listener_fddi_hostlist(void)
 {
-	register_ethereal_tap("hosts,fddi", gtk_fddi_hostlist_init);
+	register_stat_cmd_arg("hosts,fddi", gtk_fddi_hostlist_init,NULL);
 
-	register_tap_menu_item("FDDI", REGISTER_TAP_GROUP_ENDPOINT_LIST,
+	register_stat_menu_item("FDDI", REGISTER_STAT_GROUP_ENDPOINT_LIST,
 	    gtk_fddi_hostlist_cb, NULL, NULL, NULL);
 
-	register_hostlist_table(TRUE, "FDDI", "fddi", NULL /*filter*/, (void *)fddi_hostlist_packet);
+	register_hostlist_table(TRUE, "FDDI", "fddi", NULL /*filter*/, fddi_hostlist_packet);
 }
-
