@@ -2,7 +2,7 @@
  * modified from endpoint_talkers_table.c   2003 Ronnie Sahlberg
  * Helper routines common to all host list taps.
  *
- * $Id: hostlist_table.c 13463 2005-02-22 01:55:04Z lego $
+ * $Id: hostlist_table.c 16974 2006-01-07 11:08:07Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -50,7 +50,7 @@
 #include "gtk/color_dlg.h"
 #include "gtkglobals.h"
 #include "main.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "help_dlg.h"
 
@@ -86,7 +86,7 @@ hostlist_port_to_str(int port_type, guint32 port)
 #define FN_ANY_ADDRESS		0
 #define FN_ANY_PORT		1
 
-/* given an address (to distinguis between ipv4 and ipv6 for tcp/udp
+/* given an address (to distinguish between ipv4 and ipv6 for tcp/udp
    a port_type and a name_type (FN_...)
    return a string for the filter name
 
@@ -94,7 +94,7 @@ hostlist_port_to_str(int port_type, guint32 port)
    of protocols,   either ethernet, tokenring, fddi etc so we must be more
    specific there  thats why we need specific_addr_type
 */
-static char *
+static const char *
 hostlist_get_filter_name(address *addr, int specific_addr_type, int port_type, int name_type)
 {
 	switch(name_type){
@@ -108,7 +108,10 @@ hostlist_get_filter_name(address *addr, int specific_addr_type, int port_type, i
 				return "fddi.addr";
 			case SAT_TOKENRING:
 				return "tr.addr";
+                        default :
+                                ;
 			}
+                        break;
 		case AT_IPv4:
 			return "ip.addr";
 		case AT_IPv6:
@@ -117,6 +120,14 @@ hostlist_get_filter_name(address *addr, int specific_addr_type, int port_type, i
 			return "ipx.addr";
 		case AT_FC:
 			return "fc.id";
+		case AT_URI:
+			switch(specific_addr_type){
+			case SAT_JXTA:
+				return "jxta.message.address";
+                        default:
+                            ;
+			}
+                        break;
 		default:
 			;
 		}
@@ -200,7 +211,7 @@ hostlist_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 {
 	char *text1 = NULL;
 	char *text2 = NULL;
-	int i1, i2;
+	guint64 i1, i2;
 
 	const GtkCListRow *row1 = ptr1;
 	const GtkCListRow *row2 = ptr2;
@@ -209,18 +220,20 @@ hostlist_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 	text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
 
 	switch(clist->sort_column){
-	case 0:
-	case 2:
+	case 0: /* Address */
+	case 1: /* (Port) */
 		return strcmp (text1, text2);
-	case 1:
-	case 3:
-	case 4:
-	case 5:
-	case 6:
-	case 7:
-		i1=atoi(text1);
-		i2=atoi(text2);
-		return i1-i2;
+	case 2: /* Packets */
+	case 3: /* Bytes */
+	case 4: /* Tx Packets */
+	case 5: /* Tx Bytes */
+	case 6: /* Rx Packets */
+	case 7: /* Rx Bytes */
+		sscanf(text1, "%" PRIu64, &i1);
+		sscanf(text2, "%" PRIu64, &i2);
+        /* XXX - this might cause trouble because of overflow problems */
+        /* XXX - is this correct anyway? Subtracting two unsigned values will still be an unsigned value, which will never become negative */
+		return (gint) (i1-i2);
 	}
 	g_assert_not_reached();
 	
@@ -529,21 +542,21 @@ draw_hostlist_table_data(hostlist_table *hl)
 
         j=gtk_clist_find_row_from_data(hl->table, (gpointer)i);
 
-        g_snprintf(str, 16, "%u", hl->hosts[i].tx_frames+hl->hosts[i].rx_frames);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].tx_frames+hl->hosts[i].rx_frames);
         gtk_clist_set_text(hl->table, j, 2, str);
-        g_snprintf(str, 16, "%u", hl->hosts[i].tx_bytes+hl->hosts[i].rx_bytes);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].tx_bytes+hl->hosts[i].rx_bytes);
         gtk_clist_set_text(hl->table, j, 3, str);
 
 
-        g_snprintf(str, 16, "%u", hl->hosts[i].tx_frames);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].tx_frames);
         gtk_clist_set_text(hl->table, j, 4, str);
-        g_snprintf(str, 16, "%u", hl->hosts[i].tx_bytes);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].tx_bytes);
         gtk_clist_set_text(hl->table, j, 5, str);
 
 
-        g_snprintf(str, 16, "%u", hl->hosts[i].rx_frames);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].rx_frames);
         gtk_clist_set_text(hl->table, j, 6, str);
-        g_snprintf(str, 16, "%u", hl->hosts[i].rx_bytes);
+        g_snprintf(str, 16, "%" PRIu64, hl->hosts[i].rx_bytes);
         gtk_clist_set_text(hl->table, j, 7, str);
 
     }
@@ -604,7 +617,7 @@ copy_as_csv_cb(GtkWindow *win _U_, gpointer data)
 
 
 static gboolean
-init_hostlist_table_page(hostlist_table *hosttable, GtkWidget *vbox, gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+init_hostlist_table_page(hostlist_table *hosttable, GtkWidget *vbox, gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     int i;
     column_arrows *col_arrows;
@@ -716,7 +729,7 @@ init_hostlist_table_page(hostlist_table *hosttable, GtkWidget *vbox, gboolean hi
 
 
 void
-init_hostlist_table(gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+init_hostlist_table(gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     hostlist_table *hosttable;
     char title[256];
@@ -766,7 +779,7 @@ init_hostlist_table(gboolean hide_ports, char *table_name, char *tap_name, char
     gtk_widget_show_all(hosttable->win);
     window_present(hosttable->win);
 
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
 	
     /* Keep clist frozen to cause modifications to the clist (inserts, appends, others that are extremely slow
 	   in GTK2) to not be drawn, allow refreshes to occur at strategic points for performance */
@@ -954,7 +967,7 @@ init_hostlist_notebook_cb(GtkWidget *w _U_, gpointer d _U_)
     gtk_widget_show_all(win);
     window_present(win);
 
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
 
     /* after retapping, redraw table */
     for (page=1; page<=GPOINTER_TO_INT(pages[0]); page++) {
@@ -1035,19 +1048,19 @@ add_hostlist_table_data(hostlist_table *hl, const address *addr, guint32 port, g
         entries[0]="";
         entries[1]="";
 
-        g_snprintf(frames, 16, "%u", talker->tx_frames+talker->rx_frames);
+        g_snprintf(frames, 16, "%" PRIu64, talker->tx_frames+talker->rx_frames);
         entries[2]=frames;
-        g_snprintf(bytes, 16, "%u", talker->tx_bytes+talker->rx_bytes);
+        g_snprintf(bytes, 16, "%" PRIu64, talker->tx_bytes+talker->rx_bytes);
         entries[3]=bytes;
 
-        g_snprintf(txframes, 16, "%u", talker->tx_frames);
+        g_snprintf(txframes, 16, "%" PRIu64, talker->tx_frames);
         entries[4]=txframes;
-        g_snprintf(txbytes, 16, "%u", talker->tx_bytes);
+        g_snprintf(txbytes, 16, "%" PRIu64, talker->tx_bytes);
         entries[5]=txbytes;
 
-        g_snprintf(rxframes, 16, "%u", talker->rx_frames);
+        g_snprintf(rxframes, 16, "%" PRIu64, talker->rx_frames);
         entries[6]=rxframes;
-        g_snprintf(rxbytes, 16, "%u", talker->rx_bytes);
+        g_snprintf(rxbytes, 16, "%" PRIu64, talker->rx_bytes);
         entries[7]=rxbytes;
 
         gtk_clist_insert(hl->table, talker_idx, entries);
