@@ -1,7 +1,7 @@
 /* io_stat.c
  * io_stat   2002 Ronnie Sahlberg
  *
- * $Id: io_stat.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: io_stat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -41,8 +41,10 @@
 #include <epan/packet_info.h>
 
 #include "gtkglobals.h"
-#include "ui_util.h"
-#include "tap_menu.h"
+#include "gui_utils.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "alert_box.h"
@@ -73,7 +75,7 @@ static guint32 pixels_per_tick[MAX_PIXELS_PER_TICK] = {1, 2, 5, 10};
 #define PLOT_STYLE_IMPULSE	1
 #define PLOT_STYLE_FILLED_BAR	2
 #define MAX_PLOT_STYLES		3
-static char *plot_style_name[MAX_PLOT_STYLES] = {
+static const char *plot_style_name[MAX_PLOT_STYLES] = {
 	"Line",
 	"Impulse",
 	"FBar",
@@ -84,12 +86,12 @@ static char *plot_style_name[MAX_PLOT_STYLES] = {
 #define COUNT_TYPE_BYTES    1
 #define COUNT_TYPE_ADVANCED 2
 #define MAX_COUNT_TYPES 3
-static char *count_type_names[MAX_COUNT_TYPES] = {"Packets/Tick", "Bytes/Tick", "Advanced..."};
+static const char *count_type_names[MAX_COUNT_TYPES] = {"Packets/Tick", "Bytes/Tick", "Advanced..."};
 
 /* unit is in ms */
 #define MAX_TICK_VALUES 5
 #define DEFAULT_TICK_VALUE 3
-static guint tick_interval_values[MAX_TICK_VALUES] = { 1, 10, 100, 1000, 10000 };
+static const guint tick_interval_values[MAX_TICK_VALUES] = { 1, 10, 100, 1000, 10000 };
 
 #define CALC_TYPE_SUM	0
 #define CALC_TYPE_COUNT	1
@@ -98,7 +100,7 @@ static guint tick_interval_values[MAX_TICK_VALUES] = { 1, 10, 100, 1000, 10000 }
 #define CALC_TYPE_AVG	4
 #define CALC_TYPE_LOAD	5
 #define MAX_CALC_TYPES 6
-static char *calc_type_names[MAX_CALC_TYPES] = {"SUM(*)", "COUNT(*)", "MAX(*)", "MIN(*)", "AVG(*)", "LOAD(*)"};
+static const char *calc_type_names[MAX_CALC_TYPES] = {"SUM(*)", "COUNT(*)", "MAX(*)", "MIN(*)", "AVG(*)", "LOAD(*)"};
 
 
 typedef struct _io_stat_calc_type_t {
@@ -195,12 +197,9 @@ io_stat_reset(io_stat_t *io)
 			ioi->int_max=0;
 			ioi->int_min=0;
 			ioi->int_tot=0;
-			ioi->time_max.secs=0;
-			ioi->time_max.nsecs=0;
-			ioi->time_min.secs=0;
-			ioi->time_min.nsecs=0;
-			ioi->time_tot.secs=0;
-			ioi->time_tot.nsecs=0;
+			nstime_set_zero(&ioi->time_max);
+			nstime_set_zero(&ioi->time_min);
+			nstime_set_zero(&ioi->time_tot);
 		}
 	}
 	io->last_interval=0xffffffff;
@@ -238,8 +237,7 @@ gtk_iostat_packet(void *g, packet_info *pinfo, epan_dissect_t *edt, const void *
 	 * Find which interval this is supposed to to in and store the
 	 * interval index as idx
 	 */
-	time_delta.secs=pinfo->fd->rel_secs;
-	time_delta.nsecs=pinfo->fd->rel_usecs*1000;
+	time_delta=pinfo->fd->rel_ts;
 	if(time_delta.nsecs<0){
 		time_delta.secs--;
 		time_delta.nsecs+=1000000000;
@@ -326,7 +324,7 @@ gtk_iostat_packet(void *g, packet_info *pinfo, epan_dissect_t *edt, const void *
 					t=t*1000000+new_time->nsecs/1000;
 					i=idx;
 					/* handle current interval */
-					pt=pinfo->fd->rel_secs*1000000+pinfo->fd->rel_usecs;
+					pt=pinfo->fd->rel_ts.secs*1000000+pinfo->fd->rel_ts.nsecs/1000;
 					pt=pt%(git->io->interval*1000);
 					if(pt>t){
 						pt=t;
@@ -355,22 +353,15 @@ gtk_iostat_packet(void *g, packet_info *pinfo, epan_dissect_t *edt, const void *
 					||( (new_time->secs==it->time_max.secs)
 					  &&(new_time->nsecs>it->time_max.nsecs))
 					||(it->frames==0)){
-						it->time_max.secs=new_time->secs;
-						it->time_max.nsecs=new_time->nsecs;
+						it->time_max=*new_time;
 					}
 					if( (new_time->secs<it->time_min.secs)
 					||( (new_time->secs==it->time_min.secs)
 					  &&(new_time->nsecs<it->time_min.nsecs))
 					||(it->frames==0)){
-						it->time_min.secs=new_time->secs;
-						it->time_min.nsecs=new_time->nsecs;
-					}
-					it->time_tot.secs+=new_time->secs;
-					it->time_tot.nsecs+=new_time->nsecs;
-					if(it->time_tot.nsecs>=1000000000){
-						it->time_tot.nsecs-=1000000000;
-						it->time_tot.secs++;
+						it->time_min=*new_time;
 					}
+					nstime_add(&it->time_tot, new_time);
 				}
 
 			}
@@ -455,6 +446,9 @@ get_it_value(io_stat_t *io, int graph_id, int idx)
 		case CALC_TYPE_MIN:
 			value=it->time_min.secs*1000000+it->time_min.nsecs/1000;
 			break;
+		case CALC_TYPE_SUM:
+			value=it->time_tot.secs*1000000+it->time_tot.nsecs/1000;
+			break;
 		case CALC_TYPE_AVG:
 			if(it->frames){
 #ifdef G_HAVE_UINT64
@@ -623,6 +617,7 @@ io_stat_draw(io_stat_t *io)
 			switch(adv_type){
 			case FT_RELATIVE_TIME:
 				switch(io->graphs[i].calc_type){
+				case CALC_TYPE_SUM:
 				case CALC_TYPE_MAX:
 				case CALC_TYPE_MIN:
 				case CALC_TYPE_AVG:
@@ -891,20 +886,16 @@ io_stat_draw(io_stat_t *io)
 				y_pos=draw_height-1-(val*draw_height)/max_y+top_y_border;
 			}
 
-			/* dont need to draw anything if the segment
-			 * is entirely above the top of the graph 
-			 */
-			if( (prev_y_pos==0) && (y_pos==0) ){
-				prev_y_pos=y_pos;
-				prev_x_pos=x_pos;
-				continue;
-			}
-
 			switch(io->graphs[i].plot_style){
 			case PLOT_STYLE_LINE:
-				gdk_draw_line(io->pixmap, io->graphs[i].gc, 
-					prev_x_pos, prev_y_pos, 
-					x_pos, y_pos);
+				/* dont need to draw anything if the segment
+				 * is entirely above the top of the graph
+				 */
+				if( (prev_y_pos!=0) || (y_pos!=0) ){
+					gdk_draw_line(io->pixmap, io->graphs[i].gc,
+						prev_x_pos, prev_y_pos,
+						x_pos, y_pos);
+				}
 				break;
 			case PLOT_STYLE_IMPULSE:
 				if(val){
@@ -1044,7 +1035,7 @@ disable_graph(io_stat_graph_t *gio)
 }
 
 static void
-gtk_iostat_init(char *optarg _U_)
+gtk_iostat_init(const char *optarg _U_, void* userdata _U_)
 {
 	io_stat_t *io;
 	int i=0;
@@ -1067,7 +1058,7 @@ gtk_iostat_init(char *optarg _U_)
 	io->scrollbar_adjustment=NULL;
 	io->pixmap_width=500;
 	io->pixmap_height=200;
-    io->pixels_per_tick=pixels_per_tick[DEFAULT_PIXELS_PER_TICK];
+	io->pixels_per_tick=pixels_per_tick[DEFAULT_PIXELS_PER_TICK];
 	io->max_y_units=AUTO_MAX_YSCALE;
 	io->count_type=0;
 	io->last_interval=0xffffffff;
@@ -1090,6 +1081,7 @@ gtk_iostat_init(char *optarg _U_)
 		io->graphs[i].args->title = NULL;
 		io->graphs[i].args->wants_apply_button=TRUE;
 		io->graphs[i].args->activate_on_ok=TRUE;
+		io->graphs[i].args->modal_and_transient=FALSE;
 
 		io->graphs[i].filter_bt=NULL;
 	}
@@ -1110,7 +1102,7 @@ gtk_iostat_init(char *optarg _U_)
 	/* build the GUI */
 	init_io_stat_window(io);
 
-	cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 	io_stat_redraw(io);
 }
 
@@ -1128,7 +1120,7 @@ quit(GtkWidget *widget, GdkEventExpose *event _U_)
 			remove_tap_listener(&io->graphs[i]);
 			unprotect_thread_critical_region();
 
-			free(io->graphs[i].args->title);
+			g_free( (gpointer) (io->graphs[i].args->title) );
 			io->graphs[i].args->title=NULL;
 
 			g_free(io->graphs[i].args);
@@ -1270,7 +1262,7 @@ tick_interval_select(GtkWidget *item, gpointer key)
 	val=(int)OBJECT_GET_DATA(item, "tick_interval");
 
 	io->interval=val;
-	cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 	io_stat_redraw(io);
 }
 
@@ -1439,7 +1431,7 @@ create_frames_or_bytes_menu_items(io_stat_t *io, GtkWidget *menu)
 }
 
 static void
-create_ctrl_menu(io_stat_t *io, GtkWidget *box, char *name, void (*func)(io_stat_t *io, GtkWidget *menu))
+create_ctrl_menu(io_stat_t *io, GtkWidget *box, const char *name, void (*func)(io_stat_t *io, GtkWidget *menu))
 {
 	GtkWidget *hbox;
 	GtkWidget *label;
@@ -1533,7 +1525,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 		/* warn and bail out if the field could not be found */
 		hfi=proto_registrar_get_byname(field);
 		if(hfi==NULL){
-			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "'%s' isn't a valid field name.", field);
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "There is no field named '%s'.", field);
 			disable_graph(gio);
 			io_stat_redraw(gio->io);
 			return 0;
@@ -1563,6 +1555,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 		case FT_RELATIVE_TIME:
 			/* this type only supports COUNT, MAX, MIN, AVG */
 			switch(gio->calc_type){
+			case CALC_TYPE_SUM:
 			case CALC_TYPE_COUNT:
 			case CALC_TYPE_MAX:
 			case CALC_TYPE_MIN:
@@ -1634,7 +1627,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 	
 	io_stat_reset(gio->io);
 	enable_graph(gio, filter, field);
-	cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 	io_stat_redraw(gio->io);
 
 	return 0;
@@ -1673,7 +1666,7 @@ create_calc_types_menu_items(io_stat_graph_t *gio, GtkWidget *menu)
 
 
 static void
-create_advanced_menu(io_stat_graph_t *gio, GtkWidget *box, char *name, void (*func)(io_stat_graph_t *io, GtkWidget *menu))
+create_advanced_menu(io_stat_graph_t *gio, GtkWidget *box, const char *name, void (*func)(io_stat_graph_t *io, GtkWidget *menu))
 {
 	GtkWidget *hbox;
 	GtkWidget *label;
@@ -1701,7 +1694,7 @@ static void
 create_advanced_field(io_stat_graph_t *gio, GtkWidget *box)
 {
 
-	gio->calc_field=gtk_entry_new_with_max_length(30);
+	gio->calc_field=gtk_entry_new_with_max_length(50);
 	gtk_box_pack_start(GTK_BOX(box), gio->calc_field, FALSE, FALSE, 0);
 	gtk_widget_show(gio->calc_field);
 	SIGNAL_CONNECT(gio->calc_field, "activate", filter_callback, gio);
@@ -1791,9 +1784,9 @@ create_filter_box(io_stat_graph_t *gio, GtkWidget *box, int num)
 
 	g_snprintf(str, 256, "Ethereal: Display Filter  IO-Stat (Filter:%d)", num);
 	if(gio->args->title){
-		free(gio->args->title);
+		g_free( (gpointer) (gio->args->title) );
 	}
-	gio->args->title=strdup(str);	
+	gio->args->title=g_strdup(str);	
 
 	SIGNAL_CONNECT(gio->filter_bt, "clicked", filter_button_clicked, gio);
 	SIGNAL_CONNECT(gio->filter_bt, "destroy", filter_button_destroy_cb, NULL);
@@ -1921,7 +1914,7 @@ init_io_stat_window(io_stat_t *io)
 static void 
 gtk_iostat_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	gtk_iostat_init(NULL);
+	gtk_iostat_init(NULL,NULL);
 }
 
 
@@ -1930,8 +1923,8 @@ gtk_iostat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtk_iostat(void)
 {
-	register_ethereal_tap("io,stat", gtk_iostat_init);
+	register_stat_cmd_arg("io,stat", gtk_iostat_init,NULL);
 
-	register_tap_menu_item("_IO Graphs", REGISTER_TAP_GROUP_GENERIC,
+	register_stat_menu_item("_IO Graphs", REGISTER_STAT_GROUP_GENERIC,
         gtk_iostat_cb, NULL, NULL, NULL);
 }
