@@ -1,7 +1,7 @@
 /* main.h
  * Global defines, etc.
  *
- * $Id: main.h 13328 2005-02-06 23:16:05Z guy $
+ * $Id: main.h 15941 2005-09-21 20:11:55Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -57,10 +57,19 @@
  *  @ingroup windows_group
  */
 
-/** Global capture options. */
+/** Global compile time version string */
+extern GString *comp_info_str;
+/** Global runtime version string */
+extern GString *runtime_info_str;
+
+/** Global capture options type. */
 typedef struct capture_options_tag * p_capture_options_t;
+/** Pointer to global capture options. */
 extern p_capture_options_t capture_opts;
 
+extern void protect_thread_critical_region(void);
+extern void unprotect_thread_critical_region(void);
+
 /** User requested "Zoom In" by menu or toolbar.
  *
  * @param widget parent widget (unused)
@@ -180,6 +189,13 @@ extern void export_psml_cmd_cb(GtkWidget *widget, gpointer data);
  */
 extern void export_pdml_cmd_cb(GtkWidget *widget, gpointer data);
 
+/** User requested "Export as CSV" by menu.
+ *
+ * @param widget parent widget (unused)
+ * @param data unused
+ */
+extern void export_csv_cmd_cb(GtkWidget *widget, gpointer data);
+
 /** User requested "Expand Tree" by menu.
  *
  * @param widget parent widget (unused)
@@ -264,14 +280,14 @@ extern gboolean main_filter_packets(capture_file *cf, const gchar *dftext,
 
 /** Init the drag-n-drop functionality.
  *
- * @param the target widget for this dnd operations
+ * @param w the target widget for this dnd operations
  */
 extern void dnd_init(GtkWidget *w);
 
 /** Open a new file coming from drag and drop.
- * @param selection_data the selection data reported from GTK
+ * @param cf_names_freeme the selection data reported from GTK
  */
-extern void dnd_open_file_cmd(GtkSelectionData *selection_data);
+extern void dnd_open_file_cmd(gchar *cf_names_freeme);
 
 /** Update the packets statusbar to the current values. */
 extern void packets_bar_update(void);
