@@ -1,7 +1,7 @@
 /* menu.h
  * Menu definitions
  *
- * $Id: menu.h 12460 2004-11-01 11:42:08Z ulfl $
+ * $Id: menu.h 15471 2005-08-20 14:02:00Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,6 +29,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+extern void add_menu_recent_capture_file(gchar *cf_name);
+
 /** @file
  *  Menubar and context menus.
  *  @ingroup main_window_group
@@ -51,6 +53,14 @@ extern void menu_recent_read_finished(void);
 /** One of the name resolution menu items changed. */
 extern void menu_name_resolution_changed(void);
 
+/** The "Colorize Packet List" option changed. */
+extern void menu_colorize_changed(gboolean packet_list_colorize);
+
+#ifdef HAVE_LIBPCAP
+/** The "Auto Scroll Packet List in Live Capture" option changed. */
+extern void menu_auto_scroll_live_changed(gboolean auto_scroll_in);
+#endif
+
 /** Create a new menu.
  *
  * @param accel the created accelerator group
@@ -64,7 +74,7 @@ extern GtkWidget *main_menu_new(GtkAccelGroup **accel);
  * @param key the key to set
  * @param data the data to set
  */
-extern void set_menu_object_data(gchar *path, gchar *key, gpointer data);
+extern void set_menu_object_data(const gchar *path, const gchar *key, gpointer data);
 
 /** The popup menu handler.
  *
@@ -81,6 +91,14 @@ extern gint popup_menu_handler(GtkWidget *widget, GdkEvent *event, gpointer data
  */
 extern void set_menus_for_packet_history(gboolean back_history, gboolean forward_history);
 
+/** The current file has changed, we need to update the file set menu items.
+ *
+ * @param file_set the current file is part of a file set
+ * @param previous_file the previous file set (or NULL)
+ * @param next_file the next file set (or NULL)
+ */
+extern void set_menus_for_file_set(gboolean file_set, gboolean previous_file, gboolean next_file);
+
 /** The popup menu. */
 extern GtkWidget           *popup_menu_object;
 
