@@ -5,7 +5,7 @@
  *
  * Modified from gsm_map_stat.c
  *
- * $Id: mtp3_stat.c 12128 2004-09-29 00:06:36Z guy $
+ * $Id: mtp3_stat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,17 +40,20 @@
 #include "epan/packet_info.h"
 #include "epan/epan.h"
 #include "epan/value_string.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include "image/clist_ascend.xpm"
 #include "image/clist_descend.xpm"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
 #include <epan/tap.h>
+#include <epan/emem.h>
 #include "../register.h"
 #include "../globals.h"
 #include "filter_dlg.h"
 #include "compat_macros.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 
 #include <epan/dissectors/packet-mtp3.h>
 #include "mtp3_stat.h"
@@ -65,7 +68,7 @@ typedef struct column_arrows {
 
 typedef struct _my_columns_t {
     guint32		value;
-    gchar		*strptr;
+    const gchar		*strptr;
     GtkJustification	just;
 } my_columns_t;
 
@@ -95,10 +98,10 @@ static void
 mtp3_stat_reset(
     void		*tapdata)
 {
-    tapdata = tapdata;
+    mtp3_stat_t		(*stat_p)[MTP3_MAX_NUM_OPC_DPC] = tapdata;
 
     mtp3_num_used = 0;
-    memset((void *) mtp3_stat, 0, MTP3_MAX_NUM_OPC_DPC * sizeof(mtp3_stat_t));
+    memset(stat_p, 0, MTP3_MAX_NUM_OPC_DPC * sizeof(mtp3_stat_t));
 
     if (dlg.win != NULL)
     {
@@ -110,16 +113,13 @@ mtp3_stat_reset(
 static int
 mtp3_stat_packet(
     void		*tapdata,
-    packet_info		*pinfo,
+    packet_info		*pinfo _U_,
     epan_dissect_t	*edt _U_,
-    void		*data)
+    const void		*data)
 {
-    mtp3_tap_rec_t	*data_p = data;
-    int			i;
-
-
-    tapdata = tapdata;
-    pinfo = pinfo;
+    mtp3_stat_t		(*stat_p)[MTP3_MAX_NUM_OPC_DPC] = tapdata;
+    const mtp3_tap_rec_t	*data_p = data;
+    int				i;
 
     if (data_p->si_code >= MTP3_NUM_SI_CODE)
     {
@@ -136,9 +136,9 @@ mtp3_stat_packet(
     i = 0;
     while (i < mtp3_num_used)
     {
-	if (memcmp(&data_p->addr_opc, &mtp3_stat[i].addr_opc, sizeof(mtp3_addr_pc_t)) == 0)
+	if (memcmp(&data_p->addr_opc, &(*stat_p)[i].addr_opc, sizeof(mtp3_addr_pc_t)) == 0)
 	{
-	    if (memcmp(&data_p->addr_dpc, &mtp3_stat[i].addr_dpc, sizeof(mtp3_addr_pc_t)) == 0)
+	    if (memcmp(&data_p->addr_dpc, &(*stat_p)[i].addr_dpc, sizeof(mtp3_addr_pc_t)) == 0)
 	    {
 		break;
 	    }
@@ -160,10 +160,10 @@ mtp3_stat_packet(
 	mtp3_num_used++;
     }
 
-    mtp3_stat[i].addr_opc = data_p->addr_opc;
-    mtp3_stat[i].addr_dpc = data_p->addr_dpc;
-    mtp3_stat[i].si_code[data_p->si_code].num_msus++;
-    mtp3_stat[i].si_code[data_p->si_code].size += data_p->size;
+    (*stat_p)[i].addr_opc = data_p->addr_opc;
+    (*stat_p)[i].addr_dpc = data_p->addr_dpc;
+    (*stat_p)[i].si_code[data_p->si_code].num_msus++;
+    (*stat_p)[i].si_code[data_p->si_code].size += data_p->size;
 
     return(1);
 }
@@ -173,40 +173,39 @@ static void
 mtp3_stat_draw(
     void		*tapdata)
 {
+    mtp3_stat_t		(*stat_p)[MTP3_MAX_NUM_OPC_DPC] = tapdata;
     int			i, j, row_offset;
-    char		str[256];
+    char		*str;
 
-
-    tapdata = tapdata;
-
-    if (dlg.win == NULL)
+    if (!dlg.win || !tapdata)
     {
 	return;
     }
 
+    str=ep_alloc(256);
     i = 0;
 
     while (i < mtp3_num_used)
     {
 	row_offset = i * MTP3_NUM_SI_CODE;
 
-	mtp3_addr_to_str_buf((guint8 *) &mtp3_stat[i].addr_opc, str);
+	mtp3_addr_to_str_buf((guint8 *) &(*stat_p)[i].addr_opc, str, 256);
 	dlg.entries[0] = g_strdup(str);
 
-	mtp3_addr_to_str_buf((guint8 *) &mtp3_stat[i].addr_dpc, str);
+	mtp3_addr_to_str_buf((guint8 *) &(*stat_p)[i].addr_dpc, str, 256);
 	dlg.entries[1] = g_strdup(str);
 
 	for (j=0; j < MTP3_NUM_SI_CODE; j++)
 	{
 	    dlg.entries[2] = g_strdup(mtp3_service_indicator_code_short_vals[j].strptr);
 
-	    dlg.entries[3] = g_strdup_printf("%u", mtp3_stat[i].si_code[j].num_msus);
+	    dlg.entries[3] = g_strdup_printf("%u", (*stat_p)[i].si_code[j].num_msus);
 
-	    dlg.entries[4] = g_strdup_printf("%.0f", mtp3_stat[i].si_code[j].size);
+	    dlg.entries[4] = g_strdup_printf("%.0f", (*stat_p)[i].si_code[j].size);
 
 	    dlg.entries[5] =
 		g_strdup_printf("%.2f",
-		    mtp3_stat[i].si_code[j].size/mtp3_stat[i].si_code[j].num_msus);
+		    (*stat_p)[i].si_code[j].size/(*stat_p)[i].si_code[j].num_msus);
 
 	    gtk_clist_insert(GTK_CLIST(dlg.table), row_offset + j, dlg.entries);
 	}
@@ -279,8 +278,8 @@ mtp3_stat_gtk_sort_column(
     gconstpointer	ptr1,
     gconstpointer	ptr2)
 {
-    GtkCListRow		*row1 = (GtkCListRow *) ptr1;
-    GtkCListRow		*row2 = (GtkCListRow *) ptr2;
+    const GtkCListRow	*row1 = ptr1;
+    const GtkCListRow	*row2 = ptr2;
     char		*text1 = NULL;
     char		*text2 = NULL;
     int			i1, i2;
@@ -321,7 +320,7 @@ mtp3_stat_gtk_win_destroy_cb(
 static void
 mtp3_stat_gtk_win_create(
     mtp3_stat_dlg_t	*dlg_p,
-    char		*title)
+    const char		*title)
 {
     int			i;
     column_arrows	*col_arrows;
@@ -412,19 +411,6 @@ mtp3_stat_gtk_win_create(
 }
 
 
-/*
- * Never gets called ?
- */
-static void
-mtp3_stat_gtk_init(
-    char		*optarg)
-{
-    /* does not appear to be called */
-
-    optarg = optarg;
-}
-
-
 static void
 mtp3_stat_gtk_cb(
     GtkWidget		*w _U_,
@@ -446,18 +432,23 @@ mtp3_stat_gtk_cb(
 }
 
 
+static void
+mtp3_stat_gtk_init( const char *optarg _U_, void* userdata _U_)
+{
+    mtp3_stat_gtk_cb(NULL, NULL);
+}
+
+
 void
 register_tap_listener_gtkmtp3_stat(void)
 {
     GString		*err_p;
 
 
-    register_ethereal_tap("mtp3,", mtp3_stat_gtk_init);
-
     memset((void *) &mtp3_stat, 0, sizeof(mtp3_stat_t));
 
     err_p =
-	register_tap_listener("mtp3", NULL, NULL,
+	register_tap_listener("mtp3", &mtp3_stat, NULL,
 	    mtp3_stat_reset,
 	    mtp3_stat_packet,
 	    mtp3_stat_draw);
@@ -470,6 +461,7 @@ register_tap_listener_gtkmtp3_stat(void)
 	exit(1);
     }
 
-    register_tap_menu_item("MTP3/MSUs",  REGISTER_TAP_GROUP_NONE,
+    register_stat_menu_item("MTP3/MSUs",  REGISTER_STAT_GROUP_TELEPHONY,
         mtp3_stat_gtk_cb, NULL, NULL, NULL);
+    register_stat_cmd_arg("mtp3,msus", mtp3_stat_gtk_init,NULL);
 }
