@@ -1,7 +1,7 @@
 /* nameres_prefs.c
  * Dialog box for name resolution preferences
  *
- * $Id: nameres_prefs.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: nameres_prefs.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,7 +34,7 @@
 #include <epan/addr_resolv.h>
 #include <epan/prefs.h>
 #include "prefs_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "main.h"
 #include "menu.h"
 #include "compat_macros.h"
