@@ -3,7 +3,7 @@
  *
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet_win.c 13628 2005-03-06 10:26:00Z jmayer $
+ * $Id: packet_win.c 15468 2005-08-20 12:57:52Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -56,11 +56,13 @@
 #include "proto_draw.h"
 #include "keys.h"
 #include "gtkglobals.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include <epan/plugins.h>
 #include <epan/epan_dissect.h>
 #include "compat_macros.h"
 
+#include "../ui_util.h"
+
 /* Data structure holding information about a packet-detail window. */
 struct PacketWinData {
 	frame_data *frame;	   /* The frame being displayed */
