@@ -3,7 +3,7 @@
  *
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet_win.h 11400 2004-07-18 00:24:25Z guy $
+ * $Id: packet_win.h 16687 2005-12-05 22:45:54Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -37,6 +37,10 @@
  */
 extern void new_window_cb(GtkWidget *widget);
 
+/** Destroy all popup packet windows.
+ */
+void destroy_packet_wins(void);
+
 /** Redraw the hex dump panes of all packet windows. */
 void redraw_hex_dump_packet_wins(void);
 
