@@ -1,7 +1,7 @@
 /* print_dlg.c
  * Dialog boxes for printing and exporting to text files
  *
- * $Id: print_dlg.c 13293 2005-02-04 20:54:12Z guy $
+ * $Id: print_dlg.c 17289 2006-02-13 22:10:42Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,25 +31,33 @@
 #include <gtk/gtk.h>
 
 #include "globals.h"
+#include "gtkglobals.h"
 #include "keys.h"
 #include "print.h"
 #include <epan/prefs.h>
 #include "alert_box.h"
 #include "simple_dialog.h"
 #include "file_dlg.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "main.h"
 #include <epan/epan_dissect.h>
 #include <epan/filesystem.h>
 #ifdef _WIN32
-#include <io.h>
 #include "print_mswin.h"
 #endif
 #include "compat_macros.h"
 #include "range_utils.h"
 #include "help_dlg.h"
+#include "file_util.h"
+#include "tempfile.h"
+#include "util.h"
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+#include <gdk/gdkwin32.h>
+#include <windows.h>
+#include "win32-file-dlg.h"
+#endif
 
 /* dialog output action */
 typedef enum {
@@ -57,7 +65,8 @@ typedef enum {
   output_action_export_text,    /* export to plain text */
   output_action_export_ps,      /* export to postscript */
   output_action_export_psml,    /* export to packet summary markup language */
-  output_action_export_pdml     /* export to packet data markup language */
+  output_action_export_pdml,    /* export to packet data markup language */
+  output_action_export_csv      /* export to csv file */
 } output_action_e;
 
 
@@ -71,7 +80,7 @@ typedef enum {
 */
 
 static GtkWidget *
-open_print_dialog(char *title, output_action_e action, print_args_t *args);
+open_print_dialog(const char *title, output_action_e action, print_args_t *args);
 static void print_cmd_toggle_dest(GtkWidget *widget, gpointer data);
 static void print_cmd_toggle_detail(GtkWidget *widget, gpointer data);
 static void print_ok_cb(GtkWidget *ok_bt, gpointer parent_w);
@@ -84,6 +93,7 @@ static void print_destroy_cb(GtkWidget *win, gpointer user_data);
 #define PRINT_PS_RB_KEY           "printer_ps_radio_button"
 #define PRINT_PDML_RB_KEY         "printer_pdml_radio_button"
 #define PRINT_PSML_RB_KEY         "printer_psml_radio_button"
+#define PRINT_CSV_RB_KEY          "printer_csv_radio_button"
 #define PRINT_DEST_CB_KEY         "printer_destination_check_button"
 
 #define PRINT_SUMMARY_CB_KEY      "printer_summary_check_button"
@@ -111,7 +121,7 @@ static print_args_t  print_args;
 static gboolean print_prefs_init = FALSE;
 
 
-void
+static void
 file_print_cmd(gboolean print_selected)
 {
   print_args_t *args = &print_args;
@@ -141,7 +151,7 @@ file_print_cmd(gboolean print_selected)
   if(print_selected) {
       args->range.process = range_process_selected;
   }
-  
+
   print_win = open_print_dialog("Ethereal: Print", output_action_print, args);
   SIGNAL_CONNECT(print_win, "destroy", print_destroy_cb, &print_win);
 }
@@ -175,6 +185,11 @@ export_text_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
 {
   print_args_t *args = &export_text_args;
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_file(GDK_WINDOW_HWND(top_level->window), export_type_text);
+  return;
+#endif
+
   if (export_text_win != NULL) {
     /* There's already a "Export text" dialog box; reactivate it. */
     reactivate_window(export_text_win);
@@ -193,7 +208,7 @@ export_text_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
       args->print_hex           = FALSE;
       args->print_formfeed      = FALSE;
   }
-  
+
   /* init the printing range */
   packet_range_init(&args->range);
 
@@ -219,6 +234,11 @@ export_ps_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
 {
   print_args_t *args = &export_ps_args;
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_file(GDK_WINDOW_HWND(top_level->window), export_type_ps);
+  return;
+#endif
+
   if (export_ps_win != NULL) {
     /* There's already a "Export ps" dialog box; reactivate it. */
     reactivate_window(export_ps_win);
@@ -237,7 +257,7 @@ export_ps_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
       args->print_hex           = FALSE;
       args->print_formfeed      = FALSE;
   }
-  
+
   /* init the printing range */
   packet_range_init(&args->range);
 
@@ -263,6 +283,11 @@ export_psml_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
 {
   print_args_t *args = &export_psml_args;
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_file(GDK_WINDOW_HWND(top_level->window), export_type_psml);
+  return;
+#endif
+
   if (export_psml_win != NULL) {
     /* There's already a "Export psml" dialog box; reactivate it. */
     reactivate_window(export_psml_win);
@@ -281,7 +306,7 @@ export_psml_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
       args->print_hex           = FALSE;
       args->print_formfeed      = FALSE;
   }
-  
+
   /* init the printing range */
   packet_range_init(&args->range);
 
@@ -307,6 +332,11 @@ export_pdml_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
 {
   print_args_t *args = &export_pdml_args;
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_file(GDK_WINDOW_HWND(top_level->window), export_type_pdml);
+  return;
+#endif
+
   if (export_pdml_win != NULL) {
     /* There's already a "Export pdml" dialog box; reactivate it. */
     reactivate_window(export_pdml_win);
@@ -325,7 +355,7 @@ export_pdml_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
       args->print_hex           = FALSE;
       args->print_formfeed      = FALSE;
   }
-  
+
   /* init the printing range */
   packet_range_init(&args->range);
 
@@ -333,6 +363,52 @@ export_pdml_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
   SIGNAL_CONNECT(export_pdml_win, "destroy", print_destroy_cb, &export_pdml_win);
 }
 
+/*
+ * Keep a static pointer to the current "Export csv" window, if any, so that if
+ * somebody tries to do "File:Export to CSV" while there's already a "Export csv" window
+ * up, we just pop up the existing one, rather than creating a new one.
+ */
+static GtkWidget *export_csv_win = NULL;
+
+static print_args_t  export_csv_args;
+
+static gboolean export_csv_prefs_init = FALSE;
+
+void
+export_csv_cmd_cb(GtkWidget *widget _U_, gpointer data _U_)
+{
+  print_args_t *args = &export_csv_args;
+
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+  win32_export_file(GDK_WINDOW_HWND(top_level->window), export_type_csv);
+  return;
+#endif
+
+  if (export_csv_win != NULL) {
+    /* There's already a "Export csv" dialog box; reactivate it. */
+    reactivate_window(export_csv_win);
+    return;
+  }
+
+  /* get settings from preferences (and other initial values) only once */
+  if(export_csv_prefs_init == FALSE) {
+      export_csv_prefs_init     = TRUE;
+      args->format              = PR_FMT_TEXT; /* XXX */
+      args->to_file             = TRUE;
+      args->file                = g_strdup("");
+      args->cmd                 = g_strdup("");
+      args->print_summary       = FALSE;
+      args->print_dissections   = print_dissections_none;
+      args->print_hex           = FALSE;
+      args->print_formfeed      = FALSE;
+  }
+
+  /* init the printing range */
+  packet_range_init(&args->range);
+
+  export_csv_win = open_print_dialog("Ethereal: Export as \"Comma Separated Values\" File", output_action_export_csv, args);
+  SIGNAL_CONNECT(export_csv_win, "destroy", print_destroy_cb, &export_csv_win);
+}
 
 static void
 print_browse_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
@@ -345,7 +421,7 @@ print_browse_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
 
 /* Open the print dialog */
 static GtkWidget *
-open_print_dialog(char *title, output_action_e action, print_args_t *args)
+open_print_dialog(const char *title, output_action_e action, print_args_t *args)
 {
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
@@ -355,7 +431,7 @@ open_print_dialog(char *title, output_action_e action, print_args_t *args)
   GtkWidget     *main_vb;
 
   GtkWidget     *printer_fr, *printer_vb, *export_format_lb;
-  GtkWidget     *text_rb, *ps_rb, *pdml_rb, *psml_rb;
+  GtkWidget     *text_rb, *ps_rb, *pdml_rb, *psml_rb, *csv_rb;
   GtkWidget     *printer_tb, *dest_cb;
 #ifndef _WIN32
   GtkWidget     *cmd_lb, *cmd_te;
@@ -448,6 +524,16 @@ open_print_dialog(char *title, output_action_e action, print_args_t *args)
   gtk_box_pack_start(GTK_BOX(printer_vb), psml_rb, FALSE, FALSE, 0);
   /* gtk_widget_show(psml_rb); */
 
+  csv_rb = RADIO_BUTTON_NEW_WITH_MNEMONIC(text_rb, "_CSV", accel_group);
+  if (action == output_action_export_csv)
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(csv_rb), TRUE);
+  gtk_tooltips_set_tip (tooltips, csv_rb,
+      "Print output in \"Comma Separated Values\" (CSV) format, "
+      "a text format compatible with OpenOffice and Excel. "
+      "One row for each packet, with its timestamp and size.", NULL);
+  gtk_box_pack_start(GTK_BOX(printer_vb), csv_rb, FALSE, FALSE, 0);
+  /* gtk_widget_show(csv_rb); */
+
   /* printer table */
 #ifndef _WIN32
   printer_tb = gtk_table_new(2, 3, FALSE);
@@ -468,7 +554,7 @@ open_print_dialog(char *title, output_action_e action, print_args_t *args)
   gtk_table_attach_defaults(GTK_TABLE(printer_tb), dest_cb, 0, 1, 0, 1);
   if(action == output_action_print)
     gtk_widget_show(dest_cb);
-  
+
   /* File text entry */
   file_te = gtk_entry_new();
   OBJECT_SET_DATA(dest_cb, PRINT_FILE_TE_KEY, file_te);
@@ -543,7 +629,7 @@ open_print_dialog(char *title, output_action_e action, print_args_t *args)
   /*** packet format frame ***/
   format_fr = gtk_frame_new("Packet Format");
   gtk_box_pack_start(GTK_BOX(packet_hb), format_fr, TRUE, TRUE, 0);
-  if(   action == output_action_print || 
+  if(   action == output_action_print ||
         action == output_action_export_text ||
         action == output_action_export_ps)
     gtk_widget_show(format_fr);
@@ -653,6 +739,7 @@ open_print_dialog(char *title, output_action_e action, print_args_t *args)
   OBJECT_SET_DATA(ok_bt, PRINT_PS_RB_KEY, ps_rb);
   OBJECT_SET_DATA(ok_bt, PRINT_PDML_RB_KEY, pdml_rb);
   OBJECT_SET_DATA(ok_bt, PRINT_PSML_RB_KEY, psml_rb);
+  OBJECT_SET_DATA(ok_bt, PRINT_CSV_RB_KEY, csv_rb);
   OBJECT_SET_DATA(ok_bt, PRINT_DEST_CB_KEY, dest_cb);
 #ifndef _WIN32
   OBJECT_SET_DATA(ok_bt, PRINT_CMD_TE_KEY, cmd_te);
@@ -750,7 +837,7 @@ print_cmd_toggle_detail(GtkWidget *widget _U_, gpointer data)
   gtk_widget_set_sensitive(expand_all_rb, print_detail);
 
   /* if user selected nothing to print at all, disable the "ok" button */
-  gtk_widget_set_sensitive(print_bt, 
+  gtk_widget_set_sensitive(print_bt,
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (summary_cb)) ||
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (details_cb)) ||
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (hex_cb)));
@@ -766,8 +853,11 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
   gchar             *f_name;
   gchar             *dirname;
   gboolean          export_as_pdml = FALSE, export_as_psml = FALSE;
+  gboolean          export_as_csv = FALSE;
 #ifdef _WIN32
   gboolean          win_printer = FALSE;
+  int               tmp_fd;
+  char              tmp_namebuf[128+1];    /* XXX - length was used elsewhere too, why? */
 #endif
   cf_print_status_t status;
 
@@ -802,9 +892,22 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
   } else {
 #ifdef _WIN32
     win_printer = TRUE;
-    /*XXX should use temp file stuff in util routines */
+    /* We currently don't have a function in util.h to create just a tempfile */
+    /* name, so simply create a tempfile using the "official" function, */
+    /* then delete this file again. After this, the name MUST be available. */
+    /* */
+    /* Don't use tmpnam() or such, as this will fail under some ACL */
+    /* circumstances: http://bugs.ethereal.com/bugzilla/show_bug.cgi?id=358 */
+
+    tmp_fd = create_tempfile(tmp_namebuf, sizeof(tmp_namebuf), "ethprint");
+    if(tmp_fd == -1) {
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+            "Couldn't create a temporary file for printing.");
+        return;
+    }
     g_free(args->file);
-    args->file = g_strdup(tmpnam(NULL));
+    args->file = g_strdup(tmp_namebuf);
+    eth_unlink(args->file);
     args->to_file = TRUE;
 #else
     g_free(args->cmd);
@@ -823,6 +926,9 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
   button = (GtkWidget *)OBJECT_GET_DATA(ok_bt, PRINT_PSML_RB_KEY);
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (button)))
     export_as_psml = TRUE;
+  button = (GtkWidget *)OBJECT_GET_DATA(ok_bt, PRINT_CSV_RB_KEY);
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (button)))
+    export_as_csv = TRUE;
 
   button = (GtkWidget *)OBJECT_GET_DATA(ok_bt, PRINT_SUMMARY_CB_KEY);
   args->print_summary = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (button));
@@ -860,6 +966,8 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
     status = cf_write_pdml_packets(&cfile, args);
   else if (export_as_psml)
     status = cf_write_psml_packets(&cfile, args);
+  else if (export_as_csv)
+    status = cf_write_csv_packets(&cfile, args);
   else {
     switch (args->format) {
 
@@ -928,7 +1036,7 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
     print_mswin(args->file);
 
     /* trash temp file */
-    remove(args->file);
+    eth_remove(args->file);
   }
 #endif
 }
