@@ -1,7 +1,7 @@
 /* print_prefs.c
  * Dialog boxes for preferences for printing
  *
- * $Id: print_prefs.c 12146 2004-09-30 02:02:35Z guy $
+ * $Id: print_prefs.c 15465 2005-08-20 12:09:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include <epan/prefs.h>
 #include "prefs_dlg.h"
 #include "util.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "file_dlg.h"
 #include "compat_macros.h"
