@@ -1,6 +1,6 @@
 /* proto_dlg.c
  *
- * $Id: proto_dlg.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: proto_dlg.c 15471 2005-08-20 14:02:00Z jmayer $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -35,7 +35,7 @@
 #include "globals.h"
 #include "main.h"
 #include "util.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "proto_dlg.h"
 #include "simple_dialog.h"
@@ -77,13 +77,13 @@ static GtkWidget *proto_w = NULL;
 static GSList *protocol_list = NULL;
 
 typedef struct protocol_data {
-  char     *name;
-  char 	   *abbrev;
-  int  	   hfinfo_index;
-  gboolean enabled;
-  gboolean was_enabled;
+  const char  *name;
+  const char  *abbrev;
+  int  	      hfinfo_index;
+  gboolean    enabled;
+  gboolean    was_enabled;
 #if GTK_MAJOR_VERSION < 2
-  gint     row;
+  gint        row;
 #else
   GtkTreeIter iter;
 #endif
@@ -98,7 +98,7 @@ proto_cb(GtkWidget *w _U_, gpointer data _U_)
 
   GtkWidget *main_vb, *bbox, *proto_list, *label, *proto_sw, *proto_frame,
             *proto_vb, *button;
-  gchar *titles[] = { "Status", "Protocol", "Description" };
+  const gchar *titles[] = { "Status", "Protocol", "Description" };
 #if GTK_MAJOR_VERSION < 2
   gint width;
 #else
@@ -145,7 +145,7 @@ proto_cb(GtkWidget *w _U_, gpointer data _U_)
   gtk_widget_show(proto_sw);
 
 #if GTK_MAJOR_VERSION < 2
-  proto_list = gtk_clist_new_with_titles(3, titles);
+  proto_list = gtk_clist_new_with_titles(3, (gchar **) titles);
   gtk_container_add(GTK_CONTAINER(proto_sw), proto_list);
   gtk_clist_set_selection_mode(GTK_CLIST(proto_list), GTK_SELECTION_BROWSE);
   gtk_clist_column_titles_passive(GTK_CLIST(proto_list));
@@ -518,7 +518,7 @@ revert_proto_selection(void)
 
 } /* revert_proto_selection */
 
-gint
+static gint
 protocol_data_compare(gconstpointer a, gconstpointer b)
 {
   const protocol_data_t *ap = (const protocol_data_t *)a;
@@ -540,7 +540,7 @@ show_proto_selection(GtkListStore *proto_store)
   protocol_t *protocol;
   protocol_data_t *p;
 #if GTK_MAJOR_VERSION < 2
-  gchar *proto_text[3];
+  const gchar *proto_text[3];
 #endif
 
   /* Iterate over all the protocols */
@@ -572,7 +572,7 @@ show_proto_selection(GtkListStore *proto_store)
     proto_text[0] = STATUS_TXT (p->enabled);
     proto_text[1] = p->abbrev;
     proto_text[2] = p->name;
-    p->row = gtk_clist_append(proto_list, proto_text);
+    p->row = gtk_clist_append(proto_list, (gchar **) proto_text);
     gtk_clist_set_row_data(proto_list, p->row, p);
 #else
     gtk_list_store_append(proto_store, &p->iter);
