@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: proto_draw.c 17285 2006-02-13 21:25:43Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,10 +31,6 @@
 
 #include <ctype.h>
 
-#ifdef HAVE_IO_H
-#include <io.h>			/* open/close on win32 */
-#endif
-
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -64,7 +60,7 @@
 #include "proto_draw.h"
 #include "packet_win.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "gtkglobals.h"
 #include "compat_macros.h"
 #include "alert_box.h"
@@ -72,10 +68,13 @@
 #include "progress_dlg.h"
 #include "font_utils.h"
 
+#include "../ui_util.h"
+#include "file_util.h"
 
-/* Win32 needs the O_BINARY flag for open() */
-#ifndef O_BINARY
-#define O_BINARY	0
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+#include <gdk/gdkwin32.h>
+#include <windows.h>
+#include "win32-file-dlg.h"
 #endif
 
 #define BYTE_VIEW_WIDTH    16
@@ -230,9 +229,9 @@ redraw_hex_dump_all(void)
 
 #if GTK_MAJOR_VERSION >= 2
   /* XXX - this is a hack, to workaround a bug in GTK2.x!
-     when changing the font size, even refilling of the corresponding 
-     gtk_text_buffer doesn't seem to trigger an update.     
-     The only workaround is to freshly select the frame, which will remove any 
+     when changing the font size, even refilling of the corresponding
+     gtk_text_buffer doesn't seem to trigger an update.
+     The only workaround is to freshly select the frame, which will remove any
      existing notebook tabs and "restart" the whole byte view again. */
   if (cfile.current_frame != NULL)
     cf_goto_frame(&cfile, cfile.current_frame->num);
@@ -361,6 +360,8 @@ GtkTreePath *tree_find_by_field_info(GtkTreeView *tree_view, field_info *finfo)
   GtkTreeModel *model;
   struct field_lookup_info fli;
 
+  g_assert(finfo != NULL);
+
   model = gtk_tree_view_get_model(tree_view);
   fli.fi = finfo;
   gtk_tree_model_foreach(model, lookup_finfo, &fli);
@@ -703,7 +704,7 @@ add_byte_tab(GtkWidget *byte_nb, const char *name, tvbuff_t *tvb,
 			/* Horizontal */GTK_POLICY_NEVER,
 			/* Vertical*/	GTK_POLICY_ALWAYS);
 #else
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(byte_scrollw), 
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(byte_scrollw),
                                    GTK_SHADOW_IN);
 #endif
   /* Add scrolled pane to tabbed window */
@@ -750,7 +751,7 @@ add_byte_tab(GtkWidget *byte_nb, const char *name, tvbuff_t *tvb,
         gtk_notebook_set_show_tabs(GTK_NOTEBOOK(byte_nb), TRUE);
 
   /* set this page (this will print the packet data) */
-  gtk_notebook_set_page(GTK_NOTEBOOK(byte_nb), 
+  gtk_notebook_set_page(GTK_NOTEBOOK(byte_nb),
     gtk_notebook_page_num(GTK_NOTEBOOK(byte_nb), byte_nb));
 
   return byte_view;
@@ -801,8 +802,8 @@ savehex_dlg_destroy_cb(void)
         savehex_dlg = NULL;
 }
 
-extern void
-copy_hex_cb(GtkWidget * w _U_, gpointer data _U_)  
+void
+copy_hex_cb(GtkWidget * w _U_, gpointer data _U_)
 {
 	GtkWidget *bv;
 	int len;
@@ -810,16 +811,16 @@ copy_hex_cb(GtkWidget * w _U_, gpointer data _U_)
 	const guint8 *data_p = NULL;
 	GString *ASCII_representation = g_string_new("");
 	GString *byte_str = g_string_new("");
-	
+
 	bv = get_notebook_bv_ptr(byte_nb_ptr);
 	if (bv == NULL) {
-		/* shouldn't happen */ 
+		/* shouldn't happen */
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Could not find the corresponding text window!");
 		return;
 	}
-	
+
 	data_p = get_byte_view_data_and_length(GTK_WIDGET(bv), &len);
-	
+
 	g_string_sprintfa(byte_str,"%04x  ",i); /* Offset 0000 */
 	for (i=0; i<len; i++){
 	  g_string_sprintfa(ASCII_representation,"%c",isprint(*data_p) ? *data_p : '.');
@@ -829,7 +830,7 @@ copy_hex_cb(GtkWidget * w _U_, gpointer data _U_)
 	    g_string_assign (ASCII_representation,"");
 	  }
 	}
-	
+
 	if(ASCII_representation->len){
 	  for (i=ASCII_representation->len; i<16; i++){
 	    g_string_sprintfa(byte_str,"   ");
@@ -871,7 +872,7 @@ savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 		return;
 	}
 	/*
-	 * Retrieve the info we need 
+	 * Retrieve the info we need
 	 */
 	end = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_START_KEY));
 	start = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_END_KEY));
@@ -883,17 +884,17 @@ savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 		return;
 	}
 
-	fd = open(file, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0666);
+	fd = eth_open(file, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0666);
 	if (fd == -1) {
 		open_failure_alert_box(file, errno, TRUE);
 		return;
 	}
-	if (write(fd, data_p + start, end - start) < 0) {
+	if (eth_write(fd, data_p + start, end - start) < 0) {
 		write_failure_alert_box(file, errno);
-		close(fd);
+		eth_close(fd);
 		return;
 	}
-	if (close(fd) < 0) {
+	if (eth_close(fd) < 0) {
 		write_failure_alert_box(file, errno);
 		return;
 	}
@@ -907,11 +908,15 @@ void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
 {
 	int start, end, len;
 	const guint8 *data_p = NULL;
-    gchar *label;
+        gchar *label;
 
-    GtkWidget   *bv;
+        GtkWidget   *bv;
 	GtkWidget   *dlg_lb;
 
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+    win32_export_raw_file(GDK_WINDOW_HWND(top_level->window));
+    return;
+#endif
 
     /* don't show up the dialog, if no data has to be saved */
 	bv = get_notebook_bv_ptr(byte_nb_ptr);
@@ -941,7 +946,7 @@ void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
     savehex_dlg = file_selection_new("Ethereal: Export Selected Packet Bytes", FILE_SELECTION_SAVE);
 
     /* label */
-    label = g_strdup_printf("Will save %u %s of raw binary data to specified file.", 
+    label = g_strdup_printf("Will save %u %s of raw binary data to specified file.",
         end - start, plurality(end - start, "byte", "bytes"));
     dlg_lb = gtk_label_new(label);
     g_free(label);
@@ -962,7 +967,7 @@ void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
     SIGNAL_CONNECT(GTK_FILE_SELECTION (savehex_dlg)->ok_button, "clicked",
         savehex_save_clicked_cb, savehex_dlg);
 
-    window_set_cancel_button(savehex_dlg, 
+    window_set_cancel_button(savehex_dlg,
     GTK_FILE_SELECTION(savehex_dlg)->cancel_button, window_cancel_button_cb);
 
     SIGNAL_CONNECT(savehex_dlg, "delete_event", window_delete_event_cb, NULL);
@@ -1017,7 +1022,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 #else
   GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(bv));
   GtkTextIter    iter;
-  char          *revstyle;
+  const char    *revstyle;
   gchar         *convline;
   gsize          newsize;
   GtkTextMark   *mark = NULL;
@@ -1072,11 +1077,25 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
   /* When we reach the value that triggers a progress bar update,
      bump that value by this amount. */
   progbar_quantum = len/N_PROGBAR_UPDATES;
+  /* Progress so far. */
+  progbar_val = 0.0;
 
   progbar_stop_flag = FALSE;
   g_get_current_time(&progbar_start_time);
 
   while (i < len) {
+    /* Create the progress bar if necessary.
+       We check on every iteration of the loop, so that it takes no
+       longer than the standard time to create it (otherwise, for a
+       large packet, we might take considerably longer than that standard
+       time in order to get to the next progress bar step). */
+    if (progbar == NULL)
+      progbar = delayed_create_progress_dlg("Processing", "Packet Details",
+                                            TRUE,
+                                            &progbar_stop_flag,
+                                            &progbar_start_time,
+                                            progbar_val);
+
     /* Update the progress bar, but do it only N_PROGBAR_UPDATES times;
        when we update it, we have to run the GTK+ main loop to get it
        to repaint what's pending, and doing so may involve an "ioctl()"
@@ -1089,13 +1108,6 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
       g_assert(len > 0);
       progbar_val = (gfloat) i / len;
 
-      if (progbar == NULL)
-        /* Create the progress bar if necessary */
-        progbar = delayed_create_progress_dlg("Processing", "Packet Details",
-                                              &progbar_stop_flag,
-                                              &progbar_start_time,
-                                              progbar_val);
-
       if (progbar != NULL) {
         g_snprintf(progbar_status_str, sizeof(progbar_status_str),
                    "%4u of %u bytes", i, len);
@@ -1376,7 +1388,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
         convline = g_locale_to_utf8(line, cur, NULL, &newsize, NULL);
         gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
                                                  revstyle, NULL);
-        g_free(convline);
+        g_free( (gpointer) convline);
         cur = 0;
       }
       if (i < k) {
@@ -1390,7 +1402,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
         convline = g_locale_to_utf8(line, cur, NULL, &newsize, NULL);
         gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
                                                  "plain", NULL);
-        g_free(convline);
+        g_free( (gpointer) convline);
         cur = 0;
       }
       reverse = newreverse;
@@ -1400,7 +1412,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
     gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
                                              reverse ? revstyle : "plain",
                                              NULL);
-    g_free(convline);
+    g_free( (gpointer) convline);
     cur = 0;
     line[cur++] = '\n';
     gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
@@ -1599,8 +1611,29 @@ set_ptree_font_all(FONT_TYPE *font)
 }
 
 
+gboolean colors_ok = FALSE;
+GdkColor	expert_color_chat	= { 0, 0xcc00, 0xcc00, 0xe000 };	/* a pale bluegrey */
+GdkColor	expert_color_note	= { 0, 0xa000, 0xff00, 0xff00 };	/* a bright turquoise */
+GdkColor	expert_color_warn	= { 0, 0xff00, 0xff00, 0 };			/* yellow */
+GdkColor	expert_color_error	= { 0, 0xff00, 0x5c00, 0x5c00 };	/* pale red */
+
+void proto_draw_colors_init(void)
+{
+	if(colors_ok) {
+		return;
+	}
+
+	get_color(&expert_color_chat);
+	get_color(&expert_color_note);
+	get_color(&expert_color_warn);
+	get_color(&expert_color_error);
+
+	colors_ok = TRUE;
+}
+
+
 #if GTK_MAJOR_VERSION >= 2
-void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
+static void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
                                              GtkCellRenderer *cell,
                                              GtkTreeModel *tree_model,
                                              GtkTreeIter *iter,
@@ -1610,9 +1643,28 @@ void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
 
     gtk_tree_model_get(tree_model, iter, 1, &fi, -1);
 
+	if(!colors_ok) {
+		proto_draw_colors_init();
+	}
+
+	/* for the various possible attributes, see:
+	 * http://developer.gnome.org/doc/API/2.0/gtk/GtkCellRendererText.html
+	 *
+	 * color definitions can be found at:
+	 * http://cvs.gnome.org/viewcvs/gtk+/gdk-pixbuf/io-xpm.c?rev=1.42
+	 * (a good color overview: http://www.computerhope.com/htmcolor.htm)
+	 *
+	 * some experiences:
+	 * background-gdk: doesn't seem to work (probably the GdkColor must be allocated)
+	 * weight/style: doesn't take any effect
+	 */
+
     /* for each field, we have to reset the renderer attributes */
     g_object_set (cell, "foreground-set", FALSE, NULL);
 
+    g_object_set (cell, "background", "white", NULL);
+    g_object_set (cell, "background-set", TRUE, NULL);
+
     g_object_set (cell, "underline", PANGO_UNDERLINE_NONE, NULL);
     g_object_set (cell, "underline-set", FALSE, NULL);
 
@@ -1623,6 +1675,8 @@ void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
     g_object_set (cell, "weight-set", FALSE, NULL);*/
 
     if(FI_GET_FLAG(fi, FI_GENERATED)) {
+		/* we use "[...]" to mark generated items, no need to change things here */
+
         /* as some fonts don't support italic, don't use this */
         /*g_object_set (cell, "style", PANGO_STYLE_ITALIC, NULL);
         g_object_set (cell, "style-set", TRUE, NULL);
@@ -1631,6 +1685,13 @@ void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
         g_object_set (cell, "weight-set", TRUE, NULL);*/
     }
 
+    if(fi->hfinfo->type == FT_PROTOCOL) {
+        g_object_set (cell, "background", "gray90", NULL);
+        g_object_set (cell, "background-set", TRUE, NULL);
+        /*g_object_set (cell, "weight", PANGO_WEIGHT_BOLD, NULL);
+        g_object_set (cell, "weight-set", TRUE, NULL);*/
+	}
+
     if(fi->hfinfo->type == FT_FRAMENUM) {
         g_object_set (cell, "foreground", "blue", NULL);
         g_object_set (cell, "foreground-set", TRUE, NULL);
@@ -1638,10 +1699,91 @@ void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
         g_object_set (cell, "underline", PANGO_UNDERLINE_SINGLE, NULL);
         g_object_set (cell, "underline-set", TRUE, NULL);
     }
+
+	if(FI_GET_FLAG(fi, PI_SEVERITY_MASK)) {
+		switch(FI_GET_FLAG(fi, PI_SEVERITY_MASK)) {
+		case(PI_CHAT):
+			g_object_set (cell, "background-gdk", &expert_color_chat, NULL);
+			g_object_set (cell, "background-set", TRUE, NULL);
+			break;
+		case(PI_NOTE):
+			g_object_set (cell, "background-gdk", &expert_color_note, NULL);
+			g_object_set (cell, "background-set", TRUE, NULL);
+			break;
+		case(PI_WARN):
+			g_object_set (cell, "background-gdk", &expert_color_warn, NULL);
+			g_object_set (cell, "background-set", TRUE, NULL);
+			break;
+		case(PI_ERROR):
+			g_object_set (cell, "background-gdk", &expert_color_error, NULL);
+			g_object_set (cell, "background-set", TRUE, NULL);
+			break;
+		default:
+			g_assert_not_reached();
+		}
+	}
 }
 #endif
 
 
+#if GTK_MAJOR_VERSION >= 2
+static int
+tree_view_key_pressed_cb(GtkCTree *ctree _U_, GdkEventKey *event, gpointer user_data _U_)
+{
+    GtkTreeSelection* selection;
+    GtkTreeIter iter;
+    GtkTreeIter parent;
+    GtkTreeModel* model;
+    GtkTreePath* path;
+    gboolean    expanded;
+    gboolean    has_parent;
+
+
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
+    if(!selection) {
+        return FALSE;
+    }
+
+    if(!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+        return FALSE;
+    }
+
+    path = gtk_tree_model_get_path(model, &iter);
+    if(!path) {
+        return FALSE;
+    }
+
+    if (event->keyval == GDK_Left) {
+        expanded = gtk_tree_view_row_expanded(GTK_TREE_VIEW(tree_view), path);
+        if(expanded) {
+            /* subtree is expanded, collapse it */
+            gtk_tree_view_collapse_row(GTK_TREE_VIEW(tree_view), path);
+            return TRUE;
+        } else {
+            /* subtree is already collapsed, jump to parent node */
+            has_parent = gtk_tree_model_iter_parent(model, &parent, &iter);
+            path = gtk_tree_model_get_path(model, &parent);
+            if(!path) {
+                return FALSE;
+            }
+            gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree_view), path,
+                                             NULL /* focus_column */,
+                                             FALSE /* !start_editing */);
+            return TRUE;
+        }
+    }
+    if (event->keyval == GDK_Right) {
+        /* try to expand the subtree */
+        gtk_tree_view_expand_row(GTK_TREE_VIEW(tree_view), path, FALSE /* !open_all */);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+#endif
+
+
+
 GtkWidget *
 main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
 {
@@ -1656,7 +1798,7 @@ main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
   /* Tree view */
   tv_scrollw = scrolled_window_new(NULL, NULL);
 #if GTK_MAJOR_VERSION >= 2
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tv_scrollw), 
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tv_scrollw),
                                    GTK_SHADOW_IN);
 #endif
 
@@ -1690,6 +1832,7 @@ main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
                                   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   SIGNAL_CONNECT(tree_view, "row-expanded", expand_tree, NULL);
   SIGNAL_CONNECT(tree_view, "row-collapsed", collapse_tree, NULL);
+  SIGNAL_CONNECT(tree_view, "key-press-event", tree_view_key_pressed_cb, NULL );
 #endif
   gtk_container_add( GTK_CONTAINER(tv_scrollw), tree_view );
   set_ptree_sel_browse(tree_view, prefs->gui_ptree_sel_browse);
@@ -1764,7 +1907,7 @@ main_proto_tree_draw(proto_tree *protocol_tree)
 }
 
 
-void
+static void
 tree_view_follow_link(field_info   *fi)
 {
     if(fi->hfinfo->type == FT_FRAMENUM) {
@@ -1787,7 +1930,7 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
         field_info   *fi;
 
 
-        if(gtk_clist_get_selection_info(GTK_CLIST(widget), 
+        if(gtk_clist_get_selection_info(GTK_CLIST(widget),
             (gint) (((GdkEventButton *)event)->x),
             (gint) (((GdkEventButton *)event)->y),
             &row, &column))
@@ -1837,6 +1980,7 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
     return TRUE;
 }
 
+/* fill the whole protocol tree with the string values */
 void
 proto_tree_draw(proto_tree *protocol_tree, GtkWidget *tree_view)
 {
@@ -1875,6 +2019,8 @@ proto_tree_draw(proto_tree *protocol_tree, GtkWidget *tree_view)
 #endif
 }
 
+
+/* fill a single protocol tree item with the string value */
 static void
 proto_tree_draw_node(proto_node *node, gpointer data)
 {
