@@ -1,6 +1,6 @@
 /* proto_hier_stats_dlg.c
  *
- * $Id: proto_hier_stats_dlg.c 14766 2005-06-26 10:28:51Z ulfl $
+ * $Id: proto_hier_stats_dlg.c 15472 2005-08-20 14:08:53Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,8 +29,9 @@
 #include <gtk/gtk.h>
 
 #include "proto_hier_stats.h"
+#include "proto_hier_stats_dlg.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "main.h"
 #include "compat_macros.h"
 #include "help_dlg.h"
@@ -74,7 +75,7 @@ fill_in_tree_node(GNode *node, gpointer data)
     ph_stats_t      *ps = di->ps;
     gboolean        is_leaf;
     draw_info_t     child_di;
-    double	    seconds;
+    double          seconds;
     gchar           *text[NUM_STAT_COLUMNS];
 #if GTK_MAJOR_VERSION < 2
     GtkCTree        *tree = di->tree;
@@ -96,7 +97,7 @@ fill_in_tree_node(GNode *node, gpointer data)
 
     seconds = ps->last_time - ps->first_time;
 
-    text[0] = (gchar *) stats->hfinfo->name;
+    text[0] = (gchar *) (stats->hfinfo->name);
     text[1] = g_strdup_printf("%6.2f%%",
                               PCT(stats->num_pkts_total, ps->tot_packets));
     text[2] = g_strdup_printf("%u", stats->num_pkts_total);
