@@ -2,7 +2,7 @@
  * Recent "preference" handling routines
  * Copyright 2004, Ulf Lamping <ulf.lamping@web.de>
  *
- * $Id: recent.c 14654 2005-06-16 06:33:49Z guy $
+ * $Id: recent.c 16403 2005-11-06 22:43:25Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,10 +39,11 @@
 #include "main.h"
 #include <epan/prefs.h>
 #include <epan/prefs-int.h>
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "cfilter_combo_utils.h"
 #include "simple_dialog.h"
+#include "file_util.h"
 
 #define RECENT_KEY_MAIN_TOOLBAR_SHOW        "gui.toolbar_main_show"
 #define RECENT_KEY_FILTER_TOOLBAR_SHOW      "gui.filter_toolbar_show"
@@ -52,6 +53,7 @@
 #define RECENT_KEY_STATUSBAR_SHOW           "gui.statusbar_show"
 #define RECENT_KEY_PACKET_LIST_COLORIZE     "gui.packet_list_colorize"
 #define RECENT_GUI_TIME_FORMAT              "gui.time_format"
+#define RECENT_GUI_TIME_PRECISION           "gui.time_precision"
 #define RECENT_GUI_ZOOM_LEVEL               "gui.zoom_level"
 #define RECENT_GUI_GEOMETRY_MAIN_X          "gui.geometry_main_x"
 #define RECENT_GUI_GEOMETRY_MAIN_Y          "gui.geometry_main_y"
@@ -66,22 +68,21 @@
 
 #define RECENT_FILE_NAME "recent"
 
-
-/* #include "../menu.h" */
-extern void add_menu_recent_capture_file(gchar *file);
-
 recent_settings_t recent;
 
-static char *ts_type_text[] =
+static const char *ts_type_text[] =
 	{ "RELATIVE", "ABSOLUTE", "ABSOLUTE_WITH_DATE", "DELTA", NULL };
 
+static const char *ts_precision_text[] =
+	{ "AUTO", "SEC", "DSEC", "CSEC", "MSEC", "USEC", "NSEC", NULL };
+
 /* Takes an string and a pointer to an array of strings, and a default int value.
  * The array must be terminated by a NULL string. If the string is found in the array
  * of strings, the index of that string in the array is returned. Otherwise, the
  * default value that was passed as the third argument is returned.
  */
 static int
-find_index_from_string_array(char *needle, char **haystack, int default_value)
+find_index_from_string_array(const char *needle, const char **haystack, int default_value)
 {
 	int i = 0;
 
@@ -121,7 +122,7 @@ write_recent(void)
   }
 
   rf_path = get_persconffile_path(RECENT_FILE_NAME, TRUE);
-  if ((rf = fopen(rf_path, "w")) == NULL) {
+  if ((rf = eth_fopen(rf_path, "w")) == NULL) {
      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
       "Can't open recent file\n\"%s\": %s.", rf_path,
       strerror(errno));
@@ -191,6 +192,11 @@ write_recent(void)
   fprintf(rf, RECENT_GUI_TIME_FORMAT ": %s\n",
           ts_type_text[recent.gui_time_format]);
 
+  fprintf(rf, "\n# Timestamp display precision.\n");
+  fprintf(rf, "# One of: AUTO, SEC, DSEC, CSEC, MSEC, USEC, NSEC\n");
+  fprintf(rf, RECENT_GUI_TIME_PRECISION ": %s\n",
+          ts_precision_text[recent.gui_time_precision]);
+
   fprintf(rf, "\n# Zoom level.\n");
   fprintf(rf, "# A decimal number.\n");
   fprintf(rf, RECENT_GUI_ZOOM_LEVEL ": %d\n",
@@ -330,6 +336,9 @@ read_set_recent_pair_static(gchar *key, gchar *value)
   } else if (strcmp(key, RECENT_GUI_TIME_FORMAT) == 0) {
     recent.gui_time_format =
 	find_index_from_string_array(value, ts_type_text, TS_RELATIVE);
+  } else if (strcmp(key, RECENT_GUI_TIME_PRECISION) == 0) {
+    recent.gui_time_precision =
+	find_index_from_string_array(value, ts_precision_text, TS_PREC_AUTO);
   } else if (strcmp(key, RECENT_GUI_ZOOM_LEVEL) == 0) {
     num = strtol(value, &p, 0);
     if (p == value || *p != '\0')
@@ -483,6 +492,7 @@ recent_read_static(char **rf_path_return, int *rf_errno_return)
   recent.statusbar_show         = TRUE;
   recent.packet_list_colorize   = TRUE;
   recent.gui_time_format        = TS_RELATIVE;
+  recent.gui_time_precision     = TS_PREC_AUTO;
   recent.gui_zoom_level         = 0;
 
   recent.gui_geometry_main_x        =        20;
@@ -513,7 +523,7 @@ recent_read_static(char **rf_path_return, int *rf_errno_return)
 
   /* Read the user's recent file, if it exists. */
   *rf_path_return = NULL;
-  if ((rf = fopen(rf_path, "r")) != NULL) {
+  if ((rf = eth_fopen(rf_path, "r")) != NULL) {
     /* We succeeded in opening it; read it. */
     read_prefs_file(rf_path, rf, read_set_recent_pair_static);
     fclose(rf);
@@ -545,7 +555,7 @@ recent_read_dynamic(char **rf_path_return, int *rf_errno_return)
 
   /* Read the user's recent file, if it exists. */
   *rf_path_return = NULL;
-  if ((rf = fopen(rf_path, "r")) != NULL) {
+  if ((rf = eth_fopen(rf_path, "r")) != NULL) {
     /* We succeeded in opening it; read it. */
     read_prefs_file(rf_path, rf, read_set_recent_pair_dynamic);
 	/* set dfilter combobox to have an empty line */
