@@ -1,7 +1,7 @@
 /* rpc_stat.c
  * rpc_stat   2002 Ronnie Sahlberg
  *
- * $Id: rpc_stat.c 15043 2005-07-24 19:01:28Z guy $
+ * $Id: rpc_stat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,14 +37,17 @@
 #include <epan/packet_info.h>
 #include <epan/epan.h>
 
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "stat_menu.h"
+#include "gui_stat_menu.h"
 #include "simple_dialog.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include <epan/tap.h>
-#include "../register.h"
+#include "register.h"
 #include <epan/dissectors/packet-rpc.h>
-#include "../globals.h"
+#include "main.h"
+#include "globals.h"
 #include "filter_dlg.h"
 #include "compat_macros.h"
 #include "service_response_time_table.h"
@@ -197,8 +200,6 @@ rpcstat_find_vers(gpointer *key, gpointer *value _U_, gpointer *user_data _U_)
  *
  * there should not be any other critical regions in gtk2
  */
-void protect_thread_critical_region(void);
-void unprotect_thread_critical_region(void);
 static void
 win_destroy_cb(GtkWindow *win _U_, gpointer data)
 {
@@ -215,7 +216,7 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
 /* When called, this function will create a new instance of gtk2-rpcstat.
  */
 static void
-gtk_rpcstat_init(char *optarg)
+gtk_rpcstat_init(const char *optarg, void* userdata _U_)
 {
 	rpcstat_t *rs;
 	guint32 i;
@@ -227,7 +228,7 @@ gtk_rpcstat_init(char *optarg)
 	GtkWidget *bbox;
 	GtkWidget *close_bt;
 	int program, version, pos;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString *error_string;
 	int hf_index;
 	header_field_info *hfi;
@@ -307,7 +308,7 @@ gtk_rpcstat_init(char *optarg)
 	gtk_widget_show_all(rs->win);
 	window_present(rs->win);
 
-	cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 }
 
 
@@ -332,7 +333,7 @@ rpcstat_start_button_clicked(GtkWidget *item _U_, gpointer data _U_)
 		g_string_sprintfa(str, ",%s", filter);
 	}
 
-	gtk_rpcstat_init(str->str);
+	gtk_rpcstat_init(str->str,NULL);
 	g_string_free(str, TRUE);
 }
 
@@ -416,7 +417,8 @@ gtk_rpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	static construct_args_t args = {
 	  "Service Response Time Statistics Filter",
 	  TRUE,
-	  FALSE
+	  FALSE,
+      FALSE
 	};
 
 	/* if the window is already open, bring it to front */
@@ -536,8 +538,8 @@ gtk_rpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkrpcstat(void)
 {
-	register_tap_listener_cmd_arg("rpc,srt,", gtk_rpcstat_init);
+	register_stat_cmd_arg("rpc,srt,", gtk_rpcstat_init, NULL);
 
-	register_tap_menu_item("ONC-RPC...", REGISTER_TAP_GROUP_RESPONSE_TIME,
+	register_stat_menu_item("ONC-RPC...", REGISTER_STAT_GROUP_RESPONSE_TIME,
 	    gtk_rpcstat_cb, NULL, NULL, NULL);
 }
