@@ -1,7 +1,7 @@
 /* 
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_assoc_analyse.c 14908 2005-07-12 21:04:05Z tuexen $
+ * $Id: sctp_assoc_analyse.c 17803 2006-04-04 09:04:08Z tuexen $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,9 +33,10 @@
 #include <epan/epan_dissect.h>
 #include "epan/filesystem.h"
 #include "register.h"
-#include "tap_menu.h"
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "main.h"
 #include "compat_macros.h"
 
@@ -110,7 +111,7 @@ on_destroy(GtkObject *object _U_, gpointer user_data)
 }
 
 static void
-on_notebook_switch_page()
+on_notebook_switch_page(void)
 {
 }
 
@@ -354,7 +355,7 @@ static void analyse_window_set_title(struct sctp_analyse *u_data)
 	g_free(title);
 }
 
-void create_analyse_window(struct sctp_analyse* u_data)
+static void create_analyse_window(struct sctp_analyse* u_data)
 {
 	GtkWidget *window = NULL;
 	GtkWidget *notebook;
@@ -543,7 +544,11 @@ void create_analyse_window(struct sctp_analyse* u_data)
 	gtk_box_pack_start(GTK_BOX(h_button_box), graph_bt2, FALSE, FALSE, 0);
 	gtk_widget_show(graph_bt2);
 	SIGNAL_CONNECT(graph_bt2, "clicked", on_graph_byte1_dlg,u_data);
-
+	if (u_data->assoc->n_array_tsn1==0)
+	{
+		gtk_widget_set_sensitive(graph_bt1, FALSE);
+		gtk_widget_set_sensitive(graph_bt2, FALSE);
+	}
 	close_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
 	gtk_box_pack_start(GTK_BOX(h_button_box), close_bt, FALSE, FALSE, 0);
 	gtk_widget_show(close_bt);
@@ -642,13 +647,15 @@ void create_analyse_window(struct sctp_analyse* u_data)
 	gtk_box_pack_start(GTK_BOX(h_button_box), graph_bt1, FALSE, FALSE, 0);
 	gtk_widget_show(graph_bt1);
 	SIGNAL_CONNECT(graph_bt1, "clicked", on_graph2_dlg, u_data);
-
 	graph_bt2 = gtk_button_new_with_label("Graph Bytes");
 	gtk_box_pack_start(GTK_BOX(h_button_box), graph_bt2, FALSE, FALSE, 0);
 	gtk_widget_show(graph_bt2);
 	SIGNAL_CONNECT(graph_bt2, "clicked", on_graph_byte2_dlg,u_data);
-
-
+	if (u_data->assoc->n_array_tsn2==0)
+	{
+		gtk_widget_set_sensitive(graph_bt1, FALSE);
+		gtk_widget_set_sensitive(graph_bt2, FALSE);
+	}
 	close_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
 	gtk_box_pack_start(GTK_BOX(h_button_box), close_bt, FALSE, FALSE, 0);
 	gtk_widget_show(close_bt);
@@ -698,7 +705,7 @@ void assoc_analyse(sctp_assoc_info_t* assoc)
 }
 
 
-void sctp_analyse_cb(struct sctp_analyse* u_data)
+static void sctp_analyse_cb(struct sctp_analyse* u_data)
 {
 	guint8* ip_src;
 	guint16 srcport;
@@ -752,9 +759,9 @@ void sctp_analyse_cb(struct sctp_analyse* u_data)
 		return;
 	}
 
-	ip_src = malloc(edt->pi.src.len);
+	ip_src = g_malloc(edt->pi.src.len);
 	memcpy(ip_src, edt->pi.src.data, edt->pi.src.len);
-	ip_dst = malloc(edt->pi.dst.len);
+	ip_dst = g_malloc(edt->pi.dst.len);
 	memcpy(ip_dst, edt->pi.dst.data, edt->pi.dst.len);
 	srcport = edt->pi.srcport;
 	dstport = edt->pi.destport;
@@ -867,7 +874,7 @@ void sctp_analyse_start(GtkWidget *w _U_, gpointer data _U_)
 	u_data->window       = NULL;
 	u_data->num_children = 0;
 
-	cf_retap_packets(&cfile);
+	cf_retap_packets(&cfile, FALSE);
 	sctp_analyse_cb(u_data);
 }
 
@@ -875,6 +882,6 @@ void sctp_analyse_start(GtkWidget *w _U_, gpointer data _U_)
 void
 register_tap_listener_sctp_analyse(void)
 {
-	register_tap_menu_item("SCTP/Analyse Association", REGISTER_TAP_GROUP_TELEPHONY,
+	register_stat_menu_item("SCTP/Analyse Association", REGISTER_STAT_GROUP_TELEPHONY,
 	                       sctp_analyse_start, NULL, NULL, NULL);
 }
