@@ -3,7 +3,7 @@
  * Helper routines common to all service response time statistics
  * tap.
  *
- * $Id: service_response_time_table.h 12910 2005-01-01 02:57:02Z guy $
+ * $Id: service_response_time_table.h 15509 2005-08-22 07:12:20Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -26,6 +26,7 @@
 
 #include <gtk/gtk.h>
 #include "epan/nstime.h"
+#include "../timestats.h"
 
 /** @file
  *  Helper routines common to all service response time statistics tap.
@@ -34,10 +35,7 @@
 /** Procedure data */
 typedef struct _srt_procedure_t {
 	char *entries[6];   /**< column entries */
-	int num;            /**< number of calls seen */
-	nstime_t min;       /**< minimum srt */
-	nstime_t max;       /**< maximum srt */
-	nstime_t tot;       /**< average srt */
+	timestat_t stats;   /**< stats */
 } srt_procedure_t;
 
 /** Statistics table */
@@ -58,7 +56,8 @@ typedef struct _srt_stat_table {
  * @param vbox the corresponding GtkVBox to fill in
  * @param filter_string filter string or NULL
  */
-void init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox, char *filter_string);
+void init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox,
+                    const char *filter_string);
 
 /** Init an srt table row data structure.
  *
@@ -66,7 +65,7 @@ void init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox, char *f
  * @param index number of procedure
  * @param procedure the procedures name
  */
-void init_srt_table_row(srt_stat_table *rst, int index, char *procedure);
+void init_srt_table_row(srt_stat_table *rst, int index, const char *procedure);
 
 /** Add srt response to table row data. This will not draw the data!
  *
