@@ -1,7 +1,7 @@
 /* simple_dialog.c
  * Simple message dialog box routines.
  *
- * $Id: simple_dialog.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: simple_dialog.c 15886 2005-09-20 01:53:38Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,7 @@
 #include "gtkglobals.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "compat_macros.h"
 
 #include <epan/strutil.h>
@@ -41,12 +41,14 @@
 #include "image/stock_dialog_error_48.xpm"
 #include "image/stock_dialog_info_48.xpm"
 #include "image/stock_dialog_warning_48.xpm"
+#include "image/stock_dialog_stop_48.xpm"
 
 static void simple_dialog_cancel_cb(GtkWidget *, gpointer);
 
 #define CALLBACK_FCT_KEY    "ESD_Callback_Fct"
 #define CALLBACK_BTN_KEY    "ESD_Callback_Btn"
 #define CALLBACK_DATA_KEY   "ESD_Callback_Data"
+#define CHECK_BUTTON        "ESD_Check_Button"
 
 /*
  * Queue for messages requested before we have a main window.
@@ -62,13 +64,13 @@ static GSList *message_queue;
 static GtkWidget *
 display_simple_dialog(gint type, gint btn_mask, char *message)
 {
-  GtkWidget   *win, *main_vb, *top_hb, *type_pm, *msg_label,
+  GtkWidget   *win, *main_vb, *top_hb, *msg_vb, *type_pm, *msg_label, *ask_cb,
               *bbox, *ok_bt, *yes_bt, *bt, *save_bt, *dont_save_bt;
   GdkPixmap   *pixmap;
   GdkBitmap   *mask;
   GtkStyle    *style;
   GdkColormap *cmap;
-  gchar      **icon;
+  const gchar **icon;
 
   /* Main window */
   switch (type) {
@@ -81,6 +83,9 @@ display_simple_dialog(gint type, gint btn_mask, char *message)
   case ESD_TYPE_ERROR:
     icon = stock_dialog_error_48_xpm;
     break;
+  case ESD_TYPE_STOP :
+    icon = stock_dialog_stop_48_xpm;
+    break;
   case ESD_TYPE_INFO :
   default :
     icon = stock_dialog_info_48_xpm;
@@ -121,11 +126,15 @@ display_simple_dialog(gint type, gint btn_mask, char *message)
    * For now, we give it a Win32 title of just "Ethereal"; we should
    * arguably take an argument for the title.
    */
+  if(btn_mask == ESD_BTN_NONE) {
+	win = splash_window_new();
+  } else {
 #ifdef _WIN32
-  win = dlg_window_new("Ethereal");
+	win = dlg_window_new("Ethereal");
 #else
-  win = dlg_window_new("");
+    win = dlg_window_new("");
 #endif
+  }
 
   gtk_window_set_modal(GTK_WINDOW(win), TRUE);
   gtk_container_border_width(GTK_CONTAINER(win), 6);
@@ -141,15 +150,23 @@ display_simple_dialog(gint type, gint btn_mask, char *message)
   gtk_container_add(GTK_CONTAINER(main_vb), top_hb);
   gtk_widget_show(top_hb);
 
+  /* icon */
   style = gtk_widget_get_style(win);
   cmap  = gdk_colormap_get_system();
   pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, cmap,  &mask,
-    &style->bg[GTK_STATE_NORMAL], icon);
+    &style->bg[GTK_STATE_NORMAL], (gchar **) icon);
   type_pm = gtk_pixmap_new(pixmap, mask);
   gtk_misc_set_alignment (GTK_MISC (type_pm), 0.5, 0.0);
   gtk_container_add(GTK_CONTAINER(top_hb), type_pm);
   gtk_widget_show(type_pm);
 
+  /* column for message and optional check button */
+  msg_vb = gtk_vbox_new(FALSE, 6);
+  gtk_box_set_spacing(GTK_BOX(msg_vb), 24);
+  gtk_container_add(GTK_CONTAINER(top_hb), msg_vb);
+  gtk_widget_show(msg_vb);
+
+  /* message */
   msg_label = gtk_label_new(message);
 
 #if GTK_MAJOR_VERSION >= 2
@@ -159,9 +176,19 @@ display_simple_dialog(gint type, gint btn_mask, char *message)
 
   gtk_label_set_justify(GTK_LABEL(msg_label), GTK_JUSTIFY_FILL);
   gtk_misc_set_alignment (GTK_MISC (type_pm), 0.5, 0.0);
-  gtk_container_add(GTK_CONTAINER(top_hb), msg_label);
+  gtk_container_add(GTK_CONTAINER(msg_vb), msg_label);
   gtk_widget_show(msg_label);
 
+  if(btn_mask == ESD_BTN_NONE) {
+	gtk_widget_show(win);
+	return win;
+  }
+
+  /* optional check button */
+  ask_cb = gtk_check_button_new_with_label("replace with text...");
+  gtk_container_add(GTK_CONTAINER(msg_vb), ask_cb);
+  OBJECT_SET_DATA(win, CHECK_BUTTON, ask_cb);
+
   /* Button row */
   switch(btn_mask) {
   case(ESD_BTN_OK):
@@ -342,7 +369,7 @@ simple_dialog(ESD_TYPE_E type, gint btn_mask, const gchar *msg_format, ...)
 
 static void
 simple_dialog_cancel_cb(GtkWidget *w, gpointer win) {
-  gint button       = GPOINTER_TO_INT(    OBJECT_GET_DATA(w,   CALLBACK_BTN_KEY));
+  gint button = GPOINTER_TO_INT(    OBJECT_GET_DATA(w, CALLBACK_BTN_KEY));
   simple_dialog_cb_t    callback_fct    = OBJECT_GET_DATA(win, CALLBACK_FCT_KEY);
   gpointer              data            = OBJECT_GET_DATA(win, CALLBACK_DATA_KEY);
 
@@ -352,6 +379,12 @@ simple_dialog_cancel_cb(GtkWidget *w, gpointer win) {
     (callback_fct) (win, button, data);
 }
 
+void
+simple_dialog_close(gpointer dialog)
+{
+	window_destroy(GTK_WIDGET(dialog));
+}
+
 void simple_dialog_set_cb(gpointer dialog, simple_dialog_cb_t callback_fct, gpointer data)
 {
 
@@ -359,6 +392,22 @@ void simple_dialog_set_cb(gpointer dialog, simple_dialog_cb_t callback_fct, gpoi
     OBJECT_SET_DATA(GTK_WIDGET(dialog), CALLBACK_DATA_KEY, data);
 }
 
+void simple_dialog_check_set(gpointer dialog, gchar *text) {
+    GtkWidget *ask_cb = OBJECT_GET_DATA(dialog, CHECK_BUTTON);
+
+#if GTK_MAJOR_VERSION >= 2
+    /* XXX - find a way to set the GtkButton label in GTK 1.x */
+    gtk_button_set_label(GTK_BUTTON(ask_cb), text);
+#endif
+    gtk_widget_show(ask_cb);
+}
+
+gboolean simple_dialog_check_get(gpointer dialog) {
+    GtkWidget *ask_cb = OBJECT_GET_DATA(dialog, CHECK_BUTTON);
+
+    return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ask_cb));
+}
+
 char *
 simple_dialog_primary_start(void) {
     return PRIMARY_TEXT_START;
