@@ -1,7 +1,7 @@
 /* sip_stat.c
  * sip_stat   2004 Martin Mathieson
  *
- * $Id: sip_stat.c 13289 2005-02-04 18:44:44Z ulfl $
+ * $Id: sip_stat.c 17493 2006-03-07 13:11:08Z lroland $
  * Copied from http_stat.c
  *
  * Ethereal - Network traffic analyzer
@@ -34,17 +34,16 @@
 #include <epan/packet_info.h>
 #include <epan/epan.h>
 
-#include "tap_menu.h"
 #include "simple_dialog.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include <epan/dissectors/packet-sip.h>
 #include "../globals.h"
 #include "compat_macros.h"
+#include "../stat_menu.h"
 #include "../tap_dfilter_dlg.h"
-#include "tap_dfilter_dlg.h"
 
 #define SUM_STR_MAX	1024
 
@@ -73,13 +72,13 @@ typedef struct _sip_stats_t {
  * for example it can be { 3, 404, "Not Found" ,...}
  * which means we captured 3 reply sip/1.1 404 Not Found */
 typedef struct _sip_response_code_t {
-    guint32 	 packets;		    /* 3 */
-    guint	 	 response_code;		/* 404 */
-    gchar		*name;			    /* "Not Found" */
-    GtkWidget	*widget;		    /* Label where we display it */
-    GtkWidget	*table;			    /* Table in which we put it,
-                                       e.g. client_error_table */
-    sipstat_t	*sp;                /* Pointer back to main struct */
+    guint32 	 packets;		/* 3 */
+    guint 	 response_code;		/* 404 */
+    const gchar	*name;			/* "Not Found" */
+    GtkWidget	*widget;		/* Label where we display it */
+    GtkWidget	*table;			/* Table in which we put it,
+					   e.g. client_error_table */
+    sipstat_t	*sp;			/* Pointer back to main struct */
 } sip_response_code_t;
 
 /* Used to keep track of the stats for a specific request string */
@@ -120,6 +119,7 @@ static const value_string vals_status_code[] = {
     { 407, "Proxy Authentication Required"},
     { 408, "Request Timeout"},
     { 410, "Gone"},
+    { 412, "Conditional Request Failed"},
     { 413, "Request Entity Too Large"},
     { 414, "Request-URI Too Long"},
     { 415, "Unsupported Media Type"},
@@ -193,6 +193,8 @@ sip_draw_hash_requests(gchar *key _U_ , sip_request_method_t *data, gchar * unus
 {
     gchar string_buff[SUM_STR_MAX];
 
+    g_assert(data!=NULL);
+
     if (data->packets==0)
     {
         return;
@@ -222,10 +224,8 @@ sip_draw_hash_responses(gint * key _U_ , sip_response_code_t *data, gchar * unus
 {
     gchar string_buff[SUM_STR_MAX];
 
-    if (data==NULL)
-    {
-        g_warning("C'est quoi ce borderl key=%d\n", *key);
-    }
+    g_assert(data!=NULL);
+
     if (data->packets==0)
     {
         return;
@@ -498,10 +498,10 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
 
 /* Create a new instance of gtk_sipstat. */
 static void
-gtk_sipstat_init(char *optarg)
+gtk_sipstat_init(const char *optarg, void *userdata _U_)
 {
     sipstat_t *sp;
-    char *filter = NULL;
+    const char *filter = NULL;
     GString	*error_string;
     char *title = NULL;
     GtkWidget  *main_vb, *separator,
@@ -648,7 +648,7 @@ gtk_sipstat_init(char *optarg)
     window_present(sp->win);
 
     sip_init_hash(sp);
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
 }
 
 static tap_dfilter_dlg sip_stat_dlg = {
@@ -662,8 +662,5 @@ static tap_dfilter_dlg sip_stat_dlg = {
 void
 register_tap_listener_gtksipstat(void)
 {
-    register_ethereal_tap("sip,stat", gtk_sipstat_init);
-
-    register_tap_menu_item("SIP", REGISTER_TAP_GROUP_NONE,
-                           gtk_tap_dfilter_dlg_cb, NULL, NULL, &(sip_stat_dlg));
+    register_dfilter_stat(&sip_stat_dlg, "SIP", REGISTER_STAT_GROUP_TELEPHONY);
 }
