@@ -1,7 +1,7 @@
 /* stream_prefs.c
  * Dialog boxes for preferences for the stream window
  *
- * $Id: stream_prefs.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: stream_prefs.c 16027 2005-09-27 21:51:08Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -266,7 +266,7 @@ stream_prefs_apply(GtkWidget *w _U_)
 {
 	follow_redraw_all();
 
-	update_marked_frames();
+	packet_list_update_marked_frames();
 }
 
 void
