@@ -2,7 +2,7 @@
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
- * $Id: supported_protos_dlg.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: supported_protos_dlg.c 16672 2005-12-04 04:22:43Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,7 +34,7 @@
 #include <epan/prefs.h>
 #include "globals.h"
 #include "gtkglobals.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "compat_macros.h"
 #include "dlg_utils.h"
 #include "font_utils.h"
@@ -277,7 +277,7 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
   const char *type_name;
   void *cookie, *cookie2;
   protocol_t *protocol;
-  char *name, *short_name, *filter_name;
+  const char *name, *short_name, *filter_name;
   int namel = 0, short_namel = 0, filter_namel = 0;
   int count, fcount;
 
@@ -303,7 +303,7 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
     for (i = proto_get_first_protocol(&cookie); i != -1;
          i = proto_get_next_protocol(&cookie)) {
 	    count++;
-        protocol = find_protocol_by_id(i);
+	    protocol = find_protocol_by_id(i);
 	    name = proto_get_protocol_name(i);
 	    short_name = proto_get_protocol_short_name(protocol);
 	    filter_name = proto_get_protocol_filter_name(i);
@@ -328,7 +328,7 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
     /* ok, display the correctly aligned strings */
     for (i = proto_get_first_protocol(&cookie); i != -1;
          i = proto_get_next_protocol(&cookie)) {
-        protocol = find_protocol_by_id(i);
+	    protocol = find_protocol_by_id(i);
 	    name = proto_get_protocol_name(i);
 	    short_name = proto_get_protocol_short_name(protocol);
 	    filter_name = proto_get_protocol_filter_name(i);
@@ -375,8 +375,10 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
 			    maxlen = len;
 		    if ((len = strlen(hfinfo->name)) > maxlen2)
 			    maxlen2 = len;
-		    if ((len = strlen(hfinfo->blurb)) > maxlen4)
-			    maxlen4 = len;
+		    if (hfinfo->blurb != NULL) {
+			    if ((len = strlen(hfinfo->blurb)) > maxlen4)
+				maxlen4 = len;
+		    }
 	    }
     }
 
@@ -391,7 +393,7 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
     fcount = 0;
     for (i = proto_get_first_protocol(&cookie); i != -1;
          i = proto_get_next_protocol(&cookie)) {
-        protocol = find_protocol_by_id(i);
+	    protocol = find_protocol_by_id(i);
 	    name = proto_get_protocol_name(i);
 	    short_name = proto_get_protocol_short_name(protocol);
 	    filter_name = proto_get_protocol_filter_name(i);
@@ -417,11 +419,18 @@ static void set_supported_text(GtkWidget *w, supported_type_t type)
 			    continue;
 
 		    type_name = ftype_pretty_name(hfinfo->type);
-		    len = g_snprintf(buffer, BUFF_LEN, "%*s %*s %*s (%s)\n",
-				   -maxlen,  hfinfo->abbrev,
-				   -maxlen2, hfinfo->name,
-				   -maxlen4, hfinfo->blurb,
-				   type_name);
+		    if (hfinfo->blurb != NULL && hfinfo->blurb[0] != '\0') {
+			    len = g_snprintf(buffer, BUFF_LEN, "%*s %*s %*s (%s)\n",
+					     -maxlen,  hfinfo->abbrev,
+					     -maxlen2, hfinfo->name,
+					     -maxlen4, hfinfo->blurb,
+					     type_name);
+		    } else {
+			    len = g_snprintf(buffer, BUFF_LEN, "%*s %*s (%s)\n",
+					     -maxlen,  hfinfo->abbrev,
+					     -maxlen2, hfinfo->name,
+					     type_name);
+		    }
 #if GTK_MAJOR_VERSION < 2
 		    if (len > maxlen3) {
 			    maxlen3 = len;
