@@ -1,7 +1,7 @@
 /* voip_calls.h
  * VoIP calls summary addition for ethereal
  *
- * $Id: voip_calls.h 14852 2005-07-05 05:16:28Z etxrab $
+ * $Id: voip_calls.h 17676 2006-03-19 20:45:45Z guy $
  *
  * Copyright 2004, Ericsson , Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -39,6 +39,7 @@
 #include <glib.h>
 #include <stdio.h>
 #include <epan/address.h>
+#include <epan/guid-utils.h>
 
 /****************************************************************************/
 /* defines voip call state */
@@ -52,7 +53,7 @@ typedef enum _voip_call_state {
 		VOIP_UNKNOWN
 } voip_call_state;
 
-extern char *voip_call_state_name[7];
+extern const char *voip_call_state_name[7];
 
 typedef enum _voip_call_active_state {
 		VOIP_ACTIVE,
@@ -65,10 +66,11 @@ typedef enum _voip_protocol {
 		VOIP_H323,
 		VOIP_MGCP,
 		VOIP_AC_ISDN,
-		VOIP_AC_CAS
+		VOIP_AC_CAS,
+		MEDIA_T38
 } voip_protocol;
 
-extern char *voip_protocol_name[6];
+extern const char *voip_protocol_name[7];
 
 /* defines specific SIP data */
 
@@ -99,7 +101,7 @@ typedef struct _h245_address {
 
 /* defines specific H323 data */
 typedef struct _h323_calls_info {
-	guint8 *guid;	/* Call ID to identify a H225 */
+	e_guid_t *guid;	/* Call ID to identify a H225 */
 	GList*  h245_list;   /* list of H245 Address and ports for tunneling off calls*/
 	address h225SetupAddr; /* we use the SETUP H225 IP to determine if packets are forward or reverse */					
 	gboolean is_h245;
@@ -174,6 +176,7 @@ typedef struct _voip_calls_tapinfo {
 	int q931_dummy;
 	int mgcp_dummy;
 	int actrace_dummy;
+	int t38_dummy;
 } voip_calls_tapinfo_t;
 
 
@@ -229,6 +232,7 @@ void rtp_init_tap(void);
 void rtp_event_init_tap(void);
 void mgcp_calls_init_tap(void);
 void actrace_calls_init_tap(void);
+void t38_init_tap(void);
 
 /*
 * Removes the voip_calls tap listener (if not already done)
@@ -245,6 +249,7 @@ void remove_tap_listener_rtp(void);
 void remove_tap_listener_rtp_event(void);
 void remove_tap_listener_mgcp_calls(void);
 void remove_tap_listener_actrace_calls(void);
+void remove_tap_listener_t38(void);
 
 /*
 * Retrieves a constant reference to the unique info structure of the voip_calls tap listener.
