@@ -1,7 +1,7 @@
 /* voip_calls_dlg.c
  * VoIP calls summary addition for ethereal
  *
- * $Id: voip_calls_dlg.c 13699 2005-03-10 20:23:04Z gerald $
+ * $Id: voip_calls_dlg.c 17676 2006-03-19 20:45:45Z guy $
  *
  * Copyright 2004, Ericsson , Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -50,9 +50,11 @@
 #include "epan/filesystem.h"
 
 #include <epan/tap.h>
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "compat_macros.h"
 #include "gtkglobals.h"
 
@@ -89,9 +91,11 @@ static GList *last_list = NULL;
 static guint32 calls_nb = 0;     /* number of displayed calls */
 static guint32 calls_ns = 0;     /* number of selected calls */
 
-static graph_analysis_data_t *graph_analysis_data;
+static graph_analysis_data_t *graph_analysis_data = NULL;
 
 #define NUM_COLS 9
+static const GdkColor COLOR_SELECT = {0, 0x00ff, 0x80ff, 0x80ff};
+static const GdkColor COLOR_DEFAULT = {0, 0xffff, 0xffff, 0xffff};
 
 /****************************************************************************/
 /* append a line to clist */
@@ -105,10 +109,12 @@ static void add_to_clist(voip_calls_info_t* strinfo)
 	isup_calls_info_t *tmp_isupinfo;
 	h323_calls_info_t *tmp_h323info;
 	gboolean tmp_bool = FALSE;
+	GdkColor color = COLOR_SELECT;
 	for (c=0;c<NUM_COLS;c++){
 		data[c]=&field[c][0];
 	}
 
+/*	strinfo->selected = FALSE;*/
 
 	g_snprintf(field[0], 15, "%i.%2i", strinfo->start_sec, strinfo->start_usec/10000);
 	g_snprintf(field[1], 15, "%i.%2i", strinfo->stop_sec, strinfo->stop_usec/10000);
@@ -144,6 +150,12 @@ static void add_to_clist(voip_calls_info_t* strinfo)
 	
 	added_row = gtk_clist_append(GTK_CLIST(clist), data);
 
+	/* set the background color if selected */
+	if (strinfo->selected) { 
+		calls_ns++;
+		gtk_clist_set_background(GTK_CLIST(clist), added_row, &color);
+	}
+
 	/* set data pointer of last row to point to user data for that row */
 	gtk_clist_set_row_data(GTK_CLIST(clist), added_row, strinfo);
 
@@ -168,7 +180,7 @@ static void add_to_clist(voip_calls_info_t* strinfo)
 }
 
 
-void voip_calls_remove_tap_listener(void)
+static void voip_calls_remove_tap_listener(void)
 {
 	/* Remove the calls tap listener */
 	remove_tap_listener_sip_calls();
@@ -183,6 +195,8 @@ void voip_calls_remove_tap_listener(void)
 	if (find_tap_id("mgcp")) {
 		remove_tap_listener_mgcp_calls();
 	}
+	remove_tap_listener_actrace_calls();
+	remove_tap_listener_t38();
 }
 
 /****************************************************************************/
@@ -196,10 +210,12 @@ voip_calls_on_destroy                      (GtkObject       *object _U_,
 	voip_calls_remove_tap_listener();
 
 	/* Clean up memory used by calls tap */
-	voip_calls_reset((voip_calls_tapinfo_t*) voip_calls_get_info());
+	voip_calls_dlg_reset(NULL);
 
 	/* Note that we no longer have a "VoIP Calls" dialog box. */
 	voip_calls_dlg = NULL;
+
+	graph_analysis_data = NULL;
 }
 
 
@@ -226,7 +242,7 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 	const gchar *filter_string;
 	gchar c;
 	GString *filter_string_fwd;
-	gchar *filter_prepend;
+	const gchar *filter_prepend;
 	gboolean isFirst = TRUE;
 	GList* list;
 	guint filter_length = 0;
@@ -301,13 +317,12 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 			case VOIP_H323:
 				tmp_h323info = selected_call_fwd->prot_info;
 				g_string_sprintfa(filter_string_fwd,
-				   "((h225.guid == %x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x:%x || q931.call_ref == %x:%x || q931.call_ref == %x:%x) ",
-				   (guint8)tmp_h323info->guid[0], (guint8)tmp_h323info->guid[1], (guint8)tmp_h323info->guid[2],
-				   (guint8)tmp_h323info->guid[3], (guint8)tmp_h323info->guid[4], (guint8)tmp_h323info->guid[5], (guint8)tmp_h323info->guid[6],
-			   (guint8)tmp_h323info->guid[7], (guint8)tmp_h323info->guid[8], (guint8)tmp_h323info->guid[9], (guint8)tmp_h323info->guid[10],
-				   (guint8)tmp_h323info->guid[11], (guint8)tmp_h323info->guid[12], (guint8)tmp_h323info->guid[13], (guint8)tmp_h323info->guid[14],
-				   (guint8)tmp_h323info->guid[15], (guint8)(tmp_h323info->q931_crv & 0xff), (guint8)((tmp_h323info->q931_crv & 0xff00)>>8)
-				   , (guint8)(tmp_h323info->q931_crv2 & 0xff), (guint8)((tmp_h323info->q931_crv2 & 0xff00)>>8));
+				   "((h225.guid == %s || q931.call_ref == %x:%x || q931.call_ref == %x:%x) ",
+				   guid_to_str(&tmp_h323info->guid[0]),
+				   (guint8)(tmp_h323info->q931_crv & 0xff),
+				   (guint8)((tmp_h323info->q931_crv & 0xff00)>>8),
+				   (guint8)(tmp_h323info->q931_crv2 & 0xff),
+				   (guint8)((tmp_h323info->q931_crv2 & 0xff00)>>8));
 				list = g_list_first(tmp_h323info->h245_list);
 				while (list)
 				{
@@ -321,6 +336,9 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 				gtk_entry_append_text(GTK_ENTRY(main_display_filter_widget), filter_string_fwd->str);
 				break;
 			case VOIP_MGCP:
+			case VOIP_AC_ISDN:
+			case VOIP_AC_CAS:
+			case MEDIA_T38:
 				/* XXX - not supported */
 				break;
 		}
@@ -376,9 +394,6 @@ on_graph_bt_clicked                    (GtkButton       *button _U_,
 		graph_analysis_update(graph_analysis_data);		/* refresh it */
 }
 
-static const GdkColor COLOR_SELECT = {0, 0x00ff, 0x80ff, 0x80ff};
-static const GdkColor COLOR_DEFAULT = {0, 0xffff, 0xffff, 0xffff};
-
 /****************************************************************************/
 /* when the user selects a row in the calls list */
 static void
@@ -393,12 +408,15 @@ voip_calls_on_select_row(GtkCList *clist,
 	
 	selected_call_fwd = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 
+	if (selected_call_fwd==NULL)
+		return;
+
 	if (!selected_call_fwd->selected)
 		calls_ns++;
 	else
 		calls_ns--;
 
-	g_snprintf(label_text, 256,
+	g_snprintf(label_text, 80,
 	        "Detected %d VoIP %s. Selected %d %s.",
 	        calls_nb, 
             plurality(calls_nb, "Call", "Calls"),
@@ -421,10 +439,14 @@ voip_calls_on_select_row(GtkCList *clist,
 		color = COLOR_DEFAULT;
 
 	gtk_clist_set_background(GTK_CLIST(clist), row, &color);
-	
-	/*gtk_widget_set_sensitive(bt_unselect, TRUE);*/
-	gtk_widget_set_sensitive(bt_filter, TRUE);
-	gtk_widget_set_sensitive(bt_graph, TRUE);
+
+	if 	(calls_ns > 0) {
+		gtk_widget_set_sensitive(bt_filter, TRUE);
+		gtk_widget_set_sensitive(bt_graph, TRUE);
+	} else {
+		gtk_widget_set_sensitive(bt_filter, FALSE);
+		gtk_widget_set_sensitive(bt_graph, FALSE);
+	}
 
 	/* TODO: activate other buttons when implemented */
 }
@@ -478,7 +500,7 @@ voip_calls_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 {
 	char *text1 = NULL;
 	char *text2 = NULL;
-	int i1, i2, i3, i4;
+	guint i1, i2, i3, i4;
 
 	const GtkCListRow *row1 = (const GtkCListRow *) ptr1;
 	const GtkCListRow *row2 = (const GtkCListRow *) ptr2;
@@ -528,7 +550,7 @@ static void voip_calls_dlg_create (void)
 	GtkWidget *bt_close;
 	GtkTooltips *tooltips = gtk_tooltips_new();
 
-	gchar *titles[NUM_COLS] =  {"Start Time", "Stop Time", "Initial Speaker", "From",  "To", "Protocol", "Packets", "State", "Comments"};
+	const gchar *titles[NUM_COLS] =  {"Start Time", "Stop Time", "Initial Speaker", "From",  "To", "Protocol", "Packets", "State", "Comments"};
 	column_arrows *col_arrows;
 	GtkWidget *column_lb;
 	int i;
@@ -662,26 +684,27 @@ static void voip_calls_dlg_create (void)
 void voip_calls_dlg_update(GList *list)
 {
 	gchar label_text[256];
-guint foo;
 	if (voip_calls_dlg != NULL) {
-		gtk_clist_clear(GTK_CLIST(clist));
 		calls_nb = 0;
 		calls_ns = 0;
-        	g_snprintf(label_text, 256,
-        		"Total: Calls: %d   Start packets: %d   Completed calls: %d   Rejected calls: %d",
-			g_list_length(voip_calls_get_info()->strinfo_list),
-		        voip_calls_get_info()->start_packets, 
-			voip_calls_get_info()->completed_calls,
-			voip_calls_get_info()->rejected_calls);
-        	gtk_label_set(GTK_LABEL(status_label), label_text);
 
-		foo=	g_list_length(list);
+       	g_snprintf(label_text, 256,
+       		"Total: Calls: %d   Start packets: %d   Completed calls: %d   Rejected calls: %d",
+		g_list_length(voip_calls_get_info()->strinfo_list),
+        voip_calls_get_info()->start_packets, 
+		voip_calls_get_info()->completed_calls,
+		voip_calls_get_info()->rejected_calls);
+       	gtk_label_set(GTK_LABEL(status_label), label_text);
+
+		gtk_clist_freeze(GTK_CLIST(clist));
+		gtk_clist_clear(GTK_CLIST(clist));
 		list = g_list_first(list);
 		while (list)
 		{
 			add_to_clist((voip_calls_info_t*)(list->data));
 			list = g_list_next(list);
 		}
+		gtk_clist_thaw(GTK_CLIST(clist));
 
 		g_snprintf(label_text, 256,
 	        	"Detected %d VoIP %s. Selected %d %s.",
@@ -690,8 +713,6 @@ guint foo;
 			calls_ns,
 			plurality(calls_ns, "Call", "Calls"));
 		gtk_label_set(GTK_LABEL(top_label), label_text);
-
-		voip_calls_on_unselect(NULL, NULL);
 	}
 
 	last_list = list;
@@ -702,7 +723,10 @@ guint foo;
 /* draw function for tap listeners to keep the window up to date */
 void voip_calls_dlg_draw(void *ptr _U_)
 {
-	voip_calls_dlg_update(voip_calls_get_info()->strinfo_list);
+	if (voip_calls_get_info()->redraw) {
+		voip_calls_dlg_update(voip_calls_get_info()->strinfo_list);
+		voip_calls_get_info()->redraw = FALSE;
+	}
 }
 
 /* reset function for tap listeners to clear window, if necessary */
@@ -710,13 +734,29 @@ void voip_calls_dlg_reset(void *ptr _U_)
 {
 	/* Clean up memory used by calls tap */
 	voip_calls_reset((voip_calls_tapinfo_t*) voip_calls_get_info());
+
+	/* close the graph window if open */
+	if (graph_analysis_data->dlg.window != NULL) {
+		window_cancel_button_cb(NULL, graph_analysis_data->dlg.window);
+		graph_analysis_data->dlg.window = NULL;
+	}
 }
 
 /* init function for tap */
-static void
-voip_calls_init_tap(char *dummy _U_)
+/* Made extern only for "Fax T38 Analysis..." */
+void
+voip_calls_init_tap(const char *dummy _U_, void* userdata _U_)
 {
-	graph_analysis_data_init();
+	gint c;
+	gchar *data[NUM_COLS];
+	gchar field[NUM_COLS][50];
+
+	if (graph_analysis_data == NULL) {
+		graph_analysis_data_init();
+		/* init the Graph Analysys */
+		graph_analysis_data = graph_analysis_init();
+		graph_analysis_data->graph_info = voip_calls_get_info()->graph_analysis;
+	}
 
 	/* Clean up memory used by calls tap */
 	voip_calls_reset((voip_calls_tapinfo_t*) voip_calls_get_info());
@@ -735,10 +775,8 @@ voip_calls_init_tap(char *dummy _U_)
 	if (find_tap_id("mgcp")) {
 		mgcp_calls_init_tap();
 	}
-	
-	/* init the Graph Analysys */
-	graph_analysis_data = graph_analysis_init();
-	graph_analysis_data->graph_info = voip_calls_get_info()->graph_analysis;
+	actrace_calls_init_tap();
+	t38_init_tap();
 
 	/* create dialog box if necessary */
 	if (voip_calls_dlg == NULL) {
@@ -747,27 +785,37 @@ voip_calls_init_tap(char *dummy _U_)
 		/* There's already a dialog box; reactivate it. */
 		reactivate_window(voip_calls_dlg);
 	}
+
+	voip_calls_get_info()->redraw = TRUE;
+	voip_calls_dlg_draw(NULL);
+	voip_calls_get_info()->redraw = TRUE;
+	for (c=0;c<NUM_COLS;c++){
+		data[c]=&field[c][0];
+		field[c][0] = '\0';
+	}
+	g_snprintf(field[3], 50, "Please wait...");
+	gtk_clist_append(GTK_CLIST(clist), data);
 	
 	/* Scan for VoIP calls calls (redissect all packets) */
-	cf_retap_packets(&cfile);
-	
+	cf_retap_packets(&cfile, FALSE);
+
 	/* Tap listener will be removed and cleaned up in voip_calls_on_destroy */
 }
 
 
 /****************************************************************************/
 /* entry point when called via the GTK menu */
-void voip_calls_launch(GtkWidget *w _U_, gpointer data _U_)
+static void voip_calls_launch(GtkWidget *w _U_, gpointer data _U_)
 {
-	voip_calls_init_tap("");
+	voip_calls_init_tap("",NULL);
 }
 
 /****************************************************************************/
 void
 register_tap_listener_voip_calls_dlg(void)
 {
-	register_ethereal_tap("voip,calls",voip_calls_init_tap);
-	register_tap_menu_item("VoIP Calls...", REGISTER_TAP_GROUP_NONE,
+	register_stat_cmd_arg("voip,calls",voip_calls_init_tap,NULL);
+	register_stat_menu_item("VoIP Calls", REGISTER_STAT_GROUP_TELEPHONY,
 	    voip_calls_launch, NULL, NULL, NULL);
 	    
 }
