@@ -4,7 +4,7 @@
  * Web Browser Plug-in
  * Copyright (C) 2003  Henrik Brix Andersen <brix@gimp.org>
  *
- * $Id: webbrowser.c 13152 2005-01-20 23:17:23Z ulfl $
+ * $Id: webbrowser.c 17534 2006-03-08 20:55:32Z gerald $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,9 @@
 #if defined(G_OS_WIN32)
 /* Win32 - use Windows shell services to start a browser */
 #include <windows.h>
+/* We're using Unicode */
+#include <tchar.h>
+#include <epan/strutil.h>
 /* if WIN32_LEAN_AND_MEAN is defined, shellapi.h is needed too */
 #include <shellapi.h>
 #elif defined (HAVE_OS_X_FRAMEWORKS)
@@ -95,7 +98,7 @@ browser_open_url (const gchar *url)
 {
 #if defined(G_OS_WIN32)
 
-  return ((gint) ShellExecute (HWND_DESKTOP, "open", url, NULL, NULL, SW_SHOWNORMAL) > 32);
+  return ((gint) ShellExecute (HWND_DESKTOP, _T("open"), utf_8to16(url), NULL, NULL, SW_SHOWNORMAL) > 32);
 
 #elif defined(HAVE_OS_X_FRAMEWORKS)
 
