@@ -1,7 +1,7 @@
 /* wsp_stat.c
  * wsp_stat   2003 Jean-Michel FAYARD
  *
- * $Id: wsp_stat.c 14812 2005-06-28 22:15:12Z guy $
+ * $Id: wsp_stat.c 17261 2006-02-12 00:54:56Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,17 +34,16 @@
 #include <epan/packet_info.h>
 #include <epan/epan.h>
 
-#include "tap_menu.h"
 #include "simple_dialog.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "../globals.h"
 #include "compat_macros.h"
 #include <epan/dissectors/packet-wsp.h>
+#include "../stat_menu.h"
 #include "../tap_dfilter_dlg.h"
-#include "tap_dfilter_dlg.h"
 
 /* used to keep track of the stats for a specific PDU type*/
 typedef struct _wsp_pdu_t {
@@ -304,10 +303,10 @@ wsp_init_table(wspstat_t *sp)
 /* When called, this function will create a new instance of gtk2-wspstat.
  */
 static void
-gtk_wspstat_init(char *optarg)
+gtk_wspstat_init(const char *optarg, void *userdata _U_)
 {
 	wspstat_t *sp;
-	char 		*filter=NULL;
+	const char 	*filter=NULL;
 	char 		*title=NULL;
 	GString		*error_string;
 	GtkWidget	*main_vb, *pdutypes_fr, *statuscode_fr ;
@@ -412,7 +411,7 @@ gtk_wspstat_init(char *optarg)
     gtk_widget_show_all(sp->win);
     window_present(sp->win);
 	
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
 }
 
 static tap_dfilter_dlg wsp_stat_dlg = {
@@ -425,8 +424,6 @@ static tap_dfilter_dlg wsp_stat_dlg = {
 void
 register_tap_listener_gtkwspstat(void)
 {
-	register_tap_listener_cmd_arg("wsp,stat", gtk_wspstat_init);
-
-	register_tap_menu_item("WAP-WSP...", REGISTER_TAP_GROUP_TELEPHONY,
-	    gtk_tap_dfilter_dlg_cb, NULL, NULL, &(wsp_stat_dlg));
+	register_dfilter_stat(&wsp_stat_dlg, "WAP-WSP",
+	    REGISTER_STAT_GROUP_TELEPHONY);
 }
