@@ -1,6 +1,6 @@
 #! @SHELL@
 #
-#  $Id: idl2eth.sh 11400 2004-07-18 00:24:25Z guy $
+#  $Id: idl2eth.sh 15362 2005-08-15 20:31:57Z jmayer $
 #
 #  File : idl2eth          
 #
@@ -55,18 +55,24 @@ fi
 # error.
 
 if [ -f $PYTHONPATH/site-packages/ethereal_be.py ] && [ -f $PYTHONPATH/site-packages/ethereal_gen.py ]; then
-    omniidl  -p $PYTHONPATH/site-packages -b ethereal_be $1
-    exit $?
+    exec omniidl  -p $PYTHONPATH/site-packages -b ethereal_be $@
+    /* not reached */
 fi
 
 # Try current directory.
 
 if [ -f ./ethereal_be.py ] && [ -f ./ethereal_gen.py ]; then
-    omniidl  -p ./ -b ethereal_be $1
-    exit $?
+    exec omniidl  -p ./ -b ethereal_be $@
+    /* not reached */
 fi
 
 # Could not find both ethereal_be.py AND ethereal_gen.py
+# So let's just try to run it without -p, hoping that the installation
+# set up a valid path.
+
+exec omniidl -b ethereal_be $@
+
+old code: not reached
 
 echo "idl2eth Error: Could not find both ethereal_be.py AND ethereal_gen.py."
 echo "Please ensure you have the PYTHONPATH variable set, or that ethereal_be.py "
