@@ -1,7 +1,7 @@
 /* log.h
  * log output definitions
  *
- * $Id: log.h 14438 2005-05-26 17:50:27Z ulfl $
+ * $Id: log.h 16819 2005-12-16 07:43:24Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,8 +31,11 @@
 /* capture child domain (the capture child might also contain file domain messages!) */
 #define LOG_DOMAIN_CAPTURE_CHILD 	"CaptureChild"
 
+/* main domain */
+#define LOG_DOMAIN_MAIN				"Main"
 
 /* enable very verbose capture log debug output */
+/* (might slightly degrade performance) */
 /*#define LOG_CAPTURE_VERBOSE*/
 
 
