@@ -1,7 +1,7 @@
 /* packet-range.c
  * Packet range routines (save, print, ...)
  *
- * $Id: packet-range.c 12320 2004-10-16 23:32:24Z guy $
+ * $Id: packet-range.c 15355 2005-08-14 23:25:20Z jmayer $
  *
  * Dick Gooris <gooris@lucent.com>
  * Ulf Lamping <ulf.lamping@web.de>
@@ -41,7 +41,7 @@
 #include "packet-range.h"
 
 /* (re-)calculate the packet counts (except the user specified range) */
-void packet_range_calc(packet_range_t *range) {
+static void packet_range_calc(packet_range_t *range) {
   guint32       current_count;
   guint32       mark_low;
   guint32       mark_high;
