@@ -2,7 +2,7 @@
  * Routines to Dissect Appendix C TLV's
  * Copyright 2002, Anand V. Narwani <anand[AT]narwani.org>
  *
- * $Id: packet-tlv.c,v 1.16 2003/12/13 03:18:38 guy Exp $
+ * $Id: packet-tlv.c 16956 2006-01-05 13:01:08Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,9 +27,6 @@
 # include "config.h"
 #endif
 
-
-#include "plugins/plugin_api.h"
-#include "plugins/plugin_api_defs.h"
 #include "moduleinfo.h"
 
 #include <stdio.h>
@@ -258,7 +255,7 @@ static const value_string dsc_act_vals[] = {
 
 static const value_string qos_param_vals[] = {
   {0x01, "Apply to provisioned set only"},
-  {0x02, "Perform admission control add apply to addmitted set"},
+  {0x02, "Perform admission control add apply to admitted set"},
   {0x03, "Apply to provisioned and admitted set; Perform admission control"},
   {0x04, "Perform admission control if needed and apply to active set"},
   {0x05,
