@@ -3,7 +3,7 @@
  * By Jan Kiszka <jan.kiszka@web.de>
  * Copyright 2003 Jan Kiszka
  *
- * $Id: packet-ircomm.c 13596 2005-03-05 06:28:10Z lroland $
+ * $Id: packet-ircomm.c 15264 2005-08-08 18:50:39Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -39,9 +39,6 @@
 #include <epan/packet.h>
 #include <epan/proto.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
 
 /*
  * See
