@@ -6,7 +6,7 @@
  * Extended by Jan Kiszka <jan.kiszka@web.de>
  * Copyright 2003 Jan Kiszka
  *
- * $Id: packet-irda.c 14713 2005-06-20 19:35:58Z gerald $
+ * $Id: packet-irda.c 15267 2005-08-09 14:38:11Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -44,10 +44,6 @@
 #include <epan/conversation.h>
 #include <epan/xdlc.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include "irda-appl.h"
 
 /*
@@ -564,7 +560,7 @@ static void dissect_iap_request(tvbuff_t* tvb, packet_info* pinfo, proto_tree* r
             conv = find_conversation(pinfo->fd->num, &srcaddr, &destaddr, PT_NONE, pinfo->srcport, pinfo->destport, 0);
             if (conv)
             {
-                iap_conv = (iap_conversation_t*)conversation_get_proto_data(conv, PT_NONE);
+                iap_conv = (iap_conversation_t*)conversation_get_proto_data(conv, proto_iap);
                 while (1)
                 {
                     if (iap_conv->iap_query_frame == pinfo->fd->num)
@@ -585,7 +581,7 @@ static void dissect_iap_request(tvbuff_t* tvb, packet_info* pinfo, proto_tree* r
             {
                 conv = conversation_new(pinfo->fd->num, &srcaddr, &destaddr, PT_NONE, pinfo->srcport, pinfo->destport, 0);
                 iap_conv = g_mem_chunk_alloc(iap_conv_chunk);
-                conversation_add_proto_data(conv, PT_NONE, (void*)iap_conv);
+                conversation_add_proto_data(conv, proto_iap, (void*)iap_conv);
             }
 
             /* Dissect IAP query if it is new */
@@ -596,15 +592,15 @@ static void dissect_iap_request(tvbuff_t* tvb, packet_info* pinfo, proto_tree* r
                 char    attr_name[256];
 
 
+                iap_conv->pnext           = NULL;
+                iap_conv->iap_query_frame = pinfo->fd->num;
+                iap_conv->pattr_dissector = NULL;
+
                 tvb_memcpy(tvb, class_name, offset + 1 + 1, clen);
                 class_name[clen] = 0;
                 tvb_memcpy(tvb, attr_name, offset + 1 + 1 + clen + 1, alen);
                 attr_name[alen] = 0;
 
-                iap_conv->pnext           = NULL;
-                iap_conv->iap_query_frame = pinfo->fd->num;
-                iap_conv->pattr_dissector = NULL;
-
                 /* Find the attribute dissector */
                 for (i = 0; class_dissector[i].class_name != NULL; i++)
                     if (strcmp(class_name, class_dissector[i].class_name) == 0)
@@ -727,7 +723,7 @@ static void dissect_iap_result(tvbuff_t* tvb, packet_info* pinfo, proto_tree* ro
     {
         num = pinfo->fd->num;
 
-        iap_conv = (iap_conversation_t*)conversation_get_proto_data(conv, PT_NONE);
+        iap_conv = (iap_conversation_t*)conversation_get_proto_data(conv, proto_iap);
         while (iap_conv && (iap_conv->iap_query_frame >= num))
             iap_conv = iap_conv->pnext;
 
@@ -838,7 +834,7 @@ static void dissect_iap_result(tvbuff_t* tvb, packet_info* pinfo, proto_tree* ro
                         }
 
                         ti = proto_tree_add_item(tree, hf_iap_list_entry, tvb, offset, 2 + 1 + attr_len, FALSE);
-                        snprintf(buf, sizeof(buf) - 1, "%d", n + 1);
+                        g_snprintf(buf, sizeof(buf) - 1, "%d", n + 1);
                         proto_item_append_text(ti, buf);
                         entry_tree = proto_item_add_subtree(ti, ett_iap_entry[n]);
 
@@ -1016,7 +1012,7 @@ static void dissect_appl_proto(tvbuff_t* tvb, packet_info* pinfo, proto_tree* ro
     {
         num = pinfo->fd->num;
 
-        lmp_conv = (lmp_conversation_t*)conversation_get_proto_data(conv, PT_NONE);
+        lmp_conv = (lmp_conversation_t*)conversation_get_proto_data(conv, proto_irlmp);
         while (lmp_conv && (lmp_conv->iap_result_frame >= num))
             lmp_conv = lmp_conv->pnext;
 
@@ -1254,7 +1250,7 @@ void add_lmp_conversation(packet_info* pinfo, guint8 dlsap, gboolean ttp, dissec
     conv = find_conversation(pinfo->fd->num, &destaddr, &srcaddr, PT_NONE, dlsap, 0, NO_PORT_B);
     if (conv)
     {
-        lmp_conv = (lmp_conversation_t*)conversation_get_proto_data(conv, PT_NONE);
+        lmp_conv = (lmp_conversation_t*)conversation_get_proto_data(conv, proto_irlmp);
         while (1)
         {
             /* Does entry already exist? */
@@ -1274,7 +1270,7 @@ void add_lmp_conversation(packet_info* pinfo, guint8 dlsap, gboolean ttp, dissec
     {
         conv = conversation_new(pinfo->fd->num, &destaddr, &srcaddr, PT_NONE, dlsap, 0, NO_PORT_B);
         lmp_conv = g_mem_chunk_alloc(lmp_conv_chunk);
-        conversation_add_proto_data(conv, PT_NONE, (void*)lmp_conv);
+        conversation_add_proto_data(conv, proto_irlmp, (void*)lmp_conv);
     }
 
     lmp_conv->pnext            = NULL;
@@ -1770,7 +1766,7 @@ static void dissect_irlap(tvbuff_t* tvb, packet_info* pinfo, proto_tree* root)
     pinfo->circuit_id = a;
 
     /* initially set address columns to connection address */
-    snprintf(addr, sizeof(addr)-1, "0x%02X", a >> 1);
+    g_snprintf(addr, sizeof(addr)-1, "0x%02X", a >> 1);
     if (check_col(pinfo->cinfo, COL_DEF_SRC))
         col_add_str(pinfo->cinfo, COL_DEF_SRC, addr);
     if (check_col(pinfo->cinfo, COL_DEF_DST))
