@@ -3,7 +3,7 @@
 *
 * Copyright 2004, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
 *
-* $Id: mate_setup.c 15015 2005-07-23 11:41:25Z jmayer $
+* $Id: mate_setup.c 16956 2006-01-05 13:01:08Z jmayer $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
@@ -183,7 +183,7 @@ extern gboolean add_hfid(header_field_info*  hfi, gchar* how, GHashTable* where)
 	}
 
 	if (! exists) {
-		report_error("MATE Error: cannot find field %s",hfi->abbrev);
+		report_error("MATE Error: cannot find field for attribute %s",how);
 	}	
 	return exists;
 }
@@ -293,7 +293,7 @@ static void analyze_pdu_config(mate_cfg_pdu* cfg) {
 	g_array_append_val(matecfg->hfrs,hfri);
 	
 	hfri.p_id = &(cfg->hfid_pdu_time_in_gop);
-	hfri.hfinfo.name = g_strdup_printf("%s time since begining of Gop",cfg->name);
+	hfri.hfinfo.name = g_strdup_printf("%s time since beginning of Gop",cfg->name);
 	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.TimeInGop",cfg->name);
 	hfri.hfinfo.type = FT_FLOAT;
 	hfri.hfinfo.display = BASE_DEC;
@@ -333,7 +333,7 @@ static void analyze_gop_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.StartTime",cfg->name);
 	hfri.hfinfo.type = FT_FLOAT;
 	hfri.hfinfo.display = BASE_DEC;
-	hfri.hfinfo.blurb = g_strdup_printf("Seconds passed since the begining of caputre to the start of this %s",cfg->name);
+	hfri.hfinfo.blurb = g_strdup_printf("Seconds passed since the beginning of capture to the start of this %s",cfg->name);
 
 	g_array_append_val(matecfg->hfrs,hfri);
 
@@ -475,7 +475,7 @@ static void analyze_gog_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 	hfri.hfinfo.name = g_strdup_printf("%s start time",cfg->name);
 	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.StartTime",cfg->name);
 	hfri.hfinfo.type = FT_FLOAT;
-	hfri.hfinfo.blurb = g_strdup_printf("Seconds passed since the begining of caputre to the start of this %s",cfg->name);
+	hfri.hfinfo.blurb = g_strdup_printf("Seconds passed since the beginning of capture to the start of this %s",cfg->name);
 	
 	g_array_append_val(matecfg->hfrs,hfri);
 		
@@ -917,14 +917,9 @@ extern mate_config* mate_make_config(const gchar* filename, int mate_hfid) {
 	
 	if ( mate_load_config(filename,matecfg) ) {
 		analyze_config();
-
-		/* if (dbg_cfg_lvl > 0) { */
-			print_config();
-		/* } */
-		
 	} else {
 		report_failure("MATE failed to configue!\n"
-					   "It is recomended that you fix your config and restart ethereal.\n"
+					   "It is recommended that you fix your config and restart ethereal.\n"
 					   "The reported error is:\n%s\n",matecfg->config_error->str);
 		
 		/* if (matecfg) destroy_mate_config(matecfg,FALSE); */
