@@ -2,7 +2,7 @@
 * Routines for megaco packet disassembly
 * RFC 3015
 *
-* $Id: packet-megaco.c 15047 2005-07-24 21:02:40Z gerald $
+* $Id: packet-megaco.c 17897 2006-04-18 20:29:49Z gerald $
 *
 * Christian Falckenberg, 2002/10/17
 * Copyright (c) 2002 by Christian Falckenberg
@@ -54,7 +54,7 @@
 #include <epan/addr_resolv.h>
 #include <epan/prefs.h>
 #include <epan/strutil.h>
-#include "sctpppids.h"
+#include <epan/sctpppids.h>
 #include <epan/dissectors/packet-tpkt.h>
 
 #ifndef ENABLE_STATIC
@@ -420,7 +420,11 @@ dissect_megaco_text(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			}
 			return;
 			break;
-		/* transactionResponseAck 	*/
+			/* transactionResponseAck 	
+			 * transactionResponseAck = ResponseAckToken LBRKT transactionAck
+             *                           *(COMMA transactionAck) RBRKT
+			 * transactionAck = transactionID / (transactionID "-" transactionID)
+			 */
 		case 'K':
 			tvb_offset  = tvb_find_guint8(tvb, tvb_offset, tvb_len, '{');
 			tokenlen = tvb_offset - tvb_previous_offset;
@@ -429,7 +433,8 @@ dissect_megaco_text(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				"TransactionResponseAck" );
 
 			tvb_previous_offset = tvb_skip_wsp(tvb, tvb_offset+1);
-			tvb_current_offset = tvb_skip_wsp_return(tvb, tvb_len-1)-1; /* cut last RBRKT */
+			tvb_current_offset = tvb_find_guint8(tvb, tvb_offset+1, tvb_len, '}');
+			tvb_current_offset = tvb_skip_wsp_return(tvb, tvb_current_offset)-1; /* cut last RBRKT */
 			len = tvb_current_offset - tvb_previous_offset;
 
 			if (check_col(pinfo->cinfo, COL_INFO) )
@@ -560,6 +565,10 @@ dissect_megaco_text(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 						len = tvb_len - tvb_previous_offset;
 						proto_tree_add_text(megaco_tree, tvb, tvb_previous_offset, -1,
 							"%s",tvb_format_text(tvb, tvb_previous_offset, len));
+
+						my_proto_tree_add_string(megaco_tree, hf_megaco_transid, tvb,
+										tvb_offset, (tvb_next_offset - tvb_offset),
+										tvb_format_text(tvb,tvb_offset,(tvb_next_offset - tvb_offset)));
 					if(global_megaco_raw_text){
 						tvb_raw_text_add(tvb, megaco_tree);
 						}
@@ -874,8 +883,40 @@ nextcontext:
 							}
 							break;
 
-						case 'S':
+						case 'P':
+							/*
+							PackagesToken	= ("Packages"	/ "PG")
+							PendingToken	= ("Pending"	/ "PN")
+							PriorityToken	= ("Priority"	/ "PR")
+							ProfileToken	= ("Profile"	/ "PF")
+							*/
+							tempchar = tvb_get_guint8(tvb, tvb_command_start_offset+1);
 
+							switch ( tempchar ){
+							case 'G':
+								my_proto_tree_add_string(megaco_tree_command_line, hf_megaco_command, tvb,
+									tvb_command_start_offset, tokenlen,
+									"Packages");
+								break;
+							case 'N':
+								my_proto_tree_add_string(megaco_tree_command_line, hf_megaco_command, tvb,
+									tvb_command_start_offset, tokenlen,
+									"Pending");
+								break;
+							case 'R':
+								my_proto_tree_add_string(megaco_tree_command_line, hf_megaco_command, tvb,
+									tvb_command_start_offset, tokenlen,
+									"Priority");
+								break;
+							case 'F':
+								my_proto_tree_add_string(megaco_tree_command_line, hf_megaco_command, tvb,
+									tvb_command_start_offset, tokenlen,
+									"Profile");
+								break;
+							}
+							break;
+
+						case 'S':
 							tempchar = tvb_get_guint8(tvb, tvb_command_start_offset+1);
 
 							switch ( tempchar ){
@@ -2784,7 +2825,7 @@ static gint tvb_skip_wsp(tvbuff_t* tvb, gint offset ){
 
 	for(counter = offset; counter < end &&
 		((tempchar = tvb_get_guint8(tvb,counter)) == ' ' ||
-		tempchar == '\t'|| tempchar == '\n');counter++);
+		tempchar == '\t' || tempchar == '\n' || tempchar == '\r'); counter++);
 	return (counter);
 }
 static gint tvb_skip_wsp_return(tvbuff_t* tvb, gint offset){
@@ -2795,7 +2836,7 @@ static gint tvb_skip_wsp_return(tvbuff_t* tvb, gint offset){
 
 	for(counter = offset; counter > end &&
 		((tempchar = tvb_get_guint8(tvb,counter)) == ' ' ||
-		tempchar == '\t'|| tempchar == '\n');counter--);
+		tempchar == '\t' || tempchar == '\n' || tempchar == '\r'); counter--);
 	counter++;
 	return (counter);
 }
