@@ -2,7 +2,7 @@
  * Routines for OPSI protocol dissection
  * Copyright 2004, Laurent Rabret (France Telecom R&D) <laurent.rabret@i.hate.spams.org>
  *
- * $Id: packet-opsi.c 13644 2005-03-07 02:44:41Z guy $
+ * $Id: packet-opsi.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,11 +36,8 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/dissectors/packet-tcp.h>
 #include <epan/prefs.h>
 
@@ -363,7 +360,7 @@ void decode_ipv4_attribute(tvbuff_t *tvb, proto_tree *tree, int* hfValue, int of
 		proto_tree_add_text(tree, tvb, offset, length, "Too short attribute!");
 		return;
 	}
-	tvb_memcpy(tvb, (guint8 *)&ip_address, offset+4, 4);
+	ip_address = tvb_get_ipv4(tvb, offset+4);
 	proto_tree_add_ipv4(tree, *hfValue, tvb, offset+4, 4, ip_address);
 }
 
@@ -790,7 +787,7 @@ proto_register_opsi(void)
 /* We activate the desegmentation / reassembly feature */	
 	opsi_module = prefs_register_protocol(proto_opsi, NULL);
   	prefs_register_bool_preference(opsi_module, "desegment_opsi_messages",
-    		"Desegment all OPSI messages spanning multiple TCP segments",
+    		"Reassemble OPSI messages spanning multiple TCP segments",
     		"Whether the OPSI dissector should desegment all messages spanning multiple TCP segments",
     		&opsi_desegment);
 }
