@@ -1,7 +1,7 @@
 /* packet-rdm.c
  * Routines for RDM packet disassembly
  *
- * $Id: packet-rdm.c 13644 2005-03-07 02:44:41Z guy $
+ * $Id: packet-rdm.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Copyright (c) 2003 by Erwin Rol <erwin@erwinrol.com>
  *
@@ -40,7 +40,7 @@
 #include <string.h>
 #include <epan/packet.h>
 #include <epan/addr_resolv.h>
-#include "etypes.h"
+#include <epan/etypes.h>
 #include <epan/strutil.h>
 
 /* Define version if we are not building ethereal statically */
@@ -111,6 +111,13 @@ dissect_rdm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
     offset += 1;
 
     mdb_size = tvb_get_guint8(tvb, offset) - 19;
+
+    if (mdb_size < 20) {
+      proto_tree_add_text(rdm_tree, tvb, offset, 1, "Invalid MDB size: %d", 
+			mdb_size + 19);
+      return;
+    }
+
     proto_tree_add_item(rdm_tree, hf_rdm_slot_count, tvb,
                         offset, 1, FALSE);
     offset += 1;
@@ -139,6 +146,7 @@ dissect_rdm(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
                         offset, 1, FALSE);
     offset += 1;
 
+    tvb_ensure_bytes_exist(tvb, offset, mdb_size);
     proto_tree_add_item(rdm_tree, hf_rdm_mdb, tvb,
                         offset, mdb_size, FALSE);
     offset += mdb_size;
