@@ -2,7 +2,7 @@
  * Routines for RLM dissection
  * Copyright 2004, Duncan Sargeant <dunc-ethereal@rcpt.to>
  *
- * $Id: packet-rlm.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: packet-rlm.c 15264 2005-08-08 18:50:39Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -55,10 +55,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <gmodule.h>
 #include <epan/packet.h>
 
