@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c 15088 2005-07-26 15:28:39Z gerald $
+ * $Id: print.c 16403 2005-11-06 22:43:25Z ulfl $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -39,6 +39,7 @@
 #include "packet-range.h"
 #include "print.h"
 #include "ps.h"
+#include "file_util.h"
 #include <epan/charsets.h>
 #include <epan/dissectors/packet-data.h>
 #include <epan/dissectors/packet-frame.h>
@@ -83,7 +84,7 @@ open_print_dest(int to_file, const char *dest)
 
 	/* Open the file or command for output */
 	if (to_file)
-		fh = fopen(dest, "w");
+		fh = eth_fopen(dest, "w");
 	else
 		fh = popen(dest, "w");
 
