@@ -1,7 +1,7 @@
 /* print.h
  * Definitions for printing packet analysis trees.
  *
- * $Id: print.h 13240 2005-02-02 10:28:51Z guy $
+ * $Id: print.h 13724 2005-03-11 20:56:31Z ulfl $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -121,4 +121,8 @@ extern void write_psml_preamble(FILE *fh);
 extern void proto_tree_write_psml(epan_dissect_t *edt, FILE *fh);
 extern void write_psml_finale(FILE *fh);
 
+extern void write_csv_preamble(FILE *fh);
+extern void proto_tree_write_csv(epan_dissect_t *edt, FILE *fh);
+extern void write_csv_finale(FILE *fh);
+
 #endif /* print.h */
