@@ -1,7 +1,7 @@
 /* proto_hier_stats.c
  * Routines for calculating statistics based on protocol.
  *
- * $Id: proto_hier_stats.c 14819 2005-06-29 11:59:42Z sahlberg $
+ * $Id: proto_hier_stats.c 16489 2005-11-12 11:05:02Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -42,11 +42,6 @@
 #define STAT_NODE_STATS(n)   ((ph_stats_node_t*)(n)->data)
 #define STAT_NODE_HFINFO(n)  (STAT_NODE_STATS(n)->hfinfo)
 
-static double
-secs_usecs(guint32 s, guint32 us)
-{
-  return (us / 1000000.0) + (double)s;
-}
 
 static GNode*
 find_stat_node(GNode *parent_stat_node, header_field_info *needle_hfinfo)
@@ -161,7 +156,7 @@ process_frame(frame_data *frame, column_info *cinfo, ph_stats_t* ps)
 	process_tree(edt->tree, ps, frame->pkt_len);
 
 	/* Update times */
-	cur_time = secs_usecs(frame->abs_secs, frame->abs_usecs);
+	cur_time = nstime_to_sec(&frame->abs_ts);
 	if (cur_time < ps->first_time) {
 	  ps->first_time = cur_time;
 	}
@@ -184,7 +179,7 @@ ph_stats_new(void)
 	progdlg_t	*progbar = NULL;
 	gboolean	stop_flag;
 	int		count;
-	float		prog_val;
+	float		progbar_val;
 	GTimeVal	start_time;
 	gchar		status_str[100];
 	int		progbar_nextstep;
@@ -205,6 +200,8 @@ ph_stats_new(void)
 	progbar_quantum = cfile.count/N_PROGBAR_UPDATES;
 	/* Count of packets at which we've looked. */
 	count = 0;
+	/* Progress so far. */
+	progbar_val = 0.0;
 
 	stop_flag = FALSE;
 	g_get_current_time(&start_time);
@@ -213,6 +210,17 @@ ph_stats_new(void)
 	tot_bytes = 0;
 
 	for (frame = cfile.plist; frame != NULL; frame = frame->next) {
+		/* Create the progress bar if necessary.
+		   We check on every iteration of the loop, so that
+		   it takes no longer than the standard time to create
+		   it (otherwise, for a large file, we might take
+		   considerably longer than that standard time in order
+		   to get to the next progress bar step). */
+		if (progbar == NULL)
+			progbar = delayed_create_progress_dlg(
+			    "Computing", "protocol hierarchy statistics", 
+			    TRUE, &stop_flag, &start_time, progbar_val);
+
 		/* Update the progress bar, but do it only N_PROGBAR_UPDATES
 		   times; when we update it, we have to run the GTK+ main
 		   loop to get it to repaint what's pending, and doing so
@@ -225,18 +233,12 @@ ph_stats_new(void)
 			 */
 			g_assert(cfile.count > 0);
 
-			prog_val = (gfloat) count / cfile.count;
-
-			if (progbar == NULL)
-				/* Create the progress bar if necessary */
-				progbar = delayed_create_progress_dlg(
-				    "Computing", "protocol hierarchy statistics", 
-				    &stop_flag, &start_time, prog_val);
+			progbar_val = (gfloat) count / cfile.count;
 
 			if (progbar != NULL) {
 				g_snprintf(status_str, sizeof(status_str),
 					"%4u of %u frames", count, cfile.count);
-				update_progress_dlg(progbar, prog_val, status_str);
+				update_progress_dlg(progbar, progbar_val, status_str);
 			}
 
 			progbar_nextstep += progbar_quantum;
@@ -256,8 +258,7 @@ ph_stats_new(void)
 		if (frame->flags.passed_dfilter) {
 
 			if (tot_packets == 0) {
-				double cur_time = secs_usecs(frame->abs_secs,
-							     frame->abs_usecs);
+				double cur_time = nstime_to_sec(&frame->abs_ts);
 				ps->first_time = cur_time;
 				ps->last_time = cur_time;
 			}
