@@ -1,7 +1,7 @@
 /* ringbuffer.c
  * Routines for packet capture windows
  *
- * $Id: ringbuffer.c 13266 2005-02-03 21:58:33Z ulfl $
+ * $Id: ringbuffer.c 17461 2006-03-04 22:33:04Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,18 +48,10 @@
 
 #ifdef HAVE_LIBPCAP
 
-#ifdef HAVE_IO_H
-#include <io.h>
-#endif
-
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -69,17 +61,14 @@
 #include <time.h>
 #include <errno.h>
 
-#ifdef NEED_SNPRINTF_H
-#include "snprintf.h"
-#endif
+#include <pcap.h>
 
-#include "wiretap/wtap.h"
+#include <glib.h>
+
+#include "pcapio.h"
 #include "ringbuffer.h"
+#include "file_util.h"
 
-/* Win32 needs the O_BINARY flag for open() */
-#ifndef O_BINARY
-#define O_BINARY	0
-#endif
 
 /* Ringbuffer file structure */
 typedef struct _rb_file {
@@ -89,17 +78,17 @@ typedef struct _rb_file {
 /* Ringbuffer data structure */
 typedef struct _ringbuf_data {
   rb_file      *files;
-  guint         num_files;           /* Number of ringbuffer files */
-  guint         curr_file_num;       /* Number of the current file */
+  guint         num_files;           /* Number of ringbuffer files (1 to ...) */
+  guint         curr_file_num;       /* Number of the current file (ever increasing) */
   gchar        *fprefix;             /* Filename prefix */
   gchar        *fsuffix;             /* Filename suffix */
   gboolean      unlimited;           /* TRUE if unlimited number of files */
-  int           filetype;
   int           linktype;
   int           snaplen;
 
   int           fd;		     /* Current ringbuffer file descriptor */
-  wtap_dumper  *pdh;  
+  FILE         *pdh;
+  long          bytes_written;	     /* Bytes written to the current file */
 } ringbuf_data;
 
 static ringbuf_data rb_data;
@@ -117,7 +106,7 @@ static int ringbuf_open_file(rb_file *rfile, int *err)
   if (rfile->name != NULL) {
     if (rb_data.unlimited == FALSE) {
       /* remove old file (if any, so ignore error) */
-      unlink(rfile->name);
+      eth_unlink(rfile->name);
     }
     g_free(rfile->name);
   }
@@ -127,7 +116,7 @@ static int ringbuf_open_file(rb_file *rfile, int *err)
 #endif
   current_time = time(NULL);
 
-  snprintf(filenum, sizeof(filenum), "%05d", rb_data.curr_file_num + 1 /*.number*/);
+  g_snprintf(filenum, sizeof(filenum), "%05d", rb_data.curr_file_num + 1 /*.number*/);
   strftime(timestr, sizeof(timestr), "%Y%m%d%H%M%S", localtime(&current_time));
   rfile->name = g_strconcat(rb_data.fprefix, "_", filenum, "_", timestr,
 			    rb_data.fsuffix, NULL);
@@ -137,7 +126,7 @@ static int ringbuf_open_file(rb_file *rfile, int *err)
     return -1;
   }
 
-  rb_data.fd = open(rfile->name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT, 0600);
+  rb_data.fd = eth_open(rfile->name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT, 0600);
 
   if (rb_data.fd == -1 && err != NULL) {
     *err = errno;
@@ -232,19 +221,24 @@ ringbuf_init(const char *capfile_name, guint num_files)
   return rb_data.fd;
 }
 
+
+const gchar *ringbuf_current_filename(void)
+{
+  return rb_data.files[rb_data.curr_file_num % rb_data.num_files].name;
+}
+
 /*
- * Calls wtap_dump_fdopen() for the current ringbuffer file
+ * Calls libpcap_fdopen() for the current ringbuffer file
  */
-wtap_dumper*
-ringbuf_init_wtap_dump_fdopen(int filetype, int linktype, int snaplen, int *err)
+FILE *
+ringbuf_init_libpcap_fdopen(int linktype, int snaplen, int *err)
 {
 
-  rb_data.filetype = filetype;
   rb_data.linktype = linktype;
   rb_data.snaplen  = snaplen;
 
-  rb_data.pdh = wtap_dump_fdopen(rb_data.fd, filetype, linktype, snaplen, err);
-
+  rb_data.pdh = libpcap_fdopen(rb_data.fd, linktype, snaplen,
+                               &rb_data.bytes_written, err);
   return rb_data.pdh;
 }
 
@@ -252,15 +246,15 @@ ringbuf_init_wtap_dump_fdopen(int filetype, int linktype, int snaplen, int *err)
  * Switches to the next ringbuffer file
  */
 gboolean
-ringbuf_switch_file(wtap_dumper **pdh, gchar **save_file, int *save_file_fd, int *err)
+ringbuf_switch_file(FILE **pdh, gchar **save_file, int *save_file_fd, int *err)
 {
   int     next_file_num;
   rb_file *next_rfile = NULL;
 
   /* close current file */
 
-  if (!wtap_dump_close(rb_data.pdh, err)) {
-    close(rb_data.fd);	/* XXX - the above should have closed this already */
+  if (!libpcap_dump_close(rb_data.pdh, err)) {
+    eth_close(rb_data.fd);	/* XXX - the above should have closed this already */
     rb_data.pdh = NULL;	/* it's still closed, we just got an error while closing */
     rb_data.fd = -1;
     return FALSE;
@@ -271,20 +265,20 @@ ringbuf_switch_file(wtap_dumper **pdh, gchar **save_file, int *save_file_fd, int
 
   /* get the next file number and open it */
 
-  next_file_num = (rb_data.curr_file_num + 1) % rb_data.num_files;  
+  rb_data.curr_file_num++ /* = next_file_num*/;
+  next_file_num = (rb_data.curr_file_num) % rb_data.num_files;  
   next_rfile = &rb_data.files[next_file_num];
 
   if (ringbuf_open_file(next_rfile, err) == -1) {
     return FALSE;
   }
 
-  if (ringbuf_init_wtap_dump_fdopen(rb_data.filetype, rb_data.linktype,
-				    rb_data.snaplen, err) == NULL) {
+  if (ringbuf_init_libpcap_fdopen(rb_data.linktype, rb_data.snaplen,
+                                  err) == NULL) {
     return FALSE;
   }
 
   /* switch to the new file */
-  rb_data.curr_file_num = next_file_num;
   *save_file = next_rfile->name;
   *save_file_fd = rb_data.fd;
   (*pdh) = rb_data.pdh;
@@ -293,17 +287,17 @@ ringbuf_switch_file(wtap_dumper **pdh, gchar **save_file, int *save_file_fd, int
 }
 
 /*
- * Calls wtap_dump_close() for the current ringbuffer file
+ * Calls libpcap_dump_close() for the current ringbuffer file
  */
 gboolean
-ringbuf_wtap_dump_close(gchar **save_file, int *err)
+ringbuf_libpcap_dump_close(gchar **save_file, int *err)
 {
   gboolean  ret_val = TRUE;
 
   /* close current file, if it's open */
   if (rb_data.pdh != NULL) {
-    if (!wtap_dump_close(rb_data.pdh, err)) {
-      close(rb_data.fd);
+    if (!libpcap_dump_close(rb_data.pdh, err)) {
+      eth_close(rb_data.fd);
       ret_val = FALSE;
     }
 
@@ -312,7 +306,7 @@ ringbuf_wtap_dump_close(gchar **save_file, int *err)
   }
 
   /* set the save file name to the current file */
-  *save_file = rb_data.files[rb_data.curr_file_num].name;
+  *save_file = rb_data.files[rb_data.curr_file_num % rb_data.num_files].name;
   return ret_val;
 }
 
@@ -354,24 +348,24 @@ ringbuf_error_cleanup(void)
 
   /* try to close via wtap */
   if (rb_data.pdh != NULL) {
-    if (wtap_dump_close(rb_data.pdh, NULL)) {
+    if (libpcap_dump_close(rb_data.pdh, NULL)) {
       rb_data.fd = -1;
     }
     rb_data.pdh = NULL;
   }
 
   /* close directly if still open */
-  /* XXX - it shouldn't still be open; "wtap_dump_close()" should leave the
+  /* XXX - it shouldn't still be open; "libpcap_dump_close()" should leave the
      file closed even if it fails */
   if (rb_data.fd != -1) {
-    close(rb_data.fd);
+    eth_close(rb_data.fd);
     rb_data.fd = -1;
   }
 
   if (rb_data.files != NULL) {
     for (i=0; i < rb_data.num_files; i++) {
       if (rb_data.files[i].name != NULL) {
-        unlink(rb_data.files[i].name);
+        eth_unlink(rb_data.files[i].name);
       }
     }
   }
