@@ -1,7 +1,7 @@
 /* summary.c
  * Routines for capture file summary info
  *
- * $Id: summary.c,v 1.23 2003/09/02 22:10:32 guy Exp $
+ * $Id: summary.c 15520 2005-08-24 21:31:56Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,22 +27,19 @@
 #endif
 
 #include <epan/packet.h>
-#include "globals.h"
+#include "cfile.h"
 #include "summary.h"
+#ifdef HAVE_LIBPCAP
+#include "capture_ui_utils.h"
+#endif
 
 
-static double
-secs_usecs( guint32 s, guint32 us)
-{
-  return (us / 1000000.0) + (double)s;
-}
-
 static void
 tally_frame_data(frame_data *cur_frame, summary_tally *sum_tally)
 {
   double cur_time;
 
-  cur_time = secs_usecs(cur_frame->abs_secs, cur_frame->abs_usecs);
+  cur_time = nstime_to_sec(&cur_frame->abs_ts);
 
   if (cur_time < sum_tally->start_time) {
     sum_tally->start_time = cur_time;
@@ -72,7 +69,7 @@ tally_frame_data(frame_data *cur_frame, summary_tally *sum_tally)
 }
 
 void
-summary_fill_in(summary_tally *st)
+summary_fill_in(capture_file *cf, summary_tally *st)
 {
 
   frame_data    *first_frame, *cur_frame;
@@ -89,34 +86,47 @@ summary_fill_in(summary_tally *st)
   st->marked_count = 0;
 
   /* initialize the tally */
-  if (cfile.plist != NULL) {
-    first_frame = cfile.plist;
-    st->start_time 	= secs_usecs(first_frame->abs_secs,first_frame->abs_usecs);
-    st->stop_time = secs_usecs(first_frame->abs_secs,first_frame->abs_usecs);
-    cur_glist = cfile.plist;
+  if (cf->plist != NULL) {
+    first_frame = cf->plist;
+    st->start_time 	= nstime_to_sec(&first_frame->abs_ts);
+    st->stop_time = nstime_to_sec(&first_frame->abs_ts);
+    cur_glist = cf->plist;
 
-    for (i = 0; i < cfile.count; i++) {
+    for (i = 0; i < cf->count; i++) {
       cur_frame = cur_glist;
       tally_frame_data(cur_frame, st);
       cur_glist = cur_glist->next;
     }
   }
 
-  st->filename = cfile.filename;
-  st->file_length = cfile.f_len;
-  st->encap_type = cfile.cd_t;
-  st->has_snap = cfile.has_snap;
-  st->snap = cfile.snap;
-  st->elapsed_time = secs_usecs(cfile.esec, cfile.eusec);
-  st->packet_count = cfile.count;
-  st->drops_known = cfile.drops_known;
-  st->drops = cfile.drops;
-  st->iface = cfile.iface;
-  st->dfilter = cfile.dfilter;
+  st->filename = cf->filename;
+  st->file_length = cf->f_datalen;
+  st->encap_type = cf->cd_t;
+  st->has_snap = cf->has_snap;
+  st->snap = cf->snap;
+  st->elapsed_time = nstime_to_sec(&cf->elapsed_time);
+  st->packet_count = cf->count;
+  st->drops_known = cf->drops_known;
+  st->drops = cf->drops;
+  st->dfilter = cf->dfilter;
+
+  /* capture related */
+  st->cfilter = NULL;
+  st->iface = NULL;
+  st->iface_descr = NULL;
+}
+
 
 #ifdef HAVE_LIBPCAP
-  st->cfilter = cfile.cfilter;
-#else
-  st->cfilter = NULL;
-#endif
+void
+summary_fill_in_capture(capture_options *capture_opts, summary_tally *st)
+{
+  st->cfilter = capture_opts->cfilter;
+  st->iface = capture_opts->iface;
+  if(st->iface) {
+    st->iface_descr = get_interface_descriptive_name(st->iface);
+  } else {
+    st->iface_descr = NULL;
+  }
 }
+#endif
