@@ -3,7 +3,7 @@
  * Copyright 2003, Michael Lum <mlum [AT] telostech.com>
  * In association with Telos Technology Inc.
  *
- * $Id: tap-ansi_astat.c 13389 2005-02-13 00:10:15Z lroland $
+ * $Id: tap-ansi_astat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -42,6 +42,7 @@
 #include "epan/packet_info.h"
 #include "epan/value_string.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include <epan/dissectors/packet-bssap.h>
 #include <epan/dissectors/packet-ansi_a.h>
 #include "register.h"
@@ -134,14 +135,11 @@ ansi_a_stat_draw(
 
 
 static void
-ansi_a_stat_init(char *optarg)
+ansi_a_stat_init(const char *optarg _U_, void* userdata _U_)
 {
     ansi_a_stat_t	*stat_p;
     GString		*err_p;
 
-
-    optarg = optarg;
-
     stat_p = g_malloc(sizeof(ansi_a_stat_t));
 
     memset(stat_p, 0, sizeof(ansi_a_stat_t));
@@ -165,5 +163,5 @@ ansi_a_stat_init(char *optarg)
 void
 register_tap_listener_ansi_astat(void)
 {
-    register_ethereal_tap("ansi_a,", ansi_a_stat_init);
+    register_stat_cmd_arg("ansi_a,", ansi_a_stat_init,NULL);
 }
