@@ -1,7 +1,7 @@
 /* tap-bootpstat.c
  * boop_stat   2003 Jean-Michel FAYARD
  *
- * $Id: tap-bootpstat.c 13993 2005-04-01 09:08:27Z guy $
+ * $Id: tap-bootpstat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,11 +27,12 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 
 #include "epan/packet_info.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "register.h"
-#include <string.h>
 
 
 typedef const char* bootp_info_value_t;
@@ -136,10 +137,10 @@ dhcpstat_draw(void *psp)
 /* When called, this function will create a new instance of tap-boopstat.
  */
 static void
-dhcpstat_init(char *optarg)
+dhcpstat_init(const char *optarg, void* userdata _U_)
 {
 	dhcpstat_t *sp;
-	char 		*filter=NULL;
+	const char	*filter=NULL;
 	GString		*error_string;
 	
 	if (!strncmp (optarg, "bootp,stat,", 11)){
@@ -181,6 +182,6 @@ dhcpstat_init(char *optarg)
 void
 register_tap_listener_gtkdhcpstat(void)
 {
-	register_tap_listener_cmd_arg("bootp,stat,", dhcpstat_init);
+	register_stat_cmd_arg("bootp,stat,", dhcpstat_init,NULL);
 }
 
