@@ -2,7 +2,7 @@
  * h225 RAS Service Response Time statistics for ethereal
  * Copyright 2003 Lars Roland
  *
- * $Id: tap-h225rassrt.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: tap-h225rassrt.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,6 +36,7 @@
 #include <string.h>
 #include "epan/packet_info.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "epan/value_string.h"
 #include "register.h"
 #include <epan/dissectors/packet-h225.h>
@@ -211,17 +212,16 @@ h225rassrt_draw(void *phs)
 
 
 static void
-h225rassrt_init(char *optarg)
+h225rassrt_init(const char *optarg, void* userdata _U_)
 {
 	h225rassrt_t *hs;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString *error_string;
 
 	if(!strncmp(optarg,"h225,srt,",9)){
 		filter=optarg+9;
 	} else {
-		filter=g_malloc(1);
-		*filter='\0';
+		filter="";
 	}
 
 	hs = g_malloc(sizeof(h225rassrt_t));
@@ -247,5 +247,5 @@ h225rassrt_init(char *optarg)
 void
 register_tap_listener_h225rassrt(void)
 {
-	register_ethereal_tap("h225,srt", h225rassrt_init);
+	register_stat_cmd_arg("h225,srt", h225rassrt_init,NULL);
 }
