@@ -1,7 +1,7 @@
 /* tap-httpstat.c
  * tap-httpstat   2003 Jean-Michel FAYARD
  *
- * $Id: tap-httpstat.c 14739 2005-06-24 01:32:42Z guy $
+ * $Id: tap-httpstat.c 17421 2006-02-27 14:03:48Z jake $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,6 +32,7 @@
 #include "epan/packet_info.h"
 #include "epan/value_string.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "register.h"
 #include <epan/dissectors/packet-http.h>
 
@@ -145,7 +146,7 @@ static void
 http_draw_hash_responses( gint * key _U_ , http_response_code_t *data, char * format)
 {
 	if (data==NULL) {
-		g_warning("C'est quoi ce borderl key=%d\n", *key);
+		g_warning("No data available, key=%d\n", *key);
 		exit(EXIT_FAILURE);
 	}
 	if (data->packets==0)
@@ -182,7 +183,7 @@ httpstat_reset(void *psp  )
 	httpstat_t *sp=psp;
 	if (!sp) {
 		g_hash_table_foreach( sp->hash_responses, (GHFunc)http_reset_hash_responses, NULL);
-		g_hash_table_foreach( sp->hash_responses, (GHFunc)http_reset_hash_requests, NULL);
+		g_hash_table_foreach( sp->hash_requests, (GHFunc)http_reset_hash_requests, NULL);
 	}
 }
 
@@ -280,10 +281,10 @@ httpstat_draw(void *psp  )
 /* When called, this function will create a new instance of gtk_httpstat.
  */
 static void
-gtk_httpstat_init(char *optarg)
+gtk_httpstat_init(const char *optarg,void* userdata _U_)
 {
 	httpstat_t *sp;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString	*error_string;
 	
 	if (!strncmp (optarg, "http,stat,", 10)){
@@ -325,5 +326,5 @@ gtk_httpstat_init(char *optarg)
 void
 register_tap_listener_gtkhttpstat(void)
 {
-	register_tap_listener_cmd_arg("http,stat,", gtk_httpstat_init);
+	register_stat_cmd_arg("http,stat,", gtk_httpstat_init,NULL);
 }
