@@ -1,7 +1,7 @@
 /* tap-protocolinfo.c
  * protohierstat   2002 Ronnie Sahlberg
  *
- * $Id: tap-protocolinfo.c 13993 2005-04-01 09:08:27Z guy $
+ * $Id: tap-protocolinfo.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,6 +39,7 @@
 #include "epan/column-utils.h"
 #include "epan/proto.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "register.h"
 
 typedef struct _pci_t {
@@ -78,7 +79,6 @@ protocolinfo_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt, const vo
 		str=proto_construct_dfilter_string(gp->pdata[i], NULL);
 		if(str){
 			col_append_fstr(pinfo->cinfo, COL_INFO, "  %s",str);
-			g_free(str);
 		}
 	}
 	return 0;
@@ -87,11 +87,11 @@ protocolinfo_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt, const vo
 
 
 static void
-protocolinfo_init(char *optarg)
+protocolinfo_init(const char *optarg, void* userdata _U_)
 {
 	pci_t *rs;
-	char *field=NULL;
-	char *filter=NULL;
+	const char *field=NULL;
+	const char *filter=NULL;
 	header_field_info *hfi;
 	GString *error_string;
 
@@ -142,6 +142,6 @@ protocolinfo_init(char *optarg)
 void
 register_tap_listener_protocolinfo(void)
 {
-	register_tap_listener_cmd_arg("proto,colinfo,", protocolinfo_init);
+	register_stat_cmd_arg("proto,colinfo,", protocolinfo_init,NULL);
 }
 
