@@ -1,7 +1,7 @@
 /* tap-protohierstat.c
  * protohierstat   2002 Ronnie Sahlberg
  *
- * $Id: tap-protohierstat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: tap-protohierstat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,6 +39,7 @@
 #include "epan/epan_dissect.h"
 #include "epan/proto.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "register.h"
 
 typedef struct _phs_t {
@@ -47,7 +48,7 @@ typedef struct _phs_t {
 	struct _phs_t *parent;
 	char *filter;
 	int protocol;
-	char *proto_name;
+	const char *proto_name;
 	guint32 frames;
 	guint32 bytes;
 } phs_t;
@@ -135,17 +136,23 @@ protohierstat_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt, const v
 static void
 phs_draw(phs_t *rs, int indentation)
 {
-	int i;
-	char str[80];
+	int i, stroff;
+#define MAXPHSLINE 80
+	char str[MAXPHSLINE];
 	for(;rs;rs=rs->sibling){
 		if(rs->protocol==-1){
 			return;
 		}
 		str[0]=0;
+		stroff=0;
 		for(i=0;i<indentation;i++){
-			strcat(str,"  ");
+			if(i>15){
+				stroff+=g_snprintf(str+stroff, MAXPHSLINE-stroff, "...");
+				break;
+			}
+			stroff+=g_snprintf(str+stroff, MAXPHSLINE-stroff, "  ");
 		}
-		strcat(str, rs->proto_name);
+		stroff+=g_snprintf(str+stroff, MAXPHSLINE-stroff, rs->proto_name);
 		printf("%-40s frames:%d bytes:%d\n",str, rs->frames, rs->bytes);
 		phs_draw(rs->child, indentation+1);
 	}
@@ -166,11 +173,11 @@ protohierstat_draw(void *prs)
 
 
 static void
-protohierstat_init(char *optarg)
+protohierstat_init(const char *optarg, void* userdata _U_)
 {
 	phs_t *rs;
 	int pos=0;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString *error_string;
 
 	if(!strcmp("io,phs",optarg)){
@@ -216,6 +223,6 @@ protohierstat_init(char *optarg)
 void
 register_tap_listener_protohierstat(void)
 {
-	register_ethereal_tap("io,phs", protohierstat_init);
+	register_stat_cmd_arg("io,phs", protohierstat_init, NULL);
 }
 
