@@ -1,7 +1,7 @@
 /* tap_sipstat.c
  * sip message counter for ethereal
  *
- * $Id: tap-sipstat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: tap-sipstat.c 17409 2006-02-25 09:28:52Z etxrab $
  * Copied from gtk/sip_stat.c and tap-httpstat.c
  *
  * Ethereal - Network traffic analyzer
@@ -36,6 +36,7 @@
 #include <string.h>
 #include "epan/packet_info.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "epan/value_string.h"
 #include "register.h"
 #include <epan/dissectors/packet-sip.h>
@@ -55,7 +56,7 @@ typedef struct _sip_stats_t {
 typedef struct _sip_response_code_t {
 	guint32 	 packets;		/* 3 */
 	guint	 	 response_code;		/* 404 */
-	gchar		*name;			/* Not Found */
+	const gchar	*name;			/* Not Found */
 	sipstat_t	*sp;
 } sip_response_code_t;
 
@@ -96,6 +97,7 @@ static const value_string vals_status_code[] = {
     { 407, "Proxy Authentication Required"},
     { 408, "Request Timeout"},
     { 410, "Gone"},
+    { 412, "Conditional Request Failed"},
     { 413, "Request Entity Too Large"},
     { 414, "Request-URI Too Long"},
     { 415, "Unsupported Media Type"},
@@ -344,10 +346,10 @@ sipstat_draw(void *psp  )
 }
 
 static void
-sipstat_init(char *optarg)
+sipstat_init(const char *optarg, void* userdata _U_)
 {
 	sipstat_t *sp;
-	char *filter=NULL;
+	const char *filter=NULL;
 	GString	*error_string;
 
 	if (strncmp (optarg, "sip,stat,", 9) == 0){
@@ -376,7 +378,7 @@ sipstat_init(char *optarg)
 		/* error, we failed to attach to the tap. clean up */
 		g_free(sp->filter);
 		g_free(sp);
-		fprintf (stderr, "tethereal: Coulnd't register sip,stat tap: %s\n",
+		fprintf (stderr, "tethereal: Couldn't register sip,stat tap: %s\n",
 				error_string->str);
 		g_string_free(error_string, TRUE);
 		exit(1);
@@ -390,5 +392,5 @@ sipstat_init(char *optarg)
 void
 register_tap_listener_sipstat(void)
 {
-	register_ethereal_tap("sip,stat", sipstat_init);
+	register_stat_cmd_arg("sip,stat", sipstat_init,NULL);
 }
