@@ -1,7 +1,7 @@
 /* tap-smbsids.c
  * smbstat   2003 Ronnie Sahlberg
  *
- * $Id: tap-smbsids.c 13389 2005-02-13 00:10:15Z lroland $
+ * $Id: tap-smbsids.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,8 +37,9 @@
 #include <epan/dissectors/packet-smb-sidsnooping.h>
 #include "register.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "epan/value_string.h"
-#include "smb.h"
+#include <epan/dissectors/packet-smb.h>
 
 
 static int
@@ -68,7 +69,7 @@ smbsids_draw(void *pss _U_)
 
 
 static void
-smbsids_init(char *optarg _U_)
+smbsids_init(const char *optarg _U_, void* userdata _U_)
 {
 	GString *error_string;
 
@@ -95,6 +96,6 @@ smbsids_init(char *optarg _U_)
 void
 register_tap_listener_smbsids(void)
 {
-	register_ethereal_tap("smb,sids", smbsids_init);
+	register_stat_cmd_arg("smb,sids", smbsids_init,NULL);
 }
 
