@@ -2,7 +2,7 @@
  * tethereal's tap implememntation of stats_tree
  * 2005, Luis E. G. Ontanon
  *
- * $Id: $
+ * $Id: tap-stats_tree.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,6 +31,8 @@
 #include <stdio.h>
 #include <glib.h>
 #include <epan/stats_tree_priv.h>
+#include <epan/stat_cmd_args.h>
+#include <epan/report_err.h>
 
 /* actually unused */
 struct _st_node_pres {
@@ -67,7 +69,7 @@ static void draw_stats_tree(void *psp) {
 	
 }
 
-static void  init_stats_tree(char *optarg) {
+static void  init_stats_tree(const char *optarg,void* userdata _U_) {
 	guint8* abbr = stats_tree_get_abbr(optarg);
 	GString	*error_string;
 	stats_tree_cfg *cfg = NULL;
@@ -83,13 +85,15 @@ static void  init_stats_tree(char *optarg) {
 				st->filter=NULL;
 			}
 		} else {
-			g_error("no such stats_tree (%s) found in stats_tree registry",abbr);
+			report_failure("no such stats_tree (%s) found in stats_tree registry",abbr);
+			return;
 		}
 		
 		g_free(abbr);
 		
 	} else {
-		g_error("could not obtain stats_tree abbr (%s) from optarg '%s'",abbr,optarg);		
+		report_failure("could not obtain stats_tree abbr (%s) from grg '%s'",abbr,optarg);		
+		return;
 	}
 	
 	error_string = register_tap_listener( st->cfg->tapname,
@@ -100,7 +104,8 @@ static void  init_stats_tree(char *optarg) {
 										  draw_stats_tree);
 	
 	if (error_string) {
-		g_error("stats_tree for: %s failed to attach to the tap: %s",cfg->name,error_string->str);
+		report_failure("stats_tree for: %s failed to attach to the tap: %s",cfg->name,error_string->str);
+		return;
 	}
 
 	if (cfg->init) cfg->init(st);
@@ -113,7 +118,7 @@ void register_stats_tree_tap (gpointer k _U_, gpointer v, gpointer p _U_) {
 	cfg->pr = g_malloc(sizeof(tree_cfg_pres));
 	cfg->pr->init_string = g_strdup_printf("%s,tree",cfg->abbr);
 
-	register_tap_listener_cmd_arg(cfg->pr->init_string, init_stats_tree);
+	register_stat_cmd_arg(cfg->pr->init_string, init_stats_tree, NULL);
 	
 }
 
