@@ -6,7 +6,7 @@
  *
  * (c) Copyright 2001 Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: text2pcap.c,v 1.29 2004/01/06 02:58:20 guy Exp $
+ * $Id: text2pcap.c 17431 2006-03-01 00:11:16Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -176,10 +176,10 @@ static guint32 ts_usec = 0;
 static char *ts_fmt = NULL;
 
 /* Input file */
-static char *input_filename;
+static const char *input_filename;
 static FILE *input_file = NULL;
 /* Output file */
-static char *output_filename;
+static const char *output_filename;
 static FILE *output_file = NULL;
 
 /* Offset base to parse */
@@ -330,7 +330,7 @@ static unsigned long pcap_link_type = 1;   /* Default is DLT-EN10MB */
  * Pass in TRUE if this is an offset, FALSE if not
  */
 static unsigned long
-parse_num (char *str, int offset)
+parse_num (const char *str, int offset)
 {
     unsigned long num;
     char *c;
@@ -347,7 +347,7 @@ parse_num (char *str, int offset)
  * Write this byte into current packet
  */
 static void
-write_byte (char *str)
+write_byte (const char *str)
 {
     unsigned long num;
 
@@ -982,62 +982,73 @@ parse_token (token_t token, char *str)
 }
 
 /*----------------------------------------------------------------------
- * Print helpstring and exit
+ * Print usage string and exit
  */
 static void
-help (char *progname)
+usage (void)
 {
     fprintf(stderr,
+            "Text2pcap %s"
+#ifdef SVNVERSION
+            " (" SVNVERSION ")"
+#endif
             "\n"
-            "Usage: %s [-h] [-d] [-q] [-o h|o] [-l typenum] [-e l3pid] [-i proto] \n"
-            "          [-m max-packet] [-u srcp,destp] [-T srcp,destp] [-s srcp,destp,tag]\n"
-            "          [-S srcp,destp,tag] [-t timefmt] <input-filename> <output-filename>\n"
+            "Generate a capture file from an ASCII hexdump of packets.\n"
+            "See http://www.ethereal.com for more information.\n"
             "\n"
-            "where <input-filename> specifies input filename (use - for standard input)\n"
+            "Usage: text2pcap [options] <input-filename> <output-filename>\n"
+            "\n"
+            "where  <input-filename> specifies input  filename (use - for standard input)\n"
             "      <output-filename> specifies output filename (use - for standard output)\n"
             "\n"
-            "[options] are one or more of the following \n"
+            "Input:\n"
+            "  -o hex|oct             parse offsets as (h)ex or (o)ctal, default is hex\n"
+            "  -t <timefmt>           treats the text before the packet as a date/time code;\n"
+            "                         the specified argument is a format string of the sort \n"
+            "                         supported by strptime.\n"
+            "                         Example: The time \"10:15:14.5476\" has the format code\n"
+            "                         \"%%H:%%M:%%S.\"\n"
+            "                         NOTE: The subsecond component delimiter must be given\n"
+            "                          (.) but no pattern is required; the remaining number\n"
+            "                          is assumed to be fractions of a second.\n"
             "\n"
-            " -h              : Display this help message \n"
-            " -d              : Generate detailed debug of parser states \n"
-            " -o hex|oct      : Parse offsets as (h)ex or (o)ctal. Default is hex\n"
-            " -l typenum      : Specify link-layer type number. Default is 1 (Ethernet). \n"
-            "                   See net/bpf.h for list of numbers.\n"
-            " -q              : Generate no output at all (automatically turns off -d)\n"
-            " -e l3pid        : Prepend dummy Ethernet II header with specified L3PID (in\n"
-            "                   HEX)\n"
-            "                   Example: -e 0x800\n"
-            " -i proto        : Prepend dummy IP header with specified IP protocol (in\n"
-            "                   DECIMAL). \n"
-            "                   Automatically prepends Ethernet header as well.\n"
-            "                   Example: -i 46\n"
-            " -m max-packet   : Max packet length in output, default is %d\n"
-            " -u srcp,destp   : Prepend dummy UDP header with specified dest and source ports\n"
-            "                   (in DECIMAL).\n"
-            "                   Automatically prepends Ethernet and IP headers as well\n"
-            "                   Example: -u 30,40\n"
-            " -T srcp,destp   : Prepend dummy TCP header with specified dest and source ports\n"
-            "                   (in DECIMAL).\n"
-            "                   Automatically prepends Ethernet and IP headers as well\n"
-            "                   Example: -T 50,60\n"
-            " -s srcp,dstp,tag: Prepend dummy SCTP header with specified dest/source ports\n"
-            "                   and verification tag (in DECIMAL).\n"
-            "                   Automatically prepends Ethernet and IP headers as well\n"
-            "                   Example: -s 30,40,34\n"
-            " -S srcp,dstp,ppi: Prepend dummy SCTP header with specified dest/source ports\n"
-            "                   and verification tag 0. It also prepends a dummy SCTP DATA\n"
-            "                   chunk header with payload protocol identifier ppi.\n"
-            "                   Example: -S 30,40,34\n"
-            " -t timefmt      : Treats the text before the packet as a date/time code; the\n"
-            "                   specified argument is a format string of the sort supported\n"
-            "                   by strptime.\n"
-            "                   Example: The time \"10:15:14.5476\" has the format code\n"
-            "                   \"%%H:%%M:%%S.\"\n"
-            "                   NOTE:    The subsecond component delimiter must be specified\n"
-            "                            (.) but no pattern is required; the remaining number\n"
-            "                            is assumed to be fractions of a second.\n"
+            "Output:\n"
+            "  -l <typenum>           link-layer type number. Default is 1 (Ethernet). \n"
+            "                         See the file net/bpf.h for list of numbers.\n"
+            "  -m <max-packet>        max packet length in output, default is %d\n"
+            "\n"
+            "Prepend dummy header:\n"
+            "  -e <l3pid>             prepend dummy Ethernet II header with specified L3PID\n"
+            "                         (in HEX)\n"
+            "                         Example: -e 0x800\n"
+            "  -i <proto>             prepend dummy IP header with specified IP protocol\n"
+            "                         (in DECIMAL). \n"
+            "                         Automatically prepends Ethernet header as well.\n"
+            "                         Example: -i 46\n"
+            "  -u <srcp>,<destp>      prepend dummy UDP header with specified\n"
+            "                         dest and source ports (in DECIMAL).\n"
+            "                         Automatically prepends Ethernet & IP headers as well\n"
+            "                         Example: -u 30,40\n"
+            "  -T <srcp>,<destp>      prepend dummy TCP header with specified \n"
+            "                         dest and source ports (in DECIMAL).\n"
+            "                         Automatically prepends Ethernet & IP headers as well\n"
+            "                         Example: -T 50,60\n"
+            "  -s <srcp>,<dstp>,<tag> prepend dummy SCTP header with specified \n"
+            "                         dest/source ports and verification tag (in DECIMAL).\n"
+            "                         Automatically prepends Ethernet & IP headers as well\n"
+            "                         Example: -s 30,40,34\n"
+            "  -S <srcp>,<dstp>,<ppi> prepend dummy SCTP header with specified \n"
+            "                         dest/source ports and verification tag 0. \n"
+            "                         It also prepends a dummy SCTP DATA \n"
+            "                         chunk header with payload protocol identifier ppi.\n"
+            "                         Example: -S 30,40,34\n"
+            "\n"
+            "Miscellaneous:\n"
+            "  -h                     display this help and exit\n"
+            "  -d                     detailed debug of parser states \n"
+            "  -q                     generate no output at all (automatically turns off -d)\n"
             "",
-            progname, MAX_PACKET);
+            VERSION, MAX_PACKET);
 
     exit(-1);
 }
@@ -1054,8 +1065,8 @@ parse_options (int argc, char *argv[])
     /* Scan CLI parameters */
     while ((c = getopt(argc, argv, "dhqe:i:l:m:o:u:s:S:t:T:")) != -1) {
         switch(c) {
-        case '?': help(argv[0]); break;
-        case 'h': help(argv[0]); break;
+        case '?': usage(); break;
+        case 'h': usage(); break;
         case 'd': if (!quiet) debug++; break;
         case 'q': quiet = TRUE; debug = FALSE; break;
         case 'l': pcap_link_type = strtol(optarg, NULL, 0); break;
@@ -1063,7 +1074,7 @@ parse_options (int argc, char *argv[])
         case 'o':
             if (optarg[0]!='h' && optarg[0] != 'o') {
                 fprintf(stderr, "Bad argument for '-e': %s\n", optarg);
-                help(argv[0]);
+                usage();
             }
             offset_base = (optarg[0]=='o') ? 8 : 16;
             break;
@@ -1071,7 +1082,7 @@ parse_options (int argc, char *argv[])
             hdr_ethernet = TRUE;
             if (sscanf(optarg, "%lx", &hdr_ethernet_proto) < 1) {
                 fprintf(stderr, "Bad argument for '-e': %s\n", optarg);
-                help(argv[0]);
+                usage();
             }
             break;
 
@@ -1081,7 +1092,7 @@ parse_options (int argc, char *argv[])
             if (p == optarg || *p != '\0' || hdr_ip_proto < 0 ||
                   hdr_ip_proto > 255) {
                 fprintf(stderr, "Bad argument for '-i': %s\n", optarg);
-                help(argv[0]);
+                usage();
             }
             hdr_ethernet = TRUE;
             hdr_ethernet_proto = 0x800;
@@ -1092,29 +1103,29 @@ parse_options (int argc, char *argv[])
             hdr_sctp_src   = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad src port for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             if (*p == '\0') {
                 fprintf(stderr, "No dest port specified for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_sctp_dest = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad dest port for '-s'\n");
-                help(argv[0]);
+                usage();
             }
             if (*p == '\0') {
                 fprintf(stderr, "No tag specified for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_sctp_tag = strtol(optarg, &p, 10);
             if (p == optarg || *p != '\0') {
                 fprintf(stderr, "Bad tag for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
 
             hdr_ip = TRUE;
@@ -1128,28 +1139,29 @@ parse_options (int argc, char *argv[])
             hdr_sctp_src   = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad src port for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             if (*p == '\0') {
                 fprintf(stderr, "No dest port specified for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_sctp_dest = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad dest port for '-s'\n");
-                help(argv[0]);
-            }            if (*p == '\0') {
+                usage();
+            }
+            if (*p == '\0') {
                 fprintf(stderr, "No ppi specified for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_data_chunk_ppid = strtoul(optarg, &p, 10);
             if (p == optarg || *p != '\0') {
                 fprintf(stderr, "Bad ppi for '-%c'\n", c);
-                help(argv[0]);
+                usage();
             }
 
             hdr_ip = TRUE;
@@ -1168,18 +1180,18 @@ parse_options (int argc, char *argv[])
             hdr_src_port = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad src port for '-u'\n");
-                help(argv[0]);
+                usage();
             }
             if (*p == '\0') {
                 fprintf(stderr, "No dest port specified for '-u'\n");
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_dest_port = strtol(optarg, &p, 10);
             if (p == optarg || *p != '\0') {
                 fprintf(stderr, "Bad dest port for '-u'\n");
-                help(argv[0]);
+                usage();
             }
             hdr_ip = TRUE;
             hdr_ip_proto = 17;
@@ -1193,18 +1205,18 @@ parse_options (int argc, char *argv[])
             hdr_src_port = strtol(optarg, &p, 10);
             if (p == optarg || (*p != ',' && *p != '\0')) {
                 fprintf(stderr, "Bad src port for '-T'\n");
-                help(argv[0]);
+                usage();
             }
             if (*p == '\0') {
                 fprintf(stderr, "No dest port specified for '-u'\n");
-                help(argv[0]);
+                usage();
             }
             p++;
             optarg = p;
             hdr_dest_port = strtol(optarg, &p, 10);
             if (p == optarg || *p != '\0') {
                 fprintf(stderr, "Bad dest port for '-T'\n");
-                help(argv[0]);
+                usage();
             }
             hdr_ip = TRUE;
             hdr_ip_proto = 6;
@@ -1213,13 +1225,13 @@ parse_options (int argc, char *argv[])
             break;
 
         default:
-            help(argv[0]);
+            usage();
         }
     }
 
     if (optind >= argc || argc-optind < 2) {
         fprintf(stderr, "Must specify input and output filename\n");
-        help(argv[0]);
+        usage();
     }
 
     if (strcmp(argv[optind], "-")) {
