@@ -1,7 +1,7 @@
 /* util.c
  * Utility routines
  *
- * $Id: util.c 11972 2004-09-11 23:03:36Z guy $
+ * $Id: util.c 17278 2006-02-12 21:52:18Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,37 +37,9 @@
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
-#ifdef NEED_MKSTEMP
-#include "mkstemp.h"
-#endif
-
-#ifdef HAVE_IO_H
-#include <io.h>
-#ifndef __MINGW32__
-typedef int mode_t;	/* for win32 */
-#endif /* __MINGW32__ */
-#endif /* HAVE_IO_H */
-
+#include <epan/address.h>
 #include <epan/addr_resolv.h>
 
-/*
- * This has to come after the include of <pcap.h>, as the include of
- * <pcap.h> might cause <winsock2.h> to be included, and if we've
- * already included <winsock.h> as a result of including <windows.h>,
- * we get a bunch of redefinitions.
- */
-#ifdef HAVE_WINDOWS_H
-#include <windows.h>
-#endif
-
 #include "util.h"
 
 /*
@@ -110,106 +82,6 @@ get_args_as_string(int argc, char **argv, int optind)
 	return argstring;
 }
 
-static char *
-setup_tmpdir(char *dir)
-{
-	int len = strlen(dir);
-	char *newdir;
-
-	/* Append path separator if necessary */
-	if (dir[len - 1] == G_DIR_SEPARATOR) {
-		newdir = dir;
-	}
-	else {
-		newdir = g_malloc(len + 2);
-		strcpy(newdir, dir);
-		strcat(newdir, G_DIR_SEPARATOR_S);
-	}
-	return newdir;
-}
-
-static int
-try_tempfile(char *namebuf, int namebuflen, const char *dir, const char *pfx)
-{
-	static const char suffix[] = "XXXXXXXXXX";
-	int namelen = strlen(dir) + strlen(pfx) + sizeof suffix;
-	mode_t old_umask;
-	int tmp_fd;
-
-	if (namebuflen < namelen) {
-		/* Stick in a truncated name, so that if this error is
-		   reported with the file name, you at least get
-		   something. */
-		snprintf(namebuf, namebuflen, "%s%s%s", dir, pfx, suffix);
-		errno = ENAMETOOLONG;
-		return -1;
-	}
-	strcpy(namebuf, dir);
-	strcat(namebuf, pfx);
-	strcat(namebuf, suffix);
-
-	/* The Single UNIX Specification doesn't say that "mkstemp()"
-	   creates the temporary file with mode rw-------, so we
-	   won't assume that all UNIXes will do so; instead, we set
-	   the umask to 0077 to take away all group and other
-	   permissions, attempt to create the file, and then put
-	   the umask back. */
-	old_umask = umask(0077);
-	tmp_fd = mkstemp(namebuf);
-	umask(old_umask);
-	return tmp_fd;
-}
-
-static char *tmpdir = NULL;
-#ifdef _WIN32
-static char *temp = NULL;
-#endif
-static char *E_tmpdir;
-
-#ifndef P_tmpdir
-#define P_tmpdir "/var/tmp"
-#endif
-
-int
-create_tempfile(char *namebuf, int namebuflen, const char *pfx)
-{
-	char *dir;
-	int fd;
-	static gboolean initialized;
-
-	if (!initialized) {
-		if ((dir = getenv("TMPDIR")) != NULL)
-			tmpdir = setup_tmpdir(dir);
-#ifdef _WIN32
-		if ((dir = getenv("TEMP")) != NULL)
-			temp = setup_tmpdir(dir);
-#endif
-
-		E_tmpdir = setup_tmpdir(P_tmpdir);
-		initialized = TRUE;
-	}
-
-	if (tmpdir != NULL) {
-		fd = try_tempfile(namebuf, namebuflen, tmpdir, pfx);
-		if (fd != -1)
-			return fd;
-	}
-
-#ifdef _WIN32
-	if (temp != NULL) {
-		fd = try_tempfile(namebuf, namebuflen, temp, pfx);
-		if (fd != -1)
-			return fd;
-	}
-#endif
-
-	fd = try_tempfile(namebuf, namebuflen, E_tmpdir, pfx);
-	if (fd != -1)
-		return fd;
-
-	return try_tempfile(namebuf, namebuflen, G_DIR_SEPARATOR_S "tmp", pfx);
-}
-
 /* Compute the difference between two seconds/microseconds time stamps. */
 void
 compute_timestamp_diff(gint *diffsec, gint *diffusec,
@@ -265,7 +137,7 @@ compute_timestamp_diff(gint *diffsec, gint *diffusec,
    CLIENTNAME (terminal server): <remote name>
  */
 
-gchar *get_conn_cfilter(void) {
+const gchar *get_conn_cfilter(void) {
 	static GString *filter_str = NULL;
 	gchar *env, **tokens;
 
@@ -303,10 +175,8 @@ gchar *get_conn_cfilter(void) {
 			return filter_str->str;
 		}
 	} else if ((env = getenv("CLIENTNAME")) != NULL) {
-		if (g_strcasecmp("console", env) != 0) {
-			g_string_sprintf(filter_str, "not %s host %s", host_ip_af(env), env);
-			return filter_str->str;
-		}
+		g_string_sprintf(filter_str, "not tcp port 3389");
+		return filter_str->str;
 	}
 	return "";
 }
