@@ -1,7 +1,7 @@
 /* util.h
  * Utility definitions
  *
- * $Id: util.h 12172 2004-10-01 12:32:46Z jmayer $
+ * $Id: util.h 17278 2006-02-12 21:52:18Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,19 +29,29 @@
 extern "C" {
 #endif /* __cplusplus */
 
-int create_tempfile(char *, int, const char *);
-
-/*
- * Collect command-line arguments as a string consisting of the arguments,
+/* Collect command-line arguments as a string consisting of the arguments,
  * separated by spaces.
  */
-char *get_args_as_string(int, char **, int);
+char *get_args_as_string(int argc, char **argv, int optind);
 
-/* Compute the difference between two seconds/microseconds time stamps. */
-void compute_timestamp_diff(gint *, gint *, guint32, guint32, guint32, guint32);
+/* Compute the difference between two seconds/microseconds time stamps.
+ * Beware: we're using nanosecond resolution now and function is currently unused
+ */
+void compute_timestamp_diff(gint *diffsec, gint *diffusec, 
+                            guint32 sec1, guint32 usec1, guint32 sec2, guint32 usec2);
+
+/* Try to figure out if we're remotely connected, e.g. via ssh or
+   Terminal Server, and create a capture filter that matches aspects of the
+   connection.  We match the following environment variables:
+
+   SSH_CONNECTION (ssh): <remote IP> <remote port> <local IP> <local port>
+   SSH_CLIENT (ssh): <remote IP> <remote port> <local port>
+   REMOTEHOST (tcsh, others?): <remote name>
+   DISPLAY (x11): [remote name]:<display num>
+   CLIENTNAME (terminal server): <remote name>
+ */
+const char *get_conn_cfilter(void);
 
-/* Create a capture filter for the connection */
-char *get_conn_cfilter(void);
 
 #ifdef __cplusplus
 }
