@@ -1,6 +1,6 @@
 /* ascend.c
  *
- * $Id: ascend.c 13311 2005-02-05 17:39:12Z gerald $
+ * $Id: ascend.c 16423 2005-11-07 20:14:40Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -28,13 +28,10 @@
 #include "ascend.h"
 #include "ascend-int.h"
 #include "file_wrappers.h"
+#include "file_util.h"
 
 #include <errno.h>
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -62,8 +59,8 @@
 #define ASCEND_MAX_SEEK 100000
 
 typedef struct _ascend_magic_string {
-  guint    type;
-  gchar   *strptr; 
+  guint        type;
+  const gchar   *strptr; 
 } ascend_magic_string;
 
 #define ASCEND_MAGIC_STRINGS	11
@@ -106,8 +103,8 @@ static long ascend_seek(wtap *wth, int max_seek, int *err)
   while (((byte = file_getc(wth->fh)) != EOF) && bytes_read < max_seek) {
 
     for (string_i = 0; string_i < ASCEND_MAGIC_STRINGS; string_i++) {
-      gchar *strptr = ascend_magic[string_i].strptr;
-      guint len     = strlen(strptr);
+      const gchar *strptr = ascend_magic[string_i].strptr;
+      guint len           = strlen(strptr);
       
       if (byte == *(strptr + string_level[string_i])) {
         string_level[string_i]++;
@@ -218,13 +215,14 @@ int ascend_open(wtap *wth, int *err, gchar **err_info _U_)
      packet's timestamp from the capture file's ctime, which gives us an
      offset that we can apply to each packet.
    */
-  if (fstat(wtap_fd(wth), &statbuf) == -1) {
+  if (fstat(wth->fd, &statbuf) == -1) {
     *err = errno;
     g_free(wth->capture.ascend);
     return -1;
   }
   wth->capture.ascend->inittime = statbuf.st_ctime;
   wth->capture.ascend->adjusted = 0;
+  wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 
   init_parse_ascend();
 
@@ -250,7 +248,7 @@ static void config_pseudo_header(union wtap_pseudo_header *pseudo_head)
   }
 }
 
-/* Read the next packet; called from wtap_loop(). */
+/* Read the next packet; called from wtap_read(). */
 static gboolean ascend_read(wtap *wth, int *err, gchar **err_info,
 	long *data_offset)
 {
@@ -298,8 +296,8 @@ static gboolean ascend_read(wtap *wth, int *err, gchar **err_info,
     if (wth->capture.ascend->inittime > header.secs)
       wth->capture.ascend->inittime -= header.secs;
   }
-  wth->phdr.ts.tv_sec = header.secs + wth->capture.ascend->inittime;
-  wth->phdr.ts.tv_usec = header.usecs;
+  wth->phdr.ts.secs = header.secs + wth->capture.ascend->inittime;
+  wth->phdr.ts.nsecs = header.usecs * 1000;
   wth->phdr.caplen = header.caplen;
   wth->phdr.len = header.len;
   wth->data_offset = offset;
