@@ -1,6 +1,6 @@
 /* csids.c
  *
- * $Id: csids.c 14006 2005-04-03 11:00:49Z guy $
+ * $Id: csids.c 15534 2005-08-25 21:29:54Z ulfl $
  *
  * Copyright (c) 2000 by Mike Hall <mlh@io.com>
  * Copyright (c) 2000 by Cisco Systems
@@ -138,6 +138,7 @@ int csids_open(wtap *wth, int *err, gchar **err_info _U_)
   wth->subtype_read = csids_read;
   wth->subtype_seek_read = csids_seek_read;
   wth->subtype_close = csids_close;
+  wth->tsprecision = WTAP_FILE_TSPREC_SEC;
 
   return 1;
 }
@@ -180,8 +181,8 @@ static gboolean csids_read(wtap *wth, int *err, gchar **err_info _U_,
 
   wth->phdr.len = hdr.caplen;
   wth->phdr.caplen = hdr.caplen;
-  wth->phdr.ts.tv_sec = hdr.seconds;
-  wth->phdr.ts.tv_usec = 0;
+  wth->phdr.ts.secs = hdr.seconds;
+  wth->phdr.ts.nsecs = 0;
 
   if( wth->capture.csids->byteswapped == TRUE ) {
     guint16* swap = (guint16*)buf;
