@@ -1,5 +1,5 @@
 /*
- * $Id: network_instruments.c 14822 2005-06-30 02:31:15Z sahlberg $
+ * $Id: network_instruments.c 17896 2006-04-18 20:06:11Z gerald $
  */
 
 /***************************************************************************
@@ -178,6 +178,7 @@ int network_instruments_open(wtap *wth, int *err, gchar **err_info)
 	wth->subtype_close = NULL;
 	wth->subtype_sequential_close = NULL;
 	wth->snapshot_length = 0;
+	wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 
 	/* reset the pointer to the first packet */
 	if (file_seek(wth->fh, file_header.offset_to_first_packet, SEEK_SET,
@@ -205,24 +206,52 @@ static gboolean observer_read(wtap *wth, int *err, gchar **err_info,
 	
 	observer_time packet_time;
 
-	*data_offset = wth->data_offset;
+	/*
+	 * Skip records other than data records.
+	 */
+	for (;;) {
+		*data_offset = wth->data_offset;
 
-	/* pull off the packet header */
-	bytes_read = file_read(&packet_header, sizeof packet_header, 1, wth->fh);
-	if (bytes_read != sizeof packet_header) {
-		*err = file_error(wth->fh);
-		if (*err != 0)
-			return -1;
-		return 0;
-	}
-	wth->data_offset += bytes_read;
+		/* pull off the packet header */
+		bytes_read = file_read(&packet_header, sizeof packet_header, 1, wth->fh);
+		if (bytes_read != sizeof packet_header) {
+			*err = file_error(wth->fh);
+			if (*err != 0)
+				return -1;
+			return 0;
+		}
+		wth->data_offset += bytes_read;
 
-	/* check the packet's magic number; the magic number is all 8's,
-	   so the byte order doesn't matter */
-	if (packet_header.packet_magic != observer_packet_magic) {
-		*err = WTAP_ERR_BAD_RECORD;
-		*err_info = g_strdup("Observer: bad record");
-		return FALSE;
+		/* check the packet's magic number; the magic number is all 8's,
+		   so the byte order doesn't matter */
+		if (packet_header.packet_magic != observer_packet_magic) {
+			*err = WTAP_ERR_BAD_RECORD;
+			*err_info = g_strdup_printf("Observer: bad record: Invalid magic number 0x%08x",
+			    GUINT32_FROM_LE(packet_header.packet_magic));
+			return FALSE;
+		}
+
+		/* check the packet's record type, and skip non-data
+		   packets */
+		if (packet_header.packet_type == TYPE_DATA_PACKET)
+			break;
+
+		/* skip to next packet */
+		packet_header.offset_to_next_packet =
+		    GUINT16_FROM_LE(packet_header.offset_to_next_packet);
+		if (packet_header.offset_to_next_packet < sizeof(packet_header)) {
+			*err = WTAP_ERR_BAD_RECORD;
+			*err_info = g_strdup_printf("Observer: bad record (offset to next packet %u < %lu)",
+			    packet_header.offset_to_next_packet,
+			    (unsigned long)sizeof(packet_header));
+			return FALSE;
+		}
+		seek_increment = packet_header.offset_to_next_packet - sizeof(packet_header);
+		if(seek_increment>0) {
+			if (file_seek(wth->fh, seek_increment, SEEK_CUR, err) == -1)
+				return FALSE;
+		}
+		wth->data_offset += seek_increment;
 	}
 
 	/* convert from observer time to wiretap time */
@@ -240,8 +269,8 @@ static gboolean observer_read(wtap *wth, int *err, gchar **err_info,
 	wth->phdr.pkt_encap = observer_encap[packet_header.network_type];
 	wth->phdr.len    = packet_header.network_size-4; /* neglect frame markers for wiretap */
 	wth->phdr.caplen = MIN(packet_header.captured_size, wth->phdr.len);
-	wth->phdr.ts.tv_sec  = seconds;
-	wth->phdr.ts.tv_usec = useconds;
+	wth->phdr.ts.secs  = seconds;
+	wth->phdr.ts.nsecs = useconds * 1000;
 
 	/* get to the frame data */
 	packet_header.offset_to_frame =
@@ -353,8 +382,8 @@ int network_instruments_dump_can_write_encap(int encap)
 	/* Per-packet encapsulations aren't supported. */
 	if (encap == WTAP_ENCAP_PER_PACKET)
 		return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
-
-	if (encap < 0 || (unsigned) encap > NUM_FROM_WTAP_ENCAPS || from_wtap_encap[encap] == OBSERVER_UNDEFINED)
+	
+	if (encap < 0 || (unsigned) encap >= NUM_FROM_WTAP_ENCAPS || from_wtap_encap[encap] == OBSERVER_UNDEFINED)
 		return WTAP_ERR_UNSUPPORTED_ENCAP;
 
 	return 0;
@@ -430,14 +459,14 @@ static gboolean observer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	size_t nwritten;
 	guint64 capture_nanoseconds = 0;
 
-	if(phdr->ts.tv_sec<(long)seconds1970to2000) {
-		if(phdr->ts.tv_sec<0)
+	if(phdr->ts.secs<(long)seconds1970to2000) {
+		if(phdr->ts.secs<0)
 			capture_nanoseconds = 0;
 		else
-			capture_nanoseconds = phdr->ts.tv_sec;
+			capture_nanoseconds = phdr->ts.secs;
 	} else
-		capture_nanoseconds = phdr->ts.tv_sec - seconds1970to2000;
-	capture_nanoseconds = ((capture_nanoseconds*1000000) + (guint64)phdr->ts.tv_usec)*1000;
+		capture_nanoseconds = phdr->ts.secs - seconds1970to2000;
+	capture_nanoseconds = ((capture_nanoseconds*1000000) + (guint64)phdr->ts.nsecs);
 
 	memset(&packet_header, 0x00, sizeof(packet_entry_header));
 	packet_header.packet_magic = GUINT32_TO_LE(observer_packet_magic);
