@@ -1,5 +1,5 @@
 /*
- * $Id: network_instruments.h 11400 2004-07-18 00:24:25Z guy $
+ * $Id: network_instruments.h 17896 2006-04-18 20:06:11Z gerald $
  */
 
 /***************************************************************************
@@ -34,6 +34,23 @@ typedef struct capture_file_header
 	char	extra_information_present;
 } capture_file_header;
 
+#define TYPE_DATA_PACKET		0
+#define TYPE_EXPERT_INFORMATION_PACKET	1
+
+/*
+ * The Observer document indicates that the types of expert information
+ * packets are:
+ *
+ *	Network Load (markers used by Expert Time Interval and What If
+ *	analysis modes)
+ *
+ *	Start/Stop Packet Capture marker frames (with time stamps when
+ *	captures start and stop)
+ *
+ *	Wireless Channel Change (markers showing what channel was being
+ *	currently listened to)
+ */
+
 typedef struct packet_entry_header
 {
 	guint32 packet_magic;
