@@ -1,6 +1,6 @@
 /* netxray.c
  *
- * $Id: netxray.c 12341 2004-10-18 15:14:13Z gerald $
+ * $Id: netxray.c 17899 2006-04-18 20:46:48Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -46,55 +46,173 @@ static const char netxray_magic[] = {	/* magic header */
 	'X', 'C', 'P', '\0'
 };
 
-/* NetXRay file header (minus magic number). */
+/* NetXRay file header (minus magic number).			*/
+/*								*/
+/* As field usages are identified, please revise as needed	*/
+/* Please do *not* use netxray_hdr xxx... names in the code	*/
+/* (Placeholder names for all 'unknown' fields are		*/
+/*   of form xxx_x<hex_hdr_offset>				*/
+/*   where <hex_hdr_offset> *includes* the magic number)	*/
+
 struct netxray_hdr {
-	char	version[8];	/* version number */
-	guint32	start_time;	/* UNIX time when capture started */
-	guint32	nframes;	/* number of packets */
-	guint32	xxx;		/* unknown */
-	guint32	start_offset;	/* offset of first packet in capture */
-	guint32	end_offset;	/* offset after last packet in capture */
-	guint32 xxy[3];		/* unknown */
-	guint8	network;	/* datalink type */
-	guint8	xxz[3];		/* XXX - is this the upper 3 bytes of the datalink type? */
-	guint8	timeunit;	/* encodes length of a tick */
-	guint8	xxa[3];		/* XXX - is this the upper 3 bytes of the time units? */
-	guint32	timelo;		/* lower 32 bits of time stamp of capture start */
-	guint32	timehi;		/* upper 32 bits of time stamp of capture start */
-	guint32 linespeed;	/* speed of network, in bits/second */
-	guint8	xxb[64];	/* other stuff */
+	char	version[8];	/* version number				*/
+	guint32	start_time;	/* UNIX [UTC] time when capture started		*/
+
+	guint32	nframes;	/* number of packets				*/
+	guint32	xxx_x14;	/* unknown [some kind of file offset]		*/
+	guint32	start_offset;	/* offset of first packet in capture		*/
+	guint32	end_offset;	/* offset after last packet in capture		*/
+
+	guint32 xxx_x20;	/* unknown [some kind of file offset]		*/
+	guint32 xxx_x24;	/* unknown [unused ?]				*/
+	guint32 xxx_x28;	/* unknown [some kind of file offset]		*/
+	guint8	network;	/* datalink type				*/
+	guint8	network_plus;	/* [See code]					*/
+	guint8	xxx_x2E[2];	/* unknown					*/
+
+	guint8	timeunit;	/* encodes length of a tick			*/
+	guint8	xxx_x31[3];	/* XXX - upper 3 bytes of timeunit ?		*/
+	guint32	timelo;		/* lower 32 bits of capture start time stamp	*/
+	guint32	timehi;		/* upper 32 bits of capture start time stamp	*/
+	guint32 linespeed;	/* speed of network, in bits/second		*/
+
+	guint8	xxx_x40[12];	/* unknown [other stuff]			*/
+	guint8	realtick[4];	/* in v2, means ???                     	*/
+
+	guint8	xxx_x50[4];	/* unknown [other stuff]			*/
+	guint8	captype;	/* capture type					*/
+	guint8  xxx_x55[3];	/* unknown [other stuff]			*/
+	guint8  xxx_x58[4];	/* unknown [other stuff]			*/
+	guint8  wan_hdlc_subsub_captype; /* WAN HDLC subsub_captype		*/
+	guint8  xxx_x5D[3];	/* unknown [other stuff]			*/
+
+	guint8	xxx_x60[16];	/* unknown [other stuff]			*/
+
+	guint8  xxx_x70[14];    /* unknown [other stuff]			*/
+	gint16 timezone_hrs;	/* timezone hours [at least for version 2.2..];	*/
+				/*  positive values = west of UTC:		*/
+				/*  negative values = east of UTC:		*/
+				/*  e.g. +5 is American Eastern			*/
+				/* [Does not appear to be adjusted for DST ]	*/
 };
 
 /*
- * Capture type, in xxb[20].
+ * Capture type, in hdr.captype.
+ *
+ * XXX - S6040-model Sniffers with gigabit blades store 6 here for
+ * Ethernet captures, and some other Ethernet captures had a capture
+ * type of 3, so presumably the interpretation of the capture type
+ * depends on the network type.  We prefix all the capture types
+ * for WAN captures with WAN_.
  */
-#define CAPTYPE_NDIS	0	/* Capture on network interface using NDIS */
-#define CAPTYPE_BROUTER	1	/* Bridge/router captured with pod */
-#define CAPTYPE_GIGPOD	2	/* gigabit Ethernet captured with pod */
-#define CAPTYPE_PPP	3	/* PPP captured with pod */
-#define CAPTYPE_FRELAY	4	/* Frame Relay captured with pod */
-#define CAPTYPE_BROUTER2 5	/* Bridge/router captured with pod */
-#define CAPTYPE_HDLC	6	/* HDLC (X.25, ISDN) captured with pod */
-#define CAPTYPE_SDLC	7	/* SDLC captured with pod */
-#define CAPTYPE_HDLC2	8	/* HDLC captured with pod */
-#define CAPTYPE_BROUTER3 9	/* Bridge/router captured with pod */
-#define CAPTYPE_SMDS	10	/* SMDS DXI */
-#define CAPTYPE_BROUTER4 11	/* Bridge/router captured with pod */
-#define CAPTYPE_BROUTER5 12	/* Bridge/router captured with pod */
+#define CAPTYPE_NDIS	0		/* Capture on network interface using NDIS 			*/
 
 /*
- * # of ticks that equal 1 second
+ * Ethernet capture types.
+ */
+#define ETH_CAPTYPE_GIGPOD	2	/* gigabit Ethernet captured with pod				*/
+#define ETH_CAPTYPE_OTHERPOD	3	/* non-gigabit Ethernet captured with pod			*/
+#define ETH_CAPTYPE_OTHERPOD2	5	/* gigabit Ethernet via pod ??					*/
+					/*  Captype 5 seen in capture from Distributed Sniffer with:	*/
+					/*    Version 4.50.211 software					*/
+					/*    SysKonnect SK-9843 Gigabit Ethernet Server Adapter	*/
+#define ETH_CAPTYPE_GIGPOD2	6	/* gigabit Ethernet captured with pod */
+
+/*
+ * WAN capture types.
+ */
+#define WAN_CAPTYPE_BROUTER	1	/* Bridge/router captured with pod */
+#define WAN_CAPTYPE_PPP		3	/* PPP captured with pod */
+#define WAN_CAPTYPE_FRELAY	4	/* Frame Relay captured with pod */
+#define WAN_CAPTYPE_BROUTER2	5	/* Bridge/router captured with pod */
+#define WAN_CAPTYPE_HDLC	6	/* HDLC (X.25, ISDN) captured with pod */
+#define WAN_CAPTYPE_SDLC	7	/* SDLC captured with pod */
+#define WAN_CAPTYPE_HDLC2	8	/* HDLC captured with pod */
+#define WAN_CAPTYPE_BROUTER3	9	/* Bridge/router captured with pod */
+#define WAN_CAPTYPE_SMDS	10	/* SMDS DXI */
+#define WAN_CAPTYPE_BROUTER4	11	/* Bridge/router captured with pod */
+#define WAN_CAPTYPE_BROUTER5	12	/* Bridge/router captured with pod */
+
+#define CAPTYPE_ATM		15	/* ATM captured with pod */
+
+/*
+ * # of ticks that equal 1 second, in version 002.xxx files other
+ * than Ethernet captures with a captype other than CAPTYPE_NDIS;
+ * the index into this array is hdr.timeunit.
  *
- * XXX - the third item was 1193180.0, presumably because somebody found
+ * DO NOT SEND IN PATCHES THAT CHANGE ANY OF THE NON-ZERO VALUES IN
+ * ANY OF THE TpS TABLES.  THOSE VALUES ARE CORRECT FOR AT LEAST ONE
+ * CAPTURE, SO CHANGING THEM WILL BREAK AT LEAST SOME CAPTURES.  WE
+ * WILL NOT CHECK IN PATCHES THAT CHANGE THESE VALUES.
+ *
+ * Instead, if a value in a TpS table is wrong, check whether captype
+ * has a non-zero value; if so, perhaps we need a new TpS table for the
+ * corresponding network type and captype.
+ *
+ * TpS...[] entries of 0.0 mean that no capture file for the
+ * corresponding captype/timeunit values has yet been seen
+ *
+ * Note that the "realtick" value is wrong in many captures, so
+ * we no longer use it.  We don't know what significance it has.
+ * In at least one capture where "realtick" doesn't correspond
+ * to the value from the appropriate TpS table, the per-packet header's
+ * "xxx" field is all zero, so it's not as if a 2.x header includes
+ * a "compatibility" time stamp corresponding to the value from the
+ * TpS table and a "real" time stamp corresponding to "realtick".
+ *
+ * XXX - the third item is 1193180.0, presumably because somebody found
  * it gave the right answer for some captures, but 3 times that, i.e.
  * 3579540.0, appears to give the right answer for some other captures.
+ * Some captures have realtick of 1193182, some have 3579545, and some
+ * have 1193000.  Most of those, in one set of captures somebody has,
+ * are wrong.
  *
- * Is there something else in the file header to indicate which of those
- * is correct?
+ * XXX - in at least one ATM capture, hdr.realtick is 1193180.0
+ * and hdr.timeunit is 0.  Does that capture have a captype of
+ * CAPTYPE_ATM?  If so, what should the table for ATM captures with
+ * that captype be?
  */
-static double TpS[] = { 1e6, 1193000.0, 3579540.0 };
+static double TpS[] = { 1e6, 1193000.0, 1193182.0 };
 #define NUM_NETXRAY_TIMEUNITS (sizeof TpS / sizeof TpS[0])
 
+/*
+ * Table of time units for Ethernet captures with captype ETH_CAPTYPE_GIGPOD.
+ * 0.0 means "unknown.
+ *
+ * It appears that, at least for Ethernet captures, if captype is
+ * ETH_CAPTYPE_GIGPOD, that indicates that it's a gigabit Ethernet
+ * capture, possibly from a special whizzo gigabit pod, and also
+ * indicates that the time stamps have some higher resolution than
+ * in other captures, possibly thanks to a high-resolution timer
+ * on the pod.
+ *
+ * It also appears that the time units might differ for gigabit pod
+ * captures between version 002.001 and 002.002.  For 002.001,
+ * the values below are correct; for 002.002, it's claimed that
+ * the right value for TpS_gigpod[2] is 1250000.0, but at least one
+ * 002.002 gigabit pod capture has 31250000.0 as the right value.
+ */
+static double TpS_gigpod[] = { 1e9, 0.0, 31250000.0 };
+#define NUM_NETXRAY_TIMEUNITS_GIGPOD (sizeof TpS_gigpod / sizeof TpS_gigpod[0])
+
+/*
+ * Table of time units for Ethernet captures with captype ETH_CAPTYPE_OTHERPOD.
+ */
+static double TpS_otherpod[] = { 1e6, 0.0, 1250000.0 }; 
+#define NUM_NETXRAY_TIMEUNITS_OTHERPOD (sizeof TpS_otherpod / sizeof TpS_otherpod[0])
+
+/*
+ * Table of time units for Ethernet captures with captype ETH_CAPTYPE_OTHERPOD2.
+ */
+static double TpS_otherpod2[] = { 1e6, 0.0, 0.0 }; 
+#define NUM_NETXRAY_TIMEUNITS_OTHERPOD2 (sizeof TpS_otherpod2 / sizeof TpS_otherpod2[0])
+
+/*
+ * Table of time units for Ethernet captures with captype ETH_CAPTYPE_GIGPOD2. 
+ */
+static double TpS_gigpod2[] = { 1e9, 0.0, 20000000.0 };
+#define NUM_NETXRAY_TIMEUNITS_GIGPOD2 (sizeof TpS_gigpod2 / sizeof TpS_gigpod2[0])
+
 /* Version number strings. */
 static const char vers_1_0[] = {
 	'0', '0', '1', '.', '0', '0', '0', '\0'
@@ -116,6 +234,10 @@ static const char vers_2_002[] = {
 	'0', '0', '2', '.', '0', '0', '2', '\0'
 };
 
+static const char vers_2_003[] = {
+	'0', '0', '2', '.', '0', '0', '3', '\0'
+};
+
 /* Old NetXRay data record format - followed by frame data. */
 struct old_netxrayrec_hdr {
 	guint32	timelo;		/* lower 32 bits of time stamp */
@@ -189,22 +311,22 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		WTAP_ENCAP_TOKEN_RING,
 		WTAP_ENCAP_FDDI_BITSWAPPED,
 		/*
-		 * XXX - PPP captures may look like Ethernet, perhaps
-		 * because they're using NDIS to capture on the
+		 * XXX - some PPP captures may look like Ethernet,
+		 * perhaps because they're using NDIS to capture on the
 		 * same machine and it provides simulated-Ethernet
-		 * packets, but at least one ISDN capture uses the
-		 * same network type value but isn't shaped like
-		 * Ethernet.
+		 * packets, but captures taken with various serial
+		 * pods use the same network type value but aren't
+		 * shaped like Ethernet.  We handle that below.
 		 */
-		WTAP_ENCAP_ETHERNET,	/* WAN(PPP), but shaped like Ethernet */
-		WTAP_ENCAP_UNKNOWN,	/* LocalTalk */
-		WTAP_ENCAP_UNKNOWN,	/* "DIX" - should not occur */
-		WTAP_ENCAP_UNKNOWN,	/* ARCNET raw */
-		WTAP_ENCAP_UNKNOWN,	/* ARCNET 878.2 */
-		WTAP_ENCAP_ATM_PDUS_UNTRUNCATED,	/* ATM */
+		WTAP_ENCAP_ETHERNET,		/* WAN(PPP), but shaped like Ethernet */
+		WTAP_ENCAP_UNKNOWN,		/* LocalTalk */
+		WTAP_ENCAP_UNKNOWN,		/* "DIX" - should not occur */
+		WTAP_ENCAP_UNKNOWN,		/* ARCNET raw */
+		WTAP_ENCAP_UNKNOWN,		/* ARCNET 878.2 */
+		WTAP_ENCAP_ATM_PDUS_UNTRUNCATED,/* ATM */
 		WTAP_ENCAP_IEEE_802_11_WITH_RADIO,
-					/* Wireless WAN with radio information */
-		WTAP_ENCAP_UNKNOWN	/* IrDA */
+						/* Wireless WAN with radio information */
+		WTAP_ENCAP_UNKNOWN		/* IrDA */
 	};
 	#define NUM_NETXRAY_ENCAPS (sizeof netxray_encap / sizeof netxray_encap[0])
 	int file_encap;
@@ -254,12 +376,10 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		 * It also appears that version 2.00x files have per-packet
 		 * headers with some extra fields. */
 		if (memcmp(hdr.version, vers_1_0, sizeof vers_1_0) == 0) {
-			timeunit = 1000.0;
 			version_major = 1;
 			version_minor = 0;
 			file_type = WTAP_FILE_NETXRAY_1_0;
 		} else if (memcmp(hdr.version, vers_1_1, sizeof vers_1_1) == 0) {
-			timeunit = 1000000.0;
 			version_major = 1;
 			version_minor = 1;
 			file_type = WTAP_FILE_NETXRAY_1_1;
@@ -275,6 +395,10 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 			version_major = 2;
 			version_minor = 2;
 			file_type = WTAP_FILE_NETXRAY_2_00x;
+		} else if (memcmp(hdr.version, vers_2_003, sizeof vers_2_003) == 0) {
+			version_major = 2;
+			version_minor = 3;
+			file_type = WTAP_FILE_NETXRAY_2_00x;
 		} else {
 			*err = WTAP_ERR_UNSUPPORTED;
 			*err_info = g_strdup_printf("netxray: version \"%.8s\" unsupported", hdr.version);
@@ -282,7 +406,7 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		}
 	}
 
-	switch (hdr.xxz[0]) {
+	switch (hdr.network_plus) {
 
 	case 0:
 		/*
@@ -306,7 +430,7 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 	default:
 		*err = WTAP_ERR_UNSUPPORTED;
 		*err_info = g_strdup_printf("netxray: the byte after the network type has the value %u, which I don't understand",
-		    hdr.xxz[0]);
+		    hdr.network_plus);
 		return -1;
 	}
 
@@ -314,7 +438,7 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 	    || netxray_encap[network_type] == WTAP_ENCAP_UNKNOWN) {
 		*err = WTAP_ERR_UNSUPPORTED_ENCAP;
 		*err_info = g_strdup_printf("netxray: network type %u (%u) unknown or unsupported",
-		    network_type, hdr.xxz[0]);
+		    network_type, hdr.network_plus);
 		return -1;
 	}
 
@@ -327,68 +451,170 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 
 	case WTAP_FILE_NETXRAY_OLD:
 		timeunit = 1000.0;
+		wth->tsprecision = WTAP_FILE_TSPREC_MSEC;
 		break;
 
 	case WTAP_FILE_NETXRAY_1_0:
 		timeunit = 1000.0;
+		wth->tsprecision = WTAP_FILE_TSPREC_MSEC;
 		break;
 
 	case WTAP_FILE_NETXRAY_1_1:
+		/*
+		 * In version 1.1 files (as produced by Windows Sniffer
+		 * Pro 2.0.01), the time stamp is in microseconds,
+		 * rather than the milliseconds time stamps in NetXRay
+		 * and older versions of Windows Sniffer.
+		 */
 		timeunit = 1000000.0;
+		wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	case WTAP_FILE_NETXRAY_2_00x:
 		/*
-		 * It appears that, at least for Ethernet
-		 * captures, if hdr.xxb[20] is 2, that indicates
-		 * that it's a gigabit Ethernet capture, possibly
-		 * from a special whizzo gigabit pod, and also
-		 * indicates that the time stamps have some
-		 * higher resolution than in other captures,
-		 * possibly thanks to a high-resolution
-		 * timer on the pod.
-		 *
-		 * It also appears that the time units might differ
-		 * for gigabit pod captures between version 002.001
-		 * and 002.002.
+		 * Get the time stamp value from the appropriate TpS
+		 * table.
 		 */
-		if (network_type == 1 && hdr.xxb[20] == CAPTYPE_GIGPOD) {
-			if (version_minor == 1) {
+		switch (network_type) {
+
+		case 1:
+			/*
+			 * Ethernet - the table to use depends on whether
+			 * this is an NDIS or pod capture.
+			 */
+			switch (hdr.captype) {
+
+			case CAPTYPE_NDIS:
+				if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS) {
+					*err = WTAP_ERR_UNSUPPORTED;
+					*err_info = g_strdup_printf(
+					    "netxray: Unknown timeunit %u for Ethernet/CAPTYPE_NDIS version %.8s capture",
+					    hdr.timeunit, hdr.version);
+					return -1;
+				}
+				timeunit = TpS[hdr.timeunit];
+				break;
+
+			case ETH_CAPTYPE_GIGPOD:
+				if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS_GIGPOD
+				    || TpS_gigpod[hdr.timeunit] == 0.0) {
+					*err = WTAP_ERR_UNSUPPORTED;
+					*err_info = g_strdup_printf(
+					    "netxray: Unknown timeunit %u for Ethernet/ETH_CAPTYPE_GIGPOD version %.8s capture",
+					    hdr.timeunit, hdr.version);
+					return -1;
+				}
+				timeunit = TpS_gigpod[hdr.timeunit];
+
 				/*
-				 * It appears that the time units for
-				 * these captures are nanoseconds, unless
-				 * hdr.timeunit is 2, in which case it's
-				 * 1/31250000.0 of a second.
+				 * At least for 002.002 and 002.003
+				 * captures, the start time stamp is 0,
+				 * not the value in the file.
 				 */
-				if (hdr.timeunit == 2)
-					timeunit = 31250000.0;
-				else
-					timeunit = 1e9;
-			} else {
+				if (version_minor == 2 || version_minor == 3)
+					start_timestamp = 0.0;
+				break;
+
+			case ETH_CAPTYPE_OTHERPOD:
+				if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS_OTHERPOD
+				    || TpS_otherpod[hdr.timeunit] == 0.0) {
+					*err = WTAP_ERR_UNSUPPORTED;
+					*err_info = g_strdup_printf(
+					    "netxray: Unknown timeunit %u for Ethernet/ETH_CAPTYPE_OTHERPOD version %.8s capture",
+					    hdr.timeunit, hdr.version);
+					return -1;
+				}
+				timeunit = TpS_otherpod[hdr.timeunit];
+
 				/*
-				 * These just seem to be 1000 times
-				 * the regular time stamp units, unless
-				 * hdr.timeunit is 2, in which case it's
-				 * 1/1125000 of a second.
+				 * At least for 002.002 and 002.003
+				 * captures, the start time stamp is 0,
+				 * not the value in the file.
+				 */
+				if (version_minor == 2 || version_minor == 3)
+					start_timestamp = 0.0;
+				break;
+
+			case ETH_CAPTYPE_OTHERPOD2:
+				if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS_OTHERPOD2
+				    || TpS_otherpod2[hdr.timeunit] == 0.0) {
+					*err = WTAP_ERR_UNSUPPORTED;
+					*err_info = g_strdup_printf(
+					    "netxray: Unknown timeunit %u for Ethernet/ETH_CAPTYPE_OTHERPOD2 version %.8s capture",
+					    hdr.timeunit, hdr.version);
+					return -1;
+				}
+				timeunit = TpS_otherpod2[hdr.timeunit];
+				/*
+				 * XXX: start time stamp in the one capture file examined of this type was 0;
+				 *      We'll assume the start time handling is the same as for other pods.
 				 *
-				 * In addition, the start timestamp
-				 * appears to be 0.
+				 * At least for 002.002 and 002.003
+				 * captures, the start time stamp is 0,
+				 * not the value in the file.
 				 */
-				if (hdr.timeunit == 2)
-					timeunit = 1250000.0;
-				else
-					timeunit = TpS[hdr.timeunit]*1000.0;
-				start_timestamp = 0.0;
-			}
-		} else {
-			if (hdr.timeunit > NUM_NETXRAY_TIMEUNITS) {
+				if (version_minor == 2 || version_minor == 3)
+					start_timestamp = 0.0;
+				break;
+
+			case ETH_CAPTYPE_GIGPOD2:
+				if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS_GIGPOD2
+				    || TpS_gigpod2[hdr.timeunit] == 0.0) {
+					*err = WTAP_ERR_UNSUPPORTED;
+					*err_info = g_strdup_printf(
+					    "netxray: Unknown timeunit %u for Ethernet/ETH_CAPTYPE_GIGPOD2 version %.8s capture",
+					    hdr.timeunit, hdr.version);
+					return -1;
+				}
+				timeunit = TpS_gigpod2[hdr.timeunit];
+                                /*
+                                 * XXX: start time stamp in the one capture file examined of this type was 0;
+                                 *      We'll assume the start time handling is the same as for other pods.
+                                 *
+                                 * At least for 002.002 and 002.003
+                                 * captures, the start time stamp is 0,
+                                 * not the value in the file.
+                                 */
+                                if (version_minor == 2 || version_minor == 3)
+                                        start_timestamp = 0.0;
+				break;
+
+			default:
 				*err = WTAP_ERR_UNSUPPORTED;
-				*err_info = g_strdup_printf("netxray: Unknown timeunit %u",
-				    hdr.timeunit);
+				*err_info = g_strdup_printf(
+				    "netxray: Unknown capture type %u for Ethernet version %.8s capture",
+				    hdr.captype, hdr.version);
+				return -1;
+			}
+			break;
+
+		default:
+			if (hdr.timeunit >= NUM_NETXRAY_TIMEUNITS) {
+				*err = WTAP_ERR_UNSUPPORTED;
+				*err_info = g_strdup_printf(
+				    "netxray: Unknown timeunit %u for %u/%u version %.8s capture",
+				    hdr.timeunit, network_type, hdr.captype,
+				    hdr.version);
 				return -1;
 			}
 			timeunit = TpS[hdr.timeunit];
+			break;
 		}
+
+		/*
+		 * If the number of ticks per second is greater than
+		 * 1 million, make the precision be nanoseconds rather
+		 * than microseconds.
+		 *
+		 * XXX - do values only slightly greater than one million
+		 * correspond to a resolution sufficiently better than
+		 * 1 microsecond to display more digits of precision?
+		 * XXX - Seems reasonable to use nanosecs only if TPS >= 10M
+		 */
+		if (timeunit >= 1e7)
+			wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
+		else
+			wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	default:
@@ -404,33 +630,45 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		 * frames (as a result, presumably, of having passed
 		 * through NDISWAN).
 		 *
-		 * In version 2, it looks as if there's stuff in the "xxb"
-		 * words of the file header to specify what particular
-		 * type of WAN capture we have.
+		 * In version 2, it looks as if there's stuff in the 
+		 * file header to specify what particular type of WAN
+		 * capture we have.
 		 */
 		if (version_major == 2) {
-			switch (hdr.xxb[20]) {
+			switch (hdr.captype) {
 
-			case CAPTYPE_PPP:
+			case WAN_CAPTYPE_PPP:
 				/*
 				 * PPP.
 				 */
 				file_encap = WTAP_ENCAP_PPP_WITH_PHDR;
 				break;
 
-			case CAPTYPE_FRELAY:
+			case WAN_CAPTYPE_FRELAY:
 				/*
 				 * Frame Relay.
+				 *
+				 * XXX - in at least one capture, this
+				 * is Cisco HDLC, not Frame Relay, but
+				 * in another capture, it's Frame Relay.
+				 *
+				 * [Bytes in each capture:
+				 * Cisco HDLC:  hdr.xxx_x60[06:10]: 0x02 0x00 0x01 0x00 0x06
+				 * Frame Relay: hdr.xxx_x60[06:10]  0x00 0x00 0x00 0x00 0x00
+
+				 * Cisco HDLC:  hdr.xxx_x60[14:15]: 0xff 0xff
+				 * Frame Relay: hdr.xxx_x60[14:15]: 0x00 0x00
+				 * ]
 				 */
 				file_encap = WTAP_ENCAP_FRELAY_WITH_PHDR;
 				break;
 
-			case CAPTYPE_HDLC:
-			case CAPTYPE_HDLC2:
+			case WAN_CAPTYPE_HDLC:
+			case WAN_CAPTYPE_HDLC2:
 				/*
 				 * Various HDLC flavors?
 				 */
-				switch (hdr.xxb[28]) {
+				switch (hdr.wan_hdlc_subsub_captype) {
 
 				case 0:	/* LAPB/X.25 */
 					file_encap = WTAP_ENCAP_LAPB;
@@ -440,18 +678,18 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 				case 2:	/* T1 PRI */
 				case 3:	/* BRI */
 					file_encap = WTAP_ENCAP_ISDN;
-					isdn_type = hdr.xxb[28];
+					isdn_type = hdr.wan_hdlc_subsub_captype;
 					break;
 
 				default:
 					*err = WTAP_ERR_UNSUPPORTED_ENCAP;
 					*err_info = g_strdup_printf("netxray: WAN HDLC capture subsubtype 0x%02x unknown or unsupported",
-					   hdr.xxb[28]);
+					   hdr.wan_hdlc_subsub_captype);
 					return -1;
 				}
 				break;
 
-			case CAPTYPE_SDLC:
+			case WAN_CAPTYPE_SDLC:
 				/*
 				 * SDLC.
 				 */
@@ -461,7 +699,7 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 			default:
 				*err = WTAP_ERR_UNSUPPORTED_ENCAP;
 				*err_info = g_strdup_printf("netxray: WAN capture subtype 0x%02x unknown or unsupported",
-				   hdr.xxb[20]);
+				   hdr.captype);
 				return -1;
 			}
 		} else
@@ -497,15 +735,21 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		 * captures, for frames that have 0xff in hdr_2_x.xxx[2]
 		 * and hdr_2_x.xxx[3] in the per-packet header:
 		 *
-		 *	if, in the file header, hdr.xxb[13] is 0x34 and
-		 *	hdr.xxb[14] is 0x12, the frames have an FCS at
-		 *	the end;
+		 *	if, in the file header, hdr.realtick[1] is 0x34
+		 *	and hdr.realtick[2] is 0x12, the frames have an
+		 *	FCS at the end;
 		 *
 		 *	otherwise, they have 4 bytes of junk at the end.
 		 *
+		 * Yes, it's strange that you have to check the *middle*
+		 * of the time stamp field; you can't check for any
+		 * particular value of the time stamp field.
+		 *
 		 * For now, we assume that to be true for 802.11 captures
 		 * as well; it appears to be the case for at least one
-		 * such capture.
+		 * such capture - the file doesn't have 0x34 and 0x12,
+		 * and the 4 bytes at the end of the frames with 0xff
+		 * are junk, not an FCS.
 		 *
 		 * For ISDN captures, it appears, at least in some
 		 * captures, to be similar, although I haven't yet
@@ -513,11 +757,49 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 		 *
 		 * XXX - should we do this for all encapsulation types?
 		 *
-		 * XXX - is that actually a 2-byte little-endian 0x1234?
-		 * What does that field signify?
+		 * XXX - is there some other field that *really* indicates
+		 * whether we have an FCS or not?  The check of the time
+		 * stamp is bizarre, as we're checking the middle.
+		 * Perhaps hdr.realtick[0] is 0x00, in which case time
+		 * stamp units in the range 1192960 through 1193215
+		 * correspond to captures with an FCS, but that's still
+		 * a bit bizarre.
+		 *
+		 * Note that there are captures with a network type of 0
+		 * (Ethernet) and capture type of 0 (NDIS) that do, and
+		 * that don't, have 0x34 0x12 in them, and at least one
+		 * of the NDIS captures with 0x34 0x12 in it has FCSes,
+		 * so it's not as if no NDIS captures have an FCS.
+		 *
+		 * There are also captures with a network type of 4 (WAN),
+		 * capture type of 6 (HDLC), and subtype of 2 (T1 PRI) that
+		 * do, and that don't, have 0x34 0x12, so there are at least
+		 * some captures taken with a WAN pod that might lack an FCS.
+		 * (We haven't yet tried dissecting the 4 bytes at the
+		 * end of packets with hdr_2_x.xxx[2] and hdr_2_x.xxx[3]
+		 * equal to 0xff as an FCS.)
+		 *
+		 * All captures I've seen that have 0x34 and 0x12 *and*
+		 * have at least one frame with an FCS have a value of
+		 * 0x01 in xxx_x40[4].  No captures I've seen with a network
+		 * type of 0 (Ethernet) missing 0x34 0x12 have 0x01 there,
+		 * however.  However, there's at least one capture
+		 * without 0x34 and 0x12, with a network type of 0,
+		 * and with 0x01 in xxx_x40[4], *without* FCSes in the
+		 * frames - the 4 bytes at the end are all zero - so it's
+		 * not as simple as "xxx_x40[4] = 0x01 means the 4 bytes at
+		 * the end are FCSes".  Also, there's also at least one
+		 * 802.11 capture with an xxx_x40[4] value of 0x01 with junk
+		 * rather than an FCS at the end of the frame, so xxx_x40[4]
+		 * isn't an obvious flag to determine whether the
+		 * capture has FCSes.
+		 *
+		 * There don't seem to be any other values in any of the
+		 * xxx_x5..., xxx_x6...., xxx_x7.... fields
+		 * that obviously correspond to frames having an FCS.
 		 */
 		if (version_major == 2) {
-			if (hdr.xxb[13] == 0x34 && hdr.xxb[14] == 0x12)
+			if (hdr.realtick[1] == 0x34 && hdr.realtick[2] == 0x12)
 				wth->capture.netxray->fcs_valid = TRUE;
 		}
 		break;
@@ -531,9 +813,13 @@ int netxray_open(wtap *wth, int *err, gchar **err_info)
 
 	/* Remember the offset after the last packet in the capture (which
 	 * isn't necessarily the last packet in the file), as it appears
-	 * there's sometimes crud after it. */
-	wth->capture.netxray->wrapped = FALSE;
-	wth->capture.netxray->end_offset = pletohl(&hdr.end_offset);
+	 * there's sometimes crud after it.
+	 * XXX: Remember 'start_offset' to help testing for 'short file' at EOF
+	 */
+	wth->capture.netxray->wrapped      = FALSE;
+	wth->capture.netxray->nframes      = pletohl(&hdr.nframes);
+	wth->capture.netxray->start_offset = pletohl(&hdr.start_offset);
+	wth->capture.netxray->end_offset   = pletohl(&hdr.end_offset);
 
 	/* Seek to the beginning of the data records. */
 	if (file_seek(wth->fh, pletohl(&hdr.start_offset), SEEK_SET, err) == -1) {
@@ -576,7 +862,31 @@ reread:
 			return FALSE;
 		}
 
-		/* We're at EOF.  Wrap? */
+		/* We're at EOF.  Wrap?
+		 * XXX: Need to handle 'short file' cases
+		 *      (Distributed Sniffer seems to have a 
+		 *	 certain small propensity to generate 'short' files
+		 *       i.e. [many] bytes are missing from the end of the file)
+		 *   case 1: start_offset < end_offset 
+		 *           wrap will read already read packets again;
+		 *           so: error with "short file"
+		 *   case 2: start_offset > end_offset ("circular" file)
+		 *           wrap will mean there's a gap (missing packets).
+		 *	     However, I don't see a good way to identify this
+		 *           case so we'll just have to allow the wrap.
+		 *           (Maybe there can be an error message after all
+		 *            packets are read since there'll be less packets than
+		 *            specified in the file header).
+		 * Note that these cases occur *only* if a 'short' eof occurs exactly 
+		 * at the expected beginning of a frame header record; If there is a
+		 * partial frame header (or partial frame data) record, then the
+		 * netxray_read... functions will detect the short record.
+		 */
+		if (wth->capture.netxray->start_offset < wth->capture.netxray->end_offset) {
+			*err = WTAP_ERR_SHORT_READ;
+			return FALSE;
+		}
+		
 		if (!wth->capture.netxray->wrapped) {
 			/* Yes.  Remember that we did. */
 			wth->capture.netxray->wrapped = TRUE;
@@ -622,9 +932,9 @@ reread:
 		    + (double)pletohl(&hdr.old_hdr.timehi)*4294967296.0;
 		t /= wth->capture.netxray->timeunit;
 		t -= wth->capture.netxray->start_timestamp;
-		wth->phdr.ts.tv_sec = wth->capture.netxray->start_time + (long)t;
-		wth->phdr.ts.tv_usec = (unsigned long)((t-(double)(unsigned long)(t))
-			*1.0e6);
+		wth->phdr.ts.secs = wth->capture.netxray->start_time + (long)t;
+		wth->phdr.ts.nsecs = (unsigned long)((t-(double)(unsigned long)(t))
+			*1.0e9);
 		/*
 		 * We subtract the padding from the packet size, so our caller
 		 * doesn't see it.
@@ -636,9 +946,9 @@ reread:
 		    + (double)pletohl(&hdr.hdr_1_x.timehi)*4294967296.0;
 		t /= wth->capture.netxray->timeunit;
 		t -= wth->capture.netxray->start_timestamp;
-		wth->phdr.ts.tv_sec = wth->capture.netxray->start_time + (long)t;
-		wth->phdr.ts.tv_usec = (unsigned long)((t-(double)(unsigned long)(t))
-			*1.0e6);
+		wth->phdr.ts.secs = wth->capture.netxray->start_time + (long)t;
+		wth->phdr.ts.nsecs = (unsigned long)((t-(double)(unsigned long)(t))
+			*1.0e9);
 		/*
 		 * We subtract the padding from the packet size, so our caller
 		 * doesn't see it.
@@ -803,8 +1113,20 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 			 * It appears, in one 802.11 capture, that
 			 * we have 4 bytes of junk at the ends of
 			 * frames in which hdr->hdr_2_x.xxx[2] and
-			 * hdr->hdr_2_x.xxx[3] are 0xff; we assume
-			 * for now that it works like Ethernet.
+			 * hdr->hdr_2_x.xxx[3] are 0xff; I haven't
+			 * seen any frames where it's an FCS, but,
+			 * for now, we still check the fcs_valid
+			 * flag - I also haven't seen any capture
+			 * where we'd set it based on the realtick
+			 * value.
+			 *
+			 * It also appears that if the low-order bit of
+			 * hdr->hdr_2_x.xxx[8] is set, the packet has a
+			 * bad FCS.  According to Ken Mann, the 0x4 bit
+			 * is sometimes also set for errors.
+			 *
+			 * Ken also says that xxx[11] is 0x5 when the
+			 * packet is WEP-encrypted.
 			 */
 			if (hdr->hdr_2_x.xxx[2] == 0xff &&
 			    hdr->hdr_2_x.xxx[3] == 0xff) {
@@ -832,6 +1154,12 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 			    hdr->hdr_2_x.xxx[13];
 			pseudo_header->ieee_802_11.signal_level =
 			    hdr->hdr_2_x.xxx[14];
+			/*
+			 * According to Ken Mann, at least in the captures
+			 * he's seen, xxx[15] is the noise level, which
+			 * is either 0xFF meaning "none reported" or a value
+			 * from 0x00 to 0x7F for 0 to 100%.
+			 */
 			break;
 
 		case WTAP_ENCAP_ISDN:
@@ -1098,8 +1426,8 @@ gboolean netxray_dump_open_1_1(wtap_dumper *wdh, gboolean cant_seek, int *err)
 
     wdh->dump.netxray = g_malloc(sizeof(netxray_dump_t));
     wdh->dump.netxray->first_frame = TRUE;
-    wdh->dump.netxray->start.tv_sec = 0;
-    wdh->dump.netxray->start.tv_usec = 0;
+    wdh->dump.netxray->start.secs = 0;
+    wdh->dump.netxray->start.nsecs = 0;
     wdh->dump.netxray->nframes = 0;
 
     return TRUE;
@@ -1134,8 +1462,8 @@ static gboolean netxray_dump_1_1(wtap_dumper *wdh,
 
     /* build the header for each packet */
     memset(&rec_hdr, '\0', sizeof(rec_hdr));
-    timestamp = (phdr->ts.tv_sec - netxray->start.tv_sec)*1000000 +
-        phdr->ts.tv_usec;
+    timestamp = (phdr->ts.secs - netxray->start.secs)*1000000 +
+        phdr->ts.nsecs / 1000;
     rec_hdr.timelo = htolel(timestamp);
     rec_hdr.timehi = htolel(0);
     rec_hdr.orig_len = htoles(phdr->len);
@@ -1195,7 +1523,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     /* "sniffer" version ? */
     memset(&file_hdr, '\0', sizeof file_hdr);
     memcpy(file_hdr.version, vers_1_1, sizeof vers_1_1);
-    file_hdr.start_time = htolel(netxray->start.tv_sec);
+    file_hdr.start_time = htolel(netxray->start.secs);
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
@@ -1287,8 +1615,8 @@ gboolean netxray_dump_open_2_0(wtap_dumper *wdh, gboolean cant_seek, int *err)
 
     wdh->dump.netxray = g_malloc(sizeof(netxray_dump_t));
     wdh->dump.netxray->first_frame = TRUE;
-    wdh->dump.netxray->start.tv_sec = 0;
-    wdh->dump.netxray->start.tv_usec = 0;
+    wdh->dump.netxray->start.secs = 0;
+    wdh->dump.netxray->start.nsecs = 0;
     wdh->dump.netxray->nframes = 0;
 
     return TRUE;
@@ -1323,8 +1651,8 @@ static gboolean netxray_dump_2_0(wtap_dumper *wdh,
 
     /* build the header for each packet */
     memset(&rec_hdr, '\0', sizeof(rec_hdr));
-    timestamp = (phdr->ts.tv_sec - netxray->start.tv_sec)*1000000 +
-        phdr->ts.tv_usec;
+    timestamp = (phdr->ts.secs - netxray->start.secs)*1000000 +
+        phdr->ts.nsecs/1000;
     rec_hdr.timelo = htolel(timestamp);
     rec_hdr.timehi = htolel(0);
     rec_hdr.orig_len = htoles(phdr->len);
@@ -1402,7 +1730,7 @@ static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err)
     /* "sniffer" version ? */
     memset(&file_hdr, '\0', sizeof file_hdr);
     memcpy(file_hdr.version, vers_2_001, sizeof vers_2_001);
-    file_hdr.start_time = htolel(netxray->start.tv_sec);
+    file_hdr.start_time = htolel(netxray->start.secs);
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
@@ -1412,24 +1740,24 @@ static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err)
     switch (wdh->encap) {
 
     case WTAP_ENCAP_PPP_WITH_PHDR:
-	file_hdr.xxb[20] = CAPTYPE_PPP;
+	file_hdr.captype = WAN_CAPTYPE_PPP;
 	break;
 
     case WTAP_ENCAP_FRELAY_WITH_PHDR:
-	file_hdr.xxb[20] = CAPTYPE_FRELAY;
+	file_hdr.captype = WAN_CAPTYPE_FRELAY;
 	break;
 
     case WTAP_ENCAP_LAPB:
-	file_hdr.xxb[20] = CAPTYPE_HDLC;
-	file_hdr.xxb[28] = 0;
+	file_hdr.captype = WAN_CAPTYPE_HDLC;
+	file_hdr.wan_hdlc_subsub_captype = 0;
 	break;
 
     case WTAP_ENCAP_SDLC:
-	file_hdr.xxb[20] = CAPTYPE_SDLC;
+	file_hdr.captype = WAN_CAPTYPE_SDLC;
 	break;
 
     default:
-	file_hdr.xxb[20] = CAPTYPE_NDIS;
+	file_hdr.captype = CAPTYPE_NDIS;
 	break;
     }
 
