@@ -2,7 +2,7 @@
  * File read and write routines for Visual Networks cap files.
  * Copyright (c) 2001, Tom Nisbet  tnisbet@visualnetworks.com
  *
- * $Id: visual.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: visual.c 15534 2005-08-25 21:29:54Z ulfl $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -215,6 +215,7 @@ int visual_open(wtap *wth, int *err, gchar **err_info)
     wth->subtype_read = visual_read;
     wth->subtype_seek_read = visual_seek_read;
     wth->subtype_close = visual_close;
+	wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 
     /* Add Visual-specific information to the wiretap struct for later use. */
     visual = g_malloc(sizeof(struct visual_read_info));
@@ -298,8 +299,8 @@ static gboolean visual_read(wtap *wth, int *err, gchar **err_info,
     t += ((double)pletohl(&vpkt_hdr.ts_delta))*1000;
     secs = (time_t)(t/1000000);
     usecs = (guint32)(t - secs*1000000);
-    wth->phdr.ts.tv_sec = secs;
-    wth->phdr.ts.tv_usec = usecs;
+    wth->phdr.ts.secs = secs;
+    wth->phdr.ts.nsecs = usecs * 1000;
     wth->phdr.caplen = packet_size;
     wth->phdr.len = pletohs(&vpkt_hdr.orig_len);
 
@@ -489,7 +490,7 @@ static gboolean visual_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     if (visual->index_table_index == 0)
     {
         /* This is the first packet.  Save its start time as the file time. */
-        visual->start_time = phdr->ts.tv_sec;
+        visual->start_time = phdr->ts.secs;
 
         /* Initialize the index table */
         visual->index_table = g_malloc(1024 * sizeof *visual->index_table);
@@ -497,8 +498,8 @@ static gboolean visual_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     }
 
     /* Calculate milliseconds since capture start. */
-    delta_msec = phdr->ts.tv_usec / 1000;
-    delta_msec += (phdr->ts.tv_sec - visual->start_time) * 1000;
+    delta_msec = phdr->ts.nsecs / 1000000;
+    delta_msec += (phdr->ts.secs - visual->start_time) * 1000;
     vpkt_hdr.ts_delta = htolel(delta_msec);
 
     /* Fill in the length fields. */
