@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h 13311 2005-02-05 17:39:12Z gerald $
+ * $Id: wtap.h 17450 2006-03-02 21:28:05Z etxrab $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -33,6 +33,7 @@
 
 #include <glib.h>
 #include <stdio.h>
+#include <time.h>
 
 /* Encapsulation types. Choose names that truly reflect
  * what is contained in the packet trace file.
@@ -153,15 +154,34 @@
 #define WTAP_ENCAP_SYMANTEC			61
 #define WTAP_ENCAP_APPLE_IP_OVER_IEEE1394	62
 #define WTAP_ENCAP_BACNET_MS_TP			63
-#define WTAP_ENCAP_RAW_ICMP			64
-#define WTAP_ENCAP_RAW_ICMPV6			65
+#define WTAP_ENCAP_NETTL_RAW_ICMP		64
+#define WTAP_ENCAP_NETTL_RAW_ICMPV6		65
 #define WTAP_ENCAP_GPRS_LLC			67
 #define WTAP_ENCAP_JUNIPER_ATM1			68
 #define WTAP_ENCAP_JUNIPER_ATM2			69
 #define WTAP_ENCAP_REDBACK			70
+#define WTAP_ENCAP_NETTL_RAW_IP			71
+#define WTAP_ENCAP_NETTL_ETHERNET		72
+#define WTAP_ENCAP_NETTL_TOKEN_RING		73
+#define WTAP_ENCAP_NETTL_FDDI			74
+#define WTAP_ENCAP_NETTL_UNKNOWN		75
+#define WTAP_ENCAP_MTP2_WITH_PHDR               76
+#define WTAP_ENCAP_JUNIPER_PPPOE       		77
+#define WTAP_GCOM_TIE1				78
+#define WTAP_GCOM_SERIAL			79
+#define WTAP_ENCAP_NETTL_X25			80
+#define WTAP_ENCAP_K12					81
+#define WTAP_ENCAP_JUNIPER_MLPPP                82
+#define WTAP_ENCAP_JUNIPER_MLFR                 83
+#define WTAP_ENCAP_JUNIPER_ETHER                84
+#define WTAP_ENCAP_JUNIPER_PPP                  85
+#define WTAP_ENCAP_JUNIPER_FRELAY               86
+#define WTAP_ENCAP_JUNIPER_CHDLC                87
+#define WTAP_ENCAP_JUNIPER_GGSN                 88
+#define WTAP_ENCAP_LINUX_LAPD			89
 
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			71
+#define WTAP_NUM_ENCAP_TYPES			90
 
 /* File types that can be read by wiretap.
    We support writing some many of these file types, too, so we
@@ -174,41 +194,53 @@
 #define WTAP_FILE_PCAP_SS991029			5
 #define WTAP_FILE_PCAP_NOKIA			6
 #define WTAP_FILE_PCAP_AIX			7
-#define WTAP_FILE_LANALYZER			8
-#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	9
-#define WTAP_FILE_NGSNIFFER_COMPRESSED		10
-#define WTAP_FILE_SNOOP				11
-#define WTAP_FILE_SHOMITI			12
-#define WTAP_FILE_IPTRACE_1_0			13
-#define WTAP_FILE_IPTRACE_2_0			14
-#define WTAP_FILE_NETMON_1_x			15
-#define WTAP_FILE_NETMON_2_x			16
-#define WTAP_FILE_NETXRAY_OLD			17
-#define WTAP_FILE_NETXRAY_1_0			18
-#define WTAP_FILE_NETXRAY_1_1			19
-#define WTAP_FILE_NETXRAY_2_00x			20
-#define WTAP_FILE_RADCOM			21
-#define WTAP_FILE_ASCEND			22
-#define WTAP_FILE_NETTL				23
-#define WTAP_FILE_TOSHIBA			24
-#define WTAP_FILE_I4BTRACE			25
-#define WTAP_FILE_CSIDS				26
-#define WTAP_FILE_PPPDUMP			27
-#define WTAP_FILE_ETHERPEEK_V56			28
-#define WTAP_FILE_ETHERPEEK_V7			29
-#define WTAP_FILE_VMS				30
-#define WTAP_FILE_DBS_ETHERWATCH		31
-#define WTAP_FILE_VISUAL_NETWORKS		32
-#define WTAP_FILE_COSINE			33
-#define WTAP_FILE_5VIEWS			34
-#define WTAP_FILE_ERF				35
-#define WTAP_FILE_HCIDUMP			36
-#define WTAP_FILE_NETWORK_INSTRUMENTS_V9	37
-#define WTAP_FILE_AIROPEEK_V9			38
-#define WTAP_FILE_EYESDN			39
+#define WTAP_FILE_PCAP_NSEC			8
+#define WTAP_FILE_LANALYZER			9
+#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	10
+#define WTAP_FILE_NGSNIFFER_COMPRESSED		11
+#define WTAP_FILE_SNOOP				12
+#define WTAP_FILE_SHOMITI			13
+#define WTAP_FILE_IPTRACE_1_0			14
+#define WTAP_FILE_IPTRACE_2_0			15
+#define WTAP_FILE_NETMON_1_x			16
+#define WTAP_FILE_NETMON_2_x			17
+#define WTAP_FILE_NETXRAY_OLD			18
+#define WTAP_FILE_NETXRAY_1_0			19
+#define WTAP_FILE_NETXRAY_1_1			20
+#define WTAP_FILE_NETXRAY_2_00x			21
+#define WTAP_FILE_RADCOM			22
+#define WTAP_FILE_ASCEND			23
+#define WTAP_FILE_NETTL				24
+#define WTAP_FILE_TOSHIBA			25
+#define WTAP_FILE_I4BTRACE			26
+#define WTAP_FILE_CSIDS				27
+#define WTAP_FILE_PPPDUMP			28
+#define WTAP_FILE_ETHERPEEK_V56			29
+#define WTAP_FILE_ETHERPEEK_V7			30
+#define WTAP_FILE_VMS				31
+#define WTAP_FILE_DBS_ETHERWATCH		32
+#define WTAP_FILE_VISUAL_NETWORKS		33
+#define WTAP_FILE_COSINE			34
+#define WTAP_FILE_5VIEWS			35
+#define WTAP_FILE_ERF				36
+#define WTAP_FILE_HCIDUMP			37
+#define WTAP_FILE_NETWORK_INSTRUMENTS_V9	38
+#define WTAP_FILE_AIROPEEK_V9			39
+#define WTAP_FILE_EYESDN			40
+#define WTAP_FILE_K12				41
+#define WTAP_FILE_ISERIES			42
+#define WTAP_FILE_ISERIES_UNICODE		43
 
 /* last WTAP_FILE_ value + 1 */
-#define WTAP_NUM_FILE_TYPES			40
+#define WTAP_NUM_FILE_TYPES			44
+
+/* timestamp precision (currently only these values are supported) */
+#define WTAP_FILE_TSPREC_SEC		0
+#define WTAP_FILE_TSPREC_DSEC		1
+#define WTAP_FILE_TSPREC_CSEC		2
+#define WTAP_FILE_TSPREC_MSEC		3
+#define WTAP_FILE_TSPREC_USEC		6
+#define WTAP_FILE_TSPREC_NSEC		9
 
 /*
  * Maximum packet size we'll support.
@@ -422,6 +454,54 @@ struct irda_phdr {
 	guint16 pkttype;    /* packet type */
 };
 
+/* Packet "pseudo-header" for nettl (HP-UX) capture files. */
+
+struct nettl_phdr {
+	guint16 subsys;
+	guint32 devid;
+	guint32 kind;
+	gint32  pid;
+	guint16 uid;
+};
+
+/* Packet "pseudo-header" for MTP2 files. */
+
+struct mtp2_phdr {
+	guint8  sent;         
+	guint8  annex_a_used;   
+	guint16 link_number;  
+};
+
+/* Packet "pseudo-header" for K12 files. */
+
+typedef union {
+	struct {
+		guint16 vp;
+		guint16 vc;
+        guint16 cid;
+	} atm;
+	
+	guint32 ds0mask;
+} k12_input_info_t;
+
+struct k12_phdr {
+	guint32 input;
+	const gchar* input_name;
+	const gchar* stack_file;
+	guint32 input_type;
+	k12_input_info_t input_info;
+	void* stuff;
+};
+
+#define K12_PORT_DS0S      0x00010008
+#define K12_PORT_DS1       0x00100008
+#define K12_PORT_ATMPVC    0x01020000
+
+struct lapd_phdr {
+	guint16 pkttype;    /* packet type */
+	guint8 we_network;
+};
+
 union wtap_pseudo_header {
 	struct eth_phdr		eth;
 	struct x25_phdr		x25;
@@ -432,18 +512,25 @@ union wtap_pseudo_header {
 	struct ieee_802_11_phdr	ieee_802_11;
 	struct cosine_phdr	cosine;
 	struct irda_phdr	irda;
+	struct nettl_phdr	nettl;
+	struct mtp2_phdr        mtp2;
+	struct k12_phdr		k12;
+	struct lapd_phdr	lapd;
 };
 
+struct wtap_nstime {
+	time_t	secs;
+	int	nsecs;
+};
+
+
 struct wtap_pkthdr {
-	struct timeval ts;
+	struct wtap_nstime ts;
 	guint32	caplen;
 	guint32 len;
 	int pkt_encap;
 };
 
-typedef void (*wtap_handler)(guchar*, const struct wtap_pkthdr*,
-		long, union wtap_pseudo_header *pseudo_header, const guchar *);
-
 struct wtap;
 struct Buffer;
 struct wtap_dumper;
@@ -462,25 +549,27 @@ typedef struct wtap_dumper wtap_dumper;
 struct wtap* wtap_open_offline(const char *filename, int *err,
     gchar **err_info, gboolean do_random);
 
-/* Returns TRUE if entire loop-reading was successful. If read failure
- * happened, FALSE is returned and err is set. */
-gboolean wtap_loop(wtap *wth, int, wtap_handler, guchar*, int *err,
-    gchar **err_info);
-
 /* Returns TRUE if read was successful. FALSE if failure. data_offset is
  * set the the offset in the file where the data for the read packet is
  * located. */
 gboolean wtap_read(wtap *wth, int *err, gchar **err_info,
     long *data_offset);
 
+/*
+ * Return an approximation of the amount of data we've read sequentially
+ * from the file so far.  (gint64, in case that's 64 bits.)
+ */
+gint64 wtap_read_so_far(wtap *wth, int *err);
+
 struct wtap_pkthdr *wtap_phdr(wtap *wth);
 union wtap_pseudo_header *wtap_pseudoheader(wtap *wth);
 guint8 *wtap_buf_ptr(wtap *wth);
 
-int wtap_fd(wtap *wth);
+gint64 wtap_file_size(wtap *wth, int *err);
 int wtap_snapshot_length(wtap *wth); /* per file */
 int wtap_file_type(wtap *wth);
 int wtap_file_encap(wtap *wth);
+int wtap_file_tsprecision(wtap *wth);
 
 const char *wtap_file_type_string(int filetype);
 const char *wtap_file_type_short_string(int filetype);
@@ -499,13 +588,14 @@ gboolean wtap_seek_read (wtap *wth, long seek_off,
 
 gboolean wtap_dump_can_open(int filetype);
 gboolean wtap_dump_can_write_encap(int filetype, int encap);
+gboolean wtap_dump_can_compress(int filetype);
 wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
-	int snaplen, int *err);
+	int snaplen, gboolean compressed, int *err);
 wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
-	int *err);
+	gboolean compressed, int *err);
 gboolean wtap_dump(wtap_dumper *, const struct wtap_pkthdr *,
 	const union wtap_pseudo_header *pseudo_header, const guchar *, int *err);
-FILE* wtap_dump_file(wtap_dumper *);
+void wtap_dump_flush(wtap_dumper *);
 gboolean wtap_dump_close(wtap_dumper *, int *);
 long wtap_get_bytes_dumped(wtap_dumper *);
 void wtap_set_bytes_dumped(wtap_dumper *wdh, long bytes_dumped);
@@ -551,6 +641,8 @@ void wtap_set_bytes_dumped(wtap_dumper *wdh, long bytes_dumped);
 	/* LZ77 compressed data has bad offset to string */
 #define	WTAP_ERR_RANDOM_OPEN_STDIN		-18
 	/* We're trying to open the standard input for random access */
+#define WTAP_ERR_COMPRESSION_NOT_SUPPORTED -19
+	/* The filetype doesn't support output compression */
 
 /* Errors from zlib; zlib error Z_xxx turns into Wiretap error
    WTAP_ERR_ZLIB + Z_xxx.
